﻿; Windows OS
; GFX_Wizzard_BF - Desktop scaling functions for Windows, with or without activated PB DPI aware compiler option
; This functions allows you to easily determine the maximum available width for windows on any desktop with OS scaling settings above 100%

; With Windows DPIaware scaling check
; If DPI aware is disabled in the PB Compiler options,
; it may still be enabled if a user is using scalings up to 100%
; Then the process can be DPI aware even though DPI aware is disabled in the PB compiler options
; This then leads to incorrect scaling and will destroy your layout between a window, gadgets and text
; With this function you can test if a used scaling works - result = 1 is OK
; If 0 is returned there are scaling problems

; (C) W. Albus

XIncludeFile("./GFX_Wizzard_BF.pbi")

EnableExplicit

LoadFont(0, "", 14) 

Define image_1_ID=LoadImage(#PB_Any, "./Image_Set/Lion.png")

Define image_2_ID=LoadImage(#PB_Any, "./Image_Set/Chicken_1_Alpha/Chicken_1_Alpha5.png")

Define texture_ID=LoadImage(#PB_Any, "./Image_Set/stone_1.jpg")

Define window_ID=OpenWindow(#PB_Any, 0, 0,
                            DesktopWidth_UserScaled_BF()*0.60,
                            DesktopHeight_UserScaled_BF()*0.60,
                            "Desktop scaled window - GFX_Wizzard_BF - W. Albus", #PB_Window_SystemMenu | #PB_Window_ScreenCentered |#PB_Window_Invisible)

Define output_width=WindowWidth(window_ID)*DPI_ResizeFactor_BF()
Define output_height=WindowHeight(window_ID)*DPI_ResizeFactor_BF()

; ============= Init the scaling variables ==============
Global window_width_developing=1000  ; Developing size x
Global window_height_developing=1000 ; Developing size y

window_width_developing*DPI_ResizeFactor_BF()
window_height_developing*DPI_ResizeFactor_BF()
Global set_x.f=100/window_width_developing*output_width/100
Global set_y.f=100/window_height_developing*output_height/100

Define canvas_ID=CanvasGadget(#PB_Any, 0, 0, output_width, output_height, #PB_Canvas_Container)

If window_ID And canvas_ID And image_1_ID And image_2_ID And texture_ID
  
  ; Make a textured background          
  Define texture_1_ID=CreateImage(#PB_Any, output_width, output_height)
  ErrorCheck_BF(BucketFill_BF(-2, texture_1_ID, texture_ID, 0, 0))
  FreeImage(texture_ID) ; This images are not longer needed                                     
  
  If StartDrawing(CanvasOutput(canvas_ID))
    DrawImage(ImageID(texture_1_ID), 0, 0)
    StopDrawing()
    FreeImage_BF(texture_ID)
    FreeImage_BF(texture_1_ID)
  EndIf
  
  Define text_1$="GFX_Wizzard_BF - Desktop scaling functions - By W. Albus"
  
  Define text_2$="This demo demonstrate window, text and graphic scaling"+#LF$+
                 "without distortions and missmatch"+#LF$+#LF$+
                 "The GFX_Wizzard_BF can handle all scalings and outputs"+#LF$+
                 "with and without DPI aware"+#LF$+#LF$+
                 "With the here integrated Windows OS DPI aware scaling tester"+#LF$+
                 "you can check if a used Windows OS scaling is defective"+#LF$+
                 "If the PB DPI aware compiler option is disabled"+#LF$+
                 "and the process is still DPI aware there are scaling problems"
  
  ; Activate the DPI aware enhancement - For using Gadgets_BF and FitImage_BF
  ; Use it with the compiler option
  ; Look also for the other demo codes with DPI aware enhancement (Search to 'DPI' in the demo codes names)
  Activate_DPI_aware_BF(1) ; This is the magic
  
  If IsProcessDPIaware_BF()
    Define dpi_aware$="Process is DPI aware"
  Else
    dpi_aware$="Process is not DPI aware"
  EndIf
  
  If IsDPIaware_BF()
    Define dpi_aware_$="DPI aware compiler option is enabled"
  Else
    dpi_aware_$="DPI aware compiler option is disabled"
  EndIf
  
  CompilerIf #PB_Compiler_OS=#PB_OS_Windows And #PB_Compiler_Version>=570
    If IsDPIaware_BF()
      Define scaling_check$=#LF$+#LF$+
                            "To test if a used Windows OS scaling is broken, disable firstly DPI aware in the PB compiler options"
    Else
      If IsWindowsScalingOK_BF()
        Define scaling_check$=#LF$+#LF$+"The used Windows OS scaling is OK"
      Else
        scaling_check$=#LF$+#LF$+"WARNING"+#LF$+"The used advanced Windows OS scaling is defective"+#LF$+
                       "Text outputs, window and canvas size are not displayed correctly"
      EndIf
    EndIf
  CompilerElse
    Define scaling_check$
  CompilerEndIf
  
  Define text_3$="GFX_Wizzard_BF"+#LF$+
                 "With this functions you can adjust and scale any outputs on desktop"+#LF$+#LF$+
                 "It works at the same with or without activated PB DPI aware compiler option"+#LF$+
                 "The window here is automatically fitted to any user scaled desktop"+#LF$+#LF$+
                 "Desktop width  : "+Str(DesktopWidth_BF())+#LF$+
                 "Desktop height : "+Str(DesktopHeight_BF())+#LF$+#LF$+
                 "User scaled desktop width  : "+DesktopWidth_UserScaled_BF()+#LF$+
                 "User scaled desktop height : "+DesktopHeight_UserScaled_BF()+#LF$+#LF$+
                 "User scaled resulted desktop resize factor width  : "+Desktop_ResizeFactor_Width_BF()+#LF$+
                 "User scaled resulted desktop resize factor height : "+Desktop_ResizeFactor_Height_BF()+#LF$+#LF$+
                 "User set desktop scaling width  : "+Desktop_ScalingPercent_Width_BF()+" %"+#LF$+
                 "User set desktop scaling height : "+Desktop_ScalingPercent_Height_BF()+" %"+#LF$+#LF$+
                 dpi_aware$+#LF$+dpi_aware_$+scaling_check$
  
  ; Add to the FitImage functions a automatically upscaling for little images - 1=activated - 0=deactivated
  ; -You must deactivate again
  ; -As sample also for ColorButtonGadget_BF, TextGadget_BF, ImageGadget_BF, ButtonImageGadget_BF
  ; -Use you to little fonts, BF upscale, but this reduce the font output quality
  FitImageActivateUpscaling_BF(1)
  
  ; Create a font array
  Define needed_font_size, max_needed_font_size=40 ; 25 > 40 is mostly onough - Larger is slower
  Define Dim Font_Arial(max_needed_font_size)
  For needed_font_size=1 To max_needed_font_size ; Get the fonts
    Font_Arial(needed_font_size)=LoadFont(#PB_Any, "Arial", needed_font_size, #PB_Font_Bold)
  Next needed_font_size
  
  ; You can import a existing gadget_ID for creating a Gadget_BF from a Gadget PB : Set x=-2 and y the gadget_ID
  Define image_gadget_1_ID=ImageGadget_BF(image_1_ID,       ; For initializing a image_ID is not needed
                                          20 *set_x,        ; Output pos x
                                          400*set_y,        ; Output pos y
                                          200*set_x,        ; Rectangle width
                                          200*set_y,        ; Rectangle height
                                          $FF000000,        ; Background color - For a invisible background set background_color=-2 (-1 is reserved for ignore)
                                          0,                ; Padding
                                          $FFFFFF,          ; Grid color
                                          5,                ; Grid size
                                          10,               ; Grid tiling
                                          $FFFFFF,          ; Frame color
                                          3,                ; Frame size
                                          #PB_Image_Border) ; Flags same the PB function
                                                            ; The function give back the gadget ID
                                                            ; For a empty image output set image_ID=-1
                                                            ; The Gadget is compatible to the PB ImageGadget 
                                                            ; BF Supports also automatically image upscaling with the function FitImageActivateUpscaling_BF
                                                            ; Use a invisible background only for images with invisible color or you see nothing
                                                            ; ImageGadget_BF create and cache needed temporary images for WinOS outputs automatically
  
  ; Mirror the image in it self
  Mirror_and_Copy_BF(1, image_1_ID) ; This function can also a lot more, look in the GFX_Wizzard base code on declare
  
  ; You can import a existing gadget_ID for creating a Gadget_BF from a Gadget PB : Set x=-2 and y the gadget_ID
  Define image_gadget_2_ID=ImageGadget_BF(image_1_ID,       ; For initializing a image_ID is not needed
                                          780*set_x,        ; Output pos x
                                          400*set_y,        ; Output pos y
                                          200*set_x,        ; Rectangle width
                                          200*set_y,        ; Rectangle height
                                          $FF000000,        ; Background color - For a invisible background set background_color=-2 (-1 is reserved for ignore)
                                          0,                ; Padding
                                          $FFFFFF,          ; Grid color
                                          5,                ; Grid size
                                          10,               ; Grid tiling
                                          $FFFFFF,          ; Frame color
                                          3,                ; Frame size
                                          #PB_Image_Border) ; Flags same the PB function
                                                            ; The function give back the gadget ID
                                                            ; For a empty image output set image_ID=-1
                                                            ; The Gadget is compatible to the PB ImageGadget 
                                                            ; BF Supports also automatically image upscaling with the function FitImageActivateUpscaling_BF
                                                            ; Use a invisible background only for images with invisible color or you see nothing
                                                            ; ImageGadget_BF create and cache needed temporary images for WinOS outputs automatically
  
  ; Create a fitted image with dynamic size for output on canvas as BF Sprite :-)
  ; This function can also a lot more, look in the GFX_Wizzard base code on declare
  ActivateAlphaChannel_BF(1) ; Activate the alpha channel
  Define image_4_ID=FitImage_BF(1, image_2_ID,
                                300*set_x*DPI_ResizeFactor_BF(),
                                300*set_y*DPI_ResizeFactor_BF())          
  ActivateAlphaChannel_BF(0); Don't forget - Deactivate the alpha channel again or it influence other Gadgets BF
  FreeImage_BF(image_2_ID)  ; Bulletproof free images - This image is not longer needed
  
  ; This is a mighty BF core based function
  ErrorCheck_BF(AlphaChannelSprite_Pro_BF(canvas_ID, image_4_ID,
                                          -20*set_x*DPI_ResizeFactor_BF(), ; Output pos x
                                          -30*set_y*DPI_ResizeFactor_BF(), ; Output pos y
                                          0,         ; Endposition texture output  x - Preset 0 = Clipping is automatic to input width
                                          0,         ; Endposition texture output  y - Preset 0 = Clipping is automatic to input height
                                          0,         ; Rectangle width
                                          0,         ; Rectangle height
                                          0,         ; texture clip_x
                                          0,         ; texture clip_y
                                          0,         ; texture clip_width
                                          0))        ; texture clip height
  
  ; Automatic text size
  ; Automatic text adjustment
  ; Outputs all fonts with edge smoothing
  ; Can handle GradientColor_BF
  ; font_ID you must not set
  ; Use ever a large font size - Then the function adjust the needed font size automatically (about 60 for the mostly large buttons)
  ; flag : Text center=1 - Text right=2 - Text vertical=3
  ; mode : Revers text mode=1 - RTL text mode=2
  ; stretch = Headline text stretcheng function
  ; BF Supports also automatically text upscaling with the function FitImageActivateUpscaling_BF
  ; You can import a existing gadget_ID for creating a Gadget_BF from a Gadget PB : Set x=-2 and y the gadget_ID
  ; This Gadget is canvas based - A needed or setted gadget_ID is a CanvasGadget ID
  ErrorCheck_BF(TextGadget_BF(text_1$,
                              (1000/2-300)*set_x, ; Output pos x
                              20*set_y,           ; Output pos y
                              600*set_x,          ; Rectangle width
                              120*set_y,          ; Rectangle height
                              $FF,                ; Text color
                              $FFFFFF,            ; Background color
                              Font_Arial(20),     ; Font ID
                              0,                  ; Stretch x
                              0,                  ; Stretch y
                              1,                  ; Flag
                              0,                  ; Mode
                              #PB_Canvas_Border)) ; Canvas flag
  
  ErrorCheck_BF(TextGadget_BF(text_2$,
                              (1000/2-250)*set_x, ; Output pos x
                              180*set_y,          ; Output pos y
                              500*set_x,          ; Rectangle width
                              250*set_y,          ; Rectangle height
                              $00,                ; Text color
                              $FFFFFF,            ; Background color
                              Font_Arial(20),     ; Font ID
                              0,                  ; Stretch x
                              0,                  ; Stretch y
                              1,                  ; Flag
                              0,                  ; Mode
                              #PB_Canvas_Border)) ; Canvas flag
  
  Define TextGadget_3_ID=TextGadget_BF(text_3$,
                                       (1000/2-250)*set_x, ; Output pos x
                                       470*set_y,          ; Output pos y
                                       500*set_x,          ; Rectangle width
                                       500*set_y,          ; Rectangle height
                                       $00,                ; Text color
                                       $FCFC98,            ; Background color
                                       Font_Arial(20),     ; Font ID
                                       0,                  ; Stretch x
                                       0,                  ; Stretch y
                                       1,                  ; Flag
                                       0,                  ; Mode
                                       #PB_Canvas_Border)  ; Canvas flag
  
  Define text_4$="Move this window"+#LF$+"to desktop 1"+#LF$+"and get all diameters"
  
  Deactivate_TextStretching_BF(1)
  Define StretchTextGadget_1_ID=StretchTextGadget_BF(text_4$,                  ; Text
                                                     30*set_x,                 ; Output pos x
                                                     690*set_y,                ; Output pos y
                                                     180*set_x,                ; Gadget width
                                                     100*set_y,                ; Gadget height
                                                     10*DPI_ResizeFactor_BF(), ; padding x
                                                     10*DPI_ResizeFactor_BF(), ; padding y
                                                     #Black,                   ; text_color
                                                     $FCFC98)                  ; back_color
  
  Define text_5$="Move this window"+#LF$+"to desktop 2"+#LF$+"and get all diameters"
  
  Define StretchTextGadget_2_ID=StretchTextGadget_BF(text_5$,                  ; Text
                                                     790*set_x,                ; Output pos x
                                                     690*set_y,                ; Output pos y
                                                     180*set_x,                ; Gadget width
                                                     100*set_y,                ; Gadget height
                                                     10*DPI_ResizeFactor_BF(), ; padding x
                                                     10*DPI_ResizeFactor_BF(), ; padding y
                                                     #Black,                   ; text_color
                                                     #Yellow)                  ; back_color
  
  CloseGadgetList()
  
  HideWindow(window_ID, 0)
  
  Repeat ; Get the events
    
    Define win_event=WaitWindowEvent()
    
    If win_event=#PB_Event_CloseWindow
      End
    EndIf
    
    If win_event=#PB_Event_Gadget
      
      Define event_gadget=EventGadget()
      
      Select event_gadget
          
        Case StretchTextGadget_1_ID
          ErrorCheck_BF(SetTextGadget_BF(TextGadget_3_ID, text_3$, -1, $FCFC98))      
          
        Case StretchTextGadget_2_ID
          Define text_6$="GFX_Wizzard_BF"+#LF$+
                         "With this functions you can adjust and scale any outputs on desktop"+#LF$+#LF$+
                         "It works at the same with or without activated PB DPI aware compiler option"+#LF$+
                         "The window here is automatically fitted to any user scaled desktop"+#LF$+#LF$+
                         "Desktop width  : "+Str(DesktopWidth_BF(1))+#LF$+
                         "Desktop height : "+Str(DesktopHeight_BF(1))+#LF$+#LF$+
                         "User scaled desktop width  : "+DesktopWidth_UserScaled_BF(1)+#LF$+
                         "User scaled desktop height : "+DesktopHeight_UserScaled_BF(1)+#LF$+#LF$+
                         "User scaled resulted desktop resize factor width  : "+Desktop_ResizeFactor_Width_BF(1)+#LF$+
                         "User scaled resulted desktop resize factor height : "+Desktop_ResizeFactor_Height_BF(1)+#LF$+#LF$+
                         "User set desktop scaling width  : "+Desktop_ScalingPercent_Width_BF(1)+" %"+#LF$+
                         "User set desktop scaling height : "+Desktop_ScalingPercent_Height_BF(1)+" %"+#LF$+#LF$+
                         dpi_aware$+#LF$+dpi_aware_$+scaling_check$
          ErrorCheck_BF(SetTextGadget_BF(TextGadget_3_ID, text_6$, -1, #Yellow))      
          
        Case image_gadget_1_ID, image_gadget_2_ID
          End
          
          ; BF cache on Windows OS automatically Gadgets_BF contents
          ; So you can move the window outside the desktop without lose this contents
          
          FreeTextures_BF() ; BF cache all used textures for highspeed sprite output
                            ; 0=give all free (Preset) - 1=Get amount cached textures - 2=Get required size for caching all textures in bytes
                            ; This refresh the cache and can so primary fix for your eyes strange animation problems
                            ; Temporary used textures free selective with the function Free_selected_Texture_BF, this is the recommended way
                            ; This function works also with PB sprites and canvas
                            ; Exit give free automatically all used textures
                            ; Works with all BF core function based functions - "Fast output" functions are not BF core based
          
      EndSelect
      
    EndIf
  ForEver
  
EndIf

; IDE Options = PureBasic 5.72 (Windows - x64)
; CursorPosition = 253
; FirstLine = 241
; Folding = -
; EnableXP
; DPIAware
; Executable = aaa.exe
; DisableDebugger