﻿; GFX_Wizzard_BF - For PureBasic
; Author : W. Albus © 2018 - 2020
; Copyright text and references may not be removed
; The use of the GFX-Wizzard_BF is free, but the distribution is reserved for me
; Exceptions require my written permission

DeclareModule GFX_Wizzard_BF
  ; All BF functions and the BF core function can use for input and output automatically directly images, canvas and PB sprites
  ; With the function "ActivateScreenOutput_BF" you can activate screen output for allmost all BF sprite functions
  ; Exceptions are only possible for functions where a corresponding input or output is nonsensical
  
  ; BF core based functions have a lot abilities
  ; At sample automatic tiling and scrolling and can output also BF sprites on PB sprites, with alpha blending
  ; BF sprites are shapes, a screen mode is not needed for output
  
  ; BF has a function named FlipBuffers_BF
  ; This function can run on hundred different instances and you get triple buffering on images, canvas and PB sprites
  
  ; Also available, a lot functions for creating, moving and converting images, canvas and PB sprites
  ; Descriptions for all functions you found here in the declare part and inside the demo files
  
  Global SysFont_BF     ; This is pre defined font
  Global SysFontBold_BF ; This is pre defined font
  
  ; Set a little Padding, this pervent the OS word wrap on multi line ButtonGadgets
  #min_padding_x=25
  #min_padding_y=0
  
  CompilerIf Not Defined(PB_Compiler_Backend, #PB_Constant)
    #PB_Compiler_Backend=0
    #PB_Backend_C=1
    #PB_Backend_Asm=0
  CompilerEndIf
  
  Declare IsWindowsScalingOK_BF()
  
  ; This function checks if the DPIaware Compiler flag is set
  ; This must not be confused with the function IsProcessDPIaware_BF(), it is not the same !
  ; If DPI aware is disabled in the PB Compiler options,
  ; it may still be enabled if a user is using scalings up to 100%
  ; Then the process can be DPI aware even though DPI aware is disabled in the PB compiler options
  ; This then leads to incorrect scaling and will destroy your layout between a window, gadgets and text
  Declare IsDPIaware_BF()
  
  Declare IsProcessDPIaware_BF() ; Check if the Process DPI aware
  
  Declare ColorDistance_BF(color_1, color_2) ; Rudimentary - Changeable from integer to float result
  
  Declare ErrorCheck_BF(result_) ; Rudimentary error checks
  
  ; 100 delay timer available - 0 to 99
  ; time=-1 Initialize the timer - This set the first delay to zero for beginning a animation directly without a delay)
  ; You can change the number of available timers
  ; For internal things use this timer not or you influence the timers
  ; For internal timer using look for Procedure "Delay_internal" and pre check in the code for actual used internal timers
  Declare Delay_BF(timer, time)
  
  Declare FreeImage_BF(image_ID) ; Free image - This function free only the image exist
  
  Declare Free_PB_Sprite_BF(sprite_ID) ; Free PB Sprite - This function free only the sprite exist
  
  Declare FreeGadget_BF(gadget_ID) ; Free Gadgets - This function free only the gadget exist
                                   ; Its needed for BF enhanced gadgets - As sample ColorButtonGadget_BF, TextGadget_BF, ImageGadget_BF
  
  Declare FreeAllGadgets_BF(mode=0) ; BF cache needed images for enhanced Gadgets_BF - As sample ColorButtonGadget_BF, TextGadget_BF
                                    ; 0=give all free - 1=Get amount cached Gadgets_BF
                                    ; Temporary used Gadgets_BF free selective with the function FreeGadget_BF, this is the recommended way
                                    ; Exit give free automatically all used images
  
  Declare SpriteWidthUnzoomed(sprite_ID) ; After ZoomSprite SpriteWidth give back the zoomed sprite width - This function give back the physically sprite size 
  
  Declare SpriteHeightUnzoomed(sprite_ID) ; After ZoomSprite SpriteHeight give back the zoomed sprite height - This function give back the physically sprite size 
  
  Declare GetFloodArray_Adress_BF() ; The arrays are free manipulable - Manipulation do not damage things
  
  Declare GetBucketArray_Adress_BF()
  
  Declare GetSpriteArray_Adress_BF()
  
  Declare GetFloodArray_State_BF() ;  The array length is ever ImageWidth*ImageHeight*8 
  
  Declare GetBucketArray_State_BF() ; So you can test the BF arrays state before you use a BF array
  
  Declare GetSpriteArray_State_BF()
  
  Declare GetFloodArray_Point_BF(x, y) ; Get a point - -1=used - 0=unused - The arrays use quads
  
  Declare GetBucketArray_Point_BF(x, y)
  
  Declare GetSpriteArray_Point_BF(x, y)
  
  Declare PutFloodArray_Point_BF(point, x, y) ; Write in the arrays
  
  Declare PutBucketArray_Point_BF(point, x, y)
  
  Declare PutSpriteArray_Point_BF(point, x, y)
  
  Declare GetFlood_X_BF() ; Min output coordinate FloodFill
  
  Declare GetFlood_Y_BF()
  
  Declare GetFlood_XX_BF() ; Max
  
  Declare GetFlood_YY_BF()
  
  Declare GetBucket_X_BF() ; Min output coordinate BucketFill
  
  Declare GetBucket_Y_BF()
  
  Declare GetBucket_XX_BF() ; Max
  
  Declare GetBucket_YY_BF()
  
  Declare GetSprite_X_BF() ; Min output coordinate Sprite
  
  Declare GetSprite_Y_BF()
  
  Declare GetSprite_XX_BF() ; Max
  
  Declare GetSprite_YY_BF()
  
  Declare GetSprite_width_BF() ; Without mask - Before using you must activate the sprite array - Free rotate sprite activate the sprite array automatically - Points
  
  Declare GetSprite_height_BF() ; Points
  
  Declare GetFloodArray_length_BF() ; Array length - bytes
  
  Declare GetBucketArray_length_BF()
  
  Declare GetSpriteArray_length_BF()
  
  Declare GetFloodArray_width_BF() ; Array width - bytes
  
  Declare GetFloodArray_height_BF() ; Array height - bytes
  
  Declare GetBucketArray_width_BF()
  
  Declare GetBucketArray_height_BF()
  
  Declare GetSpriteArray_width_BF()
  
  Declare GetSpriteArray_height_BF()
  
  ; Reduce a large mask from a BF sprite (image) - This function change the image ID - The function grab the mask color from pos x=0 - y=0
  ; Works it not or not perfekt, the mask is not clean - Set a little color distance ~2 Or more
  ; The function give back the new image ID and delete or delete not the source image ID
  ; Sample : image_ID=ReduceSpriteMask_BF(image_ID, 10, 0) - This delete the old image and create a new with reduced mask
  Declare ReduceSpriteMask_BF(image_ID, border, color_distance.f=0, delete_source_ID=1)
  
  Declare GetImageColor_BF(image_ID, x, y) ; Get a color from a image or a texture
  
  Declare Get_PB_SpriteColor_BF(sprite_ID, x, y) ; Get a color from a PB sprite
  
  Declare GetCanvasColor_BF(canvas_ID, x, y) ; Get a color from a canvas gadget
  
  Declare.q GetColor_BF() ; Get a placed color for replacing with a BucketFill texture and also the current status of SetColor_BF
  
  Declare SetColor_BF(color.q) ; Set a color for replacing with a BucketFill texture
                               ; You must deactivate this again with -1 - For white set $FFFFFF
                               ; For using this presetted color in a BucketFill call, set the parameter .._get_color_x To -1
                               ; This function refresh also the BF texture cache 
  
  Declare ActivateSpriteArray_BF(activate) ; Activate and deactivate the BF sprite array - 1=activated - 2=get only the sprite output coordinates, without mask - You must deactivate again
  
  Declare ActivateFloodArray_BF(activate) ; Activate the min max output coordinates - 1=activated - The primary flood array is ever active - You must deactivate again
  
  Declare ActivateBucketArray_BF(activate) ; Activate and deactivate the BF BucketFill array - 1=activated - 2=get only min max output coordinates - You must deactivate again
  
  ; This function is for simple creating monochrome overlay masks for sprites
  ; Works with the alpha blending function of the used sprite function
  ; Works with all sprite outputs - colormask_color is a quad - It is presetted deactivated with -1 - Set it for white so $FFFFFF - You must deactivate again
  Declare ActivateMaskCreating_BF(colormask_color_.q, colormask_alpha_=0) 
  
  Declare SearchUnusedColor_BF(input_ID, x, y, xx, yy, delay=5) ; 0 and $FFFFFF is ignored - delay in seconds before search breaks - For images, PB sprites and canvas                                                                         
  
  Declare SearchUnusedImageColor_BF(input_ID) ; 0 and $FFFFFF is ignored - timeout 5s - after timeout the function set a randomized color - For images, PB sprites and canvas 
  
  Declare SearchUsedColor_BF(input_ID, x, y, xx, yy, search_color) ; Usable for images, PB sprites and canvas - Give back 0 or 1
  
  Declare SetColorDistanceFill_BF(percent_.f) ; Set color distance for all fill functions - This function influence borders from BucketFill and FloodFill
  
  Declare SetColorDistanceSpriteMask_BF(percent_.f) ; Set color distance for all functions with sprite output - This function influence ALL used textures
  
  Declare SetInvisibleColor_BF(color.q) ; Set the invisible color - Use this with care for sheets without a invisible color, or you can get "holes" in your output frames
                                        ; This use a QUAD, deactivate again with SetInvisibleColor_BF(-1) - For white set $FFFFFF - This is a global function
                                        ; This deactivate grabbing transparence color from pos x, y
                                        ; -2 deactivate the invisible color
                                        ; This function is equal SetColor_BF
  
  Declare SetReplacingColor_BF(color.q) ; Set the color for replacing with BucketFill - This is a quad - It is presetted deactivated with -1 - Set it for white so $FFFFFF
  
  Declare.f GetColorDistanceFill_BF() ; Get placed color distance for all fill functions
  
  Declare.f GetColorDistanceSpriteMask_BF() ; Get placed color distance for sprite mask
  
  Declare.q GetInvisibleColor_BF() ; Get placed invisible color
  
  Declare.q GetUsedInvisibleColor_BF() ; Get the actual used invisible color
  
  Declare.q GetReplacingColor_BF() ; Get the color for replacing with BucketFill
  
  Declare GetTempImageUniversalOutput_ID_BF()
  
  Declare ActivateScreenOutput_BF(screen_output_) ; Activate screen output for BF based sprite functions - Internal this function set the BF core mode parameter =1
                                                  ; Works with all BF core based sprite functions
                                                  ; Is this function activated, a output_ID and the alpha channel is ignored
                                                  ; "Fast output" functions are not BF core based - You can use, but invisible color and alpha blending is ignored
  
  Declare ActivateAlphaChannel_BF(alpha_sprite_) ; Activate the alpha channel
                                                 ; Only for handling sources with alpha channel (PNG) or GIF
                                                 ; You must deactivate again <>0 = activated
                                                 ; This call influence the BF core function and all the core function used functions, and other - As sample rotating and CSS sheet creating
                                                 ; Has not a effect on sprite and CSS sheet "Fast output" functions and directly BF output on screen
                                                 ; "Fast output" functions are not BF core based 
  
  Declare AlphaBlend_BF(color_1, color_2, alpha) ; Rudimentary function
  
  Declare FreeTextures_BF(mode=0) ; BF cache all used textures for highspeed sprite output
                                  ; 0=give all free (Preset) - 1=Get amount cached textures - 2=Get required size for caching all textures in bytes
                                  ; This refresh the cache and can so primary fix for your eyes strange animation problems
                                  ; Temporary used textures free selective with the function Free_selected_Texture_BF, this is the recommended way
                                  ; This function works also with PB sprites and canvas
                                  ; Exit give free automatically all used textures
                                  ; Works with all BF core function based functions - "Fast output" functions are not BF core based
  
  Declare Free_selected_Texture_BF(texture_ID, free=1) ; Delete a selected texture from texture cache and give free
                                                       ; Presetted is a included PB FreeImage call
                                                       ; free=0 = only remove from BF cache - free=1 = remove from BF cache and give the image free
                                                       ; Ever a texture is used from a BF function, BF cache the texture automatic
                                                       ; Sucks a animation, you must remove the image from the BF cache
                                                       ; Change you a texture content, you must remove the old unchanged texture from the cache
                                                       ; Is a texture not used from a BF core based function, you can give free with the PB FreeImage function
                                                       ; This function works also with PB sprites and canvas
                                                       ; Wworks with all BF core function based functions - "Fast output" functions are not BF core based
  
  Declare Scale_Image_to_width_BF(image_ID, new_width, create_new_image=1) ; Scale a image proportional to a new defined width - Set create_new_image=1 for creating a new image
  
  Declare Scale_Image_to_height_BF(image_ID, new_height, create_new_image=1) ; Scale a image proportional to a new defined height - Set create_new_image=1 for creating a new image
  
  Declare GrabImage_BF(image_ID) ; Create a buffer and grab - You get a pointer, give free again this memory
  
  Declare Grab_PB_Sprite_BF(sprite_ID) ; Create a buffer and grab - You get a pointer, give free again this memory
  
  Declare GrabCanvas_BF(canvas_ID) ; Create a buffer and grab - You get a pointer, give free again this memory
  
  Declare DrawingBuffer_image_BF(image_ID) ; Get the drawing buffer adress
  
  Declare DrawingBuffer_PB_Sprite_BF(sprite_ID) ; Get the drawing buffer adress
  
  Declare DrawingBuffer_canvas_BF(canvas_ID) ; Get the drawing buffer adress
  
  Declare DrawingBuffer_length_image_BF(image_ID) ; Get the drawing buffer length
  
  Declare DrawingBuffer_length_PB_Sprite_BF(sprite_ID) ; Get the drawing buffer length
  
  Declare DrawingBuffer_length_canvas_BF(canvas_ID) ; Get the drawing buffer length
  
  Declare DrawingBuffer_pitch_image_BF(image_ID) ; Get the drawing buffer pitch
  
  Declare DrawingBuffer_pitch_PB_Sprite_BF(sprite_ID) ; Get the drawing buffer pitch 
  
  Declare DrawingBuffer_pitch_canvas_BF(canvas_ID) ; Get the drawing buffer pitch
  
  Declare CopyContent_Image_Image_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0) ; Move a content from image to image
  
  Declare CopyContent_PB_Sprite_PB_sprite_BF(source_ID, destination_ID) ; Move a content from PB sprite to PB sprite - Same size 
  
  Declare CopyContent_Image_PB_Sprite_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0) ; Move a content from image to PB sprite
  
  Declare CopyContent_PB_Sprite_Image_BF(source_ID, destination_ID) ; Move a content from PB sprite to image - Same size
  
  CompilerIf #PB_Compiler_Version>571
    Declare CopyContent_PB_Sprite_Canvas_BF(source_ID, destination_ID, output_x=0, output_y=0) ; Move a content from PB sprite to canvas - Same size
  CompilerElse
    Declare CopyContent_PB_Sprite_Canvas_BF(source_ID, destination_ID) ; Move a content from PB sprite to canvas - Same size
  CompilerEndIf
  
  Declare CopyContent_Canvas_PB_Sprite_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0) ; Move a content from canvas to sprite
  
  Declare CopyContent_Canvas_Canvas_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0) ; Move a content from canvas to canvas
  
  Declare CopyContent_Image_Canvas_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0) ; Move a content from image to canvas
  
  Declare CopyContent_Image_Canvas_fast_BF(source_ID, destination_ID) ; Move a content from image to canvas
  
  Declare CopyContent_Canvas_Image_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0) ; Move a content from canvas to image
  
  Declare CopyContent_Image_Screen_BF(source_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0) ; Move a content from image to screen 
  
  Declare CopyContent_Screen_Image_BF(destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0) ; Move a content from screen to image 
  
  Declare CopyContent_Screen_Canvas_BF(destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0) ; Move a content from screen to canvas 
  
  Declare CopyContent_BF(source_ID, destination_ID, output_x=0, output_y=0) ; Move a content from image to image
                                                                            ; Canvas to canvas, canvas to image, image to canvas, PB sprite to canvas
                                                                            ; Canvas to PB sprite, PB sprite to image, image to PB sprite (same sized)
                                                                            ; Look the single CopyContent functions above for output x, output_y abilities
                                                                            ; For output on screen set ActivateScreenOutput_BF(1) - then a destination_ID is not needed
  
  Declare CopyRaw_BF(source_ID, destination_ID) ; Point and plot based copy for images, sprites and canvas (same sized)
  
  Declare Create_PB_Sprite_from_Source_BF(source_ID, free_source=0, mode=0) ; Create a PB sprite from a image or a canvas
                                                                            ; mode - #PB_Sprite_PixelCollision - #PB_Sprite_AlphaBlending
                                                                            ; free_source - Delete the source after the sprite creating
  
  Declare CreateImage_from_PB_Sprite_BF(source_ID) ; Create a image from a PB sprite
  
  Declare CreateCanvas_from_PB_Sprite_BF(source_ID, x, y, width, height, flags=0) ; Create a canvas from a PB sprite
  
  Declare CreateCanvas_from_Image_BF(source_ID, x, y, width, height, flags=0) ; Create a canvas from a image
  
  Declare CreateImage_from_Canvas_BF(source_ID) ; Create a image from a canvas
  
  Declare Create_PB_Sprite_from_Canvas_BF(source_ID) ; Create a PB sprite from a canvas
  
  Declare CreateImage_from_Image_BF(source_ID, width=0, height=0) ; This function is for creating a image from a image - also resized
  
  ; Enhanced FloodFill function with texture support, color distance, clipping and automatic tiling
  ; This function can handle color distance with SetColorDistanceFill_BF -It's ever needed for removing mask artifacts
  ; As sample, Sprite rotating BF create ever little border artifacts, because using interpolation
  ; Hint : ColorDistanceFill_BF can influence and so break a used fill function clipping
  ; This is a BF core based function and can same all BF core based functions automatically clipping, tiling, scrolling
  ; All BF core based functions use automatic BF texture caching
  ; This caching is activated by default
  ; BF core based functions which can write from a to b so even write in themselves without changing the source permanently
  ; So write operations on the source do not show any changes when caching is activated
  ; If you want to change the source permanently, switch off temporary the caching before with NoCaching_BF(1)
  Declare FloodFill_BF(mode, output_ID, texture_ID,
                       x,                     ; Output coordinate x
                       y,                     ; Output coordinate y
                       texture_x=0,           ; Startposition texture output x - Preset 0
                       texture_y=0,           ; Startposition texture output y - Preset 0
                       texture_width=0,       ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                       texture_height=0,      ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                       texture_clip_x=0,      ; Startposition inside the texture x - Preset 0
                       texture_clip_y=0,      ; Startposition inside the texture y - Preset 0
                       texture_clip_width=0,  ; Endposition inside the texture - Preset 0 = full texture width
                       texture_clip_height=0, ; Endposition inside the texture - Preset 0 = full texture height
                       texture_resize_x=0,    ; Pre resize texture x
                       texture_resize_y=0)    ; Pre resize texture y
                                              ; mode = 0         - Set only a positioning marker
                                              ;       -1         - Ignore a texture and use a color - Set as texture_ID a color, as sample $FFFF00
                                              ;       -2         - Standard preset texture mode
                                              ;       -3 to -256 - Texture mode with alpha blending
                                              ; For compatibility with BucketFill this function use negative mode values, but it works at the same also with positive values
                                              ; This function can output on images, PB sprites and canvas
  
  ; This function activate transparent FloodFill
  ; It create your selected color or texture, but it make the fill output area also invisible - You must deactivate again
  Declare TransparentFloodFill_BF(transparent_flood_fill_) ; Activate with TransparentFloodFill_BF(1)
  
  ; Enhanced BucketFill function with texture support, color distance, clipping and automatic tiling
  ; This function can handle color distance with SetColorDistanceFill_BF -It's ever needed for removing mask artifacts
  ; As sample, Sprite rotating BF create ever little border artifacts, because using interpolation
  ; Hint : ColorDistanceFill_BF can influence and so break a used fill function clipping
  ; This is a BF core based function and can same all BF core based functions automatically clipping, tiling, scrolling
  ; All BF core based functions use automatic BF texture caching
  ; This caching is activated by default
  ; BF core based functions which can write from a to b so even write in themselves without changing the source permanently
  ; So write operations on the source do not show any changes when caching is activated
  ; If you want to change the source permanently, switch off temporary the caching before with NoCaching_BF(1)
  Declare BucketFill_BF(mode, output_ID, texture_ID,
                        output_get_color_x=0,  ; Grab transparence color from sprite pos x - For preset set ..x=-1  
                        output_get_color_y=0,  ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF 
                        texture_x=0,           ; Startposition texture output x - Preset 0
                        texture_y=0,           ; Startposition texture output y - Preset 0
                        texture_width=0,       ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                        texture_height=0,      ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                        texture_clip_x=0,      ; Startposition inside the texture x - Preset 0
                        texture_clip_y=0,      ; Startposition inside the texture y - Preset 0
                        texture_clip_width=0,  ; Endposition inside the texture - Preset 0 = full texture width
                        texture_clip_height=0, ; Endposition inside the texture - Preset 0 = full texture height
                        texture_resize_x=0,    ; Pre resize texture x
                        texture_resize_y=0)    ; Pre resize texture y
                                               ; mode = 0         - Set only a positioning marker
                                               ;       -1         - Ignore a texture and use a color - Set as texture_ID a color, as sample $FFFF00
                                               ;       -2         - Standard preset texture mode
                                               ;       -3 to -256 - Texture mode with alpha blending
                                               ; For compatibility with BucketFill this function use negative mode values, but it works at the same also with positive values
                                               ; This function can output on images, PB sprites and canvas
  
  ; Main (core) function
  ; This is the BF core function and can automatically clipping, tiling, scrolling
  ; The function use automatic BF texture caching
  ; This caching is activated by default
  ; The BF core function can write from a to b so even write in themselves without changing the source permanently
  ; So write operations on the source do not show any changes when caching is activated
  ; If you want to change the source permanently, switch off temporary the caching before with NoCaching_BF(1)
  Declare BF(mode, output_ID, texture_ID,
             output_get_color_x=0,  ; Get a color from this x coordinate for replacing with a texture (-1 = ignore the mask)
             output_get_color_y=0,  ; Get a color from this y coordinate for replacing with a texture (-1 = ignore the mask)
             texture_x=0,           ; Startposition texture output x - Preset 0
             texture_y=0,           ; Startposition texture output y - Preset 0
             texture_width=0,       ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
             texture_height=0,      ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
             texture_clip_x=0,      ; Startposition inside the texture x - Preset 0
             texture_clip_y=0,      ; Startposition inside the texture y - Preset 0
             texture_clip_width=0,  ; Endposition inside the texture - Preset 0 = full texture width
             texture_clip_height=0) ; Endposition inside the texture - Preset 0 = full texture height
                                    ; mode = 0 - BucketFill  - Enable the little color positioning helper marker for texture mode
                                    ;        1 = Sprite mode - Without alpha blending - Standard preset sprite mode
                                    ;        2 to 256        - Sprite mode - With alpha blending
                                    ;       -1 = BucketFill  - Ignore a texture and use a color - Set as texture_ID a color, as sample $FFFF00
                                    ;       -2 = BucketFill  - Standard preset texture mode
                                    ;       -3 to -256       - BucketFill - Texture mode with alpha blending
                                    ; BF set the parameter for transparence (alpha) so : 1 = without transparence - 256 = full transparence (BF specific)
                                    ; BF accept as input_ID and output_ID a image, a PB sprite or a canvas
                                    ; On sprite mode output_get_color_x and output_get_color_y grab the invisible color from the texture
  
  ; This is a BF core based function and can same all BF core based functions automatically clipping, tiling, scrolling
  ; All BF core based functions use automatic BF texture caching
  ; This caching is activated by default
  ; BF core based functions which can write from a to b so even write in themselves without changing the source permanently
  ; So write operations on the source do not show any changes when caching is activated
  ; If you want to change the source permanently, switch off temporary the caching before with NoCaching_BF(1)
  Declare ScreenSprite_BF(texture_ID, ; Special function for BF sprite output directly on screen
                          output_get_color_x=0,  ; Grab transparence color from sprite pos x (-1 = ignore the mask)
                          output_get_color_y=0,  ; Grab transparence color from sprite pos y (-1 = ignore the mask)
                          texture_x=0,           ; Startposition texture output x - Preset 0
                          texture_y=0,           ; Startposition texture output y - Preset 0
                          texture_width=0,       ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                          texture_height=0,      ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                          texture_clip_x=0,      ; Startposition inside the texture x - Preset 0
                          texture_clip_y=0,      ; Startposition inside the texture y - Preset 0
                          texture_clip_width=0,  ; Endposition inside the texture - Preset 0 = full texture width
                          texture_clip_height=0) ; Endposition inside the texture - Preset 0 = full texture height
                                                 ; This function can handle sprite mask cleaning with the function SetColorDistanceSpriteMask_BF
                                                 ; Also the function can texture tiling and scrolling
                                                 ; Accept as input_ID a image, a PB sprite or a canvas
  
  ; This function grab the min x, y and the max x, y coordinates from a masked sprite without mask and put the coordinates in the sprite bounded coordinate variables
  ; The function can use a input as image, PB sprite and canvas
  ; Preset a mask color is available with SetInvisibleColor_BF(color.q) (You must deactivate again)
  ; Also SetColorDistanceSpriteMask_BF is available (You must deactivate again)
  ; CanvasGadget frame make crash (without process is DPI aware) - skip with the offsets the gadget frame - Workaround Bug PB5.72 and new feature - tested on Win OS
  Declare GrabSpriteOffsets_BF(input_ID, offset_x=0, offset_y=0, offset_xx=0, offset_yy=0)
  ; Give back :
  ; GetSprite_X_BF() ; Min output coordinate Sprite 
  ; GetSprite_Y_BF()
  ; GetSprite_XX_BF() ; Max
  ; GetSprite_YY_BF()
  
  ; Function for using any images with alpha channel as sprites
  ; Use this function only for pictures with alpha channel, or you see nothing
  ; This function can output on images, PB sprites and canvas
  ; This is a BF core based function and can same all BF core based functions automatically clipping, tiling, scrolling
  ; All BF core based functions use automatic BF texture caching
  ; This caching is activated by default
  ; BF core based functions which can write from a to b so even write in themselves without changing the source permanently
  ; Also all BF core based functions can create automatic sprite tiling (texture_width, texture_height)
  ; So write operations on the source do not show any changes when caching is activated
  ; If you want to change the source permanently, switch off temporary the caching before with NoCaching_BF(1)
  Declare AlphaChannelSprite_BF(output_ID, texture_ID,
                                texture_x,
                                texture_y,
                                texture_width=0,  ; Endposition texture output  x - Preset 0 = Clipping is automatic to input width
                                texture_height=0, ; Endposition texture output  y - Preset 0 = Clipping is automatic to input height
                                texture_clip_x=0,
                                texture_clip_y=0,
                                texture_clip_width=0,
                                texture_clip_height=0)
  
  ; Same above, but with integrated output resizing
  ; BF core based function
  Declare AlphaChannelSprite_Pro_BF(output_ID, texture_ID,
                                    texture_x,
                                    texture_y,
                                    texture_width=0,  ; Endposition texture output  x - Preset 0 = Clipping is automatic to input width
                                    texture_height=0, ; Endposition texture output  y - Preset 0 = Clipping is automatic to input height
                                    output_width=0,   ; You can here resize the output x
                                    output_height=0,  ; You can here resize the output y
                                    texture_clip_x=0,
                                    texture_clip_y=0,
                                    texture_clip_width=0,
                                    texture_clip_height=0)
  
  ; Same above, but with rotate and resize feature
  ; This function can output images with transparence also directly on screen
  ; BF core based function - can tiling
  ; With the function "ActivateScreenOutput_BF" you can activate the screen output
  Declare RotateAlphaChannelSprite_BF(output_ID, image_ID,                                                                         
                                      texture_x,             ; Output pos x
                                      texture_y,             ; Output pos y
                                      degree=0,              ; Rotating degree
                                      mode=0,                ; mode=1 rotating on - mode=2 rotating on and create a quadratic sprite for on demand rotation - Mode 1 has no effekt on linux
                                      alpha=0,               ; Set alpha blending from 1 to 255 - BF set 1=full visible - 256=invisible (BF specific)
                                      texture_width=0,       ; Endposition texture output x - Preset 0 = Clipping is automatic to input width
                                      texture_height=0,      ; Endposition texture output y - Preset 0 = Clipping is automatic to input height
                                      output_width=0,        ; You can here resize the output x
                                      output_height=0,       ; You can here resize the output y
                                      texture_clip_x=0,      ; Startposition inside the texture x - Preset 0 (Texture clipping)
                                      texture_clip_y=0,      ; Startposition inside the texture y - Preset 0 (Texture clipping)
                                      texture_clip_width=0,  ; Endposition inside the texture x - Preset 0 = full texture width (Texture clipping)
                                      texture_clip_height=0, ; Endposition inside the texture y - Preset 0 = full texture height (Texture clipping)
                                      rotating_offset_x=0,   ; Rotating offset x - For mode 1 - Do not make offsets too large - Larger=slower, but I will give you not a limit
                                      rotating_offset_y=0)   ; Rotating offset y - You can also set negative values for the rotating offsets - As sample x=50 - y=-50 - This works very easy and simple  
  
  ; Fast, not BF core based, with rotate and resize feature
  ; This function can output images with transparence also directly on screen
  ; With the function "ActivateScreenOutput_BF" you can activate the screen output
  Declare RotateAlphaChannelSprite_fast_BF(output_ID, image_ID,                                                                         
                                           texture_x,             ; Output pos x
                                           texture_y,             ; Output pos y
                                           degree=0,              ; Rotating degree
                                           mode=0,                ; mode=1 rotating on - mode=2 rotating on and create a quadratic sprite for on demand rotation - Mode 1 has no effekt on linux
                                           alpha=0,               ; Set alpha blending from 1 to 255 - BF set 1=full visible - 256=invisible (BF specific)                               
                                           output_width=0,        ; You can here resize the output x
                                           output_height=0,       ; You can here resize the output y
                                           texture_clip_x=0,      ; Startposition inside the texture x - Preset 0 (Texture clipping)
                                           texture_clip_y=0,      ; Startposition inside the texture y - Preset 0 (Texture clipping)
                                           texture_clip_width=0,  ; Endposition inside the texture x - Preset 0 = full texture width (Texture clipping)
                                           texture_clip_height=0, ; Endposition inside the texture y - Preset 0 = full texture height (Texture clipping)
                                           rotating_offset_x=0,   ; Rotating offset x - For mode 1 - Do not make offsets too large - Larger=slower, but I will give you not a limit
                                           rotating_offset_y=0)   ; Rotating offset y - You can also set negative values for the rotating offsets - As sample x=50 - y=-50 - This works very easy and simple  
  
  
  ; Function for simple using any images as sprites
  ; This function can output on images, PB sprites, canvas
  ; This is a BF core based function and can same all BF core based functions automatically clipping, tiling, scrolling
  ; All BF core based functions use automatic BF texture caching
  ; This caching is activated by default
  ; BF core based functions which can write from a to b so even write in themselves without changing the source permanently
  ; So write operations on the source do not show any changes when caching is activated
  ; If you want to change the source permanently, switch off temporary the caching before with NoCaching_BF(1)
  ; For screen output set firstly ActivateScreenOutput_BF(1)
  Declare SpriteSimple_BF(mode, output_ID, texture_ID,
                          image_get_color_x,     ; Grab transparence color from sprite pos x - For preset set ..x=-1  
                          image_get_color_y,     ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF 
                          output_pos_x,          ; Output pos x
                          output_pos_y,          ; Output pos y
                          output_width=0,        ; You can here resize the output x
                          output_height=0,       ; You can here resize the output y
                          alpha=1,               ; Set alpha blending from 2 to 256 - BF set 1=full visible - 256=invisible (BF specific)
                          texture_width=0,       ; Endposition texture output       - Preset 0 = Clipping is automatic to input width
                          texture_height=0,      ; Endposition texture output       - Preset 0 = Clipping is automatic to input height
                          texture_clip_x=0,      ; Startposition inside the texture - Preset 0 = The output starts with the texture x coordinate 0
                          texture_clip_y=0,      ; Startposition inside the texture - Preset 0 = The output starts with the texture y coordinate 0
                          texture_clip_width=0,  ; Endposition inside the texture   - Preset 0 = full texture width
                          texture_clip_height=0) ; Endposition inside the texture   - Preset 0 = full texture height
                                                 ; You use clipping, "image_get_color.." get the invisible color from the clipped texture
                                                 ; Mode 0 deactivate the invisible color - mode 1 activate the invisible color
                                                 ; For using pictures with alpha channel use mode=2
                                                 ; For using JPG based masked sprites, pre use SetColorDistanceSpriteMask_BF(percent) 
                                                 ; Resizing can change the colors a little,
                                                 ; so it is available a presetted invisible color works after resizing not longer correct
  
  ; Function for using any sprite sheets as sprites - BF Core based
  ; This function can output on images, PB sprites, canvas
  ; For screen output set firstly ActivateScreenOutput_BF(1)
  Declare Sprite_CSS_sheet_BF(mode, output_ID, texture_ID, 
                              image_get_color_x,     ; Grab transparence color from sprite pos x - For preset set ..x=-1                         
                              image_get_color_y,     ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF  
                              output_pos_x,          ; Output position x
                              output_pos_y,          ; Output position y
                              frames,                ; Numbers of frames
                              frame,                 ; Selected frame - For GIF compatibility the first frame is frame 0
                              frame_width,           ; Frame width
                              frame_height,          ; Frame height
                              frames_in_a_row=0,     ; Frames in a row - Preset 0, then BF calculate self
                              output_width=0,        ; Resizing - Define the output width  - Preset zero = without resizing - Presetted deactivated with  0
                              output_height=0,       ; Resizing - Define the output height - Presetted deactivated with  0
                              texture_width=0,       ; Endposition texture output       - Preset 0 = Clipping is automatic to output width 
                              texture_height=0,      ; Endposition texture output       - Preset 0 = Clipping is automatic to output height 
                              texture_clip_x=0,      ; Startposition inside the texture - Preset 0 = The output starts with the texture x coordinate 0
                              texture_clip_y=0,      ; Startposition inside the texture - Preset 0 = The output starts with the texture y coordinate 0
                              texture_clip_width=0,  ; Endposition inside the texture   - Preset 0 = full texture width
                              texture_clip_height=0) ; Endposition inside the texture   - Preset 0 = full texture height
                                                     ; mode 0 = Without invisible color
                                                     ; mode 1 > 256 = With invisible color and with alpha blending
                                                     ; mode -1 > -256 = Without invisible color and with alpha blending
                                                     ; Set alpha for alpha blending from 1 > 256 - 1=full visible - 256=invisible (BF specific)
                                                     ; You use clipping, "image_get_color.." get the invisible color from the clipped texture
                                                     ; Resizing can change the colors a little,
                                                     ; so it is available a presetted invisible color works after resizing not longer correct
  
  ; Function for simple using any sprite sheets as sprites - Fast output
  ; This function can output on images, PB sprites and canvas
  ; For screen output set firstly ActivateScreenOutput_BF(1)
  Declare Sprite_CSS_sheet_simple_BF(mode, output_ID, texture_ID,
                                     image_get_color_x,     ; Grab transparence color from sprite pos x - For preset set ..x=-1                         
                                     image_get_color_y,     ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF  
                                     output_pos_x,          ; Output position x
                                     output_pos_y,          ; Output position y
                                     frames,                ; Numbers of frames
                                     frame,                 ; Selected frame - For GIF compatibility the first frame is 0
                                     frame_width,           ; Frame width
                                     frame_height,          ; Frame height
                                     frames_in_a_row=0,     ; Frames in a row - Preset 0, then BF calculate self
                                     output_width=0,        ; Resizing - Define the output width - Preset zero = without resizing 
                                     output_height=0,       ; Resizing - Define the output height
                                     texture_clip_x=0,      ; Startposition inside the texture - Preset 0
                                     texture_clip_y=0,      ; Startposition inside the texture - Preset 0
                                     texture_clip_width=0,  ; Endposition inside the texture - Preset 0 = full texture width
                                     texture_clip_height=0) ; Endposition inside the texture - Preset 0 = full texture height
                                                            ; mode 0 = Without invisible color
                                                            ; mode 1 > 256 = With invisible color and with alpha blending
                                                            ; mode -1 > -256 = Without invisible color and with alpha blending
                                                            ; This is a special function for very fast CSS sheet output- mode 0 is the fastest mode - Image output is a little faster as canvas output
                                                            ; The function is not based on the GFX_Wizzard_BF core function, can so not output multiple pattern and works not with the BF sprite array
                                                            ; On PB sprites invisible color and alpha blending is ignored
  
  ; This function can output on images, PB sprites and canvas
  ; For screen output set firstly ActivateScreenOutput_BF(1)
  Declare SpriteSimple_fast_BF(mode, output_ID, texture_ID,
                               image_get_color_x,     ; Grab transparence color from sprite pos x - For preset set ..x=-1                         
                               image_get_color_y,     ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF  
                               output_pos_x,          ; Output position x
                               output_pos_y,          ; Output position y
                               output_width=0,        ; Resizing - Define the output width - Preset zero = without resizing 
                               output_height=0,       ; Resizing - Define the output height
                               texture_clip_x=0,      ; Startposition inside the texture - Preset 0
                               texture_clip_y=0,      ; Startposition inside the texture - Preset 0
                               texture_clip_width=0,  ; Endposition inside the texture - Preset 0 = full texture width
                               texture_clip_height=0) ; Endposition inside the texture - Preset 0 = full texture height
                                                      ; mode 0 = Without invisible color
                                                      ; mode 1 > 256 = With invisible color and with alpha blending
                                                      ; mode -1 > -256 = Without invisible color and with alpha blending
                                                      ; This is a special function for very fast sprite or GIF output- mode 0 is the fastest mode - Image output is a little faster as canvas output
                                                      ; The function is not based on the GFX_Wizzard_BF core function, can so not output multiple pattern and works not with the BF sprite array
                                                      ; On PB sprites invisible color and alpha blending is ignored
  
  Declare CreateSprite_from_AlphaImage_BF(image_ID, ; Create a sprite from a image with alpha channel (PNG) or a GIF
                                          mask_color=0, ; Set here a suitable color for the sprite mask (Different from the picture content color)
                                          percent_color_distance.f=0, ; You can set a color distance for the mask color in percent
                                          delete_old_image=0)         ; Set delete_old_image <>0 for deleting the source image after creating a new image
  
  Declare CreateAlphaImage_from_Sprite_BF(image_ID,
                                          percent_color_distance.f=0, ; You can set a color distance for the mask color in percent
                                          invisible_mask_color=-1,    ; The hidden color from the mask - -1 grabb the sprite mask color from sprite pos x=0 / y=0 and set the resulted color to 0
                                          alpha=0,                    ; Alpha blending - 1 is full visible - 256 invisible (BF specific)
                                          delete_old_image=0)         ; Set delete_old_image <>0 for deleting the source image after creating a new image 
  
  ; Create a image with transparence from a masked sprite
  ; This function grab the invisible color from Position 0, 0
  ; Color distance for cleaning the sprite mask available - As sample for jpg based sprites (images) 
  Global percent_create_alpha_image_from_sprite
  Declare Callback_CreateAlphaImage_from_Sprite_Fast(x, y, source_color, destination_color)
  Declare CreateAlphaImage_from_Sprite_Fast_BF(image_ID, color_distance)
  
  ; This function can output on images, PB sprites
  Declare TextureBrush_image_BF(mode, output_ID, texture_ID, 
                                x,                        ; Output coordinates
                                y,                        ;
                                texture_width=0,          ; Preset 0 = Full available width
                                texture_height=0,         ; Preset 0 = Full available width 
                                percent_visibility.f=100) ;
                                                          ; mode=-1 and 0  - Without seamless embedding - Fast
                                                          ; mode=0         - Without seamless embedding - Fast (mode -1 and mode 0 equal for parameter compatibility to BF canvas)
                                                          ; mode=1 to 3    - With seamless embedding
                                                          ; percent_visibility - Visibility in percent
  
  Declare TextureBrush_canvas_BF(mode, canvas_ID, texture_ID,
                                 x,                        ; Output coordinates
                                 y,                        ;
                                 texture_width=0,          ; Preset 0 = Full available width
                                 texture_height=0,         ; Preset 0 = Full available width
                                 percent_visibility.f=100, ;
                                 delay=0)                  ;
                                                           ; mode=-1            - Without seamless embedding - Delay not available - Fast
                                                           ; mode=0             - Without seamless embedding - Delay available - As sample for dia shows
                                                           ; mode=1 to 3        - With seamless embedding    - Delay available
                                                           ; mode=4             - Quick retouch mode         - Delay not available - Special seamless mode for picture retouching and embedding
                                                           ; percent_visibility - Visibility in percent
                                                           ; delay              - Delay for animation - ms
                                                           ; A setted delay fire WindowEvent calls for refreshing the internal created seamless animation immediately,
                                                           ; so you get not all actually window events directly back after a animation output !
                                                           ; Retouching rectangle size x, y is 7 - This can plot one 100% visible point with a little halo
  
  Declare RotateImage_BF(degree, image_ID, create_new_image=0) ; Set this parameter for creating a new rotated image
                                                               ; degree = -90 (270) - 90 - 180
                                                               ; ActivateAlphaChannel_BF for creating a new rotated image with alpha channel available
  
  Declare Create_empty_rotated_image_BF(width, height, degree, color=0) ; Create a empty free rotated image
                                                                        ; ActivateAlphaChannel_BF for creating a new rotated image with alpha channel available
  
  Declare Calculate_empty_rotated_rectangle_BF(width, height, degree) ; Calculate a rotated rectangle
  
  Declare Get_resulted_rotated_width() ; Get the resulted width for Calculate_empty_rotated_rectangle_BF and Create_empty_rotated_image_BF
  
  Declare Get_resulted_rotated_height() ; Get the resulted height for Calculate_empty_rotated_rectangle_BF and Create_empty_rotated_image_BF
  
  ; Free rotating a sprite - Base function RAW
  ; This main rotating function of the wizzard can also mirror, activate simple and you can use with many functions
  ; Activate_Mirroring_Horizontal_Rotate_BF(1) - Activate_Mirroring_Vertical_Rotate_BF(1) _ You must deactivate again
  Declare RotateSprite_BF(texture_ID, 
                          image_get_color_x=0, ; Grab transparence color from sprite pos x - For preset set ..x=-1 - x=-2 ignore the invisible color and set image_get_color_y as mask color
                          image_get_color_y=0, ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF
                          degree=0,            ; Rotating degree - The sprite rotates around its center point - With the rotating_offsets you can relocate this point
                          mode=0,              ; mode=0 create a exactly resized sprite with minimized mask - mode=1 create a quadratic sprite for on demand rotation
                          rotating_offset_x=0, ; Rotating offset x - For mode 1 - Do not make offsets too large - Larger = slower, but I will give you not a limit
                          rotating_offset_y=0, ; Rotating offset y - You can also set negative values for the rotating offsets - As sample x=50 - y=-50 - This works very easy and simple
                          create_new_image=0)  ; Set this parameter for immediately creating a new rotated image - This is recommended for on demand rotation
                                               ; After using create_new_image you must give free again with FreeImage or you get a memory leak - Then the function give back the new created image ID
                                               ; The function can create and handle sprite masks automatically or manual, from textures, images or photos
                                               ; So you can also simple create masked rotated sprites from all images without a included mask
                                               ; You see mask artifacts, set SetColorDistanceSpriteMask_BF - For jpg (lossly) compressed sprites this is ever needed - Try ~ 25 > 35 percent 
                                               ; - Hints for rotating sprites with this base function -
                                               ; BF rotated sprites are rendered with interpolation to a special rotating mask
                                               ; At the border of the two masks and at the sprite itself, mask artifacts are created which can impair the mask function (jerking, jumping, trembling)
                                               ; This must be treated separately, with the mask cleaning function SetColorDistanceSpriteMask_BF
                                               ; With functions that support clipping you can clip herewith the mask to the real sprite size
                                               ; Usually as described here
                                               ; The rotating function activates the required sprite Array automatically
                                               ; This function grab the min x,y and the max x,y coordinates from a masked sprite, without mask and put the coordinates in the sprite real coordinate variables
                                               ; This function can output on images, PB sprites and canvas
                                               ; GetSprite_X_BF()-3 ; Min output coordinate Sprite (The little offsets you should use for enlarge the mask a little)
                                               ; GetSprite_Y_BF()-3
                                               ; GetSprite_XX_BF()+6 ; Max
                                               ; GetSprite_YY_BF()+6
                                               ; As global variables : min_x_2, max_x_2, min_y_2, max_y_2 - Color distance sprite mask = percent.f
                                               ; A texture with invisible color must have one visible pixel, it doesn't work with textures that consist only of invisible color
  
  ; For screen output set firstly ActivateScreenOutput_BF(1)
  Declare RotateSprite_simple_BF(mode, output_ID, texture_ID,                              
                                 image_get_color_x=0, ; Grab transparence color from sprite pos x - For preset set ..x=-1 - x=-2 ignore the invisible color and set image_get_color_y as mask color
                                 image_get_color_y=0, ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF
                                 output_pos_x=0,      ; Output pos x
                                 output_pos_y=0,      ; Output pos y
                                 degree=0,            ; Rotating degree - On demand
                                 alpha=1,             ; Set alpha blending from 2 to 256 - BF set 1=full visible - 256=invisible (BF specific) - On demand
                                 output_width=0,      ; You can here resize the output x - On demand
                                 output_height=0,     ; You can here resize the output y - On demand
                                 rotating_offset_x=0, ; Rotating offset x - For mode 1 - Do not make offsets too large - Larger=slower, but I will give you not a limit
                                 rotating_offset_y=0) ; Rotating offset y - You can also set negative values for the rotating offsets - As sample x=50 - y=-50 - This works very easy and simple  
                                                      ; Set alpha -1 to -256 for a output without invisible color
                                                      ; mode=0 Deactivate rotating
                                                      ; mode=1 create a exactly resized sprite with minimized mask - Centered with offsets
                                                      ; mode=2 create a exactly resized sprite with minimized mask - Centered
                                                      ; mode=3 create a exactly resized sprite with minimized mask - Left-aligned
                                                      ; The function can create and handle sprite masks automatically or manual, from textures, images or photos
                                                      ; So you can also simple create masked rotated sprites from all images without a included mask
                                                      ; You see mask artifacts, set SetColorDistanceSpriteMask_BF - For jpg (lossly) compressed sprites this is ever needed - Try ~ 25 > 35 percent 
                                                      ; For images set image_get_color_x=-1 and image_get_color_y to a unused image color, best similar the background color
                                                      ; Reduce for Images SetColorDistanceSpriteMask_BF or you can get "holes", but also nice effects   
                                                      ; For mode 2 and 3 a texture with invisible color must have one visible pixel, it doesn't work with textures that consist only of invisible color
  
  Declare CreateSprite_simple_BF(type,                  ; type=0 = BF sprite - type=1 = PB sprite               
                                 input_ID,              ; Accept as input images, PB sprites, canvas 
                                 image_get_color_x=0,   ; Grab transparence color from source pos x - For preset set ..x=-1 - x=-2 ignore the invisible color and set image_get_color_y as mask color
                                 image_get_color_y=0,   ; Grab transparence color from source pos y - For preset set ..y= the color you want - as sample $FF00FF
                                 degree=0,              ; Rotating degree - The sprite rotates around its center point - With the rotating_offsets you can relocate this point
                                 mode=0,                ; mode=0 deactivate rotating - mode=1 create a quadratic sprite for on demand rotation - mode=2 create a exactly resized sprite with minimized mask
                                 output_width=0,        ; You can here resize the output x - On demand
                                 output_height=0,       ; You can here resize the output y - On demand
                                 rotating_offset_x=0,   ; Rotating offset x - For mode 1 - Do not make offsets too large - Larger = slower, but I will give you not a limit
                                 rotating_offset_y=0,   ; Rotating offset y - You can also set negative values for the rotating offsets - As sample x=50 - y=-50 - This works very easy and simple
                                 mask_color_distance=0, ; Color distance sprite mask - Mask cleaning - Important                                  
                                 delete_input=0,        ; Delete the input
                                 mirror=0,              ; Mirroring - 0=without - 1=horizontal - 2=vertical - 3=horizontal and vertical
                                 flags=0)               ; Flags for CreateSprite - same PB function call
                                                        ; The function accepts as input a image, a PB sprite or a canvas (ID)
  
  ; Activate the BF rotating function Mirroring horizontal - Fast - You must deactivate again
  ; The main rotating function of the wizzard can also mirror, activate simple and you can use with many functions
  Declare Activate_Mirroring_Horizontal_Rotate_BF(activate_mirroring_horizontal_on_rotate_)
  
  ; Activate the BF rotating function Mirroring horizontal - Fast - You must deactivate again
  ; The main rotating function of the wizzard can also mirror, activate simple and you can use with many functions
  Declare Activate_Mirroring_Vertical_Rotate_BF(activate_mirroring_vertical_on_rotate_)
  
  ; Mirroring horizontal on Images, PB Sprites and canvas - Fast
  Declare Mirroring_Horizontal_BF(input_ID)
  
  ; Mirroring vertical on Images, PB Sprites and canvas - Fast
  Declare Mirroring_Vertical_BF(input_ID)
  
  ; Mirroring horizontal on all channels - You must set self StartDrawing() - StopDrawing() - Fast
  Declare Mirroring_Horizontal_RAW_BF(input_ID)
  
  ; Mirroring vertical on all channels - You must set self StartDrawing() - StopDrawing() - Fast
  Declare Mirroring_Vertical_RAW_BF(input_ID)
  
  ; This function can output on images, PB sprites and canvas
  Declare Mirror_and_Copy_BF(mode, input_ID, create_new_output=0, flags=0, canvas_x=0, canvas_y=0)
  ; create_new_output=1 - Create a image
  ; create_new_output=2 - Create a sprite
  ; create_new_output=3 - Create a canvas
  ; mode=1 = horizontal mirroring
  ; mode=2 = vertical mirroring
  ; mode=3 Create a new image, sprite or canvas without mirroring
  ; modus - Flags for CreateSprite or CanvasGadget - same PB function call
  ; canvas_x = Output x pos for canvas
  ; canvas_y = Output y pos for canvas
  
  ; This function can output on images, PB sprites and canvas
  Declare Texture_universal_output_BF(mode, output_ID, texture_ID, temporary_color,
                                      texture_x=0,       ; Startposition texture output x   - Preset 0
                                      texture_y=0,       ; Startposition texture output y   - Preset 0
                                      texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                                      texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                                      texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                                      texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                                      texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                                      texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
                                                         ; mode= 1 - Standard preset texture mode
                                                         ; mode= 2 > 256 - Texture mode with alpha blending - 2=full visible - 256=invisible (BF specific)
                                                         ; mode=-1   Ignore a texture and use a color - Set as texture_ID a color, as sample $FFFF00 
                                                         ; mode=-2 > -256 - Ignore a texture and use a color - With alpha blending 2=full visible - 256=invisible (BF specific)
                                                         ; temporary_color = The function replace this color with a texture or a other color
                                                         ; A temporary image is additional needed - Look the demo codes
                                                         ; The function Texture_universal_output_BF create automatic a sprite mask
                                                         ; With this function you can preset a sprite mask color
                                                         ; This is only needed for vector graphic functions - For prevent ugly borders
                                                         ; You must deactivate again
                                                         ; Hint : A function call with a preseted mask color is faster and primary preferable
  
  Declare CreateImageUniversalOutput_BF(output_ID, temporary_color) ; This function create a temporary image for the Texture_universal_output_BF function
  
  Declare SetMaskColorUniversalOutput_BF(mask_color_universal_output_.q) ; This is a quad - It is presetted deactivated with -1 - Set it for white so $FFFFFF
  
  Declare GetUniversalOutputMaskColor_BF() ; Get a setted _UniversalOutputMaskColor
  
  Declare Create_animation_buffers_BF(output_ID, buffers_number) ; Create buffers for the animation - The output can are a image, a canvas or a PB sprite
                                                                 ; Set for different output targets ever a other buffers_number - 100 buffers available - 0 to 99
  
  Declare FlipBuffers_BF(buffers_number) ; Flip the animation buffers
                                         ; Set for different output targets ever a other buffers_number - 100 buffers available - 0 to 99
                                         ; This works similar the PB FlipBuffers function, but it is more flexible
                                         ; You have three separately buffers - you can read and write ever each buffer
  
  Declare Free_animation_buffers_BF(buffers_number) ; Free animation buffers
                                                    ; Set for different output targets ever a other buffers_number - 100 buffers available - 0 to 99
  
  Declare Get_animation_buffer_background_image_ID_BF(buffers_number) ; Get the animation buffer image ID for the background
                                                                      ; Set for different output targets ever a other buffers_number - 100 buffers available - 0 to 99
  
  Declare Get_animation_buffer_hidden_image_ID_BF(buffers_number) ; Get the animation buffer image ID for the temporary hidden drawing (Sprite Buffer)
                                                                  ; Set for different output targets ever a other buffers_number - 100 buffers available - 0 to 99
  
  Declare NoCaching_BF(caching) ; Deactivate texture caching - You must activate again - Activated is presetted - This is a global function
                                ; Works with all BF core function based functions - "Fast output" functions are not BF core based
  
  Declare Get_caching_state_BF() ; Check is BF caching on or off
  
  Declare SetPosMarker_BF(output_ID, x, y) ; Set a positioning help marker - For images, sprites and canvas
  
  CompilerIf #PB_Compiler_Version=>560
    Declare Create_invisible_GIF_color_BF(gif_ID) ; This function create automatic a new invisible color for a defined GIF - This change not the GIF self
                                                  ; A created invisible color is never black or white
  CompilerEndIf
  
  Declare CleanMask_BF(input_ID, get_mask_color_x_pos, get_mask_color_y_pos, percent_color_distance) ; Mask cleaning function - Works with images, PB sprites and canvas
  
  Declare CleanMask_simple_BF(input_ID, mask_color, percent_color_distance) ; Mask cleaning function - Works with images, PB sprites and canvas
  
  Declare ResizeInterpolation_BF(resize_interpolation_) ; 1 = activated - Activate interpolation for any resizing - deactivated = 0 - Preset = activated
                                                        ; Interpolation mask artifacts you can simple remove with the color distance function
  
  Declare Get_all_Window_Events_BF(get_all_window_events_) ; Activate for fixing output problems with complex text and textboxes on MacOS 
                                                           ; For using inside a callback procedure deactivate
  
  ; RotateRoundBox - Vector lib based
  ; Output automatically on image, canvas, window
  ; You can set frame_color and background_color as RGBA, but you must not - Alpha 0 is ignored and set a full visible color
  Declare RotateRoundBox_Vector_BF(output_ID,
                                   x.d,
                                   y.d,
                                   width.d,
                                   height.d,
                                   degree,            ; Rotating angle
                                   radius,            ; Rounding radius
                                   frame_thickness.d, ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                                   frame_color,       ; Frame color
                                   background_color)  ; Background color - For invisible background set to -1
  
  ; RoundBox - Vector lib based
  ; StartVectorDrawing() and StopVectorDrawing() needed
  ; You can set frame_color and background_color as RGBA, but you must not - Alpha 0 is ignored and set a full visible color
  Declare RoundBox_Vector_RAW_BF(output_ID,
                                 x.d,
                                 y.d,
                                 width.d,
                                 height.d,
                                 radius.d,          ; Rounding radius
                                 frame_thickness.d, ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                                 frame_color,       ; Frame color
                                 background_color)  ; Background color - For invisible background set to -1
  
  ; RoundBox - Vector lib based
  ; Output automatically on image, canvas, window
  ; You can set frame_color and background_color as RGBA, but you must not - Alpha 0 is ignored and set a full visible color
  Declare RoundBox_Vector_BF(output_ID,
                             x.d,
                             y.d,
                             width.d,
                             height.d,
                             radius.d,          ; Rounding radius
                             frame_thickness.d, ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                             frame_color,       ; Frame color
                             background_color)  ; Background color - For invisible background set to -1
  
  ; Padding X for Vector_Roundbox_Simple functions
  Declare Vector_Roundbox_Simple_Padding_X(vector_roundbox_simple_padding_x_=0)
  
  ; Padding Y for Vector_Roundbox_Simple functions
  Declare Vector_Roundbox_Simple_Padding_Y(vector_roundbox_simple_padding_y_=0)
  
  ; Get setted Padding X for Vector_Roundbox_Simple functions
  Declare Get_Vector_Roundbox_Simple_Padding_X()
  
  ; Get setted Padding Y for Vector_Roundbox_Simple functions
  Declare Get_Vector_Roundbox_Simple_Padding_Y()
  
  ; VectorText - DrawVectorParagraph enhanced - DPI aware - Direct output on image, canvas, sprite, window - Author (C) W. Albus
  ; After call, this function give back the rectangle coordinates and its width and height for post processing
  ; Get_Rectangle_x_BF()
  ; Get_Rectangle_y_BF()
  ; Get_Rectangle_width_BF()
  ; Get_Rectangle_height_BF() 
  ; For get only the rectangle coordinates set simple the text and background alpha = 0 - Or set output_ID = -1
  ; - So you can as sample pre fill the rectangle with what ever you want
  ; - Also you can use a post fill with BucketFill_BF, this create border effects to the text depending on the background color used
  ; - The BF Fill functions are also SetColorDistanceFill_BF() sensitive to influence border effects and other how ever you want
  ; output_width, output_height is on DPIaware internal multiplied by the DPI_ResizeFactor_BF()
  ; - Max is so 3e4/DPI_ResizeFactor_BF()
  ; Check this or the text will be cut off
  Declare VectorText_Simple_RoundBox_BF(text$,
                                        output_ID,
                                        output_x_pos,
                                        output_y_pos,
                                        font=-1,         ; -1 = Automatic - System Font
                                        output_width=0,  ;  0 = automatic
                                        output_height=0, ;  0 = automatic
                                        text_color=#Black,
                                        background_color=#White,
                                        text_alpha=255,
                                        background_alpha=255,
                                        flag=#PB_VectorParagraph_Left,
                                        frame_thickness=0,
                                        frame_color=-1,  ; -1 = Automatic - Same Text color
                                        frame_alpha=255,
                                        degree=0,
                                        radius=8)
  
  ; Same the function above
  Declare VectorText_Simple_RoundBox_small_BF(text$,                         ; Text
                                              output_ID,                     ; Output destination
                                              output_x_pos,                  ; Output x pos
                                              output_y_pos,                  ; Output y pos
                                              font=-1,                       ; -1 = Automatic - System Font
                                              text_color=#Black,             ; Text color
                                              background_color=#White,       ; Background color
                                              background_alpha=255,          ; Background alpha PB specific
                                              frame_thickness=0,             ; Frame thickness
                                              flag=#PB_VectorParagraph_Left, ; Flag
                                              radius=8)                      ; Edge rounding
  
  ; VectorText - DrawVectorParagraph enhanced - DPI aware - Direct output on image, canvas, sprite, window - Author (C) W. Albus
  ; After call, this function give back the rectangle coordinates and its width and height for post processing
  ; Get_Rectangle_x_BF()
  ; Get_Rectangle_y_BF()
  ; Get_Rectangle_width_BF()
  ; Get_Rectangle_height_BF() 
  ; For get only the rectangle coordinates set simple the text and background alpha = 0 - Or set output_ID = -1
  ; - So you can as sample pre fill the rectangle with what ever you want
  ; - Also you can use a post fill with BucketFill_BF, this create border effects to the text depending on the background color used
  ; - The BF Fill functions are also SetColorDistanceFill_BF() sensitive to influence border effects and other how ever you want
  ; output_width, output_height is on DPIaware internal multiplied by the DPI_ResizeFactor_BF()
  ; - Max is so 3e4/DPI_ResizeFactor_BF()
  ; Check this or the text will be cut off
  ; This function is GradientColor BF sensitive - (LinearGradient and CircularGradient)
  Declare VectorText_Simple_BF(text$,
                               output_ID,
                               output_x_pos,
                               output_y_pos,
                               font=-1,         ; -1 = Automatic - System Font
                               output_width=0,  ;  0 = automatic
                               output_height=0, ;  0 = automatic
                               text_color=#Black,
                               background_color=#White,
                               text_alpha=255,
                               background_alpha=255,
                               flag=#PB_VectorParagraph_Left,
                               frame_thickness=0,
                               frame_color=-1,  ; -1 = Automatic - Same Text color
                               frame_alpha=255,
                               degree=0)
  
  ; Same the function above
  Declare VectorText_Simple_small_BF(text$,                         ; Text
                                     output_ID,                     ; Output destination
                                     output_x_pos,                  ; Output x pos
                                     output_y_pos,                  ; Output y pos
                                     font=-1,                       ; -1 = Automatic - System Font
                                     text_color=#Black,             ; Text color
                                     background_color=#White,       ; Background color
                                     background_alpha=255,          ; Background alpha PB specific
                                     frame_thickness=0,             ; Frame thickness
                                     flag=#PB_VectorParagraph_Left) ; Flag
  
  ; Same the function above
  ; -but With minimized background
  ; -best for simple standard output
  Declare VectorText_BF(text$,                         ; Text
                        output_ID,                     ; Output destination
                        output_x_pos,                  ; Output x pos
                        output_y_pos,                  ; Output y pos
                        font=-1,                       ; -1 = Automatic - System Font
                        text_color=#Black,             ; Text color
                        background_color=#White,       ; Background color
                        background_alpha=255,          ; Background alpha PB specific
                        flag=#PB_VectorParagraph_Left) ; Flag
  
  ; Same the VectorText_Simple_BF function above, but it create a 32 Bit image with invisible background for any output channels
  ; As sample - The Vector lib can not output directly on screen, with this function you can
  Declare VectorText_CreateImage_RoundBox_BF(text$,
                                             font=-1,         ; -1 = Automatic - System Font
                                             output_width=0,  ;  0 = automatic
                                             output_height=0, ;  0 = automatic
                                             text_color=#White,
                                             background_color=#Black,
                                             text_alpha=255,
                                             background_alpha=255,
                                             flag=#PB_VectorParagraph_Left,
                                             frame_thickness=0,
                                             frame_color=-1,  ; -1 = Automatic - Same Text color
                                             frame_alpha=255,
                                             radius=8)
  
  ; Same the VectorText_Simple_BF function above, but it create a 32 Bit image with invisible background for any output channels
  ; As sample - The Vector lib can not output directly on screen, with this function you can
  Declare VectorText_CreateImage_BF(text$,
                                    font=-1,         ; -1 = Automatic - System Font
                                    output_width=0,  ;  0 = automatic
                                    output_height=0, ;  0 = automatic
                                    text_color=#White,
                                    background_color=#Black,
                                    text_alpha=255,
                                    background_alpha=255,
                                    flag=#PB_VectorParagraph_Left,
                                    frame_thickness=0,
                                    frame_color=-1, ; -1 = Automatic - Same Text color
                                    frame_alpha=255)
  
  ; Mono line output give back the xx output coordinate - Multi line output give back the yy output coordinate
  ; For stretched text on a textured background often DrawingMode (#PB_2DDrawing_Transparent) is needed
  ; You can set the text color (f_col) as RGBA, but you must not - Alpha 0 is ignored and set a full visible color
  ; This function is GradientColor BF sensitive - (LinearGradient and CircularGradient)
  ; A Highlight is the text stretching (Headline) and scaling feature
  ; With scaling you can simple change the text size without changing a font, so you need only one font for any text sizes
  ; StartVectorDrawing needed
  Declare DrawText_Vector_BF(txt_x,                    ; Output x pos
                             txt_y,                    ; Output y pos
                             txt$,                     ; Text
                             f_col=#Black,             ; Text color
                             flag=0,                   ; Text center=1 - Text right=2 - Text vertical=3
                             mode=0,                   ; Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
                             stretch_x.d=0,            ; Text stretching x - Headline mode
                             stretch_y.d=0,            ; Text stretching y - Headline mode
                             scale_coordinates_x.d=1,  ; Text scaling x
                             scale_coordinates_y.d=1)  ; Text scaling y
  
  ; You need only one font for any text sizes
  ; StartVectorDrawing needed
  Declare Fitted_DrawText_Vector_BF(text$,                       ; Text - Multiline supported
                                    x_pos,                       ; Output x pos
                                    y_pos,                       ; Output y pos
                                    output_width,                ; Output width
                                    output_height,               ; output height
                                    text_color=#Black,           ; Alpha channel available - Sample $FFAAAAAA - $FF is the alpha channel 
                                    text_stretching_x=0,         ; Text stretching x - Headline mode
                                    text_stretching_y=0,         ; Text stretching x - Headline mode
                                    padding_factor_x.d=0,        ; Padding x - This is a virtual padding for resizing and boxed output
                                    padding_factor_y.d=0,        ; Padding y - This is a virtual padding for resizing and boxed output
                                    flag=1,                      ; Text center=1 - Text right=2 - Text vertical=3
                                    mode=0,                      ; Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
                                    output_offset_factor_x.d=1,  ; This is the resizing factor for proportional dynamic GUI output - Look the Dynamic GUI demo codes for set_x - Default=1 = 100%
                                    output_offset_factor_y.d=1)  ; This is the resizing factor for proportional dynamic GUI output - Look the Dynamic GUI demo codes for set_y - Default=1 = 100%
  
  ; DrawText_Vector_BF - Get text width - Multiline supported - Can handle the Wizzard headline mode
  ; StartVectorDrawing needed
  Declare GetTextWidth_Vector_BF(text$,
                                 vertical=0,    ; Vertical text
                                 stretch_x=0,   ; Text stretching x - Headline mode
                                 scaling_x.d=1) ; Text scaling x
  
  ; DrawText_Vector_BF - Get text width - Multiline supported - Can handle the Wizzard headline mode
  ; StartVectorDrawing needed
  Declare GetTextHeight_Vector_BF(text$,
                                  vertical=0,    ; Vertical text
                                  stretch_y=0,   ; Text stretching x - Headline mode
                                  scaling_y.d=1) ; Text scaling y
  
  ; Get the text proportion - Multiline and text stretching supported
  Declare.d Get_Text_Proportion_Factor_X_to_Y_BF(text$,
                                                 font_ID, 
                                                 vertical=0,
                                                 stretch_x=0,
                                                 stretch_y=0)
  
  ; For vector based BF functions - You must deactivate again - As sample so : RotateCoordinates_BF()
  Declare RotateCoordinates_BF(rotate_coordinates_x_.d=0, rotate_coordinates_y_.d=0, rotate_coordinates_degree_.d=0)
  
  Declare PresetLineWidth_Vector_BF(max_1__vector) ; Preset the min output length for easy centric or right align outputs - Deactivate = 0
                                                   ; Important for permanent changed outputs
  
  ; VectorText output on screen
  ; This function is GradientColor BF sensitive - (LinearGradient and CircularGradient)
  Declare VectorText_Screen_BF(text$,
                               x,
                               y,
                               text_color,
                               text_alpha,
                               font=-1,
                               scale_width.d=1.0,
                               scale_height.d=1.0,
                               flag=#PB_VectorParagraph_Left)
  
  ; Flags : Text center=1 - Text right=2 - Text vertical=3
  ; Modes : Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
  ; Mono line output give back the xx output coordinate - Multi line output give back the yy output coordinate
  ; For stretched text on a textured background often DrawingMode (#PB_2DDrawing_Transparent) is needed
  ; DrawText_BF and FloodFill_BF are GradientColor_BF sensitive - Also all on this function based text functions !
  Declare DrawText_BF(txt_x.d, txt_y.d, txt$, f_col=#White, b_col=#Black, flag=0, mode=0, stretch_x=0, stretch_y=0)
  
  ; Create a image from the output, for post processing or using as image, give back a image (ID) - Padding -1 is automatic padding
  Declare DrawTextCreateImage_BF(txt$, f_col=#White, b_col=#Black, flag=0, mode=0, font_ID=-1, stretch_x=0, stretch_y=0, padding=0)
  
  ; Create a sprite from the output, for post processing and using as sprite, give back a sprite (ID)
  Declare DrawTextCreate_PB_Sprite_BF(txt$, f_col=#White, b_col=#Black, flag=0, mode=0, font_ID=-1, stretch_x=0, stretch_y=0, padding=0, sprite_mode=0)
  
  ; Direct output on screen - Parameters look above on DrawText_BF
  ; Additional you can set transparent for the text background
  Declare DrawTextScreenOutput_BF(txt_x, txt_y, txt$, f_col=#White, b_col=#Black, flag=0, mode=0, transparent=1, font_ID=-1, stretch_x=0, stretch_y=0)
  
  Declare PresetLineWidth_BF(max_1_) ; DrawText_BF - Preset the min output length for easy centric or right align outputs - Deactivate = 0
                                     ; Important for permanent changed outputs
  
  Declare GetTextWidth_BF(text$, ; DrawText_BF - Get text width - Multiline supported
                          font_ID,
                          vertical=0,
                          stretch_x=0,
                          padding_x=0)
  
  Declare GetTextHeight_BF(text$, ; DrawText_BF -  Get text height - Multiline supported
                           font_ID,
                           vertical=0,
                           stretch_y=0,
                           padding_y=0)
  
  ; Get vector text width - Multiline supported - Fast - #LF$ as linebreak needed
  Declare.d VectorTextWidth_Multiline_BF(text$, flags=#PB_VectorText_Default)
  
  ; Get vector text height - Multiline supported - Fast - #LF$ as linebreak needed
  Declare.d VectorTextHeight_Multiline_BF(text$, flags=#PB_VectorText_Default)
  
  ; Get vector text width - Multiline supported - All line breaks supported
  Declare.d VectorTextWidth_Multiline_Pro_BF(text$, flags=#PB_VectorText_Default)
  
  ; Get vector text height - Multiline supported - All line breaks supported
  Declare.d VectorTextHeight_Multiline_Pro_BF(text$, flags=#PB_VectorText_Default)
  
  ; DrawVectorText - Multiline - Fast - #LF$ as linebreak needed
  Declare DrawVectorTextMultiline_BF(text$)
  
  ; DrawVectorText - Multiline - All line breaks supported
  Declare DrawVectorTextMultiline_Pro_BF(text$)
  
  ; The BF gradient color functions works similar the PB functions
  ; activate_...<>0 activate a function - For the other parameters look in the PB guidance
  ; You must deactivate again
  
  ; This are color manipulating functions for the BF gradient color functions
  ; For using with textures you must change GradientColor_Alpha_BF
  ; The color manipulating functions influence the output speed
  Declare GradientColor_Alpha_BF(gradient_color_alpha_) ; Needed for output with textures - Firstly try a value about 120 > 255
  
  Declare GradientColor_Red_BF(gradient_color_red_)
  
  Declare GradientColor_Green_BF(gradient_color_green_)
  
  Declare GradientColor_Blue_BF(gradient_color_blue_)
  
  Declare GradientColor_Contrast_BF(gradient_color_contrast_)
  
  Declare GradientColor_Brightness_BF(gradient_color_brightness_)
  
  Declare GradientColor_Intensity_BF(gradient_color_intensity_)
  
  ; This Gradient color function works similar the PB functions - look in the PB guidance
  Declare BackColor_BF(activate_backcolor_=0, backcolor_=0)
  
  ; This Gradient color function works similar the PB functions - look in the PB guidance
  Declare FrontColor_BF(activate_frontcolor_=0, frontcolor_=0)
  
  ; This is a array for handling gradient colors - Available array entries 0 > 99 - You can change this preset
  Declare GradientColor_Array_BF(gradient_activate_flag, array_position, gradient_position.f, gradient_color)
  
  ; DrawText_BF and FloodFill_BF are gradient color sensitive !
  Declare LinearGradient_BF(activate_linear_gradient_=0, linear_gradient_x_=0, linear_gradient_y_=0, linear_gradient_xx_=0, linear_gradient_yy_=0)
  
  ; DrawText_BF and FloodFill_BF are gradient color sensitive !
  Declare CircularGradient_BF(activate_circular_gradient_=0, circular_gradient_x_=0, circular_gradient_y_=0, circular_gradient_radius_=0)
  
  ; DrawText_BF and FloodFill_BF are gradient color sensitive !
  Declare EllipticalGradient_BF(activate_elliptical_gradient_=0, elliptical_gradient_x_=0, elliptical_gradient_y_=0, elliptical_gradient_xx_=0, elliptical_gradient_yy_=0)
  
  ; DrawText_BF and FloodFill_BF are gradient color sensitive !
  Declare BoxedGradient_BF(activate_boxed_gradient_=0, boxed_gradient_x_=0, boxed_gradient_y_=0, boxed_gradient_width_=0, boxed_gradient_height_=0)
  
  ; DrawText_BF and FloodFill_BF are gradient color sensitive !
  Declare ConicalGradient_BF(activate_conical_gradient_=0, conical_gradient_x_=0, conical_gradient_y_=0, conical_gradient_degree_=0)
  
  ; This deactivate all gradient color functions after using
  ; Deactivate you not the functions, it can influence other outputs
  Declare DeactivateGradientColors_BF()
  
  ; This ignore all gradient color functions
  ; It can influence other outputs
  Declare IgnoreGradientColors_BF(ignore_gradient_colors_)
  
  Declare LineGo_8_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.a, lr.i) ; Moving Lines - look the sample codes in the download packet - shift=Bit shift amount - lr=moving direction
  
  Declare LineGo_16_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.u, lr.i)
  
  Declare LineGo_32_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.q, lr.i)
  
  Declare BoxGo_8_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.a, lr.i=0) ; Moving Boxes - look the sample codes in the download packet - shift=Bit shift amount - lr=moving direction
  
  Declare BoxGo_16_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.u, lr.i=0)
  
  Declare BoxGo_32_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.q, lr.i=0)
  
  Declare ActivateLineGoBackcolor_BF(activate_LineGo_backcolor_, LineGo_backcolor_=0) ; Activate a background color for LineGo and BoxGo (BoxGo is based on LineGo)
                                                                                      ; activate_LineGo_backcolor<>0 activate the function - You must deactivate again
  
  ; Check a 32Bit image if an invisible color exists - So you can check simple has a Gif or a image a invisible mask
  Declare IsImageInvisibleColor_BF(image_ID)
  
  CompilerIf #PB_Compiler_Version=>560
    Declare Get_Gif_FrameDelay_BF(gif_ID, frame) ; Delay 0 works not, so as a standard, a resulted delay 0 = delay 100ms
  CompilerEndIf                                  ; frame=-1 or fails the function, the function give back 100ms delay
                                                 ;  - This is helpfull for displaying a simple image, it slows down the GIF action loop 
  
  CompilerIf #PB_Compiler_Version=>560
    ; This function create a BF sprite (image) with defined invisible (Mask) color from a Gif frame
    Declare CreateSprite_from_Gif_Frame_BF(gif_ID, frame, invisible_color)
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    ; This function create a PB sprite with defined invisible (Mask) color from a Gif frame
    Declare Create_PB_Sprite_from_Gif_Frame_BF(gif_ID, frame, invisible_color, sprite_mode=0) ; mode = PB sprite mode
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    ; This function create a BF sprite (image) with defined invisible (Mask) color from a GIF frame and can rotate and resize the frame
    Declare CreateSprite_from_Gif_Frame_rotated_BF(gif_ID,
                                                   frame,               ; Select the GIF frame - The first frame is 0 (Because compatibility with PB)
                                                   invisible_color,     ; Select a invisible color - The function create new frame with this color
                                                   color_distance,      ; Color distance sprite mask
                                                   output_width=0,      ; You can here resize the output x - On demand
                                                   output_height=0,     ; You can here resize the output y - On demand
                                                   degree=0,            ; Rotating degree - The sprite rotates around its center point - With the rotating_offsets you can relocate this point
                                                   rotating_mode=0,     ; mode=0 without rotating - mode=1 create a exactly resized sprite with minimized mask - mode=2 create a quadratic sprite for on demand rotation 
                                                   rotating_offset_x=0, ; Rotating offset x - For mode 2 - Do not make offsets too large - Larger = slower, but I will give you not a limit
                                                   rotating_offset_y=0) ; Rotating offset y - You can also set negative values for the rotating offsets - As sample x=50 - y=-50 - This works very easy and simple
                                                                        ; rotating_mode 0 = Without rotating
                                                                        ; rotating_mode 1 = Centered with offsets
                                                                        ; rotating_mode 2 = Centered
                                                                        ; rotating_mode 3 = Left-aligned
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    ; This function create a PB sprite with defined invisible (Mask) color from a GIF frame and can rotate and resize the frame
    Declare Create_PB_Sprite_from_Gif_Frame_rotated_BF(gif_ID,
                                                       frame,               ; Select the GIF frame - The first frame is 0 (Because compatibility with PB)
                                                       invisible_color,     ; Select a invisible color - The function create new frame with this color
                                                       color_distance,      ; Color distance sprite mask
                                                       sprite_mode=0,       ; Sprite mode - Same PB function
                                                       output_width=0,      ; You can here resize the output x - On demand
                                                       output_height=0,     ; You can here resize the output y - On demand
                                                       degree=0,            ; Rotating degree - The sprite rotates around its center point - With the rotating_offsets you can relocate this point
                                                       rotating_mode=0,     ; Look down
                                                       rotating_offset_x=0, ; Rotating offset x - For mode 2 - Do not make offsets too large - Larger = slower, but I will give you not a limit
                                                       rotating_offset_y=0) ; Rotating offset y - You can also set negative values for the rotating offsets - As sample x=50 - y=-50 - This works very easy and simple
                                                                            ; rotating_mode 0 = Without rotating
                                                                            ; rotating_mode 1 = Centered with offsets
                                                                            ; rotating_mode 2 = Centered
                                                                            ; rotating_mode 3 = Left-aligned
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Declare Output_Gif_Frame_on_Canvas_rotated_BF(mode, canvas, gif_ID,
                                                  x,                   ; Output coordinate x
                                                  y,                   ; Output coordinate y
                                                  frame,               ; Sceect the GIF frame - The first frame is 0 (Because compatibility with PB)
                                                  invisible_color,     ; Select a invisible color - The function create new frame with this color
                                                  color_distance,      ; Color distance sprite mask
                                                  output_width=0,      ; You can here resize the output x - On demand
                                                  output_height=0,     ; You can here resize the output y - On demand
                                                  degree=0,            ; Rotating degree - The sprite rotates around its center point - With the rotating_offsets you can relocate this point
                                                  rotating_mode=0,     ; Look down
                                                  rotating_offset_x=0, ; Rotating offset x - For mode 1 - Do not make offsets too large - Larger = slower, but I will give you not a limit
                                                  rotating_offset_y=0) ; Rotating offset y - You can also set negative values for the rotating offsets - As sample x=50 - y=-50 - This works very easy and simple
                                                                       ; mode 0 = With invisible color
                                                                       ; mode 1 > 256 = With invisible color and with alpha blending
                                                                       ; mode -1 > -256 = Without invisible color and with alpha blending
                                                                       ; rotating_mode 0 = Without rotating
                                                                       ; rotating_mode 1 = Create a exactly resized sprite with minimized mask - Centered with offsets
                                                                       ; rotating_mode 2 = Create a exactly resized sprite with minimized mask - Centered
                                                                       ; rotating_mode 3 = Create a exactly resized sprite with minimized mask - Left-aligned
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Declare Gif_on_Image_BF(gif_ID, image_ID, x=0, y=0, frame=0, alpha=0, output_width=0, output_height=0) ; Output a Gif frame on image
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Declare Gif_on_Canvas_BF(gif_ID, canvas_ID, x=0, y=0, frame=0, alpha=0, output_width=0, output_height=0) ; Output a Gif frame on canvas
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Declare Gif_on_PB_Sprite_BF(gif_ID, sprite_ID, x=0, y=0, frame=0, alpha=0, output_width=0, output_height=0) ; Output a Gif frame on PB sprite
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Declare Gif_on_Screen_BF(gif_ID, x=0, y=0, frame=0, alpha=0, output_width=0, output_height=0) ; Output a Gif frame on screen
  CompilerEndIf
  
  ; PB sprite Rotate Bugfix (Workaround PB561) from J. Baker - I am not sure we get a bugfix in the future, so i add this function
  ; Activate absolute with 1 - This function can replace the PB function
  Declare Rotate_PB_Sprite_with_Offset_BF(sprite_ID, X, Y, angle, absolute=0, alpha=0, with_invisible_color=0)
  
  ; Change a color - Percent based as floating-point value
  ; Tree grey modes available mode=1, mode=2, mode=3 - mode=1 is more neutral
  Declare Change_Color_BF(color,
                          change_red.f=0,
                          change_green.f=0,
                          change_blue.f=0, 
                          change_contrast.f=0,
                          change_brightness.f=0,
                          change_intensity.f=0,
                          mode_grey=0)
  
  ; This function change the brightness, color, contrast - The function works with images, PB sprites, canvas
  ; The parameters are percent based - positive and negative values - Percent based as floating-point value
  ; Source and destination must have the same size
  ; mode 0 = without creating a new output - mode 1 create a image - mode 2 create a PB sprite
  ; sprite_mode - Same PB function call for PB sprite creating
  ; Tree grey modes available mode=1, mode=2, mode=3 - mode=1 is more neutral
  Declare Image_coloring_BF(source_ID, destination_ID,
                            change_red.f=0,
                            change_green.f=0,
                            change_blue.f=0, 
                            change_contrast.f=0,
                            change_brightness.f=0,
                            change_intensity.f=0,
                            mode_grey=0,
                            mode=0,
                            sprite_mode=0)
  
  ; Similar Image_coloring_BF, but can also colorize parts
  Declare Image_coloring_partition_BF(source_ID, destination_ID,
                                      x=0,             ; Output pos x
                                      y=0,             ; Output pos_y
                                      output_width=0,  ; Output width
                                      output_height=0, ; Output height
                                      swap_output=0,   ; Swap the colored part
                                      change_red.f=0,
                                      change_green.f=0,
                                      change_blue.f=0, 
                                      change_contrast.f=0,
                                      change_brightness.f=0,
                                      change_intensity.f=0,
                                      mode_grey=0,
                                      mode=0,
                                      sprite_mode=0)
  
  Declare CreateSpriteGradientColor_BF(texture_ID,
                                       image_get_color_x=0, ; Grab transparence color from source pos x - For preset set image_get_color_x=-1 and image_get_color_y as mask color
                                       image_get_color_y=0) ; Grab transparence color from source pos y - For preset set image_get_color_x=-1 and image_get_color_y as mask color
  
  Declare GetRotationExif_BF(path$) ; Check EXIF data for needed photo rotating - 6 = rotate right - 8 = rotate left
  
  Declare LoadPhoto_BF(path$) ; This function rotate automaticaly a Photo with EXIF data - Important as sample for many "Handy" Photos
                              ; The function works equal the PB function LoadImage
  
  ; Draw a grabed rectangle image
  ; This function can output on images, PB sprites, canvas
  ; For screen output set firstly ActivateScreenOutput_BF(1)
  Declare Draw_Rectangle_Image_BF(box_ID,
                                  output_ID,
                                  offset_x=0,
                                  offset_y=0,
                                  offset_width=0,
                                  offset_height=0)
  ; If you want to save a vector lib output, you have to consider the interpolation of the border lines which enlarge the rectangle
  ; This results in offsets : box_x=-1 - box_y=-1 - box_width=+2 - box_height=+2
  ; The offsets can you use also for DPI corrections of the output
  
  ; Grab multiple rectangle positions and additional their background as images
  ; For grab also a image set grab_rectangle_image=1
  ; The function give back a box ID
  ; Works on images, canvas, PB Sprites, screen - For grab from screen set ActivateScreenOutput_BF(1
  ; This Function is DPI_aware_BF sensitive  
  Declare Grab_Rectangle_BF(input_ID,
                            box_x,
                            box_y,
                            box_width,
                            box_height,
                            grab_rectangle_image=0,
                            offset_x=0,
                            offset_y=0,
                            offset_width=0,
                            offset_height=0)
  ; To Grab the rectangle as image set grab_rectangle_image=1  - 0 ignore a background image
  ; If you want to save a vector lib output, you have to consider the interpolation of the border lines which enlarge the rectangle
  ; This results in offsets : box_x=-1 - box_y=-1 - box_width=+2 - box_height=+2
  
  ; Free multiple rectangles 
  Declare FreeRectangles_BF(mode=0)
  ; 0=give all free (Preset) - 1=Get amount cached textures - 2=Get required size for caching all textures in bytes - 3=Get required images
  ; Temporary used rectangles free selective with the function Free_selected_Rectangle_BF
  ; Exit give free automatically all used rectangles
  
  ; Delete a selected rectangle from rectangle cache and give free
  Declare Free_selected_Rectangle_BF(Box_ID)
  
  ; Get a Rectangle_BF position and their image ID
  ; This function get and give back the background image ID and put the Rectangle_BF position data into the four functions below
  Declare Get_Rectangle_BF(box_ID)
  
  ; Get Rectangle_BF ID
  ; This function get and give back a Rectangle_BF ID found on Position x, y
  ; Not found a Rectangle_BF on this Position give back zero
  ; The function read the rectangle stack reverse, for better handling child rectangles
  Declare Get_Rectangle_ID_BF(x, y)
  
  ; Get Rectangle_BF image ID
  ; This function get and give back the background image ID
  Declare Get_Rectangle_ImageID_BF(box_ID)
  
  ; After a Get_Rectangle_BF call, this function give back the rectangle x position
  Declare Get_Rectangle_x_BF()
  
  ; After a Get_Rectangle_BF call, this function give back the rectangle y position
  Declare Get_Rectangle_y_BF()
  
  ; After a Get_Rectangle_BF call, this function give back the rectangle width
  Declare Get_Rectangle_width_BF()
  
  ; After a Get_Rectangle_BF call, this function give back the rectangle height
  Declare Get_Rectangle_height_BF()
  
  ; After a Get_Rectangle_BF call, this function give back the rectangle data
  Declare Get_Rectangle_data_BF()
  
  ; After a Get_Rectangle_BF call, this function give back the secound rectangle data
  Declare Get_Rectangle_data_secound_BF()
  
  ; After a Get_Rectangle_BF call, this function give back the third rectangle data
  Declare Get_Rectangle_data_third_BF()
  
  ; After a Get_Rectangle_BF call, this function give back the fourth rectangle data
  Declare Get_Rectangle_data_fourth_BF()
  
  ; After a Get_Rectangle_BF call, this function give back the rectangle data as double
  Declare.d Get_Rectangle_data_double_BF()
  
  ; After a Get_Rectangle_BF call, this function give back the rectangle string
  Declare$ Get_Rectangle_string_BF()
  
  ; After a Get_Rectangle_BF call, this function give back the rectangle string 2
  Declare$ Get_Rectangle_string_secound_BF()
  
  ; This function set the rectangle x pos
  Declare Set_Rectangle_x_BF(get_box_x_bf_)
  
  ; This function set the rectangle y pos
  Declare Set_Rectangle_y_BF(get_box_y_bf_)
  
  ; This function set the rectangle width
  Declare Set_Rectangle_width_BF(get_box_width_bf_)
  
  ; This function set the rectangle height
  Declare Set_Rectangle_height_BF(get_box_height_bf_)
  
  ; This function set the rectangle data
  Declare Set_Rectangle_data_BF(box_ID, box_data_)
  
  ; This function set the secound rectangle data
  Declare Set_Rectangle_data_secound_BF(box_ID, box_data_secound_)
  
  ; This function set the third rectangle data
  Declare Set_Rectangle_data_third_BF(box_ID, box_data_third_)
  
  ; This function set the fourth rectangle data
  Declare Set_Rectangle_data_fourth_BF(box_ID, box_data_fourth_)
  
  ; This function set the rectangle data as double
  Declare Set_Rectangle_data_double_BF(box_ID, box_data_double_.d)
  
  ; This function set the rectangle string   
  Declare Set_Rectangle_string_BF(box_ID, box_string_$)
  
  ; This function set the rectangle string 2   
  Declare Set_Rectangle_string_secound_BF(box_ID, box_string_secound_$)
  
  ; This function give back a fitted image
  Declare FitImage_StretchText(image_ID,
                               fit_width,                ; Fit image to this width
                               fit_height,               ; Fit image to this height
                               background_color=$DCDCDC, ; Background color
                               padding_x=0,              ; Padding x
                               padding_y=0,              ; Padding y
                               edge_rounding=0,          ; Edge Rounding
                               frame_color=$FFFFFF,      ; Frame color
                               frame_size=3,             ; frame_size
                               sprite_mode=0)            ; Sprite mode for PB sprite output
                                                         ; mode 0 = image output
                                                         ; mode 1 = Without background
                                                         ; mode 2 = PB sprite output
                                                         ; mode 3 = Without output - You can use GetFitSize_Width_BF() and GetFitSize_Height_BF() for get the calculated width and height
                                                         ; mode 4 = Special mode with offsets for Magic_StretchText_Pro_BF()
                                                         ; This function can output images and PB sprites
                                                         ; Look the FitImage demo code
                                                         ; You can set ActivateAlphaChannel_BF(1) before you call this function
  
  ; This function give back a fitted image
  Declare FitImage_BF(mode,
                      image_ID,
                      fit_width,                ; Fit image to this width
                      fit_height,               ; Fit image to this height
                      background_color=$DCDCDC, ; Background color
                      padding=0,                ; Padding
                      grid_color=$FFFFFF,       ; Grid color
                      grid_size=5,              ; Grid size - Set on special mode 4 Edge Rounding
                      grid_tiling=10,           ; Grid tiling
                      frame_color=$FFFFFF,      ; Frame color
                      frame_size=3,             ; frame_size
                      sprite_mode=0)            ; Sprite mode for PB sprite output
                                                ; mode 0 = image output
                                                ; mode 1 = Without background
                                                ; mode 2 = PB sprite output
                                                ; mode 3 = Without output - You can use GetFitSize_Width_BF() and GetFitSize_Height_BF() for get the calculated width and height
                                                ; mode 4 = Special mode with offsets for Magic_StretchText_Pro_BF()
                                                ; This function can output images and PB sprites
                                                ; Look the FitImage demo code
                                                ; You can set ActivateAlphaChannel_BF(1) before you call this function
  
  ; This function give back a fitted image
  Declare FitImage_DualPadding_BF(mode,
                                  image_ID,
                                  fit_width,                ; Fit image to this width
                                  fit_height,               ; Fit image to this height
                                  background_color=$DCDCDC, ; Background color
                                  padding_x=0,              ; Padding x
                                  padding_y=0,              ; Padding y
                                  grid_color=$FFFFFF,       ; Grid color
                                  grid_size=5,              ; Grid size
                                  grid_tiling=10,           ; Grid tiling
                                  frame_color=$FFFFFF,      ; Frame color
                                  frame_size=3,             ; frame_size
                                  sprite_mode=0)            ; Sprite mode for PB sprite output
                                                            ; mode 0 = image output
                                                            ; mode 1 = Without background
                                                            ; mode 2 = PB sprite output
                                                            ; mode 3 = Without output - You can use GetFitSize_Width_BF and GetFitSize_Height_BF for get the calculated width and height
                                                            ; This function can output images and PB sprites
                                                            ; Look the FitImage demo code
                                                            ; You can set ActivateAlphaChannel_BF(1) before you call this function
  
  ; Set a x, y offset for FitImage..BF - You must deactivate again - Percent based
  Declare FitImage_Offset_BF(fit_image_offset_x_.d, fit_image_offset_y_.d)
  
  ; Add to the FitImage functions a automatically upscaling for little images - 1=activated - 0=deactivated
  ; -You must deactivate again
  ; -As sample also for ColorButtonGadget_BF, TextGadget_BF, ImageGadget_BF, ButtonImageGadget_BF
  Declare FitImageActivateUpscaling_BF(fit_image_upscaling_)
  
  ; Get the state for FitImageActivateUpscaling_BF
  Declare FitImageGetUpscalingState_BF()
  
  ; Fit a source rectangle proportional in to a destination rectangle
  ; You can use for proportional resizing with many BF functions
  ; This raw base function is not needed and created for smaller images as the fitted output, you must handle this and skip the function for smaller images
  ; image_mode=1 prevent resulted values <1
  Declare GetFitSize_BF(source_width.d, source_height.d, fit_width.d, fit_height.d, image_mode=1)
  
  Declare.d GetFitOffset_X_BF() ; Give back the x offset for the fitted output
  
  Declare.d GetFitOffset_Y_BF() ; Give back the y offset for the fitted output
  
  Declare.d GetFitSize_Width_BF() ; Give back the calculated proportional width for the fitted output
  
  Declare.d GetFitSize_Height_BF() ; Give back the calculated proportional height for the fitted output
  
  ; --------------------------- Universal PB function output - start -------------------------------------
  
  ; The following functions extend PB functions
  ; They are extensively documented
  ; You can easily add any new PB functions
  ; The necessary changes are minimal in each case
  ; See the demo codes for enhanced universal PB function output
  ; The functions can output on images, PB sprites and canvas
  ; The functions can output textured, with tiling, scrolling, clipping, texture adjusting, alpha blending, color distance
  
  ; mode= 0 - Texture mode simple 
  ;           You must set a unused image color self with SetMaskColorUniversalOutput_BF() for replacing this color with a texture
  
  ; mode= 1 - Standard preset texture mode
  ;           Search a unused image color automatically for replacing with a texture
  ;           You must deactivate SetMaskColorUniversalOutput_BF(-1) with -1
  
  ; mode= 2 > 256 - Equal mode 1 but with alpha blending - 2=full visible - 256=invisible (BF specific)
  ; mode=-1   Ignore a texture and use a color - Set as texture_ID a color, as sample $FFFF00 
  ; mode=-2 > -256 - Ignore a texture and use a color - With alpha blending 2=full visible - 256=invisible (BF specific)
  
  ; For best results with vector outputs set a unused image color nearly the background image color
  ; This fix the ugly edge problem with vector drawing outputs mixed with PB standard outputs
  
  Declare AddPathLine_BF(mode, output_ID, texture_ID,
                         x.d,                               ; Output x
                         y.d,                               ; Output y
                         xx.d,                              ; Output xx
                         yy.d,                              ; Output yy
                         stroke_path.d,                     ; Stroke path
                         stroke_path_flag=#PB_Path_Default, ; Stroke path flag - As sample #PB_Path_RoundEnd
                         texture_x=0,                       ; Startposition texture output x   - Preset 0
                         texture_y=0,                       ; Startposition texture output y   - Preset 0
                         texture_width=0,                   ; Endposition texture output xx    - Preset 0 = Full available width
                         texture_height=0,                  ; Endposition texture output yy    - Preset 0 = Full available height
                         texture_clip_x=0,                  ; Startposition inside the texture - Preset 0 (Texture clipping)
                         texture_clip_y=0,                  ; Startposition inside the texture - Preset 0 (Texture clipping)
                         texture_clip_xx=0,                 ; Endposition inside the texture   - Preset 0 (Texture clipping)
                         texture_clip_yy=0)                 ; Endposition inside the texture   - Preset 0 (Texture clipping)
  
  
  Declare AddPathEllipse_BF(mode, output_ID, texture_ID,
                            x.d,               ; Output x
                            y.d,               ; Output y
                            radius_x.d,        ; Radius x
                            radius_y.d,        ; Radius y
                            starting_angle.d,  ; Starting angle
                            end_angle.d,       ; End angle
                            rotate_angle,      ; Rotate angle                             
                            texture_x=0,       ; Startposition texture output x   - Preset 0
                            texture_y=0,       ; Startposition texture output y   - Preset 0
                            texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                            texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                            texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                            texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                            texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                            texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
  
  Declare AddPathCircle_BF(mode, output_ID, texture_ID,
                           x.d,               ; Output x
                           y.d,               ; Output y
                           radius.d,          ; Radius
                           starting_angle.d,  ; Starting angle
                           end_angle.d,       ; End angle
                           texture_x=0,       ; Startposition texture output x   - Preset 0
                           texture_y=0,       ; Startposition texture output y   - Preset 0
                           texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                           texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                           texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                           texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                           texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                           texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
  
  Declare Ellipse_BF(mode, output_ID, texture_ID,
                     x,                 ; Output x
                     y,                 ; Output y
                     radius_x,          ; Radius x
                     radius_y,          ; Radius y
                     texture_x=0,       ; Startposition texture output x   - Preset 0
                     texture_y=0,       ; Startposition texture output y   - Preset 0
                     texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                     texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                     texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                     texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                     texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                     texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
  
  Declare RoundBox_BF(mode, output_ID, texture_ID,
                      x,                 ; Output x
                      y,                 ; Output y
                      width,             ; Width
                      height,            ; Height
                      radius_x,          ; radius_x
                      radius_y,          ; radius_y 
                      texture_x=0,       ; Startposition texture output x   - Preset 0
                      texture_y=0,       ; Startposition texture output y   - Preset 0
                      texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                      texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                      texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                      texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                      texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                      texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
  
  Declare Box_BF(mode, output_ID, texture_ID,
                 x,                 ; Output x
                 y,                 ; Output y
                 width,             ; Width
                 height,            ; Height
                 texture_x=0,       ; Startposition texture output x   - Preset 0
                 texture_y=0,       ; Startposition texture output y   - Preset 0
                 texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                 texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                 texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                 texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                 texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                 texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
  
  Declare DrawRotatedText_BF(mode, output_ID, texture_ID, font_ID,
                             x,                 ; Output x
                             y,                 ; Output y
                             text$,             ; Text
                             degree,            ; Degree
                             texture_x=0,       ; Startposition texture output x   - Preset 0
                             texture_y=0,       ; Startposition texture output y   - Preset 0
                             texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                             texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                             texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                             texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                             texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                             texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
                                                ; Ignore font = -1
  
  ; -------------------------- Universal PB function output - end ------------------------------------
  
  Declare WriteColorDistanceValue_BF(path$, color_distance) ; Write the color distance value to a file - As sample to a jpeg based sprite
                                                            ; On sprite sheets color distance info is automatically included directly in the sheet date
  
  Declare RemoveColorDistanceValue_BF(path$) ; Remove the color distance value from a file
  
  Declare ReadSheetInfoFile_BF(path$) ; Get and move all frame data from a CSS sheet info file to the functions above
  
  CompilerIf #PB_Compiler_Version=>560
    Declare CreateCSSSheet_simple_BF(gif_ID, mode=0) ; This function create a CSS sprite sheet from a GIF with automatic created invisible color (Background color)
                                                     ; This function fill the GetGIFandCSS... functions with frame data
                                                     ; You can set ActivateAlphaChannel_BF(1) before you call this function,
                                                     ; then the created sheet is a PNG with invisible color - you must deactivate after using the function
                                                     ; mode=0 Create a sheet
                                                     ; mode=1 Calculate only a sheet
                                                     ; mode=2 Create a sheet and delete the source GIF
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Declare CreateCSSSheet_BF(gif_ID, invisible_color) ; This function create a CSS sprite sheet from a GIF - You must set a invisible color (Background color)
                                                       ; This function fill the GetGIFandCSS... functions with frame data
                                                       ; You can set ActivateAlphaChannel_BF(1) before you call this function,
                                                       ; then the created sheet is a PNG with invisible color - you must deactivate after using the function
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    ; This function create a CSS sprite sheet from a GIF with automatic created invisible color (Background color) and a sheet data file
    ; This function fill the GetGIFandCSS... functions with frame data
    ; encoder 1 = jpg - encoder 2 = png - encoder 3 = bmp
    ; You can pass a GIF ID, but you don't have to
    ; internal_frame_data 0 = without internal frame data - internal_frame_data 1 = with internal frame data 
    Declare CreateCSSandDataSheet_simple_BF(path$, gif_ID=0, encoder=2, internal_frame_data=0)
    ; GIF can have different defined invisible colors for different frames
    ; This code define for the complete generated sheet a new invisible color, only one ! - This works very fine and make a sheet post processing very simple
    ; A sprite sheet is high flexible, mostly, i think, for using GIF like output inside a app sheet using is many better as GIF using
    ; BF can handle any sprite sheets very easy and simple, also lossly compressed sheets
    ; Movie like sheets without a invisible color you can fine compress as JPG files how ever you want
    ; For sheets with a used invisible color PNG works very fine, but you can compress this files also as JPG image
    
    ; For lossly compressed sheets with a used invisible color you must use the PB function SetColorDistanceSpriteMask_BF for clearing the sprite mask from compression artifacts
    ; For best results with lossly compressed sheets you can simple define also a own invisible color for each sheet
    ; Best results you get with a invisible color nearly the color from the sprite content edges - So you can create simple also a nice edge blur for the sprites 
    
    ; You can set ActivateAlphaChannel_BF(1) before you call this function,
    ; then the created sheet is a PNG with invisible color - you must deactivate after using the function
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    ; This function create a CSS sprite sheet from a GIF and a sheet data file - You must set a invisible color (Background color)
    ; This function fill the GetGIFandCSS... functions with frame data
    ; You can pass a GIF ID, but you don't have to
    ; internal_frame_data add the frame data (sheet info) to the file end - The latest 8 bytes from the file (quad) is the data length
    ; internal_frame_data 0 = without internal frame data - internal_frame_data 1 = with internal frame data
    ; encoder=1 create JPG sheet -  encoder=2 create PNG sheet - encoder=3 create BMP sheet
    ; internal_frame_data=1 add all frame data directly in the sheet
    Declare CreateCSSandDataSheet_BF(path$, gif_ID, invisible_gif_color, encoder=2, internal_frame_data=0, color_distance_gif=0)
    ; You can set ActivateAlphaChannel_BF(1) before you call this function,
    ; then the created sheet is a PNG with invisible color - you must deactivate after using the function
  CompilerEndIf
  
  ; Create a empty BF CSS sheet with included sheet data and external data file
  ; This function fill the GetGIFandCSS... functions with frame data
  ; encoder=1 create JPG sheet -  encoder=2 create PNG sheet - encoder=3 create BMP sheet
  ; invisible_gif_color is the background color from the created sheet
  ; You can set ActivateAlphaChannel_BF(1) before you call this function
  Declare CreateEmptyCSSandDataSheet_BF(path$, invisible_gif_color, frames, frame_width, frame_height, preset_delays=0, encoder=2, color_distance_gif=0)
  
  Declare GetGIFandCSS_color_distance_BF() ; Get gif and CSS sheet color distance after using gif and sheet read or write functions 
  
  Declare GetGIFandCSS_invisible_color_BF() ; Get gif and CSS sheet invisible color after using gif and sheet read or write functions 
  
  Declare GetGIFandCSS_Frames_BF() ; Get gif and CSS sheet number of frames after using gif and sheet read or write functions 
  
  Declare GetGIFandCSS_Frame_width_BF() ; Get gif and CSS sheet frame width after using gif and sheet read or write functions 
  
  Declare GetGIFandCSS_Frame_height_BF() ; Get gif and CSS sheet frame height after using gif and sheet read or write functions 
  
  Declare GetCSS_width_BF() ; Get CSS sheet width after using sheet read or write functions
  
  Declare GetCSS_height_BF() ; Get CSS sheet height after using sheet read or write functions
  
  Declare GetFrames_in_a_row_BF() ; Get CSS sheet frames in a row after using sheet read or write functions
  
  Declare GetGIFandCSS_delay_array_BF(gif_and_css_delay_array_) ; Get gif and CSS frame delays after using gif and sheet read or write functions
                                                                ; This function also works like Dim and can dimension the delay array
  
  Declare GetFrameData_fromSheet_BF(path$) ; Get and move all included frame data from a CSS sheet to the functions above
                                           ; Must return 1
  
  Declare IsSheet_BF(path$) ; Check - Is this file a valid BF CSS sheet - Must return 1
  
  Declare GetFrameDataFromString_BF(string$) ; Grabbing a frame data string and put it in the frame datas - Take with care
                                             ; Must return 1
  
  Declare ReadColorDistanceValue_BF(path$) ; Read the color distance value from a file - As sample to a jpeg based sprite
                                           ; On sprite sheets color distance info is automatically included directly in the sheet data
                                           ; Must return 0 > 256 - Real max is 255, but 256 is accepted also
  
  Declare IsSheetInfoFile_BF(path$) ; Check - Is this file a valid BF sheet info file
  
  Declare SetGIFandCSS_delay_array_BF(frame, delay) ; Set the Gif and CSS delay array
  
  Declare SetGIFandCSS_color_distance_BF(gif_css_color_distance_) ; Set the Gif and CSS color_distance
  
  Declare SetGIFandCSS_invisible_color_BF(gif_css_invisible_color_) ; Set the Gif and CSS invisible_color
  
  Declare SetGIFandCSS_Frames_BF(gif_css_frames_) ; Set Gif and CSS frames amount
  
  Declare SetGIFandCSS_Frame_width_BF(gif_css_frame_width_) ; Set gif and CSS sheet frame width - Take with care
  
  Declare SetGIFandCSS_Frame_height_BF(gif_css_frame_height_) ; Set gif and CSS sheet frame height - Take with care
  
  Declare SetGIFandCSS_Frames_BF(gif_css_frames_) ; Set gif and CSS sheet frame amount - Take with care
  
  Declare SetCSS_width_BF(css_width_) ; Set CSS sheet width - Take with care
  
  Declare SetCSS_height_BF(css_height_) ; Set CSS sheet height - Take with care
  
  Declare SetFrames_in_a_row_BF(css_frames_in_a_row_) ; Set css sheet frames in a row - Take with care
  
  Declare.s SetFrameDataInString_BF() ; Grabbing the frame data and put it in a string - Take with care
  
  Declare.s LoadFrameData_BF(path$) ; Get the frame data and put it in the frame variables and in a string - Fail results a empty string
  
  ; Function for grabbing a frame from any BF CSS sheet
  Declare Sprite_CSS_sheet_grab_frame_BF(texture_ID,
                                         frames,                ; Numbers of frames
                                         frame,                 ; Selected frame - For GIF compatibility the first frame is 0
                                         frame_width,           ; Frame width
                                         frame_height,          ; Frame height
                                         frames_in_a_row=0,     ; Frames in a row - Preset 0, then BF calculate self
                                         output_width=0,        ; Resizing - Define the output width - Preset zero = without resizing 
                                         output_height=0,       ; Resizing - Define the output height
                                         texture_clip_x=0,      ; Startposition inside the texture - Preset 0
                                         texture_clip_y=0,      ; Startposition inside the texture - Preset 0
                                         texture_clip_width=0,  ; Endposition inside the texture - Preset 0 = full texture width
                                         texture_clip_height=0) ; Endposition inside the texture - Preset 0 = full texture height
  
  ; Function for grabbing a frame from any BF CSS sheet and free a defined frame
  Declare Sprite_CSS_sheet_grab_frame_and_free_BF(texture_ID, free_ID,
                                                  frames,                ; Numbers of frames
                                                  frame,                 ; Selected frame - For GIF compatibility the first frame is 0
                                                  frame_width,           ; Frame width
                                                  frame_height,          ; Frame height
                                                  frames_in_a_row=0,     ; Frames in a row - Preset 0, then BF calculate self
                                                  output_width=0,        ; Resizing - Define the output width - Preset zero = without resizing 
                                                  output_height=0,       ; Resizing - Define the output height
                                                  texture_clip_x=0,      ; Startposition inside the texture - Preset 0
                                                  texture_clip_y=0,      ; Startposition inside the texture - Preset 0
                                                  texture_clip_width=0,  ; Endposition inside the texture - Preset 0 = full texture width
                                                  texture_clip_height=0) ; Endposition inside the texture - Preset 0 = full texture height
  
  ; This function fill the GetGIFandCSS... functions with frame data
  Declare CalculateCSSSheet_BF(frame_width, frame_height, frames)
  
  ; QAES AES256 OFB mode two stage special crypter for all things - binarys - strings - text files
  ; This crypter can handle automatic string termination for PB strings in compiler mode ASCII and UNICODE
  ; The crypter works with all data lengths
  ; With mode ASCII you can encrypt mixed data, string and binary - This ignore the encryption from zero bytes
  ; The rypter go ever forward, a extra decoder is unnecessary !
  ; You cipher a file blockwise, set ever the current block number (consecutive) with the counter - Important !
  ; Author Werner Albus (C) - www.nachtoptik.de - www.quick-aes-256.de
  ; No warranty whatsoever - Use at your own risk
  ; You cipher a file blockwise, set ever the current block number as counter (consecutive numbering) - This is important !
  ; You can set the counter as quad, positive and negative
  ; You can use any strings as key
  ; Set a pointer to the source and destination data - It can also are the same place
  ; Set mode=0 for binary files - Set mode=1 for ASCII strings - Set mode=2 for UNICODE strings
  Declare QAES_crypter_BF(mode, *buffer_in.word, *buffer_out.word, bytes.q, key$, counter.q=0)

  CompilerIf #PB_Compiler_Version=>560
    Declare Create_encrypted_String_File_BF(string$, destination_path_csv$, key$) ; Encrypt and save a string as file
  CompilerEndIf
  
  Declare.s Read_encrypted_String_File_BF(destination_path_csv$, key$) ; Load a encrypted string as file and decrypt to a string
  
  ; Function for convert a multicolor sprite to a sprite with multiple invisible colors
  ; You can replace ten different sprite colors with a single new color
  ; This allows you to make ten different sprite colors invisible on demand 
  ; Deactivate a invisible color with -1 - For white set $FFFFFF
  ; This function ignore color distance - Use sprites with a clean sprite mask
  Declare Convert_to_multi_invisible_Color_Sprite_BF(source_ID,
                                                     invisible_color_1=-1,
                                                     invisible_color_2=-1,
                                                     invisible_color_3=-1,
                                                     invisible_color_4=-1,
                                                     invisible_color_5=-1,
                                                     invisible_color_6=-1,
                                                     invisible_color_7=-1,
                                                     invisible_color_8=-1,
                                                     invisible_color_9=-1,
                                                     invisible_color_10=-1)
  
  ; Same the PB function, but enhanced - Without Icon output
  Declare MessageRequester_simple_BF(title_line$="",
                                     text$="",
                                     flags.a=#PB_MessageRequester_Yes,
                                     text_button_yes$="Yes",
                                     text_button_no$="No",
                                     text_button_cancel$="Cancel",
                                     text_width=470,
                                     text_height=165,
                                     buttons_width=150,
                                     buttons_height=35)
  
  ; Same the PB function, but enhanced - With Image output
  ; MagicStretchText_Add_Image_BF() sensitive
  ; Magic_StretchText_Pro_Border_BF sensitive
  ; Can handle GradientColor_BF
  Declare MessageRequester_Magic_BF(title_line$="",
                                    text$="",
                                    flags.a=#PB_MessageRequester_Yes, ; #PB_MessageRequester_Yes - #PB_MessageRequester_YesNo
                                    text_button_yes$="Yes",
                                    text_button_no$="No",
                                    text_button_cancel$="Cancel",
                                    text_width=470,
                                    text_height=165,
                                    text_stretching_x=0, ; Headline mode
                                    text_stretching_y=0, ; Headline mode
                                    text_padding_x=7,    ; This shrink the text x
                                    text_padding_y=7,    ; This shrink the text y
                                    text_color=#Black,
                                    buttons_color=#Black, ; Colors are alpha channel sensitive
                                    text_background_color=#White, ; -1 = transparent
                                    buttons_background_color=-1,  ; -1 = transparent
                                    buttons_width=150,
                                    buttons_height=35,
                                    button_yes_padding_x=7,
                                    button_yes_padding_y=7,
                                    button_no_padding_x=7,
                                    button_no_padding_y=7,
                                    button_cancel_padding_x=7,
                                    button_cancel_padding_y=7,
                                    font$="Arial",
                                    font_style=0)
  
  
  ; Make the Messagerequester_Magic_BF text box clickale - give back #PB_MessageRequester_Info - You must deactivate again
  Declare MessageRequester_Magic_make_text_clickable_BF(Messagerequester_Magic_make_text_clickable_)
  
  CompilerIf #PB_Compiler_Version=>560
    ;; Create a font array for using with this function
    ;; Mostly 30pt > 40pt is enough - smaller is faster
    ;   max_needed_font_size=40 ; Set the max needed font size for any output with this function
    ;   Global Dim Font_Arial(max_needed_font_size) ; For using in a callback procedure define as Global 
    ;   Define needed_font_size
    ;   For needed_font_size=1 To max_needed_font_size ; Get the fonts
    ;     Font_Arial(needed_font_size)=LoadFont(#PB_Any, "Courier New", needed_font_size, #PB_Font_Bold)
    ;   Next needed_font_size
    
    ; Same the PB function, but very enhanced with auto resizing text and buttons - Without OS Icon output
    ; But you can include any images or you can use also without
    ; Can handle all images, also GIF, resize and placing, with animated output and alpha blending
    ; Can handle GradientColor_BF
    ; Set simple a max requester width and height
    ; But, the requester adjusts its size itself according to a given text and requester_width, requester_height
    ; You can also influence the button text size with resize_factor_button_text - This change the button size automatically
    Declare MessageRequester_BF(Array font_ID(1), ; Set a Font array, for simple creating any look the sample above
                                requester_width,
                                requester_height,
                                title_line$="",
                                text$="",
                                flags.a=#PB_MessageRequester_Yes,
                                text_button_yes$="    Yes    ",
                                text_button_no$="No",
                                text_button_cancel$="Cancel",
                                text_color=0,
                                text_background_color=$F0F0F0,
                                background_color=$F0F0F0,
                                button_text_color=0,
                                button_background_color=$E0E0E0,
                                resize_factor_button_text.f=1.0,
                                image_ID=-1,                               
                                image_x_pos=0,
                                image_y_pos=0,
                                image_output_width=0,
                                image_output_height=0,
                                alpha=1)
  CompilerEndIf
  
  ; mode_messagerequester_=0 deactivate the function
  ; mode_messagerequester_=1 save and restore the GadgetList
  ; mode_messagerequester_=2 save and restore the GadgetList and create a centered output on the parent window
  ; mode_messagerequester_=3 save and restore the GadgetList and create a x_pos, y_ pos adjusted output
  ; A window ID is not needed, you can set to -1 for ignore
  ; Set you on mode 3 x_pos or y_pos too -1, this position is automatically centered
  ; Set you on mode 2 or mode 3 x_pos or y_pos too -1, this position is automatically centered
  Declare SetMessageRequester_BF(parent_window_message_requester_ID_, mode_messagerequester_, x_pos_messagerequester_=0, y_pos_messagerequester_=0)
  
  ; The solution for the simplest creation of color buttons and text images
  ; Automatic multi line support
  ; Automatic text size
  ; Automatic text adjustment
  ; Outputs all fonts with edge smoothing
  ; Can handle GradientColor_BF
  ; Look the ColorButtonGadget_BF demo code
  ; Flags : Text center=1 - Text right=2 - Text vertical=3
  ; Modes : Revers text mode=1 - RTL text mode=2
  ; BF Supports also automatically text upscaling with the function FitImageActivateUpscaling_BF
  Declare CreateFittedTextImage_BF(text$,
                                   image_width,
                                   image_height,
                                   text_color=$FFFFFF,
                                   background_color=0,
                                   font_ID=-1,
                                   stretch_x=0,
                                   stretch_y=0,
                                   flag=1,
                                   mode=0)
  
  ; Compatible to CreateFittedTextImage_BF, but with integrated dual padding function
  Declare CreateFittedTextImage_DualPadding_BF(text$,
                                               image_width,
                                               image_height,
                                               text_color=$FFFFFF,
                                               background_color=0,
                                               padding_x=0,
                                               padding_y=0,
                                               font_ID=-1,
                                               stretch_x=0,
                                               stretch_y=0,
                                               flag=1,
                                               mode=0)
  
  ; The subsystem Linux gtk3 can expand BF MessageRequester_Pro_BF and ButtonGadgets_BF slightly in size
  ; So you can preset a negative offset to adjust the buttons
  ; Don't select these offsets too big, the button text will get smaller and smaller
  ; Different gtk3 versions can affect button sizes differently
  ; Especially all smaller buttons are automatically enlarged, which can make precise adjustment much more difficult
  ; This is not a failure of GFX_Wizzard_BF, but a "feature" of gtk3 :-(
  ; The wizard currently automatically activates gtk3 offsets, which you can also change so :
  Declare gtk3_button_x_offset_BF(gtk3_x_offset_) ; Presetted = 14
  
  Declare gtk3_button_y_offset_BF(gtk3_y_offset_) ; presetted = 6
  
  Declare Get_gtk3_button_x_offset_BF() ; Get presetted gtk3 offset x - Only is gtk3 active <> 0
  
  Declare Get_gtk3_button_y_offset_BF() ; Get presetted gtk3 offset y - Only is gtk3 active <> 0
  
  ; Fade out - Recommendation : Set GadgetData (0 > 1) as state - This function works also with the automatic version
  Declare SetColorButtonGadget_BF_FadeOut_BF(gadget_ID,
                                             text_a$,
                                             text_b$,
                                             text_color,
                                             background_color,
                                             animation_delay,
                                             animation_alpha_step)
  
  ; Fade in - Recommendation : Set GadgetData (0 > 1) as state - This function works also with the automatic version
  Declare SetColorButtonGadget_BF_FadeIn_BF(gadget_ID,
                                            text_a$,
                                            text_b$,
                                            text_color,
                                            background_color,
                                            animation_delay,
                                            animation_alpha_step)
  
  ; Fade in - Recommendation : Set GadgetData (0 > 1) as state - This function works also With the automatic version
  Declare SetColorButtonGadget_BF_Fade_ChangeTextColor_BF(gadget_ID,
                                                          text_a$,
                                                          text_b$,
                                                          text_color_a,
                                                          text_color_b,
                                                          background_color,
                                                          animation_delay,
                                                          animation_alpha_step)
  
  ; Fade in and fade out - Full automatic mode- This function get and set GadgetData (0 > 1)
  Declare SetColorButtonGadget_BF_AutoFade_BF(gadget_ID,
                                              text_a$,
                                              text_b$,
                                              animation_delay,
                                              animation_alpha_step)
  
  ; Fade in and fade out - Recommendation : Set GadgetData (0 > 1) as state - This function works also with the automatic version
  Declare SetColorButtonGadget_BF_Fade_BF(gadget_ID,
                                          text_a$,
                                          text_b$,
                                          text_color,
                                          background_color,
                                          animation_delay,
                                          animation_alpha_step)
  
  ; Fade in and fade out - Full automatic mode- This function get and set GadgetData (0 > 1)
  Declare SetColorButtonGadget_BF_Fading_BF(gadget_ID,
                                            text_a$,
                                            text_b$,
                                            text_color_a,
                                            text_color_b,
                                            animation_delay,
                                            animation_alpha_step)
  
  Declare SetColorButtonGadget_BF_FlashText_BF(gadget_ID) ; Short text flashing
  
  ; Change the ColorButtonGadget_BF
  ; The function can handle GradientColor_BF
  ; You must not set position, size, stretch, font, but you can
  ; Use ever a large font size - Then the function adjust the needed font size automatically (about 60 for the mostly large buttons)
  ; stretch = Headline text stretcheng function
  ; BF Supports also automatically text upscaling with the function FitImageActivateUpscaling_BF
  Declare SetColorButtonGadget_BF(gadget_ID,
                                  text$="",
                                  text_color=-1,
                                  background_color=-1,
                                  x=-1,
                                  y=-1,
                                  button_width=-1,
                                  button_height=-1,
                                  stretch_x=-1,
                                  stretch_y=-1,
                                  font_ID=-1)
  
  ; Compatible to SetColorButtonGadget_BF, but with integrated dual padding function
  Declare SetColorButtonGadget_DualPadding_BF(gadget_ID,
                                              text$="",
                                              text_color=-1,
                                              background_color=-1,
                                              padding_x_gadget=-1,
                                              padding_y_gadget=-1,
                                              x=-1,
                                              y=-1,
                                              button_width=-1,
                                              button_height=-1,
                                              stretch_x=-1,
                                              stretch_y=-1,
                                              font_ID=-1)
  
  ; Automatic multi line support
  ; Automatic text size
  ; Automatic text adjustment
  ; Outputs all fonts with edge smoothing
  ; Can handle GradientColor_BF
  ; font_ID you must not set
  ; Use ever a large font size - Then the function adjust the needed font size automatically (about 60 for the mostly large buttons)
  ; flag : Text center=1 - Text right=2 - Text vertical=3
  ; mode : Revers text mode=1 - RTL text mode=2
  ; stretch = Headline text stretcheng function
  ; BF Supports also automatically text upscaling with the function FitImageActivateUpscaling_BF
  ; You can import a existing gadget_ID for creating a Gadget_BF from a Gadget PB : Set x=-2 and y the gadget_ID
  Declare ColorButtonGadget_BF(text$,
                               x=0,
                               y=0,
                               button_width=0,
                               button_height=0,
                               text_color=0,
                               background_color=$E1E1E1,
                               font_ID=-1,
                               stretch_x=0,
                               stretch_y=0,
                               flag=1,
                               mode=0,
                               button_flag=0)
  
  ; Compatible to ColorButtonGadget_BF, but with integrated dual padding function
  Declare ColorButtonGadget_DualPadding_BF(text$,
                                           x=0,
                                           y=0,
                                           button_width=0,
                                           button_height=0,
                                           text_color=0,
                                           background_color=$E1E1E1,
                                           padding_x_gadget=0,
                                           padding_y_gadget=0,
                                           font_ID=-1,
                                           stretch_x=0,
                                           stretch_y=0,
                                           flag=1,
                                           mode=0,
                                           button_flag=0)
  
  ; Fade in and fade out images with ImageGadget_BF - Full automatic mode- This function get and set GadgetData (0 > 1)
  Declare SetButtonImageGadget_BF_AutoFade_BF(gadget_ID,
                                              image_a_ID,
                                              image_b_ID,
                                              background_color,
                                              overlay_color,
                                              animation_delay,
                                              animation_alpha_step)
  
  ; Fade in and fade out images with ImageGadget_BF - Recommendation : Set GadgetData (0 > 1) as state - This function works also with the automatic version
  Declare SetButtonImageGadget_BF_Fade_BF(gadget_ID,
                                          image_a_ID,
                                          image_b_ID,
                                          background_color,
                                          overlay_color,
                                          animation_delay,
                                          animation_alpha_step)
  
  Declare SetButtonImageGadget_BF(gadget_ID,
                                  image_ID=-1,
                                  background_color=-1,
                                  x=-1,
                                  y=-1,
                                  button_width=-1,
                                  button_height=-1,
                                  padding_x_gadget=-1,
                                  padding_y_gadget=-1)
  
  ; You can import a existing gadget_ID for creating a Gadget_BF from a Gadget PB : Set x=-2 and y the gadget_ID
  Declare ButtonImageGadget_BF(image_ID,
                               background_color=$E1E1E1,
                               x=0,
                               y=0,
                               button_width=0,
                               button_height=0,
                               padding_x_gadget=14,
                               padding_y_gadget=14,
                               button_flag=0)
  
  ; Fade in and fade out images with ImageGadget_BF - Full automatic mode- This function get and set GadgetData (0 > 1)
  Declare SetImageGadget_BF_AutoFade_BF(gadget_ID,
                                        image_a_ID,
                                        image_b_ID,
                                        background_color,
                                        overlay_color,
                                        animation_delay,
                                        animation_alpha_step)
  
  ; Fade in and fade out - Recommendation : Set GadgetData (0 > 1) as state - This function works also with the automatic version
  Declare SetImageGadget_BF_Fade_BF(gadget_ID,
                                    image_a_ID,
                                    image_b_ID,
                                    background_color,
                                    overlay_color,
                                    animation_delay,
                                    animation_alpha_step)
  
  Declare SetImageGadget_BF(gadget_ID,           ; Gadget ID
                            image_ID=-1,         ; Image ID
                            x=-1,                ; Output pos x
                            y=-1,                ; Output pos y
                            button_width=-1,     ; Gadget width
                            button_height=-1,    ; Gadget height
                            background_color=-1, ; Background color - For a invisible background set background_color=-2 (-1 is reserved for ignore)
                            padding_gadget=-1,   ; Padding content
                            grid_color=-1,       ; Grid color
                            grid_size=-1,        ; Grid size
                            grid_tiling=-1,      ; Grid tiling
                            frame_color=-1,      ; Frame color
                            frame_size=-1)       ; Frame size)
                                                 ; BF Supports also automatically image upscaling with the function FitImageActivateUpscaling_BF
                                                 ; Use a invisible background only for images with invisible color or you see nothing
                                                 ; ImageGadget_BF create and cache needed temporary images for WinOS outputs automatically
  
  ; Compatible to SetImageGadget_BF, but with integrated dual padding function
  Declare SetImageGadget_DualPadding_BF(gadget_ID, ; Gadget ID
                                        image_ID=-1,         ; Image ID
                                        x=-1,                ; Output pos x
                                        y=-1,                ; Output pos y
                                        button_width=-1,     ; Gadget width
                                        button_height=-1,    ; Gadget height
                                        background_color=-1, ; Background color - For a invisible background set background_color=-2 (-1 is reserved for ignore)
                                        padding_x_gadget=-1, ; Padding content x
                                        padding_y_gadget=-1, ; Padding content y
                                        grid_color=-1,       ; Grid color
                                        grid_size=-1,        ; Grid size
                                        grid_tiling=-1,      ; Grid tiling
                                        frame_color=-1,      ; Frame color
                                        frame_size=-1)       ; Frame size)
  
  ; You can import a existing gadget_ID for creating a Gadget_BF from a Gadget PB : Set x=-2 and y the gadget_ID
  Declare ImageGadget_BF(image_ID=-1,              ; For initializing a image_ID is not needed
                         x=0,                      ; Output pos x
                         y=0,                      ; Output pos y
                         button_width=1,           ; Gadget width
                         button_height=1,          ; Gadget height
                         background_color=$808080, ; Background color - For a invisible background set background_color=-2 (-1 is reserved for ignore)
                         padding_gadget=0,         ; Padding
                         grid_color=$FFFFFF,       ; Grid color
                         grid_size=5,              ; Grid size
                         grid_tiling=10,           ; Grid tiling
                         frame_color=$FFFFFF,      ; Frame color
                         frame_size=3,             ; Frame size
                         flags=0)                  ; Flags same the PB function
                                                   ; The function give back the gadget ID
                                                   ; For a empty image output set image_ID=-1
                                                   ; The Gadget is compatible to the PB ImageGadget 
                                                   ; BF Supports also automatically image upscaling with the function FitImageActivateUpscaling_BF
                                                   ; Use a invisible background only for images with invisible color or you see nothing
                                                   ; ImageGadget_BF create and cache needed temporary images for WinOS outputs automatically
  
  ; Compatible to ImageGadget_BF, but with integrated dual padding function
  Declare ImageGadget_DualPadding_BF(image_ID=-1,              ; For initializing a image_ID is not needed
                                     x=0,                      ; Output pos x
                                     y=0,                      ; Output pos y
                                     button_width=1,           ; Gadget width
                                     button_height=1,          ; Gadget height
                                     background_color=$808080, ; Background color - For a invisible background set background_color=-2 (-1 is reserved for ignore)
                                     padding_x_gadget=0,       ; Padding x
                                     padding_y_gadget=0,       ; Padding y
                                     grid_color=$FFFFFF,       ; Grid color
                                     grid_size=5,              ; Grid size
                                     grid_tiling=10,           ; Grid tiling
                                     frame_color=$FFFFFF,      ; Frame color
                                     frame_size=3,             ; Frame size
                                     flags=0)                  ; Flags same the PB function
  
  ; Fade out - Recommendation : Set GadgetData (0 > 1) as state - This function works also with the automatic version
  Declare SetTextGadget_BF_FadeOut_BF(gadget_ID,
                                      text_a$,
                                      text_b$,
                                      text_color,
                                      background_color,
                                      animation_delay,
                                      animation_alpha_step)
  
  ; Fade in - Recommendation : Set GadgetData (0 > 1) as state - This function works also with the automatic version
  Declare SetTextGadget_BF_FadeIn_BF(gadget_ID,
                                     text_a$,
                                     text_b$,
                                     text_color,
                                     background_color,
                                     animation_delay,
                                     animation_alpha_step)
  
  ; Fade in - Recommendation : Set GadgetData (0 > 1) as state - This function works also With the automatic version
  Declare SetTextGadget_BF_Fade_ChangeTextColor_BF(gadget_ID,
                                                   text_a$,
                                                   text_b$,
                                                   text_color_a,
                                                   text_color_b,
                                                   background_color,
                                                   animation_delay,
                                                   animation_alpha_step)
  
  ; Fade in and fade out - Full automatic mode - This function get and set GadgetData (0 > 1)
  Declare SetTextGadget_BF_AutoFade_BF(gadget_ID,
                                       text_a$,
                                       text_b$,
                                       animation_delay,
                                       animation_alpha_step)
  
  ; Fade in and fade out - Recommendation : Set GadgetData (0 > 1) as state - This function works also with the automatic version
  Declare SetTextGadget_BF_Fade_BF(gadget_ID,
                                   text_a$,
                                   text_b$,
                                   text_color,
                                   background_color,
                                   animation_delay,
                                   animation_alpha_step)
  
  ; Dual color fading - Full automatic mode - This function set GadgetData (0 > 1) as state
  Declare SetTextGadget_BF_Fading_BF(gadget_ID,
                                     text_a$,
                                     text_b$,
                                     text_color_a,
                                     text_color_b,
                                     animation_delay,
                                     animation_alpha_step)
  
  ; Change the TextGadget_BF
  ; The function can handle GradientColor_BF
  ; You must not set position, size, stretch, font, but you can
  ; Use ever a large font size - Then the function adjust the needed font size automatically (about 60 for the mostly large buttons)
  ; flag : Text center=1 - Text right=2 - Text vertical=3
  ; mode : Revers text mode=1 - RTL text mode=2
  ; stretch = Headline text stretcheng function
  ; BF Supports also automatically text upscaling with the function FitImageActivateUpscaling_BF
  Declare SetTextGadget_BF(gadget_ID,
                           text$="",
                           text_color=-1,
                           background_color=-1,
                           x=-1,
                           y=-1,
                           button_width=-1,
                           button_height=-1,
                           stretch_x=-1,
                           stretch_y=-1,
                           font_ID=-1,
                           flag=-1,
                           mode=-1)
  
  ; Compatible to SetTextGadget_BF, but with integrated dual padding function
  Declare SetTextGadget_DualPadding_BF(gadget_ID,
                                       text$="",
                                       text_color=-1,
                                       background_color=-1,
                                       gadget_x_padding=-1,
                                       gadget_y_padding=-1,
                                       x=-1,
                                       y=-1,
                                       button_width=-1,
                                       button_height=-1,
                                       stretch_x=-1,
                                       stretch_y=-1,
                                       font_ID=-1,
                                       flag=-1,
                                       mode=-1)
  
  ; Automatic text size
  ; Automatic text adjustment
  ; Outputs all fonts with edge smoothing
  ; Can handle GradientColor_BF
  ; font_ID you must not set
  ; Use ever a large font size - Then the function adjust the needed font size automatically (about 60 for the mostly large buttons)
  ; flag : Text center=1 - Text right=2 - Text vertical=3
  ; mode : Revers text mode=1 - RTL text mode=2
  ; stretch = Headline text stretcheng function
  ; BF Supports also automatically text upscaling with the function FitImageActivateUpscaling_BF
  ; You can import a existing gadget_ID for creating a Gadget_BF from a Gadget PB : Set x=-2 and y the gadget_ID
  ; This Gadget is canvas based - A needed or setted gadget_ID is a CanvasGadget ID
  Declare TextGadget_BF(text$="",
                        x=0,
                        y=0,
                        button_width=0,
                        button_height=0,
                        text_color=0,
                        background_color=$F0F0F0,
                        font_ID=-1,
                        stretch_x=0,
                        stretch_y=0,
                        flag=0,
                        mode=0,
                        canvas_flag=0)
  
  ; Compatible to TextGadget_BF, but with integrated dual padding function
  Declare TextGadget_DualPadding_BF(text$="",
                                    x=0,
                                    y=0,
                                    button_width=0,
                                    button_height=0,
                                    text_color=0,
                                    background_color=$F0F0F0,
                                    padding_x_gadget=0,
                                    padding_y_gadget=0,
                                    font_ID=-1,
                                    stretch_x=0,
                                    stretch_y=0,
                                    flag=0,
                                    mode=0,
                                    canvas_flag=0)
  
  ; Calculate gadget offsets for a resized Window
  Declare Offset_x_BF(old_window_x_size.f, window_ID, position_x.f)
  
  Declare Offset_y_BF(old_window_y_size.f, window_ID, position_y.f)
  
  ; Add a background image to the function DrawTextCreateImage
  ; Works also with ColorButtonGadget_BF and TextGadget_BF
  ; BF set the parameter for transparence (alpha) so : 1 = without transparence - 256 = full transparence (BF specific)
  Declare DrawTextCreateImage_Add_Image_BF(draw_text_create_image_add_image_,              ; 1= Activate - 0 = deactivate - You must deactivate again
                                           draw_text_create_image_add_image_ID_=-1,        ; Image ID
                                           draw_text_create_image_add_image_x_=0,          ; Output x pos
                                           draw_text_create_image_add_image_y_=0,          ; Output y pos
                                           draw_text_create_image_add_image_ID_width_=-1,  ; Output width
                                           draw_text_create_image_add_image_ID_height_=-1, ; Output height  
                                           draw_text_create_image_add_image_alpha_=-1)     ; Alpha
  
  ; Add a overlay color to all Gadgets_BF
  ; Very nice to fade in smoothly or to get a overlay progress bar in a button
  ; BF set the parameter for transparence (alpha) so : 1 = without transparence - 256 = full transparence (BF specific)
  ; For a automatic full size overlay set fit_image_overlay_width=-1 or fit_image_overlay_height=-1
  ; Not suitable for images with invisible color on Windows OS - Also not useful there
  Declare FitImage_Add_Overlay_Color_BF(add_fit_image_overlay_color_, ; 1= Activate - 0 = deactivate - You must deactivate again
                                        fit_image_overlay_color_=0,   ; Overlay color
                                        fit_image_overlay_alpha_=128, ; Overlay alpha
                                        fit_image_overlay_x_=0,       ; Output x
                                        fit_image_overlay_y_=0,       ; Output y
                                        fit_image_overlay_width_=-1,  ; Overlay width
                                        fit_image_overlay_height_=-1) ; Overlay height
  
  ; Add a overlay frame to all Gadgets_BF
  ; Very nice to fade in smoothly or add to a overlay progress bar in a button
  ; BF set the parameter for transparence (alpha) so : 1 = without transparence - 256 = full transparence (BF specific)
  ; For a automatic full size overlay set fit_frame_overlay_width=-1 or fit_frame_overlay_height=-1
  ; Not suitable for images with invisible color on Windows OS - Also not useful there
  Declare FitImage_Add_Overlay_Frame_BF(add_fit_image_overlay_frame_,         ; 1= Activate - 0 = deactivate - You must deactivate again
                                        fit_image_overlay_frame_color_=0,     ; Frame color
                                        fit_image_overlay_frame_alpha_=0,     ; Image Alpha
                                        fit_image_overlay_frame_x_=0,         ; Output x pos
                                        fit_image_overlay_frame_y_=0,         ; Output y pos
                                        fit_image_overlay_frame_width_=-1,    ; Output width
                                        fit_image_overlay_frame_height_=-1,   ; Output height
                                        fit_image_overlay_frame_thickness_=3) ; Frame thickness
  
  ; For example use ColorButtonGadget_BF, ImageGadget_BF and TextGadget_BF (and all other image based functions with fitted outputs) the functions FitImage_BF and FitImage_Pro_BF
  ; With this both functions you can easily scale text and image output also on demand, which makes very nice animation effects possible
  ; You must deactivate again - 0=deactivated
  Declare FitImagePadding_Offset_x_BF(fit_image_padding_offset_x_)
  
  Declare FitImagePadding_Offset_y_BF(fit_image_padding_offset_y_)
  
  ; Manual text padding X for DrawTextCreateImage_BF - Works with all DrawTextCreateImage_BF based functions
  ; As sample with ColorButtonGadget_BF, TextGadget_BF - Nice usable for on demand text scaling - Negative values available
  ; You must deactivate again - 0=deactivated
  Declare ManualPadding_DrawTextCreateImage_X_BF(manual_padding_draw_text_create_image_X)
  
  ; Manual text padding Y for DrawTextCreateImage_BF - Works with all DrawTextCreateImage_BF based functions
  ; As sample with ColorButtonGadget_BF, TextGadget_BF - Nice usable for on demand text scaling - Negative values available
  ; You must deactivate again - 0=deactivated 
  Declare ManualPadding_DrawTextCreateImage_Y_BF(manual_padding_draw_text_create_image_Y)
  
  CompilerIf #PB_Compiler_Version=>560
    Declare ImageFrameCount_BF(gif_ID) ; Workaraound PB 570 Linux Mac - A simple image crash the function
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Declare SetImageFrame_BF(gif_ID, frame) ; Workaround PB570 - PB incompatibility - A image give back 0 on Linux and MacOS, on Windows 1
  CompilerEndIf
  
  Declare.f GetResolution_DPI_X_BF() ; Get resolution x
  
  Declare.f GetResolution_DPI_Y_BF() ; Get resolution y
  
  Declare.f DPI_ResizeFactor_BF() ; The actual DPI resize factor - Based on x
  
  Declare.f DPI_ResizeFactor_X_BF() ; The actual DPI resize factor x
  
  Declare.f DPI_ResizeFactor_Y_BF() ; The actual DPI resize factor y
  
  Declare Deactivate_DPI_ResizeFactor_BF(deactivate_dpi_aware_) ; This function set the resulted value from DPI_ResizeFactor_BF = 1
  
  ; This function activate DPI aware enhancement for many BF functions, as sample Gadgets BF and FitImage BF and many more other
  Declare Activate_DPI_aware_BF(activate_dpi_aware_) 
  
  ; Get the state from Activate_DPI_aware_BF
  Declare GetState_Activate_DPI_aware_BF()
  
  ; Grab the desktop width
  Declare DesktopWidth_BF(desktop=0) ; The first desktop is 0 - It's the preset
  
  ; Grab the desktop height
  Declare DesktopHeight_BF(desktop=0) ; The first desktop is 0 - It's the preset
  
  ; Grab the desktop resize factor width - You must not activate the PB DPI aware compiler option
  ; This function allows you to easily determine the maximum available width for output on the desktop with OS scaling settings above 100%
  Declare.f Desktop_ResizeFactor_Width_BF(desktop=0) ; The first desktop is 0 - It's the preset
  
  ; Grab the desktop resize factor height - You must not activate the PB DPI aware compiler option
  ; This function allows you to easily determine the maximum available height for output on the desktop with OS scaling settings above 100%
  Declare.f Desktop_ResizeFactor_Height_BF(desktop=0) ; The first desktop is 0 - It's the preset
  
  ; Grab the user scaled desktop width
  ; This function allows you to easily determine the maximum available width for output on the desktop with OS scaling settings above 100%
  Declare DesktopWidth_UserScaled_BF(desktop=0) ; The first desktop is 0 - It's the preset
  
  ; Grab the user scaled desktop height
  ; This function allows you to easily determine the maximum available height for output on the desktop with OS scaling settings above 100%
  Declare DesktopHeight_UserScaled_BF(desktop=0) ; The first desktop is 0 - It's the preset
  
  ; Grab the user scaled desktop scaling width
  ; This function allows you to easily determine the OS desktop width scaling as percent
  Declare Desktop_ScalingPercent_Width_BF(desktop=0) ; The first desktop is 0 - It's the preset
  
  ; Grab the user scaled desktop scaling width
  ; This function allows you to easily determine the OS desktop height scaling as percent
  Declare Desktop_ScalingPercent_Height_BF(desktop=0) ; The first desktop is 0 - It's the preset
  
  ; This function give back the type from a Gadget_BF
  ; #GadgetTypeUnknown_BF
  ; #ButtonImageGadget_BF
  ; #ColorButtonGadget_BF
  ; #ImageGadget_BF
  ; #TextGadget_BF
  ; #ColorButtonGadget_DualPadding_BF
  ; #ImageGadget_DualPadding_BF
  ; #TextGadget_DualPadding_BF
  ; #ToggleImageGadget_BF
  ; #ToggleCanvasGadget_BF
  ; #StretchTextImageGadget_BF
  ; #StretchTextCanvasGadget_BF
  ; #StretchTextImageGadget_Pro_BF
  ; #StretchTextButtonImageGadget_Pro_BF
  ; #StretchTextCanvasGadget_Pro_BF
  Declare GetGadgetTyp_BF(gadget_ID)
  
  Declare GetGadgetUsedImage_BF(gadget_ID) ; For Gadgets_BF only - Fail results -1
  
  Declare GetGadgetImage_BF(gadget_ID) ; For Gadgets_BF only - Fail results -1
  
  Declare.s GetGadgetText_BF(gadget_ID) ; For Gadgets_BF only - Fail results ""
  
  Declare GetGadgetX_BF(gadget_ID) ; For Gadgets_BF only - Fail results 0
  
  Declare GetGadgetY_BF(gadget_ID) ; For Gadgets_BF only - Fail results 0
  
  Declare GetGadgetWidth_BF(gadget_ID) ; For Gadgets_BF only - Fail results 0
  
  Declare GetGadgetHeight_BF(gadget_ID) ; For Gadgets_BF only - Fail results 0
  
  Declare GetGadgetTextColor_BF(gadget_ID) ; For Gadgets_BF only - Fail results -1
  
  Declare GetGadgetBackgroundColor_BF(gadget_ID) ; For Gadgets_BF only - Fail results -1
  
  ; Needed for resizing Gadgets_BF - Works also with PB Gadgets - Fail get a negative result
  Declare ResizeGadget_BF(gadget_ID, x, y, width, height, text_width=0, text_height=0)
  
  ; The following four functions calculate fitted font width, height and for any gadgets or rectangles
  ; DrawText_BF support padding and text stretching (Headline), so you can handle this also with this functions
  ; For padding you can also set negative values 
  ; The following sample declarations are necessary for the first three font array based functions:
  ; Dim font(max_needed_font_size) ; 40 > 60 is mostly enough
  ; Define needed_font_size
  ; For needed_font_size=1 To max_needed_font_size ; Get the fonts
  ;   font(needed_font_size)=LoadFont(#PB_Any, "", needed_font_size, #PB_Font_Bold)
  ; Next needed_font_size
  
  ; Calculate fitted font width
  Declare Calculate_fitted_Font_Width_BF(text$,
                                         width,
                                         Array font(1),
                                         vertical=0,
                                         padding_x=0,
                                         stretch_x=0,
                                         resize_factor.f=1)
  
  ; Calculate fitted font height
  Declare Calculate_fitted_Font_Height_BF(text$,
                                          height,
                                          Array font(1),
                                          vertical=0,
                                          padding_y=0,
                                          stretch_y=0,
                                          resize_factor.f=1)
  
  ; Calculate a fitted font for any gadgets or rectangles
  Declare Calculate_fitted_Font_BF(text$,
                                   width_rectangle,
                                   height_rectangle,
                                   Array font(1),
                                   vertical=0,
                                   padding_x=0,
                                   padding_y=0,
                                   stretch_x=0,
                                   stretch_y=0,
                                   resize_factor.f=1)
  
  Declare Get_Fitted_Font_Padding_X_BF() ; Get the actual padding X
  
  Declare Get_Fitted_Font_Padding_Y_BF() ; Get the actual padding Y
  
  ; Create a enhanced text output for any CanvasGadgets PB
  ; The function returns the determined font size so that it can easily be inherit on to other gadgets
  ; This is also possible on gadgets of other sizes - Sample : alterable_size=calculated_font_size/my_resize_factor
  Declare SetCanvasCustomText_BF(canvas_ID,               ; Any CanvasGadget ID
                                 Array Font(1),           ; Set a Font array, for simple creating any look the sample here
                                 output_x=0,              ; Output pos x  - Ignore = -1
                                 output_y=0,              ; Output pos y  - Ignore = -1
                                 canvas_width=0,          ; Canvas width  - Ignore = -1
                                 canvas_height=0,         ; Canvas height - Ignore = -1
                                 canvas_text$=" ",        ; Text
                                 text_color=#Black,       ; Text color
                                 background_color=#White, ; Background color
                                 flag=1,                  ; Flags : Text center=1 - Text right=2 - Text vertical=3
                                 mode=0,                  ; Modes : Revers text mode=1 - RTL text mode=2
                                 padding_x=0,             ; Text padding x
                                 padding_y=0,             ; Text padding y
                                 text_stretching_x=0,     ; Headline text stretching x
                                 text_stretching_y=0,     ; Headline text stretching y
                                 resize_factor.f=1)       ; Text resize factor
  
  ;; Create a font array for using with the function SetCanvasCustomText_BF and SetGadgetText
  ;; Mostly 30pt > 40pt is enough - smaller is faster
  ;   max_needed_font_size=40 ; Set the max needed font size for any output with this function
  ;   Global Dim Font_Arial(max_needed_font_size) ; For using in a callback procedure define as Global 
  ;   Define needed_font_size
  ;   For needed_font_size=1 To max_needed_font_size ; Get the fonts
  ;     Font_Arial(needed_font_size)=LoadFont(#PB_Any, "Courier New", needed_font_size, #PB_Font_Bold)
  ;   Next needed_font_size
  
  ; Create a fitted enhanced text output - Works with images, PB sprites, window, screen and canvas
  ; The function returns the determined font size so that it can easily be inherit
  ; This is also possible for using of other sizes - Sample : alterable_size=calculated_font_size/my_resize_factor
  ; Padding reduces or increases the size of the calculated font
  ; For screen output set firstly ActivateScreenOutput_BF(1)
  ; This function is GradientColor BF sensitive
  Declare FittedText_BF(output_ID,               ; Any image, canvas, window ID - For screen output set simple 0
                        Array Font(1),           ; Set a Font array, for simple creating any look the sample here
                        output_x=0,              ; Output pos x  - Ignore = -1
                        output_y=0,              ; Output pos y  - Ignore = -1
                        output_width=0,          ; Canvas width  - Ignore = -1
                        output_height=0,         ; Canvas height - Ignore = -1
                        text$=" ",               ; Text
                        text_color=#Black,       ; Text color
                        background_color=#White, ; Background color - For invisible background set to -1
                        frame_thickness=0,       ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                        frame_color=#Black,      ; Frame color - The frame is primary a positioning helper function for developing
                        flag=1,                  ; Flags : Text center=1 - Text right=2 - Text vertical=3
                        mode=0,                  ; Modes : Revers text mode=1 - RTL text mode=2
                        padding_x=0,             ; Text padding x
                        padding_y=0,             ; Text padding y
                        text_stretching_x=0,     ; Headline text stretching x
                        text_stretching_y=0,     ; Headline text stretching y
                        resize_factor.f=1)       ; Text resize factor
  
  ; Same FittedText above, but with additional RoundBox feature and vector output
  ; You can set frame_color and background_color as RGBA, but you must not
  ; You can set text and background color as RGBA, but you must not - Alpha 0 is ignored and equal Alpha 255
  ; Text with alpha use the vector lib for text output and reduce the text quality from little fonts because interpolation
  Declare FittedText_RoundBox_Vector_BF(output_ID,
                                        Array Font(1),           ; Set a Font array, for simple creating any look the sample here
                                        output_x=0,              ; Output pos x  - Ignore = -1
                                        output_y=0,              ; Output pos y  - Ignore = -1
                                        output_width=0,          ; Canvas width  - Ignore = -1
                                        output_height=0,         ; Canvas height - Ignore = -1
                                        radius=0,                ; Edge rounding radius x
                                        text$=" ",               ; Text
                                        text_color=#Black,       ; Text color
                                        background_color=#White, ; Background color
                                        frame_thickness=0,       ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                                        frame_color=#Black,      ; Frame color
                                        flag=1,                  ; Flags : Text center=1 - Text right=2 - Text vertical=3
                                        mode=0,                  ; Modes : Revers text mode=1 - RTL text mode=2
                                        padding_x=0,             ; Text padding x
                                        padding_y=0,             ; Text padding y
                                        text_stretching_x=0,     ; Headline text stretching x
                                        text_stretching_y=0,     ; Headline text stretching y
                                        resize_factor.f=1)       ; Text resize factor                                   
  
  ; This function scale the FittedText_RotateRoundBox_Vector_y_BF, all VectorText_Simple functions and ProgressBar_BF-text output width
  Declare.d ScaleVectorWidth_BF(set_scaling_x_.d=1.0)
  
  ; This function scale the FittedText_RotateRoundBox_Vector_y_BF, all VectorText_Simple functions and ProgressBar_BF-text output height
  Declare.d ScaleVectorHeight_BF(set_scaling_y_.d=1.0)
  
  ; Give back the scale factor for FittedText_RotateRoundBox_Vector_y_BF and all VectorText_Simple functions output width
  Declare.d Get_ScaleVectorWidth_BF()
  
  ; Give back the scale factor for FittedText_RotateRoundBox_Vector_y_BF and all VectorText_Simple functions output height
  Declare.d Get_ScaleVectorHeight_BF()
  
  ; Same FittedText above, but with additional RoundBox and rotating feature and vector output
  ; You can set frame_color and background_color as RGBA, but you must not
  ; You can set text and background color as RGBA, but you must not - Alpha 0 is ignored and equal Alpha 255
  ; Text with alpha use the vector lib for text output and reduce the text quality from little fonts because interpolation
  
  ; PB 570 Workaround - Windows
  ; For this function you can use for line breaks #CRLF$ or #LF$ - Faster is #CRLF$ - BF convert automatically #LF$ to #CRLF$
  ; Other line breaks can cause graphic errors
  Declare FittedText_RotateRoundBox_Vector_BF(output_ID,
                                              Array Font(1),           ; Set a Font array, for simple creating any look the sample here
                                              output_x=0,              ; Output pos x  - Ignore = -1
                                              output_y=0,              ; Output pos y  - Ignore = -1
                                              output_width=0,          ; Canvas width  - Ignore = -1
                                              output_height=0,         ; Canvas height - Ignore = -1
                                              degree=0,                ; Rotating angle
                                              radius=0,                ; Edge rounding radius x
                                              text$=" ",               ; Text
                                              text_color=#Black,       ; Text color
                                              background_color=#White, ; Background color
                                              frame_thickness=0,       ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                                              frame_color=#Black,      ; Frame color
                                              flag=1,                  ; Flags : Text center=1 - Text right=2 - Text vertical=3
                                              padding_x=0,             ; Text padding x
                                              padding_y=0,             ; Text padding y
                                              resize_factor.f=1)       ; Text resize factor                                  
  
  ; Same FittedText above, but with additional RoundBox feature - Also for output on sprites and screen
  Declare FittedText_RoundBox_BF(output_ID,
                                 Array Font(1),           ; Set a Font array, for simple creating any look the sample here
                                 output_x=0,              ; Output pos x  - Ignore = -1
                                 output_y=0,              ; Output pos y  - Ignore = -1
                                 output_width=0,          ; Canvas width  - Ignore = -1
                                 output_height=0,         ; Canvas height - Ignore = -1
                                 radius=0,                ; Edge rounding radius
                                 text$=" ",               ; Text
                                 text_color=#Black,       ; Text color
                                 background_color=#White, ; Background color
                                 frame_thickness=0,       ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                                 frame_color=#Black,      ; Frame color
                                 flag=1,                  ; Flags : Text center=1 - Text right=2 - Text vertical=3
                                 mode=0,                  ; Modes : Revers text mode=1 - RTL text mode=2
                                 padding_x=0,             ; Text padding x
                                 padding_y=0,             ; Text padding y
                                 text_stretching_x=0,     ; Headline text stretching x
                                 text_stretching_y=0,     ; Headline text stretching y
                                 resize_factor.f=1)       ; Text resize factor
                                                          ; Hint : Frame thickness 2 is not supported for a RoundBox output
  
  ; Fits an existing gadget text exactly for gadgets PB
  ; This is a wonderful function that can solve text size problems like magic
  ; The function returns the determined font size so that it can easily be inherit on to other gadgets
  ; This is also possible on gadgets of other sizes - Sample : alterable_size=calculated_font_size/my_resize_factor
  Declare FitGadgetText_BF(gadget_ID,         ; Any Gadget ID
                           Array Font(1),     ; Set a Font array, for simple creating any look the sample above
                           output_x,          ; Output pos x  - Ignore = -1
                           output_y,          ; Output pos x  - Ignore = -1
                           gadget_width,      ; Gadget width  - Ignore = -1
                           gadget_height,     ; Gadget height - Ignore = -1
                           padding_x=0,       ; Text padding x
                           padding_y=0,       ; Text padding y
                           resize_factor.f=1) ; Text resize factor
  
  ; Create a enhanced text output for Gadgets PB
  ; The function returns the determined font size so that it can easily be inherit on to other gadgets
  ; This is also possible on gadgets of other sizes - Sample : alterable_size=calculated_font_size/my_resize_factor
  Declare SetGadgetText_BF(gadget_ID,         ; Any Gadget ID
                           Array Font(1),     ; Set a Font array, for simple creating any look the sample here
                           output_x,          ; Output pos x  - Ignore = -1
                           output_y,          ; Output pos x  - Ignore = -1
                           gadget_width,      ; Gadget width  - Ignore = -1
                           gadget_height,     ; Gadget height - Ignore = -1
                           gadget_text$=" ",  ; Text
                           padding_x=0,       ; Text padding x
                           padding_y=0,       ; Text padding y
                           resize_factor.f=1) ; Text resize factor
  
  ; Simple BF enhanced ButtonGadget PB
  ; Font Array needed - look as sample the Dynamic_Gadgets demo codes
  Declare ButtonGadgetSimple_BF(text$,             ; Text
                                Array Font(1),     ; Font array 
                                x,                 ; x_pos
                                y,                 ; y_pos
                                width,             ; Width
                                height,            ; Height
                                padding_x=-1,      ; Text padding x - -1 = Preset 25 
                                padding_y=-1,      ; Text padding y - -1 = Preset 15
                                resize_factor.f=1) ; resize_factor.f
  
  ; Simple BF enhanced TextGadget PB
  ; Font Array needed - look as sample the Dynamic_Gadgets demo codes
  Declare TextGadgetSimple_BF(text$,             ; Text
                              Array Font(1),     ; Font array 
                              x,                 ; x_pos
                              y,                 ; y_pos
                              width,             ; Width
                              height,            ; Height
                              padding_x=-1,      ; Text padding x - -1 = Preset 25 
                              padding_y=-1,      ; Text padding y - -1 = Preset 15
                              resize_factor.f=1, ; resize_factor.f
                              flags=0)           ; Flags - Same TextGadget PB
  
  ; Background creating function
  ; Texture settings and many other you must not set, but you can
  Declare CreateBackgroundImage_BF(overlay_ID,                        ; Overlay image_ID
                                   texture_ID,                        ; Background texture ID
                                   output_width,                      ; Resulted image width
                                   output_height,                     ; Resulted image height
                                   overlay_x=0,                       ; Overlay x pos - 0 = Resulted image width
                                   overlay_y=0,                       ; Overlay y pos - 0 = Resulted image height
                                   overlay_width=0,                   ; Overlay width
                                   overlay_height=0,                  ; Overlay height
                                   overlay_alpha=180,                 ; Overlay image alpha
                                   texture_alpha=2,                   ; Texture alpha - BF specivic - 0 = full visible
                                   activate_overlay_sprite_mode=1,    ; Use a masked image as overlay - Set to 2 for a image with alpha channel
                                   color_distance_overlay=0,          ; Color distance oberlay image - This remove border artifacts
                                   set_linear_gradient_color=$808080, ; Set linear gradient color - From background color to this color - Best a grey about $808080   
                                   set_background_color=$FFFFFF,      ; Background color for use with linear gradient output
                                   texture_x=0,                       ; Startposition texture output x - Preset 0
                                   texture_y=0,                       ; Startposition texture output y - Preset 0
                                   texture_width=0,                   ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                                   texture_height=0,                  ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                                   texture_clip_x=0,                  ; Startposition inside the texture x - Preset 0
                                   texture_clip_y=0,                  ; Startposition inside the texture y - Preset 0
                                   texture_clip_width=0,              ; Endposition inside the texture - Preset 0 = full texture width
                                   texture_clip_height=0,             ; Endposition inside the texture - Preset 0 = full texture height
                                   texture_resize_x=0,                ; Pre resize texture x
                                   texture_resize_y=0)                ; Pre resize texture 
  
  ; Settings for SetFrontText_Progressbar_BF - Multiline available
  ; txt$          - Mostly as percentage
  ; txt_x         - Output pos x
  ; txt_y         - Output pos y
  ; txt_direction - rotating arc as degree
  ; txt_color     - Text color
  ; txt_alpha     - Set alpha blending for text from 1 to 256 - Set 1=full visible - 256=invisible (Tool and GFX_Wizzard_BF specific)
  ; font_ID       - You can select a font (ID) for using - but you must not - You can set as sample #SysFont_BF or #SysFontBold_BF
  ; This function add a text inside the bar_Progressbar_BF
  ; Use this function over a SetGadgeState_Progressbar_BF call
  
  Declare SetFrontText_0_Progressbar_BF(txt_$="",
                                        txt_x_=0,
                                        txt_y_=0,
                                        txt_direction_.f=0,
                                        txt_color_=0,
                                        txt_alpha_=0,
                                        font_ID_=-1)
  
  Declare SetFrontText_1_Progressbar_BF(txt_$="",
                                        txt_x_=0,
                                        txt_y_=0,
                                        txt_direction_.f=0,
                                        txt_color_=0,
                                        txt_alpha_=0,
                                        font_ID_=-1)
  
  Declare SetFrontText_2_Progressbar_BF(txt_$="",
                                        txt_x_=0,
                                        txt_y_=0,
                                        txt_direction_.f=0,
                                        txt_color_=0,
                                        txt_alpha_=0,
                                        font_ID_=-1)
  
  Declare SetFrontText_3_Progressbar_BF(txt_$="",
                                        txt_x_=0,
                                        txt_y_=0,
                                        txt_direction_.f=0,
                                        txt_color_=0,
                                        txt_alpha_=0,
                                        font_ID_=-1)
  
  Declare SetFrontText_4_Progressbar_BF(txt_$="",
                                        txt_x_=0,
                                        txt_y_=0,
                                        txt_direction_.f=0,
                                        txt_color_=0,
                                        txt_alpha_=0,
                                        font_ID_=-1)
  
  ; Set a dividing line on a Progressbar_BF - -1 is presetted for ignore
  Declare Set_DividingLine_Progressbar_BF(divisor_progressbar_pos_=-1, divisor_progressbar_color_=#Red, divisor_progressbar_alpha_=255)
  
  If Not Defined(PB_ProgressBar_Horizontal, #PB_Constant) : #PB_ProgressBar_Horizontal= 0 : EndIf
  If Not Defined(PB_ProgressBar_Remove    , #PB_Constant) : #PB_ProgressBar_Remove    =-2 : EndIf
  
  ; This function calculate the pixel based slider position from a Progressbar_BF for text positioning
  Declare Calculate_SliderPos_Progressbar_BF(bar_ID, pos, direction=#PB_ProgressBar_Horizontal, speed_factor.d=1) ; #PB_ProgressBar_Vertical - #PB_ProgressBar_Horizontal
  
  ; Progressbar_BF
  ; This function replace SetGadgetState()
  ; This function is for simplest replacing OS progressbars on all OS with multi color custom progressbars
  ; The created custom progressbars are canvas gadgets, the tool give back this canvas gadgets and it ID's
  ; Also the tool can give back only a image (ID) for fine post processing, as sample with the GFX_Wizzard_BF
  ; Replace simple SetGadgetState with SetGadgetState_Progressbar_BF and add parameters you want, more you must not do
  
  ; Sample call :
  ; SetGadgetState(bar_ID, 100*writen/file_size) ; OS progress bar
  ; SetGadgetState_Progressbar_BF(bar_ID, 100*writen/file_size), flags, mode, slider_color_1, background_color) ; Progressbar_BF
  
  ; Available flags :
  ; #PB_ProgressBar_Vertical
  ; #PB_ProgressBar_Unknown
  ; #PB_ProgressBar_Horizontal
  ; #PB_ProgressBar_Remove
  
  ; mode=1 - Gradient color - Horizontal direction - mode=-1 same mode=1 but only image creating
  ; mode=2 - Gradient color - Vertical direction - mode=-2 same mode=1 but only image creating
  ; For disable automatically gradient_color set slider_color_2=-1
  ; For moving the linear gradient output position set gradient_color_offset as percent
  ; For using #PB_ProgressBar_Unknown, you must handle the animation self (Maybe threaded - I will not preset threads here)
  
  Declare SetGadgetState_Progressbar_BF(bar_ID,                   ; Gadget ID
                                        state,                    ; Value - State
                                        flags=0,                  ; Direction
                                        mode=0,                   ; Mode
                                        slider_color_1=#Green,    ; Slider color 1
                                        background_color=$E6E6E6, ; Background color
                                        slider_color_2=-1,        ; Slider color 2 - Additional slider color
                                        border=-1,                ; Border thickness - -1 activate the CanvasGadget() border, >0 a custom border
                                        border_color=$AAAAAA,     ; Border color
                                        linear_gradient_offset=0) ; Linear gradient output offset - A negative value flip the slider colors
  
  ; Add a Background image (Texture) to MagicTextGadget_BF - You must deactivate again
  ; With automatically tilling, scrollable on demand
  Declare MagicStretchText_Add_Background_BF(MagicStretchText_Add_Background_ID_=-1,
                                             MagicStretchText_Add_Background_x_pos_=0,  ; Output x pos
                                             MagicStretchText_Add_Background_y_pos_=0,  ; Output y pos
                                             MagicStretchText_Add_Background_width_=0,  ; 0 = original width
                                             MagicStretchText_Add_Background_height_=0, ; 0 = original height
                                             MagicStretchText_Add_Background_alpha_=0)  ; BF specific 0 = full visible
  
  ; #0 Add a percent based fitted proportional image to MagicTextGadget_BF - You must deactivate again
  Declare MagicStretchText_Add_Image_BF(MagicStretchText_Add_Image_ID_=-1,
                                        MagicStretchText_Add_Image_x_=0,
                                        MagicStretchText_Add_Image_y_=0,
                                        MagicStretchText_Add_Image_width_percent_=100,
                                        MagicStretchText_Add_Image_height_percent_=100, 
                                        MagicStretchText_Add_Image_alpha_=255,
                                        MagicStretchText_Add_Image_sticky_=0)
  
  ; #1 Add a percent based fitted proportional image to MagicTextGadget_BF - You must deactivate again
  Declare MagicStretchText_Add_Image_1_BF(MagicStretchText_Add_Image_1_ID_=-1,
                                          MagicStretchText_Add_Image_1_x_=0,
                                          MagicStretchText_Add_Image_1_y_=0,
                                          MagicStretchText_Add_Image_1_width_percent_=100,
                                          MagicStretchText_Add_Image_1_height_percent_=100, 
                                          MagicStretchText_Add_Image_1_alpha_=255,
                                          MagicStretchText_Add_Image_1_sticky_=0)
  
  ; #2 Add a percent based fitted proportional image to MagicTextGadget_BF - You must deactivate again
  Declare MagicStretchText_Add_Image_2_BF(MagicStretchText_Add_Image_2_ID_=-1,
                                          MagicStretchText_Add_Image_2_x_=0,
                                          MagicStretchText_Add_Image_2_y_=0,
                                          MagicStretchText_Add_Image_2_width_percent_=100,
                                          MagicStretchText_Add_Image_2_height_percent_=100, 
                                          MagicStretchText_Add_Image_2_alpha_=255,
                                          MagicStretchText_Add_Image_2_sticky_=0)
  
  ; Calculate padding x and padding y for proportional text output with MagicTextGadget_BF 
  ; Give back GetFitSize_Width_BF() and GetFitSize_Height_BF()
  Declare Calculate_MagicText_proportional_BF(text$,
                                              font_ID,
                                              text_width,
                                              text_height,
                                              text_stretching_x=0,
                                              text_stretching_y=0,
                                              vertical=0)
  
  ; Add a ProgressBar inside the gadget
  ; Width and height -frame thickness - You must deactivate again - Alpha channel supported
  ; You can set the output to layer 0 or 1
  Declare MagicStretchText_Add_ProgressBar_BF(MagicStretchText_Add_Progressbar_percent_=0,
                                              MagicStretchText_Add_Progressbar_color_=#Green,
                                              MagicStretchText_Add_Progressbar_layer_=0)
  
  ; A fully automated text output and pixel accurate gadget available as ButtonImageGadget and CanvasGadget
  ; Works with Magic_StretchText_Pro_TextOffset_BF()
  ; This function deactivate frames
  Declare MagicTextGadget_BF(gadget_type,              ; 0 = ButtonImageGadget - 1 = CanvasGadget - 2 ImageGadget
                             text$,                    ; Text
                             x_pos,                    ; Gadget output x pos
                             y_pos,                    ; Gadget output x pos
                             button_width,             ; Button width
                             button_height,            ; Button height
                             text_width,               ; Text width
                             text_height,              ; Text height
                             stretch_x=0,              ; Text stretching x - Headline mode
                             stretch_y=0,              ; Text stretching y - Headline mode
                             padding_x=5,              ; padding_x
                             padding_y=5,              ; Padding y
                             text_color=#White,        ; Text color - Alpha channel supported
                             background_color=$EEEEEE, ; Background color - For invisible background set to -1
                             text_flag=1,              ; Text flag - Text center=1 - Text right=2 - Text vertical=3
                             font$="Arial",            ; Font type - more is not needed - accept also ""
                             font_style=0,             ; Font style
                             border_flag=0)            ; Border flag
  
  ; Set function for the Gadget above
  Declare SetMagicTextGadget_BF(gadget_ID,           ; Gadget_ID - look the demo code for using
                                button_width=-1,     ; Button width
                                button_height=-1,    ; Button height
                                text_width=-1,       ; Text width
                                text_height=-1,      ; Text height
                                x_pos=-1,            ; Gadget output x pos
                                y_pos=-1,            ; Gadget output x pos
                                text$="",            ; Text
                                stretch_x=-1,        ; Text stretching x - Headline mode
                                stretch_y=-1,        ; Text stretching y - Headline mode
                                padding_x=-1,        ; Padding x
                                padding_y=-1,        ; Padding_y
                                text_color=-1,       ; Text color
                                background_color=-1) ; Background color
  
  ; Set function for the Gadget above
  ; Very fine for using with a dynamic GUI
  ; If there is an image as background in a MagicGadget it is optimal to resize this gadget proportionally to not change the design
  Declare SetMagicTextGadget_Proportional_BF(gadget_ID,           ; Gadget_ID - look the demo code for using
                                             resize_factor.d,     ; Resize factor - As sample 0.7
                                             text_width=-1,       ; Text width
                                             text_height=-1,      ; Text height
                                             x_pos=-1,            ; Gadget output x pos
                                             y_pos=-1,            ; Gadget output x pos
                                             text$="",            ; Text
                                             stretch_x=-1,        ; Text stretching x - Headline mode
                                             stretch_y=-1,        ; Text stretching y - Headline mode
                                             padding_x=-1,        ; Padding x
                                             padding_y=-1,        ; Padding_y
                                             text_color=-1,       ; Text color
                                             background_color=-1) ; Background color
  
  ; This function create and output a Stretched text - The ToggleGadget_BF create also stretched text
  ; This function is GradientColor BF sensitive - (LinearGradient and CircularGradient)
  ; Available Canvas and ImageGadget based - Look further down, ImageGadget is presetted, changeable with CanvasToggleGadget_BF() set flag function
  ; Usable as text and button gadget
  Declare StretchTextGadget_BF(text$,                    ; Text
                               x_pos,                    ; Gadget output x pos
                               y_pos,                    ; Gadget output y pos
                               button_width,             ; Gadget width
                               button_height,            ; Gadget height
                               padding_x=0,              ; Padding x
                               padding_y=0,              ; Padding y
                               text_color=#White,        ; Text color - Alpha channel supported
                               background_color=$EEEEEE, ; Background color
                               font$="Arial",            ; Font type - more is not needed - accept also ""
                               font_style=0)             ; Font style
  
  ; This function is the base function for setting and output all StretchTextGadget_BF parameters
  ; This function is GradientColor BF sensitive - (LinearGradient and CircularGradient)
  ; The function accept -1 for ignore parameters
  Declare SetStretchTextGadget_BF(gadget_ID,           ; Gadget_ID
                                  text$="",            ; Text
                                  x_pos=-1,            ; Gadget output x pos
                                  y_pos=-1,            ; Gadget output y pos
                                  button_width=-1,     ; Gadget width
                                  button_height=-1,    ; Gadget height
                                  padding_x=-1,        ; Padding x
                                  padding_y=-1,        ; Padding y
                                  text_color=-1,       ; Text color
                                  background_color=-1) ; Background color
  
  ; This Function get the StretchTextGadget_BF Padding X
  ; Fail results also 0
  Declare GetPadding_X_StretchTextGadget_BF(gadget_ID)
  
  ; This Function get the StretchTextGadget_BF Padding Y
  ; Fail results also 0
  Declare GetPadding_Y_StretchTextGadget_BF(gadget_ID)
  
  ; Pre function call for StretchTextGadget_BF - Create and use a StretchTextGadget_BF canvas based
  Declare CanvasStretchTextGadget_BF()
  
  ; Gadget PB flags StretchTextGadget_BF - Same PB flags for CanvasGadget and ImageGadget
  Declare SetFlagsStretchTextGadget_BF(flags_stretchtext_gadget_)
  
  ; This Function set the StretchTextGadget_BF text flags - #PB_VectorParagraph_Left, #PB_VectorParagraph_Right, #PB_VectorParagraph_Center
  Declare SetTextFlagsStretchTextGadget_BF(text_flag_stretchtext_gadget_=#PB_VectorParagraph_Center)
  
  ; This function create and output a ToggleGadget_BF with automatic stretched text
  ; This function is GradientColor BF sensitive - (LinearGradient and CircularGradient)
  ; Available Canvas and ImageGadget based - Look further down, ImageGadget is presetted, changeable with CanvasToggleGadget_BF() set flag function
  ; Usable as text and button gadget
  Declare ToggleGadget_BF(state,                    ; Toggle state 0 or 1 , -1 deactivate the toggle feature
                          text$,                    ; Text
                          x_pos,                    ; Gadget output x pos
                          y_pos,                    ; Gadget output y pos
                          button_width,             ; Gadget width
                          button_height,            ; Gadget height
                          padding_x=0,              ; Padding x
                          padding_y=0,              ; Padding y
                          text_color=#White,        ; Text color
                          slider_color=#Red,        ; Slider color
                          background_color=$EEEEEE, ; Background color
                          font$="Arial",            ; Font type - more is not needed - accept also ""
                          font_style=0)             ; Font style
  
  ; This function is the base function for setting and output all ToggleGadget_BF parameters
  ; This function is GradientColor BF sensitive - (LinearGradient and CircularGradient)
  ; The function accept -1 for ignore parameters
  Declare SetToggleGadget_BF(gadget_ID,           ; Gadget_ID
                             state=-1,            ; Toggle state 0 or 1 , -1 deactivate the toggle feature
                             text$="",            ; Text
                             x_pos=-1,            ; Gadget output x pos
                             y_pos=-1,            ; Gadget output y pos
                             button_width=-1,     ; Gadget width
                             button_height=-1,    ; Gadget height
                             padding_x=-1,        ; Padding x
                             padding_y=-1,        ; Padding y
                             text_color=-1,       ; Text color
                             slider_color=-1,     ; Slider color
                             background_color=-1) ; Background color
  
  ; This Function get the ToggleGadget_BF toggle state - 0 or 1 - Fail results also 0
  Declare GetStateToggleGadget_BF(gadget_ID)
  
  ; This Function get the ToggleGadget_BF Padding X
  ; Fail results also 0
  Declare GetPadding_X_ToggleGadget_BF(gadget_ID)
  
  ; This Function get the ToggleGadget_BF Padding Y
  ; Fail results also 0
  Declare GetPadding_Y_ToggleGadget_BF(gadget_ID)
  
  ; This function toggle the ToggleGadget_BF and can set text, text_color and slider_color
  ; This function is GradientColor BF sensitive - (LinearGradient and CircularGradient)
  ; The function accept -1 for ignore parameter
  Declare ToggleToggleGadget_BF(gadget_ID,
                                state,
                                text$="",
                                text_color=-1,
                                slider_color=-1)
  
  ; For creating a full gadget width and height centered text ToggleGadget_BF - 0 or 1
  Declare SetCenteredTextToggleGadget_BF(centered_text_toggle_gadget_)
  
  ; Pre function call for ToggleGadget_BF - Create and use a ToggleGadget_BF canvas based
  Declare CanvasToggleGadget_BF()
  
  ; Gadget PB flags ToggleGadget_BF - Same PB flags for CanvasGadget and ImageGadget
  Declare SetFlagsToggleGadget_BF(flags_toggle_gadget_)
  
  ; This Function set the ToggleGadget_BF text flags - #PB_VectorParagraph_Left, #PB_VectorParagraph_Right, #PB_VectorParagraph_Center
  Declare SetTextFlagsToggleGadget_BF(text_flag_toggle_gadget_=#PB_VectorParagraph_Center)
  
  ; This Function move the ToggleGadget_BF slider 0 or 1
  Declare MoveSliderToggleGadget_BF(slider, slider_offset_)
  
  ; This Function get the ToggleGadget_BF slider moving space
  Declare.d GetSliderMovingSpaceToggleGadget_BF()
  
  ; Stretched text vector 
  ; This function create a stretched text inside a defined rectangle
  ; A font you must not preset - set simple -1 as font ID
  ; This function is GradientColor BF sensitive
  Declare StretchedText_Vector_BF(output_ID,                  ; Output_ID
                                  font_ID,                    ; Font ID      
                                  text$,                      ; Text
                                  x_pos,                      ; Output x pos
                                  y_pos,                      ; Output y pos
                                  output_width,               ; Output width
                                  output_height,              ; Output height
                                  background_color=$FF000000, ; Background color - Alpha channel available - Alpha=0 ignore the alpha channel - set to $01, this is equal 0
                                  text_color=$FFFFFFFF,       ; Text color - Alpha channel available - Alpha=0 ignore the alpha channel - set to $01, this is equal 0 
                                  text_flag_vector_paragraph=#PB_VectorParagraph_Left) ; Text flag
  
  ; Rotate MagicStretchText - This force also a left adjusted output
  Declare RotateMagic_Stretchtext(activate_rotating_stretchtext_=0, degree_stretchtext_=0)
  
  ; Can output on all channels
  ; With the function "ActivateScreenOutput_BF" you can activate the screen output
  ; For output only as image set output_ID = -1 - Use a output function with alpha channel support or you see nothing
  Declare Magic_StretchText_BF(output_ID,                ; All channels supported
                               text$,                    ; Text
                               x_pos,                    ; Output x pos
                               y_pos,                    ; Output y pos
                               output_width,             ; Output width
                               output_height,            ; output height
                               text_color=#Black,        ; Alpha channel available - Sample $FFAAAAAA - $FF is the alpha channel 
                               text_stretching_x=0,      ; Text stretching x - Headline mode
                               text_stretching_y=0,      ; Text stretching y - Headline mode
                               padding_x.d=0,            ; Padding x - This is a virtual padding for resizing and boxed output - Negative values available
                               padding_y.d=0,            ; Padding y - This is a virtual padding for resizing and boxed output - Negative values available
                               flag=0,                   ; Text center=1 - Text right=2 - Text vertical=3
                               mode=0,                   ; Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
                               font$="Arial",            ; Font type - more is not needed - accept also ""
                               font_style=0)             ; Font style
  
  ; Set a border for Magic_StretchText_Pro_BF() - Colors accept alpha channel - You must deactivate again
  Declare Magic_StretchText_Pro_Border_BF(frame_size_stretchtext_=0,
                                          frame_padding_stretchtext_=0,
                                          frame_rounding_stretchtext_=0,
                                          frame_color_stretchtext_=$FFAAAAAA)
  
  ; Set a border for Magic_StretchText_Pro_BF() - Colors accept alpha channel - You must deactivate again
  ; You can set pasdding x and padding y different
  Declare Magic_StretchText_Pro_Border_DualPadding_BF(frame_size_stretchtext_=0,
                                                      frame_padding_stretchtext_x_=0,
                                                      frame_padding_stretchtext_y_=0,
                                                      frame_rounding_stretchtext_=0,
                                                      frame_color_stretchtext_=$FFAAAAAA)  
  
  ; This function can move the text for Magic_StretchText_Pro_BF() - You must deactivate again
  Declare Magic_StretchText_Pro_TextOffset_BF(stretchtext_pro_offset_x_=0, stretchtext_pro_offset_y_=0)
  
  ; Can output on all channels
  ; With the function "ActivateScreenOutput_BF" you can activate the screen output
  ; For output only as image set output_ID = -1 - Use a output function with alpha channel support or you see nothing
  ; Fitted_text_width and fitted_text_height different text_width and text_height is needed
  ; -for dynamic GUI, but also for moving or adjust the text inside the output rectangle  
  ; Works with Magic_StretchText_Pro_TextOffset_BF()
  ; Magic_StretchText_Pro_TextOffset_BF(stretchtext_pro_offset_x_, stretchtext_pro_offset_y_)
  ; You can set a text output offset to add an image as sample or text adjustment or text animations
  Declare Magic_StretchText_Pro_BF(output_ID,                  ; All channels supported
                                   text$,                      ; Text
                                   x_pos,                      ; Output x pos
                                   y_pos,                      ; Output y pos
                                   text_width,                 ; Text width
                                   text_height,                ; Text height
                                   fitted_text_width,          ; Fitted text width  - Negative values available
                                   fitted_text_height,         ; fitted text height - Negative values available
                                   text_color=#Black,          ; Alpha channel available - Sample $FFAAAAAA - $FF is the alpha channel
                                   background_color=$FFFAFAFA, ; Alpha channel available - For invisible background set to -1
                                   text_stretching_x=0,        ; Text stretching x - Headline mode
                                   text_stretching_y=0,        ; Text stretching y - Headline mode
                                   padding_x.d=0,              ; Padding x - This is a virtual padding for resizing and boxed output - Negative values available
                                   padding_y.d=0,              ; Padding y - This is a virtual padding for resizing and boxed output - Negative values available
                                   flag=0,                     ; Text center=1 - Text right=2 - Text vertical=3
                                   mode=0,                     ; Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
                                   font$="Arial",              ; Font type - more is not needed - accept also ""
                                   font_style=0,               ; Font style
                                   inherited_font_ID=0)        ; You can inherit a font
  
  ; Same above
  Declare StretchedText_Roundbox_Vector_BF(output_ID,                                           ; Output_ID
                                           font_ID,                                             ; Font ID      
                                           text$,                                               ; Text
                                           x_pos,                                               ; Output x pos
                                           y_pos,                                               ; Output y pos
                                           output_width,                                        ; Output width
                                           output_height,                                       ; Output height
                                           radius=4,                                            ; Rounding radius
                                           frame_thickness.d=3,                                 ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                                           frame_color=$00000000,                               ; Frame color - Alpha channel available - Alpha=0 ignore the alpha channel - set to $01, this is equal 0
                                           background_color=$FFFFFFFF,                          ; Background color - Alpha channel available - Alpha=0 ignore the alpha channel - set to $01, this is equal 0
                                           padding_x.d=0,                                       ; Padding x
                                           padding_y.d=0,                                       ; Padding y
                                           text_color=$00000000,                                ; Text color - Alpha channel available - Alpha=0 ignore the alpha channel - set to $01, this is equal 0
                                           text_flag_vector_paragraph=#PB_VectorParagraph_Left) ; Text flag
  
  ; This function deactivate the automatically text stretching for vector text based Gadgets_BF,
  ; - as sample ToggleGadget_BF, StretchTextGadget_BF, MagicTextGadget_BF, MessageRequester_Magic_BF
  ; This is a mighty feature - you must activate again text stretching
  ; With an unsymmetrical padding the stretching becomes active on MagicText, positive values reduce the output, negative values increase it
  Declare  Deactivate_TextStretching_BF(deactivate_text_stretching_, preset_stretchfactor_x_.d=1, preset_stretchfactor_y_.d=1)
  
  ; This function rotate the StretchedText_Roundbox_Vector_BF() output - You must deactivate again
  Declare Rotate_StretchedText_Roundbox_Vector_BF(Rotate_StretchedText_Roundbox_Vector_)
  
  ; Functions same PB GetGadgetData(), but enhanced and for all adresses, as sample image_ID, Gadget_ID
  Declare GetData_int_1_BF(ID) 
  
  Declare GetData_int_2_BF(ID) 
  
  Declare.d GetData_double_1_BF(ID) 
  
  Declare.d GetData_double_2_BF(ID) 
  
  Declare.s GetData_string_1_BF(ID) 
  
  Declare.s GetData_string_2_BF(ID) 
  
  Declare SetData_int_1_BF(ID, data_int_1) 
  
  Declare SetData_int_2_BF(ID, data_int_2) 
  
  Declare SetData_double_1_BF(ID, data_double_1.d) 
  
  Declare SetData_double_2_BF(ID, data_double_2.d) 
  
  Declare SetData_string_1_BF(ID, data_string_1$) 
  
  Declare SetData_string_2_BF(ID, data_string_2$) 
  
  Declare FreeData_BF(ID)
  
  ; Important for all MagicText and MagicGadgets
  ; Letters are interpolated at the edges with this color
  ; 0 deactivate this function - 1 activate it
  ; On Linux you see not a effect
  ; On Windows you see a very fine effect with enabled DPI aware and a strong effekt with disabled DPI aware
  ; On MacOS the effect is good visible
  ; This effect can be very disturbing but also very beautiful
  ; The Embossing color is preset at the same as the text color
  ; Debug Hex(Embossing_Color()) ; This give back the used embossing color
  ; You must deactivate again - So : Embossing_Color()
  Declare Embossing_Color(Change_Embossing_Color_=0, Embossing_Color_=#Black)
  
  ; With this function you can texture any MagicText and MagicGadgets text
  ; With changing x and y pos you can adjust or scroll the textures
  ; You must deactivate again - So : MagicText_BucketFill_BF()
  ; Accept also color in place of the image ID, as sample #Red
  Declare MagicText_BucketFill_BF(MagicText_BucketFill_image_ID_=-1,      ; Image ID; Image ID -1 deactivate the function - Also you can set simple a color
                                  MagicText_BucketFill_x_pos_=0,          ; Output x pos
                                  MagicText_BucketFill_y_pos_=0,          ; Output y pos
                                  MagicText_BucketFill_xx_pos=0,          ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                                  MagicText_BucketFill_yy_pos=0,          ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                                  MagicText_BucketFill_texture_width_=0,  ; Resize texture width  - 0 = without resizing
                                  MagicText_BucketFill_texture_height_=0, ; Resize texture height - 0 = without resizing
                                  MagicText_BucketFill_texture_alpha_=0)  ; Texture Alpha - PB specific 0= full visibl
  
  
  Declare MagicText_BucketFill_1_BF(MagicText_BucketFill_1_image_ID_=-1,      ; Image ID; Image ID -1 deactivate the function - Also you can set simple a color
                                    MagicText_BucketFill_1_x_pos_=0,          ; Output x pos
                                    MagicText_BucketFill_1_y_pos_=0,          ; Output y pos
                                    MagicText_BucketFill_1_xx_pos=0,          ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                                    MagicText_BucketFill_1_yy_pos=0,          ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                                    MagicText_BucketFill_1_texture_width_=0,  ; Resize texture width  - 0 = without resizing
                                    MagicText_BucketFill_1_texture_height_=0, ; Resize texture height - 0 = without resizing
                                    MagicText_BucketFill_1_texture_alpha_=0)  ; Texture Alpha - PB specific 0= full visible
  
  Declare MagicText_BucketFill_2_BF(MagicText_BucketFill_2_image_ID_=-1,      ; Image ID; Image ID -1 deactivate the function - Also you can set simple a color
                                    MagicText_BucketFill_2_x_pos_=0,          ; Output x pos
                                    MagicText_BucketFill_2_y_pos_=0,          ; Output y pos
                                    MagicText_BucketFill_2_xx_pos=0,          ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                                    MagicText_BucketFill_2_yy_pos=0,          ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                                    MagicText_BucketFill_2_texture_width_=0,  ; Resize texture width  - 0 = without resizing
                                    MagicText_BucketFill_2_texture_height_=0, ; Resize texture height - 0 = without resizing
                                    MagicText_BucketFill_2_texture_alpha_=0)  ; Texture Alpha - PB specific 0= full visible
  
  ; Give back 1 if it a 32 bit image
  Declare Is_AlphaChannel_BF(image_ID)
  
  ; Add round corner to a image
  ; This function adds round corners to an image.
  ; Since it interpolates with the background, it also cuts off 1 pixel from the edge of the image, this is intentional
  Declare CreateRoundCornersImage_BF(image_ID, radius, background_color) 
  
  Enumeration Custom_Gadgets_BF
    #GadgetTypeUnknown_BF
    #ButtonImageGadget_BF
    #ColorButtonGadget_BF
    #ImageGadget_BF
    #TextGadget_BF
    #ColorButtonGadget_DualPadding_BF
    #ImageGadget_DualPadding_BF
    #TextGadget_DualPadding_BF
    #ToggleImageGadget_BF
    #ToggleCanvasGadget_BF
    #StretchTextImageGadget_BF
    #StretchTextCanvasGadget_BF
    #StretchTextImageGadget_Pro_BF
    #StretchTextButtonImageGadget_Pro_BF
    #StretchTextCanvasGadget_Pro_BF
  EndEnumeration
  
EndDeclareModule

Module GFX_Wizzard_BF
  ; GFX_Wizzard_BF - For PureBasic
  ; Author : W. Albus © 2018 - www.nachtoptik.de - www.quick-aes-256.de
  ; The use of the GFX-Wizzard_BF is free, but the distribution is reserved for me  
  ; Exceptions require my written permission
  
  EnableExplicit
  
  ; Workaround Linux PB570 window background color
  CompilerIf #PB_Compiler_OS=#PB_OS_Linux And Not Subsystem("gtk2") And Not Subsystem("qt")
    Macro SetWindowColor(window_ID, color)
      Window_SetCssBackColor(window_ID, color)
    EndMacro
    ImportC "" ; By Oma
      gtk_css_provider_load_from_data(*css_provider, data_.p-utf8, length, *error.GError)
      gtk_css_provider_new()
      gtk_style_context_add_provider(*context, *provider, priority)
      gtk_widget_get_style_context(*widget.GtkWidget)
    EndImport
    Procedure Window_SetCssBackColor(Window_ID, color)
      #GTK_STYLE_PROVIDER_PRIORITY_USER=800
      Protected color$, *gProvider=gtk_css_provider_new()
      ; Protected color$="#"+RSet(Hex(color), 6, "000000")
      color$="#"+RSet(Hex(Red(color)), 2, "00")+RSet(Hex(Green(color)), 2, "00")+RSet(Hex(Blue(color)), 2, "00")   
      Protected.s WinBkGndCSS="window.background{background-color: "+color$+";" + #LF$ + "}"
      gtk_css_provider_load_from_data(*gProvider, WinBkGndCSS, -1, #Null)
      gtk_style_context_add_provider(gtk_widget_get_style_context(WindowID(window_ID)), *gProvider, #GTK_STYLE_PROVIDER_PRIORITY_USER)
    EndProcedure
  CompilerEndIf
  
  UseSHA3Fingerprint()
  
  CompilerIf #PB_Compiler_Version=>560
    UseGIFImageDecoder()
  CompilerEndIf
  UsePNGImageDecoder()
  UseJPEGImageDecoder()
  UseTIFFImageDecoder()
  UsePNGImageEncoder()
  UseJPEGImageEncoder()
  
  If Not InitSprite() :  MessageRequester("  ERROR", "InitSprite fails"+#LF$+#LF$+"The tool is terminated") : End : EndIf
  
  #Size_internal_Timer_Array=99 ; 100 internal timers available - You can add more - Before reducing look for used internal timers - Named "Delay_internal"
  #Size_Timer_Array=99          ; 100 different timers available
  #Size_FlipBuffers_BF_Array=99 ; 100 different FlipBuffers_BF calls available - Animation Buffers - Available triple buffered outputs
  #Size_GradientColors_Array=99 ; 100 different GradientColors available
  
  Structure gadget_bf
    gadget_type.i
    gadget_ID.i
    image_ID.i
    temporary_image_ID.i
    state.i
    text_state_0$
    text_state_1$
    text$
    text_color_state_0.i
    text_color_state_1.i
    slider_color_state_0.i
    slider_color_state_1.i
    x.i
    y.i
    button_width.i
    button_height.i
    text_width.i
    text_height.i
    text_color.i
    background_color.i
    grid_color.i
    grid_size.i
    grid_tiling.i
    frame_color.i
    frame_size.i
    font$
    font_ID.i
    font_style.i
    stretch_x.i
    stretch_y.i
    flag.i
    mode.i
    gadget_flag.i
    padding_x_gadget.i
    padding_y_gadget.i
    padding_gadget.i
  EndStructure
  
  Structure grab_box_bf
    box_x.i
    box_y.i
    box_width.i
    box_height.i
    box_data.i
    box_data_secound.i
    box_data_third.i
    box_data_fourth.i
    box_background_image_ID.i
    box_data_double.d
    box_string$
    box_string_secound$
  EndStructure
  
  Structure textures
    texture_ID.i
    *texture_buffer
    texture_clip_width.i
    texture_clip_height.i
    texture_percent_transparent.f
  EndStructure
  
  Structure sprite_w_h
    sprite_w.i 
    sprite_h.i
  EndStructure
  Global sprite_w_h.sprite_w_h
  
  Structure data_bf
    ID.i
    data_int_1.i
    data_int_2.i
    data_double_1.d
    data_double_2.d
    data_string_1$
    data_string_2$
  EndStructure
  
  Global gtk3_x_offset ; With this parameters you can adjust the button size on the ColorButtonGadget_BF and the MessageRequester_Pro_BF
  Global gtk3_y_offset ; It is available this offsets can change with different gtk3 Versions, but it must not change
  
  CompilerIf #PB_Compiler_OS=#PB_OS_Linux
    CompilerIf Not Subsystem("gtk2") And Not Subsystem("qt")
      gtk3_x_offset=14 ; You can change the presetted BF color Button output width
      gtk3_y_offset=6  ; You can change the presetted BF color button output height
    CompilerEndIf
  CompilerEndIf
  
  Global mini_temp_image_ID=CreateImage(#PB_Any, 16, 16)
  If Not mini_temp_image_ID
    MessageRequester("Error", "Can not create needed temporary Image")
    End
  EndIf
  
  Global NewList textures.textures()
  Global NewList gadget_bf.gadget_bf()
  Global NewList grab_box_bf.grab_box_bf()
  Global NewList data_bf.data_bf()
  Global texture_get_color.q=-1, old_texture_get_color.q=-1, new_color, invisible_color.q=-1, replacing_color.q=-1
  Global min_x_0, min_y_0, max_x_0, max_y_0, min_x_1, min_y_1, max_x_1, max_y_1, min_x_2, min_y_2, max_x_2, max_y_2, alpha_sprite, alpha_1
  Global activate_flood_array, activate_bucket_array, activate_sprite_array, flood_array_length, bucket_array_length, sprite_array_length
  Global flood_array_width, flood_array_height, bucket_array_width, bucket_array_height, sprite_array_width, sprite_array_height, no_caching 
  Global max_1, max_1_vector, temp_image_universal_output_ID
  Global from_image, image_color, screen_output, used_invisible_color, activate_LineGo_backcolor, LineGo_backcolor, transparent_flood_fill
  Global mask_color_universal_output.q=-1, colormask_color.q=-1, colormask_alpha, flood_alpha
  Global gradient_color_alpha, gradient_color_red, gradient_color_green, gradient_color_blue, gradient_color_contrast, gradient_color_brightness, gradient_color_intensity
  Global activate_linear_gradient, activate_elliptical_gradient, activate_boxed_gradient, activate_circular_gradient, activate_conical_gradient
  Global activate_backcolor, activate_frontcolor, linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy
  Global elliptical_gradient_x, elliptical_gradient_y, elliptical_gradient_xx, elliptical_gradient_yy
  Global boxed_gradient_x, boxed_gradient_y, boxed_gradient_width, boxed_gradient_height
  Global circular_gradient_x, circular_gradient_y, circular_gradient_radius
  Global conical_gradient_x, conical_gradient_y, conical_gradient_degree, ignore_gradient_colors
  Global frontcolor, backcolor, mode_grey_, partition_x, partition_y, partition_width, partition_height, swap_output_
  Global resulted_rotated_width, resulted_rotated_height, get_all_window_events
  Global fitted_font_padding_x, fitted_font_padding_y, deactivate_dpi_aware, amount_lines_drawtext 
  Global parent_window_message_requester_ID, mode_messagerequester, x_pos_messagerequester, y_pos_messagerequester
  Global draw_text_create_image_add_image_ID, draw_text_create_image_add_image
  Global draw_text_create_image_add_image_x, draw_text_create_image_add_image_y, draw_text_create_image_add_image_alpha
  Global draw_text_create_image_add_image_ID_width, draw_text_create_image_add_image_ID_height
  Global add_fit_image_overlay_color, fit_image_overlay_color, fit_image_overlay_alpha
  Global fit_image_overlay_x, fit_image_overlay_y, fit_image_overlay_width, fit_image_overlay_height
  Global add_fit_image_overlay_frame, fit_image_overlay_frame_color, fit_image_overlay_frame_alpha, fit_image_overlay_frame_thickness
  Global fit_image_overlay_frame_x, fit_image_overlay_frame_y, fit_image_overlay_frame_width, fit_image_overlay_frame_height
  Global fit_image_padding_offset_x, fit_image_padding_offset_y, vector_roundbox_simple_padding_x, vector_roundbox_simple_padding_y, from_vector_roundbox_simple
  Global old_gadget_list, button_image_gadget_fade_in_state, activate_dpi_aware
  Global from_FittedText_RotateRoundBox_Vector_BF, set_scaling_x.d=1.0, set_scaling_y.d=1.0
  Global resize_interpolation=#PB_Image_Smooth, fit_image_upscaling, manual_padding_draw_text_create_image_x, manual_padding_draw_text_create_image_y
  Global gif_css_frames, gif_css_invisible_color, gif_css_color_distance, gif_css_frame_width, gif_css_frame_height, css_frames_in_a_row, css_width, css_height
  Global from_rectangle_image, get_box_x_bf, get_box_y_bf, get_box_width_bf, get_box_height_bf
  Global get_box_data_bf, get_box_data_secound_bf, get_box_data_third_bf, get_box_data_fourth_bf, get_box_data_double_bf.d, get_box_string_bf$, get_box_string_secound_bf$
  Global shrinked_width.d, shrinked_height.d, percent.f, percent_1.f
  Global change_red_.f, change_green_.f, change_blue_.f, change_contrast_.f, change_brightness_.f, change_intensity_.f
  Global dpi_resize_factor.f, dpi_resize_factor_x.f, dpi_resize_factor_y.f
  Global activate_mirroring_horizontal_on_rotate, activate_mirroring_vertical_on_rotate
  Global centered_text_toggle_gadget, canvas_toggle_gadget, flags_toggle_gadget, text_flag_toggle_gadget=#PB_VectorParagraph_Center
  Global Rotate_StretchedText_Roundbox_Vector, deactivate_text_stretching, canvas_stretchtext_gadget, flags_stretchtext_gadget, text_flag_stretchtext_gadget=#PB_VectorParagraph_Center
  Global slider_0_offset, slider_1_offset, slider_moving_space
  Global divisor_progressbar_pos=-1, divisor_progressbar_color=#Red, divisor_progressbar_alpha=255
  Global fit_image_offset_x.d, fit_image_offset_y.d, new_x.d, new_y.d
  Global rotate_coordinates_x.d, rotate_coordinates_y.d, rotate_coordinates_degree.d
  Global activate_rotating_stretchtext, degree_stretchtext
  Global frame_size_stretchtext, frame_padding_stretchtext_x, frame_padding_stretchtext_y, frame_rounding_stretchtext=0, frame_color_stretchtext
  Global stretchtext_pro_offset_x, stretchtext_pro_offset_y, preset_stretchfactor_x.d=1, preset_stretchfactor_y.d=1
  Global MagicStretchText_Add_Background_ID=-1, MagicStretchText_Add_Background_alpha=255
  Global MagicStretchText_Add_Background_width=100, MagicStretchText_Add_Background_height=100
  Global MagicStretchText_Add_Background_x_pos, MagicStretchText_Add_Background_y_pos
  Global MagicStretchText_Add_Image_ID=-1, MagicStretchText_Add_Image_alpha=255, MagicStretchText_Add_Image_x, MagicStretchText_Add_Image_y, MagicStretchText_Add_Image_sticky
  Global MagicStretchText_Add_Image_1_ID=-1, MagicStretchText_Add_Image_1_alpha=255, MagicStretchText_Add_Image_1_x, MagicStretchText_Add_Image_1_y, MagicStretchText_Add_Image_1_sticky
  Global MagicStretchText_Add_Image_2_ID=-1, MagicStretchText_Add_Image_2_alpha=255, MagicStretchText_Add_Image_2_x, MagicStretchText_Add_Image_2_y, MagicStretchText_Add_Image_2_sticky
  Global MagicStretchText_Add_Image_width_percent.d=100, MagicStretchText_Add_Image_height_percent.d=100
  Global MagicStretchText_Add_Image_1_width_percent.d=100, MagicStretchText_Add_Image_1_height_percent.d=100
  Global MagicStretchText_Add_Image_2_width_percent.d=100, MagicStretchText_Add_Image_2_height_percent.d=100
  Global Messagerequester_Magic_make_text_clickable, MagicStretchText_Add_Progressbar_percent, MagicStretchText_Add_Progressbar_color, MagicStretchText_Add_Progressbar_layer
  Global Change_Embossing_Color, Embossing_Color, used_Embossing_Color
  Global MagicText_BucketFill_x_pos, MagicText_BucketFill_y_pos, MagicText_BucketFill_xx_pos, MagicText_BucketFill_yy_pos
  Global MagicText_BucketFill_image_ID=-1, MagicText_BucketFill_texture_alpha, MagicText_BucketFill_texture_width, MagicText_BucketFill_texture_height
  Global MagicText_BucketFill_1_x_pos, MagicText_BucketFill_1_y_pos, MagicText_BucketFill_1_xx_pos, MagicText_BucketFill_1_yy_pos
  Global MagicText_BucketFill_1_image_ID=-1, MagicText_BucketFill_1_texture_alpha, MagicText_BucketFill_1_texture_width, MagicText_BucketFill_1_texture_height
  Global MagicText_BucketFill_2_x_pos, MagicText_BucketFill_2_y_pos, MagicText_BucketFill_2_xx_pos, MagicText_BucketFill_2_yy_pos
  Global MagicText_BucketFill_2_image_ID=-1, MagicText_BucketFill_2_texture_alpha, MagicText_BucketFill_2_texture_width, MagicText_BucketFill_2_texture_height
  Global remove_rotating_artifacts
  
  ; Workaround - Resizeimage() Windows confirmed - PB572 - exe end not if width or height = 0
  Procedure ResizeImage_(image_ID, width, height, mode=#PB_Image_Smooth)
    If width<1 Or height<1
      width=1 : height=1
    EndIf
    ProcedureReturn ResizeImage(image_ID, width, height, mode)
  EndProcedure
  Macro ResizeImage(image_ID, width, height, mode=#PB_Image_Smooth)
    ResizeImage_(image_ID, width, height, mode)
  EndMacro
  
  ; Workaround - CreateImage() Linux Subsystem qt - Damaged Alpha channel - Created image buffer is not cleaned
  CompilerIf #PB_Compiler_OS=#PB_OS_Linux And Subsystem("qt")
    Procedure CreateImage_(image_ID, image_width, image_height, image_depth=24, image_color=0)
      If image_color=#PB_Image_Transparent
        Protected resulted=CreateImage(image_ID, image_width, image_height, 32, #PB_Image_Transparent)
        If image_ID<>#PB_Any
          resulted=image_ID
        EndIf
        If Not IsImage(resulted) : ProcedureReturn 0 : EndIf
        If Not StartDrawing(ImageOutput(resulted))
          FreeImage(resulted)
          ProcedureReturn 0
        EndIf
        FillMemory(DrawingBuffer(), DrawingBufferPitch()*ImageHeight(resulted), 0)
        If Change_Embossing_Color
          DrawingMode(#PB_2DDrawing_AllChannels)
          used_Embossing_Color=Embossing_Color
          Box(0, 0, image_width, image_height, Embossing_Color)
        EndIf
        StopDrawing()
        ProcedureReturn resulted
      EndIf
      ProcedureReturn CreateImage(image_ID, image_width, image_height, image_depth, image_color)
    EndProcedure
    Macro CreateImage(image_ID, image_width, image_height, image_depth=24, image_color=0)
      CreateImage_(image_ID, image_width, image_height, image_depth, image_color)
    EndMacro
  CompilerElse
    Procedure CreateImage_(image_ID, image_width, image_height, image_depth=24, image_color=0)
      If image_color=#PB_Image_Transparent
        Protected resulted=CreateImage(image_ID, image_width, image_height, 32, #PB_Image_Transparent)
        If Not Change_Embossing_Color
          ProcedureReturn resulted
        Else
          If image_ID<>#PB_Any
            resulted=image_ID
          EndIf
          If Not IsImage(resulted) : ProcedureReturn 0 : EndIf
          If Not StartDrawing(ImageOutput(resulted))
            FreeImage(resulted)
            ProcedureReturn 0
          EndIf
          DrawingMode(#PB_2DDrawing_AllChannels)
          used_Embossing_Color=Embossing_Color
          Box(0, 0, image_width, image_height, Embossing_Color)
        EndIf
        StopDrawing()
        ProcedureReturn resulted
      EndIf
      ProcedureReturn CreateImage(image_ID, image_width, image_height, image_depth, image_color)
    EndProcedure
    Macro CreateImage(image_ID, image_width, image_height, image_depth=24, image_color=0)
      CreateImage_(image_ID, image_width, image_height, image_depth, image_color)
    EndMacro
  CompilerEndIf
  
  Macro init_progressbar_globals(number, text_)
    Global font_ID_progressbar_#number, txt_direction_progressbar_#number, txt_x_progressbar_#number,
           txt_y_progressbar_#number, txt_color_progressbar_#number, txt_alpha_progressbar_#number, txt_progressbar_#text_
  EndMacro
  init_progressbar_globals(0_BF, 0_BF$) : init_progressbar_globals(1_BF, 1_BF$) : init_progressbar_globals(2_BF, 2_BF$)
  init_progressbar_globals(3_BF, 3_BF$) : init_progressbar_globals(4_BF, 4_BF$)
  
  Global Dim picture_0.q(0, 0), Dim picture_1.q(0, 0), Dim picture_2.q(0, 0)
  Global Dim temporary_image_0_ID_BF(#Size_FlipBuffers_BF_Array), Dim temporary_image_1_ID_BF(#Size_FlipBuffers_BF_Array) ; Preset, changeable - 100 different triple buffered outputs
  Global Dim gradient_color(#Size_GradientColors_Array)
  Global Dim gradient_activate_flag(#Size_GradientColors_Array)
  Global Dim gradient_position.f(#Size_GradientColors_Array)
  Global Dim gif_and_css_delay_array(0)
  
  CompilerIf #PB_Compiler_OS=#PB_OS_Windows
    Global SysFont_BF=LoadFont(#PB_Any, "Arial", 11)
    Global SysFontBold_BF=LoadFont(#PB_Any, "Arial", 11, #PB_Font_Bold)
  CompilerEndIf
  CompilerIf #PB_Compiler_OS=#PB_OS_Linux
    Global SysFont_BF=LoadFont(#PB_Any, "Arial", 10)
    Global SysFontBold_BF=LoadFont(#PB_Any, "Arial", 10, #PB_Font_Bold)
  CompilerEndIf
  CompilerIf #PB_Compiler_OS=#PB_OS_MacOS
    Global SysFont_BF=LoadFont(#PB_Any, "Arial", 14)
    Global SysFontBold_BF=LoadFont(#PB_Any, "Arial", 14, #PB_Font_Bold)
  CompilerEndIf
  
  CompilerIf #PB_Compiler_OS=#PB_OS_Windows And #PB_Compiler_Version>=570
    dpi_resize_factor.f=DesktopResolutionX()
    dpi_resize_factor_x.f=dpi_resize_factor.f
    dpi_resize_factor_y.f=DesktopResolutionY()
  CompilerElse
    dpi_resize_factor.f=1
    dpi_resize_factor_x.f=1
    dpi_resize_factor_y.f=1
  CompilerEndIf
  
  Procedure IsWindowsScalingOK_BF()
    CompilerIf #PB_Compiler_OS=#PB_OS_Windows And #PB_Compiler_Version>=570
      If Not IsDPIaware_BF() And IsProcessDPIaware_BF()
        ProcedureReturn 0  
      EndIf
      ProcedureReturn 1
    CompilerEndIf
    ProcedureReturn 1
  EndProcedure
  
  ; CompilerIf Not Defined(PB_Compiler_Backend, #PB_Constant)
  ;   #PB_Compiler_Backend=0
  ;   #PB_Backend_C=1
  ;   #PB_Backend_Asm=0
  ; CompilerEndIf
  Procedure IsDPIaware_BF()
    CompilerIf #PB_Compiler_Backend=#PB_Backend_C
      ! extern int PB_Compiler_DPIAware ; Solution by mk-soft
      ! return PB_Compiler_DPIAware
    CompilerElse
      CompilerIf #PB_Compiler_OS=#PB_OS_Windows And #PB_Compiler_Version>=570
        Protected DPI_flag.l
        EnableASM
        MOV eax, dword [PB_Compiler_DPIAware]
        MOV DPI_flag, eax
        ProcedureReturn DPI_flag.l
        DisableASM
      CompilerEndIf 
      ProcedureReturn 0
    CompilerEndIf
  EndProcedure
  
  Global is_process_dpi_aware
  Procedure IsProcessDPIaware_BF()
    CompilerIf #PB_Compiler_OS=#PB_OS_Windows And #PB_Compiler_Version=>570
      Protected library=OpenLibrary(#PB_Any, "user32.dll")
      If library
        is_process_dpi_aware=CallFunction(library, "IsProcessDPIAware")
        CloseLibrary(library)
      EndIf
    CompilerEndIf
    ProcedureReturn is_process_dpi_aware
  EndProcedure 
  
  CompilerIf #PB_Compiler_Backend=#PB_Backend_C
    Procedure ColorDistance_BF(color_1, color_2)
      ; Procedure.f ColorDistance_BF(color_1, color_2)
      If color_1<>color_2
        Protected r1=color_1&$FFFFFF>>16, g1=(color_1&$FFFF)>>8, b1=color_1>>16
        Protected r2=color_2&$FFFFFF>>16, g2=(color_2&$FFFF)>>8, b2=color_2>>16
        Protected diff_red=Abs(r1-r2), diff_green=Abs(g1-g2), diff_blue=Abs(b1-b2)
        Protected r.f=diff_red/256, g.f=diff_green/256, b.f=diff_blue/256
        ProcedureReturn (r+g+b)/3*100
      Else
        ProcedureReturn 0
      EndIf
    EndProcedure
  CompilerElse
    ; C Backend hint - To time not supported
    Procedure ColorDistance_BF(color_1, color_2) ; Based by code from Wilbert
      !movd xmm0, [p.v_color_1]
      !movd xmm1, [p.v_color_2]
      !comisd xmm0, xmm1
      !je equal
      !pslld xmm0, 8
      !pslld xmm1, 8
      !psadbw xmm0, xmm1
      !movd eax, xmm0 ; eax range [0, 765]
      !imul eax, 0x2176cc ; div by 7.65
      !add eax, 0x800000
      !shr eax, 24
      ProcedureReturn
      !equal:
      !xor eax, eax
      ProcedureReturn
    EndProcedure
  CompilerEndIf
  
  Procedure ActivateAlphaChannel_BF(alpha_sprite_)
    alpha_sprite=alpha_sprite_
  EndProcedure
  
  Procedure color_check_valid_range(value)
    If value<0 : ProcedureReturn 0 : EndIf
    If value>255 : ProcedureReturn 255 : EndIf
    ProcedureReturn value
  EndProcedure
  
  Macro common_write_frame_data
    WriteStringN(file, "          Frames #"+RSet(Str(frames), 8, "0 "), #PB_Ascii)
    WriteStringN(file, "     Frame width #"+RSet(Str(frame_width), 8, "0 "), #PB_Ascii)
    WriteStringN(file, "    Frame height #"+RSet(Str(frame_height), 8, "0 "), #PB_Ascii)
    WriteStringN(file, "     Sheet width #"+RSet(Str(sheet_width), 8, "0 "), #PB_Ascii)
    WriteStringN(file, "    Sheet height #"+RSet(Str(sheet_height), 8, "0 "), #PB_Ascii)
    WriteStringN(file, " Frames in a row #"+RSet(Str(frames_in_a_row), 8, "0 "), #PB_Ascii)
    If invisible_gif_color<0
      WriteStringN(file, " Invisible color #"+RSet(Str(invisible_gif_color), 8, "  ")+"   $"+RSet(Hex(invisible_gif_color), 6, "  "), #PB_Ascii) ; Negative value available "  "
    Else
      WriteStringN(file, " Invisible color #"+RSet(Str(invisible_gif_color), 8, "0 ")+"   $"+RSet(Hex(invisible_gif_color), 6, "0 "), #PB_Ascii)
    EndIf
    WriteStringN(file, "  Color distance #"+RSet(Str(color_distance_gif), 8, "0 ")+"   $"+RSet(Hex(color_distance_gif), 6, "0 "), #PB_Ascii)
  EndMacro
  
  Macro AlphaBlend(color_1, color_2, alpha) ; Macro by Wilbert
    ((((color_2 & $FF00FF)*alpha+(color_1 & $FF00FF)*(256-alpha))>>8 & $FF00FF) |
     (((color_2 & $FF00)*alpha+(color_1 & $FF00)*(256-alpha))>>8 & $FF00))
  EndMacro 
  
  Macro error_message
    "The function is terminated !"+#LF$+#LF$+"##"+RSet(Str(Abs(result_)), 2, "0")
  EndMacro
  
  Global to_filtering.l                                            
  Macro filter(callback, lParam=0)                           
    to_filtering=lParam                                      
    CustomFilterCallback(callback)                                  
  EndMacro                                                           
  
  Procedure filter_callback_1(x, y, source_color, destination_color)
    If source_color&$FFFFFF=to_filtering
      ProcedureReturn destination_color
    EndIf 
    If percent
      If ColorDistance_BF(source_color, to_filtering)<=percent
        If alpha_1>1
          source_color=AlphaBlend(source_color, destination_color&$FFFFFF, alpha_1)
        EndIf 
        ProcedureReturn destination_color                                                                    
      EndIf
    EndIf
    If alpha_1>1
      source_color=AlphaBlend(source_color, destination_color&$FFFFFF, alpha_1)
    EndIf 
    ProcedureReturn source_color
  EndProcedure
  
  Procedure filter_callback_2(x, y, source_color, destination_color)
    Protected point 
    If source_color&$FFFFFF=to_filtering
      ProcedureReturn destination_color
    EndIf 
    If percent
      If ColorDistance_BF(source_color, to_filtering)<=percent
        If alpha_1>1
          point=AlphaBlend(source_color, colormask_color, alpha_1)
          source_color=AlphaBlend(destination_color&$FFFFFF, point, colormask_alpha)
        EndIf 
        ProcedureReturn destination_color                                                                    
      EndIf
    EndIf
    If alpha_1>1
      point=AlphaBlend(source_color, colormask_color, alpha_1)
      source_color=AlphaBlend(destination_color&$FFFFFF, point, colormask_alpha)
    EndIf 
    ProcedureReturn source_color
  EndProcedure
  
  Procedure filter_callback_3(x, y, source_color, destination_color)    
    source_color=AlphaBlend(source_color, destination_color, alpha_1) 
    If ColorDistance_BF(source_color, to_filtering)<=percent
      ProcedureReturn destination_color                                                                     
    EndIf
    ProcedureReturn source_color
  EndProcedure 
  
  Procedure filter_callback_4(x, y, source_color, destination_color) 
    Protected point
    point=AlphaBlend(source_color, colormask_color, alpha_1)
    source_color=AlphaBlend(destination_color, point, colormask_alpha)
    If ColorDistance_BF(source_color, to_filtering)<=percent
      ProcedureReturn destination_color                                                                     
    EndIf
    ProcedureReturn source_color
  EndProcedure 
  
  Macro filter_callback_5_6
    red=Red(source_color)
    green=Green(source_color)
    blue=Blue(source_color)
    
    If change_red_ Or change_green_ Or change_blue_
      red=color_check_valid_range(red+change_red_*2.55)
      green=color_check_valid_range(green+change_green_*2.55)
      blue=color_check_valid_range(blue+change_blue_*2.55)
    EndIf
    
    If change_contrast_
      red=color_check_valid_range(red+(red-128)*change_contrast_)
      green=color_check_valid_range(green+(green-128)*change_contrast_)
      blue=color_check_valid_range(blue+(blue-128)*change_contrast_)
    EndIf
    
    If change_intensity_
      red=color_check_valid_range(red+red*(change_intensity_/100))
      green=color_check_valid_range(green+green*(change_intensity_/100))
      blue=color_check_valid_range(blue+blue*(change_intensity_/100))
    EndIf
    
    If mode_grey_
      Select mode_grey_ ; you can simple add schemes
        Case 1
          correct_red=0.4 : correct_green=0.3 : correct_blue=0.3 ; More neutral
        Case 2
          correct_red=0.3 : correct_green=0.53 : correct_blue=0.11 ; Gimp
        Case 3
          correct_red=0.299 : correct_green=0.587 : correct_blue=0.114
      EndSelect
      grey=color_check_valid_range((red+change_red_)*correct_red+(green+change_green_)*correct_green+(blue+change_blue_)*correct_blue)
      ProcedureReturn RGBA(grey, grey, grey, Alpha(source_color))
    EndIf
    
    ProcedureReturn RGBA(red, green, blue, Alpha(source_color))
  EndMacro
  
  Procedure filter_callback_5(x, y, source_color, destination_color)
    Protected red.f, green.f, blue.f, grey.f, correct_red.f, correct_green.f, correct_blue.f
    If swap_output_
      If x=>partition_x And x<partition_x+partition_width And y=>partition_y And y<partition_y+partition_height
        ProcedureReturn source_color
      EndIf
    Else
      If x<partition_x Or y<partition_y Or x>partition_x+partition_width Or y>partition_y+partition_height
        ProcedureReturn source_color
      EndIf
    EndIf
    filter_callback_5_6
  EndProcedure
  
  Procedure filter_callback_6(x, y, source_color, destination_color)
    Protected red.f, green.f, blue.f, grey.f, correct_red.f, correct_green.f, correct_blue.f
    filter_callback_5_6
  EndProcedure
  
  ; Error Check
  Procedure ErrorCheck_BF(result_)
    Protected result
    Protected bf_type$="GFX_Wizzard_BF"
    Protected error$="  ERROR"
    Select result_
      Case -1
        MessageRequester(error$, bf_type$+" - Canvas not found"+#LF$+#LF$+error_message)                           
      Case -2
        MessageRequester(error$, bf_type$+" - Image not found"+#LF$+#LF$+error_message)
      Case -3
        MessageRequester(error$, bf_type$+" - Output start drawing fail"+#LF$+#LF$+error_message)
      Case -4
        MessageRequester(error$, bf_type$+" - Image - Start drawing fail"+#LF$+#LF$+error_message)
      Case -5
        MessageRequester(error$, bf_type$+" - Canvas - Start drawing fail"+#LF$+#LF$+error_message)
      Case -6
        MessageRequester(error$, bf_type$+" - Creating image fail"+#LF$+#LF$+error_message)
      Case -7
        MessageRequester(error$, bf_type$+" - Creating canvas fail"+#LF$+#LF$+error_message)
      Case -8
        MessageRequester(error$, bf_type$+" - Parameter wrong"+#LF$+#LF$+error_message)
      Case -9
        MessageRequester(error$, bf_type$+" - Function result fail"+#LF$+#LF$+error_message)
      Case -10
        MessageRequester(error$, bf_type$+" - Selected image frame not found"+#LF$+#LF$+error_message) 
      Case -11
        MessageRequester(error$, bf_type$+" - Output not found"+#LF$+#LF$+error_message)
      Case -12
        MessageRequester(error$, bf_type$+" - Grab output - Start drawing fail"+#LF$+#LF$+error_message)
      Case -13
        MessageRequester(error$, bf_type$+" - Grab output - Allocate memory fail"+#LF$+#LF$+error_message)
      Case -14
        MessageRequester(error$, bf_type$+" - Drawing buffer - Start drawing fail"+#LF$+#LF$+error_message)
      Case -15
        MessageRequester(error$, bf_type$+" - Drawing buffer - Drawing buffer fail"+#LF$+#LF$+error_message)
      Case -16
        MessageRequester(error$, bf_type$+" - Creating sprite fail"+#LF$+#LF$+error_message)
      Case -17
        MessageRequester(error$, bf_type$+" - Input not found"+#LF$+#LF$+error_message)
      Case -18
        MessageRequester(error$, bf_type$+" - Sprite not found"+#LF$+#LF$+error_message)
      Case -19
        MessageRequester(error$, bf_type$+" - Sprite - Start drawing fail"+#LF$+#LF$+error_message)
      Case -20
        MessageRequester(error$, bf_type$+" - Screen - Start drawing fail"+#LF$+#LF$+error_message)
      Case -21
        MessageRequester(error$, bf_type$+" - File - Can not read file"+#LF$+#LF$+error_message)
      Case -22
        MessageRequester(error$, bf_type$+" - File - Can not write file"+#LF$+#LF$+error_message)
      Case -23
        MessageRequester(error$, bf_type$+" - Creating gadget fail"+#LF$+#LF$+error_message)
      Case -24
        MessageRequester(error$, bf_type$+" - Open window fail"+#LF$+#LF$+error_message)
      Case -25
        MessageRequester(error$, bf_type$+" - Gadget not found"+#LF$+#LF$+error_message)
      Case -26
        MessageRequester(error$, bf_type$+" - String not found"+#LF$+#LF$+error_message)
      Case -27
        MessageRequester(error$, bf_type$+" - ResizeImage fails"+#LF$+#LF$+error_message)
      Case -28
        MessageRequester(error$, bf_type$+" - Image width to large >3e4"+#LF$+#LF$+error_message)
      Case -29
        MessageRequester(error$, bf_type$+" - Image height to large >3e4"+#LF$+#LF$+error_message) 
      Case -30
        MessageRequester(error$, bf_type$+" - Screen not found"+#LF$+#LF$+error_message)
      Default
        result=1
    EndSelect
    ; ProcedureReturn result ; Handle errors how you want
    ; The messages are helpfull on development - Run your programm you should never seen any again
    If Not result : End : EndIf
  EndProcedure
  
  Procedure Delay_BF(timer, time)
    If Not time : ProcedureReturn 1 : EndIf
    
    Static  Dim time_1.q(#Size_Timer_Array)
    Static Dim skip(#Size_Timer_Array)
    If skip(timer) : skip(timer)=0 : ProcedureReturn 1 : EndIf
    If time<0 : skip(timer)=1 : EndIf
    
    CompilerIf #PB_Compiler_Processor = #PB_Processor_x86 And #PB_Compiler_Version<560 ; Based on x86 timer code by Danilo
      Static ElapsedMilliseconds_64_oldValue.q=0
      Static ElapsedMilliseconds_64_overflow.q=0
      Static time_result.q
      Protected current_ms.q=ElapsedMilliseconds()&$FFFFFFFF
      If ElapsedMilliseconds_64_oldValue>current_ms
        ElapsedMilliseconds_64_overflow+1
      EndIf  
      ElapsedMilliseconds_64_oldValue=current_ms
      time_result=current_ms+ElapsedMilliseconds_64_overflow*$FFFFFFFF
      If time_result>time_1(timer)+time
        time_1(timer)=time_result
        ProcedureReturn 1
      Else
        ProcedureReturn 0
      EndIf
    CompilerElse
      If ElapsedMilliseconds()>time_1(timer)+time
        time_1(timer)=ElapsedMilliseconds()
        ProcedureReturn 1
      Else
        ProcedureReturn 0
      EndIf
    CompilerEndIf
  EndProcedure
  
  Procedure Delay_internal(timer, time)
    If Not time : ProcedureReturn 1 : EndIf
    Static Dim time_1.q(#Size_internal_Timer_Array)
    Static Dim skip(#Size_internal_Timer_Array)
    If skip(timer) : skip(timer)=0 : ProcedureReturn 1 : EndIf
    If time<0 : skip(timer)=1 : EndIf
    
    CompilerIf #PB_Compiler_Processor = #PB_Processor_x86 And #PB_Compiler_Version<560 ; Based on x86 timer code by Danilo
      Static ElapsedMilliseconds_64_oldValue.q=0
      Static ElapsedMilliseconds_64_overflow.q=0
      Static time_result.q
      Protected current_ms.q=ElapsedMilliseconds()&$FFFFFFFF
      If ElapsedMilliseconds_64_oldValue>current_ms
        ElapsedMilliseconds_64_overflow+1
      EndIf  
      ElapsedMilliseconds_64_oldValue=current_ms
      time_result=current_ms+ElapsedMilliseconds_64_overflow*$FFFFFFFF
      If time_result>time_1(timer)+time
        time_1(timer)=time_result
        ProcedureReturn 1
      Else
        ProcedureReturn 0
      EndIf
    CompilerElse
      If ElapsedMilliseconds()>time_1(timer)+time
        time_1(timer)=ElapsedMilliseconds()
        ProcedureReturn 1
      Else
        ProcedureReturn 0
      EndIf
    CompilerEndIf
  EndProcedure
  
  Procedure FreeImage_BF(image_ID)
    If IsImage(image_ID) : FreeImage(image_ID) : ProcedureReturn 1 : EndIf
    ProcedureReturn 0
  EndProcedure
  
  Procedure Free_PB_Sprite_BF(sprite_ID)
    If IsSprite(sprite_ID) : FreeSprite(sprite_ID) : ProcedureReturn 1 : EndIf
    ProcedureReturn 0
  EndProcedure
  
  Procedure FreeGadget_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        If IsImage(gadget_bf()\image_ID)
          FreeImage(gadget_bf()\image_ID)
        EndIf
        If IsImage(gadget_bf()\temporary_image_ID)
          FreeImage(gadget_bf()\temporary_image_ID)
        EndIf
        If IsGadget(gadget_ID)
          FreeGadget(gadget_ID)
        EndIf
        DeleteElement(gadget_bf())
        ProcedureReturn 1
      EndIf
    Next
    ProcedureReturn 0
  EndProcedure
  
  Procedure FreeAllGadgets_BF(mode=0)
    Protected result
    If Not mode
      ForEach(gadget_BF())
        If IsGadget(gadget_bf()\gadget_ID)
          FreeGadget(gadget_bf()\gadget_ID)
          result+1
        EndIf 
        If IsImage(gadget_bf()\image_ID)
          FreeImage(gadget_bf()\image_ID)
        EndIf
        If IsImage(gadget_bf()\temporary_image_ID)
          FreeImage(gadget_bf()\temporary_image_ID)
        EndIf
      Next
      ClearList(gadget_bf())
      ProcedureReturn result
    ElseIf mode=1
      ProcedureReturn ListSize(gadget_bf())
    EndIf
  EndProcedure
  
  Procedure SpriteWidthUnzoomed(sprite_ID)
    StartDrawing(SpriteOutput(sprite_ID))
    Protected result=OutputWidth()
    StopDrawing()
    ProcedureReturn(result)
  EndProcedure
  
  Procedure SpriteHeightUnzoomed(sprite_ID)
    StartDrawing(SpriteOutput(sprite_ID))
    Protected result=OutputHeight()
    StopDrawing()
    ProcedureReturn(result)
  EndProcedure
  
  Procedure Sprite_w_h(sprite_ID) ; ZoomSprite change SpriteWidth and SpriteHeight, but not the physicalic OutputWidth and OutputHeight
    If StartDrawing(SpriteOutput(sprite_ID))
      sprite_w_h\sprite_w=OutputWidth()
      sprite_w_h\sprite_h=OutputHeight()
      StopDrawing()
      ProcedureReturn 1
    Else
      ProcedureReturn 0
    EndIf
  EndProcedure
  
  Procedure ActivateScreenOutput_BF(screen_output_)
    screen_output=screen_output_
  EndProcedure
  
  Procedure AlphaBlend_BF(color_1, color_2, alpha)
    ProcedureReturn AlphaBlend(color_1, color_2, alpha)
  EndProcedure
  
  Procedure FreeTextures_BF(mode=0)
    Protected needed_size
    If Not mode
      ForEach(textures())
        FreeMemory(textures()\texture_buffer)
      Next
      ClearList(textures())
      ProcedureReturn 1
    ElseIf mode=1
      ProcedureReturn ListSize(textures())
    ElseIf mode=2
      ForEach(textures())
        needed_size+MemorySize(textures()\texture_buffer)
      Next
      ProcedureReturn needed_size
    EndIf
  EndProcedure
  
  Procedure Free_selected_Texture_BF(texture_ID, free=1)
    If free
      If IsImage(texture_ID)
        FreeImage(texture_ID)
      ElseIf IsSprite(texture_ID)
        FreeSprite(texture_ID)
      ElseIf IsGadget(texture_ID)
        FreeGadget(texture_ID)
      EndIf
    EndIf
    ForEach(textures())
      If textures()\texture_ID=texture_ID 
        FreeMemory(textures()\texture_buffer)
        DeleteElement(textures())
        ProcedureReturn 1
      EndIf
    Next
    ProcedureReturn 0
  EndProcedure
  
  Procedure GetFloodArray_Adress_BF()
    ProcedureReturn @picture_0()
  EndProcedure
  
  Procedure GetBucketArray_Adress_BF()
    ProcedureReturn @picture_1()
  EndProcedure
  
  Procedure GetSpriteArray_Adress_BF()
    ProcedureReturn @picture_2()
  EndProcedure
  
  Procedure GetFloodArray_State_BF()
    ProcedureReturn activate_flood_array
  EndProcedure
  
  Procedure GetBucketArray_State_BF()
    ProcedureReturn activate_bucket_array
  EndProcedure
  
  Procedure GetSpriteArray_State_BF()
    ProcedureReturn activate_sprite_array
  EndProcedure
  
  Procedure GetFloodArray_Point_BF(x, y)
    ProcedureReturn picture_0(x, y)
  EndProcedure
  
  Procedure GetBucketArray_Point_BF(x, y)
    ProcedureReturn picture_1(x, y)
  EndProcedure
  
  Procedure GetSpriteArray_Point_BF(x, y)
    ProcedureReturn picture_2(x, y)
  EndProcedure
  
  Procedure PutFloodArray_Point_BF(point, x, y)
    picture_0(x, y)=point
  EndProcedure
  
  Procedure PutBucketArray_Point_BF(point, x, y)
    picture_1(x, y)=point
  EndProcedure
  
  Procedure PutSpriteArray_Point_BF(point, x, y)
    picture_2(x, y)=point
  EndProcedure
  
  Procedure GetFlood_X_BF()
    ProcedureReturn min_x_0
  EndProcedure
  
  Procedure GetFlood_Y_BF()
    ProcedureReturn min_y_0
  EndProcedure
  
  Procedure GetFlood_XX_BF()
    ProcedureReturn max_x_0
  EndProcedure
  
  Procedure GetFlood_YY_BF()
    ProcedureReturn max_y_0
  EndProcedure
  
  Procedure GetBucket_X_BF()
    ProcedureReturn min_x_1
  EndProcedure
  
  Procedure GetBucket_Y_BF()
    ProcedureReturn min_y_1
  EndProcedure
  
  Procedure GetBucket_XX_BF()
    ProcedureReturn max_x_1
  EndProcedure
  
  Procedure GetBucket_YY_BF()
    ProcedureReturn max_y_1
  EndProcedure
  
  Procedure GetSprite_X_BF()
    ProcedureReturn min_x_2
  EndProcedure
  
  Procedure GetSprite_Y_BF()
    ProcedureReturn min_y_2
  EndProcedure
  
  Procedure GetSprite_XX_BF()
    ProcedureReturn max_x_2
  EndProcedure
  
  Procedure GetSprite_YY_BF()
    ProcedureReturn max_y_2
  EndProcedure
  
  Procedure GetSprite_width_BF()
    ProcedureReturn max_x_2-min_x_2
  EndProcedure
  
  Procedure GetSprite_height_BF()
    ProcedureReturn max_y_2-min_y_2
  EndProcedure
  
  Procedure GetFloodArray_length_BF()
    ProcedureReturn flood_array_length
  EndProcedure 
  
  Procedure GetBucketArray_length_BF()
    ProcedureReturn bucket_array_length
  EndProcedure
  
  Procedure GetSpriteArray_length_BF()
    ProcedureReturn sprite_array_length
  EndProcedure 
  
  Procedure GetFloodArray_width_BF()
    ProcedureReturn flood_array_width
  EndProcedure 
  
  Procedure GetFloodArray_height_BF()
    ProcedureReturn flood_array_height
  EndProcedure 
  
  Procedure GetBucketArray_width_BF()
    ProcedureReturn bucket_array_width
  EndProcedure 
  
  Procedure GetBucketArray_height_BF()
    ProcedureReturn bucket_array_height
  EndProcedure 
  
  Procedure GetSpriteArray_width_BF()
    ProcedureReturn sprite_array_width
  EndProcedure 
  
  Procedure GetSpriteArray_height_BF()
    ProcedureReturn sprite_array_height
  EndProcedure 
  
  Procedure ReduceSpriteMask_BF(image_ID, border, color_distance.f=0, delete_source_ID=1)
    If Not IsImage(image_ID) : ProcedureReturn -2 : EndIf
    Protected input_width=ImageWidth(image_ID)
    Protected input_height=ImageHeight(image_ID)
    Protected old_invisible_color.q=invisible_color.q
    Protected old_color_distance.f=percent.f
    Protected x, y, width, height
    percent.f=color_distance.f
    invisible_color.q=GetImageColor_BF(image_ID, 0, 0) ; Set the invisible color
    GrabSpriteOffsets_BF(image_ID)
    invisible_color.q=old_invisible_color.q
    percent.f=old_color_distance.f
    x=min_x_2-border-1
    y=min_y_2-border-1
    width=max_x_2-min_x_2+border*2+2 
    height=max_y_2-min_y_2+border*2+2
    
    If width<0 : width=0 : EndIf
    If height<0 : height=0 : EndIf
    
    If x<0
      width+x
      x=0
      If width<0 : width=0 : EndIf
    EndIf
    
    If y<0
      height+y
      y=0
      If height<0 : height=0 : EndIf
    EndIf
    
    If x+width>input_width
      width-(x+width-input_width)
      If width<0 : width=0 : EndIf
    EndIf
    
    If y+height>input_height
      height-(y+height-input_height)
      If height<0 : height=0 : EndIf
    EndIf
    
    Protected output_ID=GrabImage(image_ID, #PB_Any, x, y, width, height)
    If Not output_ID
      ProcedureReturn -6
    EndIf
    If delete_source_ID>0
      FreeImage(image_ID)
    EndIf
    ProcedureReturn output_ID
  EndProcedure
  
  Procedure GetImageColor_BF(image_ID, x, y)
    Protected point
    If Not IsImage(image_ID) : ProcedureReturn -2 : EndIf
    If StartDrawing(ImageOutput(image_ID))
      If x>-1 And x<ImageWidth(image_ID) And y>-1 And y<ImageHeight(image_ID)
        point=Point(x, y)
      EndIf
      StopDrawing()
    Else
      ProcedureReturn -4
    EndIf
    ProcedureReturn point
  EndProcedure
  
  Procedure Get_PB_SpriteColor_BF(sprite_ID, x, y)
    Protected point
    If Not IsSprite(sprite_ID) : ProcedureReturn -18 : EndIf
    If StartDrawing(SpriteOutput(sprite_ID))
      If x>-1 And x<OutputWidth() And y>-1 And y<OutputHeight()
        point=Point(x, y)
      EndIf
      StopDrawing()
    Else
      ProcedureReturn -19
    EndIf
    ProcedureReturn point
  EndProcedure
  
  Procedure GetCanvasColor_BF(canvas_ID, x, y)
    Protected point
    If Not IsGadget(canvas_ID) : ProcedureReturn -1 : EndIf
    If StartDrawing(CanvasOutput(canvas_ID))
      If x>-1 And x<GadgetWidth(canvas_ID)*DPI_ResizeFactor_BF() And y>-1 And y<GadgetHeight(canvas_ID)*DPI_ResizeFactor_BF()
        point=Point(x, y)
      EndIf
      StopDrawing()
    Else
      ProcedureReturn -4
    EndIf
    ProcedureReturn point
  EndProcedure 
  
  Procedure.q GetColor_BF()
    ProcedureReturn texture_get_color
  EndProcedure
  
  Procedure SetColor_BF(color.q)
    texture_get_color=color
  EndProcedure
  
  Procedure SetColorDistanceFill_BF(percent_.f)
    percent_1=percent_
  EndProcedure
  
  Procedure SetColorDistanceSpriteMask_BF(percent_.f)
    percent=percent_
  EndProcedure
  
  Procedure SetInvisibleColor_BF(color.q)
    invisible_color=color
  EndProcedure
  
  Procedure SetReplacingColor_BF(color.q)
    replacing_color=color
  EndProcedure
  
  Procedure.f GetColorDistanceFill_BF()
    ProcedureReturn percent_1
  EndProcedure
  
  Procedure.f GetColorDistanceSpriteMask_BF()
    ProcedureReturn percent
  EndProcedure
  
  Procedure.q GetInvisibleColor_BF()
    ProcedureReturn invisible_color
  EndProcedure
  
  Procedure.q GetUsedInvisibleColor_BF()
    ProcedureReturn used_invisible_color
  EndProcedure
  
  Procedure.q GetReplacingColor_BF()
    ProcedureReturn replacing_color
  EndProcedure
  
  Procedure GetTempImageUniversalOutput_ID_BF()
    ProcedureReturn temp_image_universal_output_ID
  EndProcedure
  
  Procedure ActivateFloodArray_BF(activate)
    If activate
      activate_flood_array=1
    Else
      activate_flood_array=0
    EndIf
  EndProcedure
  
  Procedure ActivateBucketArray_BF(activate)
    If activate
      activate_bucket_array=1
    Else
      activate_bucket_array=0
    EndIf
  EndProcedure
  
  Procedure ActivateMaskCreating_BF(colormask_color_.q, colormask_alpha_=0)
    If colormask_color_<>-1
      colormask_color=colormask_color_
      colormask_alpha_=256-colormask_alpha_
      If colormask_alpha_<0 : colormask_alpha_=0 : EndIf
      If colormask_alpha_>256 : colormask_alpha_=256 : EndIf
      colormask_alpha=colormask_alpha_
    Else
      colormask_color=-1
    EndIf
  EndProcedure
  
  Procedure ActivateSpriteArray_BF(activate)
    If activate
      activate_sprite_array=1
    Else
      activate_sprite_array=0
    EndIf
  EndProcedure
  
  Procedure SearchUnusedColor_BF(input_ID, x, y, xx, yy, delay=5)  
    Protected test_color.q, i, ii, image, sprite, output_width, output_height, result, delay_1
    
    If IsImage(input_ID) ; check for image
      image=1
    ElseIf IsSprite(input_ID) ; Check for PB sprite
      sprite=1
    ElseIf Not IsGadget(input_ID) ; Check for canvas
      ProcedureReturn -17
    EndIf
    
    If image
      output_width=ImageWidth(input_ID)-1
      output_height=ImageHeight(input_ID)-1
    ElseIf sprite
      sprite_w_h(input_ID)
      output_width=sprite_w_h\sprite_w-1
      output_height=sprite_w_h\sprite_h-1
    Else
      output_width=(GadgetWidth(input_ID)-1)*DPI_ResizeFactor_BF()
      output_height=(GadgetHeight(input_ID)-1)*DPI_ResizeFactor_BF()
    EndIf
    
    If x<0 : x=0 : EndIf
    If y<0 : y=0 : EndIf
    If x>output_width : x=output_width : EndIf
    If y>output_height : y=output_height : EndIf
    If xx>output_width : xx=output_width : EndIf
    If yy>output_height : yy=output_height : EndIf
    If x>xx : x=xx : EndIf
    If y>yy : y=yy : EndIf
    
    Delay_internal(0, -1) ; Init internal timer
    
    If image
      result=StartDrawing(ImageOutput(input_ID))
    ElseIf sprite
      result=StartDrawing(SpriteOutput(input_ID))
    Else
      result=StartDrawing(CanvasOutput(input_ID))
    EndIf
    
    If result
      search_again:
      delay_1+Delay_internal(0, 1000) ; Use internal timer 0
      If delay_1>delay
        StopDrawing()
        ProcedureReturn 0
      EndIf    
      test_color=Random($FFFFFE, 1) ; Ignore 0 and $FFFFFF
      For i=y To yy
        For ii=x To xx
          If test_color=Point(ii, i)
            Goto search_again 
          EndIf
        Next ii
      Next i
      StopDrawing()
      ProcedureReturn test_color
    Else
      ProcedureReturn -17
    EndIf
  EndProcedure
  
  Procedure SearchUnusedImageColor_BF(input_ID)
    Protected unused_color, input_width, input_height
    If IsImage(input_ID) ; check for image
      input_width=ImageWidth(input_ID) : input_height=ImageHeight(input_ID)
    ElseIf IsSprite(input_ID) ; Check for PB sprite
      input_width=SpriteWidth(input_ID) : input_height=SpriteHeight(input_ID)
    ElseIf IsGadget(input_ID) ; Check for canvas
      input_width=GadgetWidth(input_ID)*DPI_ResizeFactor_BF() : input_height=GadgetHeight(input_ID)*DPI_ResizeFactor_BF()
    Else
      ProcedureReturn -17
    EndIf
    unused_color=SearchUnusedColor_BF(input_ID, 0, 0, input_width-1, input_height-1, 5)
    If unused_color<1 : unused_color=Random($FFFFFE, 1) : EndIf ; Fuse - Error handling
    ProcedureReturn unused_color
  EndProcedure
  
  Procedure SearchUsedColor_BF(input_ID, x, y, xx, yy, search_color)
    Protected i, ii, image, sprite, output_width, output_height, result
    
    If IsImage(input_ID) ; check for image
      image=1
    ElseIf IsSprite(input_ID) ; Check for PB sprite
      sprite=1
    ElseIf Not IsGadget(input_ID) ; Check for canvas
      ProcedureReturn -17
    EndIf
    
    If image
      output_width=ImageWidth(input_ID)-1
      output_height=ImageHeight(input_ID)-1
    ElseIf sprite
      sprite_w_h(input_ID)
      output_width=sprite_w_h\sprite_w-1
      output_height=sprite_w_h\sprite_h-1
    Else
      output_width=(GadgetWidth(input_ID)-1)*DPI_ResizeFactor_BF()
      output_height=(GadgetHeight(input_ID)-1)*DPI_ResizeFactor_BF()
    EndIf
    
    If x<0 : x=0 : EndIf
    If y<0 : y=0 : EndIf
    If x>output_width : x=output_width : EndIf
    If y>output_height : y=output_height : EndIf
    If xx>output_width : xx=output_width : EndIf
    If yy>output_height : yy=output_height : EndIf
    If x>xx : x=xx : EndIf
    If y>yy : y=yy : EndIf
    
    If image
      result=StartDrawing(ImageOutput(input_ID))
    ElseIf sprite
      result=StartDrawing(SpriteOutput(input_ID))
    Else
      result=StartDrawing(CanvasOutput(input_ID))
    EndIf
    
    If result
      For i=y To yy
        For ii=x To xx
          If search_color=Point(ii, i)
            StopDrawing()
            ProcedureReturn 1
          EndIf
        Next ii
      Next i
      StopDrawing()
    Else
      ProcedureReturn -17
    EndIf
    ProcedureReturn 0
  EndProcedure
  
  Procedure Scale_Image_to_width_BF(image_ID, new_width, create_new_image=1)
    Protected image_width.d=ImageWidth(image_ID)
    Protected image_height.d=ImageHeight(image_ID)
    Protected scale_factor.d=new_width/image_width
    If create_new_image=1
      Protected image_resized_ID=CopyImage(image_ID, #PB_Any)
      If Not image_resized_ID
        ProcedureReturn -6
      EndIf 
    Else
      image_resized_ID=image_ID
    EndIf
    If Not ResizeImage(image_resized_ID, image_width*scale_factor, image_height*scale_factor, resize_interpolation)
      If create_new_image=1 : FreeImage(image_resized_ID) : EndIf
      ProcedureReturn -27
    EndIf
    ProcedureReturn(image_resized_ID)
  EndProcedure
  
  Procedure Scale_Image_to_height_BF(image_ID, new_height, create_new_image=1)
    Protected image_width.d=ImageWidth(image_ID)
    Protected image_height.d=ImageHeight(image_ID)
    Protected scale_factor.d=new_height/image_height
    If create_new_image=1
      Protected image_resized_ID=CopyImage(image_ID, #PB_Any)
      If Not image_resized_ID
        ProcedureReturn -6
      EndIf 
    Else
      image_resized_ID=image_ID
    EndIf
    If Not ResizeImage(image_resized_ID, image_width*scale_factor, image_height*scale_factor, resize_interpolation)
      If create_new_image=1 : FreeImage(image_resized_ID) : EndIf
      ProcedureReturn -27
    EndIf
    ProcedureReturn(image_resized_ID)
  EndProcedure
  
  Procedure GrabImage_BF(image_ID)
    Protected *drawing_buffer_grabed_image, drawing_buffer_len
    If Not IsImage(image_ID) : ProcedureReturn -2 : EndIf
    If Not StartDrawing(ImageOutput(image_ID))
      ProcedureReturn -12
    EndIf
    drawing_buffer_len=DrawingBufferPitch()*OutputHeight()
    *drawing_buffer_grabed_image=AllocateMemory(drawing_buffer_len, #PB_Memory_NoClear)
    If Not *drawing_buffer_grabed_image
      StopDrawing()
      ProcedureReturn -13
    EndIf
    CopyMemory(DrawingBuffer(), *drawing_buffer_grabed_image, drawing_buffer_len)
    StopDrawing()
    ProcedureReturn *drawing_buffer_grabed_image
  EndProcedure
  
  Procedure Grab_PB_Sprite_BF(sprite_ID)
    Protected *drawing_buffer_grabed_sprite, drawing_buffer_len
    If Not IsSprite(sprite_ID) : ProcedureReturn -18 : EndIf
    If Not StartDrawing(SpriteOutput(sprite_ID))
      ProcedureReturn -19
    EndIf
    drawing_buffer_len=DrawingBufferPitch()*OutputHeight()
    *drawing_buffer_grabed_sprite=AllocateMemory(drawing_buffer_len, #PB_Memory_NoClear)
    If Not *drawing_buffer_grabed_sprite
      StopDrawing()
      ProcedureReturn -13
    EndIf
    CopyMemory(DrawingBuffer(), *drawing_buffer_grabed_sprite, drawing_buffer_len)
    StopDrawing()
    ProcedureReturn *drawing_buffer_grabed_sprite
  EndProcedure
  
  Procedure GrabCanvas_BF(canvas_ID)
    Protected *drawing_buffer_grabed_canvas, drawing_buffer_len
    If Not IsGadget(canvas_ID) : ProcedureReturn -1 : EndIf
    If Not StartDrawing(CanvasOutput(canvas_ID))
      ProcedureReturn -12
    EndIf
    drawing_buffer_len=DrawingBufferPitch()*OutputHeight()
    *drawing_buffer_grabed_canvas=AllocateMemory(drawing_buffer_len, #PB_Memory_NoClear)
    If Not *drawing_buffer_grabed_canvas
      StopDrawing()
      ProcedureReturn -13
    EndIf
    CopyMemory(DrawingBuffer(), *drawing_buffer_grabed_canvas, drawing_buffer_len)
    StopDrawing()
    ProcedureReturn *drawing_buffer_grabed_canvas
  EndProcedure
  
  Procedure DrawingBuffer_image_BF(image_ID)
    Protected *drawing_buffer
    If Not IsImage(image_ID) : ProcedureReturn -2 : EndIf
    If Not StartDrawing(ImageOutput(image_ID))
      ProcedureReturn -14
    EndIf
    *drawing_buffer=DrawingBuffer()
    If Not *drawing_buffer
      StopDrawing()
      ProcedureReturn -15
    EndIf
    StopDrawing()
    ProcedureReturn *drawing_buffer
  EndProcedure
  
  Procedure DrawingBuffer_PB_Sprite_BF(sprite_ID)
    Protected *drawing_buffer
    If Not IsSprite(sprite_ID) : ProcedureReturn -18 : EndIf
    If Not StartDrawing(SpriteOutput(sprite_ID))
      ProcedureReturn -19
    EndIf
    *drawing_buffer=DrawingBuffer()
    If Not *drawing_buffer
      StopDrawing()
      ProcedureReturn -15
    EndIf
    StopDrawing()
    ProcedureReturn *drawing_buffer
  EndProcedure
  
  Procedure DrawingBuffer_canvas_BF(canvas_ID)
    Protected *drawing_buffer
    If Not IsGadget(canvas_ID) : ProcedureReturn -1 : EndIf
    If Not StartDrawing(CanvasOutput(canvas_ID))
      ProcedureReturn -14
    EndIf
    *drawing_buffer=DrawingBuffer()
    If Not *drawing_buffer
      StopDrawing()
      ProcedureReturn -15
    EndIf
    StopDrawing()
    ProcedureReturn *drawing_buffer
  EndProcedure
  
  Procedure Texture_FloodFill(x, y, output_ID, texture_ID)
    
    Protected texture_width_=ImageWidth(texture_ID)-1
    Protected texture_height_=ImageHeight(texture_ID)-1
    Protected image, sprite, result, output_width, output_height, output_width_1, output_height_1
    Protected texture_width__1=texture_width_+1, texture_height__1=texture_height_+1 
    Protected x1, y1, old_color, point, texture_1_ID, texture_2_ID, i
    Protected r1, g1, b1, r2, g2, b2, diff_red, diff_green, diff_blue
    Protected r.f, g.f, b.f
    
    Protected change_gradient_color_alpha=255-gradient_color_alpha
    If change_gradient_color_alpha<0 : change_gradient_color_alpha=0 : EndIf
    If change_gradient_color_alpha>255 : change_gradient_color_alpha=255 : EndIf
    
    If IsImage(output_ID) ; check for image
      image=1
    ElseIf IsSprite(output_ID) ; Check for PB sprite
      sprite=1
    ElseIf Not IsGadget(output_ID) ; Check for canvas
      ProcedureReturn -11
    EndIf
    
    If image
      output_width_1=ImageWidth(output_ID)
      output_height_1=ImageHeight(output_ID)
    ElseIf sprite
      sprite_w_h(output_ID)
      output_width_1=sprite_w_h\sprite_w
      output_height_1=sprite_w_h\sprite_h
    Else
      output_width_1=GadgetWidth(output_ID)
      output_height_1=GadgetHeight(output_ID)
      If activate_dpi_aware
        output_width_1*DPI_ResizeFactor_BF()
        output_height_1*DPI_ResizeFactor_BF()
      EndIf
    EndIf
    
    output_width=output_width_1-1
    output_height=output_height_1-1 
    
    Dim picture_0(output_width, output_height)
    
    If activate_flood_array
      flood_array_length=(output_width_1*output_height_1)<<3
      flood_array_width=output_width_1
      flood_array_height=output_height_1
    EndIf
    
    Structure point_
      x.i
      y.i
    EndStructure
    
    If x<0 Or x>output_width Or y<0 Or y>output_height
      ProcedureReturn -8
    EndIf
    
    max_x_0=0
    max_y_0=0
    min_x_0=output_width
    min_y_0=output_height
    
    If image
      result=StartDrawing(ImageOutput(output_ID))
    ElseIf sprite
      result=StartDrawing(SpriteOutput(output_ID))
    Else
      result=StartDrawing(CanvasOutput(output_ID))
    EndIf
    
    If result
      old_color=Point(x, y)
      NewList stack.point_()
      AddElement(stack()) : stack()\x=x : stack()\y=y
      While(LastElement(stack()))
        x=stack()\x : y=stack()\y
        DeleteElement(stack())
        If x>-1 And x<output_width_1 And y>-1 And y<output_height_1 And Not picture_0(x, y)
          point=Point(x, y)
        Else
          point=-1
          If x<0 : x= 0 : EndIf
        EndIf
        
        Macro add_stack
          picture_0(x, y)=-1
          
          If activate_flood_array
            If x<min_x_0
              min_x_0=x
            EndIf
            If y<min_y_0
              min_y_0=y
            EndIf
            If x>max_x_0
              max_x_0=x
            EndIf
            If y>max_y_0
              max_y_0=y
            EndIf
          EndIf
          
          AddElement(stack()) : stack()\x=x   : stack()\y=y+1
          AddElement(stack()) : stack()\x=x   : stack()\y=y-1
          AddElement(stack()) : stack()\x=x+1 : stack()\y=y
          AddElement(stack()) : stack()\x=x-1 : stack()\y=y
        EndMacro
        
        If point=-1
          If point=old_color
            add_stack
          EndIf
        Else
          If percent_1>0
            If ColorDistance_BF(point, old_color)<=percent_1 
              add_stack
            EndIf
          Else
            If point=old_color
              add_stack
            EndIf
          EndIf
        EndIf
      Wend
      
    Else
      
      If image
        ProcedureReturn -2
      ElseIf sprite 
        ProcedureReturn -18
      Else
        ProcedureReturn -1
      EndIf
      
    EndIf
    
    StopDrawing()
    
    If Not ignore_gradient_colors And (activate_linear_gradient Or activate_elliptical_gradient Or activate_boxed_gradient Or activate_circular_gradient Or activate_conical_gradient)
      
      texture_1_ID=CreateImage(#PB_Any, output_width, output_height)
      If Not texture_1_ID : ProcedureReturn -4 : EndIf
      texture_2_ID=CopyImage(texture_ID, #PB_Any)
      If Not texture_2_ID : FreeImage(texture_1_ID ) : ProcedureReturn -4 : EndIf
      
      If Not StartDrawing(ImageOutput(texture_1_ID))
        FreeImage(texture_1_ID) : FreeImage(texture_2_ID) : ProcedureReturn -4
      EndIf
      
      DrawingMode(#PB_2DDrawing_Gradient)
      If activate_backcolor
        BackColor(backcolor)
      EndIf
      If activate_frontcolor
        FrontColor(frontcolor)
      EndIf
      For i=0 To #Size_GradientColors_Array
        If gradient_activate_flag(i)
          GradientColor(gradient_position.f(i), gradient_color(i))
        EndIf
      Next
      
      If activate_linear_gradient
        LinearGradient(linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy)
      EndIf
      
      If activate_circular_gradient
        CircularGradient(circular_gradient_x, circular_gradient_y, circular_gradient_radius)
      EndIf
      
      If activate_elliptical_gradient
        EllipticalGradient(elliptical_gradient_x, elliptical_gradient_y, elliptical_gradient_xx, elliptical_gradient_yy)
      EndIf
      
      If activate_boxed_gradient
        BoxedGradient(boxed_gradient_x, boxed_gradient_y, boxed_gradient_width, boxed_gradient_height)
      EndIf
      
      If activate_conical_gradient
        ConicalGradient(conical_gradient_x, conical_gradient_y, conical_gradient_degree)
      EndIf
      
      Box(0, 0, output_width, output_height)
      
      DrawingMode(#PB_2DDrawing_Default)
      
      StopDrawing()
      
      If gradient_color_red Or gradient_color_green Or gradient_color_blue Or gradient_color_contrast Or gradient_color_brightness Or gradient_color_intensity
        Image_coloring_BF(texture_1_ID, texture_1_ID, ; Post processing - Optimize sprite
                          gradient_color_red,
                          gradient_color_green,
                          gradient_color_blue,
                          gradient_color_contrast,
                          gradient_color_brightness,
                          gradient_color_intensity)
      EndIf
      
      If Not StartDrawing(ImageOutput(texture_2_ID))
        FreeImage(texture_1_ID) : FreeImage(texture_2_ID) : ProcedureReturn -4
      EndIf
      
      DrawAlphaImage(ImageID(texture_1_ID), 0, 0, change_gradient_color_alpha)
      StopDrawing()
      
      FreeImage(texture_1_ID)
      
      Protected Dim texture(output_width_1, output_height_1)
      
      If StartDrawing(ImageOutput(texture_2_ID))
        For y=0 To output_height_1
          For x=0 To output_width_1
            If x>-1 And y>-1 And x<output_width_1 And y<output_height_1
              texture(x, y)=Point(x, y)
            EndIf
          Next x
        Next y
      Else
        FreeImage (texture_2_ID)
        ProcedureReturn -4
      EndIf
      StopDrawing()
      
      FreeImage(texture_2_ID)
      
      If image
        result=StartDrawing(ImageOutput(output_ID))
      ElseIf sprite
        result=StartDrawing(SpriteOutput(output_ID))
      Else
        result=StartDrawing(CanvasOutput(output_ID))
      EndIf
      
      If transparent_flood_fill
        DrawingMode(#PB_2DDrawing_AllChannels)
      EndIf
      
      If result
        For y=0 To output_height
          For x=0 To output_width
            If picture_0(x, y)=-1
              Plot(x, y, texture(x, y)) 
            EndIf
          Next x
        Next y 
      Else
        If image
          ProcedureReturn -2
        ElseIf sprite 
          ProcedureReturn -18
        Else
          ProcedureReturn -1
        EndIf
      EndIf
      
      StopDrawing()
      
    Else
      
      Dim texture(output_width, output_height)
      
      If StartDrawing(ImageOutput(texture_ID))
        For y=0 To texture_height_
          For x=0 To texture_width_
            If x>-1 And y>-1 And x<output_width_1 And y<output_height_1
              texture(x, y)=Point(x, y)
            EndIf
          Next x
        Next y
      Else
        ProcedureReturn -4
      EndIf
      StopDrawing()
      
      If image
        result=StartDrawing(ImageOutput(output_ID))
      ElseIf sprite
        result=StartDrawing(SpriteOutput(output_ID))
      Else
        result=StartDrawing(CanvasOutput(output_ID))
      EndIf
      
      If transparent_flood_fill
        DrawingMode(#PB_2DDrawing_AllChannels)
      EndIf
      
      If result
        For y=0 To output_height
          For x=0 To output_width
            If picture_0(x, y)=-1
              Plot(x, y, texture(x, y)) 
            EndIf
          Next x
        Next y
      Else
        If image
          ProcedureReturn -2
        ElseIf sprite 
          ProcedureReturn -18
        Else
          ProcedureReturn -1
        EndIf
      EndIf
      
      StopDrawing()
      
    EndIf
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure FloodFill_BF(mode, output_ID, texture_ID,
                         x,
                         y,
                         texture_x=0,
                         texture_y=0, 
                         texture_width=0,
                         texture_height=0,
                         texture_clip_x=0,
                         texture_clip_y=0,
                         texture_clip_width=0,
                         texture_clip_height=0,
                         texture_resize_x=0,
                         texture_resize_y=0)
    
    Protected image, sprite, result, output_width_1, output_height_1, temporary_texture_ID, color
    
    If IsImage(output_ID) ; check for image
      image=1
    ElseIf IsSprite(output_ID) ; Check for PB sprite
      sprite=1
    ElseIf Not IsGadget(output_ID) ; Check for canvas
      ProcedureReturn -11
    EndIf
    
    If image
      output_width_1=ImageWidth(output_ID)
      output_height_1=ImageHeight(output_ID)
    ElseIf sprite
      sprite_w_h(output_ID)
      output_width_1=sprite_w_h\sprite_w
      output_height_1=sprite_w_h\sprite_h
    Else
      output_width_1=GadgetWidth(output_ID)
      output_height_1=GadgetHeight(output_ID)
      If activate_dpi_aware
        output_width_1*DPI_ResizeFactor_BF()
        output_height_1*DPI_ResizeFactor_BF()
      EndIf
    EndIf
    
    If Not mode : SetPosMarker_BF(output_ID, x, y) : ProcedureReturn 1 : EndIf
    
    If mode<0 : mode=-mode : EndIf
    If mode<>1 And Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    
    If image
      result=StartDrawing(ImageOutput(output_ID))
    ElseIf sprite
      result=StartDrawing(SpriteOutput(output_ID))
    Else
      result=StartDrawing(CanvasOutput(output_ID))
    EndIf
    
    If result
      If x<0 Or x>output_width_1-1 : x=0 : EndIf
      If y<0 Or y>output_height_1-1 : y=0 : EndIf
      color=Point(x, y)
      StopDrawing()
    Else
      StopDrawing()
      If image
        ProcedureReturn -2
      ElseIf sprite 
        ProcedureReturn -18
      Else
        ProcedureReturn -1
      EndIf
    EndIf
    
    temporary_texture_ID=CreateImage(#PB_Any, output_width_1, output_height_1) 
    If Not temporary_texture_ID
      ProcedureReturn -6
    EndIf
    
    If StartDrawing(ImageOutput(temporary_texture_ID))
      Box(0, 0, output_width_1, output_height_1, color)
      StopDrawing()
    Else
      FreeImage(temporary_texture_ID)
      ProcedureReturn -5
    EndIf
    
    If IsImage(texture_ID)
      If activate_dpi_aware
        If texture_resize_x<1 : texture_resize_x=ImageWidth(texture_ID) : EndIf
        If texture_resize_y<1 : texture_resize_y=ImageHeight(texture_ID) : EndIf
        If texture_resize_x>0 Or texture_resize_y>0 Or activate_dpi_aware
          Protected temporary_texture_1_ID=CopyImage(texture_ID, #PB_Any)
          If Not temporary_texture_1_ID
            ProcedureReturn -6
          EndIf
          If Not ResizeImage(temporary_texture_1_ID, texture_resize_x*DPI_ResizeFactor_BF(), texture_resize_y*DPI_ResizeFactor_BF(), resize_interpolation)
            FreeImage(temporary_texture_ID)
            FreeImage(temporary_texture_1_ID)
            ProcedureReturn -27
          EndIf
          texture_ID=temporary_texture_1_ID
        EndIf
      Else
        If texture_resize_x<1 : texture_resize_x=ImageWidth(texture_ID) : EndIf
        If texture_resize_y<1 : texture_resize_y=ImageHeight(texture_ID) : EndIf
        If texture_resize_x>0 Or texture_resize_y>0  
          temporary_texture_1_ID=CopyImage(texture_ID, #PB_Any)
          If Not temporary_texture_1_ID
            ProcedureReturn -6
          EndIf
          If Not ResizeImage(temporary_texture_1_ID, texture_resize_x, texture_resize_y, resize_interpolation)
            FreeImage(temporary_texture_ID)
            FreeImage(temporary_texture_1_ID)
            ProcedureReturn -27
          EndIf
          texture_ID=temporary_texture_1_ID
        EndIf
      EndIf
    EndIf
    
    result=BF(-mode, temporary_texture_ID, texture_ID,
              0,
              0,
              texture_x,
              texture_y, 
              texture_width,
              texture_height,
              texture_clip_x,
              texture_clip_y,
              texture_clip_width,
              texture_clip_height)
    
    FreeImage_BF(temporary_texture_1_ID)
    
    If result<1 : FreeImage(temporary_texture_ID) : ProcedureReturn result : EndIf
    
    result=Texture_FloodFill(x, y, output_ID, temporary_texture_ID)
    
    FreeImage(temporary_texture_ID)
    
    ProcedureReturn result
    
  EndProcedure
  
  Procedure TransparentFloodFill_BF(transparent_flood_fill_)
    transparent_flood_fill=transparent_flood_fill_
  EndProcedure
  
  ; Enhanced BucketFill function with texture support, color distance, clipping and automatic tiling
  ; This function can handle color distance with SetColorDistanceFill_BF -It's ever needed for removing mask artifacts
  ; As sample, Sprite rotating BF create ever little border artifacts, because using interpolation
  ; Hint : ColorDistanceFill_BF can influence and so break a used fill function clipping
  ; This is a BF core based function and can same all BF core based functions automatically clipping, tiling, scrolling
  ; All BF core based functions use automatic BF texture caching
  ; This caching is activated by default
  ; BF core based functions which can write from a to b so even write in themselves without changing the source permanently
  ; So write operations on the source do not show any changes when caching is activated
  ; If you want to change the source permanently, switch off temporary the caching before with NoCaching_BF(1)
  Procedure BucketFill_BF(mode, output_ID, texture_ID,
                          output_get_color_x=0,  ; Grab transparence color from sprite pos x - For preset set ..x=-1  
                          output_get_color_y=0,  ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF 
                          texture_x=0,           ; Startposition texture output x - Preset 0
                          texture_y=0,           ; Startposition texture output y - Preset 0
                          texture_width=0,       ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                          texture_height=0,      ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                          texture_clip_x=0,      ; Startposition inside the texture x - Preset 0
                          texture_clip_y=0,      ; Startposition inside the texture y - Preset 0
                          texture_clip_width=0,  ; Endposition inside the texture - Preset 0 = full texture width
                          texture_clip_height=0, ; Endposition inside the texture - Preset 0 = full texture height
                          texture_resize_x=0,    ; Pre resize texture x
                          texture_resize_y=0)    ; Pre resize texture y
                                                 ; mode = 0         - Set only a positioning marker
                                                 ;       -1         - Ignore a texture and use a color - Set as texture_ID a color, as sample $FFFF00
                                                 ;       -2         - Standard preset texture mode
                                                 ;       -3 to -256 - Texture mode with alpha blending
                                                 ; For compatibility with BucketFill this function use negative mode values, but it works at the same also with positive values
                                                 ; This function can output on images, PB sprites and canvas
    
    If Not mode : SetPosMarker_BF(output_ID, texture_x, texture_y) : ProcedureReturn 1 : EndIf
    
    Protected old_invisible_color.q=invisible_color
    
    If mode>0 : mode=-mode : EndIf
    
    If output_get_color_x=-1
      invisible_color=output_get_color_y
    EndIf
    
    If IsImage(texture_ID)
      If activate_dpi_aware
        If texture_resize_x<1 : texture_resize_x=ImageWidth(texture_ID) : EndIf
        If texture_resize_y<1 : texture_resize_y=ImageHeight(texture_ID) : EndIf
        If texture_resize_x>0 Or texture_resize_y>0 Or activate_dpi_aware
          Protected temporary_texture_ID=CopyImage(texture_ID, #PB_Any)
          If Not temporary_texture_ID
            ProcedureReturn -6
          EndIf
          If Not ResizeImage(temporary_texture_ID, texture_resize_x*DPI_ResizeFactor_BF(), texture_resize_y*DPI_ResizeFactor_BF(), resize_interpolation)
            FreeImage(temporary_texture_ID)
            ProcedureReturn -27
          EndIf
          texture_ID=temporary_texture_ID
        EndIf
      Else
        If texture_resize_x<1 : texture_resize_x=ImageWidth(texture_ID) : EndIf
        If texture_resize_y<1 : texture_resize_y=ImageHeight(texture_ID) : EndIf
        If texture_resize_x>0 Or texture_resize_y>0  
          temporary_texture_ID=CopyImage(texture_ID, #PB_Any)
          If Not temporary_texture_ID
            ProcedureReturn -6
          EndIf
          If Not ResizeImage(temporary_texture_ID, texture_resize_x, texture_resize_y, resize_interpolation)
            FreeImage(temporary_texture_ID)
            ProcedureReturn -27
          EndIf
          texture_ID=temporary_texture_ID
        EndIf
      EndIf
    EndIf
    
    Protected result=BF(mode, output_ID, texture_ID,
                        output_get_color_x,
                        output_get_color_y,
                        texture_x,
                        texture_y, 
                        texture_width,
                        texture_height,
                        texture_clip_x,
                        texture_clip_y,
                        texture_clip_width,
                        texture_clip_height)
    
    FreeImage_BF(temporary_texture_ID)
    
    invisible_color=old_invisible_color
    
    ProcedureReturn result
  EndProcedure
  
  ; Main (core) function
  Procedure BF(mode, output_ID, texture_ID,
               output_get_color_x=0,
               output_get_color_y=0,
               texture_x=0,
               texture_y=0, 
               texture_width=0,
               texture_height=0,
               texture_clip_x=0,
               texture_clip_y=0,
               texture_clip_width=0,
               texture_clip_height=0)
    
    Protected temp_pos_s_x, temp_pos_s_y, temp_s_y, i, ii, mode_1, break_height, sprite_input, canvas_input
    Protected image_width, image_height, image_width_1, image_height_1, texture_height_offset, texture_array_size
    Protected texture_width_, texture_height_, point, refresh_texture, texture_point, result, image, sprite
    Protected used_image_color.q
    Protected *texture_buffer
    
    If mode<-256 Or mode>256 : mode=0 : EndIf
    
    If screen_output
      mode=1
      image_width_1=ScreenWidth()
      image_height_1=ScreenHeight()
    Else
      If IsImage(output_ID) ; check for image
        image=1
        image_width_1=ImageWidth(output_ID)
        image_height_1=ImageHeight(output_ID)
      ElseIf IsSprite(output_ID) ; Check for PB sprite
        sprite=1
        sprite_w_h(output_ID)
        image_width_1=sprite_w_h\sprite_w
        image_height_1=sprite_w_h\sprite_h
      ElseIf IsGadget(output_ID) ; Check for canvas
        image_width_1=GadgetWidth(output_ID)
        image_height_1=GadgetHeight(output_ID)
        If activate_dpi_aware
          image_width_1*DPI_ResizeFactor_BF()
          image_height_1*DPI_ResizeFactor_BF()
        EndIf
      Else
        ProcedureReturn -11
      EndIf
    EndIf
    
    If mode<>-1
      If IsImage(texture_ID)
        texture_width_=ImageWidth(texture_ID)-1
        texture_height_=ImageHeight(texture_ID)-1
      Else
        If IsSprite(texture_ID)
          sprite_input=1
          sprite_w_h(texture_ID)
          texture_width_=sprite_w_h\sprite_w-1
          texture_height_=sprite_w_h\sprite_h-1
        ElseIf IsGadget(texture_ID)
          canvas_input=1
          texture_width_=GadgetWidth(texture_ID)-1
          texture_height_=GadgetHeight(texture_ID)-1
          If activate_dpi_aware
            texture_width_*DPI_ResizeFactor_BF()
            texture_height_*DPI_ResizeFactor_BF()
          EndIf
        Else
          ProcedureReturn -17    
        EndIf
      EndIf
    EndIf
    
    If texture_width>image_width_1-texture_x Or texture_width<1 
      texture_width=image_width_1-texture_x
    EndIf
    If texture_height>image_height_1-texture_y Or texture_height<1
      texture_height=image_height_1-texture_y
    EndIf
    
    image_width=image_width_1-1
    image_height=image_height_1-1
    
    ; Primary clipping
    If texture_width+texture_x<0 Or texture_height+texture_y<0
      ProcedureReturn 1
    EndIf
    If texture_x>image_width Or texture_y>image_height
      ProcedureReturn 1
    EndIf
    
    If activate_bucket_array And mode<1     
      max_x_1=0
      max_y_1=0
      min_x_1=image_width
      min_y_1=image_height
      If activate_bucket_array<2
        Dim picture_1(image_width, image_height)
        bucket_array_length=(image_width_1*image_height_1)<<3
        bucket_array_width=image_width_1
        bucket_array_height=image_height_1
      EndIf
    EndIf
    
    If activate_sprite_array
      max_x_2=0
      max_y_2=0
      min_x_2=image_width
      min_y_2=image_height
      If activate_sprite_array<2
        Dim picture_2(image_width, image_height)
        sprite_array_length=(image_width_1*image_height_1)<<3
        sprite_array_width=image_width_1
        sprite_array_height=image_height_1
      EndIf
    EndIf
    
    texture_clip_width-1
    texture_clip_height-1
    
    If texture_clip_width>texture_width_ Or texture_clip_width<0
      texture_clip_width=texture_width_
    EndIf
    If texture_clip_height>texture_height_ Or texture_clip_height<1
      texture_clip_height=texture_height_
    EndIf
    
    If texture_clip_x<0 Or texture_clip_x>texture_clip_width
      texture_clip_x=0
    EndIf
    If texture_clip_y<0 Or texture_clip_y>texture_clip_height
      texture_clip_y=0
    EndIf
    
    If output_get_color_x<0 Or output_get_color_y<0
      output_get_color_x=-1
    EndIf
    
    If mode>0  
      If output_get_color_x>texture_clip_width-texture_clip_x
        output_get_color_x=0
      EndIf
      If output_get_color_y>texture_clip_height-texture_clip_y
        output_get_color_y=0
      EndIf
    Else
      If output_get_color_x>image_width
        output_get_color_x=0
      EndIf
      If output_get_color_y>image_height
        output_get_color_y=0
      EndIf
    EndIf
    
    texture_clip_width-texture_clip_x
    texture_clip_height-texture_clip_y
    
    Protected Dim texture(texture_clip_width, texture_clip_height)
    
    If mode>0 And output_get_color_x>-1
      If sprite_input
        result=StartDrawing(SpriteOutput(texture_ID))
      ElseIf canvas_input
        result=StartDrawing(CanvasOutput(texture_ID))
      Else
        result=StartDrawing(ImageOutput(texture_ID))
      EndIf
      If result
        used_image_color=Point(output_get_color_x+texture_clip_x, output_get_color_y+texture_clip_y)
        StopDrawing()
      Else
        If sprite_input : ProcedureReturn -18 : EndIf
        If canvas_input : ProcedureReturn -1 : EndIf
        ProcedureReturn -4
      EndIf
    Else
      If texture_get_color<>-1 And output_get_color_x=-1
        If old_texture_get_color<>texture_get_color
          old_texture_get_color=texture_get_color
          refresh_texture=1
        EndIf
        used_image_color=texture_get_color
      EndIf
    EndIf
    
    If invisible_color<>-1
      used_image_color=invisible_color
    EndIf
    
    Macro get_texture
      result=1
      If sprite_input
        result=StartDrawing(SpriteOutput(texture_ID))
      ElseIf canvas_input
        result=StartDrawing(CanvasOutput(texture_ID))
      Else
        result=StartDrawing(ImageOutput(texture_ID)) 
      EndIf
      If result
        If alpha_sprite
          If Not screen_output
            DrawingMode(#PB_2DDrawing_AlphaBlend)
          EndIf
        EndIf
        
        If output_get_color_x>-1
          If percent>0 And Not alpha_sprite
            For i=0 To texture_clip_height
              For ii=0 To texture_clip_width
                point=Point(ii+texture_clip_x, i+texture_clip_y)
                If ColorDistance_BF(point, used_image_color)<=percent
                  texture(ii, i)=used_image_color
                Else
                  texture(ii, i)=point
                EndIf
              Next ii
            Next i
          Else
            For i=0 To texture_clip_height
              For ii=0 To texture_clip_width
                point=Point(ii+texture_clip_x, i+texture_clip_y)
                If point=used_image_color
                  texture(ii, i)=used_image_color
                Else
                  texture(ii, i)=point
                EndIf
              Next ii
            Next i
          EndIf 
        Else
          For i=0 To texture_clip_height
            For ii=0 To texture_clip_width
              texture(ii, i)=Point(ii+texture_clip_x, i+texture_clip_y)
            Next ii
          Next i
        EndIf
      Else
        If sprite_input : ProcedureReturn -18 : EndIf
        If canvas_input : ProcedureReturn -1 : EndIf
        ProcedureReturn -4
      EndIf
      StopDrawing()
    EndMacro
    
    texture_array_size=(texture_clip_width+1)*(texture_clip_height+1)*SizeOf(integer)
    
    Macro add_element
      get_texture
      *texture_buffer=AllocateMemory(texture_array_size, #PB_Memory_NoClear)
      CopyMemory(@texture(), *texture_buffer, texture_array_size)
      AddElement(textures())
      textures()\texture_ID=texture_ID
      textures()\texture_buffer=*texture_buffer
      textures()\texture_clip_width=texture_clip_width
      textures()\texture_clip_height=texture_clip_height
      textures()\texture_percent_transparent=percent
    EndMacro
    
    If mode=-1
      texture(0, 0)=texture_ID
    Else
      If no_caching
        get_texture
      Else
        i=0
        ForEach(textures())
          If textures()\texture_ID=texture_ID
            If Not refresh_texture And
               textures()\texture_clip_width=texture_clip_width And
               textures()\texture_clip_height=texture_clip_height And
               textures()\texture_percent_transparent=percent
              *texture_buffer=textures()\texture_buffer
              CopyMemory(*texture_buffer, @texture(), texture_array_size)
            Else
              textures()\texture_clip_width=texture_clip_width
              textures()\texture_clip_height=texture_clip_height
              textures()\texture_percent_transparent=percent
              FreeMemory(textures()\texture_buffer)
              Dim texture(texture_clip_width, texture_clip_height)
              get_texture
              *texture_buffer=AllocateMemory(texture_array_size, #PB_Memory_NoClear)
              textures()\texture_buffer=*texture_buffer
              CopyMemory(@texture(), *texture_buffer, texture_array_size)
            EndIf
            i=1
            Break
          EndIf
        Next
        If Not i
          add_element
        EndIf
      EndIf
    EndIf
    
    texture_width+texture_x
    temp_pos_s_x=texture_x
    temp_pos_s_y=texture_y
    texture_height_offset=texture_height+texture_y-1
    
    If screen_output
      result=StartDrawing(ScreenOutput())
    Else
      If image
        result=StartDrawing(ImageOutput(output_ID))
      ElseIf sprite
        result=StartDrawing(SpriteOutput(output_ID))
      Else
        result=StartDrawing(CanvasOutput(output_ID))
      EndIf
    EndIf
    
    If result
      If alpha_sprite
        If Not screen_output
          DrawingMode(#PB_2DDrawing_AlphaBlend)
        EndIf
      EndIf
      
      If output_get_color_x>-1
        If mode>0 ; Get a color
          If invisible_color=-1
            used_image_color=texture(output_get_color_x, output_get_color_y)
          Else
            used_image_color=invisible_color
          EndIf
        Else
          If replacing_color=-1
            used_image_color=Point(output_get_color_x, output_get_color_y)
          Else
            used_image_color=replacing_color
          EndIf
        EndIf
      EndIf 
      
      Macro use_sprite_array
        If activate_sprite_array
          If activate_sprite_array<2 : picture_2(temp_pos_s_x, temp_pos_s_y)=-1 : EndIf
          If temp_pos_s_x<min_x_2 : min_x_2=temp_pos_s_x : EndIf
          If temp_pos_s_y<min_y_2 : min_y_2=temp_pos_s_y : EndIf
          If temp_pos_s_x>max_x_2 : max_x_2=temp_pos_s_x : EndIf
          If temp_pos_s_y>max_y_2 : max_y_2=temp_pos_s_y : EndIf
        EndIf
      EndMacro
      
      Macro use_bucket_array
        If activate_bucket_array
          If activate_bucket_array<2 : picture_1(temp_pos_s_x, temp_pos_s_y)=-1 : EndIf
          If temp_pos_s_x<min_x_1 : min_x_1=temp_pos_s_x : EndIf
          If temp_pos_s_y<min_y_1 : min_y_1=temp_pos_s_y : EndIf
          If temp_pos_s_x>max_x_1 : max_x_1=temp_pos_s_x : EndIf
          If temp_pos_s_y>max_y_1 : max_y_1=temp_pos_s_y : EndIf
        EndIf
      EndMacro
      
      If image_height<=texture_height_offset
        break_height=image_height_1
      Else
        break_height=texture_height_offset+1
      EndIf
      
      i=0 : ii=0 : mode_1=-mode
      If mode>0 And (output_get_color_x<0 Or replacing_color=-2) : used_image_color=-1 : EndIf
      While temp_pos_s_y<break_height
        If temp_pos_s_x<texture_width
          If temp_pos_s_x>-1 And temp_pos_s_y>-1
            If mode=1
              texture_point=texture(i, ii)
              If texture_point<>used_image_color
                Plot(temp_pos_s_x, temp_pos_s_y, texture_point)
                use_sprite_array
              EndIf
            ElseIf mode>1
              texture_point=texture(i, ii)
              If texture_point<>used_image_color
                If colormask_color<>-1
                  point=AlphaBlend(Point(temp_pos_s_x, temp_pos_s_y), colormask_color, colormask_alpha)
                  Plot(temp_pos_s_x, temp_pos_s_y, AlphaBlend(texture_point, point, mode))
                Else
                  Plot(temp_pos_s_x, temp_pos_s_y, AlphaBlend(texture_point, Point(temp_pos_s_x, temp_pos_s_y), mode))
                EndIf  
                use_sprite_array
              EndIf
            ElseIf mode<-2
              point=Point(temp_pos_s_x, temp_pos_s_y)
              If percent_1>0
                If ColorDistance_BF(point, used_image_color)<=percent_1
                  Plot(temp_pos_s_x, temp_pos_s_y, AlphaBlend(texture(i, ii), point, mode_1))
                  use_bucket_array
                EndIf
              Else
                If point=used_image_color
                  Plot(temp_pos_s_x, temp_pos_s_y, AlphaBlend(texture(i, ii), point, mode_1))
                  use_bucket_array
                EndIf
              EndIf
            Else ; not mode or mode=-1
              point=Point(temp_pos_s_x, temp_pos_s_y)
              If percent_1>0
                If ColorDistance_BF(point, used_image_color)<=percent_1
                  Plot(temp_pos_s_x, temp_pos_s_y, texture(i, ii))
                  use_bucket_array
                EndIf
              Else
                If point=used_image_color
                  Plot(temp_pos_s_x, temp_pos_s_y, texture(i, ii))
                  use_bucket_array
                EndIf
              EndIf
            EndIf
          EndIf
          temp_pos_s_x+1
        Else
          temp_pos_s_y+1
          ii+1 : If ii>texture_clip_height : ii=0 : EndIf
          temp_pos_s_x=texture_x : i+texture_clip_width
        EndIf
        i+1 : If i>texture_clip_width : i=0 : EndIf
      Wend 
    Else
      ProcedureReturn -3
    EndIf
    
    If Not mode ; Positioning marker
      Circle(output_get_color_x, output_get_color_y, 8, 0)
      Circle(output_get_color_x, output_get_color_y, 6, $FFFF00)
      Circle(output_get_color_x, output_get_color_y, 4, -1)
      Circle(output_get_color_x, output_get_color_y, 1, $FF)
    EndIf
    
    used_invisible_color=used_image_color
    
    StopDrawing()
    ProcedureReturn 1
  EndProcedure
  
  Procedure ScreenSprite_BF(texture_ID,
                            output_get_color_x=0,
                            output_get_color_y=0,
                            texture_x=0,
                            texture_y=0, 
                            texture_width=0,
                            texture_height=0,
                            texture_clip_x=0,
                            texture_clip_y=0,
                            texture_clip_width=0,
                            texture_clip_height=0)
    
    screen_output=1
    
    Protected result=BF(1, 0, texture_ID,
                        output_get_color_x,
                        output_get_color_y,
                        texture_x,
                        texture_y, 
                        texture_width,
                        texture_height,
                        texture_clip_x,
                        texture_clip_y,
                        texture_clip_width,
                        texture_clip_height)
    
    screen_output=0
    ProcedureReturn result
  EndProcedure
  
  Procedure GrabSpriteOffsets_BF(input_ID, offset_x=0, offset_y=0, offset_xx=0, offset_yy=0)
    Protected texture_width, texture_height
    If IsImage(input_ID)
      Protected image=1
      texture_width=ImageWidth(input_ID)
      texture_height=ImageHeight(input_ID)
    ElseIf IsSprite(input_ID)
      Protected sprite=1
      sprite_w_h(input_ID)
      texture_width=sprite_w_h\sprite_w
      texture_height=sprite_w_h\sprite_h
    ElseIf IsGadget(input_ID)
      Protected canvas=1
      texture_width=GadgetWidth(input_ID)*DPI_ResizeFactor_BF()
      texture_height=GadgetHeight(input_ID)*DPI_ResizeFactor_BF()
    Else
      ProcedureReturn -17
    EndIf
    
    Protected i, ii, texture_x, texture_y, texture_width_1, texture_height_1, used_image_color.d
    
    If canvas
      texture_x+(offset_x*DPI_ResizeFactor_BF())
      texture_y+(offset_y*DPI_ResizeFactor_BF())
      texture_width_1=texture_width-1-(offset_xx*DPI_ResizeFactor_BF())
      texture_height_1=texture_height-1-(offset_yy*DPI_ResizeFactor_BF())
    Else
      texture_x+offset_x
      texture_y+offset_y
      texture_width_1=texture_width-1-offset_xx
      texture_height_1=texture_height-1-offset_yy
    EndIf
    
    If image
      If Not StartDrawing(ImageOutput(input_ID)) : ProcedureReturn -4 : EndIf
    ElseIf sprite 
      If Not StartDrawing(SpriteOutput(input_ID)) : ProcedureReturn -19 : EndIf
    Else ; Canvas
      If Not StartDrawing(CanvasOutput(input_ID)) : ProcedureReturn -5 : EndIf
    EndIf
    
    If invisible_color<>-1
      used_image_color=invisible_color
    Else
      used_image_color=Point(0, 0)
    EndIf
    
    max_x_2=0
    max_y_2=0
    min_x_2=texture_width_1
    min_y_2=texture_height_1
    
    Macro get_min_max_sprite_pos_1
      If i<min_x_2  : min_x_2=i  : EndIf
      If ii<min_y_2 : min_y_2=ii : EndIf
      If i>max_x_2  : max_x_2=i  : EndIf
      If ii>max_y_2 : max_y_2=ii : EndIf
    EndMacro
    
    If remove_rotating_artifacts ; Remove rotating mask artifacts
      For i=texture_x To texture_width_1
        For ii=texture_y To texture_height_1
          If ColorDistance_BF(Point(i, ii), used_image_color)=>1
            get_min_max_sprite_pos_1
          EndIf
        Next ii
      Next i
    Else
      If percent>0
        For i=texture_x To texture_width_1
          For ii=texture_y To texture_height_1
            If ColorDistance_BF(Point(i, ii), used_image_color)>percent
              get_min_max_sprite_pos_1
            EndIf
          Next ii
        Next i
      Else
        For i=texture_x To texture_width_1
          For ii=texture_y To texture_height_1 
            If Point(i, ii)<>used_image_color
              get_min_max_sprite_pos_1
            EndIf
          Next ii
        Next i
      EndIf
    EndIf
    
    StopDrawing()
    ProcedureReturn 1
  EndProcedure 
  
  Procedure AlphaChannelSprite_BF(output_ID, texture_ID,
                                  texture_x,
                                  texture_y, 
                                  texture_width=0,
                                  texture_height=0,
                                  texture_clip_x=0,
                                  texture_clip_y=0,
                                  texture_clip_width=0,
                                  texture_clip_height=0)
    
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    
    If IsImage(output_ID) ; check for image
    ElseIf IsSprite(output_ID) ; Check for PB sprite
    ElseIf IsGadget(output_ID) ; Check for canvas
    Else
      ProcedureReturn -11
    EndIf
    
    If texture_width<1
      texture_width=ImageWidth(texture_ID)
    EndIf
    If texture_height<1
      texture_height=ImageHeight(texture_ID)
    EndIf
    
    Protected old_alpha_sprite=alpha_sprite
    alpha_sprite=1
    
    Protected result=BF(1, output_ID, texture_ID,
                        0,
                        0,
                        texture_x,
                        texture_y, 
                        texture_width,
                        texture_height,
                        texture_clip_x,
                        texture_clip_y,
                        texture_clip_width,
                        texture_clip_height)
    
    alpha_sprite=old_alpha_sprite
    
    ProcedureReturn result
  EndProcedure
  
  Procedure AlphaChannelSprite_Pro_BF(output_ID, texture_ID,
                                      texture_x,
                                      texture_y, 
                                      texture_width=0,
                                      texture_height=0,
                                      output_width=0,
                                      output_height=0,
                                      texture_clip_x=0,
                                      texture_clip_y=0,
                                      texture_clip_width=0,
                                      texture_clip_height=0)
    
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    
    If IsImage(output_ID) ; check for image
    ElseIf IsSprite(output_ID) ; Check for PB sprite
    ElseIf IsGadget(output_ID) ; Check for canvas
    Else
      ProcedureReturn -11
    EndIf 
    
    Protected temporary_texture_ID=CopyImage(texture_ID, #PB_Any) 
    
    If Not temporary_texture_ID
      ProcedureReturn -2
    EndIf
    
    If output_width>0 And output_height>0
      If Not ResizeImage(temporary_texture_ID, output_width, output_height, resize_interpolation)
        FreeImage(temporary_texture_ID)
        ProcedureReturn -27
      EndIf
    EndIf
    
    If texture_width<1
      texture_width=ImageWidth(temporary_texture_ID)
    EndIf
    If texture_height<1
      texture_height=ImageHeight(temporary_texture_ID)
    EndIf
    
    Protected old_alpha_sprite=alpha_sprite
    alpha_sprite=1
    
    Protected result=BF(1, output_ID, temporary_texture_ID,
                        0,
                        0,
                        texture_x,
                        texture_y, 
                        texture_width,
                        texture_height,
                        texture_clip_x,
                        texture_clip_y,
                        texture_clip_width,
                        texture_clip_height)
    
    alpha_sprite=old_alpha_sprite
    
    FreeImage(temporary_texture_ID)
    
    ProcedureReturn result
  EndProcedure
  
  Procedure RotateAlphaChannelSprite_BF(output_ID, texture_ID,                                                                         
                                        texture_x,
                                        texture_y,
                                        degree=0,
                                        mode=0,
                                        alpha=0,
                                        texture_width=0,
                                        texture_height=0,
                                        output_width=0,
                                        output_height=0,
                                        texture_clip_x=0,
                                        texture_clip_y=0,
                                        texture_clip_width=0,
                                        texture_clip_height=0,
                                        rotating_offset_x=0,
                                        rotating_offset_y=0)
    
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    
    alpha=255-alpha
    If alpha<0 : alpha=0 : EndIf
    If alpha>255 : alpha=255 : EndIf
    
    If Not screen_output
      If IsImage(output_ID) ; check for image
        Protected image=1
      ElseIf IsSprite(output_ID) ; Check for PB sprite
        Protected sprite=1
      ElseIf IsGadget(output_ID) ; Check for canvas
        Protected canvas=1
      Else
        ProcedureReturn -11
      EndIf
    EndIf
    
    Protected input_width=ImageWidth(texture_ID)
    Protected input_height=ImageHeight(texture_ID)
    
    If output_width<1 : output_width=input_width : EndIf
    If output_height<1 : output_height=input_height : EndIf
    
    If texture_width<1 : texture_width=output_width : EndIf
    If texture_height<1 : texture_height=output_height : EndIf
    
    Protected temporary_texture_ID=CreateImage(#PB_Any, texture_width, texture_height, 32, #PB_Image_Transparent)
    
    If Not temporary_texture_ID
      ProcedureReturn -2
    EndIf
    
    Protected old_screen_output=screen_output
    screen_output=0
    
    Protected result=AlphaChannelSprite_Pro_BF(temporary_texture_ID, texture_ID,
                                               0,
                                               0,
                                               texture_width,
                                               texture_height,
                                               output_width,
                                               output_height,
                                               texture_clip_x,
                                               texture_clip_y,
                                               texture_clip_width,
                                               texture_clip_height)
    
    screen_output=old_screen_output
    
    If result<1
      FreeImage(temporary_texture_ID)
      ProcedureReturn result
    EndIf
    
    Protected old_alpha_sprite=alpha_sprite
    alpha_sprite=1
    
    If mode=1
      Protected temporary_texture_1_ID=RotateSprite_BF(temporary_texture_ID,
                                                       0,
                                                       0,
                                                       degree, 
                                                       0,
                                                       rotating_offset_x,
                                                       rotating_offset_y,
                                                       0)
    ElseIf mode=2
      temporary_texture_1_ID=RotateSprite_BF(temporary_texture_ID,
                                             0,
                                             0,
                                             degree,
                                             2,
                                             rotating_offset_x,
                                             rotating_offset_y,
                                             1)
      FreeImage(temporary_texture_ID)
    EndIf
    
    If mode<2
      temporary_texture_1_ID=temporary_texture_ID
    EndIf
    
    alpha_sprite=old_alpha_sprite
    
    If Not IsImage(temporary_texture_1_ID)
      ProcedureReturn temporary_texture_1_ID
    EndIf
    
    If screen_output
      result=StartDrawing(ScreenOutput())
    Else
      If image
        result=StartDrawing(ImageOutput(output_ID))
      ElseIf sprite
        result=StartDrawing(SpriteOutput(output_ID))
      Else
        result=StartDrawing(CanvasOutput(output_ID))
      EndIf
    EndIf
    
    If Not result
      FreeImage(temporary_texture_1_ID)
      ProcedureReturn 0
    EndIf
    
    DrawAlphaImage(ImageID(temporary_texture_1_ID), texture_x, texture_y, alpha)
    
    StopDrawing()
    
    FreeImage(temporary_texture_1_ID)
    
    ProcedureReturn Bool(result)
    
  EndProcedure
  
  Procedure RotateAlphaChannelSprite_Fast_BF(output_ID, texture_ID,                                                                         
                                             texture_x,
                                             texture_y,
                                             degree=0,
                                             mode=0,
                                             alpha=0,
                                             output_width=0,
                                             output_height=0,
                                             texture_clip_x=0,
                                             texture_clip_y=0,
                                             texture_clip_width=0,
                                             texture_clip_height=0,
                                             rotating_offset_x=0,
                                             rotating_offset_y=0)
    
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    
    alpha=255-alpha
    If alpha<0 : alpha=0 : EndIf
    If alpha>255 : alpha=255 : EndIf
    
    If Not screen_output
      If IsImage(output_ID) ; check for image
        Protected image=1
      ElseIf IsSprite(output_ID) ; Check for PB sprite
        Protected sprite=1
      ElseIf IsGadget(output_ID) ; Check for canvas
        Protected canvas=1
      Else
        ProcedureReturn -11
      EndIf
    EndIf
    
    Protected temporary_texture_ID=CopyImage(texture_ID, #PB_Any)
    
    If Not temporary_texture_ID
      ProcedureReturn -2
    EndIf
    
    If output_width>0 And output_height>0
      If Not ResizeImage(temporary_texture_ID, output_width, output_height, resize_interpolation)
        FreeImage(temporary_texture_ID)
        ProcedureReturn -27
      EndIf
    EndIf
    
    Protected old_alpha_sprite=alpha_sprite
    alpha_sprite=1
    
    If mode=1
      Protected temporary_texture_1_ID=RotateSprite_BF(temporary_texture_ID,
                                                       0,
                                                       0,
                                                       degree,
                                                       0,
                                                       rotating_offset_x,
                                                       rotating_offset_y,
                                                       0)
    ElseIf mode=2
      temporary_texture_1_ID=RotateSprite_BF(temporary_texture_ID,
                                             0,
                                             0,
                                             degree,
                                             2,
                                             rotating_offset_x,
                                             rotating_offset_y,
                                             1)
      FreeImage(temporary_texture_ID)
    EndIf
    
    If mode<2
      temporary_texture_1_ID=temporary_texture_ID
    EndIf
    
    alpha_sprite=old_alpha_sprite
    
    If Not IsImage(temporary_texture_1_ID)
      ProcedureReturn temporary_texture_1_ID
    EndIf
    
    If texture_clip_x<0 : texture_clip_x=0 : EndIf
    If texture_clip_y<0 : texture_clip_y=0 : EndIf
    If texture_clip_width<1 : texture_clip_width=ImageWidth(temporary_texture_1_ID) : EndIf
    If texture_clip_height<1 : texture_clip_height=ImageHeight(temporary_texture_1_ID) : EndIf
    
    Protected grabed_image_ID=GrabImage(temporary_texture_1_ID, #PB_Any, texture_clip_x, texture_clip_y, texture_clip_width, texture_clip_height)
    
    FreeImage(temporary_texture_1_ID)
    
    If Not IsImage(grabed_image_ID)
      ProcedureReturn -2
    EndIf
    
    If screen_output
      Protected result=StartDrawing(ScreenOutput())
    Else
      If image
        result=StartDrawing(ImageOutput(output_ID))
      ElseIf sprite
        result=StartDrawing(SpriteOutput(output_ID))
      Else
        result=StartDrawing(CanvasOutput(output_ID))
      EndIf
    EndIf
    
    If Not result
      FreeImage(grabed_image_ID)
      ProcedureReturn 0
    EndIf
    
    DrawAlphaImage(ImageID(grabed_image_ID), texture_x, texture_y, alpha)
    
    StopDrawing()
    
    FreeImage(grabed_image_ID)
    
    ProcedureReturn Bool(result)
    
  EndProcedure
  
  Procedure SpriteSimple_BF(mode, output_ID, texture_ID,
                            image_get_color_x,
                            image_get_color_y,
                            output_pos_x,
                            output_pos_y,
                            output_width=0,
                            output_height=0,
                            alpha=1,
                            texture_width=0,
                            texture_height=0,
                            texture_clip_x=0,
                            texture_clip_y=0,
                            texture_clip_width=0,
                            texture_clip_height=0)
    
    If Not screen_output
      If IsImage(output_ID) ; check for image
      ElseIf IsSprite(output_ID) ; Check for PB sprite
      ElseIf Not IsGadget(output_ID) ; Check for canvas
        ProcedureReturn -11
      EndIf
    EndIf
    
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    Protected result, temporary_texture_ID, texture_size_changed
    Protected old_invisible_color.q
    
    Protected image_width=ImageWidth(texture_ID)
    Protected image_height=ImageHeight(texture_ID)
    
    If alpha<1 : alpha=1 : EndIf
    
    If alpha>256 : alpha=256 : EndIf
    
    If mode<0 Or mode>2 : mode=0 : EndIf
    
    old_invisible_color.q=invisible_color
    
    If mode
      If image_get_color_x=-1
        invisible_color=image_get_color_y
        image_get_color_x=0
        image_get_color_y=0
      EndIf
    Else
      image_get_color_x=-1
    EndIf
    
    If output_width<1
      output_width=image_width
    EndIf
    If output_height<1
      output_height=image_height
    EndIf
    
    If output_width<>image_width Or output_height<>image_height
      temporary_texture_ID=CopyImage(texture_ID, #PB_Any)
      
      If Not temporary_texture_ID
        ProcedureReturn -6
      EndIf
      
      If Not ResizeImage(temporary_texture_ID, output_width, output_height, resize_interpolation)
        FreeImage(temporary_texture_ID)
        ProcedureReturn -27
      EndIf
      
      texture_ID=temporary_texture_ID
      texture_size_changed=1
    EndIf
    
    If mode=2
      Protected old_alpha_sprite=alpha_sprite
      alpha_sprite=1
      alpha=1
    EndIf
    
    If texture_width<1 : texture_width=output_width : EndIf
    If texture_height<1 : texture_height=output_height : EndIf
    
    result=BF(alpha, output_ID, texture_ID,
              image_get_color_x,
              image_get_color_y,
              output_pos_x,
              output_pos_y,
              texture_width,
              texture_height,
              texture_clip_x,
              texture_clip_y,
              texture_clip_width,
              texture_clip_height)
    
    invisible_color=old_invisible_color
    
    If texture_size_changed
      Free_selected_Texture_BF(texture_ID)
    EndIf
    
    alpha_sprite=old_alpha_sprite
    
    If IsImage(temporary_texture_ID)
      Free_selected_Texture_BF(temporary_texture_ID) ; Not longer needed, you must remove from BF cache and give free the image
    EndIf
    
    ProcedureReturn result
  EndProcedure
  
  Procedure Sprite_CSS_sheet_BF(mode, output_ID, texture_ID,
                                image_get_color_x, 
                                image_get_color_y,
                                output_pos_x,
                                output_pos_y,
                                frames,
                                frame,
                                frame_width,
                                frame_height,
                                frames_in_a_row=0,
                                output_width=0,
                                output_height=0,
                                texture_width=0,
                                texture_height=0,
                                texture_clip_x=0,
                                texture_clip_y=0,
                                texture_clip_width=0,
                                texture_clip_height=0)
    
    If frame_width<1 Or frame_height<1 : ProcedureReturn -8 : EndIf
    
    If Not screen_output
      If IsImage(output_ID) ; check for image
      ElseIf IsSprite(output_ID) ; Check for PB sprite
      ElseIf Not IsGadget(output_ID) ; Check for canvas
        ProcedureReturn -11
      EndIf
    EndIf
    
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    
    Protected i, result, frame_=1
    Protected frame_offset_x, frame_offset_y, temporary_destination_image_ID
    Protected sheet_width=ImageWidth(texture_ID)
    Protected sheet_height=ImageHeight(texture_ID)
    
    frame+1 ; BF internal, the first frame is frame 1 - PB GIF beginns with frame 0, so we get for compatibility the first frame as frame 0
    
    If frames_in_a_row<1 : frames_in_a_row=sheet_width/frame_width : EndIf
    
    If frame<1 : frame=1 : EndIf
    
    Protected alpha=Abs(mode)
    
    If mode<1 : mode=0 : Else : mode=1 : EndIf
    
    Protected line, column    
    
    i=frame%frames_in_a_row
    If frame<=frames_in_a_row
      line=1
      column=frame
    Else
      If i
        line=frame/frames_in_a_row+1
        column=i
      Else
        line=frame/frames_in_a_row
        column=frames_in_a_row
      EndIf
    EndIf
    
    column-1 : line-1
    
    frame_offset_x=column*frame_width : frame_offset_y=line*frame_height
    
    temporary_destination_image_ID=GrabImage(texture_ID, #PB_Any, frame_offset_x, frame_offset_y, frame_width, frame_height) ; Grab frame from sheet
    
    If Not temporary_destination_image_ID
      ProcedureReturn -6
    EndIf
    
    ; - Call function #1 - Resize the frame output
    result=SpriteSimple_BF(mode, output_ID, temporary_destination_image_ID,
                           image_get_color_x,       
                           image_get_color_y,
                           output_pos_x,
                           output_pos_y,
                           output_width,
                           output_height,
                           alpha,
                           texture_width,
                           texture_height,
                           texture_clip_x,
                           texture_clip_y,
                           texture_clip_width,
                           texture_clip_height)
    
    Free_selected_Texture_BF(temporary_destination_image_ID) ; Not longer needed, you must remove from BF cache and give free the image
    ProcedureReturn result
  EndProcedure
  
  Procedure Sprite_CSS_sheet_simple_BF(mode, output_ID, texture_ID,
                                       image_get_color_x,
                                       image_get_color_y,
                                       output_pos_x,
                                       output_pos_y,
                                       frames,
                                       frame,
                                       frame_width,
                                       frame_height,
                                       frames_in_a_row=0,
                                       output_width=0,
                                       output_height=0,
                                       texture_clip_x=0,
                                       texture_clip_y=0,
                                       texture_clip_width=0,
                                       texture_clip_height=0)
    
    Protected result
    
    If frame_width<1 Or frame_height<1 : ProcedureReturn -8 : EndIf
    
    If Not screen_output
      If IsImage(output_ID) ; check for image
        Protected image=1
      ElseIf IsSprite(output_ID) ; Check for PB sprite
        Protected sprite=1
      ElseIf IsGadget(output_ID) ; Check for canvas
        Protected canvas=1
      Else
        ProcedureReturn -11
      EndIf
    EndIf
    
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    
    If texture_clip_width>frame_width : texture_clip_width=frame_width : EndIf
    If texture_clip_height>frame_height : texture_clip_height=frame_height : EndIf
    
    If texture_clip_width<0 : texture_clip_width=0 : EndIf
    If texture_clip_height<0 : texture_clip_height=0 : EndIf
    
    If texture_clip_x>frame_width : texture_clip_x=frame_width : EndIf
    If texture_clip_y>frame_height : texture_clip_y=frame_height : EndIf
    
    If texture_clip_x<0 : texture_clip_x=0 : EndIf
    If texture_clip_y<0 : texture_clip_y=0 : EndIf
    
    If texture_clip_width
      texture_clip_width=frame_width-texture_clip_width
    EndIf
    
    If texture_clip_height
      texture_clip_height=frame_height-texture_clip_height
    EndIf
    
    If Not output_width : output_width=frame_width-texture_clip_x-texture_clip_width : EndIf
    If Not output_height : output_height=frame_height-texture_clip_y-texture_clip_height : EndIf
    
    If output_width<1 : output_width=frame_width : EndIf
    If output_height<1 : output_height=frame_height : EndIf
    
    Protected i, invisible_color_, frame_=1
    Protected frame_offset_x, frame_offset_y, temporary_destination_image_ID
    Protected sheet_width=ImageWidth(texture_ID)
    Protected sheet_height=ImageHeight(texture_ID)
    
    If image_get_color_x<-1 : image_get_color_x=0 : EndIf 
    
    If image_get_color_x=-1 : invisible_color_=image_get_color_y : EndIf
    
    If image_get_color_x>-1 And image_get_color_y<0 : image_get_color_y=0 : EndIf
    
    frame+1 ; BF internal, the first frame is frame 1 - PB GIF beginns with frame 0, so we get for compatibility the first frame as frame 0
    
    If frames_in_a_row<1 : frames_in_a_row=sheet_width/frame_width : EndIf
    If frame<1 : frame=1 : EndIf
    If mode<-256 Or mode>256 Or invisible_color=-2 : mode=0 : EndIf
    
    Protected line, column    
    
    i=frame%frames_in_a_row
    If frame<=frames_in_a_row
      line=1
      column=frame
    Else
      If i
        line=frame/frames_in_a_row+1
        column=i
      Else
        line=frame/frames_in_a_row
        column=frames_in_a_row
      EndIf
    EndIf
    
    column-1 : line-1
    
    frame_offset_x=column*frame_width+texture_clip_x : frame_offset_y=line*frame_height+texture_clip_y
    
    If image_get_color_x>-1 And image_get_color_x<frame_width And image_get_color_y>-1 And image_get_color_y<frame_height
      If StartDrawing(ImageOutput(texture_ID))
        invisible_color_=Point(image_get_color_x, image_get_color_y)
        StopDrawing()
      Else
        ProcedureReturn -4
      EndIf
    EndIf
    
    frame_width=frame_width-texture_clip_x-texture_clip_width
    frame_height=frame_height-texture_clip_y-texture_clip_height
    
    If frame_offset_x>-1 And frame_offset_x<sheet_width And frame_offset_y>-1 And frame_offset_y<sheet_height And frame_width>0 And frame_height>0
      temporary_destination_image_ID=GrabImage(texture_ID, #PB_Any, frame_offset_x, frame_offset_y, frame_width, frame_height) ; Grab frame from sheet
    Else
      ProcedureReturn 1
    EndIf
    
    If Not temporary_destination_image_ID
      ProcedureReturn -6
    EndIf
    
    If screen_output
      result=StartDrawing(ScreenOutput())
    Else
      If image
        result=StartDrawing(ImageOutput(output_ID))
      ElseIf sprite
        result=StartDrawing(SpriteOutput(output_ID))
      Else
        result=StartDrawing(CanvasOutput(output_ID))
      EndIf
    EndIf
    
    If result
      If mode>0 ; Screen and PB sprite output can not handle CustomFilterCallback
        alpha_1=mode
        If invisible_color>-1
          invisible_color_=invisible_color
        EndIf
        DrawingMode(#PB_2DDrawing_CustomFilter)
        If colormask_color<>-1
          filter(@filter_callback_2(), invisible_color_)
        Else
          filter(@filter_callback_1(), invisible_color_)
        EndIf   
        If Not ResizeImage(temporary_destination_image_ID, output_width, output_height, resize_interpolation)
          FreeImage(temporary_destination_image_ID)
          StopDrawing()
          ProcedureReturn -27
        EndIf
        DrawImage(ImageID(temporary_destination_image_ID), output_pos_x, output_pos_y)
      ElseIf mode<0
        alpha_1=-mode
        If percent
          DrawingMode(#PB_2DDrawing_CustomFilter)
          If colormask_color<>-1
            CustomFilterCallback(@filter_callback_4())
          Else
            CustomFilterCallback(@filter_callback_3())
          EndIf
          If Not ResizeImage(temporary_destination_image_ID, output_width, output_height, resize_interpolation)
            FreeImage(temporary_destination_image_ID)
            StopDrawing()
            ProcedureReturn -27
          EndIf
          DrawImage(ImageID(temporary_destination_image_ID), output_pos_x, output_pos_y)
        Else
          DrawingMode(#PB_2DDrawing_AlphaBlend)
          If frame_width<>output_width Or frame_height<>output_height
            If Not ResizeImage(temporary_destination_image_ID, output_width, output_height, resize_interpolation)
              FreeImage(temporary_destination_image_ID)
              StopDrawing()
              ProcedureReturn -27
            EndIf 
          EndIf
          DrawAlphaImage(ImageID(temporary_destination_image_ID), output_pos_x, output_pos_y, 256-alpha_1)
        EndIf
      Else
        If Not ResizeImage(temporary_destination_image_ID, output_width, output_height, resize_interpolation)
          FreeImage(temporary_destination_image_ID)
          StopDrawing()
          ProcedureReturn -27
        EndIf
        DrawImage(ImageID(temporary_destination_image_ID), output_pos_x, output_pos_y)
      EndIf
      StopDrawing()
    Else
      ProcedureReturn -11
    EndIf
    
    FreeImage(temporary_destination_image_ID)
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure Sprite_CSS_sheet_grab_frame_BF(texture_ID,
                                           frames,
                                           frame,
                                           frame_width,
                                           frame_height,
                                           frames_in_a_row=0,
                                           output_width=0,
                                           output_height=0,
                                           texture_clip_x=0,
                                           texture_clip_y=0,
                                           texture_clip_width=0,
                                           texture_clip_height=0)
    
    If frame_width<1 Or frame_height<1 : ProcedureReturn -8 : EndIf
    
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    
    If texture_clip_width>frame_width : texture_clip_width=frame_width : EndIf
    If texture_clip_height>frame_height : texture_clip_height=frame_height : EndIf
    
    If texture_clip_width<0 : texture_clip_width=0 : EndIf
    If texture_clip_height<0 : texture_clip_height=0 : EndIf
    
    If texture_clip_x>frame_width : texture_clip_x=frame_width : EndIf
    If texture_clip_y>frame_height : texture_clip_y=frame_height : EndIf
    
    If texture_clip_x<0 : texture_clip_x=0 : EndIf
    If texture_clip_y<0 : texture_clip_y=0 : EndIf
    
    If texture_clip_width
      texture_clip_width=frame_width-texture_clip_width
    EndIf
    
    If texture_clip_height
      texture_clip_height=frame_height-texture_clip_height
    EndIf
    
    If Not output_width : output_width=frame_width-texture_clip_x-texture_clip_width : EndIf
    If Not output_height : output_height=frame_height-texture_clip_y-texture_clip_height : EndIf
    
    If output_width<1 : output_width=frame_width : EndIf
    If output_height<1 : output_height=frame_height : EndIf
    
    Protected i, invisible_color_, frame_=1
    Protected frame_offset_x, frame_offset_y, temporary_destination_image_ID
    Protected sheet_width=ImageWidth(texture_ID)
    Protected sheet_height=ImageHeight(texture_ID)
    
    frame+1 ; BF internal, the first frame is frame 1 - PB GIF beginns with frame 0, so we get for compatibility the first frame as frame 0
    
    If frame_width<1 : frame_width=1 : EndIf
    If frames_in_a_row<1 : frames_in_a_row=sheet_width/frame_width : EndIf
    If frame<1 : frame=1 : EndIf
    
    Protected line, column    
    
    If frames_in_a_row<1 : frames_in_a_row=1 : EndIf
    i=frame%frames_in_a_row
    If frame<=frames_in_a_row
      line=1
      column=frame
    Else
      If i
        line=frame/frames_in_a_row+1
        column=i
      Else
        line=frame/frames_in_a_row
        column=frames_in_a_row
      EndIf
    EndIf
    
    column-1 : line-1
    
    frame_offset_x=column*frame_width+texture_clip_x : frame_offset_y=line*frame_height+texture_clip_y
    
    frame_width=frame_width-texture_clip_x-texture_clip_width
    frame_height=frame_height-texture_clip_y-texture_clip_height
    
    If frame_offset_x>-1 And frame_offset_x<sheet_width And frame_offset_y>-1 And frame_offset_y<sheet_height And frame_width>0 And frame_height>0
      temporary_destination_image_ID=GrabImage(texture_ID, #PB_Any, frame_offset_x, frame_offset_y, frame_width, frame_height) ; Grab frame from sheet
      If Not temporary_destination_image_ID
        ProcedureReturn -27
      EndIf
    Else
      ProcedureReturn 0
    EndIf
    
    If frame_width<>output_width Or frame_height<>output_height
      If Not ResizeImage(temporary_destination_image_ID, output_width, output_height, resize_interpolation)
        ProcedureReturn -27
      EndIf
    EndIf
    
    ProcedureReturn temporary_destination_image_ID
  EndProcedure
  
  Procedure Sprite_CSS_sheet_grab_frame_and_free_BF(texture_ID, free_ID,
                                                    frames,
                                                    frame,
                                                    frame_width,
                                                    frame_height,
                                                    frames_in_a_row=0,
                                                    output_width=0,
                                                    output_height=0,
                                                    texture_clip_x=0,
                                                    texture_clip_y=0,
                                                    texture_clip_width=0,
                                                    texture_clip_height=0)
    
    Protected result=Sprite_CSS_sheet_grab_frame_BF(texture_ID,
                                                    frames,
                                                    frame,
                                                    frame_width,
                                                    frame_height,
                                                    frames_in_a_row,
                                                    output_width,
                                                    output_height,
                                                    texture_clip_x,
                                                    texture_clip_y,
                                                    texture_clip_width,
                                                    texture_clip_height)
    If IsImage(free_ID) : FreeImage(free_ID) : EndIf
    ProcedureReturn result
  EndProcedure
  
  Procedure SpriteSimple_fast_BF(mode, output_ID, texture_ID,
                                 image_get_color_x,
                                 image_get_color_y,
                                 output_pos_x,
                                 output_pos_y,
                                 output_width=0,
                                 output_height=0,
                                 texture_clip_x=0,
                                 texture_clip_y=0,
                                 texture_clip_width=0,
                                 texture_clip_height=0)
    
    If IsImage(texture_ID)
      ProcedureReturn Sprite_CSS_sheet_simple_BF(mode, output_ID, texture_ID,
                                                 image_get_color_x,
                                                 image_get_color_y,
                                                 output_pos_x,
                                                 output_pos_y,
                                                 1,
                                                 0,
                                                 ImageWidth(texture_ID),
                                                 ImageHeight(texture_ID),
                                                 1,
                                                 output_width,
                                                 output_height,
                                                 texture_clip_x,
                                                 texture_clip_y,
                                                 texture_clip_width,
                                                 texture_clip_height)
    Else
      ProcedureReturn -2
    EndIf 
    
  EndProcedure
  
  Procedure CreateSprite_from_AlphaImage_BF(image_ID,
                                            mask_color=0,
                                            percent_color_distance.f=0, delete_old_image=0)
    If Not IsImage(image_ID) : ProcedureReturn -2 : EndIf
    Protected image_width=ImageWidth(image_ID)-1
    Protected image_height=ImageHeight(image_ID)-1
    Protected image_width_1=image_width+1
    Protected image_height_1=image_height+1
    Protected point_, i, ii, result
    
    Protected texture_ID=CreateImage(#PB_Any, image_width_1 ,image_height_1, 24, mask_color)
    If Not texture_ID
      ProcedureReturn -6
    EndIf
    
    result=AlphaChannelSprite_BF(texture_ID, image_ID,
                                 0,
                                 0, 
                                 image_width_1,
                                 image_height_1)
    
    If StartDrawing(ImageOutput(texture_ID))
      For i=0 To image_height
        For ii=0 To image_width
          point_=Point(ii, i)
          If ColorDistance_BF(point_, mask_color)<=percent_color_distance
            Plot(ii, i, mask_color)
          EndIf
        Next ii
      Next i
      StopDrawing()
    Else
      FreeImage(texture_ID)
      ProcedureReturn -4
    EndIf
    If delete_old_image
      FreeImage(image_ID)
    EndIf
    ProcedureReturn texture_ID
  EndProcedure
  
  Procedure CreateAlphaImage_from_Sprite_BF(image_ID,
                                            percent_color_distance.f=0,
                                            invisible_mask_color=-1,
                                            alpha=0,
                                            delete_old_image=0)
    If Not IsImage(image_ID) : ProcedureReturn -2 : EndIf
    Protected image_width=ImageWidth(image_ID)-1
    Protected image_height=ImageHeight(image_ID)-1
    Protected image_width_1=image_width+1
    Protected image_height_1=image_height+1
    Protected point, i, ii, alpha_image_ID
    Protected old_percent_color_distance.f
    
    Dim image.q(image_width, image_height)
    
    If alpha<1 : alpha=1 : EndIf
    If alpha>255 : alpha=255 : EndIf
    
    If StartDrawing(ImageOutput(image_ID)) ; Put image in array
      
      If invisible_mask_color=-1
        point=Point(0,0)
        invisible_mask_color=0
      Else
        point=invisible_mask_color
      EndIf
      
      For i=0 To image_height
        For ii=0 To image_width
          If ColorDistance_BF(point, Point(ii,i))<=percent_color_distance 
            image(ii, i)=-1
          Else
            image(ii, i)=Point(ii, i)   
          EndIf
        Next ii
      Next i
    Else
      ProcedureReturn -5
    EndIf
    
    StopDrawing()
    
    alpha_image_ID=CreateImage(#PB_Any, image_width_1, image_height_1, 32, #PB_Image_Transparent)
    If Not alpha_image_ID
      ProcedureReturn -6
    EndIf
    
    old_percent_color_distance=percent_1
    percent_1=percent_color_distance
    
    If StartDrawing(ImageOutput(alpha_image_ID)) ; Put array in image
      
      DrawingMode(#PB_2DDrawing_AllChannels)
      
      For i=0 To image_height
        For ii=0 To image_width
          point=image(ii,i)
          If point<>-1
            Plot(ii, i, RGBA(Red(point), Green(point), Blue(point), 256-alpha))
          Else
            Plot(ii, i, invisible_mask_color) ; Hide the mask
          EndIf
        Next ii
      Next i
    Else
      FreeImage(alpha_image_ID)
      ProcedureReturn -5
    EndIf
    
    StopDrawing()
    
    percent_1=old_percent_color_distance
    If delete_old_image
      FreeImage(image_ID)
    EndIf
    ProcedureReturn alpha_image_ID
  EndProcedure
  
  Procedure Callback_CreateAlphaImage_from_Sprite_Fast(x, y, source_color, destination_color)
    If ColorDistance_BF(source_color&$FFFFFF, to_filtering)>percent_create_alpha_image_from_sprite
      ProcedureReturn source_color
    EndIf
  EndProcedure
  
  Procedure CreateAlphaImage_from_Sprite_Fast_BF(image_ID, color_distance)
    percent_create_alpha_image_from_sprite=color_distance
    
    If Not IsImage(image_ID)
      ProcedureReturn -2
    EndIf
    
    If StartDrawing(ImageOutput(Image_ID))
      to_filtering=Point(0, 0)
      StopDrawing()
    Else
      ProcedureReturn -4
    EndIf
    
    Protected resulted_image_ID=CreateImage(#PB_Any, ImageWidth(image_ID), ImageHeight(image_ID), 32, #PB_Image_Transparent)
    If Not resulted_image_ID
      ProcedureReturn -6
    EndIf
    
    If StartDrawing(ImageOutput(resulted_image_ID))
      DrawingMode(#PB_2DDrawing_CustomFilter)
      CustomFilterCallback(@Callback_CreateAlphaImage_from_Sprite_Fast())
      DrawImage(ImageID(image_ID), 0, 0)
      StopDrawing()
      ProcedureReturn resulted_image_ID
    EndIf
    ProcedureReturn -3
  EndProcedure
  
  Procedure CopyContent_Image_Image_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0)
    If Not StartDrawing(ImageOutput(destination_ID)) : ProcedureReturn -9 : EndIf
    DrawingMode(#PB_2DDrawing_AllChannels)
    If new_size_x>0 And new_size_y>0
      DrawImage(ImageID(source_ID), output_x, output_y, new_size_x, new_size_y)
    Else
      DrawImage(ImageID(source_ID), output_x, output_y)
    EndIf
    StopDrawing()
    ProcedureReturn 1
  EndProcedure
  
  CompilerIf #PB_Compiler_OS=#PB_OS_Windows
    Procedure CopyContent_PB_Sprite_PB_Sprite_BF(source_ID, destination_ID)
      If Not StartDrawing(SpriteOutput(source_ID)) : ProcedureReturn -19 : EndIf
      Protected *drawing_buffer_source_adress=DrawingBuffer()
      Protected drawing_buffer_source_pitch=DrawingBufferPitch()
      Protected source_height=OutputHeight()
      Protected drawing_buffer_source_length=drawing_buffer_source_pitch*source_height
      StopDrawing()
      If Not StartDrawing(SpriteOutput(destination_ID)) : ProcedureReturn -19 : EndIf
      Protected *drawing_buffer_destination_adress=DrawingBuffer()
      Protected drawing_buffer_destination_length=drawing_buffer_source_pitch*source_height
      StopDrawing()
      CopyMemory(*drawing_buffer_source_adress, *drawing_buffer_destination_adress, drawing_buffer_source_length)
      ProcedureReturn 1
    EndProcedure
  CompilerElse
    Procedure CopyContent_PB_Sprite_PB_Sprite_BF(source_ID, destination_ID)
      Protected result=CopyRaw_BF(source_ID, destination_ID)
      ProcedureReturn result
    EndProcedure
  CompilerEndIf
  
  Procedure CopyContent_Image_PB_Sprite_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0)  
    If Not StartDrawing(SpriteOutput(destination_ID)) : ProcedureReturn -19 : EndIf
    DrawingMode(#PB_2DDrawing_AllChannels)
    If new_size_x>0 And new_size_y>0
      DrawImage(ImageID(source_ID), output_x, output_y, new_size_x, new_size_y)
    Else
      DrawImage(ImageID(source_ID), output_x, output_y)
    EndIf
    StopDrawing()
    ProcedureReturn 1
  EndProcedure
  
  Procedure CopyContent_PB_Sprite_Image_BF(source_ID, destination_ID)
    Protected result=CopyRaw_BF(source_ID, destination_ID)
    ProcedureReturn result
  EndProcedure
  
  CompilerIf #PB_Compiler_Version>571
    Procedure CopyContent_PB_Sprite_Canvas_BF(source_ID, destination_ID, output_x=0, output_y=0)
      ; PB (PB560) GrabDrawing works not on Win (Mirrored output) and Mac (Memory overwritten, PB crash) 
      If Not StartDrawing(SpriteOutput(source_ID)) : ProcedureReturn -19 : EndIf
      Protected temp_image_ID=GrabDrawingImage(#PB_Any ,0 ,0 , OutputWidth(), OutputHeight())
      If Not temp_image_ID : StopDrawing() : ProcedureReturn -6 : EndIf
      StopDrawing() 
      If Not StartDrawing(CanvasOutput(destination_ID)) : ProcedureReturn -5 : EndIf
      DrawImage(ImageID(temp_image_ID), 0, 0, output_x, output_y)
      StopDrawing()
      FreeImage(temp_image_ID)
    EndProcedure
  CompilerElse
    Procedure CopyContent_PB_Sprite_Canvas_BF(source_ID, destination_ID)
      Protected result=CopyRaw_BF(source_ID, destination_ID)
      ProcedureReturn result
    EndProcedure
  CompilerEndIf
  
  Procedure CopyContent_Canvas_PB_Sprite_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0)
    If Not StartDrawing(SpriteOutput(destination_ID))
      ProcedureReturn -1
    EndIf
    If new_size_x>0 And new_size_y>0
      DrawImage(GetGadgetAttribute(source_ID, #PB_Canvas_Image), output_x, output_y, new_size_x, new_size_y)
    Else
      DrawImage(GetGadgetAttribute(source_ID, #PB_Canvas_Image), output_x, output_y)
    EndIf
    StopDrawing()
    ProcedureReturn 1
  EndProcedure
  
  Procedure CopyContent_Canvas_Canvas_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0)
    ; SetGadgetAttribute(destination_ID, #PB_Canvas_Image, GetGadgetAttribute(source_ID, #PB_Canvas_Image))
    If Not StartDrawing(CanvasOutput(destination_ID)) : ProcedureReturn -9 : EndIf
    If new_size_x>0 And new_size_y>0
      DrawImage(GetGadgetAttribute(source_ID, #PB_Canvas_Image), output_x, output_y, new_size_x, new_size_y)
    Else
      DrawImage(GetGadgetAttribute(source_ID, #PB_Canvas_Image), output_x, output_y)
    EndIf
    StopDrawing()
    ProcedureReturn 1
  EndProcedure
  
  Procedure CopyContent_Image_Canvas_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0)
    ; SetGadgetAttribute(destination_ID, #PB_Canvas_Image, ImageID(source_ID))
    If Not StartDrawing(CanvasOutput(destination_ID)) : ProcedureReturn -9 : EndIf
    If new_size_x>0 And new_size_y>0
      DrawImage(ImageID(source_ID), output_x, output_y, new_size_x, new_size_y)
    Else
      DrawImage(ImageID(source_ID), output_x, output_y)
    EndIf
    StopDrawing()
    ProcedureReturn 1
  EndProcedure
  
  Procedure CopyContent_Image_Canvas_fast_BF(source_ID, destination_ID)
    SetGadgetAttribute(destination_ID, #PB_Canvas_Image, ImageID(source_ID))
    ProcedureReturn 1
  EndProcedure
  
  Procedure CopyContent_Canvas_Image_BF(source_ID, destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0)
    If Not StartDrawing(ImageOutput(destination_ID))
      ProcedureReturn -1
    EndIf
    If new_size_x>0 And new_size_y>0
      DrawImage(GetGadgetAttribute(source_ID, #PB_Canvas_Image), output_x, output_y, new_size_x, new_size_y)
    Else
      DrawImage(GetGadgetAttribute(source_ID, #PB_Canvas_Image), output_x, output_y)
    EndIf
    StopDrawing()
    ProcedureReturn 1
  EndProcedure
  
  Procedure CopyContent_Image_Screen_BF(source_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0)  
    If Not StartDrawing(ScreenOutput()) : ProcedureReturn -20 : EndIf
    DrawingMode(#PB_2DDrawing_AllChannels)
    If new_size_x>0 And new_size_y>0
      DrawImage(ImageID(source_ID), output_x, output_y, new_size_x, new_size_y)
    Else
      DrawImage(ImageID(source_ID), output_x, output_y)
    EndIf
    StopDrawing()
    ProcedureReturn 1
  EndProcedure
  
  Procedure CopyContent_Screen_Image_BF(destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0)
    If Not IsImage(destination_ID) : ProcedureReturn -2 : EndIf
    If Not ScreenWidth() Or Not ScreenHeight() : ProcedureReturn -30 : EndIf
    If Not StartDrawing(ScreenOutput()) : ProcedureReturn -20 : EndIf
    Protected temp_image_ID=GrabDrawingImage(#PB_Any, 0, 0, ScreenWidth(), ScreenHeight())
    If Not temp_image_ID : StopDrawing() : ProcedureReturn -6 : EndIf
    StopDrawing()
    If Not StartDrawing(ImageOutput(destination_ID)) : FreeImage(temp_image_ID) : ProcedureReturn -4 : EndIf
    DrawingMode(#PB_2DDrawing_AllChannels)
    If new_size_x>0 And new_size_y>0
      DrawImage(ImageID(temp_image_ID), output_x, output_y, new_size_x, new_size_y)
    Else
      DrawImage(ImageID(temp_image_ID), output_x, output_y)
    EndIf
    StopDrawing()
    FreeImage(temp_image_ID)
    ProcedureReturn 1
  EndProcedure
  
  Procedure CopyContent_Screen_Canvas_BF(destination_ID, output_x=0, output_y=0, new_size_x=0, new_size_y=0)
    If Not IsGadget(destination_ID) : ProcedureReturn -1 : EndIf
    If Not ScreenWidth() Or Not ScreenHeight() : ProcedureReturn -30 : EndIf
    If Not StartDrawing(ScreenOutput()) : ProcedureReturn -20 : EndIf
    Protected temp_image_ID=GrabDrawingImage(#PB_Any, 0, 0, ScreenWidth(), ScreenHeight())
    If Not temp_image_ID : StopDrawing() : ProcedureReturn -6 : EndIf
    StopDrawing()
    If Not StartDrawing(CanvasOutput(destination_ID)) : FreeImage(temp_image_ID) : ProcedureReturn -4 : EndIf
    DrawingMode(#PB_2DDrawing_AllChannels)
    If new_size_x>0 And new_size_y>0
      DrawImage(ImageID(temp_image_ID), output_x, output_y, new_size_x, new_size_y)
    Else
      DrawImage(ImageID(temp_image_ID), output_x, output_y)
    EndIf
    StopDrawing()
    FreeImage(temp_image_ID)
    ProcedureReturn 1
  EndProcedure
  
  Procedure CopyContent_BF(source_ID, destination_ID, output_x=0, output_y=0)
    
    If IsImage(source_ID) : Protected source_image=1 : EndIf
    If IsGadget(destination_ID) : Protected destination_canvas=1 : EndIf
    
    If source_image And destination_canvas ; Mostly
      CopyContent_Image_Canvas_BF(source_ID, destination_ID, output_x, output_y)
      ProcedureReturn 1
    EndIf
    
    If screen_output
      CopyContent_Image_Screen_BF(source_ID, output_x, output_y)
      ProcedureReturn 1
    EndIf
    
    If IsImage(destination_ID) : Protected destination_image=1 : EndIf
    
    If source_image And destination_image 
      CopyContent_Image_Image_BF(source_ID, destination_ID, output_x, output_y)
      ProcedureReturn 1
    EndIf 
    
    If IsGadget(source_ID) : Protected source_canvas=1 : EndIf
    
    If source_canvas And destination_image 
      CopyContent_Canvas_Image_BF(source_ID, destination_ID, output_x, output_y) 
      ProcedureReturn 1
    EndIf
    
    If source_canvas And destination_canvas
      CopyContent_Canvas_Canvas_BF(source_ID, destination_ID)
      ProcedureReturn 1
    EndIf
    
    If IsSprite(source_ID) : Protected source_sprite=1 : EndIf
    
    If source_sprite And destination_image
      CopyContent_PB_Sprite_Image_BF(source_ID, destination_ID)
    EndIf
    
    If source_sprite And destination_canvas
      CopyContent_PB_Sprite_Canvas_BF(source_ID, destination_ID)
    EndIf
    
    If IsSprite(destination_ID) : Protected destination_sprite=1 : EndIf
    
    If source_canvas And destination_sprite
      CopyContent_Canvas_PB_Sprite_BF(source_ID, destination_ID, output_x, output_y)
      ProcedureReturn 1
    EndIf
    
    If source_image And destination_sprite
      CopyContent_Image_PB_Sprite_BF(source_ID, destination_ID, output_x, output_y)
      ProcedureReturn 1
    EndIf
    
    ProcedureReturn -9
  EndProcedure
  
  Procedure CopyRaw_BF(source_ID, destination_ID)
    Protected i, ii, width, height
    
    If IsImage(destination_ID)
      Protected destination_image=1
    ElseIf IsSprite(destination_ID)
      Protected destination_sprite=1
    ElseIf IsGadget(destination_ID)
      Protected destination_canvas=1
    Else
      ProcedureReturn -11
    EndIf
    
    If IsImage(source_ID)
      If Not StartDrawing(ImageOutput(source_ID)) : ProcedureReturn -4 : EndIf
      Protected source_image=1
    ElseIf IsSprite(source_ID)
      If Not StartDrawing(SpriteOutput(source_ID)) : ProcedureReturn -19 : EndIf
      Protected source_sprite=1
    ElseIf IsGadget(source_ID)
      If Not StartDrawing(CanvasOutput(source_ID)) : ProcedureReturn -5 : EndIf
      Protected source_canvas=1
    Else
      ProcedureReturn -17
    EndIf
    
    DrawingMode(#PB_2DDrawing_AllChannels)
    
    width=OutputWidth()
    height=OutputHeight()
    
    Protected Dim points(width, height)
    
    For i=0 To width-1 : For ii=0 To height-1 : points(i, ii)=Point(i,ii) : Next : Next
    StopDrawing()
    
    If destination_sprite
      If Not StartDrawing(SpriteOutput(destination_ID)) : ProcedureReturn -19 : EndIf
    ElseIf destination_image
      If Not StartDrawing(ImageOutput(destination_ID)) : ProcedureReturn -4 : EndIf
    ElseIf destination_canvas
      If Not StartDrawing(CanvasOutput(destination_ID)) : ProcedureReturn -5 : EndIf
    EndIf
    
    DrawingMode(#PB_2DDrawing_AllChannels)
    
    For i=0 To width-1 : For ii=0 To height-1 : Plot(i, ii, points(i, ii)) : Next : Next
    StopDrawing()
    ProcedureReturn 1
  EndProcedure
  
  Procedure DrawingBuffer_length_image_BF(image_ID)
    Protected result=StartDrawing(ImageOutput(image_ID))
    If Not result : ProcedureReturn -4 : EndIf
    result=DrawingBufferPitch()*OutputHeight()
    StopDrawing()
    ProcedureReturn result
  EndProcedure
  
  Procedure DrawingBuffer_length_PB_Sprite_BF(sprite_ID)
    Protected result=StartDrawing(SpriteOutput(sprite_ID))
    If Not result : ProcedureReturn -19 : EndIf
    result=DrawingBufferPitch()*OutputHeight()
    StopDrawing()
    ProcedureReturn result
  EndProcedure
  
  Procedure DrawingBuffer_length_canvas_BF(canvas_ID)
    Protected result=StartDrawing(CanvasOutput(canvas_ID))
    If Not result : ProcedureReturn -1 : EndIf
    result=DrawingBufferPitch()*OutputHeight()
    StopDrawing()
    ProcedureReturn result
  EndProcedure
  
  Procedure DrawingBuffer_pitch_image_BF(image_ID)
    Protected result=StartDrawing(ImageOutput(image_ID))
    If Not result : ProcedureReturn -4 : EndIf
    result=DrawingBufferPitch()
    StopDrawing()
    ProcedureReturn result
  EndProcedure
  
  Procedure DrawingBuffer_pitch_PB_Sprite_BF(sprite_ID)
    Protected result=StartDrawing(SpriteOutput(sprite_ID))
    If Not result : ProcedureReturn -19 : EndIf
    result=DrawingBufferPitch()
    StopDrawing()
    ProcedureReturn result
  EndProcedure
  
  Procedure DrawingBuffer_pitch_canvas_BF(canvas_ID)
    Protected result=StartDrawing(CanvasOutput(canvas_ID))
    If Not result : ProcedureReturn -1 : EndIf
    result=DrawingBufferPitch()
    StopDrawing()
    ProcedureReturn result
  EndProcedure
  
  ; Procedure CreateImage_from_PB_Sprite_BF(source_ID)
  ; To time PB (PB560) GrabDrawing bug on Win (Mirrored output) and Mac (Memory overwritten, PB crash) 
  ; If Not StartDrawing(SpriteOutput(source_ID)) : ProcedureReturn -19 : EndIf
  ; Protected new_image_ID=GrabDrawingImage(#PB_Any, 0 ,0, OutputWidth(), OutputHeight())
  ; If Not new_image_ID : StopDrawing() : ProcedureReturn -6 : EndIf
  ; StopDrawing() 
  ;  ProcedureReturn new_image_ID
  ; EndProcedure
  Procedure CreateImage_from_PB_Sprite_BF(source_ID)
    sprite_w_h(source_ID)
    Protected new_image_ID=CreateImage(#PB_Any, sprite_w_h\sprite_w, sprite_w_h\sprite_h)
    If Not new_image_ID
      ProcedureReturn -6
    EndIf
    CopyContent_PB_Sprite_Image_BF(source_ID, new_image_ID)
    ProcedureReturn new_image_ID
  EndProcedure
  
  Procedure CreateCanvas_from_PB_Sprite_BF(source_ID, x, y, width, height, flags=0)
    sprite_w_h(source_ID)
    Protected new_canvas_ID=CanvasGadget(#PB_Any, x, y, sprite_w_h\sprite_w, sprite_w_h\sprite_h, flags)
    If Not new_canvas_ID : ProcedureReturn -7 : EndIf
    CopyContent_PB_Sprite_Canvas_BF(source_ID, new_canvas_ID)
    ProcedureReturn new_canvas_ID
  EndProcedure
  
  Procedure CreateCanvas_from_Image_BF(source_ID, x, y, width, height, flags=0)
    Protected new_canvas_ID=CanvasGadget(#PB_Any, x, y, ImageWidth(source_ID), ImageHeight(source_ID), flags)
    If Not new_canvas_ID : ProcedureReturn -7 : EndIf
    CopyContent_Image_Canvas_BF(source_ID, new_canvas_ID)
    ProcedureReturn new_canvas_ID
  EndProcedure
  
  Procedure CreateImage_from_Canvas_BF(source_ID)
    Protected new_image_ID=CreateImage(#PB_Any,
                                       GadgetWidth(source_ID)*DPI_ResizeFactor_BF(),
                                       GadgetHeight(source_ID)*DPI_ResizeFactor_BF())
    If Not new_image_ID
      ProcedureReturn -6
    EndIf
    CopyContent_Canvas_Image_BF(source_ID, new_image_ID)
    ProcedureReturn new_image_ID
  EndProcedure
  
  Procedure Create_PB_Sprite_from_Canvas_BF(source_ID)
    Protected new_PB_sprite_ID=CreateSprite(#PB_Any,
                                            GadgetWidth(source_ID)*DPI_ResizeFactor_BF(),
                                            GadgetHeight(source_ID)*DPI_ResizeFactor_BF())
    If Not new_PB_Sprite_ID : ProcedureReturn -16 : EndIf
    CopyContent_Canvas_PB_Sprite_BF(source_ID, new_PB_sprite_ID)
    ProcedureReturn new_PB_sprite_ID
  EndProcedure
  
  Procedure CreateImage_from_Image_BF(source_ID, width=0, height=0)
    Protected new_image_ID
    If Not IsImage(source_ID) : ProcedureReturn -2 : EndIf
    If width<1 Or height<1
      width=ImageWidth(source_ID)
      height=ImageHeight(source_ID)
    EndIf
    new_image_ID=CreateImage(#PB_Any, width, height)
    If Not new_image_ID
      ProcedureReturn -6
    EndIf
    If StartDrawing(ImageOutput(new_image_ID))
      DrawImage(ImageID(source_ID), 0, 0, width, height)
      StopDrawing()
    Else
      ProcedureReturn -4
    EndIf
    ProcedureReturn new_image_ID
  EndProcedure
  
  Procedure Create_PB_Sprite_from_Source_BF(source_ID, free_source=0, mode=0)
    Protected sprite_ID, image
    If IsImage(source_ID)
      image=1
      sprite_ID=CreateSprite(#PB_Any, ImageWidth(source_ID), ImageHeight(source_ID), mode)
    ElseIf IsGadget(source_ID)
      sprite_ID=CreateSprite(#PB_Any, GadgetWidth(source_ID)*DPI_ResizeFactor_BF(), GadgetHeight(source_ID)*DPI_ResizeFactor_BF(), mode)
    EndIf
    If Not sprite_ID : ProcedureReturn -16 : EndIf
    If image
      If CopyContent_Image_PB_Sprite_BF(source_ID, sprite_ID)<0
        FreeSprite(sprite_ID)
        ProcedureReturn -9
      EndIf
    Else
      If CopyContent_Canvas_PB_Sprite_BF(source_ID, sprite_ID)<0
        FreeSprite(sprite_ID)
        ProcedureReturn -9
      EndIf
    EndIf
    If free_source
      If image : FreeImage(source_ID) : Else : FreeGadget(source_ID) : EndIf
    EndIf
    ProcedureReturn sprite_ID
  EndProcedure
  
  Procedure TextureBrush_image_BF(mode, output_ID, texture_ID,
                                  x,
                                  y,
                                  texture_width=0,
                                  texture_height=0,
                                  percent_visibility.f=100)
    If Not IsImage(output_ID) And Not IsSprite(output_ID) : ProcedureReturn -11 : EndIf
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    Protected i, ii, result, max_repeats, temporary_texture_ID
    Protected repeats.f, divisor.f=250
    
    If Not texture_width Or Not texture_height
      texture_width=ImageWidth(texture_ID)
      texture_height=ImageHeight(texture_ID)
    EndIf
    
    If texture_width<2 : texture_width=2 : EndIf
    If texture_height<2 : texture_height=2 : EndIf
    
    If mode>3 : mode=0 : EndIf
    
    If mode<-1 : mode=-1 : EndIf
    
    If percent_visibility<1 : percent_visibility=1 : EndIf
    
    If percent_visibility>100 : percent_visibility=100 : EndIf
    
    If texture_height>texture_width
      max_repeats=texture_width>>1
    Else
      max_repeats=texture_height>>1
    EndIf 
    
    If max_repeats>100 : max_repeats=100 : EndIf
    
    temporary_texture_ID=CopyImage(texture_ID, #PB_Any)
    
    If Not temporary_texture_ID
      ProcedureReturn -6
    EndIf
    
    If ImageWidth(temporary_texture_ID)<>texture_width Or
       ImageHeight(temporary_texture_ID)<>texture_height
      If Not ResizeImage(temporary_texture_ID, texture_width, texture_height, resize_interpolation)
        FreeImage(temporary_texture_ID)
        ProcedureReturn -27
      EndIf
    EndIf
    
    percent_visibility-1
    
    If mode<1
      
      divisor=250-divisor/(65)*percent_visibility
      If divisor<1 : divisor=1 : EndIf
      
      result=BF(divisor, output_ID, temporary_texture_ID,
                -1,
                -1,
                x,
                y,
                texture_width,
                texture_height)
      
    Else
      
      If mode=1
        repeats=max_repeats/(230*mode)*percent_visibility
      ElseIf mode=2
        repeats=max_repeats/(140*mode)*percent_visibility
      ElseIf mode=3
        repeats=max_repeats/(120*mode)*percent_visibility
      EndIf
      
      If repeats>max_repeats : repeats=max_repeats : EndIf
      
      If repeats<1 : repeats=1 : EndIf
      
      For i=0 To repeats
        
        result=BF(250-ii, output_ID, temporary_texture_ID,
                  -1,
                  -1,
                  x+ii,
                  y+ii,
                  texture_width-ii-ii,
                  texture_height-ii-ii,
                  ii,
                  ii,
                  texture_width-ii,
                  texture_height-ii) 
        
        ii+mode
        
      Next i
      
    EndIf
    
    Free_selected_Texture_BF(temporary_texture_ID)
    ProcedureReturn result
  EndProcedure
  
  Procedure TextureBrush_canvas_BF(mode, canvas_ID, texture_ID,
                                   x,
                                   y,
                                   texture_width=0,
                                   texture_height=0,
                                   percent_visibility.f=100,
                                   delay=0)
    If Not IsGadget(canvas_ID) : ProcedureReturn -1 : EndIf
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    Protected i, ii, result, repeats_1, max_repeats, temporary_texture_ID
    Protected repeats.f, divisor.f=250
    
    If Not texture_width Or Not texture_height
      texture_width=ImageWidth(texture_ID)
      texture_height=ImageHeight(texture_ID)
    EndIf
    
    If mode>0 And mode<4
      If texture_width<7 : texture_width=3 : mode=1 : EndIf
      If texture_height<7 : texture_height=3 : mode=1 : EndIf
    ElseIf mode=4
      If texture_width<7 : texture_width=7 : mode=1 : EndIf
      If texture_height<7 : texture_height=7 : mode=1 : EndIf
    Else
      If texture_width<2 : texture_width=2 : EndIf
      If texture_height<2 : texture_height=2 : EndIf
    EndIf
    
    If mode>4 : mode=0 : EndIf
    
    If mode<-1 : mode=-1 : EndIf
    
    If percent_visibility<0.1 : percent_visibility=0.1 : EndIf
    
    If percent_visibility>100 : percent_visibility=100 : EndIf
    
    If texture_height>texture_width
      max_repeats=texture_width>>1
    Else
      max_repeats=texture_height>>1
    EndIf 
    
    If max_repeats>100 : max_repeats=100 : EndIf
    
    temporary_texture_ID=CopyImage(texture_ID, #PB_Any)
    
    If Not temporary_texture_ID
      ProcedureReturn -6
    EndIf
    
    If ImageWidth(temporary_texture_ID)<>texture_width Or
       ImageHeight(temporary_texture_ID)<>texture_height
      If Not ResizeImage(temporary_texture_ID, texture_width, texture_height, resize_interpolation)
        FreeImage(temporary_texture_ID)
        ProcedureReturn -6
      EndIf
    EndIf
    
    percent_visibility-1
    
    If mode=-1
      
      divisor=250-divisor/(65)*percent_visibility
      If divisor<1 : divisor=1 : EndIf
      
      result=BF(divisor, canvas_ID, temporary_texture_ID,
                -1,
                -1,
                x,
                y,
                texture_width,
                texture_height)
      
    Else
      
      If mode=1
        repeats=max_repeats/(230*mode)*percent_visibility
      ElseIf mode=2
        repeats=max_repeats/(140*mode)*percent_visibility
      ElseIf mode=3
        repeats=max_repeats/(120*mode)*percent_visibility
      ElseIf mode=4
        repeats=max_repeats/(120*mode)*99   
      Else ; not mode
        repeats=percent_visibility/2
      EndIf
      
      If repeats>max_repeats : repeats=max_repeats : EndIf
      
      If repeats<1 : repeats=1 : EndIf
      
      repeats_1=repeats/2
      
      If repeats_1>1 : repeats_1=1 : EndIf
      
      For i=0 To repeats
        
        If Not mode 
          
          result=BF(250-ii, canvas_ID, temporary_texture_ID,
                    -1,
                    -1,
                    x,
                    y,
                    texture_width,
                    texture_height)
          
        Else 
          
          result=BF(250-ii, canvas_ID, temporary_texture_ID,
                    -1,
                    -1,
                    x+ii,
                    y+ii,
                    texture_width-ii-ii,
                    texture_height-ii-ii,
                    ii,
                    ii,
                    texture_width-ii,
                    texture_height-ii)
          
          If mode=4
            If i>repeats_1 Or repeats_1=1
              result=253-(257/100*percent_visibility)
              If result<1 : result=1 : EndIf
              result=BF(result, canvas_ID, temporary_texture_ID,
                        -1,
                        -1,
                        x+ii,
                        y+ii,
                        texture_width-ii-ii,
                        texture_height-ii-ii,
                        ii,
                        ii,
                        texture_width-ii,
                        texture_height-ii)
            EndIf
          EndIf
          
        EndIf
        
        If mode<4 And delay
          While WindowEvent() : Wend
          Delay(delay)
        EndIf 
        
        If Not mode
          ii+1
        Else
          If mode<4 
            ii+mode
          Else
            ii+3
          EndIf
        EndIf
        
      Next i
      
    EndIf
    
    Free_selected_Texture_BF(temporary_texture_ID)
    ProcedureReturn result
  EndProcedure
  
  Procedure RotateImage_BF(degree, image_ID,
                           create_new_image=0)
    
    If Not IsImage(image_ID)
      ProcedureReturn -6
    EndIf
    
    Protected Dim points(ImageWidth(image_ID), ImageHeight(image_ID))
    Protected image_width=ImageWidth(image_ID)
    Protected image_height=ImageHeight(image_ID)
    Protected i, ii, iii, new_image_ID
    
    If Not StartDrawing(ImageOutput(image_ID))
      ProcedureReturn -4     
    EndIf
    
    DrawingMode(#PB_2DDrawing_AllChannels)
    
    For i=0 To ImageWidth(image_ID)-1
      For ii=0 To ImageHeight(image_ID)-1
        points(i, ii)=Point(i,ii)
      Next
    Next
    
    StopDrawing()
    
    If create_new_image
      
      If alpha_sprite
        If Abs(degree)=90 Or degree=270
          new_image_ID=CreateImage(#PB_Any, image_height, image_width, 32, #PB_Image_Transparent)
        Else
          new_image_ID=CreateImage(#PB_Any, image_width, image_height, 32, #PB_Image_Transparent)
        EndIf
      Else
        If Abs(degree)=90 Or degree=270
          new_image_ID=CreateImage(#PB_Any, image_height, image_width)
        Else
          new_image_ID=CreateImage(#PB_Any, image_width, image_height)
        EndIf
      EndIf
      
      If Not new_image_ID
        ProcedureReturn -6
      EndIf
      If Not StartDrawing(ImageOutput(new_image_ID))
        ProcedureReturn -4     
      EndIf 
    Else
      If Abs(degree)=90 Or degree=270
        If Not ResizeImage(image_ID, image_height, image_width)
          ProcedureReturn -27
        EndIf
      EndIf
      If Not StartDrawing(ImageOutput(image_ID))
        ProcedureReturn -4     
      EndIf
    EndIf
    
    DrawingMode(#PB_2DDrawing_AllChannels)
    
    Select degree
      Case -90, 270
        iii=image_width-1 ; Rotate 90 degrees left
        For i=0 To image_width-1
          For ii=0 To image_height-1
            Plot(ii, iii, points(i, ii)) 
          Next
          iii-1
        Next
        
      Case 90
        iii=image_height-1 ; Rotate 90 degrees right
        For i=0 To image_width-1
          For ii=0 To image_height-1
            Plot(iii, i, points(i, ii))
            iii-1
          Next
          iii=image_height-1
        Next
        
      Case 180
        iii=image_height-1 ; Rotate 180 degrees
        For i=0 To image_width-1
          For ii=0 To image_height-1
            Plot(i, iii, points(i, ii))
            iii-1 
          Next
          iii=image_height-1
        Next
    EndSelect
    
    StopDrawing ()
    
    If create_new_image
      ProcedureReturn new_image_ID
    EndIf
    
    ProcedureReturn 1
  EndProcedure
  
  ; --- Calculate size rotated image and create a empty resized image ---
  Procedure.f Max(f_first.f, f_second.f)
    If f_first>f_second
      ProcedureReturn f_first
    Else
      ProcedureReturn f_second
    EndIf
  EndProcedure
  
  Procedure.f Min(f_first.f, f_second.f)
    If f_first>f_second
      ProcedureReturn f_second
    Else
      ProcedureReturn f_first
    EndIf
  EndProcedure
  
  Macro calculate_rotated_rectangle
    Protected radians.f=(2*#PI*degree)/360
    Protected cos.f=Cos(radians)
    Protected sin.f=Sin(radians)
    
    Protected point_1_x.f=-height*sin
    Protected point_1_y.f=height*cos
    Protected point_2_x.f=width*cos-height*sin
    Protected point_2_y.f=height*cos+width*sin
    Protected point_3_x.f=width*cos
    Protected point_3_y.f=width*sin
    
    Protected min_x.f=min(0, min(point_1_x, min(point_2_x, point_3_x)))
    Protected min_y.f=min(0, min(point_1_y, min(point_2_y, point_3_y)))
    Protected max_x.f=max(0, max(point_1_x, max(point_2_x, point_3_x)))
    Protected max_y.f=max(0, max(point_1_y, max(point_2_y, point_3_y)))
    
    Protected resulted_width.f=Abs(max_x)-min_x
    Protected resulted_height.f=Abs(max_y)-min_y
  EndMacro
  
  Procedure Create_empty_rotated_image_BF(width, height, degree, color=0)
    calculate_rotated_rectangle
    resulted_rotated_width=resulted_width
    resulted_rotated_height=resulted_height
    If alpha_sprite
      Protected resulted_ID=CreateImage(#PB_Any, resulted_width, resulted_height, 32, #PB_Image_Transparent)
    Else
      resulted_ID=CreateImage(#PB_Any, resulted_width, resulted_height, 24, color)
    EndIf
    ProcedureReturn resulted_ID
  EndProcedure
  
  Procedure Calculate_empty_rotated_rectangle_BF(width, height, degree)
    calculate_rotated_rectangle
    resulted_rotated_width=resulted_width
    resulted_rotated_height=resulted_height
    ProcedureReturn 1
  EndProcedure
  
  Procedure Get_resulted_rotated_width()
    ProcedureReturn resulted_rotated_width
  EndProcedure
  
  Procedure Get_resulted_rotated_height()
    ProcedureReturn resulted_rotated_height
  EndProcedure
  
  ; -----------------------------------------------------------------------
  
  Procedure RotateSprite_BF(texture_ID,
                            image_get_color_x=0,                       
                            image_get_color_y=0,                 
                            degree=0,
                            mode=0,
                            rotating_offset_x=0,
                            rotating_offset_y=0,
                            create_new_image=0) 
    
    If Not IsImage(texture_ID)
      ProcedureReturn -2
    EndIf
    
    Protected texture_width=ImageWidth(texture_ID)
    Protected texture_height=ImageHeight(texture_ID)
    
    If texture_width<1 Or texture_height<1
      ProcedureReturn -9
    EndIf
    
    Protected temporary_texture_0_ID, temporary_texture_1_ID
    Protected invisible_color_0, resulted_x_y
    
    If rotating_offset_x<0 : Protected negative_x=1 : EndIf
    If rotating_offset_y<0 : Protected negative_y=1 : EndIf
    
    rotating_offset_x=Abs(rotating_offset_x)
    rotating_offset_y=Abs(rotating_offset_y)
    
    If mode And (rotating_offset_x Or rotating_offset_y)
      resulted_x_y=Sqr(Pow(texture_width+rotating_offset_x*5, 2)+Pow(texture_height+rotating_offset_y*5, 2))+6
    Else
      rotating_offset_x=0 : rotating_offset_y=0
      resulted_x_y=Sqr(Pow(texture_width, 2)+Pow(texture_height, 2))+6
    EndIf
    
    If image_get_color_x>texture_width-1 Or image_get_color_x<-2: image_get_color_x=0 : EndIf
    
    If image_get_color_x<0
      invisible_color_0=image_get_color_y
    Else
      If StartDrawing(ImageOutput(texture_ID))
        If image_get_color_y<0 Or image_get_color_y>texture_height-1: image_get_color_y=0 : EndIf
        invisible_color_0=Point(image_get_color_x, image_get_color_y)
      Else
        ProcedureReturn -4     
      EndIf
      StopDrawing()
    EndIf
    
    temporary_texture_0_ID=CreateImage(#PB_Any, resulted_x_y, resulted_x_y, 32, #PB_Image_Transparent)
    If Not IsImage(temporary_texture_0_ID)
      ProcedureReturn -6
    EndIf
    
    If Not StartVectorDrawing(ImageVectorOutput(temporary_texture_0_ID))
      FreeImage(temporary_texture_0_ID)
      ProcedureReturn -4     
    EndIf
    If alpha_sprite
      VectorSourceColor(invisible_color_0)
    Else
      VectorSourceColor(invisible_color_0|$FF000000)
    EndIf
    FillVectorOutput()
    If degree<0 : degree=360-Abs(degree) : EndIf ; Workaround because of output offset at negative grade
    If negative_x And negative_y
      RotateCoordinates(VectorOutputWidth()/2-rotating_offset_x, VectorOutputHeight()/2-rotating_offset_y, degree)
    ElseIf Not negative_x And Not negative_y
      RotateCoordinates(VectorOutputWidth()/2+rotating_offset_x, VectorOutputHeight()/2+rotating_offset_y, degree)
    ElseIf negative_x And Not negative_y
      RotateCoordinates(VectorOutputWidth()/2-rotating_offset_x, VectorOutputHeight()/2+rotating_offset_y, degree)
    ElseIf Not negative_x And negative_y
      RotateCoordinates(VectorOutputWidth()/2+rotating_offset_x, VectorOutputHeight()/2-rotating_offset_y, degree)
    EndIf
    If activate_mirroring_horizontal_on_rotate
      FlipCoordinatesX(VectorOutputWidth()/2)
    EndIf
    If activate_mirroring_vertical_on_rotate
      FlipCoordinatesY(VectorOutputHeight()/2)
    EndIf
    MovePathCursor(VectorOutputWidth()/2-texture_width/2, VectorOutputHeight()/2-texture_height/2)
    DrawVectorImage(ImageID(texture_ID))
    StopVectorDrawing()
    
    Protected old_invisible_color.q=invisible_color
    invisible_color=invisible_color_0
    remove_rotating_artifacts=1
    GrabSpriteOffsets_BF(temporary_texture_0_ID)
    remove_rotating_artifacts=0
    invisible_color=old_invisible_color
    
    If mode
      If create_new_image
        ProcedureReturn temporary_texture_0_ID
      EndIf
      If Not ResizeImage(texture_ID, resulted_x_y, resulted_x_y, resize_interpolation)
        ProcedureReturn -27
      EndIf
      CopyContent_Image_Image_BF(temporary_texture_0_ID, texture_ID)
      FreeImage(temporary_texture_0_ID)
    Else
      If max_x_2-min_x_2<-4 Or max_y_2-min_y_2<-4 ; Fuse
        Protected rotating_fuse=1
        temporary_texture_1_ID=CreateImage(#PB_Any, 1, 1, 32, #PB_Image_Transparent)
      Else
        Protected min_x=min_x_2-3, min_y=min_y_2-3
        Protected max_x=max_x_2-min_x_2+6, max_y=max_y_2-min_y_2+6
        If min_x<0 : min_x=0 : EndIf : If min_y<0 : min_y=0 : EndIf ; Fuse for GrabImage
        If max_x>resulted_x_y : max_x=resulted_x_y : EndIf : If max_y>resulted_x_y : max_y=resulted_x_y : EndIf
        temporary_texture_1_ID=GrabImage(temporary_texture_0_ID, #PB_Any, min_x, min_y, max_x, max_y)
      EndIf
      
      FreeImage(temporary_texture_0_ID)
      
      If Not temporary_texture_1_ID
        ProcedureReturn -6
      EndIf
      
      If create_new_image
        ProcedureReturn temporary_texture_1_ID
      Else
        If rotating_fuse
          If Not ResizeImage(texture_ID, 1, 1, #PB_Image_Raw)
            ProcedureReturn -27
          EndIf
        Else
          If Not ResizeImage(texture_ID, max_x_2-min_x_2+6, max_y_2-min_y_2+6, #PB_Image_Raw)
            ProcedureReturn -27
          EndIf
        EndIf
        
        StartDrawing(ImageOutput(texture_ID))
        DrawingMode(#PB_2DDrawing_AlphaChannel)
        Box(0, 0, max_x_2-min_x_2+6, max_y_2-min_y_2+6, 0)
        StopDrawing()
        CopyContent_Image_Image_BF(temporary_texture_1_ID, texture_ID)
        FreeImage(temporary_texture_1_ID)
        
      EndIf
    EndIf
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure RotateSprite_simple_BF(mode, output_ID, texture_ID,                              
                                   image_get_color_x=0,                     
                                   image_get_color_y=0,   
                                   output_pos_x=0,
                                   output_pos_y=0,
                                   degree=0,
                                   alpha=1,
                                   output_width=0,
                                   output_height=0,
                                   rotating_offset_x=0,
                                   rotating_offset_y=0)
    
    If Not screen_output
      If Not IsImage(output_ID) ; Check for image
        If IsSprite(output_ID)  ; Check for sprite
          Protected sprite=1
        ElseIf Not IsGadget(output_ID) ; Check for canvas
          ProcedureReturn -11
        EndIf
      EndIf
    EndIf
    
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    
    Protected texture_1_ID, old_invisible_color, result, mode_=1
    
    If mode<0 Or mode>3 : mode=0 : EndIf
    
    If mode<>1 : mode_=0 : EndIf
    
    If Not alpha : alpha=1 : EndIf
    
    If alpha>256 : alpha=256 : EndIf
    
    If alpha<-256 : alpha=-256 : EndIf
    
    If image_get_color_x=-2
      old_invisible_color=invisible_color
      invisible_color=-2
    EndIf
    
    ; - Rotate sprite #1 -
    texture_1_ID=CopyImage(texture_ID, #PB_Any)
    
    If Not texture_1_ID
      ProcedureReturn -6
    EndIf
    
    If output_width>0 And output_height>0
      If output_width<3 : output_width=3 : EndIf
      If output_height<3 : output_height=3 : EndIf
      If Not ResizeImage(texture_1_ID, output_width, output_height, resize_interpolation)
        FreeImage(texture_1_ID)
        ProcedureReturn -27
      EndIf
    EndIf
    
    If mode
      result=RotateSprite_BF(texture_1_ID,
                             image_get_color_x,
                             image_get_color_y,
                             degree,
                             mode_,
                             rotating_offset_x,
                             rotating_offset_y)
      If result<1
        If image_get_color_x=-2
          invisible_color=old_invisible_color
        EndIf
        ProcedureReturn result
      EndIf
    EndIf
    
    If (#PB_Compiler_OS=#PB_OS_MacOS And (output_width<11 Or output_height<11) Or alpha>1) Or screen_output Or sprite
      
      ; - Call function #1 -
      If mode=1
        result=BF(alpha, output_ID, texture_1_ID,
                  0,   
                  0,  
                  output_pos_x+min_x_2,
                  output_pos_y+min_y_2,
                  max_x_2-min_x_2+6,
                  max_y_2-min_y_2+6,
                  min_x_2-3, 
                  min_y_2-3)
      ElseIf mode=2
        result=BF(alpha, output_ID, texture_1_ID,
                  0,   
                  0,  
                  output_pos_x+min_x_2,
                  output_pos_y+min_y_2,
                  ImageWidth(texture_1_ID),
                  ImageHeight(texture_1_ID))
      Else ; mode=3
        result=BF(alpha, output_ID, texture_1_ID,
                  0,   
                  0,  
                  output_pos_x,
                  output_pos_y,
                  ImageWidth(texture_1_ID),
                  ImageHeight(texture_1_ID)) 
      EndIf
      
    Else
      
      ; - Call function #1 -
      If mode=1
        result=Sprite_CSS_sheet_simple_BF(alpha, output_ID, texture_1_ID,
                                          0,
                                          0,
                                          output_pos_x+min_x_2,
                                          output_pos_y+min_y_2,
                                          1,
                                          0,
                                          ImageWidth(texture_1_ID),
                                          ImageHeight(texture_1_ID),
                                          1,
                                          0,
                                          0,
                                          min_x_2-3,
                                          min_y_2-3,
                                          max_x_2+6, 
                                          max_y_2+6)
        
        min_x_2+output_pos_x+3 : min_y_2+output_pos_y+3
        max_x_2+output_pos_x+3 : max_y_2+output_pos_y+3
        
      ElseIf mode=2
        result=Sprite_CSS_sheet_simple_BF(alpha, output_ID, texture_1_ID,
                                          0,
                                          0,
                                          output_pos_x+min_x_2,
                                          output_pos_y+min_y_2,
                                          1,
                                          0,
                                          ImageWidth(texture_1_ID),
                                          ImageHeight(texture_1_ID))
        
        min_x_2+output_pos_x+3 : min_y_2+output_pos_y+3
        max_x_2+output_pos_x+3 : max_y_2+output_pos_y+3
        
      Else ; mode=3
        result=Sprite_CSS_sheet_simple_BF(alpha, output_ID, texture_1_ID,
                                          0,
                                          0,
                                          output_pos_x,
                                          output_pos_y,
                                          1,
                                          0,
                                          ImageWidth(texture_1_ID),
                                          ImageHeight(texture_1_ID))
        
        max_x_2=output_pos_x+(max_x_2-min_x_2)+3 : max_y_2=output_pos_y+(max_y_2-min_y_2)+3
        min_x_2=output_pos_x+3 : min_y_2=output_pos_y+3
      EndIf
      
    EndIf
    
    If image_get_color_x=-2
      invisible_color=old_invisible_color
    EndIf
    
    Free_selected_Texture_BF(texture_1_ID)
    
    If result<1 : ProcedureReturn -9 : EndIf
    
    ProcedureReturn 1
    
  EndProcedure
  
  Procedure CreateSprite_simple_BF(type,
                                   input_ID,
                                   image_get_color_x=0,
                                   image_get_color_y=0,
                                   degree=0,
                                   mode=0,
                                   output_width=0,
                                   output_height=0,
                                   rotating_offset_x=0,
                                   rotating_offset_y=0,
                                   mask_color_distance=0,
                                   delete_input=0,
                                   mirror=0,
                                   flags=0)
    
    Protected temporary_texture_0_ID, temporary_texture_1_ID, sprite_ID
    Protected invisible_color_0, result, resulted_x_y, old_mask_color_distance, old_sprite_color_distance, texture_width, texture_height
    
    If IsImage(input_ID) ; check for image
      Protected image=1
    ElseIf IsSprite(input_ID) ; Check for PB sprite
      Protected sprite=1
    ElseIf IsGadget(input_ID) ; Check for canvas
      Protected canvas=1
    Else
      ProcedureReturn -17
    EndIf
    
    If mirror
      If mirror<3
        temporary_texture_1_ID=Mirror_and_Copy_BF(mirror, input_ID, 1)
        If Not temporary_texture_1_ID
          ProcedureReturn -6
        EndIf
      Else ; mirror=3
        temporary_texture_1_ID=Mirror_and_Copy_BF(1, input_ID, 1)
        If Not temporary_texture_1_ID
          ProcedureReturn -6
        EndIf
        If Not Mirror_and_Copy_BF(2, temporary_texture_1_ID)
          ProcedureReturn -6
        EndIf 
      EndIf
    Else  
      If sprite
        sprite_w_h(input_ID)
        temporary_texture_1_ID=CreateImage(#PB_Any, sprite_w_h\sprite_w, sprite_w_h\sprite_h)
        If Not temporary_texture_1_ID
          ProcedureReturn -6
        EndIf
        CopyContent_PB_Sprite_Image_BF(input_ID, temporary_texture_1_ID)
      ElseIf canvas
        temporary_texture_1_ID=CreateImage(#PB_Any,
                                           GadgetWidth(input_ID)*DPI_ResizeFactor_BF(),
                                           GadgetHeight(input_ID)*DPI_ResizeFactor_BF())
        If Not temporary_texture_1_ID 
          ProcedureReturn -6
        EndIf
        CopyContent_Canvas_Image_BF(input_ID, temporary_texture_1_ID) 
      ElseIf image
        temporary_texture_1_ID=CopyImage(input_ID, #PB_Any)
        If Not temporary_texture_1_ID
          ProcedureReturn -6
        EndIf
      Else
        ProcedureReturn -17
      EndIf 
    EndIf
    
    If delete_input
      If image : FreeImage(input_ID) : EndIf
      If sprite : FreeSprite(input_ID) : EndIf
      If canvas : FreeGadget(input_ID) : EndIf
    EndIf
    
    If output_width>0 And output_height>0
      If Not ResizeImage(temporary_texture_1_ID, output_width, output_height, resize_interpolation)
        FreeImage(temporary_texture_1_ID)
        ProcedureReturn -27
      EndIf
    EndIf
    
    texture_width=ImageWidth(temporary_texture_1_ID)
    texture_height=ImageHeight(temporary_texture_1_ID)
    
    If texture_width<1 Or texture_height<1
      FreeImage(temporary_texture_1_ID)
      ProcedureReturn -9
    EndIf
    
    If mode <0 Or mode>2 : mode=1 : EndIf
    
    If mode And mode<>2
      If rotating_offset_x<0 : Protected negative_x=1 : EndIf
      If rotating_offset_y<0 : Protected negative_y=1 : EndIf
      
      rotating_offset_x=Abs(rotating_offset_x)
      rotating_offset_y=Abs(rotating_offset_y)
      
      If (rotating_offset_x Or rotating_offset_y)
        resulted_x_y=Sqr(Pow(texture_width+rotating_offset_x*5, 2)+Pow(texture_height+rotating_offset_y*5, 2))+7
      Else
        rotating_offset_x=0 : rotating_offset_y=0
        resulted_x_y=Sqr(Pow(texture_width, 2)+Pow(texture_height, 2))+7
      EndIf
      
      temporary_texture_0_ID=CreateImage(#PB_Any, resulted_x_y, resulted_x_y)
      If Not IsImage(temporary_texture_0_ID)
        FreeImage(temporary_texture_1_ID)
        ProcedureReturn -6
      EndIf
    ElseIf mode=2
      temporary_texture_0_ID=Create_empty_rotated_image_BF(texture_width, texture_height, degree)
      If Not IsImage(temporary_texture_0_ID)
        FreeImage(temporary_texture_1_ID)
        ProcedureReturn -6
      EndIf
    EndIf
    
    If image_get_color_x>texture_width-1 Or image_get_color_x<-2: image_get_color_x=0 : EndIf
    
    If image_get_color_x<0
      invisible_color_0=image_get_color_y
    Else
      If StartDrawing(ImageOutput(temporary_texture_1_ID))
        If image_get_color_y<0 Or image_get_color_y>texture_height-1: image_get_color_y=0 : EndIf
        invisible_color_0=Point(image_get_color_x, image_get_color_y)
      Else
        If temporary_texture_0_ID : FreeImage(temporary_texture_0_ID) : EndIf
        FreeImage(temporary_texture_1_ID)
        ProcedureReturn -4     
      EndIf
      StopDrawing()
    EndIf
    
    If Not mode
      If mask_color_distance>0 And image_get_color_x<>-2
        old_mask_color_distance=percent_1
        percent_1=mask_color_distance
        result=BF(-1, temporary_texture_1_ID, invisible_color_0)
        percent_1=old_mask_color_distance ; Color distance fill
        If result<1 : FreeImage(temporary_texture_1_ID) : ProcedureReturn result : EndIf
      Else
        used_invisible_color=invisible_color_0
      EndIf
      If Not type ; BF sprite
        ProcedureReturn temporary_texture_1_ID
      Else ; PB sprite
        sprite_ID=Create_PB_Sprite_from_Source_BF(temporary_texture_1_ID, 0, flags)
        FreeImage(temporary_texture_1_ID)
        If Not sprite_ID
          ProcedureReturn -16
        EndIf
        ProcedureReturn sprite_ID
      EndIf
    EndIf
    
    If Not StartVectorDrawing(ImageVectorOutput(temporary_texture_0_ID))
      FreeImage(temporary_texture_0_ID)
      FreeImage(temporary_texture_1_ID)
      ProcedureReturn -4     
    EndIf
    VectorSourceColor(invisible_color_0|$FF000000)
    FillVectorOutput()
    If degree<0 : degree=360-Abs(degree) : EndIf ; Workaround because of output offset at negative grade
    If negative_x And negative_y
      RotateCoordinates(VectorOutputWidth()/2-rotating_offset_x, VectorOutputHeight()/2-rotating_offset_y, degree)
    ElseIf Not negative_x And Not negative_y
      RotateCoordinates(VectorOutputWidth()/2+rotating_offset_x, VectorOutputHeight()/2+rotating_offset_y, degree)
    ElseIf negative_x And Not negative_y
      RotateCoordinates(VectorOutputWidth()/2-rotating_offset_x, VectorOutputHeight()/2+rotating_offset_y, degree)
    ElseIf Not negative_x And negative_y
      RotateCoordinates(VectorOutputWidth()/2+rotating_offset_x, VectorOutputHeight()/2-rotating_offset_y, degree)
    EndIf
    If activate_mirroring_horizontal_on_rotate
      FlipCoordinatesX(VectorOutputWidth()/2)
    EndIf
    If activate_mirroring_vertical_on_rotate
      FlipCoordinatesY(VectorOutputHeight()/2)
    EndIf
    MovePathCursor(VectorOutputWidth()/2-texture_width/2, VectorOutputHeight()/2-texture_height/2)
    DrawVectorImage(ImageID(temporary_texture_1_ID))
    StopVectorDrawing()
    
    If mask_color_distance>0 And image_get_color_x<>-2
      old_mask_color_distance=percent_1
      percent_1=mask_color_distance
      result=BF(-1, temporary_texture_0_ID, invisible_color_0)
      percent_1=old_mask_color_distance ; Color distance fill
      If result<1
        FreeImage(temporary_texture_0_ID)
        FreeImage(temporary_texture_1_ID)
      ProcedureReturn result : EndIf
    Else
      used_invisible_color=invisible_color_0
    EndIf
    If Not type ; BF sprite
      FreeImage(temporary_texture_1_ID)
      ProcedureReturn temporary_texture_0_ID
    Else ; PB sprite
      sprite_ID=Create_PB_Sprite_from_Source_BF(temporary_texture_0_ID, 0, flags)
      FreeImage(temporary_texture_0_ID)
      FreeImage(temporary_texture_1_ID)
      If Not sprite_ID
        ProcedureReturn -16
      EndIf
      ProcedureReturn sprite_ID
    EndIf
  EndProcedure
  
  Procedure Activate_Mirroring_Horizontal_Rotate_BF(activate_mirroring_horizontal_on_rotate_)
    activate_mirroring_horizontal_on_rotate=activate_mirroring_horizontal_on_rotate_
    ProcedureReturn activate_mirroring_horizontal_on_rotate
  EndProcedure 
  
  Procedure Activate_Mirroring_Vertical_Rotate_BF(activate_mirroring_vertical_on_rotate_)
    activate_mirroring_vertical_on_rotate=activate_mirroring_vertical_on_rotate_
    ProcedureReturn activate_mirroring_vertical_on_rotate
  EndProcedure
  
  Procedure Mirroring_Horizontal_BF(input_ID)
    Protected temp_input_1_ID, temp_input_2_ID, width, height, i, ii, div_2
    If IsImage(input_ID)
      If Not StartDrawing(ImageOutput(input_ID)) : ProcedureReturn -4 : EndIf
    ElseIf IsSprite(input_ID)
      If Not StartDrawing(SpriteOutput(input_ID)) : ProcedureReturn -19 : EndIf
    ElseIf IsGadget(input_ID)
      If Not StartDrawing(CanvasOutput(input_ID)) : ProcedureReturn -5 : EndIf
    Else
      ProcedureReturn -17
    EndIf
    width=OutputWidth()
    height=OutputHeight()
    DrawingMode(#PB_2DDrawing_AllChannels)
    ii=width-1
    div_2=width>>1
    For i=0 To div_2
      temp_input_1_ID=GrabDrawingImage(#PB_Any, ii, 0, 1, height)
      If Not temp_input_1_ID
        StopDrawing()
        ProcedureReturn -12
      EndIf
      temp_input_2_ID=GrabDrawingImage(#PB_Any, i, 0, 1, height)
      If Not temp_input_2_ID
        StopDrawing()
        FreeImage(temp_input_1_ID)
        ProcedureReturn -12
      EndIf
      If Not DrawImage(ImageID(temp_input_2_ID), ii, 0)
        StopDrawing()
        FreeImage(temp_input_1_ID)
        FreeImage(temp_input_2_ID)
        ProcedureReturn 0
      EndIf
      If Not DrawImage(ImageID(temp_input_1_ID), i, 0)
        StopDrawing()
        FreeImage(temp_input_1_ID)
        FreeImage(temp_input_2_ID)
        ProcedureReturn 0
      EndIf
      FreeImage(temp_input_1_ID)
      FreeImage(temp_input_2_ID)
      ii-1
    Next i
    StopDrawing()
    ProcedureReturn 1
  EndProcedure
  
  Procedure Mirroring_Vertical_BF(input_ID)
    Protected temp_input_1_ID, temp_input_2_ID, width, height, i, ii, div_2
    If IsImage(input_ID)
      If Not StartDrawing(ImageOutput(input_ID)) : ProcedureReturn -4 : EndIf
    ElseIf IsSprite(input_ID)
      If Not StartDrawing(SpriteOutput(input_ID)) : ProcedureReturn -19 : EndIf
    ElseIf IsGadget(input_ID)
      If Not StartDrawing(CanvasOutput(input_ID)) : ProcedureReturn -5 : EndIf
    Else
      ProcedureReturn -17
    EndIf
    width=OutputWidth()
    height=OutputHeight()
    DrawingMode(#PB_2DDrawing_AllChannels)
    ii=height-1
    div_2=height>>1
    For i=0 To div_2
      temp_input_1_ID=GrabDrawingImage(#PB_Any, 0, ii, width, 1) 
      If Not temp_input_1_ID
        StopDrawing()
        ProcedureReturn -12
      EndIf
      temp_input_2_ID=GrabDrawingImage(#PB_Any, 0, i, width, 1)
      If Not temp_input_2_ID
        StopDrawing()
        FreeImage(temp_input_1_ID)
        ProcedureReturn -12
      EndIf
      If Not DrawImage(ImageID(temp_input_2_ID), 0, ii)
        StopDrawing()
        FreeImage(temp_input_1_ID)
        FreeImage(temp_input_2_ID)
        ProcedureReturn 0
      EndIf
      If Not DrawImage(ImageID(temp_input_1_ID), 0, i)
        StopDrawing()
        FreeImage(temp_input_1_ID)
        FreeImage(temp_input_2_ID)
        ProcedureReturn 0
      EndIf
      FreeImage(temp_input_1_ID)
      FreeImage(temp_input_2_ID)
      ii-1
    Next i 
    StopDrawing()
    ProcedureReturn 1
  EndProcedure
  
  Procedure Mirroring_Horizontal_RAW_BF(input_ID)
    Protected temp_input_1_ID, temp_input_2_ID, width, height, i, ii, div_2
    width=OutputWidth()
    height=OutputHeight()
    DrawingMode(#PB_2DDrawing_AllChannels)
    ii=width-1
    div_2=width>>1
    For i=0 To div_2
      temp_input_1_ID=GrabDrawingImage(#PB_Any, ii, 0, 1, height)
      If Not temp_input_1_ID
        ProcedureReturn -12
      EndIf
      temp_input_2_ID=GrabDrawingImage(#PB_Any, i, 0, 1, height)
      If Not temp_input_2_ID
        FreeImage(temp_input_1_ID)
        ProcedureReturn -12
      EndIf
      If Not DrawImage(ImageID(temp_input_2_ID), ii, 0)
        FreeImage(temp_input_1_ID)
        FreeImage(temp_input_2_ID)
        ProcedureReturn 0
      EndIf
      If Not DrawImage(ImageID(temp_input_1_ID), i, 0)
        FreeImage(temp_input_1_ID)
        FreeImage(temp_input_2_ID)
        ProcedureReturn 0
      EndIf
      FreeImage(temp_input_1_ID)
      FreeImage(temp_input_2_ID)
      ii-1
    Next i
    ProcedureReturn 1
  EndProcedure
  
  Procedure Mirroring_Vertical_RAW_BF(input_ID)
    Protected temp_input_1_ID, temp_input_2_ID, width, height, i, ii, div_2
    width=OutputWidth()
    height=OutputHeight()
    DrawingMode(#PB_2DDrawing_AllChannels)
    ii=height-1
    div_2=height>>1
    For i=0 To div_2
      temp_input_1_ID=GrabDrawingImage(#PB_Any, 0, ii, width, 1) 
      If Not temp_input_1_ID
        ProcedureReturn -12
      EndIf
      temp_input_2_ID=GrabDrawingImage(#PB_Any, 0, i, width, 1)
      If Not temp_input_2_ID
        FreeImage(temp_input_1_ID)
        ProcedureReturn -12
      EndIf
      If Not DrawImage(ImageID(temp_input_2_ID), 0, ii)
        FreeImage(temp_input_1_ID)
        FreeImage(temp_input_2_ID)
        ProcedureReturn 0
      EndIf
      If Not DrawImage(ImageID(temp_input_1_ID), 0, i)
        FreeImage(temp_input_1_ID)
        FreeImage(temp_input_2_ID)
        ProcedureReturn 0
      EndIf
      FreeImage(temp_input_1_ID)
      FreeImage(temp_input_2_ID)
      ii-1
    Next i 
    ProcedureReturn 1
  EndProcedure
  
  Procedure Mirror_and_Copy_BF(mode, input_ID, create_new_output=0, flags=0, canvas_x=0, canvas_y=0)
    Protected i, ii, iii, image, sprite, canvas, width, height, new_ID
    
    If mode<1 Or mode>3 : ProcedureReturn 0 : EndIf 
    
    If IsImage(input_ID)
      If Not StartDrawing(ImageOutput(input_ID)) : ProcedureReturn -4 : EndIf
      image=1
    ElseIf IsSprite(input_ID)
      If Not StartDrawing(SpriteOutput(input_ID)) : ProcedureReturn -19 : EndIf
      sprite=1
    ElseIf IsGadget(input_ID)
      If Not StartDrawing(CanvasOutput(input_ID)) : ProcedureReturn -5 : EndIf
      canvas=1
    Else
      ProcedureReturn -17
    EndIf
    
    DrawingMode(#PB_2DDrawing_AllChannels)
    
    width=OutputWidth()
    height=OutputHeight()
    
    Protected Dim points(width, height)
    For i=0 To width-1 : For ii=0 To height-1 : points(i, ii)=Point(i,ii) : Next : Next
    
    If create_new_output
      StopDrawing()
      If create_new_output=2 ; Sprite
        new_ID=CreateSprite(#PB_Any, width, height, flags)
        If Not new_ID : ProcedureReturn -16 : EndIf
        If Not StartDrawing(SpriteOutput(new_ID)) : FreeSprite(new_ID) : ProcedureReturn -19 : EndIf
      ElseIf create_new_output=1 ; Image
        new_ID=CreateImage(#PB_Any, width, height)
        If Not new_ID
          ProcedureReturn -6
        EndIf
        If Not StartDrawing(ImageOutput(new_ID)) : FreeImage(new_ID) : ProcedureReturn -4 : EndIf
      ElseIf create_new_output=3 ; Canvas
        new_ID=CanvasGadget(#PB_Any, canvas_x, canvas_y, width, height, flags)
        If Not new_ID : ProcedureReturn -7 : EndIf
        If Not StartDrawing(CanvasOutput(new_ID)) : FreeGadget(new_ID) : ProcedureReturn -5 : EndIf
      EndIf
    EndIf
    
    DrawingMode(#PB_2DDrawing_AllChannels)
    
    If mode=1 ; Horizontal mirroring
      iii=width-1 : For i=0 To width-1 : For ii=0 To height-1 : Plot(iii, ii, points(i, ii)) : Next : iii-1 : Next
    ElseIf mode=2 ; Vertical mirroring
      iii=height-1 : For i=0 To width-1 : For ii=0 To height-1 : Plot(i, iii, points(i, ii)) : iii-1 : Next : iii=height-1 : Next
    ElseIf mode=3 And create_new_output ; Copy without mirroring
      For i=0 To width-1 : For ii=0 To height-1 : Plot(i, ii, points(i, ii)) : Next : Next
    EndIf
    
    StopDrawing()
    If create_new_output : ProcedureReturn new_ID : EndIf
    ProcedureReturn 1
  EndProcedure
  
  Procedure Texture_universal_output_BF(mode, output_ID, texture_ID, temporary_color,
                                        texture_x=0,
                                        texture_y=0,
                                        texture_width=0,
                                        texture_height=0,
                                        texture_clip_x=0,
                                        texture_clip_y=0,
                                        texture_clip_xx=0,
                                        texture_clip_yy=0)
    
    If mode>-1 And Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    
    Protected mode_1, temporary_texture_ID
    Protected old_color_bf.q, old_invisible_color.q
    
    old_color_bf=GetColor_BF()
    SetColor_BF(temporary_color)
    
    Protected old_bucket_array_state=GetBucketArray_State_BF()
    ActivateBucketArray_BF(1) ; Activate the BucketFill array
    
    If mode<0 : mode_1=-1 : Else : mode_1=-2 : EndIf
    If Not mode Or mode<-256 Or mode>256 : mode=1 : EndIf
    
    ; - Call function #1 -
    Protected result=BF(mode_1, temp_image_universal_output_ID, texture_ID,
                        -1,
                        -1,
                        texture_x,
                        texture_y,
                        texture_width,
                        texture_height,
                        texture_clip_x,
                        texture_clip_y,
                        texture_clip_xx,
                        texture_clip_yy)
    
    Protected grab_x=GetBucket_X_BF()
    Protected grab_y=GetBucket_Y_BF()
    Protected grab_xx=GetBucket_XX_BF()
    Protected grab_yy=GetBucket_YY_BF()
    
    ActivateBucketArray_BF(old_bucket_array_state)
    
    SetColor_BF(old_color_bf)
    
    If result<1
      ProcedureReturn result
    EndIf
    
    If grab_xx-grab_x<1 Or grab_yy-grab_y<1 : ProcedureReturn 0 : EndIf
    
    temporary_texture_ID=GrabImage(temp_image_universal_output_ID, #PB_Any, grab_x, grab_y, grab_xx-grab_x+1, grab_yy-grab_y+1)
    
    If Not temporary_texture_ID
      ProcedureReturn -6
    EndIf
    
    old_invisible_color.q=GetInvisibleColor_BF()
    SetInvisibleColor_BF(temporary_color-1)
    
    ; The function is now ready for output
    ; The output is a sprite, created in the picture temporary_texture_ID
    ; The invisible color from this sprite is temporary_color-1
    ; You can make with this sprite how ever you want, as sample rotate, catch for other functions, or other things
    ; Call function #2 is only the output function for the sprite
    
    If IsGadget(output_ID) ; Canvas
                           ; - Call function #2 -
      result=BF(Abs(mode), output_ID, temporary_texture_ID,
                0,
                0,
                grab_x,
                grab_y,
                ImageWidth(temporary_texture_ID),
                ImageHeight(temporary_texture_ID))
    Else
      result=SpriteSimple_fast_BF(Abs(mode), output_ID, temporary_texture_ID,                                     
                                  0,      ; Grab transparence color from the frame pos x - Set to -1 for output without invisible color 
                                  0,      ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - As sample $FF00FF  
                                  grab_x, ; Output position x
                                  grab_y) ; Output position y
    EndIf
    
    SetInvisibleColor_BF(old_invisible_color)
    
    Free_selected_Texture_BF(temporary_texture_ID)
    
    ProcedureReturn result   
  EndProcedure
  
  Procedure CreateImageUniversalOutput_BF(output_ID, temporary_color)
    Protected output_width, output_height
    If IsImage(output_ID)
      output_width=ImageWidth(output_ID) : output_height=ImageHeight(output_ID)
    ElseIf IsGadget(output_ID)
      output_width=GadgetWidth(output_ID) : output_height=GadgetHeight(output_ID)*DPI_ResizeFactor_BF()
    ElseIf IsSprite(output_ID)
      output_width=SpriteWidth(output_ID) : output_height=SpriteHeight(output_ID)*DPI_ResizeFactor_BF()
    Else
      ProcedureReturn -11
    EndIf
    temp_image_universal_output_ID=CreateImage(#PB_Any, output_width, output_height, 24, temporary_color) ; Create a temporary image
    If temp_image_universal_output_ID
      ProcedureReturn 1
    Else
      ProcedureReturn -6
    EndIf
  EndProcedure
  
  Procedure SetMaskColorUniversalOutput_BF(mask_color_universal_output_.q)
    mask_color_universal_output=mask_color_universal_output_
  EndProcedure
  
  Procedure GetUniversalOutputMaskColor_BF()
    ProcedureReturn mask_color_universal_output
  EndProcedure
  
  Procedure Create_animation_buffers_BF(output_ID, buffers_number)
    Protected output_width, output_height, result, canvas, image, sprite
    If buffers_number<0 Or buffers_number>#Size_FlipBuffers_BF_Array : ProcedureReturn -8 : EndIf
    If IsImage(temporary_image_0_ID_BF(buffers_number)) : Free_selected_Texture_BF(temporary_image_0_ID_BF(buffers_number)) : EndIf
    If IsImage(temporary_image_1_ID_BF(buffers_number)) : FreeImage(temporary_image_1_ID_BF(buffers_number)) : EndIf
    If IsGadget(output_ID)
      canvas=1
      output_width=GadgetWidth(output_ID)*DPI_ResizeFactor_BF()
      output_height=GadgetHeight(output_ID)*DPI_ResizeFactor_BF()
    ElseIf IsImage(output_ID)
      image=1
      output_width=ImageWidth(output_ID)
      output_height=ImageHeight(output_ID)
    ElseIf IsSprite(output_ID)
      sprite=1
      sprite_w_h(output_ID)
      output_width=sprite_w_h\sprite_w
      output_height=sprite_w_h\sprite_h
    Else
      ProcedureReturn -17
    EndIf
    temporary_image_0_ID_BF(buffers_number)=CreateImage(#PB_Any, output_width, output_height) ; Create a temporary image for the BF output
    temporary_image_1_ID_BF(buffers_number)=CreateImage(#PB_Any, output_width, output_height) ; Create a temporary image for the background
    If canvas
      result=CopyContent_Canvas_Image_BF(output_ID, temporary_image_0_ID_BF(buffers_number)) ; Put the background in the temporary image 0
    ElseIf image
      result=CopyContent_Image_Image_BF(output_ID, temporary_image_0_ID_BF(buffers_number)) ; Put the background in the temporary image 0
    ElseIf sprite
      result=CopyContent_PB_Sprite_Image_BF(output_ID, temporary_image_0_ID_BF(buffers_number)) ; Put the background in the temporary image 0
    EndIf
    result+CopyContent_Image_Image_BF(temporary_image_0_ID_BF(buffers_number), temporary_image_1_ID_BF(buffers_number)) ; Put the background in the temporary image 1
    
    If result>1 : ProcedureReturn 1 : EndIf
    ProcedureReturn -9
  EndProcedure
  
  Procedure FlipBuffers_BF(buffers_number)
    If IsImage(temporary_image_0_ID_BF(buffers_number))
      If Not StartDrawing(ImageOutput(temporary_image_0_ID_BF(buffers_number))) : ProcedureReturn -5 : EndIf
      If IsImage(temporary_image_1_ID_BF(buffers_number))
        DrawImage(ImageID(temporary_image_1_ID_BF(buffers_number)), 0, 0)
      EndIf
      StopDrawing()
    EndIf
  EndProcedure
  
  Procedure Free_animation_buffers_BF(buffers_number)
    If IsImage(temporary_image_0_ID_BF(buffers_number)) : Free_selected_Texture_BF(temporary_image_0_ID_BF(buffers_number)) : EndIf
    If IsImage(temporary_image_1_ID_BF(buffers_number)) : FreeImage(temporary_image_1_ID_BF(buffers_number)) : EndIf
  EndProcedure
  
  Procedure Get_animation_buffer_background_image_ID_BF(buffers_number)
    ProcedureReturn temporary_image_1_ID_BF(buffers_number)
  EndProcedure
  
  Procedure Get_animation_buffer_hidden_image_ID_BF(buffers_number)
    ProcedureReturn temporary_image_0_ID_BF(buffers_number)
  EndProcedure
  
  Procedure NoCaching_BF(caching)
    no_caching=caching
  EndProcedure
  
  Procedure Get_caching_state_BF()
    ProcedureReturn 1-no_caching
  EndProcedure
  
  Procedure SetPosMarker_BF(output_ID, x, y) ; Set a positioning help marker
    Protected image, sprite, result 
    If IsImage(output_ID) ; check for image
      image=1
    ElseIf IsSprite(output_ID) ; Check for PB sprite
      sprite=1
    ElseIf Not IsGadget(output_ID) ; Check for canvas
      ProcedureReturn -11
    EndIf
    If image
      result=StartDrawing(ImageOutput(output_ID))
    ElseIf sprite
      result=StartDrawing(SpriteOutput(output_ID))
    Else
      result=StartDrawing(CanvasOutput(output_ID))
    EndIf
    If Not result : ProcedureReturn -3 : EndIf
    Circle(x, y, 8, 0)
    Circle(x, y, 6, $FFFF00)
    Circle(x, y, 4, -1)
    Circle(x, y, 1, $FF)
    StopDrawing()
  EndProcedure
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure Create_invisible_GIF_color_BF(gif_ID)
      Protected sheet_ID, i, ii, iii, iiii, file, invisible_color
      Protected gif_width, gif_height, sheet_width, sheet_height, frames, frames_x, frames_y
      
      If Not IsImage(gif_ID) : ProcedureReturn -2 : EndIf
      
      gif_width=ImageWidth(gif_ID)
      gif_height=ImageHeight(gif_ID)
      frames=ImageFrameCount_BF(gif_ID)
      If gif_height>gif_width And frames>1
        frames_x=Sqr(frames)+1
      Else
        frames_x=Sqr(frames)
      EndIf
      If Not frames_x : frames_x=1 : EndIf
      frames_y=frames/frames_x+1
      sheet_width=frames_x*gif_width
      sheet_height=frames_y*gif_height
      
      For i=0 To frames-1
        iiii=iii : ii+gif_width
        If ii=>sheet_width
          ii=0 : iii+gif_height
        EndIf   
      Next i
      ii=0 : iii=0
      
      sheet_height=iiii+gif_height
      
      sheet_ID=CreateImage(#PB_Any, sheet_width, sheet_height)
      
      For i=0 To frames-1
        SetImageFrame(gif_ID, i)
        StartDrawing(ImageOutput(sheet_ID))
        DrawImage(ImageID(gif_ID), ii, iii)
        StopDrawing()
        ii+gif_width
        If ii=>sheet_width
          ii=0 : iii+gif_height
        EndIf   
      Next i
      ii=0 : iii=0
      
      invisible_color=SearchUnusedColor_BF(sheet_ID, 0, 0, sheet_width-1, sheet_height-1)
      
      FreeImage(sheet_ID)
      
      If invisible_color<1 : ProcedureReturn 0 : EndIf
      
      ProcedureReturn invisible_color
      
    EndProcedure
  CompilerEndIf
  
  Procedure CleanMask_BF(input_ID, get_mask_color_x_pos, get_mask_color_y_pos, percent_color_distance)
    Protected mask_color, old_percent_1, result
    If IsImage(input_ID)
      mask_color=GetImageColor_BF(input_ID, get_mask_color_x_pos, get_mask_color_y_pos)
    ElseIf IsSprite(input_ID)
      mask_color=Get_PB_SpriteColor_BF(input_ID, get_mask_color_x_pos, get_mask_color_y_pos)
    ElseIf IsGadget(input_ID)
      mask_color=GetCanvasColor_BF(input_ID, get_mask_color_x_pos, get_mask_color_y_pos)
    Else
      ProcedureReturn -17
    EndIf
    old_percent_1=percent_1
    percent_1=percent_color_distance
    result=BF(-1, input_ID, mask_color) 
    percent_1=old_percent_1
    ProcedureReturn result
  EndProcedure
  
  Procedure CleanMask_simple_BF(input_ID, mask_color, percent_color_distance)
    Protected old_percent_1, result
    old_percent_1=percent_1
    percent_1=percent_color_distance
    result=BF(-1, input_ID, mask_color) 
    percent_1=old_percent_1
    ProcedureReturn result
  EndProcedure
  
  Procedure ResizeInterpolation_BF(resize_interpolation_)
    If resize_interpolation_=1
      resize_interpolation=#PB_Image_Smooth
    Else
      resize_interpolation=#PB_Image_Raw 
    EndIf
  EndProcedure
  
  Procedure Get_all_Window_Events_BF(get_all_window_events_)
    get_all_window_events=get_all_window_events_
    ProcedureReturn get_all_window_events
  EndProcedure
  
  Procedure RotateRoundBox_Vector_BF(output_ID,
                                     x.d,
                                     y.d,
                                     width.d,
                                     height.d,
                                     degree,
                                     radius,
                                     frame_thickness.d,
                                     frame_color,
                                     background_color)
    
    If frame_thickness<0 : Protected negative_frame_thickness=1 : frame_thickness*-1*2 : EndIf
    x+frame_thickness/2 : y+frame_thickness/2 : width-frame_thickness : height-frame_thickness : frame_thickness+1 
    
    If from_vector_roundbox_simple
      width+vector_roundbox_simple_padding_x*2*DPI_ResizeFactor_BF()
      height+vector_roundbox_simple_padding_y*2*DPI_ResizeFactor_BF()
    EndIf
    If IsImage(output_ID) ; check for image
      Protected image=1
    ElseIf IsGadget(output_ID) ; Check for canvas
      Protected canvas=1
    ElseIf IsWindow(output_ID) ; Check for window
      Protected window=1  
    Else
      from_FittedText_RotateRoundBox_Vector_BF=0
      ProcedureReturn -11
    EndIf
    
    If image
      Protected result=StartVectorDrawing(ImageVectorOutput(output_ID))
    ElseIf canvas
      result=StartVectorDrawing(CanvasVectorOutput(output_ID))
    Else
      result=StartVectorDrawing(WindowVectorOutput(output_ID)) 
    EndIf
    
    If Not result
      from_FittedText_RotateRoundBox_Vector_BF=0
      ProcedureReturn -11
    EndIf
    
    If from_FittedText_RotateRoundBox_Vector_BF
      ScaleCoordinates(set_scaling_x, set_scaling_y)
    EndIf
    
    If Not Alpha(frame_color)
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), 255))
    ElseIf Alpha(frame_color)=1
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), 0))
    Else
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), Alpha(frame_color)))
    EndIf
    
    If Not negative_frame_thickness And frame_thickness>1
      StrokePath(frame_thickness)   
    EndIf
    
    RotateCoordinates (width/2+x, height/2+y, degree)
    MovePathCursor(x+radius, y)
    
    If Not radius
      AddPathLine(width, 0, #PB_Path_Relative)
      AddPathLine(0, height, #PB_Path_Relative)
      AddPathLine(-width, 0, #PB_Path_Relative)
      AddPathLine(0, -height, #PB_Path_Relative)
    Else
      MovePathCursor(x+radius, y)
      AddPathArc(width-radius, 0, width-radius, radius, radius, #PB_Path_Relative)
      AddPathArc(0, height-radius, -radius, height-radius, radius, #PB_Path_Relative)
      AddPathArc(-width+radius, 0, -width+radius, -radius, radius, #PB_Path_Relative)
      AddPathArc(0, -height+radius, radius, -height+radius, radius, #PB_Path_Relative)
    EndIf
    
    ClosePath()
    
    If Not Alpha(background_color)
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), 255))
    ElseIf Alpha(background_color)=1
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), 0))
    Else
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), Alpha(background_color)))
    EndIf
    If background_color>>8<>-1
      FillPath(#PB_Path_Preserve)
    EndIf
    If Not Alpha(frame_color)
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), 255)) 
    ElseIf Alpha(frame_color)=1
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), 0))
    Else
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), Alpha(frame_color)))
    EndIf
    If Not negative_frame_thickness And frame_thickness>1
      StrokePath(frame_thickness, #PB_Path_Default)
    EndIf
    
    StopVectorDrawing()
    from_FittedText_RotateRoundBox_Vector_BF=0
    If get_all_window_events : While WindowEvent() : Wend : EndIf
    ProcedureReturn 1
  EndProcedure
  
  Macro macro_common_RoundBox_Vector_BF
    If Not Alpha(frame_color)
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), 255))
    ElseIf Alpha(frame_color)=1
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), 0))
    Else
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), Alpha(frame_color)))
    EndIf
    
    StrokePath(frame_thickness)
    
    MovePathCursor(x, y)
    
    If radius>0
      AddPathArc(width-radius, 0, width-radius, radius, radius, #PB_Path_Relative)
      AddPathArc(0, height-radius, -radius, height-radius, radius, #PB_Path_Relative)
      AddPathArc(-width+radius, 0, -width+radius, -radius, radius, #PB_Path_Relative)
      AddPathArc(0, -height+radius, radius, -height+radius, radius, #PB_Path_Relative)
    Else
      AddPathLine(width, 0, #PB_Path_Relative)
      AddPathLine(0, height, #PB_Path_Relative)
      AddPathLine(-width, 0, #PB_Path_Relative)
      AddPathLine(0, -height, #PB_Path_Relative)
    EndIf
    
    ClosePath()
    
    If Not Alpha(background_color)
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), 255))
    ElseIf Alpha(frame_color)=1
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), 0))
    Else
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), Alpha(background_color)))
    EndIf
    
    If background_color>>8<>-1
      FillPath(#PB_Path_Preserve)
    EndIf
    
    If Not Alpha(frame_color)
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), 255))
    ElseIf Alpha(frame_color)=1
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), 0))
    Else
      VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), Alpha(frame_color)))
    EndIf
    
    If Not negative_frame_thickness And frame_thickness>1
      If radius>1
        StrokePath(frame_thickness, #PB_Path_RoundCorner)
      Else
        StrokePath(frame_thickness, #PB_Path_Default)
      EndIf
    EndIf
  EndMacro
  
  Procedure RoundBox_Vector_RAW_BF(output_ID,
                                   x.d,
                                   y.d,
                                   width.d,
                                   height.d,
                                   radius.d,
                                   frame_thickness.d,
                                   frame_color,
                                   background_color)
    
    If radius<0 : radius=0 : EndIf
    If radius>0 : radius+1 : EndIf
    
    If frame_thickness<0 : Protected negative_frame_thickness=1 : frame_thickness*-1*2 : EndIf 
    x+frame_thickness/2+radius : y+frame_thickness/2 : width-frame_thickness : height-frame_thickness : frame_thickness+1
    If width-radius*2<0 : width=radius*2 : EndIf : If height-radius*2<0 : height=radius*2 : EndIf 
    If IsImage(output_ID) ; check for image
      Protected image=1
    ElseIf IsGadget(output_ID) ; Check for canvas
      Protected canvas=1
    ElseIf IsWindow(output_ID) ; Check for window
      Protected window=1  
    Else
      ProcedureReturn -11
    EndIf
    
    macro_common_RoundBox_Vector_BF
    
    If get_all_window_events : While WindowEvent() : Wend : EndIf
    ProcedureReturn 1
  EndProcedure
  
  Procedure RoundBox_Vector_BF(output_ID, 
                               x.d,
                               y.d,
                               width.d,
                               height.d,
                               radius.d,
                               frame_thickness.d,
                               frame_color,
                               background_color)
    
    If radius<0 : radius=0 : EndIf
    If radius>0 : radius+1 : EndIf
    
    If frame_thickness<0 : Protected negative_frame_thickness=1 : frame_thickness*-1*2 : EndIf 
    x+frame_thickness/2+radius : y+frame_thickness/2 : width-frame_thickness : height-frame_thickness : frame_thickness+1
    If width-radius*2<0 : width=radius*2 : EndIf : If height-radius*2<0 : height=radius*2 : EndIf 
    If IsImage(output_ID) ; check for image
      Protected image=1
    ElseIf IsGadget(output_ID) ; Check for canvas
      Protected canvas=1
    ElseIf IsWindow(output_ID) ; Check for window
      Protected window=1  
    Else
      ProcedureReturn -11
    EndIf
    
    If image
      Protected result=StartVectorDrawing(ImageVectorOutput(output_ID))
    ElseIf canvas
      result=StartVectorDrawing(CanvasVectorOutput(output_ID))
    Else
      result=StartVectorDrawing(WindowVectorOutput(output_ID)) 
    EndIf
    
    If Not result : ProcedureReturn -11 : EndIf
    
    macro_common_RoundBox_Vector_BF
    
    StopVectorDrawing()
    If get_all_window_events : While WindowEvent() : Wend : EndIf
    ProcedureReturn 1
  EndProcedure
  
  Procedure Vector_Roundbox_Simple_Padding_X(vector_roundbox_simple_padding_x_=0)
    vector_roundbox_simple_padding_x=vector_roundbox_simple_padding_x_
    ProcedureReturn vector_roundbox_simple_padding_x
  EndProcedure
  
  Procedure Vector_Roundbox_Simple_Padding_Y(vector_roundbox_simple_padding_y_=0)
    vector_roundbox_simple_padding_y=vector_roundbox_simple_padding_y_
    ProcedureReturn vector_roundbox_simple_padding_y
  EndProcedure
  
  Procedure Get_Vector_Roundbox_Simple_Padding_X()
    ProcedureReturn vector_roundbox_simple_padding_x
  EndProcedure
  
  Procedure Get_Vector_Roundbox_Simple_Padding_Y()
    ProcedureReturn vector_roundbox_simple_padding_y
  EndProcedure
  
  Procedure VectorText_Simple_RoundBox_BF(text$,
                                          output_ID,
                                          output_x_pos,
                                          output_y_pos,
                                          font=-1,
                                          output_width=0,
                                          output_height=0,
                                          text_color=#Black,
                                          background_color=#White,
                                          text_alpha=255,
                                          background_alpha=255,
                                          flag=#PB_VectorParagraph_Left,
                                          frame_thickness=0,
                                          frame_color=-1,
                                          frame_alpha=255,
                                          degree=0,
                                          radius=8)
    
    Protected old_set_scaling_y=set_scaling_y
    If degree And set_scaling_x<>set_scaling_y And (frame_thickness Or background_alpha)
      set_scaling_y=set_scaling_x
    EndIf
    
    Protected old_vector_roundbox_simple_padding_x=vector_roundbox_simple_padding_x
    If degree And vector_roundbox_simple_padding_x<0
      vector_roundbox_simple_padding_x=1.0
    EndIf
    
    Protected old_vector_roundbox_simple_padding_y=vector_roundbox_simple_padding_y
    If degree And vector_roundbox_simple_padding_y<0
      vector_roundbox_simple_padding_y=1.0
    EndIf
    
    from_vector_roundbox_simple=1
    output_x_pos+vector_roundbox_simple_padding_x+frame_thickness/2
    output_y_pos+vector_roundbox_simple_padding_y+frame_thickness/2
    
    If frame_color=-1 : frame_color=text_color : EndIf
    
    Protected old_activate_dpi_aware=activate_dpi_aware
    activate_dpi_aware=1
    
    Protected result=VectorText_Simple_BF(text$, ; Firstly get only the rectangle coordinates
                                          -1,
                                          output_x_pos,
                                          output_y_pos,
                                          font,
                                          output_width,
                                          output_height,
                                          0,
                                          0,
                                          0,
                                          0,
                                          0,
                                          0,
                                          0,
                                          0,
                                          0)
    
    If result<1
      activate_dpi_aware=old_activate_dpi_aware
      from_vector_roundbox_simple=0
      set_scaling_y=old_set_scaling_y
      vector_roundbox_simple_padding_x=old_vector_roundbox_simple_padding_x
      vector_roundbox_simple_padding_y=old_vector_roundbox_simple_padding_y
      ProcedureReturn result
    EndIf
    
    result=RotateRoundBox_Vector_BF(output_ID,
                                    Get_Rectangle_x_BF()*DPI_ResizeFactor_BF(),
                                    Get_Rectangle_y_BF()*DPI_ResizeFactor_BF(),
                                    Get_Rectangle_Width_BF()*DPI_ResizeFactor_BF(),
                                    Get_Rectangle_Height_BF()*DPI_ResizeFactor_BF(),
                                    degree,
                                    radius*DPI_ResizeFactor_BF(),
                                    frame_thickness*DPI_ResizeFactor_BF(),
                                    frame_color|frame_alpha<<24,
                                    background_color|background_alpha<<24)
    
    If result<1
      activate_dpi_aware=old_activate_dpi_aware
      from_vector_roundbox_simple=0
      set_scaling_y=old_set_scaling_y
      vector_roundbox_simple_padding_x=old_vector_roundbox_simple_padding_x
      vector_roundbox_simple_padding_y=old_vector_roundbox_simple_padding_y
      ProcedureReturn result
    EndIf
    
    result=VectorText_Simple_BF(text$,
                                output_ID,
                                output_x_pos,
                                output_y_pos,
                                font,
                                output_width,
                                output_height,
                                text_color,
                                0,
                                text_alpha,
                                0,
                                flag,
                                0,
                                frame_color,
                                frame_alpha,
                                degree)
    
    If result<1
      activate_dpi_aware=old_activate_dpi_aware
      from_vector_roundbox_simple=0
      set_scaling_y=old_set_scaling_y
      vector_roundbox_simple_padding_x=old_vector_roundbox_simple_padding_x
      vector_roundbox_simple_padding_y=old_vector_roundbox_simple_padding_y
      ProcedureReturn result
    EndIf
    
    activate_dpi_aware=old_activate_dpi_aware
    from_vector_roundbox_simple=0
    set_scaling_y=old_set_scaling_y
    vector_roundbox_simple_padding_x=old_vector_roundbox_simple_padding_x
    vector_roundbox_simple_padding_y=old_vector_roundbox_simple_padding_y
    If get_all_window_events : While WindowEvent() : Wend : EndIf
    ProcedureReturn 1
  EndProcedure
  
  Procedure VectorText_Simple_BF(text$,
                                 output_ID,
                                 output_x_pos,
                                 output_y_pos,
                                 font=-1,
                                 output_width=0,
                                 output_height=0,
                                 text_color=#Black,
                                 background_color=#White,
                                 text_alpha=255,
                                 background_alpha=255,
                                 flag=#PB_VectorParagraph_Left,
                                 frame_thickness=0,
                                 frame_color=-1,
                                 frame_alpha=255,
                                 degree=0)
    
    If output_width<0 : output_width=0 : EndIf
    
    Protected temp_image_ID, is_temp_ID, result, output_width_=output_width
    Protected mult_0, mult_1, mult_2, mult_3, get_box_width_bf_
    
    If frame_color=-1 : frame_color=text_color : EndIf
    
    If activate_dpi_aware
      output_x_pos*DPI_ResizeFactor_BF()
      output_y_pos*DPI_ResizeFactor_BF()
      output_width*DPI_ResizeFactor_BF()
      output_height*DPI_ResizeFactor_BF()
      frame_thickness*DPI_ResizeFactor_BF()
    EndIf
    
    If frame_thickness>1
      frame_thickness+1
      frame_thickness-(frame_thickness%2)
    EndIf
    
    ; output_x_pos*DesktopResolutionX()
    ; output_y_pos*DesktopResolutionY()
    ; output_width*DesktopResolutionX()
    ; output_height*DesktopResolutionY()
    
    If output_width<1
      output_width=1
    EndIf
    If output_width>3e4
      output_width=3e4
    EndIf
    
    If output_height>3e4
      output_height=3e4
    EndIf
    
    If FindString(text$, #CR$) ; A part from DrawText_BF ;-)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Linux
      text$=#LF$+text$
    CompilerElse
      text$=#LF$+text$+#LF$
    CompilerEndIf
    
    If Not IsFont(font)
      font=SysFont_BF
    EndIf
    
    If output_ID=-1
      temp_image_ID=CreateImage(#PB_Any, 256, 256)
      If temp_image_ID
        is_temp_ID=1
        Swap output_ID, temp_image_ID
      Else
        ProcedureReturn -6
      EndIf
    EndIf
    
    If IsGadget(output_ID)
      result=StartVectorDrawing(CanvasVectorOutput(output_ID))
    ElseIf IsImage(output_ID)
      result=StartVectorDrawing(ImageVectorOutput(output_ID))
    ElseIf IsWindow(output_ID)
      result=StartVectorDrawing(WindowOutput(output_ID)) 
    ElseIf IsSprite(output_ID)
      result=StartVectorDrawing(SpriteOutput(output_ID)) 
    Else
      If is_temp_ID
        Swap output_ID, temp_image_ID
      EndIf
      ProcedureReturn -11
    EndIf
    
    If result
      
      ScaleCoordinates(set_scaling_x, set_scaling_y)
      output_x_pos/set_scaling_x.d
      output_y_pos/set_scaling_y.d
      
      If output_width_>0
        If output_width<1  : output_width=1    : EndIf
      EndIf 
      
      If output_height<1 : output_height=3e4 : EndIf
      
      VectorFont(FontID(font))
      
      If VectorTextWidth_Multiline_BF(text$)=>output_width-VectorTextHeight_Multiline_BF(text$)*2
        mult_0=1 : mult_1=2 : mult_3=1
      Else
        mult_0=2 : mult_1=1 : mult_3=2
      EndIf
      
      If VectorTextWidth(StringField(text$, 1, #LF$))=>output_width+VectorTextHeight(" ")*2 And
         VectorTextWidth_Multiline_BF(text$)=>output_width+VectorTextHeight(" ")*2
        mult_2=1
      Else
        mult_0=1 : mult_2=2
      EndIf
      
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), background_alpha))
      
      If from_vector_roundbox_simple
        get_box_x_bf=output_x_pos-(vector_roundbox_simple_padding_x/set_scaling_x)*DPI_ResizeFactor_BF()
        get_box_y_bf=output_y_pos-(vector_roundbox_simple_padding_y/set_scaling_y)*DPI_ResizeFactor_BF()
      Else
        get_box_x_bf=output_x_pos
        get_box_y_bf=output_y_pos
      EndIf
      
      Protected VectorTextWidth_=VectorTextWidth_Multiline_BF(text$)+VectorTextHeight(" ")/2
      
      If output_width_>0
        get_box_width_bf=output_width+VectorTextHeight(" ")*mult_1
        get_box_height_bf=VectorParagraphHeight(text$, output_width, output_height)+VectorTextHeight(" ")*mult_0
      Else
        get_box_width_bf=VectorTextWidth_+VectorTextHeight(" ")
        get_box_width_bf_=VectorTextWidth_
        get_box_height_bf=VectorParagraphHeight(text$, get_box_width_bf, output_height)+VectorTextHeight(" ")*mult_0
      EndIf 
      
      If degree<>0
        RotateCoordinates (get_box_x_bf+get_box_width_bf/2, get_box_y_bf+get_box_height_bf/2, degree)
      EndIf
      
      If Not is_temp_ID
        AddPathBox(get_box_x_bf, get_box_y_bf, get_box_width_bf, get_box_height_bf)
        FillPath()
        
        VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), text_alpha))
        
        If activate_linear_gradient
          VectorSourceLinearGradient(linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy)
        EndIf
        
        If activate_circular_gradient
          VectorSourceCircularGradient(circular_gradient_x, circular_gradient_y, circular_gradient_radius)
        EndIf
        
        Protected i
        For i=0 To #Size_GradientColors_Array
          If gradient_activate_flag(i)
            VectorSourceGradientColor(gradient_color(i), gradient_position.f(i))
          EndIf
        Next 
        
        ; Fine adjust
        If output_width_>0
          If flag=#PB_VectorParagraph_Center
            output_x_pos-VectorTextHeight(" ")/10
          EndIf
          If from_vector_roundbox_simple
            output_y_pos+VectorTextHeight(" ")/10
          EndIf
          MovePathCursor(output_x_pos+VectorTextHeight(" ")/mult_0/mult_3, output_y_pos)
          DrawVectorParagraph(Text$, output_width, output_height ,flag)
        Else
          If flag<>#PB_VectorParagraph_Center
            output_x_pos+VectorTextHeight(" ")/1.3
          EndIf
          MovePathCursor(output_x_pos+VectorTextHeight(" ")/mult_0/mult_3-(get_box_width_bf-get_box_width_bf_), output_y_pos)
          DrawVectorParagraph(Text$, get_box_width_bf, get_box_height_bf, flag)
        EndIf
      EndIf
      
      If frame_thickness>0
        Protected is_frame=1 
      EndIf
      
      If frame_thickness>VectorTextHeight(" ")/4 : frame_thickness=VectorTextHeight(" ")/4 : EndIf
      
      If is_frame And Not frame_thickness
        frame_thickness=1
      EndIf
      
      If frame_thickness>0
        VectorSourceColor(RGBA(Red(frame_color), Green(frame_color), Blue(frame_color), frame_alpha))
        AddPathBox(get_box_x_bf+frame_thickness/2,
                   get_box_y_bf+frame_thickness/2,
                   get_box_width_bf-frame_thickness,
                   get_box_height_bf-frame_thickness)
        StrokePath(frame_thickness)
      EndIf
      
      StopVectorDrawing()
      
      If is_temp_ID
        Swap output_ID, temp_image_ID
      EndIf
      
      If activate_dpi_aware
        get_box_x_bf*set_scaling_x/DPI_ResizeFactor_BF()
        get_box_y_bf*set_scaling_y/DPI_ResizeFactor_BF()
        get_box_width_bf*set_scaling_x/DPI_ResizeFactor_BF()
        get_box_height_bf*set_scaling_y/DPI_ResizeFactor_BF()
      EndIf
      
      If get_all_window_events : While WindowEvent() : Wend : EndIf
      ProcedureReturn 1
    EndIf
    
    If is_temp_ID
      Swap output_ID, temp_image_ID
    EndIf
    ProcedureReturn -3
  EndProcedure
  
  Procedure VectorText_Simple_RoundBox_small_BF(text$,                         ; Text
                                                output_ID,                     ; Output destination
                                                output_x_pos,                  ; Output x pos
                                                output_y_pos,                  ; Output y pos
                                                font=-1,                       ; -1 = Automatic - System Font
                                                text_color=#Black,             ; Text color
                                                background_color=#White,       ; Background color
                                                background_alpha=255,          ; Background alpha PB specific
                                                frame_thickness=0,             ; Frame thickness
                                                flag=#PB_VectorParagraph_Left, ; Flag
                                                radius=8)                      ; Edge rounding
    
    Protected result=VectorText_Simple_RoundBox_BF(text$,            ; Text
                                                   output_ID,        ; Output destination
                                                   output_x_pos,     ; Output x pos
                                                   output_y_pos,     ; Output y pos
                                                   font,             ; -1 = Font preset system font
                                                   -1,               ; Output_width - Line break is ever automatically, -1 is automatically width
                                                   0,                ; Output height 0 = automatically height - This is mostly recommended
                                                   text_color,       ; Text color
                                                   background_color, ; Background color
                                                   255,              ; Text alpha PB specific
                                                   background_alpha, ; Background alpha PB specific
                                                   flag,             ; Flag Text adjustment
                                                   frame_thickness,  ; Frame thickness
                                                   -1,               ; Frame color=-1, ; -1 = Automatic - Same Text color
                                                   255,              ; Frame alpha PB specific
                                                   0,                ; Rotating angle
                                                   radius)           ; Edge rounding
    
    ProcedureReturn result
  EndProcedure
  
  Procedure VectorText_Simple_small_BF(text$,                         ; Text
                                       output_ID,                     ; Output destination
                                       output_x_pos,                  ; Output x pos
                                       output_y_pos,                  ; Output y pos
                                       font=-1,                       ; -1 = Automatic - System Font
                                       text_color=#Black,             ; Text color
                                       background_color=#White,       ; Background color
                                       background_alpha=255,          ; Background alpha PB specific
                                       frame_thickness=0,             ; Frame thickness
                                       flag=#PB_VectorParagraph_Left) ; Flag
    
    Protected result=VectorText_Simple_BF(text$,            ; Text
                                          output_ID,        ; Output destination
                                          output_x_pos,     ; Output x pos
                                          output_y_pos,     ; Output y pos
                                          font,             ; -1 = Font preset system font
                                          -1,               ; Output_width - Line break is ever automatically, -1 is automatically width
                                          0,                ; Output height 0 = automatically height - This is mostly recommended
                                          text_color,       ; Text color
                                          background_color, ; Background colo
                                          255,              ; Text alpha PB specific
                                          background_alpha, ; Background alpha PB specific
                                          flag,             ; Flag Text adjustment
                                          frame_thickness,  ; Frame thickness
                                          -1,               ; Frame color=-1, ; -1 = Automatic - Same Text color
                                          255,              ; Frame alpha PB specific
                                          0)                ; Rotating angle
    
    ProcedureReturn result
  EndProcedure
  
  Procedure VectorText_BF(text$,                           ; Text
                          output_ID,                       ; Output destination
                          output_x_pos,                    ; Output x pos
                          output_y_pos,                    ; Output y pos
                          font=-1,                         ; -1 = Automatic - System Font
                          text_color=#Black,               ; Text color
                          background_color=#White,         ; Background color
                          background_alpha=255,            ; Background alpha PB specific
                          flag=#PB_VectorParagraph_Left)   ; Flag
    
    Protected result=VectorText_Simple_RoundBox_BF(text$,
                                                   output_ID,        ; Output destination
                                                   output_x_pos,     ; Output x pos
                                                   output_y_pos,     ; Output y pos
                                                   font,             ; -1 = Font preset system font
                                                   0,                ; Output_width - Line break is ever automatically, 0 is automatically width
                                                   0,                ; Output height 0 = automatically height - This is mostly recommended
                                                   text_color,       ; Text color
                                                   background_color, ; Background colo
                                                   255,              ; Text alpha PB specific
                                                   background_alpha, ; Background alpha PB specific
                                                   flag,             ; Flag Text adjustment
                                                   0,                ; Frame thickness
                                                   -1,               ; Frame color=-1, ; -1 = Automatic - Same Text color
                                                   255,              ; Frame alpha PB specific
                                                   0,                ; Rotating angle
                                                   0)                ; Edge rounding 
    
    ProcedureReturn result
  EndProcedure
  
  Procedure VectorText_CreateImage_RoundBox_BF(text$,
                                               font=-1,
                                               output_width=0,
                                               output_height=0,
                                               text_color=#White,
                                               background_color=#Black,
                                               text_alpha=255,
                                               background_alpha=255,
                                               flag=#PB_VectorParagraph_Left,
                                               frame_thickness=0,
                                               frame_color=-1,
                                               frame_alpha=255,
                                               radius=8)
    
    Protected image_ID, result
    
    If output_height<1
      output_height=3e4
    EndIf
    
    If activate_dpi_aware
      output_width*DPI_ResizeFactor_BF()
      output_height*DPI_ResizeFactor_BF()
    EndIf
    
    If output_width>3e4
      output_width=3e4
    EndIf
    If output_height>3e4
      output_height=3e4
    EndIf
    
    result=VectorText_Simple_BF(text$,
                                -1,            ; Output destination
                                0,             ; Output x pos
                                0,             ; Output y pos,
                                font,          ; -1 = Font preset system font
                                output_width,  ; Output_width - Line break is ever automatically
                                output_height, ; Output height 0 = automatically height - This is mostly recommended
                                0,
                                0,
                                0,
                                0,
                                0,
                                frame_thickness,
                                0,
                                0,
                                0)
    
    If result<1 : ProcedureReturn result : EndIf
    
    image_ID=CreateImage(#PB_Any,
                         Get_Rectangle_width_BF()*DPI_ResizeFactor_BF()+(frame_thickness+(vector_roundbox_simple_padding_x*2))*DPI_ResizeFactor_BF(),
                         Get_Rectangle_height_BF()*DPI_ResizeFactor_BF()+(frame_thickness+(vector_roundbox_simple_padding_y*2))*DPI_ResizeFactor_BF(),
                         32, #PB_Image_Transparent)
    
    If Not image_ID : ProcedureReturn -6 : EndIf
    
    result=VectorText_Simple_RoundBox_BF(text$,
                                         image_ID,         ; Output destination
                                         0,                ; Output x pos
                                         0,                ; Output y pos
                                         font,             ; -1 = Font preset system font
                                         output_width,     ; Output_width - Line break is ever automatically
                                         output_height,    ; Output height 0 = automatically height - This is mostly recommended
                                         text_color,       ; Text color
                                         background_color, ; Background color
                                         text_alpha,       ; Text alpha PB specific
                                         background_alpha, ; Background alpha PB specific
                                         flag,             ; Flag Text adjustment
                                         frame_thickness,  ; Frame thickness
                                         frame_color,      ; Frame color=-1, ; -1 = Automatic - Same Text color
                                         frame_alpha,      ; Frame alpha PB specific
                                         0,                ; Rotating degree)
                                         radius)           ; Edge rounding
    
    If result<1
      FreeImage(image_ID)
      ProcedureReturn result
    EndIf
    
    ProcedureReturn image_ID
    
  EndProcedure
  
  Procedure VectorText_CreateImage_BF(text$,
                                      font=-1,
                                      output_width=0,
                                      output_height=0,
                                      text_color=#White,
                                      background_color=#Black,
                                      text_alpha=255,
                                      background_alpha=255,
                                      flag=#PB_VectorParagraph_Left,
                                      frame_thickness=0,
                                      frame_color=-1,
                                      frame_alpha=255)
    
    Protected image_ID, result
    
    If output_height<1
      output_height=3e4
    EndIf
    
    If activate_dpi_aware
      output_width*DPI_ResizeFactor_BF()
      output_height*DPI_ResizeFactor_BF()
    EndIf
    
    If output_width>3e4
      output_width=3e4
    EndIf
    If output_height>3e4
      output_height=3e4
    EndIf
    
    Protected old_activate_dpi_aware=activate_dpi_aware
    activate_dpi_aware=0
    
    result=VectorText_Simple_BF(text$,
                                -1,
                                0, 
                                0,
                                font,
                                output_width,
                                output_height)
    
    If result<1
      activate_dpi_aware=old_activate_dpi_aware
      ProcedureReturn result
    EndIf
    
    image_ID=CreateImage(#PB_Any, Get_Rectangle_width_BF()*set_scaling_x.d, Get_Rectangle_height_BF()*set_scaling_y.d, 32, #PB_Image_Transparent) 
    If Not image_ID
      activate_dpi_aware=old_activate_dpi_aware
      ProcedureReturn -6
    EndIf
    
    result=VectorText_Simple_BF(text$,
                                image_ID,
                                0,
                                0,
                                font,
                                output_width,
                                output_height,
                                text_color,
                                background_color,
                                text_alpha,
                                background_alpha,
                                flag,
                                frame_thickness*DPI_ResizeFactor_BF(),
                                frame_color,
                                frame_alpha,
                                0)
    
    activate_dpi_aware=old_activate_dpi_aware
    
    If result<1
      FreeImage(image_ID)
      ProcedureReturn result
    EndIf
    
    ProcedureReturn image_ID
    
  EndProcedure
  
  Procedure VectorText_Screen_BF(text$,
                                 x,
                                 y,
                                 text_color,
                                 text_alpha,
                                 font=-1,
                                 scale_width.d=1.0,
                                 scale_height.d=1.0,
                                 flag=#PB_VectorParagraph_Left)
    
    If FindString(text$, #CR$)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    If Not IsFont(font)
      font=SysFont_BF
    EndIf
    
    If Not StartVectorDrawing(ImageVectorOutput(mini_temp_image_ID)) : ProcedureReturn -3 : EndIf
    VectorFont(FontID(font), 11)
    Protected temp_1_image_width=VectorTextWidth(text$)*scale_width*DPI_ResizeFactor_BF()+1
    Protected temp_1_image_height=VectorParagraphHeight(text$, 3e4, 3e4)*scale_height*DPI_ResizeFactor_BF()+1
    StopVectorDrawing()
    
    If Not StartDrawing(ScreenOutput()) : ProcedureReturn -3 : EndIf
    Protected temp_1_ID=GrabDrawingImage(#PB_Any, x, y, temp_1_image_width, temp_1_image_height)
    If Not temp_1_ID : StopDrawing() : ProcedureReturn -6 : EndIf
    StopDrawing()
    
    If StartVectorDrawing(ImageVectorOutput(temp_1_ID))
      VectorFont(FontID(font), 11)
      ScaleCoordinates(scale_width*DPI_ResizeFactor_BF(), scale_height*DPI_ResizeFactor_BF())
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), 255))
      
      If activate_linear_gradient
        VectorSourceLinearGradient(linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy)
      EndIf
      
      If activate_circular_gradient
        VectorSourceCircularGradient(circular_gradient_x, circular_gradient_y, circular_gradient_radius)
      EndIf
      
      Protected i
      For i=0 To #Size_GradientColors_Array
        If gradient_activate_flag(i)
          VectorSourceGradientColor(gradient_color(i), gradient_position.f(i)/scale_width.d)
        EndIf
      Next 
      
      DrawVectorParagraph(text$, VectorTextWidth(text$), VectorTextHeight(text$), flag)  
      
      StopVectorDrawing()
      StartDrawing(ScreenOutput())
      DrawAlphaImage(ImageID(temp_1_ID), x, y, text_alpha)
      StopDrawing()
      FreeImage(temp_1_ID)
      ProcedureReturn 1
    Else
      FreeImage(temp_1_ID)
      ProcedureReturn -3
    EndIf
  EndProcedure
  
  Procedure DrawText_Vector_BF(txt_x,
                               txt_y,
                               txt$,
                               f_col=#Black,
                               flag=0,
                               mode=0,
                               stretch_x.d=0,
                               stretch_y.d=0,
                               scale_coordinates_x.d=1,
                               scale_coordinates_y.d=1)
    
    If txt$="" : ProcedureReturn -26 : EndIf
    
    Protected rotate_coordinates_x_.d=rotate_coordinates_x.d
    Protected rotate_coordinates_y_.d=rotate_coordinates_y.d
    
    If activate_dpi_aware
      txt_x*DPI_ResizeFactor_BF()
      txt_y*DPI_ResizeFactor_BF()
      stretch_x*DPI_ResizeFactor_BF()
      stretch_y*DPI_ResizeFactor_BF()
      rotate_coordinates_x_.d*DPI_ResizeFactor_BF()
      rotate_coordinates_y_.d*DPI_ResizeFactor_BF()
    Else
      If IsDPIaware_BF()
        rotate_coordinates_x_.d*DPI_ResizeFactor_BF()
        rotate_coordinates_y_.d*DPI_ResizeFactor_BF()
        If scale_coordinates_x.d>0
          scale_coordinates_x.d/DPI_ResizeFactor_BF()
        EndIf
        If scale_coordinates_y.d>0
          scale_coordinates_y.d/DPI_ResizeFactor_BF()
        EndIf
      EndIf
    EndIf
    
    Protected i1=VectorTextHeight(Space(1))+stretch_y, i2, i3, i4, i5, i6, i7, padding_1
    Protected result, max, max_len, max_2, max_3, txt_x_0=txt_x, txt_y_0=txt_y, amount_lines
    Protected txt1$, chr$, ln$, ln1$
    Protected Dim f1$(0), Dim f2$(0)
    
    If scale_coordinates_x.d>0
      txt_x/scale_coordinates_x.d
    EndIf
    
    If scale_coordinates_y.d>0
      txt_y/scale_coordinates_y.d
    EndIf
    
    ResetCoordinates()
    RotateCoordinates(rotate_coordinates_x_.d, rotate_coordinates_y_.d, rotate_coordinates_degree.d)
    ScaleCoordinates(scale_coordinates_x.d, scale_coordinates_y.d)
    
    Protected i_x
    Macro DrawText_Vector(txt_x, txt_y, txt, f_col)
      If Alpha(f_col)=0 
        VectorSourceColor(RGBA(Red(f_col), Green(f_col), Blue(f_col), 255))
      Else
        VectorSourceColor(RGBA(Red(f_col), Green(f_col), Blue(f_col), Alpha(f_col))) 
      EndIf
      
      If activate_linear_gradient
        VectorSourceLinearGradient(linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy)
      EndIf
      
      If activate_circular_gradient
        VectorSourceCircularGradient(circular_gradient_x, circular_gradient_y, circular_gradient_radius)
      EndIf
      
      For i_x=0 To #Size_GradientColors_Array
        If gradient_activate_flag(i_x)
          VectorSourceGradientColor(gradient_color(i_x), gradient_position.f(i_x))
        EndIf
      Next 
      
      MovePathCursor(txt_x, txt_y)
      DrawVectorText(txt)
      
    EndMacro
    
    If FindString(txt$, #CR$) ; A part from DrawText_BF :-)
      txt$=ReplaceString(txt$, #CRLF$, #LF$)
      txt$=ReplaceString(txt$, #LFCR$, #LF$)
      ReplaceString(txt$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    txt1$=txt$
    
    i7=Len(txt1$)
    
    If flag<>3 And FindString(txt1$, #LF$)
      amount_lines=CountString(txt1$, #LF$)+1
      txt_y-i1
      For result=1 To i7
        ln$=StringField(txt1$, result, #LF$)
        i4=Len(ln$)
        If i4
          max_2=VectorTextWidth(ln$)+stretch_x*i4
          If max_2>max_3
            max_3=max_2
            ln1$=ln$
            i2+i1
            max=VectorTextWidth(ln$)+i4
            max_len=i4
          EndIf
        EndIf
      Next
      
      If max_1_vector>0
        max=max_1_vector
      EndIf
      
      If flag=1 ; Centered
        txt_x+max_len*(stretch_x/2)-max_len/2
      ElseIf flag=2 ; Right
        txt_x+max_len*(stretch_x)-max_len
      EndIf
    EndIf
    
    If mode=3 And flag<>3
      If i2
        If Not stretch_x
          ProcedureReturn VectorTextWidth(ln1$)*scale_Coordinates_x.d
        Else
          i6=Len(ln1$)
          For i4=1 To i6
            i5+VectorTextWidth(Mid(ln1$, i4, 1))
          Next
          ProcedureReturn (i5+stretch_x*i6+stretch_x/12)*scale_Coordinates_x.d
        EndIf
      Else
        If Not stretch_x
          ProcedureReturn VectorTextWidth(txt1$)*scale_Coordinates_x.d
        Else
          For i4=1 To i7
            i5+VectorTextWidth(Mid(txt1$, i4, 1))
          Next
          ProcedureReturn (i5+stretch_x*Len(txt1$)+stretch_x/12)*scale_Coordinates_x.d
        EndIf
      EndIf
    EndIf
    
    If mode=4 And flag<>3
      If i2
        ProcedureReturn (amount_lines*(i1+1)-stretch_y)*scale_Coordinates_y.d
      Else
        ProcedureReturn (i1+1-stretch_y)*scale_Coordinates_y.d
      EndIf
    EndIf
    
    Macro first_char_width_Vector
      i5=0
      For i4=1 To i7
        i6=VectorTextWidth(Mid(txt1$, i4, 1))
        If i6>i5
          i5=i6
        EndIf
      Next
    EndMacro
    
    padding_1=VectorTextWidth(Space(3))
    
    If mode=3 And flag=3
      first_char_width_Vector
      ProcedureReturn i5*scale_Coordinates_x.d
    EndIf
    
    If mode=4 And flag=3
      ProcedureReturn (i1*i7-stretch_y)*scale_Coordinates_y.d
    EndIf
    
    Macro spread_x_Vector(txt_x, txt_y, ln)
      i6=0
      i4=Len(ln)
      For i5=1 To Len(ln)
        ln1$=Mid(ln, i5, 1)
        If flag=1 And i2
          DrawText_Vector(txt_x+i6-i4*stretch_x/2, txt_y, ln1$, f_col)
        ElseIf flag=2 And i2
          DrawText_Vector(txt_x+i6-i4*stretch_x, txt_y, ln1$, f_col)
        Else
          DrawText_Vector(txt_x+i6, txt_y, ln1$, f_col)
        EndIf
        i6+VectorTextWidth(ln1$)+stretch_x
      Next   
    EndMacro
    
    Macro rtl_Vector(txt)
      i5=0
      i3=Len(txt)
      For i4=1 To i3
        If Mid(txt, i4, 1)=" "
          i6+1
        EndIf
        ln1$=StringField(txt, i4, " ")
        If ln1$<>""
          ReDim f1$(i5)
          ReDim f2$(i5)
          f1$(i5)=ln1$
          f2$(i5)=ReverseString(ln1$)
          i5+1
        EndIf
      Next
      txt=ReverseString(txt)
      For i4=0 To i5-1
        txt=ReplaceString(txt, f2$(i4), f1$(i4))
      Next
      i3=0 : i5=0 : i6=0
    EndMacro 
    
    Macro txt_out_Vector
      If mode=1
        ln$=ReverseString(ln$)
      ElseIf mode=2
        rtl_Vector(ln$)
      EndIf
      If flag=2 ; Right
        If stretch_x
          spread_x_Vector(txt_x+max-VectorTextWidth(ln$), txt_y+i2, ln$)
        Else
          DrawText_Vector(txt_x+max-VectorTextWidth(ln$), txt_y+i2, ln$, f_col)
        EndIf
      ElseIf flag=1 ; Centered
        If stretch_x
          spread_x_Vector(txt_x+max/2-VectorTextWidth(ln$)/2, txt_y+i2, ln$)
        Else
        DrawText_Vector(txt_x+max/2-VectorTextWidth(ln$)/2, txt_y+i2, ln$, f_col) : EndIf
      Else ; Left
        If stretch_x
          spread_x_Vector(txt_x, txt_y+i2, ln$)
        Else
          DrawText_Vector(txt_x, txt_y+i2, ln$, f_col)
        EndIf
      EndIf
    EndMacro
    
    If flag=3 And mode=1
      rtl_Vector(txt1$)
    EndIf
    If flag=3 ; Vertical
      txt1$=ReplaceString(txt1$, #LF$, " ")
      If mode
        txt1$=ReverseString(txt1$)
      EndIf
      For result=1 To i7
        chr$=Mid(txt1$, result, 1)
        first_char_width_Vector
        DrawText_Vector(i5/2+txt_x-VectorTextWidth(chr$)/2, txt_y+i3, chr$, f_col)
        i3+i1
      Next
      ProcedureReturn txt_y+i3
    Else ; Horizontal
      If i2
        i2=0 : ln$=""
        For result=1 To i7
          chr$=Mid(txt1$, result, 1)
          If chr$=#LF$
            i2+i1
            txt_out_Vector
            ln$=""
          Else
            ln$+chr$
          EndIf
        Next
        i2+i1
        txt_out_Vector
        ProcedureReturn txt_y+i2+i1
      Else ; One line
        If mode=1
          txt$=ReverseString(txt$)
        ElseIf mode=2
          rtl_Vector(txt$)
        EndIf
        If stretch_x
          spread_x_Vector(txt_x, txt_y, txt$)
        Else
          DrawText_Vector(txt_x, txt_y, txt$, f_col)
        EndIf
      EndIf
    EndIf
    
    If i6
      ProcedureReturn txt_x+i6
    Else
      ProcedureReturn txt_x+VectorTextWidth(txt$)
    EndIf
  EndProcedure
  
  Procedure Fitted_DrawText_Vector_BF(text$,
                                      x_pos,
                                      y_pos,
                                      output_width,
                                      output_height,
                                      text_color=#Black,
                                      text_stretching_x=0,
                                      text_stretching_y=0,
                                      padding_factor_x.d=0,
                                      padding_factor_y.d=0,
                                      flag=1,
                                      mode=0,
                                      output_offset_factor_x.d=1,
                                      output_offset_factor_y.d=1)
    
    ; Get text width - Base calculation
    Define text_width.d=GetTextWidth_Vector_BF(text$,
                                               Bool(flag=3),      ; Vertical text = 1
                                               text_stretching_x, ; Text stretching x - Headline mode
                                               1)                 ; Text scaling x
    
    ; Get text height - Base calculation
    Define text_height.d=GetTextHeight_Vector_BF(text$,
                                                 Bool(flag=3),      ; Vertical text = 1
                                                 text_stretching_y, ; Text stretching y - Headline mode
                                                 1)                 ; Text scaling y
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Windows
      x_pos+2
    CompilerElse
      x_pos+5
    CompilerEndIf
    
    text_width=output_width/text_width/output_offset_factor_x
    text_height=output_height/text_height/output_offset_factor_y
    
    If activate_dpi_aware
      x_pos*DPI_ResizeFactor_BF()
      y_pos*DPI_ResizeFactor_BF()
      text_width*DPI_ResizeFactor_BF()
      text_height*DPI_ResizeFactor_BF()
    EndIf
    
    Define scale_coordinates_x_.d=text_width-padding_factor_x/200
    Define scale_coordinates_y_.d=text_height-padding_factor_y/200
    Define scale_coordinates_x.d=scale_coordinates_x_*output_offset_factor_x
    Define scale_coordinates_y.d=scale_coordinates_y_*output_offset_factor_y
    
    GetFitSize_BF(scale_coordinates_x_, scale_coordinates_y_, scale_coordinates_x, scale_coordinates_y, 0)
    
    ; Get text width
    Define text_width.d=GetTextWidth_Vector_BF(text$,
                                               Bool(flag=3),          ; Vertical text = 1
                                               text_stretching_x,     ; Text stretching x - Headline mode
                                               GetFitSize_Width_BF()) ; Text scaling x
    
    ; Get text height
    Define text_height.d=GetTextHeight_Vector_BF(text$,
                                                 Bool(flag=3),           ; Vertical text = 1
                                                 text_stretching_y,      ; Text stretching x - Headline mode
                                                 GetFitSize_Height_BF()) ; Text scaling x
    
    If activate_dpi_aware
      x_pos+output_width/2-text_width/2/DPI_ResizeFactor_BF()
      y_pos+output_height/2-text_height/2/DPI_ResizeFactor_BF()
    Else
      x_pos+output_width/2-text_width/2
      y_pos+output_height/2-text_height/2
    EndIf
    
    Define result=DrawText_Vector_BF(x_pos,                  ; X pos
                                     y_pos,                  ; Y pos                                                                                          ;                                        
                                     text$,                  ; Text                                             
                                     text_color,             ; Text Color
                                     flag,                   ; Flag
                                     mode,                   ; mode
                                     text_stretching_x,      ; Text stretching x - Headline mode
                                     text_stretching_y,      ; Text stretching y - Headline mode
                                     GetFitSize_Width_BF(),  ; Scale coordinates x
                                     GetFitSize_Height_BF()) ; Scale coordinates y
    ProcedureReturn 1
  EndProcedure
  
  Procedure GetTextWidth_Vector_BF(text$,
                                   vertical=0,
                                   stretch_x=0,
                                   scaling_x.d=1)
    
    If vertical>0 : Protected flag=3 : EndIf
    ProcedureReturn DrawText_Vector_BF(0,          ; X pos
                                       0 ,         ; Y pos
                                       text$,      ; Text
                                       0,          ; Text Color
                                       flag,       ; Flag
                                       3,          ; mode
                                       stretch_x,  ; Text stretching x
                                       0,          ; Text stretching y
                                       scaling_x,  ; Text scaling x
                                       1)          ; Text scaling y
  EndProcedure
  
  Procedure GetTextHeight_Vector_BF(text$,
                                    vertical=0,
                                    stretch_y=0,
                                    scaling_y.d=1)
    
    If vertical>0 : Protected flag=3 : EndIf
    ProcedureReturn DrawText_Vector_BF(0,         ; X pos
                                       0 ,        ; Y pos
                                       text$,     ; Text
                                       0,         ; Text Color
                                       flag,      ; Flag
                                       4,         ; mode
                                       0,         ; Text stretching x
                                       stretch_y, ; Text stretching y
                                       1,         ; Text scaling x
                                       scaling_y) ; Text scaling y
  EndProcedure
  
  ; Get the text proportion - Multiline and text stretching supported
  Procedure.d Get_Text_Proportion_Factor_X_to_Y_BF(text$,
                                                   font_ID, 
                                                   vertical=0,
                                                   stretch_x=0,
                                                   stretch_y=0)
    
    If StartVectorDrawing(ImageVectorOutput(mini_temp_image_ID))
      
      VectorFont(FontID(font_ID))
      
      Protected get_text_width=GetTextWidth_Vector_BF(text$,
                                                      vertical,  ; Vertical text
                                                      stretch_x, ; Text stretching x - Headline mode
                                                      1)         ; Text scaling x
      
      Protected get_text_height=GetTextHeight_Vector_BF(text$,
                                                        vertical,  ; Vertical text
                                                        stretch_y, ; Text stretching x - Headline mode
                                                        1)         ; Text scaling y
      StopVectorDrawing()
      If get_text_width And get_text_height
        ProcedureReturn  get_text_width/get_text_height
      EndIf
    EndIf
    ProcedureReturn 0
  EndProcedure
  
  Procedure RotateCoordinates_BF(rotate_coordinates_x_.d=0, rotate_coordinates_y_.d=0, rotate_coordinates_degree_.d=0)
    rotate_coordinates_x=rotate_coordinates_x_
    rotate_coordinates_y=rotate_coordinates_y_
    rotate_coordinates_degree.d=rotate_coordinates_degree_.d
    ProcedureReturn 1
  EndProcedure
  
  Procedure PresetLineWidth_Vector_BF(max_1__vector)
    max_1_vector=max_1__vector
  EndProcedure
  
  Procedure DrawText_BF(txt_x.d, txt_y.d, txt$, f_col=#White, b_col=#Black, flag=0, mode=0, stretch_x=0, stretch_y=0)
    If txt$="" : ProcedureReturn -26 : EndIf
    Protected i1.d=TextHeight(" ")+stretch_y, i2.d, i3.d, i4, i5, i6, i7, padding.d, i
    Protected result, max.d, max_len.d, max_2.d, max_3.d, txt_x_0.d=txt_x.d, txt_y_0.d=txt_y.d, amount_lines.d
    Protected txt1$, chr$, ln$, ln1$
    Protected Dim f1$(0), Dim f2$(0)
    
    If FindString(txt$, #CR$)
      txt$=ReplaceString(txt$, #CRLF$, #LF$)
      txt$=ReplaceString(txt$, #LFCR$, #LF$)
      ReplaceString(txt$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    txt1$=txt$
    
    i7=Len(txt1$)
    
    If flag<>3 And FindString(txt1$, #LF$)
      amount_lines=CountString(txt1$, #LF$)+1
      amount_lines_drawtext=amount_lines
      txt_y-i1
      For result=1 To i7
        ln$=StringField(txt1$, result, #LF$)
        i4=Len(ln$)
        If i4 
          max_2=TextWidth(ln$)+stretch_x*i4
          If max_2>max_3
            max_3=max_2
            ln1$=ln$
            i2+i1
            max=TextWidth(ln$)+i4
            max_len=i4
          EndIf
        EndIf
      Next
      
      If max_1>0
        max=max_1
      EndIf
      
      If flag=1 ; Centered
        txt_x+max_len*(stretch_x/2)-max_len/2
      ElseIf flag=2 ; Right
        txt_x+max_len*(stretch_x)-max_len
      EndIf
    EndIf
    
    If mode=3 And flag<>3
      If i2
        If Not stretch_x
          ProcedureReturn TextWidth(ln1$)+2
        Else
          i6=Len(ln1$)
          For i4=1 To i6
            i5+TextWidth(Mid(ln1$, i4, 1))
          Next
          ProcedureReturn i5+stretch_x*i6+stretch_x/12
        EndIf
      Else
        If Not stretch_x
          ProcedureReturn TextWidth(txt1$)+2
        Else
          For i4=1 To i7
            i5+TextWidth(Mid(txt1$, i4, 1))
          Next
          ProcedureReturn i5+stretch_x*Len(txt1$)+stretch_x/12
        EndIf
      EndIf
    EndIf
    
    If mode=4 And flag<>3
      If i2
        ProcedureReturn amount_lines*(i1+1)-stretch_y
      Else
        ProcedureReturn i1+1-stretch_y
      EndIf
    EndIf
    
    Macro first_char_width
      i5=0
      For i4=1 To i7
        i6=TextWidth(Mid(txt1$, i4, 1))
        If i6>i5
          i5=i6
        EndIf
      Next
    EndMacro
    
    If mode=3 And flag=3
      first_char_width
      ProcedureReturn i5
    EndIf
    
    If mode=4 And flag=3
      ProcedureReturn i1*i7-stretch_y
    EndIf
    
    If from_image
      DrawingMode(#PB_2DDrawing_Transparent)
    EndIf
    
    If Not ignore_gradient_colors And (activate_linear_gradient Or activate_elliptical_gradient Or activate_boxed_gradient Or activate_circular_gradient Or activate_conical_gradient) 
      
      DrawingMode(#PB_2DDrawing_Transparent | #PB_2DDrawing_Gradient)
      If activate_backcolor
        BackColor(backcolor)
      EndIf
      If activate_frontcolor
        FrontColor(frontcolor)
      EndIf
      For i=0 To #Size_GradientColors_Array
        If gradient_activate_flag(i)
          GradientColor(gradient_position.f(i), gradient_color(i))
        EndIf
      Next
      
      If activate_linear_gradient
        LinearGradient(linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy)
      EndIf
      
      If activate_circular_gradient
        CircularGradient(circular_gradient_x, circular_gradient_y, circular_gradient_radius)
      EndIf 
      
      If activate_elliptical_gradient
        EllipticalGradient(elliptical_gradient_x, elliptical_gradient_y, elliptical_gradient_xx, elliptical_gradient_yy)
      EndIf
      
      If activate_boxed_gradient
        BoxedGradient(boxed_gradient_x, boxed_gradient_y, boxed_gradient_width, boxed_gradient_height)
      EndIf 
      
      If activate_conical_gradient
        ConicalGradient(conical_gradient_x, conical_gradient_y, conical_gradient_degree)
      EndIf
      
    EndIf
    
    Macro spread_x(txt_x, txt_y, ln)
      i6=0
      i4=Len(ln)
      For i5=1 To Len(ln)
        ln1$=Mid(ln, i5, 1)
        If flag=1 And i2
          DrawText(txt_x+i6-i4*stretch_x/2, txt_y, ln1$, f_col, b_col)
        ElseIf flag=2 And i2
          DrawText(txt_x+i6-i4*stretch_x, txt_y, ln1$, f_col, b_col)
        Else
          DrawText(txt_x+i6, txt_y, ln1$, f_col, b_col)
        EndIf
        i6+TextWidth(ln1$)+stretch_x
      Next   
    EndMacro
    
    Macro rtl(txt)
      i5=0
      i3=Len(txt)
      For i4=1 To i3
        If Mid(txt, i4, 1)=" "
          i6+1
        EndIf
        ln1$=StringField(txt, i4, " ")
        If ln1$<>""
          ReDim f1$(i5)
          ReDim f2$(i5)
          f1$(i5)=ln1$
          f2$(i5)=ReverseString(ln1$)
          i5+1
        EndIf
      Next
      txt=ReverseString(txt)
      For i4=0 To i5-1
        txt=ReplaceString(txt, f2$(i4), f1$(i4))
      Next
      i3=0 : i5=0 : i6=0
    EndMacro 
    
    Macro txt_out
      If mode=1
        ln$=ReverseString(ln$)
      ElseIf mode=2
        rtl(ln$)
      EndIf
      If flag=2 ; Right
        If stretch_x
          spread_x(txt_x+max-TextWidth(ln$), txt_y+i2, ln$)
        Else
          DrawText(txt_x+max-TextWidth(ln$), txt_y+i2, ln$, f_col, b_col)
        EndIf
      ElseIf flag=1 ; Centered
        If stretch_x
          spread_x(txt_x+max/2-TextWidth(ln$)/2, txt_y+i2, ln$)
        Else
        DrawText(txt_x+max/2-TextWidth(ln$)/2, txt_y+i2, ln$, f_col, b_col) : EndIf
      Else ; Left
        If stretch_x
          spread_x(txt_x, txt_y+i2, ln$)
        Else
          DrawText(txt_x, txt_y+i2, ln$, f_col, b_col)
        EndIf
      EndIf
    EndMacro
    
    If flag=3 And mode=1
      rtl(txt1$)
    EndIf
    If flag=3 ; Vertical
      txt1$=ReplaceString(txt1$, #LF$, " ")
      If mode
        txt1$=ReverseString(txt1$)
      EndIf
      For result=1 To i7
        chr$=Mid(txt1$, result, 1)
        first_char_width
        DrawText(i5/2+txt_x-TextWidth(chr$)/2, txt_y+i3, chr$, f_col, b_col)
        i3+i1
      Next
      ProcedureReturn txt_y+i3
    Else ; Horizontal
      If i2
        i2=0 : ln$=""
        For result=1 To i7
          chr$=Mid(txt1$, result, 1)
          If chr$=#LF$
            i2+i1
            txt_out
            ln$=""
          Else
            ln$+chr$
          EndIf
        Next
        i2+i1
        txt_out
        ProcedureReturn txt_y+i2+i1
      Else ; One line
        If mode=1
          txt$=ReverseString(txt$)
        ElseIf mode=2
          rtl(txt$)
        EndIf
        If stretch_x
          spread_x(txt_x, txt_y, txt$)
        Else
          DrawText(txt_x, txt_y, txt$, f_col, b_col)
        EndIf
      EndIf
    EndIf
    
    If i6
      ProcedureReturn txt_x+i6
    Else
      ProcedureReturn txt_x+TextWidth(txt$)
    EndIf
  EndProcedure
  
  Procedure DrawTextCreateImage_BF(txt$, f_col=#White, b_col=#Black, flag=0, mode=0, font_ID=-1, stretch_x=0, stretch_y=0, padding=0)
    If txt$="" : ProcedureReturn -26 : EndIf
    Static little_temporary_image_ID
    If Not little_temporary_image_ID
      little_temporary_image_ID=CreateImage(#PB_Any, 16, 16)
    EndIf
    
    If Not StartDrawing(ImageOutput(little_temporary_image_ID)) ; Get the needed width and height for the output
      ProcedureReturn -4
    EndIf
    
    If IsFont(font_ID) : DrawingFont(FontID(font_ID)) : EndIf
    
    If padding=-1
      padding=TextWidth(Space(3))
    EndIf
    
    Protected padding_x=padding
    Protected padding_y=padding
    
    If manual_padding_draw_text_create_image_x<>0
      padding_x=manual_padding_draw_text_create_image_x
    EndIf
    
    If manual_padding_draw_text_create_image_y<>0
      padding_y=manual_padding_draw_text_create_image_y
    EndIf
    
    If padding_x<0 : padding_x=padding : EndIf
    If padding_y<0 : padding_y=padding : EndIf
    
    If max_1>0
      Protected output_width=max_1 ; Get the needed width
    Else
      If mode<>4
        output_width=DrawText_BF(0, 0, txt$, 0, 0, flag, 3, stretch_x, stretch_y)-stretch_x ; Get the needed width
      EndIf
      If mode=3
        StopDrawing()
        ProcedureReturn output_width+padding_x
      EndIf
    EndIf
    Protected output_height=DrawText_BF(0, 0, txt$, 0, 0, flag, 4, stretch_x, stretch_y)-stretch_y ; Get the needed height
    StopDrawing()
    
    If mode=4
      ProcedureReturn output_height+padding_y
    EndIf
    
    Protected image_ID=CreateImage(#PB_Any, output_width+padding_x<<1, output_height+padding_y<<1)
    
    If Not image_ID
      ProcedureReturn -6
    EndIf
    
    If Not StartDrawing(ImageOutput(image_ID)) ; Draw the text in the image
      FreeImage(image_ID)
      ProcedureReturn -4
    EndIf
    
    Box(0, 0, output_width+padding_x<<1, output_height+padding_y<<1, b_col)
    
    Protected intensity=255-draw_text_create_image_add_image_alpha
    If intensity<0 : intensity=0 : EndIf    
    If intensity>255 : intensity=255 : EndIf
    
    If draw_text_create_image_add_image And IsImage(draw_text_create_image_add_image_ID)
      Protected width=draw_text_create_image_add_image_ID_width
      Protected height=draw_text_create_image_add_image_ID_height
      If width<0 : width=ImageWidth(draw_text_create_image_add_image_ID): EndIf
      If height<0 : height=ImageHeight(draw_text_create_image_add_image_ID) : EndIf
      Protected temp_draw_text_create_image_add_image_ID=CopyImage(draw_text_create_image_add_image_ID, #PB_Any)
      If IsImage(temp_draw_text_create_image_add_image_ID)
        If Not ResizeImage(temp_draw_text_create_image_add_image_ID, width, height, resize_interpolation)
          FreeImage(temp_draw_text_create_image_add_image_ID)
          StopDrawing()
          ProcedureReturn -27
        EndIf
        DrawAlphaImage(ImageID(temp_draw_text_create_image_add_image_ID), draw_text_create_image_add_image_x, draw_text_create_image_add_image_y, intensity)
        FreeImage(temp_draw_text_create_image_add_image_ID)
      EndIf
    EndIf
    
    If IsFont(font_ID) : DrawingFont(FontID(font_ID)) : EndIf 
    
    from_image=1
    image_color=b_col
    DrawText_BF(padding_x, padding_y, txt$, f_col, 0, flag, mode, stretch_x, stretch_y)
    from_image=0
    StopDrawing()
    
    If gradient_color_red Or gradient_color_green Or gradient_color_blue Or gradient_color_contrast Or gradient_color_brightness Or gradient_color_intensity
      Image_coloring_BF(image_ID, image_ID, ; Post processing - Optimize sprite
                        gradient_color_red,
                        gradient_color_green,
                        gradient_color_blue,
                        gradient_color_contrast,
                        gradient_color_brightness,
                        gradient_color_intensity)
    EndIf
    
    ProcedureReturn image_ID
  EndProcedure
  
  Procedure DrawTextCreate_PB_Sprite_BF(txt$, f_col=#White, b_col=#Black, flag=0, mode=0, font_ID=-1, stretch_x=0, stretch_y=0, padding=0, sprite_mode=0)
    If txt$="" : ProcedureReturn -26 : EndIf
    Protected image_ID=DrawTextCreateImage_BF(txt$, f_col, b_col, flag, mode, font_ID, stretch_x, stretch_y, padding)
    If Not IsImage(image_ID) : ProcedureReturn -2 : EndIf
    Protected sprite_ID=Create_PB_Sprite_from_Source_BF(image_ID, 1, sprite_mode)
    If Not IsSprite(sprite_ID) : ProcedureReturn -18 : EndIf
    ProcedureReturn sprite_ID
  EndProcedure
  
  Procedure DrawTextScreenOutput_BF(txt_x, txt_y, txt$, f_col=#White, b_col=#Black, flag=0, mode=0, transparent=1, font_ID=-1, stretch_x=0, stretch_y=0)
    If Not StartDrawing(ScreenOutput()) : ProcedureReturn -20 : EndIf
    If IsFont(font_ID) : DrawingFont(FontID(font_ID)) : EndIf
    If transparent : DrawingMode(#PB_2DDrawing_Transparent) : EndIf
    Protected result=DrawText_BF(txt_x, txt_y, txt$, f_col, b_col, flag, mode, stretch_x, stretch_y)
    StopDrawing()
    ProcedureReturn result
  EndProcedure
  
  Procedure PresetLineWidth_BF(max_1_)
    max_1=max_1_
  EndProcedure
  
  Procedure GetTextWidth_BF(text$,
                            font_ID,
                            vertical=0,
                            stretch_x=0,
                            padding_x=0)
    
    If vertical>0 : Protected flag=3 : EndIf
    ProcedureReturn DrawTextCreateImage_BF(text$, 0, 0, flag, 3, font_ID, stretch_x, 0, padding_x)
  EndProcedure
  
  Procedure GetTextHeight_BF(text$,
                             font_ID,
                             vertical=0,
                             stretch_y=0,
                             padding_y=0)
    
    If vertical>0 : Protected flag=3 : EndIf
    ProcedureReturn DrawTextCreateImage_BF(text$, 0, 0, flag, 4, font_ID, 0, stretch_y, padding_y)
  EndProcedure
  
  Procedure.d VectorTextWidth_Multiline_BF(text$, flag=#PB_VectorText_Default)
    Protected i, width.d, result.d=0.0, count=CountString(text$,#LF$)
    For i=0 To count
      width=VectorTextWidth(StringField(text$,i+1,#LF$),flag)
      If width>result
        result=width
      EndIf
    Next
    ProcedureReturn result
  EndProcedure
  
  Procedure.d VectorTextHeight_Multiline_BF(text$, flag=#PB_VectorText_Default)
    ProcedureReturn VectorTextHeight(" ", flag)*(CountString(text$, #LF$)+1)
  EndProcedure
  
  Procedure.d VectorTextWidth_Multiline_Pro_BF(text$, flag=#PB_VectorText_Default)
    If FindString(text$, #CR$)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    Protected i, width.d, result.d=0.0, count=CountString(text$, #LF$)
    For i=0 To count
      width=VectorTextWidth(StringField(text$, i+1, #LF$), flag)
      If width>result
        result=width
      EndIf
    Next
    ProcedureReturn result
  EndProcedure
  
  Procedure.d VectorTextHeight_Multiline_Pro_BF(text$, flag=#PB_VectorText_Default)
    If FindString(text$, #CR$)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    ProcedureReturn VectorTextHeight(" ", flag)*(CountString(text$,#LF$)+1)
  EndProcedure
  
  Procedure DrawVectorTextMultiline_BF(text$)
    Protected count=CountString(text$, #LF$)
    Protected height.d = VectorTextHeight(text$)
    Protected i, x=PathCursorX(), y=PathCursorY()
    For i=0 To count
      MovePathCursor(x, y+(i*height))
      DrawVectorText(StringField(text$, i+1, #LF$))
    Next
    ProcedureReturn 1
  EndProcedure
  
  Procedure DrawVectorTextMultiline_Pro_BF(text$)
    If FindString(text$, #CR$)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    Protected count=CountString(text$, #LF$)
    Protected height.d = VectorTextHeight(text$)
    Protected i, x=PathCursorX(), y=PathCursorY()
    For i=0 To count
      MovePathCursor(x, y+(i*height))
      DrawVectorText(StringField(text$, i+1, #LF$))
      ResetPath()
    Next
    ProcedureReturn 1
  EndProcedure
  
  Procedure GradientColor_Alpha_BF(gradient_color_alpha_)
    gradient_color_alpha=gradient_color_alpha_
  EndProcedure
  
  Procedure GradientColor_Red_BF(gradient_color_red_)
    gradient_color_red=gradient_color_red_
  EndProcedure
  
  Procedure GradientColor_Green_BF(gradient_color_green_)
    gradient_color_green=gradient_color_green_
  EndProcedure
  
  Procedure GradientColor_Blue_BF(gradient_color_blue_)
    gradient_color_blue=gradient_color_blue_
  EndProcedure
  
  Procedure GradientColor_Contrast_BF(gradient_color_contrast_) 
    gradient_color_contrast=gradient_color_contrast_
  EndProcedure
  
  Procedure GradientColor_Brightness_BF(gradient_color_brightness_)
    gradient_color_brightness=gradient_color_brightness_
  EndProcedure
  
  Procedure GradientColor_Intensity_BF(gradient_color_intensity_)
    gradient_color_intensity=gradient_color_intensity_
  EndProcedure
  
  Procedure BackColor_BF(activate_backcolor_=0, backcolor_=0)
    activate_backcolor=activate_backcolor_
    backcolor=backcolor_
  EndProcedure
  
  Procedure FrontColor_BF(activate_frontcolor_=0, frontcolor_=0)
    activate_frontcolor=activate_frontcolor_
    frontcolor=frontcolor_
  EndProcedure
  
  Procedure GradientColor_Array_BF(gradient_activate_flag, array_position, gradient_position.f, gradient_color)
    If array_position<0 : array_position=0 : EndIf
    If array_position>#Size_GradientColors_Array : array_position=#Size_GradientColors_Array : EndIf
    gradient_activate_flag(array_position)=gradient_activate_flag
    gradient_position(array_position)=gradient_position
    gradient_color(array_position)=gradient_color
  EndProcedure
  
  Procedure LinearGradient_BF(activate_linear_gradient_=0, linear_gradient_x_=0, linear_gradient_y_=0, linear_gradient_xx_=0, linear_gradient_yy_=0)
    activate_linear_gradient=activate_linear_gradient_
    linear_gradient_x=linear_gradient_x_
    linear_gradient_y=linear_gradient_y_
    linear_gradient_xx=linear_gradient_xx_
    linear_gradient_yy=linear_gradient_yy_
  EndProcedure
  
  Procedure CircularGradient_BF(activate_circular_gradient_=0, circular_gradient_x_=0, circular_gradient_y_=0, circular_gradient_radius_=0)
    activate_circular_gradient=activate_circular_gradient_
    circular_gradient_x=circular_gradient_x_
    circular_gradient_y=circular_gradient_y_
    circular_gradient_radius=circular_gradient_radius_
  EndProcedure
  
  Procedure EllipticalGradient_BF(activate_elliptical_gradient_=0, elliptical_gradient_x_=0, elliptical_gradient_y_=0, elliptical_gradient_xx_=0, elliptical_gradient_yy_=0)
    activate_elliptical_gradient=activate_elliptical_gradient_
    elliptical_gradient_x=elliptical_gradient_x_
    elliptical_gradient_y=elliptical_gradient_y_
    elliptical_gradient_xx=elliptical_gradient_xx_
    elliptical_gradient_yy=elliptical_gradient_yy_
  EndProcedure
  
  Procedure BoxedGradient_BF(activate_boxed_gradient_=0, boxed_gradient_x_=0, boxed_gradient_y_=0, boxed_gradient_width_=0, boxed_gradient_height_=0)
    activate_boxed_gradient=activate_boxed_gradient_
    boxed_gradient_x=boxed_gradient_x_
    boxed_gradient_y=boxed_gradient_y_
    boxed_gradient_width=boxed_gradient_width_
    boxed_gradient_height=boxed_gradient_height_
  EndProcedure
  
  Procedure ConicalGradient_BF(activate_conical_gradient_=0, conical_gradient_x_=0, conical_gradient_y_=0, conical_gradient_degree_=0)
    activate_conical_gradient=activate_conical_gradient_
    conical_gradient_x=conical_gradient_x_
    conical_gradient_y=conical_gradient_y_
    conical_gradient_degree=conical_gradient_degree_
  EndProcedure
  
  Procedure DeactivateGradientColors_BF()
    Protected i
    For i=0 To #Size_GradientColors_Array
      gradient_color(i)=0
      gradient_activate_flag(i)=0
      gradient_position.f(i)=0
    Next
    LinearGradient_BF()
    CircularGradient_BF()
    EllipticalGradient_BF()
    BoxedGradient_BF()
    ConicalGradient_BF()
    gradient_color_alpha=0
    gradient_color_red=0
    gradient_color_green=0
    gradient_color_blue=0
    gradient_color_contrast=0
    gradient_color_brightness=0
    gradient_color_intensity=0
  EndProcedure
  
  Procedure IgnoreGradientColors_BF(ignore_gradient_colors_)
    ignore_gradient_colors=ignore_gradient_colors_
  EndProcedure
  
  Macro make_box(type)
    xx-1 : yy-1 
    If lr
      type(x, y, x+xx, y, color, move, shift, 1)
      type(x+xx, y, x+xx, y+yy, color, move, shift, 1)
      type(x, y+yy, x+xx, y+yy, color, move, shift, 0)
      type(x, y+yy, x, y, color, move, shift, 0)
    Else
      type(x, y, x+xx, y, color, move, shift, 0)
      type(x+xx, y, x+xx, y+yy, color, move, shift, 0)
      type(x, y+yy, x+xx, y+yy, color, move, shift, 1)
      type(x, y+yy, x, y, color, move, shift, 1)
    EndIf
  EndMacro
  
  Procedure Line8(x.i, y.i, xx.i, yy.i, shift.a, color.l, lr.i) 
    Protected a.i, dx.i, dy.i, add.i = 1
    Protected *plotX.Integer=@xx, *plotY.Integer=@yy 
    If Abs(x-xx)<=Abs(y-yy) : Swap x, y : Swap xx, yy : *plotX=@yy : *plotY=@xx : EndIf
    If x < xx : Swap x, xx : Swap y, yy : EndIf
    If y<yy : y=yy<<1-y : add=-1 : EndIf
    dy=(y-yy)<<1 : a=x-xx : dx=a<<1
    While xx<=x
      If shift&1
        Box(*plotX\i, *plotY\i, 1, 1, color)
      EndIf
      If lr
        shift=(shift<<1)|((shift>>7)&$7f)
      Else
        shift=((shift>>1)&$7F)|(shift<<7)
      EndIf
      xx+1 : a-dy : If a<=0 : a+dx : yy+add : EndIf
    Wend
  EndProcedure
  
  Procedure Line16(x.i, y.i, xx.i, yy.i, shift.u, color.l, lr.i)
    Protected a.i, dx.i, dy.i, add.i=1
    Protected *plotX.Integer=@xx, *plotY.Integer=@yy
    If Abs(x-xx)<=Abs(y-yy) : Swap x, y : Swap xx, yy : *plotX=@yy : *plotY=@xx : EndIf
    If x < xx : Swap x, xx : Swap y, yy : EndIf
    If y<yy : y=yy<<1-y : add=-1 : EndIf
    dy=(y-yy)<<1 : a=x-xx : dx=a<<1
    While xx<=x
      If shift&1 : Box(*plotX\i, *plotY\i, 1, 1, color) : EndIf
      If lr
        shift=(shift<<1)|((shift>>15)&$7fff)
      Else
        shift=((shift>>1)&$7fff)|(shift<<15)
      EndIf
      xx+1 : a-dy : If a<=0 : a+dx : yy+add : EndIf
    Wend
  EndProcedure
  
  Procedure Line32(x.i, y.i, xx.i, yy.i, shift.q, color.l, lr.i) 
    Protected a.i, dx.i, dy.i, add.i = 1
    Protected *plotX.Integer=@xx, *plotY.Integer=@yy
    If Abs(x-xx)<=Abs(y-yy) : Swap x, y : Swap xx, yy : *plotX=@yy : *plotY=@xx : EndIf
    If x < xx : Swap x, xx : Swap y, yy : EndIf
    If y<yy : y=yy<<1-y : add=-1 : EndIf
    dy=(y-yy)<<1 : a=x-xx : dx=a<<1
    While xx<=x
      If shift&1 : Box(*plotX\i, *plotY\i, 1, 1, color) : EndIf
      If lr
        shift=(shift<<1)|((shift>>31)&$7fffffff)
      Else
        shift=((shift>>1)&$7fffffff)|(shift<<31)
      EndIf
      xx+1 : a-dy : If a<=0 : a+dx : yy+add : EndIf
    Wend  
  EndProcedure
  
  Procedure LineGo_8_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.a, lr.i)
    Protected i
    For i=0 To move
      shift=(shift<<1)|((shift>>7)&$7f)  
    Next i
    If activate_LineGo_backcolor : LineXY(x, y, xx, yy, LineGo_backcolor) : EndIf
    Line8(x, y, xx, yy, shift, color, lr)     
  EndProcedure
  
  Procedure LineGo_16_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.u, lr.i)
    Protected i
    For i=0 To move
      shift=(shift<<1)|((shift>>15)&$7fff)  
    Next i
    If activate_LineGo_backcolor : LineXY(x, y, xx, yy, LineGo_backcolor) : EndIf
    Line16(x, y, xx, yy, shift, color, lr)      
  EndProcedure
  
  Procedure LineGo_32_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.q, lr.i)
    Protected i
    For i=0 To move
      shift=(shift<<1)|((shift>>31)&$7fffffff)  
    Next i
    If activate_LineGo_backcolor : LineXY(x, y, xx, yy, LineGo_backcolor) : EndIf
    Line32(x, y, xx, yy, shift, color, lr)     
  EndProcedure
  
  Procedure BoxGo_8_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.a, lr.i=0) : make_box(LineGo_8_BF) : EndProcedure
  
  Procedure BoxGo_16_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.u, lr.i=0) : make_box(LineGo_16_BF) : EndProcedure
  
  Procedure BoxGo_32_BF(x.i, y.i, xx.i, yy.i, color.l, move.i, shift.q, lr.i=0) : make_box(LineGo_32_BF) : EndProcedure
  
  Procedure ActivateLineGoBackcolor_BF(activate_LineGo_backcolor_, LineGo_backcolor_=0)
    activate_LineGo_backcolor=activate_LineGo_backcolor_
    LineGo_backcolor=LineGo_backcolor_
  EndProcedure
  
  Procedure IsImageInvisibleColor_BF(image_ID)
    If Not IsImage(image_ID) : ProcedureReturn -2 : EndIf
    Protected i, ii
    Protected image_width=ImageWidth(image_ID)-1
    Protected image_height=ImageHeight(image_ID)-1
    If Not StartDrawing(ImageOutput(image_ID))
      ProcedureReturn -4
    EndIf
    DrawingMode(#PB_2DDrawing_AllChannels)
    For i=0 To image_width
      For ii=0 To image_height
        If Not Alpha(Point(i, ii))
          StopDrawing()
          ProcedureReturn 1
        EndIf
      Next
    Next
    StopDrawing()
    ProcedureReturn 0
  EndProcedure
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure Get_Gif_FrameDelay_BF(gif_ID, frame)
      If Not IsImage(gif_ID) Or ImageFrameCount_BF(gif_ID)<2 : ProcedureReturn 100 : EndIf ; Workaround PB570 - PB incompatibility, a image give back 0, on Windows 1
      If frame<0 Or frame>ImageFrameCount_BF(gif_ID) : ProcedureReturn 100 : EndIf         ; Delay 0 works not, so as a standard, a resulted delay 0 = delay 100ms
      SetImageFrame(gif_ID, frame)                                                         ; - This is helpfull for displaying a simple image, it slows down the action loop 
      Protected frame_delay=GetImageFrameDelay(gif_ID)                                     ; Get the GIF frame delay
      If Not frame_delay : frame_delay=100 : EndIf
      ProcedureReturn frame_delay
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure CreateSprite_from_Gif_Frame_BF(gif_ID, frame, invisible_color)
      If frame<0 Or frame>ImageFrameCount_BF(gif_ID) : ProcedureReturn -10 : EndIf
      Protected frame_width=(ImageWidth(gif_ID))
      Protected frame_height=(ImageHeight(gif_ID))
      SetImageFrame(gif_ID, frame) ; Set the GIF frame for processing
      Protected temporary_frame_ID=CreateImage(#PB_Any, frame_width, frame_height, 24, invisible_color)
      If Not StartDrawing(ImageOutput(temporary_frame_ID)) : ProcedureReturn -4 : EndIf ; Draw in the temporary frame buffer
      DrawAlphaImage(ImageID(gif_ID), 0, 0)                                             ; Draw the GIF frame in the temporary frame buffer
      StopDrawing()
      ProcedureReturn temporary_frame_ID
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure Create_PB_Sprite_from_Gif_Frame_BF(gif_ID, frame, invisible_color, sprite_mode=0)
      If frame<0 Or frame>ImageFrameCount_BF(gif_ID) : ProcedureReturn -10 : EndIf
      Protected frame_width=(ImageWidth(gif_ID))
      Protected frame_height=(ImageHeight(gif_ID))
      SetImageFrame(gif_ID, frame) ; Set the GIF frame for processing
      Protected temporary_frame_ID=CreateSprite(#PB_Any, frame_width, frame_height, sprite_mode)
      If Not StartDrawing(SpriteOutput(temporary_frame_ID)) : ProcedureReturn -16 : EndIf ; Draw in the temporary frame buffer
      Box(0, 0, frame_width, frame_height, invisible_color)                               ; Draw the invisible color
      DrawAlphaImage(ImageID(gif_ID), 0, 0)                                               ; Draw the GIF frame in the temporary frame buffer
      StopDrawing()
      ProcedureReturn temporary_frame_ID
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure CreateSprite_from_Gif_Frame_rotated_BF(gif_ID,
                                                     frame, 
                                                     invisible_color,
                                                     color_distance,
                                                     output_width=0,
                                                     output_height=0,
                                                     degree=0,
                                                     rotating_mode=0,
                                                     rotating_offset_x=0,
                                                     rotating_offset_y=0)
      
      If frame<0 Or frame>ImageFrameCount_BF(gif_ID) : ProcedureReturn -10 : EndIf
      Protected resulted_x_y, old_percent_color_distance.f
      Protected frame_width=(ImageWidth(gif_ID))
      Protected frame_height=(ImageHeight(gif_ID))
      SetImageFrame(gif_ID, frame) ; Set the GIF frame for processing
      Protected temporary_frame_1_ID=CreateImage(#PB_Any, frame_width, frame_height)
      If Not temporary_frame_1_ID
        ProcedureReturn -6
      EndIf
      Protected temporary_frame_ID, temporary_frame_2_ID
      If Not StartDrawing(ImageOutput(temporary_frame_1_ID))
        FreeImage(temporary_frame_1_ID)
        ProcedureReturn -6
      EndIf
      Box(0, 0, frame_width, frame_height, invisible_color)
      DrawAlphaImage(ImageID(gif_ID), 0, 0)
      StopDrawing()
      
      If output_width>0 And output_height>0
        If Not ResizeImage(temporary_frame_1_ID, output_width, output_height, resize_interpolation)
          FreeImage(temporary_frame_1_ID)
          ProcedureReturn -27
        EndIf
        frame_width=output_width
        frame_height=output_height
      EndIf
      
      If rotating_mode
        resulted_x_y=Sqr(Pow(frame_width+Abs(rotating_offset_x)*4, 2)+Pow(frame_height+Abs(rotating_offset_y)*4, 2))+7
        temporary_frame_2_ID=CreateImage(#PB_Any, resulted_x_y, resulted_x_y, 24, invisible_color)
      Else
        If color_distance
          old_percent_color_distance=percent_1
          percent_1=color_distance
          BF(-1, temporary_frame_1_ID, invisible_color) ; Clean sprite mask
          percent_1=old_percent_color_distance
        EndIf
        ProcedureReturn temporary_frame_1_ID
      EndIf 
      
      old_percent_color_distance=percent
      percent=color_distance
      
      Protected result=RotateSprite_simple_BF(rotating_mode, temporary_frame_2_ID, temporary_frame_1_ID,                              
                                              -1,
                                              invisible_color,
                                              0,
                                              0,
                                              degree,
                                              0,
                                              output_width,
                                              output_height,
                                              rotating_offset_x,
                                              rotating_offset_y)   
      
      percent=old_percent_color_distance
      
      If result<0 : FreeImage(temporary_frame_1_ID) : FreeImage(temporary_frame_2_ID) : ProcedureReturn result : EndIf
      
      FreeImage(temporary_frame_1_ID)
      ProcedureReturn temporary_frame_2_ID
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure Create_PB_Sprite_from_Gif_Frame_rotated_BF(gif_ID,
                                                         frame, 
                                                         invisible_color,
                                                         color_distance,
                                                         sprite_mode=0,
                                                         output_width=0,
                                                         output_height=0,
                                                         degree=0,
                                                         rotating_mode=0,
                                                         rotating_offset_x=0,
                                                         rotating_offset_y=0)
      
      If frame<0 Or frame>ImageFrameCount_BF(gif_ID) : ProcedureReturn -10 : EndIf
      Protected resulted_x_y, old_percent_color_distance.f
      Protected frame_width=(ImageWidth(gif_ID))
      Protected frame_height=(ImageHeight(gif_ID))
      SetImageFrame(gif_ID, frame) ; Set the GIF frame for processing
      Protected temporary_frame_1_ID=CreateImage(#PB_Any, frame_width, frame_height)
      Protected temporary_frame_ID, temporary_frame_2_ID
      If Not StartDrawing(ImageOutput(temporary_frame_1_ID))
        FreeImage(temporary_frame_1_ID)
        ProcedureReturn -6
      EndIf
      Box(0, 0, frame_width, frame_height, invisible_color)
      DrawAlphaImage(ImageID(gif_ID), 0, 0)
      StopDrawing()
      
      If output_width>0 And output_height>0
        If Not ResizeImage(temporary_frame_1_ID, output_width, output_height, resize_interpolation)
          FreeImage(temporary_frame_1_ID)
          ProcedureReturn -27
        EndIf
        frame_width=output_width
        frame_height=output_height
      EndIf
      
      If rotating_mode
        resulted_x_y=Sqr(Pow(frame_width+Abs(rotating_offset_x)*4, 2)+Pow(frame_height+Abs(rotating_offset_y)*4, 2))+7
        temporary_frame_2_ID=CreateImage(#PB_Any, resulted_x_y, resulted_x_y, 24, invisible_color)
      Else
        temporary_frame_ID=CreateSprite(#PB_Any, frame_width, frame_height, sprite_mode)
        If color_distance
          old_percent_color_distance=percent_1
          percent_1=color_distance
          BF(-1, temporary_frame_1_ID, invisible_color) ; Clean sprite mask
          percent_1=old_percent_color_distance
        EndIf
        If Not StartDrawing(SpriteOutput(temporary_frame_ID)) : FreeSprite(temporary_frame_ID) : FreeImage(temporary_frame_1_ID) : ProcedureReturn -16 : EndIf
        DrawImage(ImageID(temporary_frame_1_ID), 0, 0)
        StopDrawing()
        FreeImage(temporary_frame_1_ID) 
        ProcedureReturn temporary_frame_ID
      EndIf 
      
      old_percent_color_distance=percent
      percent=color_distance
      
      Protected result=RotateSprite_simple_BF(rotating_mode, temporary_frame_2_ID, temporary_frame_1_ID,                              
                                              -1,
                                              invisible_color,
                                              0,
                                              0,
                                              degree,
                                              0,
                                              output_width,
                                              output_height,
                                              rotating_offset_x,
                                              rotating_offset_y)   
      
      percent=old_percent_color_distance
      
      If result<0 : FreeImage(temporary_frame_1_ID) : FreeImage(temporary_frame_2_ID) : ProcedureReturn result : EndIf
      
      temporary_frame_ID=CreateSprite(#PB_Any, ImageWidth(temporary_frame_2_ID), ImageHeight(temporary_frame_2_ID), sprite_mode)
      
      If Not StartDrawing(SpriteOutput(temporary_frame_ID))
        FreeSprite(temporary_frame_ID) : FreeImage(temporary_frame_1_ID) : : FreeImage(temporary_frame_2_ID)
        ProcedureReturn -16
      EndIf
      DrawImage(ImageID(temporary_frame_2_ID), 0, 0)
      StopDrawing()
      FreeImage(temporary_frame_1_ID)
      FreeImage(temporary_frame_2_ID) 
      ProcedureReturn temporary_frame_ID
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure Output_Gif_Frame_on_Canvas_rotated_BF(mode, canvas_ID, gif_ID,
                                                    x,
                                                    y,
                                                    frame, 
                                                    invisible_color,
                                                    color_distance,
                                                    output_width=0,
                                                    output_height=0,
                                                    degree=0,
                                                    rotating_mode=0,
                                                    rotating_offset_x=0,
                                                    rotating_offset_y=0)
      
      If frame<0 Or frame>ImageFrameCount_BF(gif_ID) : ProcedureReturn -10 : EndIf
      Protected result, transparence_mode, alpha, old_percent_color_distance.f
      Protected frame_width=(ImageWidth(gif_ID))
      Protected frame_height=(ImageHeight(gif_ID))
      SetImageFrame(gif_ID, frame) ; Set the GIF frame for processing
      Protected temporary_frame_ID=CreateImage(#PB_Any, frame_width, frame_height)
      If Not StartDrawing(ImageOutput(temporary_frame_ID))
        FreeImage(temporary_frame_ID)
        ProcedureReturn -6
      EndIf
      Box(0, 0, frame_width, frame_height, invisible_color)
      DrawAlphaImage(ImageID(gif_ID), 0, 0)
      StopDrawing()
      
      old_percent_color_distance=percent
      percent=color_distance
      result=RotateSprite_simple_BF(rotating_mode, canvas_ID, temporary_frame_ID,                              
                                    -1,
                                    invisible_color,
                                    x,
                                    y,
                                    degree,
                                    mode, ; Alpha
                                    output_width,
                                    output_height,
                                    rotating_offset_x,
                                    rotating_offset_y) 
      
      If result <0 : FreeImage(temporary_frame_ID) : ProcedureReturn result : EndIf
      percent=old_percent_color_distance
      
      FreeImage(temporary_frame_ID)
      If result<0 : ProcedureReturn result : EndIf
      ProcedureReturn 1
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure Gif_on_Image_BF(gif_ID, image_ID, x=0, y=0, frame=0, alpha=0, output_width=0, output_height=0)
      SetImageFrame(gif_ID, frame) ; Set the GIF frame for processing
      If output_width>0 And output_height>0
        Protected temporary_image_ID=CopyImage(gif_ID, #PB_Any)
        If Not temporary_image_ID
          ProcedureReturn -6
        EndIf
        If Not ResizeImage(temporary_image_ID, output_width, output_height, resize_interpolation)
          FreeImage(temporary_image_ID)
          ProcedureReturn -27
        EndIf
        If Not StartDrawing(ImageOutput(image_ID)) : ProcedureReturn -3 : EndIf
        DrawAlphaImage(ImageID(temporary_image_ID), x, y, alpha)
        FreeImage(temporary_image_ID)
      Else
        If Not StartDrawing(ImageOutput(image_ID)) : ProcedureReturn -3 : EndIf
        DrawAlphaImage(ImageID(gif_ID), x, y, alpha)
      EndIf
      StopDrawing()
      ProcedureReturn 1
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure Gif_on_Canvas_BF(gif_ID, canvas_ID, x=0, y=0, frame=0, alpha=0, output_width=0, output_height=0)
      SetImageFrame(gif_ID, frame) ; Set the GIF frame for processing
      If output_width>0 And output_height>0
        Protected temporary_image_ID=CopyImage(gif_ID, #PB_Any)
        If Not temporary_image_ID
          ProcedureReturn -6
        EndIf
        If Not ResizeImage(temporary_image_ID, output_width, output_height, resize_interpolation)
          FreeImage(temporary_image_ID)
          ProcedureReturn -27
        EndIf
        If Not StartDrawing(CanvasOutput(canvas_ID)) : ProcedureReturn -3 : EndIf
        DrawAlphaImage(ImageID(temporary_image_ID), x, y, alpha)
        FreeImage(temporary_image_ID)
      Else
        If Not StartDrawing(CanvasOutput(canvas_ID)) : ProcedureReturn -3 : EndIf
        DrawAlphaImage(ImageID(gif_ID), x, y, alpha)
      EndIf
      StopDrawing()
      ProcedureReturn 1
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure Gif_on_PB_Sprite_BF(gif_ID, sprite_ID, x=0, y=0, frame=0, alpha=0, output_width=0, output_height=0)
      SetImageFrame(gif_ID, frame) ; Set the GIF frame for processing
      If output_width>0 And output_height>0
        Protected temporary_image_ID=CopyImage(gif_ID, #PB_Any)
        If Not temporary_image_ID
          ProcedureReturn -6
        EndIf
        If Not ResizeImage(temporary_image_ID, output_width, output_height, resize_interpolation)
          FreeImage(temporary_image_ID)
          ProcedureReturn -27
        EndIf
        If Not StartDrawing(SpriteOutput(sprite_ID)) : ProcedureReturn -3 : EndIf
        DrawAlphaImage(ImageID(temporary_image_ID), x, y, alpha)
        FreeImage(temporary_image_ID)
      Else
        If Not StartDrawing(SpriteOutput(sprite_ID)) : ProcedureReturn -3 : EndIf
        DrawAlphaImage(ImageID(gif_ID), x, y, alpha)
      EndIf
      StopDrawing()
      ProcedureReturn 1
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure Gif_on_Screen_BF(gif_ID, x=0, y=0, frame=0, alpha=0, output_width=0, output_height=0)
      SetImageFrame(gif_ID, frame) ; Set the GIF frame for processing
      If output_width>0 And output_height>0
        Protected temporary_image_ID=CopyImage(gif_ID, #PB_Any)
        If Not temporary_image_ID
          ProcedureReturn -6
        EndIf
        If Not ResizeImage(temporary_image_ID, output_width, output_height, resize_interpolation)
          FreeImage(temporary_image_ID)
          ProcedureReturn -27
        EndIf
        If Not StartDrawing(ScreenOutput()) : ProcedureReturn -3 : EndIf
        DrawAlphaImage(ImageID(temporary_image_ID), x, y, alpha)
        FreeImage(temporary_image_ID)
      Else
        If Not StartDrawing(ScreenOutput()) : ProcedureReturn -3 : EndIf
        DrawAlphaImage(ImageID(gif_ID), x, y, alpha)
      EndIf
      StopDrawing()
      ProcedureReturn 1
    EndProcedure
  CompilerEndIf
  
  ; PB sprite Rotate Bugfix (Workaround PB561) from J. Baker - I am not sure we get a bugfix in the future, so i add this function
  Procedure Rotate_PB_Sprite_with_Offset_BF(sprite_ID, X, Y, angle, absolute=0, alpha=0, with_invisible_color=0)
    Protected degree
    degree+angle
    If degree>=360 : degree=0 : EndIf
    If absolute=1
      RotateSprite(sprite_ID, angle, #PB_Absolute)
    Else
      RotateSprite(sprite_ID, angle, #PB_Relative)
    EndIf
    If with_invisible_color
      DisplayTransparentSprite(sprite_ID, X + Cos(Radian(degree)) * SpriteWidth(sprite_ID) / 2048, Y + Sin(Radian(degree)) * SpriteHeight(sprite_ID) / 2048, alpha)
    Else
      DisplaySprite(sprite_ID, X + Cos(Radian(degree)) * SpriteWidth(sprite_ID) / 2048, Y + Sin(Radian(degree)) * SpriteHeight(sprite_ID) / 2048)
    EndIf
  EndProcedure
  
  Macro common_image_coloring
    change_red+change_brightness : change_green+change_brightness : change_blue+change_brightness
    
    change_contrast/100
    
    If Not mode
      If IsImage(destination_ID)
        Protected destination_image=1
      ElseIf IsSprite(destination_ID)
        Protected destination_sprite=1
      ElseIf IsGadget(destination_ID)
        Protected destination_canvas=1
      Else
        ProcedureReturn -11
      EndIf
    Else
      If mode=1
        destination_image=2
      ElseIf mode=2
        destination_sprite=2
      Else
        ProcedureReturn -8
      EndIf
    EndIf
    
    If IsImage(source_ID)
      If Not StartDrawing(ImageOutput(source_ID)) : ProcedureReturn -4 : EndIf
    ElseIf IsSprite(source_ID)
      If Not StartDrawing(SpriteOutput(source_ID)) : ProcedureReturn -19 : EndIf
    ElseIf IsGadget(source_ID)
      If Not StartDrawing(CanvasOutput(source_ID)) : ProcedureReturn -5 : EndIf
    Else
      ProcedureReturn -17
    EndIf
    
    DrawingMode(#PB_2DDrawing_AllChannels)
    
    width=OutputWidth()-1 : height=OutputHeight()-1
    
    Protected Dim points(width, height) 
    
    For i=0 To width : For ii=0 To height : points(i, ii)=Point(i,ii) : Next : Next
    StopDrawing()
    
    If destination_sprite
      If destination_sprite=2
        destination_ID=CreateSprite(#PB_Any, ImageWidth(source_ID), ImageHeight(source_ID), sprite_mode)
        If Not destination_ID : ProcedureReturn -16 : EndIf
      EndIf
      If Not StartDrawing(SpriteOutput(destination_ID)) : ProcedureReturn -19 : EndIf
    ElseIf destination_image
      If destination_image=2
        destination_ID=CreateImage(#PB_Any, ImageWidth(source_ID), ImageHeight(source_ID))
        If Not destination_ID
          ProcedureReturn -6
        EndIf
      EndIf
      If Not StartDrawing(ImageOutput(destination_ID)) : ProcedureReturn -4 : EndIf
    ElseIf destination_canvas
      If Not StartDrawing(CanvasOutput(destination_ID)) : ProcedureReturn -5 : EndIf
    EndIf
    
    DrawingMode(#PB_2DDrawing_AllChannels)
    
    Select mode_grey ; you can simple add schemes
      Case 1
        correct_red=0.4 : correct_green=0.3 : correct_blue=0.3 ; More neutral
      Case 2
        correct_red=0.3 : correct_green=0.53 : correct_blue=0.11 ; Gimp
      Case 3
        correct_red=0.299 : correct_green=0.587 : correct_blue=0.114
    EndSelect
  EndMacro
  
  Procedure Change_Color_BF(color,
                            change_red.f=0,
                            change_green.f=0,
                            change_blue.f=0, 
                            change_contrast.f=0,
                            change_brightness.f=0,
                            change_intensity.f=0,
                            mode_grey=0)
    
    Protected i, ii, width, height, red.f, green.f, blue.f, grey.f, correct_red.f, correct_green.f, correct_blue.f
    
    change_red+change_brightness : change_green+change_brightness : change_blue+change_brightness
    
    change_contrast/100
    
    Select mode_grey ; you can simple add schemes
      Case 1
        correct_red=0.4 : correct_green=0.3 : correct_blue=0.3 ; More neutral
      Case 2
        correct_red=0.3 : correct_green=0.53 : correct_blue=0.11 ; Gimp
      Case 3
        correct_red=0.299 : correct_green=0.587 : correct_blue=0.114
    EndSelect
    
    red=Red(color)
    green=Green(color)
    blue=Blue(color)
    
    If change_red Or change_green Or change_blue
      red=color_check_valid_range(red+change_red*2.55)
      green=color_check_valid_range(green+change_green*2.55)
      blue=color_check_valid_range(blue+change_blue*2.55)
    EndIf
    
    If change_contrast 
      red=color_check_valid_range(red+(red-128)*change_contrast)
      green=color_check_valid_range(green+(green-128)*change_contrast)
      blue=color_check_valid_range(blue+(blue-128)*change_contrast)
    EndIf
    
    If change_intensity
      red=color_check_valid_range(red+red*(change_intensity/100))
      green=color_check_valid_range(green+green*(change_intensity/100))
      blue=color_check_valid_range(blue+blue*(change_intensity/100))
    EndIf
    
    If mode_grey
      grey=color_check_valid_range((red+change_red)*correct_red+(green+change_green)*correct_green+(blue+change_blue)*correct_blue)
      red=grey : green=grey : blue=grey
    EndIf
    
    ProcedureReturn RGBA(red, green, blue, Alpha(color))
  EndProcedure
  
  Procedure Image_coloring_BF(source_ID, destination_ID,
                              change_red.f=0,
                              change_green.f=0,
                              change_blue.f=0, 
                              change_contrast.f=0,
                              change_brightness.f=0,
                              change_intensity.f=0,
                              mode_grey=0,
                              mode=0,
                              sprite_mode=0)
    
    If mode<0 Or mode>2 : mode=0 : EndIf
    
    Protected i, ii, width, height, red.f, green.f, blue.f, grey.f, correct_red.f, correct_green.f, correct_blue.f, color
    
    common_image_coloring
    
    For i=0 To width
      For ii=0 To height
        color=points(i, ii) 
        red=Red(color)
        green=Green(color)
        blue=Blue(color)
        
        If change_red Or change_green Or change_blue
          red=color_check_valid_range(red+change_red*2.55)
          green=color_check_valid_range(green+change_green*2.55)
          blue=color_check_valid_range(blue+change_blue*2.55)
        EndIf
        
        If change_contrast 
          red=color_check_valid_range(red+(red-128)*change_contrast)
          green=color_check_valid_range(green+(green-128)*change_contrast)
          blue=color_check_valid_range(blue+(blue-128)*change_contrast)
        EndIf
        
        If change_intensity
          red=color_check_valid_range(red+red*(change_intensity/100))
          green=color_check_valid_range(green+green*(change_intensity/100))
          blue=color_check_valid_range(blue+blue*(change_intensity/100))
        EndIf
        
        If mode_grey
          grey=color_check_valid_range((red+change_red)*correct_red+(green+change_green)*correct_green+(blue+change_blue)*correct_blue)
          red=grey : green=grey : blue=grey
        EndIf
        
        Plot(i, ii, RGBA(red, green, blue, Alpha(color)))
        
      Next
    Next
    
    StopDrawing()
    
    If destination_image>1 Or destination_sprite>1
      ProcedureReturn destination_ID
    EndIf
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure Image_coloring_partition(source_ID, destination_ID,
                                     x=0,
                                     y=0,
                                     output_width=0,
                                     output_height=0,
                                     swap_output=0,     
                                     change_red.f=0,
                                     change_green.f=0,
                                     change_blue.f=0, 
                                     change_contrast.f=0,
                                     change_brightness.f=0,
                                     change_intensity.f=0,
                                     mode_grey=0,
                                     mode=0,
                                     sprite_mode=0)
    
    If mode<0 Or mode>2 : mode=0 : EndIf
    
    Protected i, ii, width, height, red.f, green.f, blue.f, grey.f, correct_red.f, correct_green.f, correct_blue.f, color, old_color
    
    common_image_coloring
    
    For i=0 To width
      For ii=0 To height
        color=points(i, ii)
        old_color=color
        red=Red(color)
        green=Green(color)
        blue=Blue(color)
        
        If change_red Or change_green Or change_blue
          red=color_check_valid_range(red+change_red*2.55)
          green=color_check_valid_range(green+change_green*2.55)
          blue=color_check_valid_range(blue+change_blue*2.55)
        EndIf
        
        If change_contrast 
          red=color_check_valid_range(red+(red-128)*change_contrast)
          green=color_check_valid_range(green+(green-128)*change_contrast)
          blue=color_check_valid_range(blue+(blue-128)*change_contrast)
        EndIf
        
        If change_intensity
          red=color_check_valid_range(red+red*(change_intensity/100))
          green=color_check_valid_range(green+green*(change_intensity/100))
          blue=color_check_valid_range(blue+blue*(change_intensity/100))
        EndIf
        
        If mode_grey
          grey=color_check_valid_range((red+change_red)*correct_red+(green+change_green)*correct_green+(blue+change_blue)*correct_blue)
          red=grey : green=grey : blue=grey
        EndIf
        
        color=RGBA(red, green, blue, Alpha(color))
        
        If swap_output
          If i=>x And i<x+output_width And ii=>y And ii<y+output_height
            color=old_color
          EndIf
        Else
          If i<x Or ii<y Or i>x+output_width Or ii>y+output_height
            color=old_color
          EndIf
        EndIf 
        
        Plot(i, ii, color)
        
      Next
    Next
    
    StopDrawing()
    
    If destination_image>1 Or destination_sprite>1
      ProcedureReturn destination_ID
    EndIf
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure Image_coloring_partition_filter_callback(source_ID, destination_ID,
                                                     x=0,
                                                     y=0,
                                                     output_width=0,
                                                     output_height=0,
                                                     swap_output=0,
                                                     change_red.f=0,
                                                     change_green.f=0,
                                                     change_blue.f=0, 
                                                     change_contrast.f=0,
                                                     change_brightness.f=0,
                                                     change_intensity.f=0,
                                                     mode_grey=0,
                                                     mode=0,
                                                     sprite_mode=0)
    
    If mode<0 Or mode>2 : mode=0 : EndIf
    
    Protected red.f, green.f, blue.f, grey.f, correct_red.f, correct_green.f, correct_blue.f
    Protected temporary_image_ID, color
    
    change_red+change_brightness : change_green+change_brightness : change_blue+change_brightness
    
    change_contrast/100
    
    partition_x=x
    partition_y=y
    swap_output_=swap_output
    change_red_.f=change_red
    change_green_.f=change_green
    change_blue_.f=change_blue
    change_contrast_.f=change_contrast
    change_brightness_.f=change_brightness
    change_intensity_.f=change_intensity
    mode_grey_=mode_grey
    
    If Not mode
      If IsImage(destination_ID)
        Protected destination_image=1
      ElseIf IsSprite(destination_ID)
        Protected destination_sprite=1
      ElseIf IsGadget(destination_ID)
        Protected destination_canvas=1
      Else
        ProcedureReturn -11
      EndIf
    Else
      If mode=1
        destination_image=2
      ElseIf mode=2
        destination_sprite=2
      Else
        ProcedureReturn -8
      EndIf
    EndIf
    
    If IsImage(source_ID)
      temporary_image_ID=CopyImage(source_ID, #PB_Any)
      If output_width<1 Or output_height<1
        output_width=(ImageWidth(source_ID))
        output_height=(ImageHeight(source_ID))
      EndIf
    ElseIf IsGadget(source_ID)
      temporary_image_ID=CreateImage_from_Canvas_BF(source_ID)
      If output_width<1 Or output_height<1
        output_width=GadgetWidth(source_ID)*DPI_ResizeFactor_BF()
        output_height=GadgetHeight(source_ID)*DPI_ResizeFactor_BF()
      EndIf
    Else
      ProcedureReturn -17
    EndIf
    
    partition_width=output_width
    partition_height=output_height
    
    If Not temporary_image_ID
      ProcedureReturn -6
    EndIf
    
    If destination_sprite
      If destination_sprite=2
        destination_ID=CreateSprite(#PB_Any, ImageWidth(source_ID), ImageHeight(source_ID), sprite_mode)
        If Not destination_ID : FreeImage(temporary_image_ID): ProcedureReturn -16 : EndIf
      EndIf
      If Not StartDrawing(SpriteOutput(destination_ID)) : FreeImage(temporary_image_ID) : ProcedureReturn -19 : EndIf
    ElseIf destination_image
      If destination_image=2
        destination_ID=CreateImage(#PB_Any, ImageWidth(source_ID), ImageHeight(source_ID))
        If Not destination_ID
          FreeImage(temporary_image_ID)
          ProcedureReturn -6
        EndIf
      EndIf
      If Not StartDrawing(ImageOutput(destination_ID)) : FreeImage(temporary_image_ID) : ProcedureReturn -4 : EndIf
    ElseIf destination_canvas
      If Not StartDrawing(CanvasOutput(destination_ID)) : FreeImage(temporary_image_ID) : ProcedureReturn -5 : EndIf
    EndIf
    
    Select mode_grey ; you can simple add schemes
      Case 1
        correct_red=0.4 : correct_green=0.3 : correct_blue=0.3 ; More neutral
      Case 2
        correct_red=0.3 : correct_green=0.53 : correct_blue=0.11 ; Gimp
      Case 3
        correct_red=0.299 : correct_green=0.587 : correct_blue=0.114
    EndSelect
    
    DrawingMode(#PB_2DDrawing_CustomFilter)
    
    If partition_x>0 Or partition_y>0 Or partition_width>0 Or partition_height>0
      CustomFilterCallback(@filter_callback_5())
    Else
      CustomFilterCallback(@filter_callback_6())
    EndIf
    
    If mode=2
      DrawImage(ImageID(temporary_image_ID), 0, 0)
    Else
      DrawImage(ImageID(source_ID), 0, 0)
    EndIf
    
    StopDrawing()
    
    FreeImage(temporary_image_ID)
    
    If destination_image>1 Or destination_sprite>1
      ProcedureReturn destination_ID
    EndIf
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure Image_coloring_partition_BF(source_ID, destination_ID,
                                        x=0,
                                        y=0,
                                        output_width=0,
                                        output_height=0,
                                        swap_output=0,     
                                        change_red.f=0,
                                        change_green.f=0,
                                        change_blue.f=0, 
                                        change_contrast.f=0,
                                        change_brightness.f=0,
                                        change_intensity.f=0,
                                        mode_grey=0,
                                        mode=0,
                                        sprite_mode=0)
    Protected result
    If IsSprite(source_ID) Or IsSprite(destination_ID)  
      
      result=Image_coloring_partition(source_ID, destination_ID,
                                      x,
                                      y,
                                      output_width,
                                      output_height,
                                      swap_output,     
                                      change_red.f,
                                      change_green.f,
                                      change_blue.f, 
                                      change_contrast.f,
                                      change_brightness.f,
                                      change_intensity.f,
                                      mode_grey,
                                      mode,
                                      sprite_mode)
    Else
      result=Image_coloring_partition_filter_callback(source_ID, destination_ID,
                                                      x,
                                                      y,
                                                      output_width,
                                                      output_height,
                                                      swap_output,     
                                                      change_red.f,
                                                      change_green.f,
                                                      change_blue.f, 
                                                      change_contrast.f,
                                                      change_brightness.f,
                                                      change_intensity.f,
                                                      mode_grey,
                                                      mode,
                                                      sprite_mode)
    EndIf
    
    ProcedureReturn result
    
  EndProcedure 
  
  Procedure CreateSpriteGradientColor_BF(texture_ID,
                                         image_get_color_x=0,
                                         image_get_color_y=0)
    
    If Not IsImage(texture_ID) : ProcedureReturn -2 : EndIf
    
    Protected image_width=ImageWidth(texture_ID), image_width_minus_1=image_width-1
    Protected image_height=ImageHeight(texture_ID), image_height_minus_1=image_height-1
    
    Protected texture_1_ID=CreateImage(#PB_Any, image_width, image_height)
    If Not texture_1_ID
      ProcedureReturn -6
    EndIf
    
    Protected intensity=255-gradient_color_alpha
    
    If intensity<0 : intensity=0 : EndIf    
    If intensity>255 : intensity=255 : EndIf
    
    Protected result, x, y, i
    Protected old_activate_sprite_array=activate_sprite_array
    Protected old_invisible_color=invisible_color
    Protected old_no_caching=no_caching
    Protected old_screen_output=screen_output
    
    activate_sprite_array=1
    no_caching=1
    screen_output=0
    
    If image_get_color_x<0
      invisible_color=image_get_color_y
      image_get_color_x=0
      image_get_color_y=0
    EndIf
    
    result=BF(1, texture_1_ID, texture_ID, image_get_color_x, image_get_color_y)
    If result<1 : FreeImage(texture_1_ID) : ProcedureReturn result : EndIf
    
    activate_sprite_array=old_activate_sprite_array
    invisible_color=old_invisible_color
    no_caching=old_no_caching
    screen_output=old_screen_output
    
    If Not StartDrawing(ImageOutput(texture_1_ID))
      FreeImage(texture_1_ID) : ProcedureReturn -4
    EndIf
    
    If Not ignore_gradient_colors And (activate_linear_gradient Or activate_elliptical_gradient Or activate_boxed_gradient Or activate_circular_gradient Or activate_conical_gradient)
      
      DrawingMode(#PB_2DDrawing_Gradient)
      If activate_backcolor
        BackColor(backcolor)
      EndIf
      If activate_frontcolor
        FrontColor(frontcolor)
      EndIf
      For i=0 To #Size_GradientColors_Array
        If gradient_activate_flag(i)
          GradientColor(gradient_position.f(i), gradient_color(i))
        EndIf
      Next
      
      If activate_linear_gradient
        LinearGradient(linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy)
      EndIf
      
      If activate_circular_gradient
        CircularGradient(circular_gradient_x, circular_gradient_y, circular_gradient_radius)
      EndIf
      
      If activate_elliptical_gradient
        EllipticalGradient(elliptical_gradient_x, elliptical_gradient_y, elliptical_gradient_xx, elliptical_gradient_yy)
      EndIf
      
      If activate_boxed_gradient
        BoxedGradient(boxed_gradient_x, boxed_gradient_y, boxed_gradient_width, boxed_gradient_height)
      EndIf
      
      If activate_conical_gradient
        ConicalGradient(conical_gradient_x, conical_gradient_y, conical_gradient_degree)
      EndIf
      
    EndIf
    
    Box(0, 0, image_width_minus_1, image_height_minus_1)
    
    DrawingMode(#PB_2DDrawing_Default)
    
    DrawAlphaImage(ImageID(texture_ID), 0, 0, intensity)
    
    For y=0 To image_height_minus_1 ; Read the BF SpriteArray
      For x=0 To image_width_minus_1
        If Not GetSpriteArray_Point_BF(x, y)
          Plot(x, y, used_invisible_color)
        EndIf
      Next x
    Next y
    StopDrawing()
    
    If gradient_color_red Or gradient_color_green Or gradient_color_blue Or gradient_color_contrast Or gradient_color_brightness Or gradient_color_intensity
      Image_coloring_BF(texture_1_ID, texture_1_ID, ; Post processing - Optimize sprite
                        gradient_color_red,
                        gradient_color_green,
                        gradient_color_blue,
                        gradient_color_contrast,
                        gradient_color_brightness,
                        gradient_color_intensity)
    EndIf
    
    ProcedureReturn texture_1_ID
  EndProcedure
  
  ; - Exif reading ------------------------------------------------------------------------
  Procedure GetRotationExif_BF(path$)
    ; Based by code from Sparkie
    Protected image_ID=ReadFile(#PB_Any, path$)
    If image_ID
      ;--> Byte 0 of Exif begins after JPEG header
      FileSeek(image_ID, 12)
      ;--> Bytes 0-1 is word order 18761 ($4949) is Intel and 19789 ($4D4D) is Motorola
      Protected byteOrder.i=ReadWord(image_ID)
      ;--> For now I only handle Little Endian
      If byteOrder=$4949
        ;--> Bytes 2-3 is TIFF format, it's always 42 ($2A). If not, give up.
        Protected tifFormat.i=ReadWord(image_ID)
        ;--> This is always $2A. If not, give up.
        If tifFormat=$2A
          ;--> Bytes 4-7 is starting offset for IFD (Image File Directory)
          Protected ifd1.i=ReadLong(image_ID)
          ;--> Move to start of IFD
          FileSeek(image_ID, ifd1 + 12)
          ;--> First 2 bytes of IFD is number of field entries
          Protected nFields.i=ReadWord(image_ID)
          ;--> Loop through all fields to find Date/Time stamp
          Protected i.i
          For i=1 To nFields
            ;--> Bytes 0-1 contain the Tag for the field.
            Protected currentTag.i=ReadWord(image_ID) & $FFFF
            Select currentTag
              Case 274 ; orientation
                FileSeek(image_ID, Loc(image_ID) + 6)
                Protected orientation.i=ReadWord(image_ID)
                ReadWord(image_ID)
              Default
                Protected num_champ.i
                num_champ + 1
                ;--> Move to next field. Each field is 12 bytes.
                ;--> currentTag (2 bytes) is current Loc(0) so we add 10
                FileSeek(image_ID, Loc(image_ID) + 10)
            EndSelect
          Next i
          Protected.i exifResult=orientation
        Else
          ;--> Wrong format, display Unavailable
          exifResult=0
        EndIf
      Else
        ;--> Wrong byte order, display Unavailable
        exifResult=0
      EndIf
      CloseFile(image_ID)
    Else
      ;--> File not readable
      exifResult=-21
    EndIf
    ProcedureReturn exifResult
  EndProcedure
  
  Procedure LoadPhoto_BF(path$)
    Protected degree, exif_result
    
    exif_result=GetRotationExif_BF(path$)
    
    If exif_result<0 : ProcedureReturn exif_result : EndIf
    
    Protected image_ID=LoadImage(#PB_Any, path$)
    
    If exif_result=6
      RotateImage_BF(90, image_ID)
    ElseIf exif_result=8
      RotateImage_BF(-90, image_ID)
    EndIf
    If Not IsImage(image_ID)
      ProcedureReturn -6
    EndIf
    ProcedureReturn image_ID
  EndProcedure
  
  Macro common_fit_image
    Protected.f shrink_percent_x, shrink_percent_y
    Protected output_ID, image_1_ID, i, ii
    
    If activate_dpi_aware
      frame_size*DPI_ResizeFactor_BF()
    EndIf
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Linux
      If new_fit_width<16 : new_fit_width=16 : EndIf
      If new_fit_height<16 : new_fit_height=16 : EndIf
    CompilerElse
      If new_fit_width<1 : new_fit_width=1 : EndIf
      If new_fit_height<1 : new_fit_height=1 : EndIf
    CompilerEndIf
    
    If Not IsImage(image_ID) : ProcedureReturn -2 : EndIf
    
    Protected image_width=ImageWidth(image_ID)
    Protected image_height=ImageHeight(image_ID)
    
    If Not fit_image_upscaling
      If image_width<new_fit_width
        new_x=new_fit_width/2-image_width/2
        new_fit_width=image_width
      EndIf
      If image_height<new_fit_height
        new_y=new_fit_height/2-image_height/2
        new_fit_height=image_height
      EndIf
    EndIf
    
    If image_width*new_fit_width>0
      shrink_percent_x=100/image_width*new_fit_width
    Else
      shrink_percent_x=100
    EndIf
    
    If image_height*new_fit_height>0
      shrink_percent_y=100/image_height*new_fit_height
    Else
      shrink_percent_y=100
    EndIf
    
    If shrink_percent_x<shrink_percent_y
      shrinked_width=image_width/100*shrink_percent_x
      shrinked_height=image_height/100*shrink_percent_x 
    Else
      shrinked_width=image_width/100*shrink_percent_y
      shrinked_height=image_height/100*shrink_percent_y
    EndIf
    
    If shrinked_width<fit_width
      new_x=fit_width/2-shrinked_width/2-1
    Else
      new_x=0
    EndIf
    If shrinked_height<fit_height
      new_y=fit_height/2-shrinked_height/2-1
    Else
      new_y=0
    EndIf
    
    ; Fuse for resizing functions
    If shrinked_width<1 : shrinked_width=1 : EndIf
    If shrinked_height<1 : shrinked_height=1 : EndIf
    
    If mode=3 : ProcedureReturn 1 : EndIf
    
    If mode=1    
      output_ID=CopyImage(image_ID, #PB_Any)
      If Not output_ID
        ProcedureReturn -6
      EndIf
      If Not ResizeImage(output_ID, shrinked_width, shrinked_height, resize_interpolation)
        FreeImage(output_ID)
        ProcedureReturn -27
      EndIf
      ProcedureReturn output_ID
    EndIf
    
    If mode=4
      output_ID=CopyImage(image_ID, #PB_Any)
      If Not output_ID
        ProcedureReturn -6
      EndIf 
      Protected output_1_ID=CreateImage(#PB_Any, fit_width-2, fit_height-2, 32, #PB_Image_Transparent)
      If Not output_1_ID
        FreeImage(output_ID)
        ProcedureReturn -6
      EndIf
      If Not ResizeImage(output_ID, shrinked_width, shrinked_height, resize_interpolation)
        FreeImage(output_ID)
        FreeImage(output_1_ID)
        ProcedureReturn -27
      EndIf
      If Not StartVectorDrawing(ImageVectorOutput(output_1_ID))
        ProcedureReturn -4
      EndIf
      
      If degree_stretchtext And Not background_color ; Simple blackmask rotating fuse
        frame_color_stretchtext=4
        If frame_size_stretchtext<2 
          frame_size_stretchtext=1
        EndIf
      EndIf
      
      If activate_dpi_aware
        Protected result=RoundBox_Vector_RAW_BF(output_1_ID,
                                                0,
                                                0,
                                                fit_width-2,
                                                fit_height-2,
                                                grid_size*DPI_ResizeFactor_BF(),              ; Set on special mode 4 edge rounding radius
                                                frame_size_stretchtext*DPI_ResizeFactor_BF(), ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                                                frame_color_stretchtext,                      ; Frame color
                                                background_color)                             ; Background color - For invisible background set to -1
      Else
        result=RoundBox_Vector_RAW_BF(output_1_ID,
                                      0,
                                      0,
                                      fit_width-2,
                                      fit_height-2,
                                      grid_size,               ; Set on special mode 4 edge rounding radius
                                      frame_size_stretchtext,  ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                                      frame_color_stretchtext, ; Frame color
                                      background_color)        ; Background color - For invisible background set to -1
      EndIf
      If result<1
        FreeImage(output_ID)
        FreeImage(output_1_ID)
        ProcedureReturn result
      EndIf
      
      MovePathCursor(((fit_width-2)/2-shrinked_width/2)+stretchtext_pro_offset_x, ((fit_height-2)/2-shrinked_height/2)+stretchtext_pro_offset_y)
      DrawVectorImage(ImageID(output_ID))
      
      StopVectorDrawing()
      FreeImage(output_ID)
      ProcedureReturn output_1_ID  
    EndIf  
    
    If mode=2
      output_ID=CreateSprite(#PB_Any, fit_width-2, fit_height-2, sprite_mode)
      If Not output_ID : ProcedureReturn -16 : EndIf
      StartDrawing(SpriteOutput(output_ID))
      Box(0, 0, fit_width-2, fit_height-2, $FF)
    Else
      If alpha_sprite
        output_ID=CreateImage(#PB_Any, fit_width-2, fit_height-2, 32, #PB_Image_Transparent)
      Else
        output_ID=CreateImage(#PB_Any, fit_width-2, fit_height-2, 24, background_color)
      EndIf
      If Not output_ID
        ProcedureReturn -6
      EndIf
      StartDrawing(ImageOutput(output_ID))
    EndIf
    If grid_size>0
      For i=0 To fit_width-2
        If grid_tiling And ii>grid_tiling
          LineXY(i, 0, i, fit_height, grid_color)
          i+2*DPI_ResizeFactor_BF() : ii=0
        EndIf
        LineXY(i, 0, i, fit_height, grid_color)
        i+grid_size*DPI_ResizeFactor_BF() : ii+1
      Next i
      ii=0
      For i=0 To fit_height-2
        If grid_tiling And ii>grid_tiling
          LineXY(0, i, fit_width, i, grid_color)
          i+2*DPI_ResizeFactor_BF() : ii=0
        EndIf
        LineXY(0, i, fit_width, i, grid_color)
        i+grid_size*DPI_ResizeFactor_BF() : ii+1
      Next i
    EndIf
    DrawingMode(#PB_2DDrawing_Outlined)
    Box(-1, -1, fit_width, fit_height, grid_color)
    For i=0 To frame_size-1
      Box(i, i, fit_width-i*2-2, fit_height-i*2-2, frame_color)
    Next i
    image_1_ID=CopyImage(image_ID, #PB_Any)
    If Not image_1_ID
      StopDrawing()
      If mode=2
        FreeSprite(output_ID)
        ProcedureReturn -6
      EndIf
      FreeImage(output_ID) : ProcedureReturn -6
    EndIf
    If Not ResizeImage(image_1_ID, shrinked_width, shrinked_height, resize_interpolation)
      FreeImage(image_1_ID)
      ProcedureReturn -27
    EndIf
    DrawAlphaImage(ImageID(Image_1_ID), new_x+shrinked_width/100*fit_image_offset_x, new_y+shrinked_height/100*fit_image_offset_y)
    FreeImage(image_1_ID)
    
    If add_fit_image_overlay_color
      Protected overlay_alpha, fit_width_=fit_width, fit_height_=fit_height
      overlay_alpha=255-fit_image_overlay_alpha
      If overlay_alpha<0 : overlay_alpha=0 : EndIf    
      If overlay_alpha>255 : overlay_alpha=255 : EndIf
      If fit_image_overlay_width>-1 And fit_image_overlay_height>-1
        fit_width_=fit_image_overlay_width
        fit_height_=fit_image_overlay_height
      EndIf
      DrawingMode(#PB_2DDrawing_AlphaBlend)
      Box(fit_image_overlay_x, fit_image_overlay_y, fit_width_, fit_height_, RGBA(Red(fit_image_overlay_color),
                                                                                  Green(fit_image_overlay_color),
                                                                                  Blue(fit_image_overlay_color),
                                                                                  overlay_alpha))
    EndIf
    
    If add_fit_image_overlay_frame
      If fit_image_overlay_frame_thickness>0
        fit_width_=fit_width : fit_height_=fit_height
        overlay_alpha=255-fit_image_overlay_frame_alpha
        If overlay_alpha<0 : overlay_alpha=0 : EndIf    
        If overlay_alpha>255 : overlay_alpha=255 : EndIf
        If fit_image_overlay_frame_width>-1 And fit_image_overlay_frame_height>-1
          fit_width_=fit_image_overlay_frame_width
          fit_height_=fit_image_overlay_frame_height
        EndIf
        DrawingMode(#PB_2DDrawing_AlphaBlend|#PB_2DDrawing_Outlined)
        For i=0 To fit_image_overlay_frame_thickness-1
          Box(fit_image_overlay_frame_x+i, fit_image_overlay_frame_y+i, fit_width_-i-i, fit_height_-i-i, RGBA(Red(fit_image_overlay_frame_color),
                                                                                                              Green(fit_image_overlay_frame_color),
                                                                                                              Blue(fit_image_overlay_frame_color),
                                                                                                              overlay_alpha))
        Next i
      EndIf
    EndIf
    
    StopDrawing()
    ProcedureReturn output_ID 
  EndMacro 
  
  Procedure Draw_Rectangle_Image_BF(box_ID, output_ID, offset_x=0, offset_y=0, offset_width=0, offset_height=0)
    from_rectangle_image=1
    Protected get_rectangle=Get_Rectangle_BF(box_ID)
    from_rectangle_image=0
    If  get_rectangle ; Get the rectangle values
      ProcedureReturn SpriteSimple_fast_BF(0, output_ID, get_rectangle,
                                           0, ; Grab transparence color from sprite pos x - For preset set ..x=-1  
                                           0, ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF  
                                           Get_Rectangle_x_BF()+offset_x, ; Output position x                      
                                           Get_Rectangle_y_BF()+offset_y, ; Output position y
                                           Get_Rectangle_width_BF()+offset_width, ; Output width
                                           Get_Rectangle_height_BF()+offset_height) ; Output height
    EndIf
  EndProcedure
  
  Procedure Grab_Rectangle_BF(input_ID, box_x, box_y, box_width, box_height, grab_rectangle_image=0, offset_x=0, offset_y=0, offset_width=0, offset_height=0)
    Protected input_width, input_height, box_background_image_ID, temporary_image_ID
    
    If grab_rectangle_image
      If screen_output
        input_width=ScreenWidth() : input_height=ScreenHeight()
      Else
        If IsImage(input_ID) ; check for image
          input_width=ImageWidth(input_ID) : input_height=ImageHeight(input_ID)
        ElseIf IsSprite(input_ID) ; Check for PB sprite
          input_width=SpriteWidth(input_ID) : input_height=SpriteHeight(input_ID)
        ElseIf IsGadget(input_ID) ; Check for canvas
          input_width=GadgetWidth(input_ID)*DPI_ResizeFactor_BF() : input_height=GadgetHeight(input_ID)*DPI_ResizeFactor_BF()
        Else
          ProcedureReturn 0
        EndIf
      EndIf
      If Not input_width Or Not input_height : ProcedureReturn 0 : EndIf
    EndIf
    
    If activate_dpi_aware
      box_x*DPI_ResizeFactor_BF()
      box_y*DPI_ResizeFactor_BF()
      box_width*DPI_ResizeFactor_BF()
      box_height*DPI_ResizeFactor_BF()
      offset_x*DPI_ResizeFactor_BF()
      offset_y*DPI_ResizeFactor_BF()
      input_width*DPI_ResizeFactor_BF()
      input_height*DPI_ResizeFactor_BF()
    EndIf
    
    Protected output_x=box_x+offset_x
    Protected output_y=box_y+offset_y
    
    Protected rectangle_width=box_width+offset_width
    Protected rectangle_height=box_height+offset_height
    
    If grab_rectangle_image 
      
      If output_x<0
        rectangle_width+output_x
        output_x=0
        If rectangle_width<0 : rectangle_width=0 : EndIf
      EndIf
      
      If output_y<0
        rectangle_height+output_y
        output_y=0
        If rectangle_height<0 : rectangle_height=0 : EndIf
      EndIf
      
      If output_x+rectangle_width>input_width
        rectangle_width-(output_x+rectangle_width-input_width)
        If rectangle_width<0 : rectangle_width=0 : EndIf
      EndIf
      
      If output_y+rectangle_height>input_height
        rectangle_height-(output_y+rectangle_height-input_height)
        If rectangle_height<0 : rectangle_height=0 : EndIf
      EndIf
      
      If screen_output
        If Not StartDrawing(ScreenOutput()) : ProcedureReturn 0 : EndIf
        box_background_image_ID=GrabDrawingImage(#PB_Any, output_x, output_y, rectangle_width, rectangle_height)
        If Not box_background_image_ID : ProcedureReturn 0 : EndIf
        StopDrawing()
      Else
        If Not IsImage(input_ID)
          temporary_image_ID=CreateImage(#PB_Any, input_width, input_height)
          If Not temporary_image_ID : ProcedureReturn 0 : EndIf
          CopyContent_BF(input_ID, temporary_image_ID)
          box_background_image_ID=GrabImage(temporary_image_ID, #PB_Any, output_x, output_y, rectangle_width, rectangle_height)
          FreeImage(temporary_image_ID)
        Else
          box_background_image_ID=GrabImage(input_ID, #PB_Any, output_x, output_y, rectangle_width, rectangle_height)
        EndIf
      EndIf
    EndIf
    
    AddElement(grab_box_bf())
    grab_box_bf()\box_x=output_x
    grab_box_bf()\box_y=output_y
    grab_box_bf()\box_width=rectangle_width
    grab_box_bf()\box_height=rectangle_height
    If grab_rectangle_image 
      grab_box_bf()\box_background_image_ID=box_background_image_ID
    EndIf
    ProcedureReturn @grab_box_bf()\box_x ; Create and give back a box ID
  EndProcedure
  
  Procedure FreeRectangles_BF(mode=0)
    Protected i, result, drawing_buffer_len, amount_used_images
    If Not mode
      ForEach(grab_box_bf())
        If IsImage(grab_box_bf()\box_background_image_ID)
          FreeImage(grab_box_bf()\box_background_image_ID)
        EndIf
      Next
      ClearList(grab_box_bf())
      ProcedureReturn 1
    ElseIf mode=1
      ProcedureReturn ListSize(grab_box_bf())
    ElseIf mode=2
      ForEach(grab_box_bf()) 
        If IsImage(grab_box_bf()\box_background_image_ID)
          If Not StartDrawing(ImageOutput(grab_box_bf()\box_background_image_ID))
            ProcedureReturn -12
          EndIf
          drawing_buffer_len=DrawingBufferPitch()*OutputHeight()
          StopDrawing()
        Else
          drawing_buffer_len=0
        EndIf
        result+drawing_buffer_len+SizeOf(integer)*4 ; Image size + variables size
      Next
      ProcedureReturn result
    ElseIf mode=3
      ForEach(grab_box_bf()) 
        If IsImage(grab_box_bf()\box_background_image_ID)
          i=1
        Else
          i=0
        EndIf
        result+i ; Found images
      Next
      ProcedureReturn result
    EndIf
  EndProcedure
  
  Procedure Free_selected_Rectangle_BF(Box_ID)
    ForEach(grab_box_bf())
      If @grab_box_bf()\box_x=box_ID 
        If IsImage(grab_box_bf()\box_background_image_ID)
          FreeImage(grab_box_bf()\box_background_image_ID)
        EndIf
        DeleteElement(grab_box_bf())
        ProcedureReturn 1
      EndIf
    Next
    ProcedureReturn 0
  EndProcedure
  
  Procedure Get_Rectangle_BF(box_ID)
    ForEach(grab_box_bf())
      If @grab_box_bf()\box_x=box_ID
        get_box_x_bf=grab_box_bf()\box_x
        get_box_y_bf=grab_box_bf()\box_y
        get_box_width_bf=grab_box_bf()\box_width
        get_box_height_bf=grab_box_bf()\box_height
        If Not from_rectangle_image
          get_box_data_bf=grab_box_bf()\box_data
          get_box_data_secound_bf=grab_box_bf()\box_data_secound
          get_box_data_third_bf=grab_box_bf()\box_data_third
          get_box_data_fourth_bf=grab_box_bf()\box_data_fourth
          get_box_data_double_bf.d=grab_box_bf()\box_data_double.d
          get_box_string_bf$=grab_box_bf()\box_string$
          get_box_string_secound_bf$=grab_box_bf()\box_string_secound$
        EndIf
        ProcedureReturn grab_box_bf()\box_background_image_ID
      EndIf
    Next
    get_box_x_bf=0
    get_box_y_bf=0
    get_box_width_bf=0
    get_box_height_bf=0
    If Not from_rectangle_image
      get_box_data_bf=0
      get_box_data_secound_bf=0
      get_box_data_third_bf=0
      get_box_data_fourth_bf=0
      get_box_data_double_bf.d=0
      get_box_string_bf$=""
      get_box_string_secound_bf$=""
    EndIf
    ProcedureReturn 0
  EndProcedure
  
  Procedure Get_Rectangle_ID_BF(x, y)
    If LastElement(grab_box_bf()) ; Read list inverse - For better handling child rectangles
      Repeat
        If x=>grab_box_bf()\box_x And
           y=>grab_box_bf()\box_y And
           x<=grab_box_bf()\box_x+grab_box_bf()\box_width And
           y<=grab_box_bf()\box_y+grab_box_bf()\box_height
          ProcedureReturn @grab_box_bf()\box_x
        EndIf
      Until Not PreviousElement(grab_box_bf())
    EndIf
    ProcedureReturn 0
  EndProcedure
  
  Procedure Get_Rectangle_ImageID_BF(box_ID)
    ForEach grab_box_bf()
      If @grab_box_bf()\box_x=box_ID And
         IsImage(grab_box_bf()\box_background_image_ID)
        get_box_x_bf=grab_box_bf()\box_x
        get_box_y_bf=grab_box_bf()\box_y
        get_box_width_bf=grab_box_bf()\box_width
        get_box_height_bf=grab_box_bf()\box_height
        ProcedureReturn grab_box_bf()\box_background_image_ID
      EndIf
    Next
    get_box_x_bf=0
    get_box_y_bf=0
    get_box_width_bf=0
    get_box_height_bf=0
    ProcedureReturn 0
  EndProcedure
  
  Procedure Get_Rectangle_x_BF()
    ProcedureReturn get_box_x_bf
  EndProcedure
  
  Procedure Get_Rectangle_y_BF()
    ProcedureReturn get_box_y_bf
  EndProcedure
  
  Procedure Get_Rectangle_width_BF()
    ProcedureReturn get_box_width_bf
  EndProcedure
  
  Procedure Get_Rectangle_height_BF()
    ProcedureReturn get_box_height_bf
  EndProcedure
  
  Procedure Get_Rectangle_data_BF()
    ProcedureReturn get_box_data_bf
  EndProcedure
  
  Procedure Get_Rectangle_data_secound_BF()
    ProcedureReturn get_box_data_secound_bf
  EndProcedure
  
  Procedure Get_Rectangle_data_third_BF()
    ProcedureReturn get_box_data_third_bf
  EndProcedure
  
  Procedure Get_Rectangle_data_fourth_BF()
    ProcedureReturn get_box_data_fourth_bf
  EndProcedure
  
  Procedure.d Get_Rectangle_data_double_BF()
    ProcedureReturn get_box_data_double_bf.d
  EndProcedure
  
  Procedure$ Get_Rectangle_string_BF()
    ProcedureReturn get_box_string_bf$
  EndProcedure
  
  Procedure$ Get_Rectangle_string_secound_BF()
    ProcedureReturn get_box_string_secound_bf$
  EndProcedure
  
  Procedure Set_Rectangle_x_BF(get_box_x_bf_)
    get_box_x_bf=get_box_x_bf_
    ProcedureReturn get_box_x_bf
  EndProcedure
  
  Procedure Set_Rectangle_y_BF(get_box_y_bf_)
    get_box_y_bf=get_box_y_bf_
    ProcedureReturn get_box_y_bf
  EndProcedure
  
  Procedure Set_Rectangle_width_BF(get_box_width_bf_)
    get_box_width_bf=get_box_width_bf_
    ProcedureReturn get_box_width_bf
  EndProcedure
  
  Procedure Set_Rectangle_height_BF(get_box_height_bf_)
    get_box_height_bf=get_box_height_bf_
    ProcedureReturn get_box_height_bf
  EndProcedure
  
  Procedure Set_Rectangle_data_BF(box_ID, box_data_)
    ForEach(grab_box_bf())
      If @grab_box_bf()\box_x=box_ID 
        grab_box_bf()\box_data=box_data_
        ProcedureReturn 1
      EndIf
    Next
    ProcedureReturn 0 
  EndProcedure
  
  Procedure Set_Rectangle_data_secound_BF(box_ID, box_data_secound_)
    ForEach(grab_box_bf())
      If @grab_box_bf()\box_x=box_ID 
        grab_box_bf()\box_data_secound=box_data_secound_
        ProcedureReturn 1
      EndIf
    Next
    ProcedureReturn 0 
  EndProcedure
  
  Procedure Set_Rectangle_data_third_BF(box_ID, box_data_third_)
    ForEach(grab_box_bf())
      If @grab_box_bf()\box_x=box_ID 
        grab_box_bf()\box_data_third=box_data_third_
        ProcedureReturn 1
      EndIf
    Next
    ProcedureReturn 0 
  EndProcedure
  
  Procedure Set_Rectangle_data_fourth_BF(box_ID, box_data_fourth_)
    ForEach(grab_box_bf())
      If @grab_box_bf()\box_x=box_ID 
        grab_box_bf()\box_data_fourth=box_data_fourth_
        ProcedureReturn 1
      EndIf
    Next
    ProcedureReturn 0 
  EndProcedure
  
  Procedure Set_Rectangle_data_double_BF(box_ID, box_data_double_.d)
    ForEach(grab_box_bf())
      If @grab_box_bf()\box_x=box_ID 
        grab_box_bf()\box_data_double.d=box_data_double_.d
        ProcedureReturn 1
      EndIf
    Next
    ProcedureReturn 0 
  EndProcedure
  
  Procedure Set_Rectangle_string_BF(box_ID, box_string_$)
    ForEach(grab_box_bf())
      If @grab_box_bf()\box_x=box_ID 
        grab_box_bf()\box_string$=box_string_$
        ProcedureReturn 1
      EndIf
    Next
    ProcedureReturn 0 
  EndProcedure
  
  Procedure Set_Rectangle_string_secound_BF(box_ID, box_string_secound_$)
    ForEach(grab_box_bf())
      If @grab_box_bf()\box_x=box_ID 
        grab_box_bf()\box_string_secound$=box_string_secound_$
        ProcedureReturn 1
      EndIf
    Next
    ProcedureReturn 0 
  EndProcedure 
  
  Procedure FitImage_StretchText(image_ID,
                                 fit_width,
                                 fit_height,
                                 background_color=$DCDCDC,
                                 padding_x=0, ; Padding x
                                 padding_y=0, ; Padding y
                                 edge_rounding=0,
                                 frame_color=$FFFFFF,
                                 frame_size=3,
                                 sprite_mode=0)
    
    Protected mode=4, grid_color, grid_tiling, grid_size=edge_rounding
    
    grid_tiling-1
    
    If (fit_width-2)<1 : fit_width=3 : EndIf
    If (fit_height-2)<1 : fit_height=3 : EndIf 
    
    padding_x+fit_image_padding_offset_x+2
    padding_x+padding_x % 2
    
    padding_y+fit_image_padding_offset_y+2
    padding_y+padding_y % 2
    
    If padding_x<0 : padding_x=0 : EndIf
    If padding_y<0 : padding_y=0 : EndIf
    
    Protected new_fit_width=fit_width-padding_x
    Protected new_fit_height=fit_height-padding_y
    
    common_fit_image
  EndProcedure
  
  Procedure FitImage_BF(mode,
                        image_ID,
                        fit_width,
                        fit_height,
                        background_color=$DCDCDC,
                        padding=0,
                        grid_color=$FFFFFF,
                        grid_size=5,
                        grid_tiling=10,
                        frame_color=$FFFFFF,
                        frame_size=3,
                        sprite_mode=0)
    
    Protected padding_x=padding, padding_y=padding
    
    grid_tiling-1
    
    If (fit_width-2)<1 : fit_width=3 : EndIf
    If (fit_height-2)<1 : fit_height=3 : EndIf 
    
    padding_x+fit_image_padding_offset_x+2
    padding_x+padding_x % 2
    
    padding_y+fit_image_padding_offset_y+2
    padding_y+padding_y % 2
    
    If padding_x<0 : padding_x=0 : EndIf
    If padding_y<0 : padding_y=0 : EndIf
    
    Protected new_fit_width=fit_width-padding_x
    Protected new_fit_height=fit_height-padding_y
    
    common_fit_image
  EndProcedure
  
  Procedure FitImage_DualPadding_BF(mode,
                                    image_ID,
                                    fit_width,
                                    fit_height,
                                    background_color=$DCDCDC,
                                    padding_x=0,
                                    padding_y=0,
                                    grid_color=$FFFFFF,
                                    grid_size=5,
                                    grid_tiling=10,
                                    frame_color=$FFFFFF,
                                    frame_size=3,
                                    sprite_mode=0)
    
    grid_tiling-1
    
    If (fit_width-2)<1 : fit_width=3 : EndIf
    If (fit_height-2)<1 : fit_height=3 : EndIf 
    
    padding_x+fit_image_padding_offset_x+2
    padding_x+padding_x % 2
    
    padding_y+fit_image_padding_offset_y+2
    padding_y+padding_y % 2
    
    If padding_x<0 : padding_x=0 : EndIf
    If padding_y<0 : padding_y=0 : EndIf
    
    Protected new_fit_width=fit_width-padding_x
    Protected new_fit_height=fit_height-padding_y
    
    common_fit_Image
  EndProcedure
  
  Procedure FitImage_Offset_BF(fit_image_offset_x_.d, fit_image_offset_y_.d)
    fit_image_offset_x=fit_image_offset_x_
    fit_image_offset_y=fit_image_offset_y_
    ProcedureReturn 1
  EndProcedure
  
  Procedure FitImageActivateUpscaling_BF(fit_image_upscaling_)
    fit_image_upscaling=fit_image_upscaling_
  EndProcedure
  
  Procedure FitImageGetUpscalingState_BF()
    ProcedureReturn fit_image_upscaling
  EndProcedure
  
  Procedure GetFitSize_BF(source_width.d, source_height.d, fit_width.d, fit_height.d, image_mode=1)
    
    Protected.d shrink_percent_x, shrink_percent_y
    
    If source_width*fit_width>0
      shrink_percent_x=100/source_width*fit_width
    Else
      shrink_percent_x=100
    EndIf
    
    If source_height*fit_height>0
      shrink_percent_y=100/source_height*fit_height
    Else
      shrink_percent_y=100
    EndIf
    
    If shrink_percent_x<shrink_percent_y
      shrinked_width=source_width/100*shrink_percent_x
      shrinked_height=source_height/100*shrink_percent_x 
    Else
      shrinked_width=source_width/100*shrink_percent_y
      shrinked_height=source_height/100*shrink_percent_y
    EndIf
    
    If shrinked_width<fit_width
      new_x=fit_width/2-shrinked_width/2-1
    Else
      new_x=0
    EndIf
    If shrinked_height<fit_height
      new_y=fit_height/2-shrinked_height/2-1
    Else
      new_y=0
    EndIf
    
    If image_mode
      ; Fuse for resizing functions
      If shrinked_width<1 : shrinked_width=1 : EndIf
      If shrinked_height<1 : shrinked_height=1 : EndIf
    EndIf
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure.d GetFitOffset_X_BF() ; Give back the x offset for the FitImage_BF output
    ProcedureReturn new_x
  EndProcedure
  
  Procedure.d GetFitOffset_Y_BF() ; Give back the y offset for the FitImage_BF output
    ProcedureReturn new_y
  EndProcedure
  
  Procedure.d GetFitSize_Width_BF()
    ProcedureReturn shrinked_width
  EndProcedure
  
  Procedure.d GetFitSize_Height_BF()
    ProcedureReturn shrinked_height
  EndProcedure
  
  Procedure AddPathLine_BF(mode, output_ID, texture_ID,
                           x.d,                               ; Output x
                           y.d,                               ; Output y
                           xx.d,                              ; Output xx
                           yy.d,                              ; Output yy
                           stroke_path.d,                     ; Stroke path
                           stroke_path_flag=#PB_Path_Default, ; Stroke path flag - As sample #PB_Path_RoundEnd
                           texture_x=0,                       ; Startposition texture output x   - Preset 0
                           texture_y=0,                       ; Startposition texture output y   - Preset 0
                           texture_width=0,                   ; Endposition texture output xx    - Preset 0 = Full available width
                           texture_height=0,                  ; Endposition texture output yy    - Preset 0 = Full available height
                           texture_clip_x=0,                  ; Startposition inside the texture - Preset 0 (Texture clipping)
                           texture_clip_y=0,                  ; Startposition inside the texture - Preset 0 (Texture clipping)
                           texture_clip_xx=0,                 ; Endposition inside the texture   - Preset 0 (Texture clipping)
                           texture_clip_yy=0)                 ; Endposition inside the texture   - Preset 0 (Texture clipping)
    
    Protected mask_color, temporary_color
    
    If GetUniversalOutputMaskColor_BF()<>-1
      ; Mask color preset - Get firstly a unused color from the actual used texture
      ; For best results with vector graphics use this, harmonic to the output content colors - This fix ugly edges on vector outputs
      mask_color=GetUniversalOutputMaskColor_BF()
    Else  
      ; This is the automatic standard mask color generating function
      If mode>0 : mask_color=SearchUnusedImageColor_BF(texture_ID) : EndIf
    EndIf
    
    temporary_color=mask_color+1 ; Change this line not
    CreateImageUniversalOutput_BF(output_ID, mask_color) ; Change this line not
    
    If StartVectorDrawing(ImageVectorOutput(GetTempImageUniversalOutput_ID_BF())) ; Set here the PB graphic function for using - Set ever this defined color
      MovePathCursor(x, y)
      AddPathLine(xx ,yy)
      VectorSourceColor(temporary_color|$FF000000)
      StrokePath(stroke_path, stroke_path_flag)
      StopVectorDrawing()
    EndIf
    
    Protected result=Texture_universal_output_BF(mode, output_ID, texture_ID, temporary_color,
                                                 texture_x,
                                                 texture_y,
                                                 texture_width,
                                                 texture_height,
                                                 texture_clip_x,
                                                 texture_clip_y,
                                                 texture_clip_xx,
                                                 texture_clip_yy)
    FreeImage(GetTempImageUniversalOutput_ID_BF())
    ProcedureReturn result
  EndProcedure
  
  Procedure AddPathEllipse_BF(mode, output_ID, texture_ID,
                              x.d,               ; Output x
                              y.d,               ; Output y
                              radius_x.d,        ; Radius x
                              radius_y.d,        ; Radius y
                              starting_angle.d,  ; Starting angle
                              end_angle.d,       ; End angle
                              rotate_angle,      ; Rotate angle                             
                              texture_x=0,       ; Startposition texture output x   - Preset 0
                              texture_y=0,       ; Startposition texture output y   - Preset 0
                              texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                              texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                              texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                              texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                              texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                              texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
    
    Protected mask_color, temporary_color
    
    If GetUniversalOutputMaskColor_BF()<>-1
      ; Mask color preset - Get firstly a unused color from the actual used texture
      ; For best results with vector graphics use this, harmonic to the output content colors - This fix ugly edges on vector outputs
      mask_color=GetUniversalOutputMaskColor_BF()
    Else  
      ; This is the automatic standard mask color generating function
      If mode>0 : mask_color=SearchUnusedImageColor_BF(texture_ID) : EndIf
    EndIf
    
    temporary_color=mask_color+1 ; Change this line not
    CreateImageUniversalOutput_BF(output_ID, mask_color) ; Change this line not
    
    If StartVectorDrawing(ImageVectorOutput(GetTempImageUniversalOutput_ID_BF())) ; Set here the PB graphic function for using - Set ever this defined color
      VectorSourceColor(temporary_color|$FF000000)
      MovePathCursor(x, y)
      RotateCoordinates (x, y, rotate_angle)
      AddPathEllipse(x, y, radius_x, radius_y, starting_angle, end_angle, #PB_Path_Connected)
      FillPath()
      StopVectorDrawing()
    EndIf
    
    Protected result=Texture_universal_output_BF(mode, output_ID, texture_ID, temporary_color,
                                                 texture_x,
                                                 texture_y,
                                                 texture_width,
                                                 texture_height,
                                                 texture_clip_x,
                                                 texture_clip_y,
                                                 texture_clip_xx,
                                                 texture_clip_yy)
    FreeImage(GetTempImageUniversalOutput_ID_BF())
    ProcedureReturn result
  EndProcedure
  
  Procedure AddPathCircle_BF(mode, output_ID, texture_ID,
                             x.d,               ; Output x
                             y.d,               ; Output y
                             radius.d,          ; Radius
                             starting_angle.d,  ; Starting angle
                             end_angle.d,       ; End angle
                             texture_x=0,       ; Startposition texture output x   - Preset 0
                             texture_y=0,       ; Startposition texture output y   - Preset 0
                             texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                             texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                             texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                             texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                             texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                             texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
    
    Protected mask_color, temporary_color
    
    If GetUniversalOutputMaskColor_BF()<>-1
      ; Mask color preset - Get firstly a unused color from the actual used texture
      ; For best results with vector graphics use this, harmonic to the output content colors - This fix ugly edges on vector outputs
      mask_color=GetUniversalOutputMaskColor_BF()
    Else  
      ; This is the automatic standard mask color generating function
      If mode>0 : mask_color=SearchUnusedImageColor_BF(texture_ID) : EndIf
    EndIf
    
    temporary_color=mask_color+1 ; Change this line not  
    CreateImageUniversalOutput_BF(output_ID, mask_color) ; Change this line not
    
    If StartVectorDrawing(ImageVectorOutput(GetTempImageUniversalOutput_ID_BF())) ; Set here the PB graphic function for using - Set ever this defined color
      VectorSourceColor(temporary_color|$FF000000)
      MovePathCursor(x, y)
      AddPathCircle(x, y, radius, starting_angle, end_angle, #PB_Path_Connected)
      FillPath()
      StopVectorDrawing()
    EndIf
    
    Protected result=Texture_universal_output_BF(mode, output_ID, texture_ID, temporary_color,
                                                 texture_x,
                                                 texture_y,
                                                 texture_width,
                                                 texture_height,
                                                 texture_clip_x,
                                                 texture_clip_y,
                                                 texture_clip_xx,
                                                 texture_clip_yy)
    FreeImage(GetTempImageUniversalOutput_ID_BF())
    ProcedureReturn result
  EndProcedure
  
  Procedure Ellipse_BF(mode, output_ID, texture_ID,
                       x,                 ; Output x
                       y,                 ; Output y
                       radius_x,          ; Radius x
                       radius_y,          ; Radius y
                       texture_x=0,       ; Startposition texture output x   - Preset 0
                       texture_y=0,       ; Startposition texture output y   - Preset 0
                       texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                       texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                       texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                       texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                       texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                       texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
    
    Protected mask_color, temporary_color
    
    If GetUniversalOutputMaskColor_BF()<>-1
      ; Mask color preset - Get firstly a unused color from the actual used texture
      ; For best results with vector graphics use this, harmonic to the output content colors - This fix ugly edges on vector outputs
      mask_color=GetUniversalOutputMaskColor_BF()
    Else  
      ; This is the automatic standard mask color generating function
      If mode>0 : mask_color=SearchUnusedImageColor_BF(texture_ID) : EndIf
    EndIf
    
    temporary_color=mask_color+1 ; Change this line not  
    CreateImageUniversalOutput_BF(output_ID, mask_color) ; Change this line not
    
    If StartDrawing(ImageOutput(GetTempImageUniversalOutput_ID_BF())) ; Set here the PB graphic function for using - Set ever this defined color
      Ellipse(x, y, radius_x, radius_y, temporary_color)
      StopDrawing()
    EndIf
    
    Protected result=Texture_universal_output_BF(mode, output_ID, texture_ID, temporary_color,
                                                 texture_x,
                                                 texture_y,
                                                 texture_width,
                                                 texture_height,
                                                 texture_clip_x,
                                                 texture_clip_y,
                                                 texture_clip_xx,
                                                 texture_clip_yy)
    FreeImage(GetTempImageUniversalOutput_ID_BF())
    ProcedureReturn result
  EndProcedure
  
  Procedure RoundBox_BF(mode, output_ID, texture_ID,
                        x,                 ; Output x
                        y,                 ; Output y
                        width,             ; Width
                        height,            ; Height
                        radius_x,          ; radius_x
                        radius_y,          ; radius_y 
                        texture_x=0,       ; Startposition texture output x   - Preset 0
                        texture_y=0,       ; Startposition texture output y   - Preset 0
                        texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                        texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                        texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                        texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                        texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                        texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
    
    Protected mask_color, temporary_color
    
    If GetUniversalOutputMaskColor_BF()<>-1
      ; Mask color preset - Get firstly a unused color from the actual used texture
      ; For best results with vector graphics use this, harmonic to the output content colors - This fix ugly edges on vector outputs
      mask_color=GetUniversalOutputMaskColor_BF()
    Else  
      ; This is the automatic standard mask color generating function
      If mode>0 : mask_color=SearchUnusedImageColor_BF(texture_ID) : EndIf
    EndIf
    
    temporary_color=mask_color+1 ; Change this line not  
    CreateImageUniversalOutput_BF(output_ID, mask_color) ; Change this line not
    
    If StartDrawing(ImageOutput(GetTempImageUniversalOutput_ID_BF())) ; Set here the PB graphic function for using - Set ever this defined color
      RoundBox(x, y, width, height, radius_x, radius_y, temporary_color)
      StopDrawing()
    EndIf
    
    Protected result=Texture_universal_output_BF(mode, output_ID, texture_ID, temporary_color,
                                                 texture_x,
                                                 texture_y,
                                                 texture_width,
                                                 texture_height,
                                                 texture_clip_x,
                                                 texture_clip_y,
                                                 texture_clip_xx,
                                                 texture_clip_yy)
    FreeImage(GetTempImageUniversalOutput_ID_BF())
    ProcedureReturn result
  EndProcedure
  
  Procedure Box_BF(mode, output_ID, texture_ID,
                   x,                 ; Output x
                   y,                 ; Output y
                   width,             ; Width
                   height,            ; Height
                   texture_x=0,       ; Startposition texture output x   - Preset 0
                   texture_y=0,       ; Startposition texture output y   - Preset 0
                   texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                   texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                   texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                   texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                   texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                   texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
    
    Protected mask_color, temporary_color
    
    If GetUniversalOutputMaskColor_BF()<>-1
      ; Mask color preset - Get firstly a unused color from the actual used texture
      ; For best results with vector graphics use this, harmonic to the output content colors - This fix ugly edges on vector outputs
      mask_color=GetUniversalOutputMaskColor_BF()
    Else  
      ; This is the automatic standard mask color generating function
      If mode>0 : mask_color=SearchUnusedImageColor_BF(texture_ID) : EndIf
    EndIf
    
    temporary_color=mask_color+1 ; Change this line not  
    CreateImageUniversalOutput_BF(output_ID, mask_color) ; Change this line not
    
    If StartDrawing(ImageOutput(GetTempImageUniversalOutput_ID_BF())) ; Set here the PB graphic function for using - Set ever this defined color
      Box(x, y, width, height, temporary_color)
      StopDrawing()
    EndIf
    
    Protected result=Texture_universal_output_BF(mode, output_ID, texture_ID, temporary_color,
                                                 texture_x,
                                                 texture_y,
                                                 texture_width,
                                                 texture_height,
                                                 texture_clip_x,
                                                 texture_clip_y,
                                                 texture_clip_xx,
                                                 texture_clip_yy)
    FreeImage(GetTempImageUniversalOutput_ID_BF())
    ProcedureReturn result
  EndProcedure
  
  Procedure DrawRotatedText_BF(mode, output_ID, texture_ID, font_ID,
                               x,                 ; Output x
                               y,                 ; Output y
                               text$,             ; Text
                               degree,            ; Degree
                               texture_x=0,       ; Startposition texture output x   - Preset 0
                               texture_y=0,       ; Startposition texture output y   - Preset 0
                               texture_width=0,   ; Endposition texture output xx    - Preset 0 = Full available width
                               texture_height=0,  ; Endposition texture output yy    - Preset 0 = Full available height
                               texture_clip_x=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                               texture_clip_y=0,  ; Startposition inside the texture - Preset 0 (Texture clipping)
                               texture_clip_xx=0, ; Endposition inside the texture   - Preset 0 (Texture clipping)
                               texture_clip_yy=0) ; Endposition inside the texture   - Preset 0 (Texture clipping)
                                                  ; Ignore font = -1
    
    Protected mask_color, temporary_color
    
    If GetUniversalOutputMaskColor_BF()<>-1
      ; Mask color preset - Get firstly a unused color from the actual used texture
      ; For best results with vector graphics use this, harmonic to the output content colors - This fix ugly edges on vector outputs
      mask_color=GetUniversalOutputMaskColor_BF()
    Else  
      ; This is the automatic standard mask color generating function
      If mode>0 : mask_color=SearchUnusedImageColor_BF(texture_ID) : EndIf
    EndIf
    
    temporary_color=mask_color+1 ; Change this line not  
    CreateImageUniversalOutput_BF(output_ID, mask_color) ; Change this line not
    
    StartDrawing(ImageOutput(GetTempImageUniversalOutput_ID_BF())) ; Set here the PB graphic function for using - Set ever this defined color
    If IsFont(font_ID)
      DrawingFont(FontID(font_ID))
    EndIf
    DrawRotatedText(x, y, text$, degree, temporary_color)
    StopDrawing()
    
    Protected result=Texture_universal_output_BF(mode, output_ID, texture_ID, temporary_color,
                                                 texture_x,
                                                 texture_y,
                                                 texture_width,
                                                 texture_height,
                                                 texture_clip_x,
                                                 texture_clip_y,
                                                 texture_clip_xx,
                                                 texture_clip_yy)
    FreeImage(GetTempImageUniversalOutput_ID_BF())
    ProcedureReturn result
  EndProcedure
  
  
  Procedure ReadColorDistanceValue_BF(path$)
    Protected color_distance
    Protected file=ReadFile(#PB_Any, path$)
    If Not file
      ProcedureReturn -21
    EndIf
    FileSeek(file, Lof(file)-25)
    If ReadString(file, #PB_UTF8, 13)<>"ColorDistance"
      CloseFile(file)
      ProcedureReturn -21
    EndIf
    FileSeek(file, Lof(file)-5)
    color_distance=Val(ReadString(file, #PB_Ascii, 3)) ; Color distance
    CloseFile(file)
    ProcedureReturn color_distance
  EndProcedure
  
  Procedure WriteColorDistanceValue_BF(path$, color_distance)
    If color_distance<0 Or color_distance>256 ; Max is 255 but 256 is accepted
      ProcedureReturn -8
    EndIf
    
    If ReadColorDistanceValue_BF(path$)=>0 : ProcedureReturn 0 : EndIf
    
    Protected file=OpenFile(#PB_Any, path$)
    If file
      FileSeek(file, Lof(file))
    Else
      ProcedureReturn -21
    EndIf
    Protected result=WriteStringN(file, "   ColorDistance +"+RSet(Str(color_distance), 8, "0 "), #PB_Ascii)
    If Not result
      CloseFile(file)
      ProcedureReturn -22
    EndIf
    CloseFile(file)
    ProcedureReturn 1
  EndProcedure
  
  Procedure RemoveColorDistanceValue_BF(path$) 
    Protected file=OpenFile(#PB_Any, path$)
    If Not file
      ProcedureReturn -21
    EndIf
    FileSeek(file, Lof(file)-25)
    If ReadString(file, #PB_UTF8, 13)<>"ColorDistance"
      CloseFile(file)
      ProcedureReturn -21
    Else
      FileSeek(file, Lof(file)-28)
      TruncateFile(file)
      CloseFile(file)
    EndIf
    ProcedureReturn 1
  EndProcedure
  
  Macro calculate_CSS_sheet
    
    If Not IsImage(gif_ID) : ProcedureReturn -2 : EndIf
    
    gif_width=ImageWidth(gif_ID)
    gif_height=ImageHeight(gif_ID)
    gif_css_frame_width=gif_width
    gif_css_frame_height=gif_height
    frames=ImageFrameCount_BF(gif_ID)
    gif_css_frames=frames
    Dim gif_and_css_delay_array(gif_css_frames-1)
    If gif_height>gif_width And frames>1
      frames_x=Sqr(frames)+1
    Else
      frames_x=Sqr(frames)
    EndIf
    If Not frames_x : frames_x=1 : EndIf
    css_frames_in_a_row=frames_x
    frames_y=frames/frames_x+1
    sheet_width=frames_x*gif_width
    sheet_height=frames_y*gif_height
    For i=0 To frames-1
      iiii=iii : ii+gif_width
      If ii=>sheet_width
        ii=0 : iii+gif_height
      EndIf   
    Next i
    ii=0 : iii=0
    
    sheet_height=iiii+gif_height 
    
    css_width=sheet_width
    css_height=sheet_height
    
    If Not mode Or mode=2
      If alpha_sprite
        sheet_ID=CreateImage(#PB_Any, sheet_width, sheet_height, 32, #PB_Image_Transparent)
      Else
        sheet_ID=CreateImage(#PB_Any, sheet_width, sheet_height)
      EndIf
    EndIf
    
    If Not sheet_ID
      ProcedureReturn -6
    EndIf
    
    For i=0 To frames-1
      SetImageFrame_BF(gif_ID, i)
      gif_frame_delay=GetImageFrameDelay(gif_ID)
      If Not gif_frame_delay : gif_frame_delay=100 : EndIf
      gif_and_css_delay_array(i)=gif_frame_delay
      If Not mode Or mode=2
        If Not StartDrawing(ImageOutput(sheet_ID))
          FreeImage(sheet_ID)
          ProcedureReturn -4
        EndIf
        DrawAlphaImage(ImageID(gif_ID), ii, iii)
        StopDrawing()
      EndIf
      ii+gif_width
      If ii=>sheet_width
        ii=0 : iii+gif_height
      EndIf   
    Next i
    
    ii=0 : iii=0
    
  EndMacro
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure CreateCSSSheet_simple_BF(gif_ID, mode=0)
      Protected sheet_ID, i, ii, iii, iiii, frames, frames_x, frames_y, gif_frame_delay
      Protected gif_width, gif_height, sheet_width, sheet_height, invisible_gif_color
      
      calculate_CSS_sheet
      
      If mode=1 : ProcedureReturn 1 : EndIf
      
      invisible_gif_color=SearchUnusedColor_BF(sheet_ID, 0, 0, sheet_width, sheet_height+1)
      If invisible_gif_color<1 : invisible_gif_color=Random($FFFFFE, 1) : EndIf ; Fuse - Error handling for automatic mask color generating
      
      gif_css_invisible_color=invisible_gif_color
      
      If Not StartDrawing(ImageOutput(sheet_ID)) : FreeImage(sheet_ID) : ProcedureReturn -4 : EndIf
      Box(0, 0, sheet_width, sheet_height, invisible_gif_color)
      StopDrawing()
      For i=0 To frames-1
        SetImageFrame_BF(gif_ID, i)
        If Not StartDrawing(ImageOutput(sheet_ID)) : FreeImage(sheet_ID) : ProcedureReturn -4 : EndIf
        DrawAlphaImage(ImageID(gif_ID), ii, iii)
        StopDrawing()
        ii+gif_width
        If ii=>sheet_width
          ii=0 : iii+gif_height
        EndIf   
      Next i
      
      If alpha_sprite
        sheet_ID=CreateAlphaImage_from_Sprite_BF(sheet_ID,
                                                 0,
                                                 invisible_gif_color,
                                                 0,
                                                 1)
      EndIf
      
      If mode=2 : FreeImage(gif_ID) : EndIf
      
      ProcedureReturn sheet_ID
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure CreateCSSSheet_BF(gif_ID, invisible_gif_color)
      Protected sheet_ID, i, ii, iii, iiii, frames, frames_x, frames_y, gif_frame_delay
      Protected gif_width, gif_height, sheet_width, sheet_height, mode
      
      calculate_CSS_sheet
      
      gif_css_invisible_color=invisible_gif_color
      If Not StartDrawing(ImageOutput(sheet_ID)) : FreeImage(sheet_ID) : ProcedureReturn -4 : EndIf
      Box(0, 0, sheet_width, sheet_height, invisible_gif_color)
      StopDrawing()
      For i=0 To frames-1
        SetImageFrame_BF(gif_ID, i)
        If Not StartDrawing(ImageOutput(sheet_ID)) : FreeImage(sheet_ID) : ProcedureReturn -4 : EndIf
        DrawAlphaImage(ImageID(gif_ID), ii, iii)
        StopDrawing()
        ii+gif_width
        If ii=>sheet_width
          ii=0 : iii+gif_height
        EndIf   
      Next i
      
      If alpha_sprite
        sheet_ID=CreateAlphaImage_from_Sprite_BF(sheet_ID,
                                                 0,
                                                 invisible_gif_color,
                                                 0,
                                                 1)
      EndIf
      
      ProcedureReturn sheet_ID
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure CreateCSSandDataSheet_simple_BF(path$, gif_ID=0, encoder=2, internal_frame_data=0)
      Protected sheet_ID, i, file, gif_frame_delay, result
      
      If Not IsImage(gif_ID)
        gif_ID=LoadImage(#PB_Any, path$)
      EndIf
      
      If Not gif_ID : ProcedureReturn -17 : EndIf
      
      sheet_ID=CreateCSSSheet_simple_BF(gif_ID)
      
      If Not IsImage(sheet_ID) : ProcedureReturn sheet_ID : EndIf
      
      Protected frame_width=GetGIFandCSS_Frame_width_BF()
      Protected frame_height=GetGIFandCSS_Frame_height_BF()
      Protected sheet_width=ImageWidth(sheet_ID)
      Protected sheet_height=ImageHeight(sheet_ID)
      Protected frames=GetGIFandCSS_Frames_BF()
      Protected frames_in_a_row=GetFrames_in_a_row_BF()
      Protected invisible_gif_color=GetGIFandCSS_invisible_color_BF()
      Protected color_distance_gif=GetGIFandCSS_color_distance_BF()
      
      If frames>1
        
        file=CreateFile(#PB_Any, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+"_sheet_info_BF.txt")
        
        If Not file : FreeImage(sheet_ID) : ProcedureReturn -22 : EndIf
        
        common_write_frame_data
        
        For i=0 To frames-1
          gif_frame_delay=GetGIFandCSS_delay_array_BF(i)
          result=WriteStringN(file, "   Frame / delay :"+RSet(Str(i), 8, "0 ")+" *"+RSet(Str(gif_frame_delay), 8, "0"), #PB_Ascii) 
        Next i
        
        If Not result : FreeImage(sheet_ID) : CloseFile(file) : ProcedureReturn -22 : EndIf
        
        CloseFile(file)
        
      EndIf
      
      Select encoder
        Case 1
          result=SaveImage(sheet_ID, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".jpg", #PB_ImagePlugin_JPEG, 10)
        Case 2
          result=SaveImage(sheet_ID, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".png", #PB_ImagePlugin_PNG, 10)
        Case 3
          result=SaveImage(sheet_ID, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".png", #PB_ImagePlugin_BMP) 
      EndSelect
      
      FreeImage(sheet_ID)
      
      If Not result : ProcedureReturn -22 : EndIf
      
      If internal_frame_data And frames>1
        
        Select encoder
          Case 1     
            file=OpenFile(#PB_Any,  GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".jpg")
          Case 2   
            file=OpenFile(#PB_Any, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".png")
          Case 3     
            file=OpenFile(#PB_Any, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".bmp")
        EndSelect 
        
        If Not file : ProcedureReturn -22 : EndIf
        
        Protected file_length_0=Lof(file)
        
        FileSeek(file, file_length_0)
        
        common_write_frame_data
        
        For i=0 To frames-1
          gif_frame_delay=GetGIFandCSS_delay_array_BF(i)
          result=WriteStringN(file, "   Frame / delay :"+RSet(Str(i), 8, "0 ")+" *"+RSet(Str(gif_frame_delay), 8, "0"), #PB_Ascii)
        Next i
        
        If Not result : CloseFile(file) : ProcedureReturn -22 : EndIf
        
        Protected file_length_1=Lof(file)
        
        result=WriteQuad(file, file_length_1-file_length_0)  
        
        CloseFile(file)
        
        If Not result : ProcedureReturn -22 : EndIf
        
      EndIf
      
      ProcedureReturn 1
      
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure CreateCSSandDataSheet_BF(path$, gif_ID, invisible_gif_color, encoder=2, internal_frame_data=0, color_distance_gif=0)
      Protected sheet_ID, i, file, gif_frame_delay, result
      
      If Not IsImage(gif_ID)
        gif_ID=LoadImage(#PB_Any, path$)
      EndIf
      
      If Not gif_ID : ProcedureReturn -17 : EndIf
      
      sheet_ID=CreateCSSSheet_BF(gif_ID, invisible_gif_color)
      
      If Not IsImage(sheet_ID) : ProcedureReturn sheet_ID : EndIf
      
      Protected frame_width=GetGIFandCSS_Frame_width_BF()
      Protected frame_height=GetGIFandCSS_Frame_height_BF()
      Protected sheet_width=ImageWidth(sheet_ID)
      Protected sheet_height=ImageHeight(sheet_ID)
      Protected frames=GetGIFandCSS_Frames_BF()
      Protected frames_in_a_row=GetFrames_in_a_row_BF()
      
      If frames>1
        
        file=CreateFile(#PB_Any, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+"_sheet_info_BF.txt")
        
        If Not file : FreeImage(sheet_ID) : ProcedureReturn -22 : EndIf
        
        common_write_frame_data
        
        For i=0 To frames-1
          gif_frame_delay=GetGIFandCSS_delay_array_BF(i)
          result=WriteStringN(file, "   Frame / delay :"+RSet(Str(i), 8, "0 ")+" *"+RSet(Str(gif_frame_delay), 8, "0"), #PB_Ascii)  
        Next i
        
        If Not result : FreeImage(sheet_ID) : CloseFile(file) : ProcedureReturn -22 : EndIf
        
        CloseFile(file)
        
      EndIf
      
      Select encoder
        Case 1
          result=SaveImage(sheet_ID, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".jpg", #PB_ImagePlugin_JPEG, 10)
        Case 2
          result=SaveImage(sheet_ID, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".png", #PB_ImagePlugin_PNG, 10)
        Case 3
          result=SaveImage(sheet_ID, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".png", #PB_ImagePlugin_BMP) 
      EndSelect
      
      FreeImage(sheet_ID)
      
      If Not result : ProcedureReturn -22 : EndIf
      
      If internal_frame_data And frames>1
        Select encoder
          Case 1      
            file=OpenFile(#PB_Any,  GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".jpg")
          Case 2   
            file=OpenFile(#PB_Any, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".png")
          Case 3     
            file=OpenFile(#PB_Any, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".bmp")
        EndSelect
        
        If Not file : ProcedureReturn -22 : EndIf
        
        Protected file_length_0=Lof(file)
        
        FileSeek(file, file_length_0)
        
        common_write_frame_data
        
        For i=0 To frames-1
          gif_frame_delay=GetGIFandCSS_delay_array_BF(i)
          result=WriteStringN(file, "   Frame / delay :"+RSet(Str(i), 8, "0 ")+" *"+RSet(Str(gif_frame_delay), 8, "0"), #PB_Ascii)
        Next i
        
        If Not result : CloseFile(file) : ProcedureReturn -22 : EndIf
        
        Protected file_length_1=Lof(file)
        
        result=WriteQuad(file, file_length_1-file_length_0)  
        
        CloseFile(file)
        
        If Not result : ProcedureReturn -22 : EndIf
        
      EndIf
      
      ProcedureReturn 1
      
    EndProcedure
  CompilerEndIf
  
  Procedure CreateEmptyCSSandDataSheet_BF(path$, invisible_gif_color, frames, frame_width, frame_height, preset_delays=0, encoder=2, color_distance_gif=0)
    Protected sheet_ID, i, file, result, sheet_width, sheet_height, frames_in_a_row
    
    CalculateCSSSheet_BF(frame_width, frame_height, frames)
    
    frame_width=GetGIFandCSS_Frame_width_BF()
    frame_height=GetGIFandCSS_Frame_height_BF()
    sheet_width=GetCSS_width_BF()
    sheet_height=GetCSS_height_BF()
    frames_in_a_row=GetFrames_in_a_row_BF()
    
    If alpha_sprite
      sheet_ID=CreateImage(#PB_Any, sheet_width, sheet_height, 32, #PB_Image_Transparent)
    Else
      sheet_ID=CreateImage(#PB_Any, sheet_width, sheet_height, 24, invisible_gif_color)
    EndIf
    
    If Not IsImage(sheet_ID) : ProcedureReturn sheet_ID : EndIf
    
    If frames>1
      
      file=CreateFile(#PB_Any, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+"_sheet_info_BF.txt")
      
      If Not file : FreeImage(sheet_ID) : ProcedureReturn -22 : EndIf
      
      common_write_frame_data
      
      For i=0 To frames-1
        result=WriteStringN(file, "   Frame / delay :"+RSet(Str(i), 8, "0 ")+" *"+RSet(Str(preset_delays), 8, "0"), #PB_Ascii)  
      Next i
      
      If Not result : FreeImage(sheet_ID) : CloseFile(file) : ProcedureReturn -22 : EndIf
      
      CloseFile(file)
      
    EndIf
    
    Select encoder
      Case 1
        result=SaveImage(sheet_ID, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".jpg", #PB_ImagePlugin_JPEG, 10)
      Case 2
        result=SaveImage(sheet_ID, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".png", #PB_ImagePlugin_PNG, 10)
      Case 3
        result=SaveImage(sheet_ID, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".png", #PB_ImagePlugin_BMP) 
    EndSelect
    
    FreeImage(sheet_ID)
    
    If Not result : ProcedureReturn -22 : EndIf
    
    If frames>1
      
      Select encoder
        Case 1      
          file=OpenFile(#PB_Any,  GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".jpg")
        Case 2   
          file=OpenFile(#PB_Any, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".png")
        Case 3     
          file=OpenFile(#PB_Any, GetPathPart(path$)+GetFilePart(path$, #PB_FileSystem_NoExtension)+".bmp")
      EndSelect
      
      If Not file : ProcedureReturn -22 : EndIf
      
      Protected file_length_0=Lof(file)
      
      FileSeek(file, file_length_0)
      
      common_write_frame_data
      
      For i=0 To frames-1
        result=WriteStringN(file, "   Frame / delay :"+RSet(Str(i), 8, "0 ")+" *"+RSet(Str(preset_delays), 8, "0"), #PB_Ascii)
      Next i
      
      If Not result : CloseFile(file) : ProcedureReturn -22 : EndIf
      
      Protected file_length_1=Lof(file)
      
      result=WriteQuad(file, file_length_1-file_length_0)  
      
      CloseFile(file)
      
      If Not result : ProcedureReturn -22 : EndIf
      
    EndIf
    
    ProcedureReturn 1
    
  EndProcedure
  
  Procedure GetGIFandCSS_color_distance_BF()
    ProcedureReturn gif_css_color_distance
  EndProcedure
  
  Procedure GetGIFandCSS_invisible_color_BF()
    ProcedureReturn gif_css_invisible_color
  EndProcedure
  
  Procedure GetGIFandCSS_Frames_BF()
    ProcedureReturn gif_css_frames
  EndProcedure
  
  Procedure GetGIFandCSS_Frame_width_BF()
    ProcedureReturn gif_css_frame_width 
  EndProcedure
  
  Procedure GetGIFandCSS_Frame_height_BF()
    ProcedureReturn gif_css_frame_height
  EndProcedure
  
  Procedure GetCSS_width_BF()
    ProcedureReturn css_width 
  EndProcedure
  
  Procedure GetCSS_height_BF()
    ProcedureReturn css_height
  EndProcedure
  
  Procedure GetFrames_in_a_row_BF()
    ProcedureReturn css_frames_in_a_row
  EndProcedure
  
  Procedure GetGIFandCSS_delay_array_BF(gif_and_css_delay_array_)
    If gif_and_css_delay_array_>ArraySize(gif_and_css_delay_array())
      ReDim gif_and_css_delay_array(gif_and_css_delay_array_)
    EndIf
    ProcedureReturn gif_and_css_delay_array(gif_and_css_delay_array_)
  EndProcedure
  
  Procedure GetFrameData_fromSheet_BF(path$)
    Protected i, file, file_length_0.q, file_length_1.q, frame_delay
    file=ReadFile(#PB_Any, path$)
    If Not file : ProcedureReturn -21 : EndIf
    file_length_0=Lof(file)
    If file_length_0<208 : ProcedureReturn -21 : EndIf  ; New BF <246
    FileSeek(file, file_length_0-28)
    If ReadString(file, #PB_UTF8, 18)="   ColorDistance +"
      i=28
      FileSeek(file, file_length_0-36)
    Else
      FileSeek(file, file_length_0-8)
    EndIf
    file_length_1=ReadQuad(file)
    If file_length_1>Lof(file)  Or file_length_1<208 : CloseFile(file) : ProcedureReturn -21 : EndIf ; Fuse - New BF <246
    file_length_0=file_length_0-file_length_1
    FileSeek(file, file_length_0-i)
    If ReadString(file, #PB_UTF8, 10)<>"  Frames #" : CloseFile(file) : ProcedureReturn -21 : EndIf
    FileSeek(file, -18, #PB_Relative)
    gif_css_frames=Val(Mid(ReadString(file), 19, 8))           ; Count frames in the sheet
    gif_css_frame_width=Val(Mid(ReadString(file), 19, 8))      ; Frame width
    gif_css_frame_height=Val(Mid(ReadString(file), 19, 8))     ; Frame height
    css_width=Val(Mid(ReadString(file), 19, 8))                ; Sheet width
    css_height=Val(Mid(ReadString(file), 19, 8))               ; Sheet height 
    css_frames_in_a_row=Val(Mid(ReadString(file), 19, 8))      ; Frames in a row
    gif_css_invisible_color=Val(Mid(ReadString(file), 19, 8))  ; Invisible color
    
    ; Compatibility for older version
    ; gif_css_color_distance=Val(Mid(ReadString(file), 19, 8)) ; Color distance
    Protected result$=ReadString(file)
    If Left(result$, 16)="  Color distance"
      gif_css_color_distance=Val(Mid(result$, 19, 8)) ; Color distance
    Else
      FileSeek(file, -36, #PB_Relative)
    EndIf
    
    Dim gif_and_css_delay_array(gif_css_frames-1)
    i=0
    While Not Eof(file) And i<gif_css_frames
      frame_delay=Val(Mid(ReadString(file), 29, 8) ) ; Frame delay
      If Not frame_delay : frame_delay=100 : EndIf
      gif_and_css_delay_array(i)=frame_delay ; Frame delay
      
      ; Debug GetGIFandCSS_delay_array_BF(i)
      i+1
    Wend
    
    ; Debug GetGIFandCSS_frames_BF()
    ; Debug GetGIFandCSS_frame_width_BF()
    ; Debug GetGIFandCSS_frame_height_BF()
    ; Debug GetCSS_width_BF()
    ; Debug GetCSS_height_BF()
    ; Debug GetFrames_in_a_row_BF()
    ; Debug GetGIFandCSS_invisible_color_BF()
    ; Debug GetGIFandCSS_color_distance_BF()
    
    CloseFile(file)
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure IsSheet_BF(path$)
    Protected i, file, file_length_0.q, file_length_1.q, frame_delay, search_string$="   Frame / delay"
    file=ReadFile(#PB_Any, path$)
    If Not file : ProcedureReturn -21 : EndIf
    file_length_0=Lof(file)
    If file_length_0<208 : ProcedureReturn -21 : EndIf ; New BF <246
    FileSeek(file, file_length_0-74)
    If ReadString(file, #PB_UTF8, 16)=search_string$
      CloseFile(file)
      ProcedureReturn 1
    Else
      FileSeek(file, file_length_0-46)
      If ReadString(file, #PB_UTF8, 16)=search_string$
        CloseFile(file)
        ProcedureReturn 1
      EndIf
    EndIf
    CloseFile(file)
    ProcedureReturn 0
  EndProcedure
  
  Procedure GetFrameDataFromString_BF(string$)
    Protected i, ii=Len(string$)-1, iii
    Protected delays=CountString(string$, "*")
    If Not delays Or CountString(string$, "#")<>6 : ProcedureReturn -9 : EndIf ; Fuse
    Dim gif_and_css_delay_array(delays-1)
    i=2
    SetGIFandCSS_Frames_BF(Val(Mid(string$, i, 8))) : i+9
    SetGIFandCSS_frame_width_BF(Val(Mid(string$, i, 8))) : i+9
    SetGIFandCSS_frame_height_BF(Val(Mid(string$, i, 8))) : i+9
    SetCSS_width_BF(Val(Mid(string$, i, 8))) : i+9
    SetCSS_height_BF(Val(Mid(string$, i, 8))) : i+9
    SetFrames_in_a_row_BF(Val(Mid(string$, i, 8))) : i+9
    SetGIFandCSS_invisible_color_BF(Val(Mid(string$, i, 8))) : i+9
    SetGIFandCSS_color_distance_BF(Val(Mid(string$, i, 8))) : i+9
    For ii=i To Len(string$) Step 9
      gif_and_css_delay_array(iii)=Val(Mid(string$, ii, 8))
      iii+1
    Next ii
    ProcedureReturn 1
  EndProcedure
  
  Procedure ReadSheetInfoFile_BF(path$)
    Protected i, file, frame_delay
    file=ReadFile(#PB_Any, path$)
    If Not file : ProcedureReturn -21 : EndIf
    FileSeek(file, 8, #PB_Relative)
    If ReadString(file, #PB_UTF8, 10)<>"  Frames #" : CloseFile(file) : ProcedureReturn -21 : EndIf
    FileSeek(file, -18, #PB_Relative)
    gif_css_frames=Val(Mid(ReadString(file), 19, 8))           ; Count frames in the sheet
    gif_css_frame_width=Val(Mid(ReadString(file), 19, 8))      ; Frame width
    gif_css_frame_height=Val(Mid(ReadString(file), 19, 8))     ; Frame height
    css_width=Val(Mid(ReadString(file), 19, 8))                ; Sheet width
    css_height=Val(Mid(ReadString(file), 19, 8))               ; Sheet height 
    css_frames_in_a_row=Val(Mid(ReadString(file), 19, 8))      ; Frames in a row
    gif_css_invisible_color=Val(Mid(ReadString(file), 19, 8))  ; Invisible color
    
    ; Compatibility for older version
    ; gif_css_color_distance=Val(Mid(ReadString(file), 19, 8)) ; Color distance
    Protected result$=ReadString(file)
    If Left(result$, 16)="  Color distance"
      gif_css_color_distance=Val(Mid(result$, 19, 8)) ; Color distance
    Else
      FileSeek(file, -36, #PB_Relative)
    EndIf
    
    Dim gif_and_css_delay_array(gif_css_frames-1)
    
    While Not Eof(file) And i<gif_css_frames
      frame_delay=Val(Mid(ReadString(file), 29, 8) ) ; Frame delay
      If Not frame_delay : frame_delay=100 : EndIf
      gif_and_css_delay_array(i)=frame_delay ; Frame delay
      
      ; Debug GetGIFandCSS_delay_array_BF(i)
      i+1
    Wend
    
    ; Debug GetGIFandCSS_frames_BF()
    ; Debug GetGIFandCSS_frame_width_BF()
    ; Debug GetGIFandCSS_frame_height_BF()
    ; Debug GetCSS_width_BF()
    ; Debug GetCSS_height_BF()
    ; Debug GetFrames_in_a_row_BF()
    ; Debug GetGIFandCSS_invisible_color_BF()
    ; Debug GetGIFandCSS_color_distance_BF()
    
    CloseFile(file)
    
    ProcedureReturn 1
    
  EndProcedure
  
  Procedure IsSheetInfoFile_BF(path$)
    If FileSize(path$)<208 : ProcedureReturn 0 : EndIf  ; New BF <246
    Protected file=ReadFile(#PB_Any, path$)
    If Not file : ProcedureReturn -21 : EndIf
    FileSeek(file, 8, #PB_Relative)
    If ReadString(file, #PB_UTF8, 10)<>"  Frames #" : CloseFile(file) : ProcedureReturn 0
    Else : CloseFile(file) : ProcedureReturn 1
    EndIf
  EndProcedure
  
  Procedure SetSheet_delay_array_BF(frame, delay)
    gif_and_css_delay_array(frame)=delay
    ProcedureReturn 0
  EndProcedure
  
  Procedure SetGIFandCSS_delay_array_BF(frame, delay)
    gif_and_css_delay_array(frame)=delay
    ProcedureReturn 0
  EndProcedure
  
  Procedure SetGIFandCSS_color_distance_BF(gif_css_color_distance_)
    gif_css_color_distance=gif_css_color_distance_
    ProcedureReturn 0
  EndProcedure
  
  Procedure SetGIFandCSS_invisible_color_BF(gif_css_invisible_color_)
    gif_css_invisible_color=gif_css_invisible_color_
    ProcedureReturn 0
  EndProcedure
  
  Procedure SetGIFandCSS_Frames_BF(gif_css_frames_)
    gif_css_frames=gif_css_frames_
    ProcedureReturn
  EndProcedure
  
  Procedure SetGIFandCSS_Frame_width_BF(gif_css_frame_width_)
    gif_css_frame_width=gif_css_frame_width_
    ProcedureReturn 0
  EndProcedure
  
  Procedure SetGIFandCSS_Frame_height_BF(gif_css_frame_height_)
    gif_css_frame_height=gif_css_frame_height_
    ProcedureReturn 0
  EndProcedure
  
  Procedure SetCSS_width_BF(css_width_)
    css_width=css_width_
    ProcedureReturn 0
  EndProcedure
  
  Procedure SetCSS_height_BF(css_height_)
    css_height=css_height_
    ProcedureReturn 0
  EndProcedure
  
  Procedure SetFrames_in_a_row_BF(css_frames_in_a_row_)
    css_frames_in_a_row=css_frames_in_a_row_
  EndProcedure
  
  Procedure.s SetFrameDataInString_BF()
    Protected i, fillstring$, ii=ArraySize(gif_and_css_delay_array())
    
    If GetGIFandCSS_invisible_color_BF()<0 : fillstring$=" " : Else : fillstring$="0" : EndIf
    
    Protected result$="#"+RSet(Str(GetGIFandCSS_Frames_BF()), 8, "0")+
                      "#"+RSet(Str(GetGIFandCSS_frame_width_BF()), 8, "0")+
                      "#"+RSet(Str(GetGIFandCSS_frame_height_BF()), 8, "0")+
                      "#"+RSet(Str(GetCSS_width_BF()), 8, "0")+
                      "#"+RSet(Str(GetCSS_height_BF()), 8, "0")+
                      "#"+RSet(Str(GetFrames_in_a_row_BF()), 8, "0")+
                      "$"+RSet(Str(GetGIFandCSS_invisible_color_BF()), 8, fillstring$)+ ; Negative value available " "
                      "$"+RSet(Str(GetGIFandCSS_color_distance_BF()), 8, "0")
    For i=0 To ii
      result$+"*"+RSet(Str(gif_and_css_delay_array(i)), 8, "0")
    Next i
    ProcedureReturn result$
  EndProcedure
  
  Procedure.s LoadFrameData_BF(path$)
    If GetFrameData_fromSheet_BF(path$)<>1
      ProcedureReturn ""
    EndIf
    ProcedureReturn SetFrameDataInString_BF()
  EndProcedure
  
  Procedure CalculateCSSSheet_BF(frame_width, frame_height, frames)
    Protected i, ii, iii, iiii, frames_x, frames_y, frames_in_a_row, sheet_width, sheet_height
    If frame_height>frame_width And frames>1
      frames_x=Sqr(frames)+1
    Else
      frames_x=Sqr(frames)
    EndIf
    If Not frames_x : frames_x=1 : EndIf
    frames_in_a_row=frames_x
    frames_y=frames/frames_x+1
    sheet_width=frames_x*frame_width
    sheet_height=frames_y*frame_height
    For i=0 To frames-1
      iiii=iii : ii+frame_width
      If ii=>sheet_width
        ii=0 : iii+frame_height
      EndIf   
    Next i
    
    sheet_height=iiii+frame_height 
    
    SetGIFandCSS_Frames_BF(frames)
    SetGIFandCSS_Frame_width_BF(frame_width)
    SetGIFandCSS_Frame_height_BF(frame_height)
    SetCSS_width_BF(sheet_width)
    SetCSS_height_BF(sheet_height)
    SetFrames_in_a_row_BF(frames_in_a_row)
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure QAES_crypter_BF(mode, *buffer_in.word, *buffer_out.word, bytes.q, key$, counter.q=0)
    
    If Not bytes.q Or key$="" : ProcedureReturn -26 : EndIf
    
    Protected.q i
    Protected ii, iii, bytes_minus_x, stepp=SizeOf(character)<<1
    Protected *buffer_in_asc.ascii, *buffer_out_asc.ascii
    Protected hash$=key$+Str(counter)+"t8690352cj2p1ch7fgw34uotmq09745$%()=)&%" ; + Salt, you can change
    Static fixed_key_string${64}
    Static Dim register.q(3)
    
    fixed_key_string$=Fingerprint(@hash$, StringByteLength(hash$), #PB_Cipher_SHA3, 256)
    
    If mode=2
      bytes_minus_x=bytes-3
    Else
      bytes_minus_x=bytes-2
      *buffer_in_asc.ascii=*buffer_in
      *buffer_out_asc.ascii=*buffer_out
    EndIf
    
    For ii = 0 To 31 : PokeA(@register(0)+ii, Val("$"+PeekS(@fixed_key_string$+iii, 2))) : iii+stepp : Next ; Create a key
    
    Repeat
      If Not AESEncoder(@register(0), @register(0), 32, @register(0), 256, 0, #PB_Cipher_ECB) : ProcedureReturn -9 : EndIf
      Swap register(0), register(3) : Swap register(1), register(2)
      If Not AESEncoder(@register(0), @register(0), 16, @register(0), 256, 0, #PB_Cipher_ECB) : ProcedureReturn -9 : EndIf
      
      If mode=2
        Protected *register.word=@register(0)
        For ii=0 To 15 Step 2
          If *buffer_in\w And *buffer_in\w ! *register\w
            *buffer_out\w=*buffer_in\w ! *register\w
          Else
            *buffer_out\w=*buffer_in\w
          EndIf
          If i>bytes_minus_x : Break 2 : EndIf
          *buffer_in+2 : *buffer_out+2 : *register+2 : i+2
        Next ii
      ElseIf mode=1
        Protected *register_asc.ascii=@register(0)
        For ii=0 To 15
          If *buffer_in_asc\a And *buffer_in_asc\a ! *register_asc\a
            *buffer_out_asc\a=*buffer_in_asc\a ! *register_asc\a
          Else
            *buffer_out_asc\a=*buffer_in_asc\a
          EndIf
          If i>bytes_minus_x : Break 2 : EndIf
          *buffer_in_asc+1 : *buffer_out_asc+1 : *register_asc+1 : i+1
        Next ii
      Else
        *register_asc.ascii=@register(0)
        For ii=0 To 15
          *buffer_out_asc\a=*buffer_in_asc\a ! *register_asc\a
          If i>bytes_minus_x : Break 2 : EndIf
          *buffer_in_asc+1 : *buffer_out_asc+1 : *register_asc+1 : i+1
        Next ii
      EndIf
    ForEver
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure Create_encrypted_String_File_BF(string$, destination_path$, key$)
    ; Create a encrypted string file
    Protected file=CreateFile(#PB_Any, destination_path$)
    If file
      Protected counter.q
      If OpenCryptRandom()
        CryptRandomData(@counter, 8)
      Else
        RandomData(@counter, 8)
      EndIf
      Protected strg_size=StringByteLength(string$)
      If strg_size
        Protected *buffer=AllocateMemory(strg_size)
      EndIf
      If Not *buffer : ProcedureReturn 0 : EndIf
      CopyMemory(@string$, *buffer, strg_size)
      If Not QAES_Crypter_BF(0, *buffer, *buffer, MemorySize(*buffer), key$, counter)
        FreeMemory(*buffer) : ProcedureReturn 0
      EndIf
      If WriteData(file, *buffer, MemorySize(*buffer))<>MemorySize(*buffer)
        FreeMemory(*buffer) : ProcedureReturn 0
      EndIf 
      FileSeek(file, Lof(file))
      If WriteQuad(file, counter)<>8 : FreeMemory(*buffer) : ProcedureReturn 0 : EndIf
      CloseFile(file)
      FreeMemory(*buffer)
    Else
      ProcedureReturn 0
    EndIf
    ProcedureReturn 1
  EndProcedure
  
  Procedure.s Read_encrypted_String_File_BF(destination_path$, key$)
    ; Read a encrypted string file
    Protected file=ReadFile(#PB_Any, destination_path$)
    If Not file : ProcedureReturn "" : EndIf
    Protected *buffer=AllocateMemory(Lof(file)-8)
    If Not *buffer : ProcedureReturn "" : EndIf
    FileSeek(file, Lof(file)-8)
    Protected counter.q=ReadQuad(file)
    If Not counter : FreeMemory(*buffer) : ProcedureReturn "" : EndIf
    FileSeek(file, 0)
    If ReadData(file, *buffer, MemorySize(*buffer))<>MemorySize(*buffer)
    FreeMemory(*buffer) : ProcedureReturn "" : EndIf
    Protected string$=Space(MemorySize(*buffer)/SizeOf(character))
    If Not QAES_Crypter_BF(0, *buffer, @string$, MemorySize(*buffer), key$, counter)
      FreeMemory(*buffer) : ProcedureReturn ""
    EndIf
    CloseFile(file)
    FreeMemory(*buffer)
    ProcedureReturn string$
  EndProcedure
  
  Procedure Convert_to_multi_invisible_Color_Sprite_BF(source_ID,
                                                       invisible_color_1=-1,
                                                       invisible_color_2=-1,
                                                       invisible_color_3=-1,
                                                       invisible_color_4=-1,
                                                       invisible_color_5=-1,
                                                       invisible_color_6=-1,
                                                       invisible_color_7=-1,
                                                       invisible_color_8=-1,
                                                       invisible_color_9=-1,
                                                       invisible_color_10=-1)
    
    Protected new=SearchUnusedImageColor_BF(source_ID), old_texture_get_color=texture_get_color, old_color_distance.f=percent
    percent=0
    
    If invisible_color_1<>-1  : texture_get_color=invisible_color_1  : If BF(-1, source_ID, new, -1, -1)<0 : ProcedureReturn -1 : EndIf : EndIf
    If invisible_color_2<>-1  : texture_get_color=invisible_color_2  : If BF(-1, source_ID, new, -1, -1)<0 : ProcedureReturn -1 : EndIf : EndIf
    If invisible_color_3<>-1  : texture_get_color=invisible_color_3  : If BF(-1, source_ID, new, -1, -1)<0 : ProcedureReturn -1 : EndIf : EndIf
    If invisible_color_4<>-1  : texture_get_color=invisible_color_4  : If BF(-1, source_ID, new, -1, -1)<0 : ProcedureReturn -1 : EndIf : EndIf
    If invisible_color_5<>-1  : texture_get_color=invisible_color_5  : If BF(-1, source_ID, new, -1, -1)<0 : ProcedureReturn -1 : EndIf : EndIf
    If invisible_color_6<>-1  : texture_get_color=invisible_color_6  : If BF(-1, source_ID, new, -1, -1)<0 : ProcedureReturn -1 : EndIf : EndIf
    If invisible_color_7<>-1  : texture_get_color=invisible_color_7  : If BF(-1, source_ID, new, -1, -1)<0 : ProcedureReturn -1 : EndIf : EndIf
    If invisible_color_8<>-1  : texture_get_color=invisible_color_8  : If BF(-1, source_ID, new, -1, -1)<0 : ProcedureReturn -1 : EndIf : EndIf
    If invisible_color_9<>-1  : texture_get_color=invisible_color_9  : If BF(-1, source_ID, new, -1, -1)<0 : ProcedureReturn -1 : EndIf : EndIf
    If invisible_color_10<>-1 : texture_get_color=invisible_color_10 : If BF(-1, source_ID, new, -1, -1)<0 : ProcedureReturn -1 : EndIf : EndIf
    
    texture_get_color=old_texture_get_color
    percent=old_color_distance
    
    ProcedureReturn new  
  EndProcedure
  
  Macro procedurereturn_messagerequester
    If IsWindow(parent_window_message_requester_ID)
      If mode_messagerequester
        UseGadgetList(old_gadget_list)
      EndIf
    EndIf
  EndMacro
  
  Procedure MessageRequester_simple_BF(title_line$="",
                                       text$="",
                                       flags.a=#PB_MessageRequester_Yes,
                                       text_button_yes$="Yes",
                                       text_button_no$="No",
                                       text_button_cancel$="Cancel",
                                       text_width=470,
                                       text_height=165,
                                       buttons_width=150,
                                       buttons_height=35)
    
    Protected text_flag=#PB_Text_Center ; You can change
    
    flags << 5 : flags >> 5
    
    If text_width<120 : text_width=120 : EndIf
    If text_height<20 : text_height=20 : EndIf
    
    text_width+(1-text_width%2)
    Protected window_ID, button_yes_ID, button_no_ID, button_cancel_ID, text_ID, result, window_event
    
    If flags=#PB_MessageRequester_Yes
      If buttons_width>text_width-10 : buttons_width=text_width-10 : EndIf
    ElseIf flags=#PB_MessageRequester_YesNo
      If buttons_width*2+40>text_width-10 : buttons_width=text_width/2-20 : EndIf 
    ElseIf flags=#PB_MessageRequester_YesNoCancel
      If buttons_width*3+60>text_width-10 : buttons_width=text_width/3-20 : EndIf 
    EndIf
    
    window_ID=OpenWindow(#PB_Any, 0, 0, text_width+20, buttons_height+30+text_height, " "+title_line$, #PB_Window_Tool|#PB_Window_ScreenCentered|#PB_Window_SystemMenu|#PB_Window_Invisible)
    If IsWindow(window_ID)
      StickyWindow(window_ID, #True)
      text_ID=TextGadget(#PB_Any, 10, 10, text_width, text_height, text$, text_flag)
      
      button_yes_ID=ButtonGadget(#PB_Any, -buttons_width, WindowHeight(window_ID)-buttons_height-10, buttons_width, buttons_height, " "+text_button_yes$, #PB_Button_MultiLine)
      button_no_ID=ButtonGadget(#PB_Any, -buttons_width, WindowHeight(window_ID)-buttons_height-10, buttons_width, buttons_height, " "+text_button_no$, #PB_Button_MultiLine)
      button_cancel_ID=ButtonGadget(#PB_Any, -buttons_width, WindowHeight(window_ID)-buttons_height-10, buttons_width, buttons_height, " "+text_button_cancel$, #PB_Button_MultiLine)
      
      If Not IsGadget(text_ID) Or Not IsGadget(button_yes_ID) Or Not IsGadget(button_no_ID) Or Not IsGadget(button_cancel_ID)
        CloseWindow(window_ID)
        ProcedureReturn 0
      EndIf
      
      If flags=#PB_MessageRequester_Yes : HideGadget(button_no_ID, 1) : HideGadget(button_cancel_ID, 1)
        ResizeGadget(button_yes_ID, WindowWidth(window_ID)/2-buttons_width/2, #PB_Ignore, #PB_Ignore, #PB_Ignore)
      ElseIf flags=#PB_MessageRequester_YesNo : HideGadget(button_cancel_ID, 1)
        ResizeGadget(button_yes_ID, WindowWidth(window_ID)/2-buttons_width-10, #PB_Ignore, #PB_Ignore, #PB_Ignore)
        ResizeGadget(button_no_ID, WindowWidth(window_ID)/2+10, #PB_Ignore, #PB_Ignore, #PB_Ignore)
      ElseIf flags=#PB_MessageRequester_YesNoCancel
        ResizeGadget(button_yes_ID, WindowWidth(window_ID)/2-buttons_width*1.5-20, #PB_Ignore, #PB_Ignore, #PB_Ignore)
        ResizeGadget(button_no_ID, WindowWidth(window_ID)/2-buttons_width/2, #PB_Ignore, #PB_Ignore, #PB_Ignore)
        ResizeGadget(button_cancel_ID, WindowWidth(window_ID)/2+buttons_width/2+20, #PB_Ignore, #PB_Ignore, #PB_Ignore)
      EndIf
      
      HideWindow(window_ID, 0)
      
      Repeat
        window_event=WaitWindowEvent()
        If window_event=#PB_Event_Gadget
          Select EventGadget()
            Case button_yes_ID
              result=#PB_MessageRequester_Yes
            Case button_no_ID
              result=#PB_MessageRequester_No
            Case button_cancel_ID
              result=#PB_MessageRequester_Cancel
          EndSelect
        EndIf
        If window_event=#PB_Event_CloseWindow
          End
        EndIf
      Until result
      CloseWindow(window_ID)
      ProcedureReturn result
    EndIf
  EndProcedure
  
  Procedure MessageRequester_Magic_BF(title_line$="",
                                      text$="",
                                      flags.a=#PB_MessageRequester_Yes,
                                      text_button_yes$="Yes",
                                      text_button_no$="No",
                                      text_button_cancel$="Cancel",
                                      text_width=470,
                                      text_height=165,
                                      text_stretching_x=0, ; Headline mode
                                      text_stretching_y=0, ; Headline mode
                                      text_padding_x=7,    ; This shrink the text x
                                      text_padding_y=7,    ; This shrink the text y
                                      text_color=#Black,
                                      buttons_color=#Black,
                                      text_background_color=#White,
                                      buttons_background_color=-1,
                                      buttons_width=150,
                                      buttons_height=35,
                                      button_yes_padding_x=7,
                                      button_yes_padding_y=7,
                                      button_no_padding_x=7,
                                      button_no_padding_y=7,
                                      button_cancel_padding_x=7,
                                      button_cancel_padding_y=7,
                                      font$="Arial",
                                      font_style=0)
    
    Protected text_flag=#PB_Text_Center ; You can change
    Protected window_ID, button_yes_ID, button_no_ID, button_cancel_ID, text_ID, result, window_event
    
    If flags=#PB_MessageRequester_Yes
      If buttons_width>text_width-10 : buttons_width=text_width-10 : EndIf
    ElseIf flags=#PB_MessageRequester_YesNo
      If buttons_width*2+40>text_width-10 : buttons_width=text_width/2-20 : EndIf 
    ElseIf flags=#PB_MessageRequester_YesNoCancel
      If buttons_width*3+60>text_width-10 : buttons_width=text_width/3-20 : EndIf 
    EndIf
    
    Static font_ID, font_style_, font_$, text_flag_
    
    If Not IsFont(font_ID) Or font_style<>font_style_ Or font$<>font_$
      If IsFont(font_ID) ; PB 5.72 Bug - Linux free not fonts - This method load only absolut necessary fonts
        FreeFont(font_ID)
      EndIf
      font_ID=LoadFont(#PB_Any, font$, 11, font_style) ; Fontsize 11 you must not change
      If Not IsFont(font_ID) : ProcedureReturn 0 : EndIf
      font_style_=font_style
      font_$=font$
    EndIf
    
    flags << 5 : flags >> 5
    
    If text_width<120 : text_width=120 : EndIf
    If text_height<20 : text_height=20 : EndIf
    
    text_width+(1-text_width%2)
    
    Protected gtk3_x_offset_, gtk3_y_offset_
    CompilerIf #PB_Compiler_OS=#PB_OS_Linux And Not Subsystem("gtk2") And Not Subsystem("qt") ; gtk3
      If Messagerequester_Magic_make_text_clickable
        gtk3_x_offset_=gtk3_x_offset
        gtk3_y_offset_=gtk3_y_offset
      EndIf
    CompilerEndIf
    
    window_ID=OpenWindow(#PB_Any, 0, 0,
                         text_width+20+gtk3_x_offset_/2-1,
                         buttons_height+30+text_height+gtk3_y_offset_, " "+title_line$, #PB_Window_Tool|#PB_Window_ScreenCentered|#PB_Window_SystemMenu|#PB_Window_Invisible)
    
    If IsWindow(window_ID)
      StickyWindow(window_ID, #True)
      
      Protected swap_=Bool(Messagerequester_Magic_make_text_clickable)
      swap_=1-swap_
      text_ID=MagicTextGadget_BF(swap_,                                                ; 0 = ButtonImageGadget - 1 = CanvasGadget
                                 text$,                                                ; Text
                                 9,                                                    ; Gadget output x pos
                                 10,                                                   ; Gadget output y pos
                                 text_width-gtk3_x_offset_/1.4,                        ; Gadget width
                                 text_height-gtk3_y_offset_/1.4,                       ; Gadget height
                                 text_width-gtk3_x_offset_/1.4,                        ; Text width
                                 text_height-gtk3_y_offset_/1.4,                       ; Text height
                                 text_stretching_x,                                    ; Text stretching x
                                 text_stretching_y,                                    ; Text stretching y
                                 text_padding_x,                                       ; Padding_x
                                 text_padding_y,                                       ; Padding y
                                 text_color,                                           ; Text color - Alpha channel supported
                                 text_background_color,                                ; Background color - Alpha channel supported - For invisible background set to -1
                                 1,                                                    ; Text flag - Text center=1 - Text right=2 - Text vertical=3
                                 font$,                                                ; Font type - more is not needed - accept also ""
                                 font_style,                                           ; Font style
                                 0)                                                    ; Border flag - #PB_Canvas_Border
      
      Protected old_MagicStretchText_Add_Image_ID=MagicStretchText_Add_Image_ID
      Protected old_MagicStretchText_Add_Image_1_ID=MagicStretchText_Add_Image_1_ID 
      Protected old_MagicStretchText_Add_Image_2_ID=MagicStretchText_Add_Image_2_ID
      MagicStretchText_Add_Image_ID=-1
      MagicStretchText_Add_Image_1_ID=-1
      MagicStretchText_Add_Image_2_ID=-1 
      
      Protected old_frame_size_stretchtext=frame_size_stretchtext
      Protected old_frame_padding_stretchtext_x=frame_padding_stretchtext_x
      Protected old_frame_padding_stretchtext_y=frame_padding_stretchtext_y
      frame_size_stretchtext=0
      frame_padding_stretchtext_x=0
      frame_padding_stretchtext_y=0
      
      button_yes_ID=MagicTextGadget_BF(0,                                                          ; 0 = ButtonImageGadget - 1 = CanvasGadget
                                       text_button_yes$,                                           ; Text
                                       -buttons_width-10-gtk3_y_offset,                            ; Gadget output x pos
                                       WindowHeight(window_ID)-buttons_height-10-gtk3_y_offset_/2, ; Gadget output y pos
                                       buttons_width-gtk3_x_offset,                                ; Gadget width
                                       buttons_height-gtk3_y_offset,                               ; Gadget height
                                       buttons_width-gtk3_x_offset,                                ; Text width
                                       buttons_height-gtk3_y_offset,                               ; Text height
                                       0,                                                          ; Text stretching x - Headline mode
                                       0,                                                          ; Text stretching y - Headline mode
                                       button_yes_padding_x,                                       ; Button yes Padding_x
                                       button_yes_padding_y,                                       ; Button yes Padding_y
                                       buttons_color,                                              ; Button color - Alpha channel supported
                                       buttons_background_color,                                   ; Background color - Alpha channel supported - For invisible background set to -1
                                       1,                                                          ; Text flag - Text center=1 - Text right=2 - Text vertical=3
                                       font$,                                                      ; Font type - more is not needed - accept also ""
                                       font_style,                                                 ; Font style
                                       0)                                                          ; Border flag
      
      button_no_ID=MagicTextGadget_BF(0,                                                         ; 0 = ButtonImageGadget - 1 = CanvasGadget
                                      text_button_no$,                                           ; Text
                                      -buttons_width-gtk3_x_offset,                              ; Gadget output x pos
                                      WindowHeight(window_ID)-buttons_height-10-gtk3_y_offset/2, ; Gadget output y pos
                                      buttons_width-gtk3_x_offset,                               ; Gadget width
                                      buttons_height-gtk3_y_offset,                              ; Gadget height
                                      buttons_width-gtk3_x_offset,                               ; Text width
                                      buttons_height-gtk3_y_offset,                              ; Text height
                                      0,                                                         ; Text stretching x - Headline mode
                                      0,                                                         ; Text stretching y - Headline mode
                                      button_no_padding_x,                                       ; Button no Padding_x
                                      button_no_padding_y,                                       ; Button no Padding_y
                                      buttons_color,                                             ; Button color - Alpha channel supported
                                      buttons_background_color,                                  ; Background color - Alpha channel supported - For invisible background set to -1
                                      1,                                                         ; Text flag - Text center=1 - Text right=2 - Text vertical=3
                                      font$,                                                     ; Font type - more is not needed - accept also ""
                                      font_style,                                                ; Font style
                                      0)                                                         ; Border flag
      
      button_cancel_ID=MagicTextGadget_BF(0,                                                         ; 0 = ButtonImageGadget - 1 = CanvasGadget
                                          text_button_cancel$,                                       ; Text
                                          -buttons_width-gtk3_y_offset,                              ; Gadget output x pos
                                          WindowHeight(window_ID)-buttons_height-10-gtk3_y_offset/2, ; Gadget output y pos
                                          buttons_width-gtk3_y_offset,                               ; Gadget width
                                          buttons_height-gtk3_y_offset,                              ; Gadget height
                                          buttons_width-gtk3_y_offset,                               ; Text width
                                          buttons_height-gtk3_y_offset,                              ; Text height
                                          0,                                                         ; Text stretching x - Headline mode
                                          0,                                                         ; Text stretching y - Headline mode
                                          button_cancel_padding_x,                                   ; Button cancel Padding_x
                                          button_cancel_padding_y,                                   ; Button cancel Padding_y
                                          buttons_color,                                             ; Button color - Alpha channel supported
                                          buttons_background_color,                                  ; Background color - Alpha channel supported - For invisible background set to -1
                                          1,                                                         ; Text flag - Text center=1 - Text right=2 - Text vertical=3
                                          font$,                                                     ; Font type - more is not needed - accept also ""
                                          font_style,                                                ; Font style
                                          0)                                                         ; Border flag 
      
      frame_size_stretchtext=old_frame_size_stretchtext
      frame_padding_stretchtext_x=old_frame_padding_stretchtext_x
      frame_padding_stretchtext_y=old_frame_padding_stretchtext_y
      MagicStretchText_Add_Image_ID=old_MagicStretchText_Add_Image_ID
      MagicStretchText_Add_Image_1_ID=old_MagicStretchText_Add_Image_1_ID
      MagicStretchText_Add_Image_2_ID=old_MagicStretchText_Add_Image_2_ID 
      
      Macro free_gadgets_BF_MessageRequester_Magic_BF
        FreeGadget_BF(text_ID)
        FreeGadget_BF(button_yes_ID)
        FreeGadget_BF(button_no_ID)
        FreeGadget_BF(button_cancel_ID)
      EndMacro
      
      If Not IsGadget(text_ID) Or Not IsGadget(button_yes_ID) Or Not IsGadget(button_no_ID) Or Not IsGadget(button_cancel_ID)
        free_gadgets_BF_MessageRequester_Magic_BF
        CloseWindow(window_ID)
        ProcedureReturn 0
      EndIf
      
      If flags=#PB_MessageRequester_Yes : HideGadget(button_no_ID, 1) : HideGadget(button_cancel_ID, 1)
        ResizeGadget(button_yes_ID, WindowWidth(window_ID)/2-buttons_width/2, #PB_Ignore, #PB_Ignore, #PB_Ignore)
      ElseIf flags=#PB_MessageRequester_YesNo : HideGadget(button_cancel_ID, 1)
        ResizeGadget(button_yes_ID, WindowWidth(window_ID)/2-buttons_width-10, #PB_Ignore, #PB_Ignore, #PB_Ignore)
        ResizeGadget(button_no_ID, WindowWidth(window_ID)/2+10, #PB_Ignore, #PB_Ignore, #PB_Ignore)
      ElseIf flags=#PB_MessageRequester_YesNoCancel
        ResizeGadget(button_yes_ID, WindowWidth(window_ID)/2-buttons_width*1.5-20, #PB_Ignore, #PB_Ignore, #PB_Ignore)
        ResizeGadget(button_no_ID, WindowWidth(window_ID)/2-buttons_width/2, #PB_Ignore, #PB_Ignore, #PB_Ignore)
        ResizeGadget(button_cancel_ID, WindowWidth(window_ID)/2+buttons_width/2+20, #PB_Ignore, #PB_Ignore, #PB_Ignore)
      EndIf
      
      HideWindow(window_ID, 0)     
      
      Macro messagerequester_magic_events
        If window_event=#PB_Event_CloseWindow : result=#PB_MessageRequester_Cancel : EndIf
        If window_event=#PB_Event_Gadget
          Select EventGadget()
            Case button_yes_ID
              result=#PB_MessageRequester_Yes
              Break
            Case button_no_ID
              result=#PB_MessageRequester_No
              Break
            Case button_cancel_ID
              result=#PB_MessageRequester_Cancel
              Break
            Case text_ID
              If Not swap_
                result=#PB_MessageRequester_Info
                Break
              EndIf
          EndSelect
        EndIf
      EndMacro
      
      Repeat
        window_event=WaitWindowEvent()
        messagerequester_magic_events
      Until result
      
      CloseWindow(window_ID)
      free_gadgets_BF_MessageRequester_Magic_BF
    EndIf
    
    ProcedureReturn result
    
  EndProcedure
  
  Procedure MessageRequester_Magic_make_text_clickable_BF(Messagerequester_Magic_make_text_clickable_)
    Messagerequester_Magic_make_text_clickable=Messagerequester_Magic_make_text_clickable_
    ProcedureReturn 1
  EndProcedure
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure MessageRequester_BF(Array font_ID(1),
                                  requester_width,
                                  requester_height,
                                  title_line$="",
                                  text$="",
                                  flags.a=#PB_MessageRequester_Yes,
                                  text_button_yes$="    Yes    ",
                                  text_button_no$="No",
                                  text_button_cancel$="Cancel",
                                  text_color=0,
                                  text_background_color=$F0F0F0,
                                  background_color=$F0F0F0,
                                  button_text_color=0,
                                  button_background_color=$E0E0E0,
                                  resize_factor_button_text.f=1.0,
                                  image_ID=-1,                               
                                  image_x_pos=0,
                                  image_y_pos=0,
                                  image_output_width=0,
                                  image_output_height=0,
                                  alpha=1)
      
      Protected offset_vertical=25
      
      Protected frame, frames, frame_delay, delay, image_width_text_image_ID=100, image_height_text_image_ID=100, temporary_frame_ID, temporary_image_ID
      Protected text_flag=#PB_Text_Center ; You can change
      Protected window_ID, button_yes_ID, button_no_ID, button_cancel_ID, canvas_ID, animation_buffer_ID, background_animation_buffer_ID
      Protected win_event, result, result_1, frame_width, frame_height, use_alpha_channel_image, is_gif, old_no_caching, old_ignore_gradient_colors
      Protected old_invisible_color.q, invisible_color_.q
      Protected pixel_format.a
      
      flags << 5 : flags >> 5
      
      If IsImage(image_ID) And (image_output_width=0 Or image_output_height=0)
        image_output_width=ImageWidth(image_ID)
        image_output_height=ImageHeight(image_ID)
      EndIf
      
      If activate_dpi_aware
        image_x_pos*DPI_ResizeFactor_BF()
        image_y_pos*DPI_ResizeFactor_BF()
      EndIf
      
      Protected calculated_font_size=Calculate_fitted_Font_BF(text$,
                                                              requester_width,  ; Requester width
                                                              requester_height, ; Requester height
                                                              Font_ID(),        ; Font for calculation
                                                              0,
                                                              20,               ; Text padding x 
                                                              20,               ; Text padding y
                                                              0,
                                                              0,
                                                              1)                ; resize_factor.f
      
      ; You can set a resize factor text, but this influence image positions on different scalings and is primary not needed
      ; calculated_font_size/resize_factor_text
      
      Protected calculated_font_size_buttons=calculated_font_size/resize_factor_button_text.f/1.2 ; Reduce the button font size a little presetted
      Protected text_font_ID=Font_ID(calculated_font_size_buttons)
      
      ; Use ever a large font for the buttons - Then the function adjust the needed font size automatically (about 60 for the mostly large buttons)
      Protected button_font_ID=Font_ID(ArraySize(Font_ID()))
      
      If flags=#PB_MessageRequester_Yes
        text_button_no$=" " : text_button_cancel$=" "
      ElseIf flags=#PB_MessageRequester_YesNo
        text_button_cancel$=" "
      EndIf
      
      ; Get the needed text width for the buttons - Multiline supported
      Protected buttons_width=GetTextWidth_BF(text_button_yes$+#LF$+text_button_no$+#LF$+text_button_cancel$, ; Get tricky the max needed width
                                              text_font_ID,
                                              0,  ; vertical
                                              0,  ; Stretch x
                                              30) ; Text padding x
      
      Protected get_amount_lines_drawtext=amount_lines_drawtext
      
      ; Get the needed text height for the buttons - Multiline supported
      Protected buttons_height=GetTextHeight_BF(" ", ; Get tricky the max needed height - Preset for Monoline
                                                text_font_ID,
                                                0, ; vertical
                                                0, ; Stretch y
                                                0) ; Text padding y
      
      buttons_height*(get_amount_lines_drawtext-2)+8
      
      If IsImage(image_ID)
        frames=ImageFrameCount_BF(image_ID) ; Count frames
        If frames>1 : is_gif=1 : EndIf
      EndIf
      
      If activate_dpi_aware
        calculated_font_size*DPI_ResizeFactor_BF()
        If calculated_font_size>ArraySize(Font_ID()) ; Fuse
          calculated_font_size=ArraySize(Font_ID())
        EndIf
        text_font_ID=Font_ID(calculated_font_size)
      EndIf
      
      Protected text_width.f=DrawTextCreateImage_BF(text$, 0, 0, 1, 3, text_font_ID, 0, 0, 20)
      Protected text_height.f=DrawTextCreateImage_BF(text$, 0, 0, 1, 4, text_font_ID, 0, 0, 20)
      
      Protected percent_x.f=100/requester_width*image_output_width
      Protected percent_y.f=100/requester_height*image_output_height
      
      image_output_width=text_width/100*percent_x
      image_output_height=text_height/100*percent_y
      
      text_height+20 
      
      animation_buffer_ID=CreateImage(#PB_Any, text_width, text_height, 24, text_background_color)
      background_animation_buffer_ID=CreateImage(#PB_Any, text_width, text_height, 24, text_background_color)
      
      If Not IsImage(animation_buffer_ID) Or Not IsImage(background_animation_buffer_ID)
        procedurereturn_messagerequester : ProcedureReturn -6
      EndIf
      
      If activate_dpi_aware
        image_width_text_image_ID=ImageWidth(animation_buffer_ID)/DPI_ResizeFactor_BF()
        image_height_text_image_ID=ImageHeight(animation_buffer_ID)/DPI_ResizeFactor_BF()
        Protected x_pos_messagerequester_=x_pos_messagerequester
        Protected y_pos_messagerequester_=y_pos_messagerequester
      Else
        image_width_text_image_ID=ImageWidth(animation_buffer_ID)
        image_height_text_image_ID=ImageHeight(animation_buffer_ID)
        x_pos_messagerequester_=x_pos_messagerequester
        y_pos_messagerequester_=y_pos_messagerequester
      EndIf
      
      If mode_messagerequester
        If IsWindow(parent_window_message_requester_ID)
          old_gadget_list=UseGadgetList(WindowID(parent_window_message_requester_ID))
        EndIf  
      EndIf
      If mode_messagerequester<2 Or x_pos_messagerequester=-1 Or y_pos_messagerequester=-1
        Protected PB_Window_ScreenCentered=#PB_Window_ScreenCentered
      EndIf
      
      If flags=#PB_MessageRequester_Yes
        If image_width_text_image_ID<buttons_width
          window_ID=OpenWindow(#PB_Any, x_pos_messagerequester_, y_pos_messagerequester_, buttons_width+20, image_height_text_image_ID+buttons_height+offset_vertical, " "+title_line$, #PB_Window_SystemMenu|PB_Window_ScreenCentered|#PB_Window_Invisible) 
          If Not IsWindow(window_ID) : procedurereturn_messagerequester : ProcedureReturn -9 : EndIf
          canvas_ID=CanvasGadget(#PB_Any, 0, 0, buttons_width+20, image_height_text_image_ID)
          If Not IsGadget(canvas_ID) : procedurereturn_messagerequester : ProcedureReturn -9 : EndIf
        Else
          window_ID=OpenWindow(#PB_Any, x_pos_messagerequester_, y_pos_messagerequester_, image_width_text_image_ID+20, image_height_text_image_ID+buttons_height+offset_vertical, " "+title_line$, #PB_Window_SystemMenu|PB_Window_ScreenCentered|#PB_Window_Invisible)
          If Not IsWindow(window_ID) : procedurereturn_messagerequester : ProcedureReturn -9 : EndIf
          canvas_ID=CanvasGadget(#PB_Any, 0, 0, image_width_text_image_ID+20, image_height_text_image_ID)
          If Not IsGadget(canvas_ID) : procedurereturn_messagerequester : ProcedureReturn -9 : EndIf
        EndIf
      ElseIf flags=#PB_MessageRequester_YesNo
        If image_width_text_image_ID<buttons_width*2
          window_ID=OpenWindow(#PB_Any, x_pos_messagerequester_, y_pos_messagerequester_, buttons_width*2+40, image_height_text_image_ID+buttons_height+offset_vertical, " "+title_line$, #PB_Window_SystemMenu|PB_Window_ScreenCentered|#PB_Window_Invisible)
          If Not IsWindow(window_ID) : procedurereturn_messagerequester : ProcedureReturn -9 : EndIf
          canvas_ID=CanvasGadget(#PB_Any, 0, 0, buttons_width*2+40, image_height_text_image_ID)
        Else
          window_ID=OpenWindow(#PB_Any, x_pos_messagerequester_, y_pos_messagerequester_, image_width_text_image_ID+40, image_height_text_image_ID+buttons_height+offset_vertical, " "+title_line$, #PB_Window_SystemMenu|PB_Window_ScreenCentered|#PB_Window_Invisible)
          If Not IsWindow(window_ID) : procedurereturn_messagerequester : ProcedureReturn -9 : EndIf
          canvas_ID=CanvasGadget(#PB_Any, 0, 0, image_width_text_image_ID+40, image_height_text_image_ID)
          If Not IsGadget(canvas_ID) : procedurereturn_messagerequester : ProcedureReturn -9 : EndIf
        EndIf
      Else
        If image_width_text_image_ID<buttons_width*3
          window_ID=OpenWindow(#PB_Any, x_pos_messagerequester_, y_pos_messagerequester_, buttons_width*3+60, image_height_text_image_ID+buttons_height+offset_vertical, " "+title_line$, #PB_Window_SystemMenu|PB_Window_ScreenCentered|#PB_Window_Invisible)
          If Not IsWindow(window_ID) : procedurereturn_messagerequester : ProcedureReturn -9 : EndIf
          canvas_ID=CanvasGadget(#PB_Any, 0, 0, buttons_width*3+60, image_height_text_image_ID)
        Else
          window_ID=OpenWindow(#PB_Any, x_pos_messagerequester_, y_pos_messagerequester_, image_width_text_image_ID+60, image_height_text_image_ID+buttons_height+offset_vertical, " "+title_line$, #PB_Window_SystemMenu|PB_Window_ScreenCentered|#PB_Window_Invisible)
          If Not IsWindow(window_ID) : procedurereturn_messagerequester : ProcedureReturn -9 : EndIf
          canvas_ID=CanvasGadget(#PB_Any, 0, 0, image_width_text_image_ID+60, image_height_text_image_ID)
          If Not IsGadget(canvas_ID) : procedurereturn_messagerequester : ProcedureReturn -9 : EndIf
        EndIf
      EndIf
      
      If mode_messagerequester=3
        If x_pos_messagerequester=-1
          ResizeWindow(window_ID,
                       #PB_Ignore,
                       y_pos_messagerequester_,
                       #PB_Ignore,
                       #PB_Ignore)
        EndIf
        If y_pos_messagerequester=-1
          ResizeWindow(window_ID,
                       x_pos_messagerequester_,
                       #PB_Ignore,    
                       #PB_Ignore,
                       #PB_Ignore)
        EndIf
      EndIf
      
      If IsWindow(parent_window_message_requester_ID)
        If mode_messagerequester=2
          If Not x_pos_messagerequester And Not y_pos_messagerequester
            ResizeWindow(window_ID, WindowX(parent_window_message_requester_ID)+(WindowWidth(parent_window_message_requester_ID)/2)-WindowWidth(window_ID)/2,
                         WindowY(parent_window_message_requester_ID)+(WindowHeight(parent_window_message_requester_ID)/2)-WindowHeight(window_ID)/2,
                         #PB_Ignore,
                         #PB_Ignore)
          EndIf
          If x_pos_messagerequester=-1
            ResizeWindow(window_ID, WindowX(parent_window_message_requester_ID)+(WindowWidth(parent_window_message_requester_ID)/2)-WindowWidth(window_ID)/2,
                         WindowY(parent_window_message_requester_ID)+y_pos_messagerequester_,
                         #PB_Ignore,
                         #PB_Ignore)
          EndIf
          If y_pos_messagerequester=-1
            ResizeWindow(window_ID, WindowX(parent_window_message_requester_ID)+x_pos_messagerequester_,
                         WindowY(parent_window_message_requester_ID)+(WindowHeight(parent_window_message_requester_ID)/2)-WindowHeight(window_ID)/2,
                         #PB_Ignore,
                         #PB_Ignore)
          EndIf
        EndIf
      EndIf
      
      StickyWindow(window_ID, 1)
      
      If StartDrawing(CanvasOutput(canvas_ID))
        If activate_dpi_aware
          Box(0, 0, WindowWidth(window_ID)*DPI_ResizeFactor_BF(), WindowHeight(window_ID)*DPI_ResizeFactor_BF(), background_color)
        Else
          Box(0, 0, WindowWidth(window_ID), WindowHeight(window_ID), background_color)
        EndIf
        StopDrawing()
      Else
        procedurereturn_messagerequester : ProcedureReturn -5
      EndIf
      
      SetWindowColor(window_ID, background_color)
      
      If IsFont(button_font_ID)
        SetGadgetFont(#PB_Default, FontID(button_font_ID))
      EndIf
      
      old_ignore_gradient_colors=ignore_gradient_colors
      ignore_gradient_colors=1
      
      Protected old_manual_padding_y=manual_padding_draw_text_create_image_y
      manual_padding_draw_text_create_image_y=15
      
      button_yes_ID=ColorButtonGadget_DualPadding_BF(text_button_yes$,
                                                     WindowWidth(window_ID)/2-buttons_width-buttons_width/2-10,
                                                     WindowHeight(window_ID)-buttons_height-10,
                                                     buttons_width,
                                                     buttons_height,
                                                     button_text_color,
                                                     button_background_color,
                                                     15, ; Padding x
                                                     5,  ; Padding y
                                                     button_font_ID)
      
      button_no_ID=ColorButtonGadget_DualPadding_BF(text_button_no$,
                                                    WindowWidth(window_ID)/2-buttons_width/2,
                                                    WindowHeight(window_ID)-buttons_height-10,
                                                    buttons_width,
                                                    buttons_height,
                                                    button_text_color,
                                                    button_background_color,
                                                    15, ; Padding x
                                                    5,  ; Padding y 
                                                    button_font_ID)
      
      button_cancel_ID=ColorButtonGadget_DualPadding_BF(text_button_cancel$,
                                                        WindowWidth(window_ID)/2+buttons_width/2+10,
                                                        WindowHeight(window_ID)-buttons_height-10,
                                                        buttons_width,
                                                        buttons_height,
                                                        button_text_color,
                                                        button_background_color,
                                                        15, ; Padding x
                                                        5,  ; Padding y
                                                        button_font_ID)
      
      manual_padding_draw_text_create_image_y=old_manual_padding_y
      
      ignore_gradient_colors=old_ignore_gradient_colors
      
      If IsFont(button_font_ID)
        SetGadgetFont(#PB_Default, #PB_Default)
      EndIf
      
      If Not canvas_ID Or Not button_yes_ID Or Not button_no_ID Or Not button_cancel_ID
        procedurereturn_messagerequester : ProcedureReturn -9
      EndIf
      
      If flags=#PB_MessageRequester_Yes
        HideGadget(button_no_ID, 1)
        HideGadget(button_cancel_ID, 1)
        ResizeGadget(button_yes_ID, WindowWidth(window_ID)/2-GadgetWidth(button_yes_ID)/2, #PB_Ignore, #PB_Ignore, #PB_Ignore)
      ElseIf flags=#PB_MessageRequester_YesNo
        HideGadget(button_cancel_ID, 1)
        ResizeGadget(button_yes_ID, WindowWidth(window_ID)/2-GadgetWidth(button_yes_ID)-5, #PB_Ignore, #PB_Ignore, #PB_Ignore)
        ResizeGadget(button_no_ID, WindowWidth(window_ID)/2, #PB_Ignore, #PB_Ignore, #PB_Ignore)
      EndIf
      
      If IsImage(image_ID)
        frame_width=ImageWidth(image_ID)   ; Frame width
        frame_height=ImageHeight(image_ID) ; Frame height
        
        temporary_frame_ID=CreateImage(#PB_Any, frame_width, frame_height) ; Create a temporary frame
        
        old_invisible_color.q=invisible_color
        
        If is_gif
          ; invisible_color_=$A7A8A9 ; You can preset here a invisible color - Define not the same as the GIF content color or you get "Holes"
          invisible_color_=Create_invisible_GIF_color_BF(image_ID) ; This create automatic a new and common invisible color for the GIF output as sprite
          If invisible_color_<1 : invisible_color_=Random($FFFFFE, 1) : EndIf ; Fuse - Error handling for automatic invisible color generating
        EndIf
        StartDrawing(ImageOutput(image_ID))
        pixel_format=DrawingBufferPixelFormat()
        If frames<2 And (pixel_format=#PB_PixelFormat_32Bits_RGB Or pixel_format=#PB_PixelFormat_32Bits_BGR)
          use_alpha_channel_image=1
        EndIf
        StopDrawing()
      EndIf
      
      If use_alpha_channel_image
        temporary_image_ID=CopyImage(image_ID, #PB_Any)
        If Not temporary_image_ID
          ProcedureReturn -6
        EndIf
        If Not ResizeImage(temporary_image_ID, image_output_width, image_output_height, resize_interpolation)
          FreeImage(temporary_image_ID)
          ProcedureReturn -27
        EndIf
      EndIf
      
      invisible_color=invisible_color_
      old_no_caching=no_caching
      no_caching=1
      
      Delay_internal(1, -1) ; Init internal timer 1 (Set at start to zero - This skip the first timer delay)
      
      HideWindow(window_ID, 0)
      
      While WindowEvent() : Wend
      
      Repeat
        
        If Delay_internal(1, 10) ; Animation Refresh
          
          If IsImage(image_ID)
            
            If is_gif
              
              SetImageFrame(image_ID, frame) ; Set the GIF frame for processing
              frame_delay=GetImageFrameDelay(image_ID) ; Get the GIF frame delay
              
              If Not frame_delay : frame_delay=100 : EndIf ; Delay 0 works not, so as a standard, delay 0 = delay 100 
              If delay : delay=0 : frame+1 : If frame=>frames : frame=0 : EndIf : Else : delay+Delay_internal(2, frame_delay) : EndIf
              
            EndIf 
            
            StartDrawing(ImageOutput(temporary_frame_ID))          ; Draw in the temporary frame buffer
            Box(0, 0, frame_width, frame_height, invisible_color_) ; Draw the invisible color
            DrawAlphaImage(ImageID(image_ID), 0, 0)                ; Draw the GIF frame in the temporary frame buffer
            StopDrawing()
            
            If use_alpha_channel_image
              result_1=RotateAlphaChannelSprite_fast_BF(animation_buffer_ID, temporary_image_ID,                                                                         
                                                        image_x_pos,
                                                        image_y_pos,
                                                        0,     ; Rotating degree
                                                        0,     ; mode=1 rotating on - mode=2 rotating on and create a quadratic sprite for on demand rotation - Mode 1 has no effekt on linux
                                                        alpha) ; Set alpha blending from 1 to 255 - BF set 1=full visible - 256=invisible (BF specific)
                                                               ; Additional parameters here not used
            Else
              result_1=SpriteSimple_fast_BF(alpha, animation_buffer_ID, temporary_frame_ID,                                     
                                            0, ; Grab transparence color from the frame pos x - Set to -1 for output without invisible color 
                                            0, ; Grab transparence color from the frame pos y - You can set x or y, or both parameters to -1 
                                            image_x_pos,
                                            image_y_pos, 
                                            image_output_width,
                                            image_output_height)
            EndIf
            
            If result_1<1 : Break: EndIf
          EndIf
          
          StartDrawing(ImageOutput(animation_buffer_ID)) ; Draw in the temporary frame buffer
          DrawingMode(#PB_2DDrawing_Transparent)
          If IsFont(text_font_ID) : DrawingFont(FontID(text_font_ID)) : EndIf
          DrawText_BF(10, 10, text$, text_color, 0, 1, 0, 0, -1)
          StopDrawing()
          
          ; Get the result
          If activate_dpi_aware
            CopyContent_Image_Canvas_BF(animation_buffer_ID, canvas_ID, WindowWidth(window_ID)/2*DPI_ResizeFactor_BF()-ImageWidth(animation_buffer_ID)/2, offset_vertical*DPI_ResizeFactor_BF()/2) ; Put the hidden animation buffer in the canvas
          Else
            CopyContent_Image_Canvas_BF(animation_buffer_ID, canvas_ID, WindowWidth(window_ID)/2-ImageWidth(animation_buffer_ID)/2, offset_vertical/2) ; Put the hidden animation buffer in the canvas
          EndIf
          CopyContent_Image_Image_BF(background_animation_buffer_ID, animation_buffer_ID)
        EndIf
        
        If is_gif
          win_event=WaitWindowEvent(1)
        Else
          Repeat
            win_event=WaitWindowEvent()
            If win_event=#PB_Event_CloseWindow : result=#PB_MessageRequester_Cancel : EndIf
            If win_event=#PB_Event_Gadget
              Select EventGadget()
                Case button_yes_ID
                  result=#PB_MessageRequester_Yes
                Case button_no_ID
                  result=#PB_MessageRequester_No
                Case button_cancel_ID
                  result=#PB_MessageRequester_Cancel
              EndSelect
            EndIf
            If result : Break : EndIf
          ForEver
        EndIf
        If win_event=#PB_Event_CloseWindow : result=#PB_MessageRequester_Cancel : EndIf
        If win_event=#PB_Event_Gadget
          Select EventGadget()
            Case button_yes_ID
              result=#PB_MessageRequester_Yes
            Case button_no_ID
              result=#PB_MessageRequester_No
            Case button_cancel_ID
              result=#PB_MessageRequester_Cancel
          EndSelect
        EndIf
        
      Until result
      
      invisible_color=old_invisible_color.q
      no_caching=old_no_caching
      If IsImage(temporary_frame_ID)
        FreeImage(temporary_frame_ID)
      EndIf
      If IsImage(temporary_image_ID)
        FreeImage(temporary_image_ID)
      EndIf
      FreeImage(animation_buffer_ID)
      FreeImage(background_animation_buffer_ID)
      FreeGadget_BF(button_yes_ID)
      FreeGadget_BF(button_no_ID)
      FreeGadget_BF(button_cancel_ID)
      FreeGadget(canvas_ID)
      CloseWindow(window_ID) ; At least
      procedurereturn_messagerequester : ProcedureReturn result
      
    EndProcedure
  CompilerEndIf
  
  Procedure SetMessageRequester_BF(parent_window_message_requester_ID_, mode_messagerequester_, x_pos_messagerequester_=0, y_pos_messagerequester_=0)
    If mode_messagerequester_>3 Or mode_messagerequester_<0
      ProcedureReturn 0
    EndIf
    x_pos_messagerequester=x_pos_messagerequester_
    y_pos_messagerequester=y_pos_messagerequester_
    mode_messagerequester=mode_messagerequester_
    parent_window_message_requester_ID=parent_window_message_requester_ID_
    ProcedureReturn 1
  EndProcedure
  
  Procedure CreateFittedTextImage_BF(text$,
                                     image_width,
                                     image_height,
                                     text_color=$FFFFFF,
                                     background_color=0,
                                     font_ID=-1,
                                     stretch_x=0,
                                     stretch_y=0,
                                     flag=1,
                                     mode=0)
    
    Protected image_1_ID=DrawTextCreateImage_BF(text$,
                                                text_color,
                                                background_color,
                                                flag,
                                                mode,
                                                font_ID,
                                                stretch_x, 
                                                stretch_y,
                                                -1)
    
    Protected image_ID=FitImage_BF(0,
                                   image_1_ID,
                                   image_width,
                                   image_height,
                                   background_color,
                                   0,
                                   $FF,
                                   0,
                                   0,
                                   0,
                                   0)
    FreeImage_BF(image_1_ID)
    
    ProcedureReturn image_ID
  EndProcedure
  
  Procedure CreateFittedTextImage_DualPadding_BF(text$,
                                                 image_width,
                                                 image_height,
                                                 text_color=$FFFFFF,
                                                 background_color=0,
                                                 padding_x=0,
                                                 padding_y=0,
                                                 font_ID=-1,
                                                 stretch_x=0,
                                                 stretch_y=0,
                                                 flag=1,
                                                 mode=0)
    
    Protected old_manual_padding_draw_text_create_image_x=manual_padding_draw_text_create_image_x
    Protected old_manual_padding_draw_text_create_image_y=manual_padding_draw_text_create_image_x
    manual_padding_draw_text_create_image_x=padding_x
    manual_padding_draw_text_create_image_y=padding_y
    
    Protected image_1_ID=DrawTextCreateImage_BF(text$,
                                                text_color,
                                                background_color,
                                                flag,
                                                mode,
                                                font_ID,
                                                stretch_x, 
                                                stretch_y,
                                                -1)
    
    manual_padding_draw_text_create_image_x=old_manual_padding_draw_text_create_image_x
    manual_padding_draw_text_create_image_y=old_manual_padding_draw_text_create_image_x
    
    Protected image_ID=FitImage_DualPadding_BF(0,
                                               image_1_ID,
                                               image_width,
                                               image_height,
                                               background_color,
                                               padding_x,
                                               padding_y,
                                               $FF,
                                               0,
                                               0,
                                               0,
                                               0)
    FreeImage_BF(image_1_ID)
    
    ProcedureReturn image_ID
  EndProcedure
  
  Procedure gtk3_button_x_offset_BF(gtk3_x_offset_)
    gtk3_x_offset=gtk3_x_offset_
  EndProcedure
  
  Procedure gtk3_button_y_offset_BF(gtk3_y_offset_)
    gtk3_y_offset=gtk3_y_offset_
  EndProcedure
  
  Procedure Get_gtk3_button_x_offset_BF()
    ProcedureReturn gtk3_x_offset
  EndProcedure
  
  Procedure Get_gtk3_button_y_offset_BF()
    ProcedureReturn gtk3_y_offset
  EndProcedure
  
  Procedure SetColorButtonGadget_BF_FadeOut_BF(gadget_ID,
                                               text_a$,
                                               text_b$,
                                               text_color,
                                               background_color,
                                               animation_delay,
                                               animation_alpha_step)
    
    Protected i, i1=255
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    For i=0 To 255 
      i1-animation_alpha_step
      If i1<0 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    background_color, ; Overlay color
                                    i1)               ; Overlay alpha    
      SetColorButtonGadget_DualPadding_BF(gadget_ID,
                                          text_a$,
                                          text_color,
                                          background_color)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    SetColorButtonGadget_DualPadding_BF(gadget_ID,
                                        text_b$,
                                        text_color,
                                        background_color)
    
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetColorButtonGadget_BF_FadeIn_BF(gadget_ID,
                                              text_a$,
                                              text_b$,
                                              text_color,
                                              background_color,
                                              animation_delay,
                                              animation_alpha_step)
    
    Protected i, i1
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    
    For i=0 To 255
      i1+animation_alpha_step
      If i1>255 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    background_color, ; Overlay color
                                    i1)               ; Overlay alpha
      SetColorButtonGadget_DualPadding_BF(gadget_ID,
                                          text_b$,
                                          text_color,
                                          background_color)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetColorButtonGadget_BF_Fade_ChangeTextColor_BF(gadget_ID,
                                                            text_a$,
                                                            text_b$,
                                                            text_color_a,
                                                            text_color_b,
                                                            background_color,
                                                            animation_delay,
                                                            animation_alpha_step)
    
    SetColorButtonGadget_BF_FadeOut_BF(gadget_ID,
                                       text_b$,
                                       text_b$,
                                       text_color_b,
                                       background_color,
                                       animation_delay,
                                       animation_alpha_step)
    
    SetColorButtonGadget_BF_FadeIn_BF(gadget_ID,
                                      text_a$,
                                      text_a$,
                                      text_color_a,
                                      background_color,
                                      animation_delay,
                                      animation_alpha_step)
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetColorButtonGadget_BF_AutoFade_BF(gadget_ID,
                                                text_a$,
                                                text_b$,
                                                animation_delay,
                                                animation_alpha_step)
    
    Protected i, i1=255, old_get_gadget_data=GetGadgetData(gadget_ID)
    old_get_gadget_data=1-old_get_gadget_data ; Toggle flag
    SetGadgetData(gadget_ID, old_get_gadget_data) ; Set gadget states for post processing
    If Not GetGadgetData(gadget_ID)
      Protected text_c$=text_a$ : text_a$=text_b$ : text_b$=text_c$ : text_c$=""
    EndIf
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    For i=0 To 255 
      i1-animation_alpha_step
      If i1<0 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    GetGadgetBackgroundColor_BF(gadget_ID), ; Overlay color
                                    i1)                                     ; Overlay alpha    
      SetColorButtonGadget_DualPadding_BF(gadget_ID,
                                          text_a$,
                                          GetGadgetTextColor_BF(gadget_ID),
                                          GetGadgetBackgroundColor_BF(gadget_ID))
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    SetColorButtonGadget_DualPadding_BF(gadget_ID,
                                        text_b$,
                                        GetGadgetTextColor_BF(gadget_ID),
                                        GetGadgetBackgroundColor_BF(gadget_ID))
    i1=0
    For i=0 To 255
      i1+animation_alpha_step
      If i1>255 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    GetGadgetBackgroundColor_BF(gadget_ID), ; Overlay color
                                    i1)                                     ; Overlay alpha
      SetColorButtonGadget_DualPadding_BF(gadget_ID,
                                          text_b$,
                                          GetGadgetTextColor_BF(gadget_ID),
                                          GetGadgetBackgroundColor_BF(gadget_ID))
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn GetGadgetData(gadget_ID)
  EndProcedure
  
  Procedure SetColorButtonGadget_BF_Fade_BF(gadget_ID,
                                            text_a$,
                                            text_b$,
                                            text_color,
                                            background_color,
                                            animation_delay,
                                            animation_alpha_step)
    
    Protected i, i1=255
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    For i=0 To 255 
      i1-animation_alpha_step
      If i1<0 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    background_color, ; Overlay color
                                    i1)               ; Overlay alpha    
      SetColorButtonGadget_DualPadding_BF(gadget_ID,
                                          text_a$,
                                          text_color,
                                          background_color)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    SetColorButtonGadget_DualPadding_BF(gadget_ID,
                                        text_b$,
                                        text_color,
                                        background_color)
    i1=0
    For i=0 To 255
      i1+animation_alpha_step
      If i1>255 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    background_color, ; Overlay color
                                    i1)               ; Overlay alpha
      SetColorButtonGadget_DualPadding_BF(gadget_ID,
                                          text_b$,
                                          text_color,
                                          background_color)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetColorButtonGadget_BF_Fading_BF(gadget_ID,
                                              text_a$,
                                              text_b$,
                                              text_color_a,
                                              text_color_b,
                                              animation_delay,
                                              animation_alpha_step)
    
    SetGadgetData(gadget_ID, ~GetGadgetData(gadget_ID))
    
    If GetGadgetData(gadget_ID) 
      SetColorButtonGadget_BF_FadeOut_BF(gadget_ID,
                                         text_a$,
                                         text_a$,
                                         text_color_a,
                                         GetGadgetBackgroundColor_BF(gadget_ID),
                                         animation_delay,
                                         animation_alpha_step)
      
      SetColorButtonGadget_BF_FadeIn_BF(gadget_ID,
                                        text_b$,
                                        text_b$,
                                        text_color_b,
                                        GetGadgetBackgroundColor_BF(gadget_ID),
                                        animation_delay,
                                        animation_alpha_step)
      
    Else
      
      SetColorButtonGadget_BF_FadeOut_BF(gadget_ID,
                                         text_b$,
                                         text_b$,
                                         text_color_b,
                                         GetGadgetBackgroundColor_BF(gadget_ID),
                                         animation_delay,
                                         animation_alpha_step)
      
      SetColorButtonGadget_BF_FadeIn_BF(gadget_ID,
                                        text_a$,
                                        text_a$,
                                        text_color_a,
                                        GetGadgetBackgroundColor_BF(gadget_ID),
                                        animation_delay,
                                        animation_alpha_step)
      
    EndIf
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetColorButtonGadget_BF_FlashText_BF(gadget_ID)
    SetColorButtonGadget_BF_AutoFade_BF(gadget_ID,
                                        GetGadgetText_BF(gadget_ID),
                                        GetGadgetText_BF(gadget_ID),
                                        3,
                                        10)
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetColorButtonGadget_BF(gadget_ID,
                                    text$="",
                                    text_color=-1,
                                    background_color=-1,
                                    x=-1,
                                    y=-1,
                                    button_width=-1,
                                    button_height=-1,
                                    stretch_x=-1,
                                    stretch_y=-1,
                                    font_ID=-1)
    
    If Not IsGadget(gadget_ID)
      ProcedureReturn -23
    EndIf
    
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        If IsImage(gadget_bf()\image_ID)
          FreeImage(gadget_bf()\image_ID)
        EndIf
        If text$=""
          text$=gadget_bf()\text$
        EndIf
        If text_color<0
          text_color=gadget_bf()\text_color
        EndIf
        If background_color<0
          background_color=gadget_bf()\background_color
        EndIf
        If x=-1
          x=gadget_bf()\x
        EndIf
        If y=-1
          y=gadget_bf()\y
        EndIf
        If button_width<0
          button_width=gadget_bf()\button_width
        EndIf
        If button_height<0
          button_height=gadget_bf()\button_height
        EndIf
        If stretch_x<0
          stretch_x=gadget_bf()\stretch_x
        EndIf
        If stretch_y<0
          stretch_y=gadget_bf()\stretch_y
        EndIf
        If font_ID<0
          font_ID=gadget_bf()\font_ID
        EndIf
        Protected flag=gadget_bf()\flag
        Protected mode=gadget_bf()\mode
        Break
      EndIf
    Next
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected image_ID=CreateFittedTextImage_BF(text$,
                                                button_width_-gtk3_x_offset,
                                                button_height_-gtk3_y_offset,
                                                text_color,
                                                background_color,
                                                font_ID,
                                                stretch_x,
                                                stretch_y,
                                                flag,
                                                mode)
    
    If Not IsImage(image_ID)
      ProcedureReturn -6
    EndIf
    
    ResizeGadget(gadget_ID, x, y, button_width, button_height)
    SetGadgetAttribute(gadget_ID, #PB_Button_Image, ImageID(image_ID))
    
    gadget_bf()\image_ID=image_ID
    gadget_bf()\text$=text$
    gadget_bf()\text_color=text_color
    gadget_bf()\background_color=background_color
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\stretch_x=stretch_x
    gadget_bf()\stretch_y=stretch_y
    gadget_bf()\font_ID=font_ID
    
    ProcedureReturn image_ID 
  EndProcedure
  
  Procedure ColorButtonGadget_BF(text$,
                                 x=0,
                                 y=0,
                                 button_width=0,
                                 button_height=0,
                                 text_color=0,
                                 background_color=$E1E1E1,
                                 font_ID=-1,
                                 stretch_x=0,
                                 stretch_y=0,
                                 flag=1,
                                 mode=0,
                                 button_flag=0)
    
    If text$=""
      ProcedureReturn -26
    EndIf
    
    button_width-gtk3_x_offset
    button_height-gtk3_y_offset
    
    If button_width<1 : button_width=1 : EndIf
    If button_height<1 : button_height=1 : EndIf
    
    If x=-2 And IsGadget(y) ; Import a gadget
      ResizeGadget(y, #PB_Ignore, #PB_Ignore, button_width, button_height)
      Protected gadget_ID=y : x=GadgetX(y) : y=GadgetY(y)
    Else
      gadget_ID=ButtonImageGadget(#PB_Any, x, y, button_width, button_height, 0, button_flag)
      If Not IsGadget(gadget_ID)
        ProcedureReturn -23
      EndIf
    EndIf
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected image_ID=CreateFittedTextImage_BF(text$,
                                                button_width_-gtk3_x_offset,
                                                button_height_-gtk3_y_offset,
                                                text_color,
                                                background_color,
                                                font_ID,
                                                stretch_x,
                                                stretch_y,
                                                flag,
                                                mode)
    
    If Not IsImage(image_ID)
      ProcedureReturn -6
    EndIf
    
    SetGadgetAttribute(gadget_ID, #PB_Button_Image, ImageID(image_ID))
    
    AddElement(gadget_bf())
    gadget_bf()\gadget_type=#ColorButtonGadget_BF
    gadget_bf()\gadget_ID=gadget_ID
    gadget_bf()\image_ID=image_ID
    gadget_bf()\text$=text$
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\text_color=text_color
    gadget_bf()\background_color=background_color
    gadget_bf()\font_ID=font_ID
    gadget_bf()\stretch_x=stretch_x
    gadget_bf()\stretch_y=stretch_y
    gadget_bf()\flag=flag
    gadget_bf()\mode=mode
    gadget_bf()\gadget_flag=button_flag
    
    ProcedureReturn gadget_ID
  EndProcedure
  
  Procedure SetColorButtonGadget_DualPadding_BF(gadget_ID,
                                                text$="",
                                                text_color=-1,
                                                background_color=-1,
                                                padding_x_gadget=-1,
                                                padding_y_gadget=-1,
                                                x=-1,
                                                y=-1,
                                                button_width=-1,
                                                button_height=-1,
                                                stretch_x=-1,
                                                stretch_y=-1,
                                                font_ID=-1)
    
    If Not IsGadget(gadget_ID)
      ProcedureReturn -23
    EndIf
    
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        If IsImage(gadget_bf()\image_ID)
          FreeImage(gadget_bf()\image_ID)
        EndIf
        If text$=""
          text$=gadget_bf()\text$
        EndIf
        If text_color<0
          text_color=gadget_bf()\text_color
        EndIf
        If background_color<0
          background_color=gadget_bf()\background_color
        EndIf
        If padding_x_gadget<0
          padding_x_gadget=gadget_bf()\padding_x_gadget
        EndIf
        If padding_y_gadget<0
          padding_y_gadget=gadget_bf()\padding_y_gadget
        EndIf
        If x=-1
          x=gadget_bf()\x
        EndIf
        If y=-1
          y=gadget_bf()\y
        EndIf
        If button_width<0
          button_width=gadget_bf()\button_width
        EndIf
        If button_height<0
          button_height=gadget_bf()\button_height
        EndIf
        If stretch_x<0
          stretch_x=gadget_bf()\stretch_x
        EndIf
        If stretch_y<0
          stretch_y=gadget_bf()\stretch_y
        EndIf
        If font_ID<0
          font_ID=gadget_bf()\font_ID
        EndIf
        Protected flag=gadget_bf()\flag
        Protected mode=gadget_bf()\mode
        Break
      EndIf
    Next
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected image_ID=CreateFittedTextImage_DualPadding_BF(text$,
                                                            button_width_-gtk3_x_offset,
                                                            button_height_-gtk3_y_offset,
                                                            text_color,
                                                            background_color,
                                                            padding_x_gadget,
                                                            padding_y_gadget,
                                                            font_ID,
                                                            stretch_x,
                                                            stretch_y,
                                                            flag,
                                                            mode)
    
    If Not IsImage(image_ID)
      ProcedureReturn -6
    EndIf
    
    ResizeGadget(gadget_ID, x, y, button_width, button_height)
    SetGadgetAttribute(gadget_ID, #PB_Button_Image, ImageID(image_ID))
    
    gadget_bf()\image_ID=image_ID
    gadget_bf()\text$=text$
    gadget_bf()\text_color=text_color
    gadget_bf()\background_color=background_color
    gadget_bf()\padding_x_gadget=padding_x_gadget
    gadget_bf()\padding_y_gadget=padding_y_gadget
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\stretch_x=stretch_x
    gadget_bf()\stretch_y=stretch_y
    gadget_bf()\font_ID=font_ID
    
    ProcedureReturn image_ID 
  EndProcedure
  
  Procedure ColorButtonGadget_DualPadding_BF(text$,
                                             x=0,
                                             y=0,
                                             button_width=0,
                                             button_height=0,
                                             text_color=0,
                                             background_color=$E1E1E1,
                                             padding_x_gadget=0,
                                             padding_y_gadget=0,
                                             font_ID=-1,
                                             stretch_x=0,
                                             stretch_y=0,
                                             flag=1,
                                             mode=0,
                                             button_flag=0)
    
    If text$=""
      ProcedureReturn -26
    EndIf
    
    button_width-gtk3_x_offset
    button_height-gtk3_y_offset
    
    If button_width<1 : button_width=1 : EndIf
    If button_height<1 : button_height=1 : EndIf
    
    If x=-2 And IsGadget(y) ; Import a gadget
      ResizeGadget(y, #PB_Ignore, #PB_Ignore, button_width, button_height)
      Protected gadget_ID=y : x=GadgetX(y) : y=GadgetY(y)
    Else
      gadget_ID=ButtonImageGadget(#PB_Any, x, y, button_width, button_height, 0, button_flag)
      If Not IsGadget(gadget_ID)
        ProcedureReturn -23
      EndIf
    EndIf
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected image_ID=CreateFittedTextImage_DualPadding_BF(text$,
                                                            button_width_-gtk3_x_offset,
                                                            button_height_-gtk3_y_offset,
                                                            text_color,
                                                            background_color,
                                                            padding_x_gadget,
                                                            padding_y_gadget,
                                                            font_ID,
                                                            stretch_x,
                                                            stretch_y,
                                                            flag,
                                                            mode)
    
    If Not IsImage(image_ID)
      ProcedureReturn -6
    EndIf
    
    SetGadgetAttribute(gadget_ID, #PB_Button_Image, ImageID(image_ID))
    
    AddElement(gadget_bf())
    gadget_bf()\gadget_type=#ColorButtonGadget_DualPadding_BF
    gadget_bf()\gadget_ID=gadget_ID
    gadget_bf()\image_ID=image_ID
    gadget_bf()\text$=text$
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\text_color=text_color
    gadget_bf()\background_color=background_color
    gadget_bf()\padding_x_gadget=padding_x_gadget
    gadget_bf()\padding_y_gadget=padding_y_gadget
    gadget_bf()\font_ID=font_ID
    gadget_bf()\stretch_x=stretch_x
    gadget_bf()\stretch_y=stretch_y
    gadget_bf()\flag=flag
    gadget_bf()\mode=mode
    gadget_bf()\gadget_flag=button_flag
    
    ProcedureReturn gadget_ID
  EndProcedure
  
  Procedure SetButtonImageGadget_BF_AutoFade_BF(gadget_ID,
                                                image_a_ID,
                                                image_b_ID,
                                                background_color,
                                                overlay_color,
                                                animation_delay,
                                                animation_alpha_step)
    Protected i, i1, old_get_gadget_data=GetGadgetData(gadget_ID)
    old_get_gadget_data=1-old_get_gadget_data ; Toggle flag
    SetGadgetData(gadget_ID, old_get_gadget_data) ; Set gadget states for post processing
    If Not GetGadgetData(gadget_ID) : Swap image_a_ID, image_b_ID : EndIf
    i1=255
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    For i=0 To 255 
      i1-animation_alpha_step
      If i1<0 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    overlay_color, ; Overlay color
                                    i1)            ; Overlay alpha    
      SetButtonImageGadget_BF(gadget_ID, image_a_ID)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    SetButtonImageGadget_BF(gadget_ID, image_b_ID)
    i1=0
    For i=0 To 255
      i1+animation_alpha_step
      If i1>255 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    overlay_color, ; Overlay color
                                    i1)            ; Overlay alpha
      SetButtonImageGadget_BF(gadget_ID, image_b_ID)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn GetGadgetData(gadget_ID)
  EndProcedure
  
  Procedure SetButtonImageGadget_BF_Fade_BF(gadget_ID,
                                            image_a_ID,
                                            image_b_ID,
                                            background_color,
                                            overlay_color,
                                            animation_delay,
                                            animation_alpha_step)
    Protected i, i1
    i1=255
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    For i=0 To 255 
      i1-animation_alpha_step
      If i1<0 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    overlay_color, ; Overlay color
                                    i1)            ; Overlay alpha    
      SetButtonImageGadget_BF(gadget_ID, image_a_ID)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    SetButtonImageGadget_BF(gadget_ID, image_b_ID)
    i1=0
    For i=0 To 255
      i1+animation_alpha_step
      If i1>255 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    overlay_color, ; Overlay color
                                    i1)            ; Overlay alpha
      SetButtonImageGadget_BF(gadget_ID, image_b_ID)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetButtonImageGadget_BF(gadget_ID,
                                    image_ID=-1,
                                    background_color=-1,
                                    x=-1,
                                    y=-1,
                                    button_width=-1,
                                    button_height=-1,
                                    padding_x_gadget=-1,
                                    padding_y_gadget=-1)
    
    If Not IsGadget(gadget_ID)
      ProcedureReturn -23
    EndIf
    
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        If image_ID=-1
          image_ID=gadget_bf()\image_ID
        EndIf
        If background_color<0
          background_color=gadget_bf()\background_color
        EndIf
        If x=-1
          x=gadget_bf()\x
        EndIf
        If y=-1
          y=gadget_bf()\y
        EndIf
        If button_width<0
          button_width=gadget_bf()\button_width
        EndIf
        If button_height<0
          button_height=gadget_bf()\button_height
        EndIf
        If padding_x_gadget<0
          padding_x_gadget=gadget_bf()\padding_x_gadget
        EndIf
        If padding_y_gadget<0
          padding_y_gadget=gadget_bf()\padding_y_gadget
        EndIf
        Break
      EndIf
    Next
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected temporary_image_ID=FitImage_DualPadding_BF(0,
                                                         image_ID,
                                                         button_width_-gtk3_x_offset,
                                                         button_height_-gtk3_y_offset,
                                                         background_color,
                                                         padding_x_gadget,
                                                         padding_y_gadget,
                                                         0,
                                                         0)
    
    If Not IsImage(temporary_image_ID)
      ProcedureReturn -6
    EndIf
    
    ResizeGadget(gadget_ID, x, y, button_width, button_height)
    SetGadgetAttribute(gadget_ID, #PB_Button_Image, ImageID(temporary_image_ID))
    
    FreeImage_BF(gadget_bf()\temporary_image_ID)
    gadget_bf()\temporary_image_ID=temporary_image_ID
    
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\background_color=background_color
    gadget_bf()\padding_x_gadget=padding_x_gadget
    gadget_bf()\padding_y_gadget=padding_y_gadget
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure ButtonImageGadget_BF(image_ID,
                                 background_color=$E1E1E1,
                                 x=0,
                                 y=0,
                                 button_width=0,
                                 button_height=0,
                                 padding_x_gadget=14,
                                 padding_y_gadget=14,
                                 button_flag=0)
    
    If Not IsImage(image_ID)
      ProcedureReturn -2
    EndIf
    
    button_width-gtk3_x_offset
    button_height-gtk3_y_offset
    
    If button_width<1 : button_width=1 : EndIf
    If button_height<1 : button_height=1 : EndIf 
    
    If x=-2 And IsGadget(y) ; Import a gadget
      ResizeGadget(y, #PB_Ignore, #PB_Ignore, button_width, button_height)
      Protected gadget_ID=y : x=GadgetX(y) : y=GadgetY(y)
    Else
      gadget_ID=ButtonImageGadget(#PB_Any, x, y, button_width, button_height, 0, button_flag)
      If Not IsGadget(gadget_ID)
        ProcedureReturn -23
      EndIf
    EndIf
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected padding_x_gadget_=padding_x_gadget*DPI_ResizeFactor_BF()
      Protected padding_y_gadget_=padding_y_gadget*DPI_ResizeFactor_BF()
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      padding_x_gadget_=padding_x_gadget
      padding_y_gadget_=padding_y_gadget
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected temporary_image_ID=FitImage_DualPadding_BF(0,
                                                         image_ID,
                                                         button_width_-gtk3_x_offset,
                                                         button_height_-gtk3_y_offset,
                                                         background_color,
                                                         padding_x_gadget_,
                                                         padding_y_gadget_,
                                                         0,
                                                         0)
    
    If Not IsImage(temporary_image_ID)
      ProcedureReturn -6
    EndIf
    
    SetGadgetAttribute(gadget_ID, #PB_Button_Image, ImageID(temporary_image_ID))
    
    AddElement(gadget_bf())
    gadget_bf()\gadget_type=#ButtonImageGadget_BF
    gadget_bf()\gadget_ID=gadget_ID
    gadget_bf()\image_ID=image_ID
    gadget_bf()\temporary_image_ID=temporary_image_ID
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\background_color=background_color
    gadget_bf()\padding_x_gadget=padding_x_gadget
    gadget_bf()\padding_y_gadget=padding_y_gadget
    gadget_bf()\gadget_flag=button_flag
    
    ProcedureReturn gadget_ID
  EndProcedure
  
  Procedure SetImageGadget_BF_AutoFade_BF(gadget_ID,
                                          image_a_ID,
                                          image_b_ID,
                                          background_color,
                                          overlay_color,
                                          animation_delay,
                                          animation_alpha_step)
    
    Protected dual_padding
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        If gadget_bf()\gadget_type=#TextGadget_DualPadding_BF
          dual_padding=1
        EndIf
      EndIf
    Next
    
    Protected i, i1, old_get_gadget_data=GetGadgetData(gadget_ID)
    old_get_gadget_data=1-old_get_gadget_data ; Toggle flag
    SetGadgetData(gadget_ID, old_get_gadget_data) ; Set gadget states for post processing
    If Not GetGadgetData(gadget_ID) : Swap image_a_ID, image_b_ID : EndIf
    i1=255
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    For i=0 To 255 
      i1-animation_alpha_step
      If i1<0 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    overlay_color, ; Overlay color
                                    i1)            ; Overlay alpha
      If dual_padding
        SetImageGadget_DualPadding_BF(gadget_ID, image_a_ID, -1, -1, -1, -1, background_color)
      Else
        SetImageGadget_BF(gadget_ID, image_a_ID, -1, -1, -1, -1, background_color)
      EndIf
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    i1=0
    For i=0 To 255
      i1+animation_alpha_step
      If i1>255 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    overlay_color, ; Overlay color
                                    i1)            ; Overlay alpha
      If dual_padding
        SetImageGadget_DualPadding_BF(gadget_ID, image_b_ID, -1, -1, -1, -1, background_color)
      Else
        SetImageGadget_BF(gadget_ID, image_b_ID, -1, -1, -1, -1, background_color)
      EndIf
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn GetGadgetData(gadget_ID)
  EndProcedure
  
  Procedure SetImageGadget_BF_Fade_BF(gadget_ID,
                                      image_a_ID,
                                      image_b_ID,
                                      background_color,
                                      overlay_color,
                                      animation_delay,
                                      animation_alpha_step)
    
    Protected dual_padding
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        If gadget_bf()\gadget_type=#TextGadget_DualPadding_BF
          dual_padding=1
        EndIf
      EndIf
    Next
    
    Protected i, i1
    i1=255
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    For i=0 To 255 
      i1-animation_alpha_step
      If i1<0 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    overlay_color, ; Overlay color
                                    i1)            ; Overlay alpha   
      If dual_padding
        SetImageGadget_DualPadding_BF(gadget_ID, image_a_ID, -1, -1, -1, -1, background_color)
      Else
        SetImageGadget_BF(gadget_ID, image_a_ID, -1, -1, -1, -1, background_color)
      EndIf
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    i1=0
    For i=0 To 255
      i1+animation_alpha_step
      If i1>255 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    overlay_color, ; Overlay color
                                    i1)            ; Overlay alpha
      If dual_padding
        SetImageGadget_DualPadding_BF(gadget_ID, image_b_ID, -1, -1, -1, -1, background_color)
      Else
        SetImageGadget_BF(gadget_ID, image_b_ID, -1, -1, -1, -1, background_color)
      EndIf
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetImageGadget_BF(gadget_ID,
                              image_ID=-1,
                              x=-1,
                              y=-1,
                              button_width=-1,
                              button_height=-1,
                              background_color=-1,
                              padding_gadget=-1,
                              grid_color=-1,
                              grid_size=-1,
                              grid_tiling=-1,
                              frame_color=-1,
                              frame_size=-1)
    
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        If image_ID=-1
          image_ID=gadget_bf()\image_ID
        EndIf
        If x=-1
          x=gadget_bf()\x
        EndIf
        If y=-1
          y=gadget_bf()\y
        EndIf
        If button_width<0
          button_width=gadget_bf()\button_width
        EndIf
        If button_height<0
          button_height=gadget_bf()\button_height
        EndIf
        If background_color=-1
          background_color=gadget_bf()\background_color
        EndIf
        If padding_gadget<0
          padding_gadget=gadget_bf()\padding_gadget
        EndIf
        If grid_color=-1
          grid_color=gadget_bf()\grid_color
        EndIf
        If grid_size<0
          grid_size=gadget_bf()\grid_size
        EndIf
        If grid_tiling<0
          grid_tiling=gadget_bf()\grid_tiling
        EndIf
        If frame_color=-1
          frame_color=gadget_bf()\frame_color
        EndIf
        If frame_size<0
          frame_size=gadget_bf()\frame_size
        EndIf
        Break
      EndIf
    Next
    
    If Not gadget_ID : ProcedureReturn -2 : EndIf
    
    If background_color=-2
      Protected old_alpha_sprite=alpha_sprite
      alpha_sprite=1
    EndIf
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected temporary_image_ID=FitImage_BF(0,
                                             image_ID,
                                             button_width_,
                                             button_height_,
                                             background_color,
                                             padding_gadget,
                                             grid_color,
                                             grid_size,
                                             grid_tiling,
                                             frame_color,
                                             frame_size)
    
    alpha_sprite=old_alpha_sprite 
    
    If IsGadget(gadget_ID)
      If IsImage(temporary_image_ID)
        SetGadgetState(gadget_ID, ImageID(temporary_image_ID))
      EndIf
      ResizeGadget(gadget_ID, x, y, #PB_Ignore, #PB_Ignore)
    Else
      FreeImage_BF(temporary_image_ID)
      ProcedureReturn -23
    EndIf
    
    FreeImage_BF(gadget_bf()\temporary_image_ID)
    gadget_bf()\temporary_image_ID=temporary_image_ID
    
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\background_color=background_color
    gadget_bf()\padding_gadget=padding_gadget
    gadget_bf()\grid_color=grid_color
    gadget_bf()\grid_size=grid_size
    gadget_bf()\grid_tiling=grid_tiling
    gadget_bf()\frame_color=frame_color
    gadget_bf()\frame_size=frame_size
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure ImageGadget_BF(image_ID=-1,
                           x=0,
                           y=0,
                           button_width=1,
                           button_height=1,
                           background_color=$808080,
                           padding_gadget=0,
                           grid_color=$FFFFFF,
                           grid_size=5,
                           grid_tiling=10,
                           frame_color=$FFFFFF,
                           frame_size=3,
                           flags=0)
    
    If button_width<1 : button_width=1 : EndIf : If button_height<1 : button_height=1 : EndIf
    
    If x=-2 And IsGadget(y) ; Import a gadget
      button_width=GadgetWidth(y) : button_height=GadgetHeight(y)
      If button_width<1 : button_width=1 : EndIf : If button_height<1 : button_height=1 : EndIf
      ResizeGadget(y, #PB_Ignore, #PB_Ignore, button_width, button_height)
      Protected gadget_ID=y : x=GadgetX(y) : y=GadgetY(y)
    Else
      gadget_ID=ImageGadget(#PB_Any, x, y, button_width, button_height, 0, flags) 
      If Not IsGadget(gadget_ID)
        ProcedureReturn -23
      EndIf
    EndIf
    
    If Not IsImage(image_ID) : image_ID=CreateImage(#PB_Any, button_width, button_height) : EndIf
    
    If Not IsImage(image_ID)
      ProcedureReturn -6
    EndIf
    
    If background_color=-2
      Protected old_alpha_sprite=alpha_sprite
      alpha_sprite=1
    EndIf
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected padding_gadget_=padding_gadget*DPI_ResizeFactor_BF()
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      padding_gadget_=padding_gadget
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected temporary_image_ID=FitImage_BF(0,
                                             image_ID,
                                             button_width_,
                                             button_height_,
                                             background_color,
                                             padding_gadget_,
                                             grid_color,
                                             grid_size,
                                             grid_tiling,
                                             frame_color,
                                             frame_size)
    
    alpha_sprite=old_alpha_sprite 
    
    If Not IsImage(temporary_image_ID)
      ProcedureReturn -6
    EndIf
    
    SetGadgetState(gadget_ID, ImageID(temporary_image_ID))
    
    AddElement(gadget_bf())
    gadget_bf()\gadget_type=#ImageGadget_BF
    gadget_bf()\gadget_ID=gadget_ID
    gadget_bf()\image_ID=image_ID
    gadget_bf()\temporary_image_ID=temporary_image_ID
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\background_color=background_color
    gadget_bf()\padding_gadget=padding_gadget
    gadget_bf()\grid_color=grid_color
    gadget_bf()\grid_size=grid_size
    gadget_bf()\grid_tiling=grid_tiling
    gadget_bf()\frame_color=frame_color
    gadget_bf()\frame_size=frame_size
    gadget_bf()\gadget_flag=flags
    
    ProcedureReturn gadget_ID
  EndProcedure
  
  Procedure SetImageGadget_DualPadding_BF(gadget_ID,
                                          image_ID=-1,
                                          x=-1,
                                          y=-1,
                                          button_width=-1,
                                          button_height=-1,
                                          background_color=-1,
                                          padding_x_gadget=-1,
                                          padding_y_gadget=-1,
                                          grid_color=-1,
                                          grid_size=-1,
                                          grid_tiling=-1,
                                          frame_color=-1,
                                          frame_size=-1)
    
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        If image_ID=-1
          image_ID=gadget_bf()\image_ID
        EndIf
        If x=-1
          x=gadget_bf()\x
        EndIf
        If y=-1
          y=gadget_bf()\y
        EndIf
        If button_width<0
          button_width=gadget_bf()\button_width
        EndIf
        If button_height<0
          button_height=gadget_bf()\button_height
        EndIf
        If background_color=-1
          background_color=gadget_bf()\background_color
        EndIf
        If padding_x_gadget<0
          padding_x_gadget=gadget_bf()\padding_x_gadget
        EndIf
        If padding_y_gadget<0
          padding_y_gadget=gadget_bf()\padding_y_gadget
        EndIf
        If grid_color=-1
          grid_color=gadget_bf()\grid_color
        EndIf
        If grid_size<0
          grid_size=gadget_bf()\grid_size
        EndIf
        If grid_tiling<0
          grid_tiling=gadget_bf()\grid_tiling
        EndIf
        If frame_color=-1
          frame_color=gadget_bf()\frame_color
        EndIf
        If frame_size<0
          frame_size=gadget_bf()\frame_size
        EndIf
        Break
      EndIf
    Next
    
    If Not gadget_ID : ProcedureReturn -2 : EndIf
    
    If background_color=-2
      Protected old_alpha_sprite=alpha_sprite
      alpha_sprite=1
    EndIf
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected temporary_image_ID=FitImage_DualPadding_BF(0,
                                                         image_ID,
                                                         button_width_,
                                                         button_height_,
                                                         background_color,
                                                         padding_x_gadget,
                                                         padding_y_gadget, 
                                                         grid_color,
                                                         grid_size,
                                                         grid_tiling,
                                                         frame_color,
                                                         frame_size)
    
    alpha_sprite=old_alpha_sprite 
    
    If IsGadget(gadget_ID)
      If IsImage(temporary_image_ID)
        SetGadgetState(gadget_ID, ImageID(temporary_image_ID))
      EndIf
      ResizeGadget(gadget_ID, x, y, #PB_Ignore, #PB_Ignore)
    Else
      FreeImage_BF(temporary_image_ID)
      ProcedureReturn -23
    EndIf
    
    FreeImage_BF(gadget_bf()\temporary_image_ID)
    gadget_bf()\temporary_image_ID=temporary_image_ID
    
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\background_color=background_color
    gadget_bf()\padding_x_gadget=padding_x_gadget
    gadget_bf()\padding_y_gadget=padding_y_gadget
    gadget_bf()\grid_color=grid_color
    gadget_bf()\grid_size=grid_size
    gadget_bf()\grid_tiling=grid_tiling
    gadget_bf()\frame_color=frame_color
    gadget_bf()\frame_size=frame_size
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure ImageGadget_DualPadding_BF(image_ID=-1,
                                       x=0,
                                       y=0,
                                       button_width=1,
                                       button_height=1,
                                       background_color=$808080,
                                       padding_x_gadget=0,
                                       padding_y_gadget=0,
                                       grid_color=$FFFFFF,
                                       grid_size=5,
                                       grid_tiling=10,
                                       frame_color=$FFFFFF,
                                       frame_size=3,
                                       flags=0)
    
    If button_width<1 : button_width=1 : EndIf : If button_height<1 : button_height=1 : EndIf
    
    If x=-2 And IsGadget(y) ; Import a gadget
      button_width=GadgetWidth(y) : button_height=GadgetHeight(y)
      If button_width<1 : button_width=1 : EndIf : If button_height<1 : button_height=1 : EndIf
      ResizeGadget(y, #PB_Ignore, #PB_Ignore, button_width, button_height)
      Protected gadget_ID=y : x=GadgetX(y) : y=GadgetY(y)
    Else
      gadget_ID=ImageGadget(#PB_Any, x, y, button_width, button_height, 0, flags) 
      If Not IsGadget(gadget_ID)
        ProcedureReturn -23
      EndIf
    EndIf
    
    If Not IsImage(image_ID) : image_ID=CreateImage(#PB_Any, button_width, button_height) : EndIf
    
    If Not IsImage(image_ID)
      ProcedureReturn -6
    EndIf
    
    If background_color=-1
      Protected old_alpha_sprite=alpha_sprite
      alpha_sprite=1
    EndIf
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected temporary_image_ID=FitImage_DualPadding_BF(0,
                                                         image_ID,
                                                         button_width_,
                                                         button_height_,
                                                         background_color,
                                                         padding_x_gadget,
                                                         padding_y_gadget,
                                                         grid_color,
                                                         grid_size,
                                                         grid_tiling,
                                                         frame_color,
                                                         frame_size)
    
    alpha_sprite=old_alpha_sprite 
    
    If Not IsImage(temporary_image_ID)
      ProcedureReturn -6
    EndIf
    
    SetGadgetState(gadget_ID, ImageID(temporary_image_ID))
    
    AddElement(gadget_bf())
    gadget_bf()\gadget_type=#ImageGadget_DualPadding_BF
    gadget_bf()\gadget_ID=gadget_ID
    gadget_bf()\image_ID=image_ID
    gadget_bf()\temporary_image_ID=temporary_image_ID
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\background_color=background_color
    gadget_bf()\padding_x_gadget=padding_x_gadget
    gadget_bf()\padding_y_gadget=padding_y_gadget
    gadget_bf()\grid_color=grid_color
    gadget_bf()\grid_size=grid_size
    gadget_bf()\grid_tiling=grid_tiling
    gadget_bf()\frame_color=frame_color
    gadget_bf()\frame_size=frame_size
    gadget_bf()\gadget_flag=flags
    
    ProcedureReturn gadget_ID
  EndProcedure
  
  Procedure SetTextGadget_BF_FadeOut_BF(gadget_ID,
                                        text_a$,
                                        text_b$,
                                        text_color,
                                        background_color,
                                        animation_delay,
                                        animation_alpha_step)
    
    Protected i, i1=255
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    For i=0 To 255 
      i1-animation_alpha_step
      If i1<0 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    background_color, ; Overlay color
                                    i1)               ; Overlay alpha    
      SetTextGadget_DualPadding_BF(gadget_ID,
                                   text_a$,
                                   text_color,
                                   background_color)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    SetTextGadget_DualPadding_BF(gadget_ID,
                                 text_b$,
                                 text_color,
                                 background_color)
    
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetTextGadget_BF_FadeIn_BF(gadget_ID,
                                       text_a$,
                                       text_b$,
                                       text_color,
                                       background_color,
                                       animation_delay,
                                       animation_alpha_step)
    
    Protected i, i1
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    
    For i=0 To 255
      i1+animation_alpha_step
      If i1>255 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    background_color, ; Overlay color
                                    i1)               ; Overlay alpha
      SetTextGadget_DualPadding_BF(gadget_ID,
                                   text_b$,
                                   text_color,
                                   background_color)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetTextGadget_BF_Fade_ChangeTextColor_BF(gadget_ID,
                                                     text_a$,
                                                     text_b$,
                                                     text_color_a,
                                                     text_color_b,
                                                     background_color,
                                                     animation_delay,
                                                     animation_alpha_step)
    
    SetTextGadget_BF_FadeOut_BF(gadget_ID,
                                text_b$,
                                text_b$,
                                text_color_b,
                                background_color,
                                animation_delay,
                                animation_alpha_step)
    
    SetTextGadget_BF_FadeIn_BF(gadget_ID,
                               text_a$,
                               text_a$,
                               text_color_a,
                               background_color,
                               animation_delay,
                               animation_alpha_step)
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetTextGadget_BF_AutoFade_BF(gadget_ID,
                                         text_a$,
                                         text_b$,
                                         animation_delay,
                                         animation_alpha_step)
    
    Protected i, i1=255, old_get_gadget_data=GetGadgetData(gadget_ID)
    old_get_gadget_data=1-old_get_gadget_data ; Toggle flag
    SetGadgetData(gadget_ID, old_get_gadget_data) ; Set gadget states for post processing
    If Not GetGadgetData(gadget_ID)
      Protected text_c$=text_a$ : text_a$=text_b$ : text_b$=text_c$ : text_c$=""
    EndIf
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    For i=0 To 255 
      i1-animation_alpha_step
      If i1<0 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    GetGadgetBackgroundColor_BF(gadget_ID), ; Overlay color
                                    i1)                                     ; Overlay alpha    
      SetTextGadget_DualPadding_BF(gadget_ID,
                                   text_a$,
                                   GetGadgetTextColor_BF(gadget_ID),
                                   GetGadgetBackgroundColor_BF(gadget_ID))
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    SetTextGadget_DualPadding_BF(gadget_ID,
                                 text_b$,
                                 GetGadgetTextColor_BF(gadget_ID),
                                 GetGadgetBackgroundColor_BF(gadget_ID))
    i1=0
    For i=0 To 255
      i1+animation_alpha_step
      If i1>255 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    GetGadgetBackgroundColor_BF(gadget_ID), ; Overlay color
                                    i1)                                     ; Overlay alpha
      SetTextGadget_DualPadding_BF(gadget_ID,
                                   text_b$,
                                   GetGadgetTextColor_BF(gadget_ID),
                                   GetGadgetBackgroundColor_BF(gadget_ID))
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn GetGadgetData(gadget_ID)
  EndProcedure
  
  Procedure SetTextGadget_BF_Fade_BF(gadget_ID,
                                     text_a$,
                                     text_b$,
                                     text_color,
                                     background_color,
                                     animation_delay,
                                     animation_alpha_step)
    
    Protected i, i1=255
    Protected old_add_fit_image_overlay_color=add_fit_image_overlay_color
    For i=0 To 255 
      i1-animation_alpha_step
      If i1<0 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    background_color, ; Overlay color
                                    i1)               ; Overlay alpha    
      SetTextGadget_DualPadding_BF(gadget_ID,
                                   text_a$,
                                   text_color,
                                   background_color)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    SetTextGadget_DualPadding_BF(gadget_ID,
                                 text_b$,
                                 text_color,
                                 background_color)
    i1=0
    For i=0 To 255
      i1+animation_alpha_step
      If i1>255 : Break : EndIf
      FitImage_Add_Overlay_Color_BF(1,
                                    background_color, ; Overlay color
                                    i1)               ; Overlay alpha
      SetTextGadget_DualPadding_BF(gadget_ID,
                                   text_b$,
                                   text_color,
                                   background_color)
      While WindowEvent() : Wend
      Delay(animation_delay)
    Next
    add_fit_image_overlay_color=old_add_fit_image_overlay_color
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetTextGadget_BF_Fading_BF(gadget_ID,
                                       text_a$,
                                       text_b$,
                                       text_color_a,
                                       text_color_b,
                                       animation_delay,
                                       animation_alpha_step)
    
    SetGadgetData(gadget_ID, ~GetGadgetData(gadget_ID))
    
    If GetGadgetData(gadget_ID)    
      SetTextGadget_BF_FadeOut_BF(gadget_ID,
                                  text_a$,
                                  text_a$,
                                  text_color_a,
                                  GetGadgetBackgroundColor_BF(gadget_ID),
                                  animation_delay,
                                  animation_alpha_step)
      
      SetTextGadget_BF_FadeIn_BF(gadget_ID,
                                 text_b$,
                                 text_b$,
                                 text_color_b,
                                 GetGadgetBackgroundColor_BF(gadget_ID),
                                 animation_delay,
                                 animation_alpha_step)
      
    Else
      
      SetTextGadget_BF_FadeOut_BF(gadget_ID,
                                  text_b$,
                                  text_b$,
                                  text_color_b,
                                  GetGadgetBackgroundColor_BF(gadget_ID),
                                  animation_delay,
                                  animation_alpha_step)
      
      SetTextGadget_BF_FadeIn_BF(gadget_ID,
                                 text_a$,
                                 text_a$,
                                 text_color_a,
                                 GetGadgetBackgroundColor_BF(gadget_ID),
                                 animation_delay,
                                 animation_alpha_step)
    EndIf
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetTextGadget_DualPadding_BF(gadget_ID,
                                         text$="",
                                         text_color=-1,
                                         background_color=-1,
                                         padding_x_gadget=-1,
                                         padding_y_gadget=-1,
                                         x=-1,
                                         y=-1,
                                         button_width=-1,
                                         button_height=-1,
                                         stretch_x=-1,
                                         stretch_y=-1,
                                         font_ID=-1,
                                         flag=-1,
                                         mode=-1)
    
    If Not IsGadget(gadget_ID)
      ProcedureReturn -23
    EndIf
    
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        If IsImage(gadget_bf()\image_ID)
          FreeImage(gadget_bf()\image_ID)
        EndIf
        If text$=""
          text$=gadget_bf()\text$
        EndIf
        If text_color=-1
          text_color=gadget_bf()\text_color
        EndIf
        If background_color=-1
          background_color=gadget_bf()\background_color
        EndIf
        If padding_x_gadget<0
          padding_x_gadget=gadget_bf()\padding_x_gadget
        EndIf
        If padding_y_gadget<0
          padding_y_gadget=gadget_bf()\padding_y_gadget
        EndIf
        If x=-1
          x=gadget_bf()\x
        EndIf
        If y=-1
          y=gadget_bf()\y
        EndIf
        If button_width<0
          button_width=gadget_bf()\button_width
        EndIf
        If button_height<0
          button_height=gadget_bf()\button_height
        EndIf
        If stretch_x<0
          stretch_x=gadget_bf()\stretch_x
        EndIf
        If stretch_y<0
          stretch_y=gadget_bf()\stretch_y
        EndIf
        If font_ID<0
          font_ID=gadget_bf()\font_ID
        EndIf
        If flag<0
          flag=gadget_bf()\flag
        EndIf
        If mode<0
          mode=gadget_bf()\mode
        EndIf
        Break
      EndIf
    Next
    
    If text$="" : text$=" " : EndIf
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected image_ID=CreateFittedTextImage_DualPadding_BF(text$,
                                                            button_width_,
                                                            button_height_,
                                                            text_color,
                                                            background_color,
                                                            padding_x_gadget,
                                                            padding_y_gadget,
                                                            font_ID,
                                                            stretch_x,
                                                            stretch_y, 
                                                            flag,
                                                            mode)
    
    If IsImage(image_ID) 
      ResizeGadget(gadget_ID, x, y, button_width, button_height)
      SetGadgetAttribute(gadget_ID, #PB_Canvas_Image, ImageID(image_ID))
      StartDrawing(CanvasOutput(gadget_ID))
      Box(0, 0, button_width_, button_height_, background_color)
      DrawImage(ImageID(image_ID), 0, 0)
      StopDrawing()
    Else
      ProcedureReturn -6
    EndIf
    
    gadget_bf()\image_ID=image_ID
    gadget_bf()\text$=text$
    gadget_bf()\text_color=text_color
    gadget_bf()\background_color=background_color
    gadget_bf()\padding_x_gadget=padding_x_gadget
    gadget_bf()\padding_y_gadget=padding_y_gadget
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\stretch_x=stretch_x
    gadget_bf()\stretch_y=stretch_y
    gadget_bf()\font_ID=font_ID
    gadget_bf()\flag=flag
    gadget_bf()\mode=mode
    
    ProcedureReturn image_ID
  EndProcedure
  
  Procedure SetTextGadget_BF(gadget_ID,
                             text$="",
                             text_color=-1,
                             background_color=-1,
                             x=-1,
                             y=-1,
                             button_width=-1,
                             button_height=-1,
                             stretch_x=-1,
                             stretch_y=-1,
                             font_ID=-1,
                             flag=-1,
                             mode=-1)
    
    If Not IsGadget(gadget_ID)
      ProcedureReturn -23
    EndIf
    
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        If IsImage(gadget_bf()\image_ID)
          FreeImage(gadget_bf()\image_ID)
        EndIf
        If text$=""
          text$=gadget_bf()\text$
        EndIf
        If text_color=-1
          text_color=gadget_bf()\text_color
        EndIf
        If background_color=-1
          background_color=gadget_bf()\background_color
        EndIf
        If x=-1
          x=gadget_bf()\x
        EndIf
        If y=-1
          y=gadget_bf()\y
        EndIf
        If button_width<0
          button_width=gadget_bf()\button_width
        EndIf
        If button_height<0
          button_height=gadget_bf()\button_height
        EndIf
        If stretch_x<0
          stretch_x=gadget_bf()\stretch_x
        EndIf
        If stretch_y<0
          stretch_y=gadget_bf()\stretch_y
        EndIf
        If font_ID<0
          font_ID=gadget_bf()\font_ID
        EndIf
        If flag<0
          flag=gadget_bf()\flag
        EndIf
        If mode<0
          mode=gadget_bf()\mode
        EndIf
        Break
      EndIf
    Next
    
    If text$="" : text$=" " : EndIf
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected image_ID=CreateFittedTextImage_BF(text$,
                                                button_width_,
                                                button_height_,
                                                text_color,
                                                background_color,
                                                font_ID,
                                                stretch_x,
                                                stretch_y, 
                                                flag,
                                                mode)
    
    If IsImage(image_ID) 
      ResizeGadget(gadget_ID, x, y, button_width, button_height)
      SetGadgetAttribute(gadget_ID, #PB_Canvas_Image, ImageID(image_ID))
      StartDrawing(CanvasOutput(gadget_ID))
      Box(0, 0, button_width_+1, button_height_+1, background_color)
      DrawImage(ImageID(image_ID), 0, 0)
      StopDrawing()
    Else
      ProcedureReturn -6
    EndIf
    
    gadget_bf()\image_ID=image_ID
    gadget_bf()\text$=text$
    gadget_bf()\text_color=text_color
    gadget_bf()\background_color=background_color
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\stretch_x=stretch_x
    gadget_bf()\stretch_y=stretch_y
    gadget_bf()\font_ID=font_ID
    gadget_bf()\flag=flag
    gadget_bf()\mode=mode
    
    ProcedureReturn image_ID
  EndProcedure
  
  Procedure TextGadget_BF(text$="",
                          x=0,
                          y=0,
                          button_width=0,
                          button_height=0,
                          text_color=0,
                          background_color=$F0F0F0,
                          font_ID=-1,
                          stretch_x=0,
                          stretch_y=0,
                          flag=0,
                          mode=0,
                          canvas_flag=0)
    
    If text$="" : text$=" " : EndIf 
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected image_ID=CreateFittedTextImage_BF(text$,
                                                button_width_,
                                                button_height_,
                                                text_color,
                                                background_color,
                                                font_ID,
                                                stretch_x,
                                                stretch_y, 
                                                flag,
                                                mode)
    If IsImage(image_ID)
      
      If button_width<1 : button_width=1 : EndIf
      If button_height<1 : button_height=1 : EndIf
      
      Protected gadget_ID=CanvasGadget(#PB_Any, x, y, button_width, button_height, canvas_flag)
      If Not IsGadget(gadget_ID)
        FreeImage(image_ID)
        ProcedureReturn -23
      EndIf
      StartDrawing(CanvasOutput(gadget_ID))
      Box(0, 0, button_width_+1, button_height_+1, background_color)
      DrawImage(ImageID(image_ID), 0, 0)
      StopDrawing()
    Else
      ProcedureReturn -7
    EndIf
    
    AddElement(gadget_bf())
    gadget_bf()\gadget_type=#TextGadget_BF
    gadget_bf()\gadget_ID=gadget_ID
    gadget_bf()\image_ID=image_ID
    gadget_bf()\text$=text$
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\text_color=text_color
    gadget_bf()\background_color=background_color
    gadget_bf()\font_ID=font_ID
    gadget_bf()\stretch_x=stretch_x
    gadget_bf()\stretch_y=stretch_y
    gadget_bf()\flag=flag
    gadget_bf()\mode=mode
    gadget_bf()\gadget_flag=canvas_flag
    
    ProcedureReturn gadget_ID
    
  EndProcedure
  
  Procedure TextGadget_DualPadding_BF(text$="",
                                      x=0,
                                      y=0,
                                      button_width=0,
                                      button_height=0,
                                      text_color=0,
                                      background_color=$F0F0F0,
                                      padding_x_gadget=0,
                                      padding_y_gadget=0,
                                      font_ID=-1,
                                      stretch_x=0,
                                      stretch_y=0,
                                      flag=0,
                                      mode=0,
                                      canvas_flag=0)
    
    If text$="" : text$=" " : EndIf
    
    If activate_dpi_aware And button_width>0 And button_height>0
      Protected button_width_=button_width*DPI_ResizeFactor_BF()
      Protected button_height_=button_height*DPI_ResizeFactor_BF()
    Else
      button_width_=button_width
      button_height_=button_height
    EndIf
    
    Protected image_ID=CreateFittedTextImage_DualPadding_BF(text$,
                                                            button_width_,
                                                            button_height_,
                                                            text_color,
                                                            background_color,
                                                            padding_x_gadget,
                                                            padding_y_gadget,
                                                            font_ID,
                                                            stretch_x,
                                                            stretch_y, 
                                                            flag,
                                                            mode)
    If IsImage(image_ID)
      
      If button_width<1 : button_width=1 : EndIf
      If button_height<1 : button_height=1 : EndIf
      
      Protected gadget_ID=CanvasGadget(#PB_Any, x, y, button_width, button_height, canvas_flag)
      If Not IsGadget(gadget_ID)
        FreeImage(image_ID)
        ProcedureReturn -23
      EndIf
      StartDrawing(CanvasOutput(gadget_ID))
      Box(0, 0, button_width_+1, button_height_+1, background_color)
      DrawImage(ImageID(image_ID), 0, 0)
      StopDrawing()
    Else
      ProcedureReturn -7
    EndIf
    
    AddElement(gadget_bf())
    gadget_bf()\gadget_type=#TextGadget_DualPadding_BF
    gadget_bf()\gadget_ID=gadget_ID
    gadget_bf()\image_ID=image_ID
    gadget_bf()\text$=text$
    gadget_bf()\x=x
    gadget_bf()\y=y
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\text_color=text_color
    gadget_bf()\background_color=background_color
    gadget_bf()\padding_x_gadget=padding_x_gadget
    gadget_bf()\padding_y_gadget=padding_y_gadget
    gadget_bf()\font_ID=font_ID
    gadget_bf()\stretch_x=stretch_x
    gadget_bf()\stretch_y=stretch_y
    gadget_bf()\flag=flag
    gadget_bf()\mode=mode
    gadget_bf()\gadget_flag=canvas_flag
    
    ProcedureReturn gadget_ID
    
  EndProcedure
  
  Procedure Offset_x_BF(old_window_x_size.f, window_ID, position_x.f)
    ProcedureReturn position_x/(old_window_x_size/WindowWidth(window_ID))
  EndProcedure
  
  Procedure Offset_y_BF(old_window_y_size.f, window_ID, position_y.f)
    ProcedureReturn position_y/(old_window_y_size/WindowHeight(window_ID))
  EndProcedure
  
  Procedure DrawTextCreateImage_Add_Image_BF(draw_text_create_image_add_image_,
                                             draw_text_create_image_add_image_ID_=-1,
                                             draw_text_create_image_add_image_x_=0,
                                             draw_text_create_image_add_image_y_=0,
                                             draw_text_create_image_add_image_ID_width_=-1,
                                             draw_text_create_image_add_image_ID_height_=-1,   
                                             draw_text_create_image_add_image_alpha_=-1)
    
    draw_text_create_image_add_image=draw_text_create_image_add_image_
    draw_text_create_image_add_image_ID=draw_text_create_image_add_image_ID_
    draw_text_create_image_add_image_x=draw_text_create_image_add_image_x_
    draw_text_create_image_add_image_y=draw_text_create_image_add_image_y_
    draw_text_create_image_add_image_ID_width=draw_text_create_image_add_image_ID_width_
    draw_text_create_image_add_image_ID_height=draw_text_create_image_add_image_ID_height_  
    draw_text_create_image_add_image_alpha=draw_text_create_image_add_image_alpha_
    ProcedureReturn 1
  EndProcedure
  
  Procedure FitImage_Add_Overlay_Color_BF(add_fit_image_overlay_color_,
                                          fit_image_overlay_color_=0,
                                          fit_image_overlay_alpha_=128,
                                          fit_image_overlay_x_=0,
                                          fit_image_overlay_y_=0,
                                          fit_image_overlay_width_=-1,
                                          fit_image_overlay_height_=-1)
    
    add_fit_image_overlay_color=add_fit_image_overlay_color_
    fit_image_overlay_color=fit_image_overlay_color_
    fit_image_overlay_alpha=fit_image_overlay_alpha_
    fit_image_overlay_x=fit_image_overlay_x_
    fit_image_overlay_y=fit_image_overlay_y_
    fit_image_overlay_width=fit_image_overlay_width_
    fit_image_overlay_height=fit_image_overlay_height_
    ProcedureReturn 1
  EndProcedure
  
  Procedure FitImage_Add_Overlay_Frame_BF(add_fit_image_overlay_frame_,
                                          fit_image_overlay_frame_color_=0,
                                          fit_image_overlay_frame_alpha_=0,
                                          fit_image_overlay_frame_x_=0,
                                          fit_image_overlay_frame_y_=0,
                                          fit_image_overlay_frame_width_=-1,
                                          fit_image_overlay_frame_height_=-1,
                                          fit_image_overlay_frame_thickness_=3)
    
    add_fit_image_overlay_frame=add_fit_image_overlay_frame_
    fit_image_overlay_frame_color=fit_image_overlay_frame_color_
    fit_image_overlay_frame_alpha=fit_image_overlay_frame_alpha_
    fit_image_overlay_frame_x=fit_image_overlay_frame_x_
    fit_image_overlay_frame_y=fit_image_overlay_frame_y_
    fit_image_overlay_frame_width=fit_image_overlay_frame_width_
    fit_image_overlay_frame_height=fit_image_overlay_frame_height_
    fit_image_overlay_frame_thickness=fit_image_overlay_frame_thickness_
    ProcedureReturn 1
  EndProcedure
  
  Procedure FitImagePadding_Offset_x_BF(fit_image_padding_offset_x_)
    fit_image_padding_offset_x=fit_image_padding_offset_x_
    ProcedureReturn 1
  EndProcedure
  
  Procedure FitImagePadding_Offset_y_BF(fit_image_padding_offset_y_)
    fit_image_padding_offset_y=fit_image_padding_offset_y_
    ProcedureReturn 1
  EndProcedure
  
  Procedure ManualPadding_DrawTextCreateImage_X_BF(manual_padding_draw_text_create_image_x_)
    manual_padding_draw_text_create_image_x=manual_padding_draw_text_create_image_x_
    ProcedureReturn 1
  EndProcedure
  
  Procedure ManualPadding_DrawTextCreateImage_Y_BF(manual_padding_draw_text_create_image_y_)
    manual_padding_draw_text_create_image_y=manual_padding_draw_text_create_image_y_
    ProcedureReturn 1
  EndProcedure
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure ImageFrameCount_BF(gif_ID) ; Workaraound PB 570 Linux Mac
      If IsImage(gif_ID)
        Protected frames=ImageFrameCount(gif_ID)
      Else
        ProcedureReturn 0
      EndIf
      If frames<2 : frames=1 : EndIf
      ProcedureReturn frames
    EndProcedure
  CompilerEndIf
  
  CompilerIf #PB_Compiler_Version=>560
    Procedure SetImageFrame_BF(gif_ID, frame) ; Workaround PB570 Linux and Mac
      If ImageFrameCount(gif_ID)>1
        SetImageFrame(gif_ID, frame)
      EndIf
      ProcedureReturn 1
    EndProcedure
  CompilerEndIf
  
  Procedure.f GetResolution_DPI_X_BF()
    CompilerIf #PB_Compiler_Version=>570
      ProcedureReturn DesktopResolutionX()
    CompilerElse
      Static little_temporary_image_ID
      If Not little_temporary_image_ID
        little_temporary_image_ID=CreateImage(#PB_Any, 16, 16)
      EndIf
      If little_temporary_image_ID
        If StartVectorDrawing(ImageVectorOutput(little_temporary_image_ID))
          Protected dpi=VectorResolutionX()
          StopDrawing()
        EndIf
        ProcedureReturn dpi
      Else
        ProcedureReturn 1
      EndIf
    CompilerEndIf
  EndProcedure
  
  Procedure.f GetResolution_DPI_Y_BF()
    CompilerIf #PB_Compiler_Version=>570
      ProcedureReturn DesktopResolutionY()
    CompilerElse
      Static little_temporary_image_ID
      If Not little_temporary_image_ID
        little_temporary_image_ID=CreateImage(#PB_Any, 16, 16)
      EndIf  
      If little_temporary_image_ID
        If StartVectorDrawing(ImageVectorOutput(little_temporary_image_ID))
          Protected dpi=VectorResolutionY()
          StopDrawing()
        EndIf
        ProcedureReturn dpi
      Else
        ProcedureReturn 1
      EndIf
    CompilerEndIf
  EndProcedure
  
  Procedure.f DPI_ResizeFactor_BF()
    If deactivate_dpi_aware
      ProcedureReturn 1
    Else
      ProcedureReturn dpi_resize_factor.f
    EndIf
  EndProcedure
  
  Procedure.f DPI_ResizeFactor_X_BF()
    If deactivate_dpi_aware
      ProcedureReturn 1
    Else
      ProcedureReturn dpi_resize_factor_x.f
    EndIf
  EndProcedure
  
  Procedure.f DPI_ResizeFactor_Y_BF()
    If deactivate_dpi_aware
      ProcedureReturn 1
    Else
      ProcedureReturn dpi_resize_factor_y.f
    EndIf
  EndProcedure
  
  Procedure Deactivate_DPI_ResizeFactor_BF(deactivate_dpi_aware_)
    If deactivate_dpi_aware_<>0 : deactivate_dpi_aware_=1 : EndIf
    deactivate_dpi_aware=deactivate_dpi_aware_
  EndProcedure
  
  Procedure Activate_DPI_aware_BF(activate_dpi_aware_)
    activate_dpi_aware=activate_dpi_aware_
    ProcedureReturn activate_dpi_aware
  EndProcedure
  
  Procedure GetState_Activate_DPI_aware_BF()
    ProcedureReturn activate_dpi_aware
  EndProcedure 
  
  ; Windows OS
  ; Desktop scaling functions for Windows OS, without activated PB DPI aware compiler option
  ; This functions allows you to easily determine the maximum available width and height for output on the desktop on any desktop scaling settings above 100%
  
  ; Grab the desktop width
  Procedure DesktopWidth_BF(desktop=0) ; The first desktop is 0 - It's the preset
    Protected debugger_state, amount_desktops, desktop_width
    
    CompilerIf #PB_Compiler_Debugger
      debugger_state=1
      DisableDebugger
    CompilerEndIf
    amount_desktops=ExamineDesktops()
    If amount_desktops
      desktop_width=DesktopWidth(desktop)
    EndIf
    If debugger_state
      EnableDebugger
    EndIf
    If desktop_width
      ProcedureReturn desktop_width
    EndIf
    ProcedureReturn 0
  EndProcedure
  
  ; Grab the desktop height
  Procedure DesktopHeight_BF(desktop=0) ; The first desktop is 0 - It's the preset
    Protected debugger_state, amount_desktops, desktop_height
    
    CompilerIf #PB_Compiler_Debugger
      debugger_state=1
      DisableDebugger
    CompilerEndIf
    amount_desktops=ExamineDesktops()
    If amount_desktops
      desktop_height=DesktopHeight(desktop)
    EndIf
    If debugger_state
      EnableDebugger
    EndIf
    If desktop_height
      ProcedureReturn desktop_height
    EndIf
    ProcedureReturn 0
  EndProcedure
  
  Procedure.f Desktop_ResizeFactor_Width_BF(desktop=0)   
    Protected window_ID, desktop_width, res.f
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Windows
      
      If ExamineDesktops()
        desktop_width=DesktopWidth(desktop)
        If desktop_width
          window_ID=OpenWindow(#PB_Any, 0, 0, 0, 0, "", #PB_Window_NoGadgets | #PB_Window_BorderLess | #PB_Window_Maximize | #PB_Window_Invisible)
          If window_ID
            res.f=desktop_width/WindowWidth(window_ID)
            CloseWindow(window_ID)   
          EndIf
          ProcedureReturn res
        EndIf
      Else
        ProcedureReturn 1
      EndIf
      
    CompilerElse
      ProcedureReturn 1
    CompilerEndIf
    
  EndProcedure
  
  Procedure.f Desktop_ResizeFactor_Height_BF(desktop=0)
    Protected window_ID, desktop_height, res.f
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Windows
      
      If ExamineDesktops()
        desktop_height=DesktopHeight(desktop)
        If desktop_height
          window_ID=OpenWindow(#PB_Any, 0, 0, 0, 0, "", #PB_Window_NoGadgets | #PB_Window_BorderLess | #PB_Window_Maximize | #PB_Window_Invisible)
          If window_ID
            res.f=desktop_height/WindowHeight(window_ID)
            CloseWindow(window_ID)   
          EndIf
          ProcedureReturn res
        EndIf
      Else
        ProcedureReturn 1
      EndIf
      
    CompilerElse
      ProcedureReturn 1
    CompilerEndIf
    
  EndProcedure
  
  Procedure DesktopWidth_UserScaled_BF(desktop=0)
    Protected window_ID, res
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Windows
      
      If ExamineDesktops()
        window_ID=OpenWindow(#PB_Any, 0, 0, 0, 0, "", #PB_Window_NoGadgets | #PB_Window_BorderLess | #PB_Window_Maximize | #PB_Window_Invisible)
        If window_ID
          res=WindowWidth(window_ID)
          CloseWindow(window_ID)
          ProcedureReturn res
        EndIf
      Else
        ProcedureReturn 0
      EndIf
      
    CompilerElse
      If ExamineDesktops()
        ProcedureReturn DesktopWidth(desktop)
      Else
        ProcedureReturn 0
      EndIf
    CompilerEndIf
    
  EndProcedure
  
  Procedure DesktopHeight_UserScaled_BF(desktop=0)  
    Protected window_ID, res
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Windows
      
      If ExamineDesktops()
        window_ID=OpenWindow(#PB_Any, 0, 0, 0, 0, "", #PB_Window_NoGadgets | #PB_Window_BorderLess | #PB_Window_Maximize | #PB_Window_Invisible)
        If window_ID
          res=WindowHeight(window_ID)
          CloseWindow(window_ID)
          ProcedureReturn res
        EndIf 
      Else
        ProcedureReturn 0
      EndIf
      
    CompilerElse
      If ExamineDesktops()
        ProcedureReturn DesktopHeight(desktop)
      Else
        ProcedureReturn 0
      EndIf
    CompilerEndIf
    
  EndProcedure
  
  Procedure Desktop_ScalingPercent_Width_BF(desktop=0) 
    Protected window_ID, desktop_width.f, window_width.f, res
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Windows
      
      If ExamineDesktops()
        desktop_width=DesktopWidth(desktop)
        If desktop_width
          window_ID=OpenWindow(#PB_Any, 0, 0, 0, 0, "", #PB_Window_NoGadgets | #PB_Window_BorderLess | #PB_Window_Maximize | #PB_Window_Invisible)
          If window_ID
            window_width=WindowWidth(window_ID)
            res=desktop_width/window_width*100
            CloseWindow(window_ID) 
            ProcedureReturn res
          EndIf
        EndIf 
      Else
        ProcedureReturn 0
      EndIf
      
    CompilerElse
      ProcedureReturn 100
    CompilerEndIf
    
  EndProcedure
  
  Procedure Desktop_ScalingPercent_Height_BF(desktop=0)    
    Protected window_ID, desktop_height.f, window_height.f, res
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Windows
      
      If ExamineDesktops()
        desktop_height=DesktopHeight(desktop)
        If desktop_height
          window_ID=OpenWindow(#PB_Any, 0, 0, 0, 0, "", #PB_Window_NoGadgets | #PB_Window_BorderLess | #PB_Window_Maximize | #PB_Window_Invisible)
          If window_ID
            window_height=WindowHeight(window_ID)
            res=desktop_height/window_height*100
            CloseWindow(window_ID) 
            ProcedureReturn res
          EndIf
        EndIf  
      Else
        ProcedureReturn 0
      EndIf
      
    CompilerElse
      ProcedureReturn 100
    CompilerEndIf
    
  EndProcedure
  
  Procedure GetGadgetTyp_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\gadget_type
      EndIf
    Next
    ProcedureReturn #GadgetTypeUnknown_BF 
  EndProcedure
  
  Procedure GetGadgetUsedImage_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\image_ID
      EndIf
    Next
    ProcedureReturn -1
  EndProcedure
  
  Procedure GetGadgetImage_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\temporary_image_ID
      EndIf
    Next
    ProcedureReturn -1
  EndProcedure
  
  Procedure.s GetGadgetText_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\text$
      EndIf
    Next
    ProcedureReturn ""
  EndProcedure
  
  Procedure GetGadgetX_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\x
      EndIf
    Next
    ProcedureReturn 0
  EndProcedure
  
  Procedure GetGadgetY_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\y
      EndIf
    Next
    ProcedureReturn 0
  EndProcedure
  
  Procedure GetGadgetWidth_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\button_width
      EndIf
    Next
    ProcedureReturn 0
  EndProcedure
  
  Procedure GetGadgetHeight_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\button_height
      EndIf
    Next
    ProcedureReturn -1
  EndProcedure
  
  Procedure GetGadgetTextColor_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\text_color
      EndIf
    Next
    ProcedureReturn -1
  EndProcedure
  
  Procedure GetGadgetBackgroundColor_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\background_color
      EndIf
    Next
    ProcedureReturn -1
  EndProcedure
  
  Procedure ResizeGadget_BF(gadget_ID, x, y, width, height, text_width=0, text_height=0)
    If Not IsGadget(gadget_ID) : ProcedureReturn -25 : EndIf
    Protected result
    Select GetGadgetTyp_BF(gadget_ID)
      Case #ButtonImageGadget_BF
        result=SetButtonImageGadget_BF(gadget_ID, -1, -1, x, y, width, height)
      Case #ColorButtonGadget_BF
        result=SetColorButtonGadget_BF(gadget_ID, "", -1, -1, x, y, width, height)
      Case #ImageGadget_BF
        result=SetImageGadget_BF(gadget_ID, -1, x, y, width, height)
      Case #TextGadget_BF
        result=SetTextGadget_BF(gadget_ID, "", -1, -1, x, y, width, height)
      Case #ColorButtonGadget_DualPadding_BF
        result=SetColorButtonGadget_DualPadding_BF(gadget_ID, "", -1, -1, -1, -1, x, y, width, height) 
      Case #ImageGadget_DualPadding_BF
        result=SetImageGadget_DualPadding_BF(gadget_ID, -1, x, y, width, height)
      Case #TextGadget_DualPadding_BF
        result=SetTextGadget_DualPadding_BF(gadget_ID, "", -1, -1, -1, -1, x, y, width, height)
      Case #ToggleImageGadget_BF, #ToggleCanvasGadget_BF
        result=SetToggleGadget_BF(gadget_ID, -1, "", x, y, width, height)
      Case #StretchTextImageGadget_BF, #StretchTextCanvasGadget_BF
        result=SetStretchTextGadget_BF(gadget_ID, "", x, y, width, height)
      Case #StretchTextImageGadget_Pro_BF, #StretchTextButtonImageGadget_Pro_BF, #StretchTextCanvasGadget_Pro_BF
        result=SetMagicTextGadget_BF(gadget_ID, text_width, text_height, width, height, x, y) 
      Default
        result=ResizeGadget(gadget_ID, x, y, width, height)
    EndSelect
    ProcedureReturn result
  EndProcedure
  
  Procedure Calculate_fitted_Font_Width_BF(text$,
                                           width_rectangle,
                                           Array font(1),
                                           vertical=0,
                                           padding_x=0,
                                           stretch_x=0,
                                           resize_factor.f=1)
    
    If text$="" : ProcedureReturn 0 : EndIf
    
    padding_x+#min_padding_x
    
    If vertical>0 : Protected flag=3 : EndIf
    
    Protected max_needed_font_size, text_width
    Protected array_size=ArraySize(font())
    
    Macro calculate_font_width
      If max_needed_font_size>array_size
        Break
      EndIf
      text_width=DrawTextCreateImage_BF(text$, 0, 0, flag, 3, font(max_needed_font_size), stretch_x, 0, 0)
      If text_width>width_rectangle-padding_x
        Break
      EndIf
    EndMacro
    
    Repeat
      max_needed_font_size+20
      calculate_font_width
    ForEver
    max_needed_font_size-20
    
    Repeat
      max_needed_font_size+10
      calculate_font_width
    ForEver
    max_needed_font_size-10
    
    Repeat
      max_needed_font_size+5
      calculate_font_width
    ForEver
    max_needed_font_size-5
    
    Repeat
      max_needed_font_size+1
      calculate_font_width
    ForEver  
    max_needed_font_size-1
    
    If text_width+padding_x>>1>width_rectangle
      fitted_font_padding_x=padding_x-((text_width+padding_x>>1)-width_rectangle)
    Else
      fitted_font_padding_x=padding_x
    EndIf
    
    If resize_factor
      max_needed_font_size/resize_factor
    EndIf
    
    If max_needed_font_size<1 : max_needed_font_size=1 : EndIf
    
    If max_needed_font_size>array_size : max_needed_font_size=array_size : EndIf
    ProcedureReturn max_needed_font_size
  EndProcedure
  
  Procedure Calculate_fitted_Font_Height_BF(text$,
                                            height_rectangle,
                                            Array font(1),
                                            vertical=0,
                                            padding_y=0,
                                            stretch_y=0,
                                            resize_factor.f=1)
    
    If text$="" : ProcedureReturn 0 : EndIf
    
    padding_y+#min_padding_y
    
    If vertical>0 : Protected flag=3 : EndIf
    
    Protected max_needed_font_size, text_height
    Protected array_size=ArraySize(font())
    
    Macro calculate_font_height
      If max_needed_font_size>array_size
        Break
      EndIf
      text_height=DrawTextCreateImage_BF(text$, 0, 0, flag, 4, font(max_needed_font_size), 0, stretch_y, 0)
      If text_height>height_rectangle-padding_y
        Break
      EndIf
    EndMacro
    
    Repeat
      max_needed_font_size+20
      calculate_font_height
    ForEver
    max_needed_font_size-20
    
    Repeat
      max_needed_font_size+10
      calculate_font_height
    ForEver
    max_needed_font_size-10
    
    Repeat
      max_needed_font_size+5
      calculate_font_height
    ForEver
    max_needed_font_size-5
    
    Repeat
      max_needed_font_size+1
      calculate_font_height
    ForEver
    max_needed_font_size-1
    
    If text_height+padding_y>>1>height_rectangle
      fitted_font_padding_y=padding_y-((text_height+padding_y>>1)-height_rectangle)
    Else
      fitted_font_padding_y=padding_y
    EndIf
    
    If resize_factor
      max_needed_font_size/resize_factor
    EndIf
    If max_needed_font_size<1 : max_needed_font_size=1 : EndIf
    
    If max_needed_font_size>array_size : max_needed_font_size=array_size : EndIf
    ProcedureReturn max_needed_font_size
  EndProcedure
  
  Procedure Calculate_fitted_Font_BF(text$,
                                     width_rectangle,
                                     height_rectangle,
                                     Array font(1),
                                     vertical=0,
                                     padding_x=0,
                                     padding_y=0,
                                     stretch_x=0,
                                     stretch_y=0,
                                     resize_factor.f=1)
    
    If text$="" : ProcedureReturn 0 : EndIf
    
    padding_x+#min_padding_x
    padding_y+#min_padding_y
    
    If vertical>0 : Protected flag=3 : EndIf
    
    Protected max_needed_font_size, text_width, text_height
    Protected array_size=ArraySize(font())
    
    Macro calculate_font_BF
      If max_needed_font_size>array_size
        Break
      EndIf
      text_width=DrawTextCreateImage_BF(text$, 0, 0, flag, 3, font(max_needed_font_size), stretch_x, 0, 0)
      text_height=DrawTextCreateImage_BF(text$, 0, 0, flag, 4, font(max_needed_font_size), 0, stretch_y, 0)
      If text_width>width_rectangle-padding_x Or text_height>height_rectangle-padding_y
        Break
      EndIf
    EndMacro
    
    Repeat
      max_needed_font_size+20
      calculate_font_BF
    ForEver
    max_needed_font_size-20
    
    Repeat
      max_needed_font_size+10
      calculate_font_BF
    ForEver
    max_needed_font_size-10
    
    Repeat
      max_needed_font_size+5
      calculate_font_BF
    ForEver
    max_needed_font_size-5
    
    Repeat
      max_needed_font_size+1
      calculate_font_BF
    ForEver
    max_needed_font_size-1
    
    If text_width+padding_x>>1>width_rectangle
      fitted_font_padding_x=padding_x-((text_width+padding_x>>1)-width_rectangle)
    Else
      fitted_font_padding_x=padding_x
    EndIf
    If text_height+padding_y>>1>height_rectangle
      fitted_font_padding_y=padding_y-((text_height+padding_y>>1)-height_rectangle)
    Else
      fitted_font_padding_y=padding_y
    EndIf
    
    If resize_factor
      max_needed_font_size/resize_factor
    EndIf
    If max_needed_font_size<1 : max_needed_font_size=1 : EndIf
    
    If max_needed_font_size>array_size : max_needed_font_size=array_size : EndIf
    ProcedureReturn max_needed_font_size
  EndProcedure
  
  Procedure Get_Fitted_Font_Padding_X_BF()
    ProcedureReturn fitted_font_padding_x
  EndProcedure
  
  Procedure Get_Fitted_Font_Padding_Y_BF()
    ProcedureReturn fitted_font_padding_y
  EndProcedure
  
  Procedure SetCanvasCustomText_BF(canvas_ID,
                                   Array Font(1),
                                   output_x=0,
                                   output_y=0,
                                   canvas_width=0,
                                   canvas_height=0,
                                   canvas_text$=" ",
                                   text_color=#Black,
                                   background_color=#White,
                                   flag=1,
                                   mode=0,
                                   padding_x=0,
                                   padding_y=0,
                                   text_stretching_x=0,
                                   text_stretching_y=0,
                                   resize_factor.f=1)
    
    If Not IsGadget(canvas_ID) : ProcedureReturn -1 : EndIf
    
    If output_x=-1 : output_x=GadgetX(canvas_ID) : EndIf
    If output_y=-1 : output_y=GadgetY(canvas_ID) : EndIf
    If canvas_width<0 : canvas_width=GadgetWidth(canvas_ID) : EndIf
    If canvas_height<0 : canvas_height=GadgetHeight(canvas_ID) : EndIf
    
    If canvas_text$="" : canvas_text$=" " : EndIf
    
    If flag=3 : Protected vertical=1 : EndIf
    
    If activate_dpi_aware And canvas_width>0 And canvas_height>0
      Protected canvas_width_=canvas_width*DPI_ResizeFactor_BF()
      Protected canvas_height_=canvas_height*DPI_ResizeFactor_BF()
    Else
      canvas_width_=canvas_width
      canvas_height_=canvas_height
    EndIf
    
    Protected calculated_font_size=Calculate_fitted_Font_BF(canvas_text$,
                                                            canvas_width_,
                                                            canvas_height_,
                                                            Font(),
                                                            vertical,
                                                            padding_x, 
                                                            padding_y,
                                                            text_stretching_x,
                                                            text_stretching_y,
                                                            resize_factor.f)
    
    ResizeGadget(canvas_ID, output_x, output_y, canvas_width, canvas_height)
    
    Protected text_width=GetTextWidth_BF(canvas_text$, Font(calculated_font_size), vertical, text_stretching_x)
    Protected text_height=GetTextHeight_BF(canvas_text$, Font(calculated_font_size), vertical, text_stretching_y)
    
    Protected output_x_=canvas_width_/2.04-text_width/2   ; Correction factor output x = 0.04
    Protected output_y_=canvas_height_/1.96-text_height/2 ; Correction factor output y = 0.04
    
    If Not StartDrawing(CanvasOutput(canvas_ID)) ; Use DrawText_BF for text output on canvas
      ProcedureReturn -5
    EndIf
    Box(0, 0, canvas_width_+1, canvas_height_+1, background_color)
    DrawingMode(#PB_2DDrawing_Transparent)
    DrawingFont(FontID(Font(calculated_font_size)))
    
    DrawText_BF(output_x_,
                output_y_,
                canvas_text$,
                text_color,
                background_color,
                flag,
                mode,
                text_stretching_x,
                text_stretching_y)
    
    StopDrawing()
    If get_all_window_events : While WindowEvent() : Wend : EndIf
    ProcedureReturn calculated_font_size
  EndProcedure
  
  Procedure FittedText_BF(output_ID,
                          Array Font(1),           ; Set a Font array, for simple creating any look the sample here
                          output_x=0,              ; Output pos x  - Ignore = -1
                          output_y=0,              ; Output pos y  - Ignore = -1
                          output_width=0,          ; Canvas width  - Ignore = -1
                          output_height=0,         ; Canvas height - Ignore = -1
                          text$=" ",               ; Text
                          text_color=#Black,       ; Text color
                          background_color=#White, ; Background color
                          frame_thickness=0,       ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                          frame_color=#Black,      ; Frame color
                          flag=1,                  ; Flags : Text center=1 - Text right=2 - Text vertical=3
                          mode=0,                  ; Modes : Revers text mode=1 - RTL text mode=2
                          padding_x=0,             ; Text padding x
                          padding_y=0,             ; Text padding y
                          text_stretching_x=0,     ; Headline text stretching x
                          text_stretching_y=0,     ; Headline text stretching y
                          resize_factor.f=1)       ; Text resize factor
    
    Protected i, ii, result, background_offset
    
    If Not screen_output
      If IsImage(output_ID) ; check for image
        Protected image=1
      ElseIf IsSprite(output_ID) ; Check for PB sprite
        Protected sprite=1
      ElseIf IsGadget(output_ID) ; Check for canvas
        Protected canvas=1
      ElseIf IsWindow(output_ID) ; Check for window
        Protected window=1  
      Else
        ProcedureReturn -11
      EndIf
    EndIf
    
    If text$="" : text$=" " : EndIf
    
    If flag=3 : Protected vertical=1 : EndIf
    
    If activate_dpi_aware
      padding_x*DPI_ResizeFactor_BF()
      padding_y*DPI_ResizeFactor_BF()
      output_x*DPI_ResizeFactor_BF()
      output_y*DPI_ResizeFactor_BF()
      output_width*DPI_ResizeFactor_BF()
      output_height*DPI_ResizeFactor_BF()
    EndIf
    
    Protected calculated_font_size=Calculate_fitted_Font_BF(text$,
                                                            output_width,
                                                            output_height,
                                                            Font(),
                                                            vertical,
                                                            padding_x, 
                                                            padding_y,
                                                            text_stretching_x,
                                                            text_stretching_y,
                                                            resize_factor.f)
    
    Protected text_width=GetTextWidth_BF(text$, Font(calculated_font_size), vertical, text_stretching_x)
    Protected text_height=GetTextHeight_BF(text$, Font(calculated_font_size), vertical, text_stretching_y)
    
    If screen_output
      result=StartDrawing(ScreenOutput())
    Else
      If image
        result=StartDrawing(ImageOutput(output_ID))
      ElseIf sprite
        result=StartDrawing(SpriteOutput(output_ID))
      ElseIf canvas
        result=StartDrawing(CanvasOutput(output_ID))
      Else
        result=StartDrawing(WindowOutput(output_ID)) 
      EndIf
    EndIf
    
    If Not result : ProcedureReturn -11 : EndIf
    
    If frame_thickness<0
      background_offset=-frame_thickness
    EndIf
    
    If background_color<>-1
      Box(output_x+background_offset,
          output_y+background_offset,
          output_width-1-background_offset<<1,
          output_height-1-background_offset<<1, background_color)
    EndIf
    
    DrawingMode(#PB_2DDrawing_Outlined)
    If frame_thickness>0
      For i=0 To frame_thickness-1
        Box(output_x+i, output_y+i, output_width-i*2-1, output_height-i*2-1, frame_color)
      Next i
    EndIf
    
    DrawingMode(#PB_2DDrawing_Transparent)
    DrawingFont(FontID(Font(calculated_font_size)))
    
    DrawText_BF(output_x+output_width/2-text_width/2,
                output_y+output_height/2-text_height/2,
                text$,
                text_color,
                background_color,
                flag,
                mode,
                text_stretching_x,
                text_stretching_y)
    
    StopDrawing()
    If get_all_window_events : While WindowEvent() : Wend : EndIf
    ProcedureReturn calculated_font_size
  EndProcedure
  
  Procedure FittedText_RoundBox_Vector_BF(output_ID,
                                          Array Font(1),
                                          output_x=0,
                                          output_y=0,
                                          output_width=0,
                                          output_height=0,
                                          radius=0,
                                          text$=" ", 
                                          text_color=#Black,
                                          background_color=#White,
                                          frame_thickness=0,
                                          frame_color=#Black,
                                          flag=1,
                                          mode=0,
                                          padding_x=0,
                                          padding_y=0,
                                          text_stretching_x=0,
                                          text_stretching_y=0,
                                          resize_factor.f=1)
    
    Protected i, ii, result, background_offset
    
    If Not screen_output
      If IsImage(output_ID) ; check for image
        Protected image=1
      ElseIf IsSprite(output_ID) ; Check for PB sprite
        Protected sprite=1
      ElseIf IsGadget(output_ID) ; Check for canvas
        Protected canvas=1
      ElseIf IsWindow(output_ID) ; Check for window
        Protected window=1  
      Else
        ProcedureReturn -11
      EndIf
    EndIf
    
    If text$="" : text$=" " : EndIf
    
    If flag=3 : Protected vertical=1 : EndIf
    
    If activate_dpi_aware
      padding_x*DPI_ResizeFactor_BF()
      padding_y*DPI_ResizeFactor_BF()
      output_x*DPI_ResizeFactor_BF()
      output_y*DPI_ResizeFactor_BF()
      output_width*DPI_ResizeFactor_BF()
      output_height*DPI_ResizeFactor_BF()
    EndIf
    
    Protected calculated_font_size=Calculate_fitted_Font_BF(text$,
                                                            output_width,
                                                            output_height,
                                                            Font(),
                                                            vertical,
                                                            padding_x, 
                                                            padding_y,
                                                            text_stretching_x,
                                                            text_stretching_y,
                                                            resize_factor.f)
    
    Protected text_width=GetTextWidth_BF(text$, Font(calculated_font_size), vertical, text_stretching_x)
    Protected text_height=GetTextHeight_BF(text$, Font(calculated_font_size), vertical, text_stretching_y)
    
    If frame_thickness<0
      background_offset=-frame_thickness
    EndIf
    
    result=RoundBox_Vector_BF(output_ID,
                              output_x+background_offset,
                              output_y+background_offset,
                              output_width-1-background_offset*2,
                              output_height-1-background_offset*2,
                              radius,
                              frame_thickness,
                              frame_color,
                              background_color)
    
    If result<1 : ProcedureReturn result : EndIf
    
    If Alpha(text_color)>0
      If image
        result=StartVectorDrawing(ImageVectorOutput(output_ID))
      ElseIf canvas
        result=StartVectorDrawing(CanvasVectorOutput(output_ID))
      Else
        result=StartVectorDrawing(WindowVectorOutput(output_ID)) 
      EndIf
      
      If Not result : ProcedureReturn -11 : EndIf
      
      VectorFont(FontID(Font(calculated_font_size)))
      
      DrawText_Vector_BF(output_x+output_width/2-text_width/2,
                         output_y+output_height/2-text_height/2,
                         text$,
                         text_color,
                         flag,
                         mode,
                         text_stretching_x,
                         text_stretching_y)
      StopVectorDrawing()
    Else
      If image
        result=StartDrawing(ImageOutput(output_ID))
      ElseIf canvas
        result=StartDrawing(CanvasOutput(output_ID))
      Else
        result=StartDrawing(WindowOutput(output_ID)) 
      EndIf
      If Not result : ProcedureReturn -11 : EndIf
      
      DrawingMode(#PB_2DDrawing_Transparent)
      DrawingFont(FontID(Font(calculated_font_size)))
      
      DrawText_BF(output_x+output_width/2-text_width/2,
                  output_y+output_height/2-text_height/2,
                  text$,
                  text_color,
                  background_color,
                  flag,
                  mode,
                  text_stretching_x,
                  text_stretching_y)
      
      StopDrawing()   
    EndIf
    
    If get_all_window_events : While WindowEvent() : Wend : EndIf
    ProcedureReturn calculated_font_size
  EndProcedure
  
  Procedure.d ScaleVectorWidth_BF(set_scaling_x_.d=1.0)
    set_scaling_x=set_scaling_x_
    ProcedureReturn set_scaling_x
  EndProcedure
  
  Procedure.d ScaleVectorHeight_BF(set_scaling_y_.d=1.0)
    set_scaling_y=set_scaling_y_
    ProcedureReturn set_scaling_y
  EndProcedure
  
  Procedure.d Get_ScaleVectorWidth_BF()
    ProcedureReturn set_scaling_x
  EndProcedure
  
  Procedure.d Get_ScaleVectorHeight_BF()
    ProcedureReturn set_scaling_y
  EndProcedure
  
  Procedure FittedText_RotateRoundBox_Vector_BF(output_ID,
                                                Array Font(1),
                                                output_x=0,
                                                output_y=0,
                                                output_width=0,
                                                output_height=0,
                                                degree=0,
                                                radius=0,
                                                text$=" ", 
                                                text_color=#Black,
                                                background_color=#White,
                                                frame_thickness=0,
                                                frame_color=#Black,
                                                flag=1,
                                                padding_x=0,
                                                padding_y=0,
                                                resize_factor.f=1)
    
    output_x/set_scaling_x
    output_y/set_scaling_y
    
    Protected i, ii, result, background_offset
    
    If Not screen_output
      If IsImage(output_ID) ; check for image
        Protected image=1
      ElseIf IsSprite(output_ID) ; Check for PB sprite
        Protected sprite=1
      ElseIf IsGadget(output_ID) ; Check for canvas
        Protected canvas=1
      ElseIf IsWindow(output_ID) ; Check for window
        Protected window=1  
      Else
        ProcedureReturn -11
      EndIf
    EndIf
    
    If text$="" : text$=" " : EndIf
    
    If FindString(text$, #CR$) ; A part from DrawText_BF ;-)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    If activate_dpi_aware
      padding_x*DPI_ResizeFactor_BF()
      padding_y*DPI_ResizeFactor_BF()
      output_x*DPI_ResizeFactor_BF()
      output_y*DPI_ResizeFactor_BF()
      output_width*DPI_ResizeFactor_BF()
      output_height*DPI_ResizeFactor_BF()
    EndIf
    
    Protected calculated_font_size=Calculate_fitted_Font_BF(text$,
                                                            output_width,
                                                            output_height,
                                                            Font(),
                                                            0,
                                                            padding_x, 
                                                            padding_y,
                                                            0,
                                                            0,
                                                            resize_factor.f)
    
    Protected reduce=GetTextHeight_BF(" ", Font(calculated_font_size)/2) ; Correction because prevent automatically line break
    
    Protected text_width=GetTextWidth_BF(text$, Font(calculated_font_size), 0, 0)
    Protected text_height=GetTextHeight_BF(text$, Font(calculated_font_size), 0, 0)
    
    If frame_thickness<0
      background_offset=-frame_thickness
    EndIf
    
    from_FittedText_RotateRoundBox_Vector_BF=1
    
    result=RotateRoundBox_Vector_BF(output_ID,
                                    output_x+background_offset,
                                    output_y+background_offset,
                                    output_width-1-background_offset*2+reduce,
                                    output_height-1-background_offset*2,
                                    degree,
                                    radius,
                                    frame_thickness,
                                    frame_color,
                                    background_color)
    
    from_FittedText_RotateRoundBox_Vector_BF=0
    
    If result<1 : ProcedureReturn result : EndIf
    
    If image
      result=StartVectorDrawing(ImageVectorOutput(output_ID))
    ElseIf canvas
      result=StartVectorDrawing(CanvasVectorOutput(output_ID))
    Else
      result=StartVectorDrawing(WindowVectorOutput(output_ID)) 
    EndIf
    
    If Not result : ProcedureReturn -11 : EndIf
    
    ScaleCoordinates(set_scaling_x, set_scaling_y)
    
    VectorFont(FontID(Font(calculated_font_size)))
    
    If Not Alpha(text_color)
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), 255))
    ElseIf Alpha(text_color)=1
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), 0))
    Else
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), Alpha(text_color)))
    EndIf
    
    RotateCoordinates(output_width/2+output_x ,output_height/2+output_y, degree)
    
    MovePathCursor(output_width/2+output_x-text_width/2, output_height/2+output_y-text_height/2)
    
    text_width+reduce ; Correction because prevent automatically line break
    
    If flag=1 
      DrawVectorParagraph(text$, text_width, text_height, #PB_VectorParagraph_Center)
    ElseIf flag=2
      DrawVectorParagraph(text$, text_width, text_height, #PB_VectorParagraph_Right)
    ElseIf flag=3
      DrawVectorParagraph(text$, text_width, text_height, #PB_VectorParagraph_Block)
    Else
      DrawVectorParagraph(text$, text_width, text_height, #PB_VectorParagraph_Left)
    EndIf
    
    StopVectorDrawing()
    
    If get_all_window_events : While WindowEvent() : Wend : EndIf
    ProcedureReturn calculated_font_size
  EndProcedure
  
  Procedure FittedText_RoundBox_BF(output_ID,
                                   Array Font(1),           ; Set a Font array, for simple creating any look the sample here
                                   output_x=0,              ; Output pos x  - Ignore = -1
                                   output_y=0,              ; Output pos y  - Ignore = -1
                                   output_width=0,          ; Canvas width  - Ignore = -1
                                   output_height=0,         ; Canvas height - Ignore = -1
                                   radius=0,                ; Edge rounding radius
                                   text$=" ",               ; Text
                                   text_color=#Black,       ; Text color
                                   background_color=#White, ; Background color
                                   frame_thickness=0,       ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                                   frame_color=#Black,      ; Frame color
                                   flag=1,                  ; Flags : Text center=1 - Text right=2 - Text vertical=3
                                   mode=0,                  ; Modes : Revers text mode=1 - RTL text mode=2
                                   padding_x=0,             ; Text padding x
                                   padding_y=0,             ; Text padding y
                                   text_stretching_x=0,     ; Headline text stretching x
                                   text_stretching_y=0,     ; Headline text stretching y
                                   resize_factor.f=1)       ; Text resize factor
    
    If radius<0 : radius=0 : EndIf
    
    Protected i, ii, result, background_offset
    
    If frame_Thickness=2 : frame_thickness=3 : EndIf
    
    If Not screen_output
      If IsImage(output_ID) ; check for image
        Protected image=1
      ElseIf IsSprite(output_ID) ; Check for PB sprite
        Protected sprite=1
      ElseIf IsGadget(output_ID) ; Check for canvas
        Protected canvas=1
      ElseIf IsWindow(output_ID) ; Check for window
        Protected window=1  
      Else
        ProcedureReturn -11
      EndIf
    EndIf
    
    If text$="" : text$=" " : EndIf
    
    If flag=3 : Protected vertical=1 : EndIf
    
    If activate_dpi_aware
      padding_x*DPI_ResizeFactor_BF()
      padding_y*DPI_ResizeFactor_BF()
      output_x*DPI_ResizeFactor_BF()
      output_y*DPI_ResizeFactor_BF()
      output_width*DPI_ResizeFactor_BF()
      output_height*DPI_ResizeFactor_BF()
    EndIf
    
    Protected calculated_font_size=Calculate_fitted_Font_BF(text$,
                                                            output_width,
                                                            output_height,
                                                            Font(),
                                                            vertical,
                                                            padding_x, 
                                                            padding_y,
                                                            text_stretching_x,
                                                            text_stretching_y,
                                                            resize_factor.f)
    
    Protected text_width=GetTextWidth_BF(text$, Font(calculated_font_size), vertical, text_stretching_x)
    Protected text_height=GetTextHeight_BF(text$, Font(calculated_font_size), vertical, text_stretching_y)
    
    If screen_output
      result=StartDrawing(ScreenOutput())
    Else
      If image
        result=StartDrawing(ImageOutput(output_ID))
      ElseIf sprite
        result=StartDrawing(SpriteOutput(output_ID))
      ElseIf canvas
        result=StartDrawing(CanvasOutput(output_ID))
      Else
        result=StartDrawing(WindowOutput(output_ID)) 
      EndIf
    EndIf
    
    If Not result : ProcedureReturn -11 : EndIf
    
    If frame_thickness<0
      background_offset=-frame_thickness
    EndIf
    
    If background_color<>-1
      RoundBox(output_x+background_offset,
               output_y+background_offset,
               output_width-1-background_offset<<1,
               output_height-1-background_offset<<1, radius, radius, background_color)
    EndIf
    
    DrawingMode(#PB_2DDrawing_Outlined)
    If frame_thickness>0
      RoundBox(output_x+i, output_y+i, output_width-i*2-1, output_height-i*2-1, radius, radius, frame_color)  
      i=frame_thickness-1
      RoundBox(output_x+i, output_y+i, output_width-i*2-1, output_height-i*2-1, radius, radius, frame_color)
      If frame_thickness>2
        FillArea(output_x+output_width>>1, output_y+1, frame_color, frame_color)
      EndIf
    EndIf
    
    DrawingMode(#PB_2DDrawing_Transparent)
    DrawingFont(FontID(Font(calculated_font_size)))
    
    DrawText_BF(output_x+output_width/2-text_width/2,
                output_y+output_height/2-text_height/2,
                text$,
                text_color,
                background_color,
                flag,
                mode,
                text_stretching_x,
                text_stretching_y)
    
    StopDrawing()
    If get_all_window_events : While WindowEvent() : Wend : EndIf
    ProcedureReturn calculated_font_size
  EndProcedure
  
  Procedure FitGadgetText_BF(gadget_ID,
                             Array Font(1),
                             output_x,
                             output_y,
                             gadget_width,
                             gadget_height,
                             padding_x=0,
                             padding_y=0,
                             resize_factor.f=1)
    
    If Not IsGadget(gadget_ID) : ProcedureReturn -25 : EndIf
    
    If output_x=-1 : output_x=GadgetX(gadget_ID) : EndIf
    If output_y=-1 : output_y=GadgetY(gadget_ID) : EndIf
    If gadget_width<0 : gadget_width=GadgetWidth(gadget_ID) : EndIf
    If gadget_height<0 : gadget_height=GadgetHeight(gadget_ID) : EndIf
    
    Protected gadget_text$=GetGadgetText(gadget_ID)
    If gadget_text$="" : gadget_text$=" " : EndIf
    
    If activate_dpi_aware And gadget_width>0 And gadget_height>0
      Protected gadget_width_=gadget_width*DPI_ResizeFactor_BF()
      Protected gadget_height_=gadget_height*DPI_ResizeFactor_BF()
    Else
      gadget_width_=gadget_width
      gadget_height_=gadget_height
    EndIf
    
    Protected calculated_font_size=Calculate_fitted_Font_BF(gadget_text$,
                                                            gadget_width_,
                                                            gadget_height_,
                                                            Font(),
                                                            0,
                                                            padding_x, 
                                                            padding_y,
                                                            0,
                                                            0,
                                                            resize_factor.f)
    
    ResizeGadget(gadget_ID, output_x, output_y, gadget_width, gadget_height)
    SetGadgetFont(gadget_ID, FontID(Font(calculated_font_size)))
    SetGadgetText(gadget_ID, gadget_text$)
    If get_all_window_events : While WindowEvent() : Wend : EndIf
    ProcedureReturn calculated_font_size
  EndProcedure
  
  Procedure SetGadgetText_BF(gadget_ID,
                             Array Font(1),
                             output_x,
                             output_y,
                             gadget_width,
                             gadget_height,
                             gadget_text$=" ",
                             padding_x=0,
                             padding_y=0,
                             resize_factor.f=1)
    
    If Not IsGadget(gadget_ID) : ProcedureReturn -25 : EndIf
    
    If output_x=-1 : output_x=GadgetX(gadget_ID) : EndIf
    If output_y=-1 : output_y=GadgetY(gadget_ID) : EndIf
    If gadget_width<0 : gadget_width=GadgetWidth(gadget_ID) : EndIf
    If gadget_height<0 : gadget_height=GadgetHeight(gadget_ID) : EndIf
    
    If gadget_text$="" : gadget_text$=" " : EndIf
    
    If activate_dpi_aware And gadget_width>0 And gadget_height>0
      Protected gadget_width_=gadget_width*DPI_ResizeFactor_BF()
      Protected gadget_height_=gadget_height*DPI_ResizeFactor_BF()
    Else
      gadget_width_=gadget_width
      gadget_height_=gadget_height
    EndIf
    
    Protected calculated_font_size=Calculate_fitted_Font_BF(gadget_text$,
                                                            gadget_width_,
                                                            gadget_height_,
                                                            Font(),
                                                            0,
                                                            padding_x, 
                                                            padding_y,
                                                            0,
                                                            0,
                                                            resize_factor.f)
    
    ResizeGadget(gadget_ID, output_x, output_y, gadget_width, gadget_height)
    SetGadgetFont(gadget_ID, FontID(Font(calculated_font_size)))
    SetGadgetText(gadget_ID, gadget_text$)
    If get_all_window_events : While WindowEvent() : Wend : EndIf
    ProcedureReturn calculated_font_size
  EndProcedure
  
  Procedure ButtonGadgetSimple_BF(text$,
                                  Array Font(1),
                                  x,
                                  y,
                                  width,
                                  height,
                                  padding_x=-1,
                                  padding_y=-1,
                                  resize_factor.f=1)
    
    Protected gadget_ID=ButtonGadget(#PB_Any, x, y, width, height, text$, #PB_Button_MultiLine)
    If Not gadget_ID : ProcedureReturn 0 : EndIf
    
    If padding_x<0 : padding_x=25 : EndIf
    If padding_y<0 : padding_y=15 : EndIf
    
    FitGadgetText_BF(gadget_ID,       ; Enhance a ButtonGadget PB
                     Font(),          ; Font array
                     -1,              ; x
                     -1,              ; y
                     -1,              ; Width
                     -1,              ; Height
                     padding_x,       ; Padding x
                     padding_y,       ; Padding y
                     resize_factor.f) ; resize_factor.f
    
    ProcedureReturn gadget_ID
  EndProcedure
  
  Procedure TextGadgetSimple_BF(text$,
                                Array Font(1),
                                x,
                                y,
                                width,
                                height,
                                padding_x=-1,
                                padding_y=-1,
                                resize_factor.f=1,
                                flags=0)
    
    Protected gadget_ID=TextGadget(#PB_Any, x, y, width, height, text$, flags)
    If Not gadget_ID : ProcedureReturn 0 : EndIf
    
    If padding_x<0 : padding_x=25 : EndIf
    If padding_y<0 : padding_y=15 : EndIf
    
    FitGadgetText_BF(gadget_ID,       ; Enhance a ButtonGadget PB
                     Font(),          ; Font array
                     -1,              ; x
                     -1,              ; y
                     -1,              ; Width
                     -1,              ; Height
                     padding_x,       ; Padding x
                     padding_y,       ; Padding y
                     resize_factor.f) ; resize_factor.f
    
    ProcedureReturn gadget_ID
  EndProcedure
  
  Procedure CreateBackgroundImage_BF(overlay_ID,
                                     texture_ID,
                                     output_width,
                                     output_height,
                                     overlay_x=0,
                                     overlay_y=0,
                                     overlay_width=0,
                                     overlay_height=0,
                                     overlay_alpha=180,
                                     texture_alpha=2,
                                     activate_overlay_sprite_mode=1,
                                     color_distance_overlay=0,
                                     set_linear_gradient_color=$808080,
                                     set_background_color=$FFFFFF,
                                     texture_x=0,
                                     texture_y=0,
                                     texture_width=0,
                                     texture_height=0,
                                     texture_clip_x=0,
                                     texture_clip_y=0,
                                     texture_clip_width=0,
                                     texture_clip_height=0,
                                     texture_resize_x=0,
                                     texture_resize_y=0)
    
    If texture_alpha<2 : texture_alpha=2 : EndIf
    If overlay_width<=1 : overlay_width=output_width : EndIf
    If overlay_height<=1 : overlay_height=output_height : EndIf
    
    Protected output_ID=CreateImage(#PB_Any, output_width, output_height, 24, set_background_color)
    If Not output_ID
      ProcedureReturn -6
    EndIf
    
    If IsImage(texture_ID)
      
      If texture_resize_x<1 : texture_resize_x=ImageWidth(texture_ID) : EndIf
      If texture_resize_y<1 : texture_resize_y=ImageHeight(texture_ID) : EndIf
      
      If texture_resize_x>0 Or texture_resize_y>0  
        Protected temporary_texture_ID=CopyImage(texture_ID, #PB_Any)
        If temporary_texture_ID
          If Not ResizeImage(temporary_texture_ID, texture_resize_x, texture_resize_y, resize_interpolation)
            FreeImage(temporary_texture_ID)
            ProcedureReturn -27
          EndIf 
          texture_ID=temporary_texture_ID
        EndIf
      EndIf
      
      Protected result=BF(-texture_alpha, output_ID, texture_ID, ; Make a background texture
                          0,
                          0,
                          texture_x,
                          texture_y,
                          texture_width,
                          texture_height,
                          texture_clip_x,
                          texture_clip_y,
                          texture_clip_width,
                          texture_clip_height)
      
      FreeImage_BF(temporary_texture_ID)
      
      If result<>1 : FreeImage(output_ID) : ProcedureReturn result : EndIf
      
    EndIf
    
    Protected gradient_ID=CreateImage(#PB_Any, output_width, output_height)
    If Not gradient_ID
      FreeImage(output_ID)
      ProcedureReturn -6
    EndIf
    If StartDrawing(ImageOutput(gradient_ID))
      DrawingMode(#PB_2DDrawing_Gradient)
      BackColor(set_background_color)
      FrontColor(set_linear_gradient_color)
      LinearGradient(output_width, 0, output_width, output_height)
      Box(0, 0, output_width, output_height, set_background_color)
      StopDrawing()
      If StartDrawing(ImageOutput(output_ID))
        DrawingMode(#PB_2DDrawing_Default)
        DrawAlphaImage(ImageID(gradient_ID), 0, 0, 150)
        StopDrawing()
      EndIf
    EndIf
    FreeImage(gradient_ID)
    
    If IsImage(overlay_ID)
      Protected overlay_temp_ID=CopyImage(overlay_ID, #PB_Any)
      If Not overlay_temp_ID
        FreeImage(output_ID)
        ProcedureReturn -6
      EndIf
      If Not ResizeImage(overlay_temp_ID, overlay_width, overlay_height, resize_interpolation)
        FreeImage(output_ID)
        FreeImage(overlay_temp_ID)
        ProcedureReturn -27
      EndIf
      If activate_overlay_sprite_mode<0 Or activate_overlay_sprite_mode>2
        activate_overlay_sprite_mode=0
      EndIf
      
      Protected old_color_distance_overlay=percent
      percent=color_distance_overlay
      result=SpriteSimple_BF(activate_overlay_sprite_mode, output_ID, overlay_temp_ID,
                             0,             ; overlay_grab_invisible_color_x, ; Grab transparence color from sprite pos x - For preset set ..x=-1                         
                             0,             ; overlay_grab_invisible_color_y, ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF      
                             overlay_x,     ; Overlay texture x pos
                             overlay_y,     ; Overlay texture y pos
                             0,             ; You can here resize the output x
                             0,             ; You can here resize the output y
                             overlay_alpha) ; Set alpha blending from 2 To 256 - BF set 1=full visible - 256=invisible (BF specific)
      percent=old_color_distance_overlay
      
      FreeImage(overlay_temp_ID)
    EndIf
    
    ProcedureReturn output_ID
  EndProcedure
  
  Procedure SetFrontText_0_Progressbar_BF(txt_$="",
                                          txt_x_=0,
                                          txt_y_=0,
                                          txt_direction_.f=0,
                                          txt_color_=0,
                                          txt_alpha_=0,
                                          font_ID_=-1)
    
    If FindString(txt_$, #CR$) ; A part from DrawText_BF ;-)
      txt_$=ReplaceString(txt_$, #CRLF$, #LF$)
      txt_$=ReplaceString(txt_$, #LFCR$, #LF$)
      ReplaceString(txt_$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    font_ID_progressbar_0_BF=font_ID_
    txt_direction_progressbar_0_BF=txt_direction_
    txt_x_progressbar_0_BF=txt_x_
    txt_y_progressbar_0_BF=txt_y_
    txt_color_progressbar_0_BF=txt_color_
    txt_progressbar_0_BF$=txt_$
    txt_alpha_progressbar_0_BF=255-txt_alpha_
  EndProcedure
  
  Procedure SetFrontText_1_Progressbar_BF(txt_$="",
                                          txt_x_=0,
                                          txt_y_=0,
                                          txt_direction_.f=0,
                                          txt_color_=0,
                                          txt_alpha_=0,
                                          font_ID_=-1)
    
    If FindString(txt_$, #CR$) ; A part from DrawText_BF ;-)
      txt_$=ReplaceString(txt_$, #CRLF$, #LF$)
      txt_$=ReplaceString(txt_$, #LFCR$, #LF$)
      ReplaceString(txt_$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    font_ID_progressbar_1_BF=font_ID_
    txt_direction_progressbar_1_BF=txt_direction_
    txt_x_progressbar_1_BF=txt_x_
    txt_y_progressbar_1_BF=txt_y_
    txt_color_progressbar_1_BF=txt_color_
    txt_progressbar_1_BF$=txt_$
    txt_alpha_progressbar_1_BF=255-txt_alpha_
  EndProcedure
  
  Procedure SetFrontText_2_Progressbar_BF(txt_$="",
                                          txt_x_=0,
                                          txt_y_=0,
                                          txt_direction_.f=0,
                                          txt_color_=0,
                                          txt_alpha_=0,
                                          font_ID_=-1)
    
    If FindString(txt_$, #CR$) ; A part from DrawText_BF ;-)
      txt_$=ReplaceString(txt_$, #CRLF$, #LF$)
      txt_$=ReplaceString(txt_$, #LFCR$, #LF$)
      ReplaceString(txt_$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    font_ID_progressbar_2_BF=font_ID_
    txt_direction_progressbar_2_BF=txt_direction_
    txt_x_progressbar_2_BF=txt_x_
    txt_y_progressbar_2_BF=txt_y_
    txt_color_progressbar_2_BF=txt_color_
    txt_progressbar_2_BF$=txt_$
    txt_alpha_progressbar_2_BF=255-txt_alpha_
  EndProcedure
  
  Procedure SetFrontText_3_Progressbar_BF(txt_$="",
                                          txt_x_=0,
                                          txt_y_=0,
                                          txt_direction_.f=0,
                                          txt_color_=0,
                                          txt_alpha_=0,
                                          font_ID_=-1)
    
    If FindString(txt_$, #CR$) ; A part from DrawText_BF ;-)
      txt_$=ReplaceString(txt_$, #CRLF$, #LF$)
      txt_$=ReplaceString(txt_$, #LFCR$, #LF$)
      ReplaceString(txt_$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    font_ID_progressbar_3_BF=font_ID_
    txt_direction_progressbar_3_BF=txt_direction_
    txt_x_progressbar_3_BF=txt_x_
    txt_y_progressbar_3_BF=txt_y_
    txt_color_progressbar_3_BF=txt_color_
    txt_progressbar_3_BF$=txt_$
    txt_alpha_progressbar_3_BF=255-txt_alpha_
  EndProcedure
  
  Procedure SetFrontText_4_Progressbar_BF(txt_$="",
                                          txt_x_=0,
                                          txt_y_=0,
                                          txt_direction_.f=0,
                                          txt_color_=0,
                                          txt_alpha_=0,
                                          font_ID_=-1)
    
    If FindString(txt_$, #CR$) ; A part from DrawText_BF ;-)
      txt_$=ReplaceString(txt_$, #CRLF$, #LF$)
      txt_$=ReplaceString(txt_$, #LFCR$, #LF$)
      ReplaceString(txt_$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    font_ID_progressbar_4_BF=font_ID_
    txt_direction_progressbar_4_BF=txt_direction_
    txt_x_progressbar_4_BF=txt_x_
    txt_y_progressbar_4_BF=txt_y_
    txt_color_progressbar_4_BF=txt_color_
    txt_progressbar_4_BF$=txt_$
    txt_alpha_progressbar_4_BF=255-txt_alpha_
  EndProcedure
  
  Procedure Set_DividingLine_Progressbar_BF(divisor_progressbar_pos_=-1, divisor_progressbar_color_=#Red, divisor_progressbar_alpha_=255)
    divisor_progressbar_pos=divisor_progressbar_pos_
    divisor_progressbar_color=divisor_progressbar_color_
    divisor_progressbar_alpha=divisor_progressbar_alpha_
  EndProcedure 
  
  Procedure Calculate_SliderPos_Progressbar_BF(bar_ID, pos, direction=#PB_ProgressBar_Horizontal, speed_factor.d=1) ; #PB_ProgressBar_Vertical - #PB_ProgressBar_Horizontal
    If Not IsGadget(bar_ID) : ProcedureReturn 0 : EndIf
    Protected min=GetGadgetAttribute(bar_ID, #PB_ProgressBar_Minimum), max=GetGadgetAttribute(bar_ID, #PB_ProgressBar_Maximum)
    If pos>max-min Or pos<0 : ProcedureReturn 0 : EndIf 
    If direction=#PB_ProgressBar_Horizontal
      Protected diameter.d=GadgetWidth(bar_ID)
    ElseIf direction=#PB_ProgressBar_Vertical
      diameter.d=GadgetHeight(bar_ID)
    Else
      ProcedureReturn 0
    EndIf
    If Not pos : pos=1 : EndIf
    ProcedureReturn (diameter/(max-min)+0.06*speed_factor)*pos-4
  EndProcedure
  
  Procedure SetGadgetState_Progressbar_BF(bar_ID,
                                          state,
                                          flags=0,
                                          mode=0,
                                          slider_color_1=#Green,
                                          background_color=$E6E6E6,
                                          slider_color_2=-1,
                                          border=-1,
                                          border_color=$AAAAAA,
                                          linear_gradient_offset=0)
    
    If Not IsGadget(bar_ID) : ProcedureReturn 0 : EndIf
    
    If border<0 : Protected canvas_flag_progressbar=#PB_Canvas_Border : border=0 : EndIf
    
    If linear_gradient_offset<0 : linear_gradient_offset*-1 : Protected flip=1 : EndIf
    
    Protected min=GetGadgetAttribute(bar_ID, #PB_ProgressBar_Minimum), max=GetGadgetAttribute(bar_ID, #PB_ProgressBar_Maximum)
    If state<>#PB_ProgressBar_Unknown
      If state<min : state=min : EndIf
      If state>max : state=max : EndIf
    EndIf
    SetGadgetState(bar_ID, state)
    
    If mode<0
      Protected hidden_image=1
      mode*-1
    EndIf
    
    Protected image_ID, image_1_ID, image_2_ID, output_ID
    Protected width.d=GadgetWidth(bar_ID), height.d=GadgetHeight(bar_ID), step_width.d, i.d
    
    Structure bars_progressbar_BF
      image_ID.i
      output_ID.i
      bar_ID.i
      progress.i
      old_border.i
      old_width.f
      old_height.f
    EndStructure
    
    Static NewList bars_progressbar_BF.bars_progressbar_BF()
    
    Macro delete_variables_Progressbar_BF
      txt_progressbar_0_BF$=""
      txt_progressbar_1_BF$=""
      txt_progressbar_2_BF$=""
      txt_progressbar_3_BF$=""
      txt_progressbar_4_BF$=""
      divisor_progressbar_pos=-1
    EndMacro
    
    If flags=-2
      error:
      ForEach bars_progressbar_BF()
        While NextElement(bars_progressbar_BF())
          If bar_ID=bars_progressbar_BF()\bar_ID
            If IsImage(bars_progressbar_BF()\image_ID)
              FreeImage(bars_progressbar_BF()\image_ID)
            EndIf
            If IsGadget(bars_progressbar_BF()\output_ID)
              FreeGadget(bars_progressbar_BF()\output_ID)
            EndIf
            If IsGadget(bar_ID)
              HideGadget(bar_ID, 0)
            EndIf
            DeleteElement(bars_progressbar_BF())
          EndIf
        Wend
      Next
      delete_variables_Progressbar_BF
      ProcedureReturn 0
    EndIf
    
    If min<1 : min=1 : EndIf : If max<1 : max=1 : EndIf
    HideGadget(bar_ID, 1)
    ForEach bars_progressbar_BF()
      If bar_ID=bars_progressbar_BF()\bar_ID
        If width<>bars_progressbar_BF()\old_width Or
           height<>bars_progressbar_BF()\old_height Or
           border<>bars_progressbar_BF()\old_border
          Goto error
        EndIf
        image_ID=bars_progressbar_BF()\image_ID
        output_ID=bars_progressbar_BF()\output_ID
        bars_progressbar_BF()\progress+2
        Break
      EndIf
    Next
    
    image_1_ID=CreateImage(#PB_Any,
                           (width+border*2+1)*DPI_ResizeFactor_BF(),
                           (height+border*2+1)*DPI_ResizeFactor_BF(),
                           24,
                           border_color)
    
    If Not image_1_ID : Goto error : EndIf
    
    If Not image_ID  
      image_ID=CreateImage(#PB_Any,
                           width*DPI_ResizeFactor_BF(), 
                           height*DPI_ResizeFactor_BF(),
                           24,
                           background_color)
      
      If Not image_ID : Goto error : EndIf 
      
      AddElement(bars_progressbar_BF())
      bars_progressbar_BF()\image_ID=image_ID
      bars_progressbar_BF()\bar_ID=bar_ID
      bars_progressbar_BF()\old_border=border
      bars_progressbar_BF()\old_width=width
      bars_progressbar_BF()\old_height=height
    EndIf
    
    If Not StartDrawing(ImageOutput(image_ID)) : Goto error : EndIf
    
    Box(0,
        0,
        width*DPI_ResizeFactor_BF(),
        height*DPI_ResizeFactor_BF(),
        background_color)
    
    If flags=#PB_ProgressBar_Vertical
      
      If slider_color_2>-1
        DrawingMode(#PB_2DDrawing_Gradient)
        If flip
          FrontColor(slider_color_2) : BackColor(slider_color_1)
        Else
          FrontColor(slider_color_1) : BackColor(slider_color_2)
        EndIf
        GradientColor(Linear_gradient_offset/100, slider_color_2)
        If mode>1
          LinearGradient(0, 0, 0, height*DPI_ResizeFactor_BF())
        Else
          LinearGradient(0, 0, width*DPI_ResizeFactor_BF(), 0)
        EndIf
      EndIf
      If state=#PB_ProgressBar_Unknown
        Protected height_div=height/2
        If bars_progressbar_BF()\progress>height+height_div
          bars_progressbar_BF()\progress=0
        EndIf
        Box(0,
            (height-bars_progressbar_BF()\progress)*DPI_ResizeFactor_BF(),
            width*DPI_ResizeFactor_BF(),
            height/2*DPI_ResizeFactor_BF(),
            slider_color_1)
        
        DrawingMode(#PB_2DDrawing_Default)
      Else
        Box(0,
            ((height-(state-min)*(height/(max-min-1)))-1)*DPI_ResizeFactor_BF(),
            width*DPI_ResizeFactor_BF(),
            height*DPI_ResizeFactor_BF(),
            slider_color_1) 
      EndIf
    Else
      If slider_color_2>-1
        DrawingMode(#PB_2DDrawing_Gradient)
        If flip
          FrontColor(slider_color_1) : BackColor(slider_color_2)
        Else
          FrontColor(slider_color_2) : BackColor(slider_color_1)
        EndIf
        GradientColor(Linear_gradient_offset/100, slider_color_1)
        If mode>1
          LinearGradient(0, 0, width*DPI_ResizeFactor_BF(), 0)
        Else
          LinearGradient(0, 0, 0, height*DPI_ResizeFactor_BF())
        EndIf
      EndIf
      If state=#PB_ProgressBar_Unknown
        Protected width_div=width/2
        If bars_progressbar_BF()\progress>width
          bars_progressbar_BF()\progress=-width_div
        EndIf
        Box(bars_progressbar_BF()\progress*DPI_ResizeFactor_BF(),
            0,
            width_div*DPI_ResizeFactor_BF(),
            height*DPI_ResizeFactor_BF(),
            slider_color_1)
        
        DrawingMode(#PB_2DDrawing_Default)
      Else
        Box(0,
            0,
            ((state-min)*(width/(max-min-1))-1)*DPI_ResizeFactor_BF(),
            height*DPI_ResizeFactor_BF(),
            slider_color_1)
      EndIf
    EndIf
    
    StopDrawing()
    
    If StartVectorDrawing(ImageVectorOutput(image_ID))
      
      If txt_progressbar_0_BF$<>"" Or txt_progressbar_1_BF$ Or txt_progressbar_2_BF$ Or txt_progressbar_3_BF$ Or txt_progressbar_4_BF$
        
        ScaleCoordinates(set_scaling_x, set_scaling_y)
        
        Macro use_text_progressbar_BF(number, number_)
          Protected txt_x_#number=txt_x_progressbar_#number*DPI_ResizeFactor_BF()
          Protected txt_y_#number=txt_y_progressbar_#number*DPI_ResizeFactor_BF()
          If txt_progressbar_#number_<>""
            VectorSourceColor(RGBA(Red(txt_color_progressbar_#number), Green(txt_color_progressbar_#number), Blue(txt_color_progressbar_#number), txt_alpha_progressbar_#number))
            MovePathCursor(txt_x_#number, txt_y_#number)            
            If IsFont(font_ID_progressbar_#number) : VectorFont(FontID(font_ID_progressbar_#number)) : Else : VectorFont(FontID(SysFont_BF)) : EndIf 
            RotateCoordinates(txt_x_#number, txt_y_#number, txt_direction_progressbar_#number)
            DrawVectorParagraph(txt_progressbar_#number_, VectorTextWidth(txt_progressbar_#number_), VectorTextHeight(txt_progressbar_#number_), #PB_VectorParagraph_Center)
          EndIf
        EndMacro
        use_text_progressbar_BF(0_BF, 0_BF$)
        use_text_progressbar_BF(1_BF, 1_BF$)
        use_text_progressbar_BF(2_BF, 2_BF$)
        use_text_progressbar_BF(3_BF, 3_BF$)
        use_text_progressbar_BF(4_BF, 4_BF$)
        
      EndIf
      
      If divisor_progressbar_pos<>-1
        VectorSourceColor(RGBA(Red(divisor_progressbar_color), Green(divisor_progressbar_color), Blue(divisor_progressbar_color), divisor_progressbar_alpha))
        If flags=#PB_ProgressBar_Horizontal
          AddPathBox(((divisor_progressbar_pos-min)*(width/(max-min-1))-1)*DPI_ResizeFactor_BF(),
                     0,
                     3*DPI_ResizeFactor_BF(),
                     height*DPI_ResizeFactor_BF()) 
          StrokePath(3*DPI_ResizeFactor_BF())
        ElseIf flags=#PB_ProgressBar_Vertical
          AddPathBox(0,
                     ((height-(divisor_progressbar_pos-min)*(height/(max-min-1)))-1)*DPI_ResizeFactor_BF(),
                     width*DPI_ResizeFactor_BF(),
                     3*DPI_ResizeFactor_BF())  
          StrokePath(3*DPI_ResizeFactor_BF())
        EndIf
      EndIf
      
      StopVectorDrawing()
      
    EndIf
    
    If hidden_image ; Output as hidden image
      If Not output_ID
        output_ID=CreateImage(#PB_Any,
                              (width+(border*2))*DPI_ResizeFactor_BF(),
                              (height+(border*2))*DPI_ResizeFactor_BF())    
        If Not output_ID : Goto error : EndIf
        bars_progressbar_BF()\output_ID=output_ID
      EndIf
      If Not output_ID : Goto error : EndIf
      If Not StartDrawing(ImageOutput(output_ID)) : Goto error : EndIf
    Else ; Output as canvas
      If Not output_ID
        output_ID=CanvasGadget(#PB_Any,
                               GadgetX(bar_ID)-border,
                               GadgetY(bar_ID)-border,
                               width+(border*2),
                               height+(border*2),
                               canvas_flag_progressbar)
        If Not output_ID : Goto error : EndIf
        bars_progressbar_BF()\output_ID=output_ID
      EndIf
      If Not StartDrawing(CanvasOutput(output_ID)) : Goto error : EndIf
    EndIf
    
    DrawImage(ImageID(image_1_ID), 0, 0)
    DrawImage(ImageID(image_ID), border*DPI_ResizeFactor_BF(), border*DPI_ResizeFactor_BF())
    StopDrawing() : FreeImage(image_1_ID)
    delete_variables_Progressbar_BF
    ProcedureReturn output_ID
  EndProcedure
  
  Macro magictext_gadget_add_background
    If IsImage(MagicStretchText_Add_Background_ID)
      
      old_no_caching=no_caching
      no_caching=1
      ; If you want to change the source permanently, switch off temporary the caching before with NoCaching_BF(1)
      alpha_=MagicStretchText_Add_Background_alpha*-1
      If alpha_>-2 : alpha_=-2 : EndIf
      
      Protected old_alpha_sprite=alpha_sprite
      alpha_sprite=Is_AlphaChannel_BF(MagicStretchText_Add_Background_ID)
      
      BucketFill_BF(alpha_, image_1_ID, MagicStretchText_Add_Background_ID,
                    0,                                      ; Grab transparence color from sprite pos x - For preset set ..x=-1  
                    0,                                      ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF 
                    MagicStretchText_Add_Background_x_pos,  ; Startposition texture output x - Preset 0
                    MagicStretchText_Add_Background_y_pos,  ; Startposition texture output y - Preset 0
                    0,                                      ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                    0,                                      ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                    0,                                      ; Startposition inside the texture x - Preset 0
                    0,                                      ; Startposition inside the texture y - Preset 0
                    0,                                      ; Endposition inside the texture - Preset 0 = full texture width
                    0,                                      ; Endposition inside the texture - Preset 0 = full texture height
                    MagicStretchText_Add_Background_width,  ; Pre resize texture x
                    MagicStretchText_Add_Background_height) ; Pre resize texture y
                                                            ; mode = 0         - Set only a positioning marker
                                                            ;       -1         - Ignore a texture and use a color - Set as texture_ID a color, as sample $FFFF00
                                                            ;       -2         - Standard preset texture mode
                                                            ;       -3 to -256 - Texture mode with alpha blending
                                                            ; For compatibility with BucketFill this function use negative mode values, but it works at the same also with positive values
                                                            ; This function can output on images, PB sprites and canvas
      alpha_sprite=old_alpha_sprite
      no_caching=old_no_caching 
      
    EndIf
  EndMacro
  
  Macro magictext_gadget_add_image
    If IsImage(MagicStretchText_Add_Image_ID)
      temp_loaded_image_ID=CopyImage(MagicStretchText_Add_Image_ID, #PB_Any)
      If Not temp_loaded_image_ID
        FreeImage(image_1_ID)
        ProcedureReturn-6
      EndIf
      If Not StartDrawing(ImageOutput(image_1_ID))
        FreeImage(image_1_ID)
        FreeImage(temp_loaded_image_ID)
        ProcedureReturn -4
      EndIf
      GetFitSize_BF(ImageWidth(temp_loaded_image_ID),
                    ImageHeight(temp_loaded_image_ID),
                    ImageWidth(image_1_ID)/100*MagicStretchText_Add_Image_width_percent,
                    ImageHeight(image_1_ID)/100*MagicStretchText_Add_Image_height_percent)
      ResizeImage(temp_loaded_image_ID, GetFitSize_Width_BF(), GetFitSize_Height_BF())
      DrawAlphaImage(ImageID(temp_loaded_image_ID), MagicStretchText_Add_Image_x, MagicStretchText_Add_Image_y, MagicStretchText_Add_Image_alpha)
      StopDrawing()
      FreeImage(temp_loaded_image_ID)
    EndIf
  EndMacro
  
  Macro magictext_gadget_add_image_1
    If IsImage(MagicStretchText_Add_Image_1_ID)
      temp_loaded_image_ID=CopyImage(MagicStretchText_Add_Image_1_ID, #PB_Any)
      If Not temp_loaded_image_ID
        FreeImage(image_1_ID)
        ProcedureReturn-6
      EndIf
      If Not StartDrawing(ImageOutput(image_1_ID))
        FreeImage(image_1_ID)
        FreeImage(temp_loaded_image_ID)
        ProcedureReturn -4
      EndIf
      GetFitSize_BF(ImageWidth(temp_loaded_image_ID),
                    ImageHeight(temp_loaded_image_ID),
                    ImageWidth(image_1_ID)/100*MagicStretchText_Add_Image_1_width_percent,
                    ImageHeight(image_1_ID)/100*MagicStretchText_Add_Image_1_height_percent)
      ResizeImage(temp_loaded_image_ID, GetFitSize_Width_BF(), GetFitSize_Height_BF())
      DrawingMode(#PB_2DDrawing_AlphaBlend)
      DrawAlphaImage(ImageID(temp_loaded_image_ID), MagicStretchText_Add_Image_1_x, MagicStretchText_Add_Image_1_y, MagicStretchText_Add_Image_1_alpha)
      StopDrawing()
      FreeImage(temp_loaded_image_ID)
    EndIf
  EndMacro
  
  Macro magictext_gadget_add_image_2
    If IsImage(MagicStretchText_Add_Image_2_ID)
      temp_loaded_image_ID=CopyImage(MagicStretchText_Add_Image_2_ID, #PB_Any)
      If Not temp_loaded_image_ID
        FreeImage(image_1_ID)
        ProcedureReturn-6
      EndIf
      If Not StartDrawing(ImageOutput(image_1_ID))
        FreeImage(image_1_ID)
        FreeImage(temp_loaded_image_ID)
        ProcedureReturn -4
      EndIf
      GetFitSize_BF(ImageWidth(temp_loaded_image_ID),
                    ImageHeight(temp_loaded_image_ID),
                    ImageWidth(image_1_ID)/100*MagicStretchText_Add_Image_2_width_percent,
                    ImageHeight(image_1_ID)/100*MagicStretchText_Add_Image_2_height_percent)
      ResizeImage(temp_loaded_image_ID, GetFitSize_Width_BF(), GetFitSize_Height_BF())
      DrawingMode(#PB_2DDrawing_AlphaBlend)
      DrawAlphaImage(ImageID(temp_loaded_image_ID), MagicStretchText_Add_Image_2_x, MagicStretchText_Add_Image_2_y, MagicStretchText_Add_Image_2_alpha)
      StopDrawing()
      FreeImage(temp_loaded_image_ID)
    EndIf
  EndMacro
  
  Macro MagicStretchText_Add_Progressbar
    
    If MagicStretchText_Add_Progressbar_percent>0
      If Not StartDrawing(ImageOutput(image_1_ID))
        FreeImage(image_1_ID)
        ProcedureReturn -4
      EndIf
      DrawingMode(#PB_2DDrawing_AllChannels)
      Box((frame_size_stretchtext)*DPI_ResizeFactor_BF(),
          (frame_size_stretchtext)*DPI_ResizeFactor_BF(),
          ((ImageWidth(image_1_ID)-frame_size_stretchtext*DPI_ResizeFactor_BF())/100)*MagicStretchText_Add_Progressbar_percent,
          ImageHeight(image_1_ID)-frame_size_stretchtext*2*DPI_ResizeFactor_BF(),
          MagicStretchText_Add_Progressbar_color)
      StopDrawing()
    EndIf
    
  EndMacro
  
  Macro common_magictext_gadget
    
    If text_width*DPI_ResizeFactor_BF()<1 Or text_height*DPI_ResizeFactor_BF()<1 ; Fuse
      ProcedureReturn -6
    EndIf
    
    If gadget_type=1
      If Alpha(background_color)<>255 Or background_color=-1
        image_1_ID=CreateImage(#PB_Any,
                               button_width*DPI_ResizeFactor_BF(),
                               button_height*DPI_ResizeFactor_BF(), 32, #PB_2DDrawing_Transparent)
      Else
        image_1_ID=CreateImage(#PB_Any,
                               button_width*DPI_ResizeFactor_BF(),
                               button_height*DPI_ResizeFactor_BF(), 24, background_color)
      EndIf
    ElseIf Not gadget_type Or gadget_type=2
      If Alpha(background_color)<>255 Or background_color=-1
        image_1_ID=CreateImage(#PB_Any,
                               button_width*DPI_ResizeFactor_BF(),
                               button_height*DPI_ResizeFactor_BF(), 32, background_color)
      Else
        image_1_ID=CreateImage(#PB_Any,
                               button_width*DPI_ResizeFactor_BF(),
                               button_height*DPI_ResizeFactor_BF(), 24, background_color)
      EndIf
    EndIf
    
    If Not image_1_ID
      ProcedureReturn 0
    EndIf
    
    If Not IsGadget(gadget_ID)
      If gadget_type=1
        gadget_ID=CanvasGadget(#PB_Any, x_pos, y_pos, text_width, text_height, border_flag)
      ElseIf gadget_type=2
        gadget_ID=ImageGadget(#PB_Any, x_pos, y_pos, text_width, text_height, 0, border_flag)
      Else
        gadget_ID=ButtonImageGadget(#PB_Any, x_pos, y_pos, text_width, text_height, 0, border_flag)
      EndIf
    EndIf
    
    If background_color<>-1
      If Not StartDrawing(ImageOutput(image_1_ID))
        FreeImage(image_1_ID)
        ProcedureReturn -4
      EndIf
      If Alpha(background_color)
        DrawingMode(#PB_2DDrawing_AllChannels)
      EndIf
      Box(0, 0, ImageWidth(image_1_ID), ImageHeight(image_1_ID), background_color)
      StopDrawing()
    EndIf
    
    If background_color=-1 And gadget_type=1
      If Not StartDrawing(ImageOutput(image_1_ID))
        FreeImage(image_1_ID)
        ProcedureReturn -4
      EndIf
      If Alpha(background_color)
        DrawingMode(#PB_2DDrawing_AllChannels)
      EndIf
      Box(0, 0, ImageWidth(image_1_ID), ImageHeight(image_1_ID), $FFFFFFFF)
      StopDrawing()
    EndIf
    
    If Not MagicStretchText_Add_Progressbar_layer
      MagicStretchText_Add_Progressbar
    EndIf
    
    If Not MagicStretchText_Add_Image_sticky
      magictext_gadget_add_image
    EndIf
    
    If Not MagicStretchText_Add_Image_1_sticky
      magictext_gadget_add_image_1
    EndIf
    
    If Not MagicStretchText_Add_Image_2_sticky
      magictext_gadget_add_image_2
    EndIf
    
    If MagicStretchText_Add_Progressbar_layer=1
      MagicStretchText_Add_Progressbar
    EndIf
    
    Protected old_activate_dpi_aware=activate_dpi_aware  
    activate_dpi_aware=1
    
    ResizeGadget(gadget_ID, x_pos, y_pos, button_width, button_height)
    
    magictext_gadget_add_background
    
    If text$<>""
      Protected old_activate_rotating_stretchtext=activate_rotating_stretchtext
      activate_rotating_stretchtext=0
      Magic_StretchText_Pro_BF(image_1_ID,                 ; All channels supported
                               text$,                      ; Text
                               1+stretchtext_pro_offset_x, ; Output x pos
                               1+stretchtext_pro_offset_y, ; Output y pos
                               text_width,                 ; Text width
                               text_height,                ; Text height
                               button_width,               ; Fitted text width
                               button_height,              ; fitted text height
                               text_color,                 ; text_color - Alpha channel available - Sample $FFAAAAAA - $FF is the alpha channel
                               -1,                         ; background_color - Alpha channel available - For invisible background set To -1
                               stretch_x,                  ; Text stretching x - Headline mode
                               stretch_y,                  ; Text stretching y - Headline mode
                               padding_x,                  ; Padding x - This is a virtual padding for resizing and boxed output
                               padding_y,                  ; Padding y - This is a virtual padding for resizing and boxed output
                               text_flag,                  ; Text center=1 - Text right=2 - Text vertical=3
                               0,                          ; Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
                               "",                         ; Font type - more is not needed - accept also ""
                               0,                          ; Font style
                               font_ID)                    ; You can inherit a font
      activate_rotating_stretchtext=old_activate_rotating_stretchtext
    EndIf   
    
    If MagicStretchText_Add_Image_sticky
      magictext_gadget_add_image
    EndIf
    
    If MagicStretchText_Add_Image_1_sticky
      magictext_gadget_add_image_1
    EndIf
    
    If MagicStretchText_Add_Image_2_sticky
      magictext_gadget_add_image_2
    EndIf
    
    activate_dpi_aware=old_activate_dpi_aware
    
    If gadget_type=1
      If Not StartDrawing(CanvasOutput(gadget_ID))
        FreeImage(image_1_ID)
        ProcedureReturn -5
      EndIf
      DrawAlphaImage(ImageID(image_1_ID), 0, 0)
      StopDrawing()
    Else
      If gadget_type=2
        SetGadgetState(gadget_ID, ImageID(image_1_ID))  
      Else
        SetGadgetAttribute(gadget_ID, #PB_Button_Image, ImageID(image_1_ID))
      EndIf
    EndIf
    
    gadget_bf()\gadget_ID=gadget_ID
    gadget_bf()\temporary_image_ID=image_1_ID
    If gadget_type=1
      gadget_bf()\gadget_type=#StretchTextCanvasGadget_Pro_BF
    ElseIf gadget_type=2
      gadget_bf()\gadget_type=#StretchTextImageGadget_Pro_BF
    Else
      gadget_bf()\gadget_type=#StretchTextButtonImageGadget_Pro_BF 
    EndIf
    gadget_bf()\text$=text$
    gadget_bf()\x=x_pos
    gadget_bf()\y=y_pos
    gadget_bf()\text_width=text_width
    gadget_bf()\text_height=text_height
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\stretch_x=stretch_x 
    gadget_bf()\stretch_y=stretch_y
    gadget_bf()\padding_x_gadget=padding_x
    gadget_bf()\padding_y_gadget=padding_y
    gadget_bf()\text_color=text_color
    gadget_bf()\background_color=background_color
    
  EndMacro
  
  Procedure MagicStretchText_Add_Background_BF(MagicStretchText_Add_Background_ID_=-1,
                                               MagicStretchText_Add_Background_x_pos_=0,
                                               MagicStretchText_Add_Background_y_pos_=0,
                                               MagicStretchText_Add_Background_width_=0,
                                               MagicStretchText_Add_Background_height_=0, 
                                               MagicStretchText_Add_Background_alpha_=0)
    MagicStretchText_Add_Background_ID=MagicStretchText_Add_Background_ID_
    MagicStretchText_Add_Background_x_pos=MagicStretchText_Add_Background_x_pos_
    MagicStretchText_Add_Background_y_pos=MagicStretchText_Add_Background_y_pos_
    MagicStretchText_Add_Background_width=MagicStretchText_Add_Background_width_
    MagicStretchText_Add_Background_height=MagicStretchText_Add_Background_height_
    MagicStretchText_Add_Background_alpha=MagicStretchText_Add_Background_alpha_
    ProcedureReturn MagicStretchText_Add_Background_ID
  EndProcedure
  
  Procedure MagicStretchText_Add_Image_BF(MagicStretchText_Add_Image_ID_=-1,
                                          MagicStretchText_Add_Image_x_=0,
                                          MagicStretchText_Add_Image_y_=0,
                                          MagicStretchText_Add_Image_width_percent_=100,
                                          MagicStretchText_Add_Image_height_percent_=100, 
                                          MagicStretchText_Add_Image_alpha_=255,
                                          MagicStretchText_Add_Image_sticky_=0)
    MagicStretchText_Add_Image_ID=MagicStretchText_Add_Image_ID_
    MagicStretchText_Add_Image_x=MagicStretchText_Add_Image_x_
    MagicStretchText_Add_Image_y=MagicStretchText_Add_Image_y_
    MagicStretchText_Add_Image_width_percent=MagicStretchText_Add_Image_width_percent_
    MagicStretchText_Add_Image_height_percent=MagicStretchText_Add_Image_height_percent_
    MagicStretchText_Add_Image_alpha=MagicStretchText_Add_Image_alpha_
    MagicStretchText_Add_Image_sticky=MagicStretchText_Add_Image_sticky_
    ProcedureReturn MagicStretchText_Add_Image_ID
  EndProcedure
  
  Procedure MagicStretchText_Add_Image_1_BF(MagicStretchText_Add_Image_1_ID_=-1,
                                            MagicStretchText_Add_Image_1_x_=0,
                                            MagicStretchText_Add_Image_1_y_=0,
                                            MagicStretchText_Add_Image_1_width_percent_=100,
                                            MagicStretchText_Add_Image_1_height_percent_=100, 
                                            MagicStretchText_Add_Image_1_alpha_=255,
                                            MagicStretchText_Add_Image_1_sticky_=0)
    MagicStretchText_Add_Image_1_ID=MagicStretchText_Add_Image_1_ID_
    MagicStretchText_Add_Image_1_x=MagicStretchText_Add_Image_1_x_
    MagicStretchText_Add_Image_1_y=MagicStretchText_Add_Image_1_y_
    MagicStretchText_Add_Image_1_width_percent=MagicStretchText_Add_Image_1_width_percent_
    MagicStretchText_Add_Image_1_height_percent=MagicStretchText_Add_Image_1_height_percent_
    MagicStretchText_Add_Image_1_alpha=MagicStretchText_Add_Image_1_alpha_
    MagicStretchText_Add_Image_1_sticky=MagicStretchText_Add_Image_1_sticky_
    ProcedureReturn MagicStretchText_Add_Image_1_ID
  EndProcedure
  
  Procedure MagicStretchText_Add_Image_2_BF(MagicStretchText_Add_Image_2_ID_=-1,
                                            MagicStretchText_Add_Image_2_x_=0,
                                            MagicStretchText_Add_Image_2_y_=0,
                                            MagicStretchText_Add_Image_2_width_percent_=100,
                                            MagicStretchText_Add_Image_2_height_percent_=100, 
                                            MagicStretchText_Add_Image_2_alpha_=255,
                                            MagicStretchText_Add_Image_2_sticky_=0)
    MagicStretchText_Add_Image_2_ID=MagicStretchText_Add_Image_2_ID_
    MagicStretchText_Add_Image_2_x=MagicStretchText_Add_Image_2_x_
    MagicStretchText_Add_Image_2_y=MagicStretchText_Add_Image_2_y_
    MagicStretchText_Add_Image_2_width_percent=MagicStretchText_Add_Image_2_width_percent_
    MagicStretchText_Add_Image_2_height_percent=MagicStretchText_Add_Image_2_height_percent_
    MagicStretchText_Add_Image_2_alpha=MagicStretchText_Add_Image_2_alpha_
    MagicStretchText_Add_Image_2_sticky=MagicStretchText_Add_Image_2_sticky_
    ProcedureReturn MagicStretchText_Add_Image_2_ID
  EndProcedure
  
  Procedure Calculate_MagicText_proportional_BF(text$,
                                                font_ID,
                                                text_width,
                                                text_height,
                                                text_stretching_x=0,
                                                text_stretching_y=0,
                                                vertical=0)
    
    If StartVectorDrawing(ImageVectorOutput(mini_temp_image_ID))
      
      VectorFont(FontID(font_ID))
      
      Protected get_text_width=GetTextWidth_Vector_BF(text$,
                                                      vertical,               ; Vertical text
                                                      text_stretching_x,      ; Text stretching x - Headline mode
                                                      preset_stretchfactor_x) ; Text scaling x
      
      Protected get_text_height=GetTextHeight_Vector_BF(text$,
                                                        vertical,               ; Vertical text
                                                        text_stretching_y,      ; Text stretching x - Headline mode
                                                        preset_stretchfactor_y) ; Text scaling y
      StopVectorDrawing()
    Else
      ProcedureReturn -4
    EndIf
    
    GetFitSize_BF(get_text_width, get_text_height, text_width, text_height)
    
    shrinked_width=text_width-GetFitSize_Width_BF()
    shrinked_height=text_height-GetFitSize_height_BF()
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure MagicStretchText_Add_ProgressBar_BF(MagicStretchText_Add_Progressbar_percent_=0,
                                                MagicStretchText_Add_Progressbar_color_=#Green,
                                                MagicStretchText_Add_Progressbar_layer_=0)
    MagicStretchText_Add_Progressbar_percent=MagicStretchText_Add_Progressbar_percent_
    MagicStretchText_Add_Progressbar_color=MagicStretchText_Add_Progressbar_color_
    MagicStretchText_Add_Progressbar_layer=MagicStretchText_Add_Progressbar_layer_
    ProcedureReturn MagicStretchText_Add_Progressbar_percent
  EndProcedure
  
  Procedure MagicTextGadget_BF(gadget_type,              ; 0 = ButtonImageGadget - 1 = CanvasGadget - 2 ImageGadget
                               text$,                    ; Text
                               x_pos,                    ; Gadget output x pos
                               y_pos,                    ; Gadget output x pos                                   
                               button_width,             ; Button width
                               button_height,            ; Button height
                               text_width,               ; Text width
                               text_height,              ; Text height
                               stretch_x=0,              ; Text stretching x - Headline mode
                               stretch_y=0,              ; Text stretching y - Headline mode
                               padding_x=5,              ; padding_x
                               padding_y=5,              ; Padding y
                               text_color=#White,        ; Text color - Alpha channel supported
                               background_color=$EEEEEE, ; Background color
                               text_flag=1,              ; Text flag - Text center=1 - Text right=2 - Text vertical=3
                               font$="Arial",            ; Font type - more is not needed - accept also ""
                               font_style=0,             ; Font style
                               border_flag=0)            ; Border flag
    
    If FindString(text$, #CR$) ; A part from DrawText_BF ;-)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    stretch_x*DPI_ResizeFactor_BF()
    stretch_y*DPI_ResizeFactor_BF()
    
    Protected font_ID, gadget_ID, image_1_ID, temp_loaded_image_ID, old_no_caching, alpha_
    
    ForEach(gadget_bf())
      If gadget_bf()\font$=font$ And gadget_bf()\font_style=font_style
        font$=gadget_bf()\font$
        font_ID=gadget_bf()\font_ID
        Break
      EndIf
    Next
    
    If Not IsFont(font_ID)
      font_ID=LoadFont(#PB_Any, font$, 11, font_style) ; Change not fontsize 11
      If Not IsFont(font_ID) : ProcedureReturn 0 : EndIf
    EndIf
    
    If deactivate_text_stretching
      
      Calculate_MagicText_proportional_BF(text$,
                                          font_ID,
                                          text_width,
                                          text_height,
                                          stretch_x,
                                          stretch_y)
      
      padding_x=(GetFitSize_Width_BF()+padding_x)
      padding_y=(GetFitSize_Height_BF()+padding_y)
    EndIf
    
    AddElement(gadget_bf())
    
    common_magictext_gadget
    
    gadget_bf()\flag=text_flag
    gadget_bf()\font$=font$
    gadget_bf()\font_ID=font_ID
    gadget_bf()\font_style=font_style
    
    ProcedureReturn gadget_ID
  EndProcedure
  
  Procedure SetMagicTextGadget_BF(gadget_ID,           ; Gadget_ID - look the demo code for using
                                  button_width=-1,     ; Button width
                                  button_height=-1,    ; Button height
                                  text_width=-1,       ; Text width
                                  text_height=-1,      ; Text height
                                  x_pos=-1,            ; Gadget output x pos
                                  y_pos=-1,            ; Gadget output x pos
                                  text$="",            ; Text
                                  stretch_x=-1,        ; Text stretching x - Headline mode
                                  stretch_y=-1,        ; Text stretching y - Headline mode
                                  padding_x=-1,        ; Padding x
                                  padding_y=-1,        ; Padding_y
                                  text_color=-1,       ; Text color
                                  background_color=-1) ; Background color
    
    If Not IsGadget(gadget_ID)
      ProcedureReturn 0
    EndIf
    
    If FindString(text$, #CR$) ; A part from DrawText_BF ;-)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    Protected font_ID, image_1_ID, temp_loaded_image_ID, font$, font_style, gadget_type, border_flag, text_flag, old_no_caching, alpha_
    
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        gadget_type=gadget_bf()\gadget_type
        If text$=""
          text$=gadget_bf()\text$
        EndIf
        If button_width=-1
          button_width=gadget_bf()\button_width
        EndIf
        If button_height=-1
          button_height=gadget_bf()\button_height
        EndIf
        If text_width=-1
          text_width=gadget_bf()\text_width
        EndIf
        If text_height=-1
          text_height=gadget_bf()\text_height
        EndIf
        If x_pos=-1
          x_pos=gadget_bf()\x
        EndIf
        If y_pos=-1
          y_pos=gadget_bf()\y
        EndIf
        If stretch_x<0
          stretch_x=gadget_bf()\stretch_x
        EndIf
        If stretch_y<0
          stretch_y=gadget_bf()\stretch_y
        EndIf
        If padding_x<0
          padding_x=gadget_bf()\padding_x_gadget
        EndIf
        If padding_y<0
          padding_y=gadget_bf()\padding_y_gadget
        EndIf
        If text_color=-1
          text_color=gadget_bf()\text_color
        EndIf
        If background_color=-1
          background_color=gadget_bf()\background_color
        EndIf
        
        text_flag=gadget_bf()\flag
        font$=gadget_bf()\font$
        font_ID=gadget_bf()\font_ID
        image_1_ID=gadget_bf()\temporary_image_ID
        
        Break
      EndIf
    Next
    
    If gadget_type=#StretchTextCanvasGadget_Pro_BF
      gadget_type=1
    ElseIf gadget_type=#StretchTextImageGadget_Pro_BF
      gadget_type=2
    ElseIf gadget_type=#StretchTextButtonImageGadget_Pro_BF
      gadget_type=0
    Else
      ProcedureReturn 0
    EndIf
    
    If Not IsFont(font_ID)
      ProcedureReturn 0
    EndIf
    
    If Not IsImage (image_1_ID)
      ProcedureReturn 0
    EndIf
    FreeImage(image_1_ID)
    
    If text_width<2 : text_width=2 : EndIf
    If text_height<2 : text_height=2 : EndIf
    If button_width<2 : button_width=2 : EndIf ; Fuse
    If button_height<2 : button_height=2 : EndIf
    
    common_magictext_gadget
    
    ProcedureReturn 1
  EndProcedure 
  
  Procedure SetMagicTextGadget_Proportional_BF(gadget_ID,           ; Gadget_ID - look the demo code for using
                                               resize_factor.d,     ; Resize factor - As sample 0.7
                                               text_width=-1,       ; Text width
                                               text_height=-1,      ; Text height
                                               x_pos=-1,            ; Gadget output x pos
                                               y_pos=-1,            ; Gadget output x pos
                                               text$="",            ; Text
                                               stretch_x=-1,        ; Text stretching x - Headline mode
                                               stretch_y=-1,        ; Text stretching y - Headline mode
                                               padding_x=-1,        ; Padding x
                                               padding_y=-1,        ; Padding_y
                                               text_color=-1,       ; Text color
                                               background_color=-1) ; Background color
    
    If Not IsGadget(gadget_ID)
      ProcedureReturn 0
    EndIf
    
    If FindString(text$, #CR$) ; A part from DrawText_BF ;-)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    Protected font_ID, image_1_ID, temp_loaded_image_ID, font$, font_style, gadget_type, border_flag, text_flag, old_no_caching, alpha_
    
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        gadget_type=gadget_bf()\gadget_type
        If text$=""
          text$=gadget_bf()\text$
        EndIf
        If text_width=-1
          text_width=gadget_bf()\text_width
        EndIf
        If text_height=-1
          text_height=gadget_bf()\text_height
        EndIf
        If x_pos=-1
          x_pos=gadget_bf()\x
        EndIf
        If y_pos=-1
          y_pos=gadget_bf()\y
        EndIf
        If stretch_x<0
          stretch_x=gadget_bf()\stretch_x
        EndIf
        If stretch_y<0
          stretch_y=gadget_bf()\stretch_y
        EndIf
        If padding_x<0
          padding_x=gadget_bf()\padding_x_gadget
        EndIf
        If padding_y<0
          padding_y=gadget_bf()\padding_y_gadget
        EndIf
        If text_color=-1
          text_color=gadget_bf()\text_color
        EndIf
        If background_color=-1
          background_color=gadget_bf()\background_color
        EndIf
        
        Protected button_width=gadget_bf()\button_width   
        Protected button_height=gadget_bf()\button_height
        Protected init_button_width=button_width
        Protected init_button_height=button_height
        
        text_flag=gadget_bf()\flag
        font$=gadget_bf()\font$
        font_ID=gadget_bf()\font_ID
        image_1_ID=gadget_bf()\temporary_image_ID
        
        Break
      EndIf
    Next
    
    If gadget_type=#StretchTextCanvasGadget_Pro_BF
      gadget_type=1
    ElseIf gadget_type=#StretchTextImageGadget_Pro_BF
      gadget_type=2
    ElseIf gadget_type=#StretchTextButtonImageGadget_Pro_BF
      gadget_type=0
    Else
      ProcedureReturn 0
    EndIf
    
    If Not IsFont(font_ID)
      ProcedureReturn 0
    EndIf
    
    If Not IsImage (image_1_ID)
      ProcedureReturn 0
    EndIf
    FreeImage(image_1_ID)
    
    If resize_factor>0
      button_width*resize_factor
      button_height*resize_factor
      If button_width>32e3 Or button_height>32e3 ; Fuse
        button_width=init_button_width
        button_height=init_button_height
      EndIf
    EndIf
    
    If text_width<2 : text_width=2 : EndIf
    If text_height<2 : text_height=2 : EndIf
    If button_width<2 : button_width=2 : EndIf ; Fuse
    If button_height<2 : button_height=2 : EndIf
    
    common_magictext_gadget
    
    If resize_factor>0
      gadget_bf()\button_width=init_button_width
      gadget_bf()\button_height=init_button_height
    EndIf
    
    ProcedureReturn 1
  EndProcedure 
  
  Macro macro_reset_variables_toggle_gadget
    slider_0_offset=0
    slider_1_offset=0
    centered_text_toggle_gadget=0
    canvas_toggle_gadget=0
    flags_toggle_gadget=0
    text_flag_toggle_gadget=#PB_VectorParagraph_Center
    canvas_stretchtext_gadget=0
    flags_stretchtext_gadget=0
    text_flag_stretchtext_gadget=#PB_VectorParagraph_Center
  EndMacro
  
  Macro common_toggle_gadget
    
    Protected resize_factor_fitted_image_width.d, resize_factor_fitted_image_height.d
    Protected adjust_fitted_image_x.d, adjust_fitted_image_y.d
    Protected enlarge_image_width, enlarge_image_height
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Windows
      
      If gadget_bf()\gadget_type=#ToggleImageGadget_BF Or gadget_bf()\gadget_type=#StretchTextImageGadget_BF ; ImageGadget based
        If state                                                                                             ; Left
          adjust_fitted_image_x=4                                                                            ; Adjust the output x position from the half sized embedded image
        Else                                                                                                 ; Right
          adjust_fitted_image_x=0                                                                            ; Adjust the output x position from the half sized embedded image
        EndIf
        resize_factor_fitted_image_width.d=2.0  ; Adjust the width from the full and half sized embedded image
        resize_factor_fitted_image_height.d=1.0 ; Adjust the height from the full and half sized embedded image
        adjust_fitted_image_y=2                 ; Adjust the output y position from the half sized embedded image
      EndIf
      
      If gadget_bf()\gadget_type=#ToggleCanvasGadget_BF Or gadget_bf()\gadget_type=#StretchTextCanvasGadget_BF ; CanvasGadget based
        If state                                                                                               ; Left
          adjust_fitted_image_x=1                                                                              ; Adjust the output position from the half sized embedded image
        Else                                                                                                   ; Right
          adjust_fitted_image_x=1                                                                              ; Adjust the output position from the half sized embedded image
        EndIf
        resize_factor_fitted_image_width.d=2.0  ; Adjust the width from the full and half sized embedded image
        resize_factor_fitted_image_height.d=1.0 ; Adjust the height from the full and half sized embedded image
        adjust_fitted_image_y=1.0               ; Adjust the output y position from the half sized embedded image
      EndIf
      
      If centered_text_toggle_gadget
        enlarge_image_width=2  ; Moves the main image right
        enlarge_image_height=3 ; Moves the main image down
      Else
        enlarge_image_width=0  ; Moves the tile image left
        enlarge_image_height=0 ; Moves the tile image up
      EndIf
      
    CompilerEndIf 
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Linux
      
      If Subsystem("gtk2")
        If gadget_bf()\gadget_type=#ToggleImageGadget_BF Or gadget_bf()\gadget_type=#StretchTextImageGadget_BF ; ImageGadget based
          If state                                                                                             ; Left
            adjust_fitted_image_x=2                                                                            ; Adjust the output x position from the half sized embedded image
          Else                                                                                                 ; Right
            adjust_fitted_image_x=2                                                                            ; Adjust the output x position from the half sized embedded image
          EndIf
          resize_factor_fitted_image_width.d=2.0  ; Adjust the width from the full and half sized embedded image
          resize_factor_fitted_image_height.d=1.0 ; Adjust the height from the full and half sized embedded image
          adjust_fitted_image_y=2                 ; Adjust the output y position from the half sized embedded image
        EndIf
        
        If gadget_bf()\gadget_type=#ToggleCanvasGadget_BF Or gadget_bf()\gadget_type=#StretchTextCanvasGadget_BF ; CanvasGadget based
          If state                                                                                               ; Left
            adjust_fitted_image_x=1                                                                              ; Adjust the output position from the half sized embedded image
          Else                                                                                                   ; Right
            adjust_fitted_image_x=1                                                                              ; Adjust the output position from the half sized embedded image
          EndIf
          resize_factor_fitted_image_width.d=2.0  ; Adjust the width from the full and half sized embedded image
          resize_factor_fitted_image_height.d=1.0 ; Adjust the height from the full and half sized embedded image
          adjust_fitted_image_y=1.0               ; Adjust the output y position from the half sized embedded image
        EndIf
        
        If centered_text_toggle_gadget
          enlarge_image_width=2  ; Moves the main image right
          enlarge_image_height=3 ; Moves the main image down
        Else
          enlarge_image_width=2  ; Moves the tile image left
          enlarge_image_height=2 ; Moves the tile image up
        EndIf
        
      ElseIf Subsystem("qt")
        
        If gadget_bf()\gadget_type=#ToggleImageGadget_BF Or gadget_bf()\gadget_type=#StretchTextImageGadget_BF ; ImageGadget based
          If state                                                                                             ; Left
            adjust_fitted_image_x=2                                                                            ; Adjust the output x position from the half sized embedded image
          Else                                                                                                 ; Right
            adjust_fitted_image_x=2                                                                            ; Adjust the output x position from the half sized embedded image
          EndIf
          resize_factor_fitted_image_width.d=2.0  ; Adjust the width from the full and half sized embedded image
          resize_factor_fitted_image_height.d=1.0 ; Adjust the height from the full and half sized embedded image
          adjust_fitted_image_y=2                 ; Adjust the output y position from the half sized embedded image
        EndIf
        
        If gadget_bf()\gadget_type=#ToggleCanvasGadget_BF Or gadget_bf()\gadget_type=#StretchTextCanvasGadget_BF ; CanvasGadget based
          If state                                                                                               ; Left
            adjust_fitted_image_x=2                                                                              ; Adjust the output position from the half sized embedded image
          Else                                                                                                   ; Right
            adjust_fitted_image_x=0                                                                              ; Adjust the output position from the half sized embedded image
          EndIf
          resize_factor_fitted_image_width.d=2.0   ; Adjust the width from the full and half sized embedded image
          resize_factor_fitted_image_height.d=1.05 ; Adjust the height from the full and half sized embedded image
          adjust_fitted_image_y=1.0                ; Adjust the output y position from the half sized embedded image
        EndIf
        
        If centered_text_toggle_gadget
          enlarge_image_width=2  ; Moves the main image right
          enlarge_image_height=3 ; Moves the main image down
        Else
          enlarge_image_width=2  ; Moves the tile image left
          enlarge_image_height=2 ; Moves the tile image up
        EndIf
        
      Else ; "gtk3"
        
        If gadget_bf()\gadget_type=#ToggleImageGadget_BF Or gadget_bf()\gadget_type=#StretchTextImageGadget_BF ; ImageGadget based
          If state                                                                                             ; Left
            adjust_fitted_image_x=2                                                                            ; Adjust the output x position from the half sized embedded image
          Else                                                                                                 ; Right
            adjust_fitted_image_x=2                                                                            ; Adjust the output x position from the half sized embedded image
          EndIf
          resize_factor_fitted_image_width.d=2.0  ; Adjust the width from the full and half sized embedded image
          resize_factor_fitted_image_height.d=1.0 ; Adjust the height from the full and half sized embedded image
          adjust_fitted_image_y=2                 ; Adjust the output y position from the half sized embedded image
        EndIf
        
        If gadget_bf()\gadget_type=#ToggleCanvasGadget_BF Or gadget_bf()\gadget_type=#StretchTextCanvasGadget_BF ; CanvasGadget based
          If state                                                                                               ; Left
            adjust_fitted_image_x=2                                                                              ; Adjust the output position from the half sized embedded image
          Else                                                                                                   ; Right
            adjust_fitted_image_x=0                                                                              ; Adjust the output position from the half sized embedded image
          EndIf
          resize_factor_fitted_image_width.d=2.0   ; Adjust the width from the full and half sized embedded image
          resize_factor_fitted_image_height.d=1.05 ; Adjust the height from the full and half sized embedded image
          adjust_fitted_image_y=0.7                ; Adjust the output y position from the half sized embedded image
        EndIf
        
        If centered_text_toggle_gadget
          enlarge_image_width=2  ; Moves the main image right
          enlarge_image_height=3 ; Moves the main image down
        Else
          enlarge_image_width=2  ; Moves the tile image left
          enlarge_image_height=2 ; Moves the tile image up
        EndIf
        
      EndIf
    CompilerEndIf
    
    CompilerIf #PB_Compiler_OS=#PB_OS_MacOS
      
      If gadget_bf()\gadget_type=#ToggleImageGadget_BF Or gadget_bf()\gadget_type=#StretchTextImageGadget_BF ; ImageGadget based
        If state                                                                                             ; Left
          adjust_fitted_image_x=2                                                                            ; Adjust the output x position from the half sized embedded image
        Else                                                                                                 ; Right
          adjust_fitted_image_x=2                                                                            ; Adjust the output x position from the half sized embedded image
        EndIf
        resize_factor_fitted_image_width.d=2.0  ; Adjust the width from the full and half sized embedded image
        resize_factor_fitted_image_height.d=1.0 ; Adjust the height from the full and half sized embedded image
        adjust_fitted_image_y=2                 ; Adjust the output y position from the half sized embedded image
      EndIf
      
      If gadget_bf()\gadget_type=#ToggleCanvasGadget_BF Or gadget_bf()\gadget_type=#StretchTextCanvasGadget_BF ; CanvasGadget based
        If state                                                                                               ; Left
          adjust_fitted_image_x=1                                                                              ; Adjust the output position from the half sized embedded image
        Else                                                                                                   ; Right
          adjust_fitted_image_x=1                                                                              ; Adjust the output position from the half sized embedded image
        EndIf
        resize_factor_fitted_image_width.d=2.0  ; Adjust the width from the full and half sized embedded image
        resize_factor_fitted_image_height.d=1.0 ; Adjust the height from the full and half sized embedded image
        adjust_fitted_image_y=1.0               ; Adjust the output y position from the half sized embedded image
      EndIf
      
      If centered_text_toggle_gadget
        enlarge_image_width=2  ; Moves the main image right
        enlarge_image_height=3 ; Moves the main image down
      Else
        enlarge_image_width=2  ; Moves the tile image left
        enlarge_image_height=2 ; Moves the tile image up
      EndIf
      
    CompilerEndIf
    
    If centered_text_toggle_gadget
      used_value=slider_color
      resize_factor_fitted_image_width/2
    Else
      used_value=background_color
    EndIf
    
    If IsGadget(gadget_ID) And IsImage(image_1_ID)
      FreeImage(image_1_ID)
    EndIf
    
    If button_width*DPI_ResizeFactor_BF()+enlarge_image_width<1 Or button_height*DPI_ResizeFactor_BF()+enlarge_image_height<1 ; Fuse
      ProcedureReturn -6
    EndIf
    image_1_ID=CreateImage(#PB_Any,
                           button_width*DPI_ResizeFactor_BF()+enlarge_image_width,
                           button_height*DPI_ResizeFactor_BF()+enlarge_image_height, 24, used_value)
    
    If Not image_1_ID
      macro_reset_variables_toggle_gadget
      ProcedureReturn 0
    EndIf
    
    Protected image_2_ID=CreateImage(#PB_Any, 16, 16, 24, slider_color)
    If Not image_2_ID
      FreeImage(image_1_ID)
      macro_reset_variables_toggle_gadget
      ProcedureReturn 0
    EndIf
    
    If Not StartVectorDrawing(ImageVectorOutput(image_2_ID))
      FreeImage(image_1_ID)
      FreeImage(image_2_ID)
      macro_reset_variables_toggle_gadget
      ProcedureReturn 0
    EndIf
    VectorFont(FontID(font_ID))
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Windows
      ; Enlarge width prevent automatically line breaks on DrawVectorParagraph()
      Protected get_text_width.d=VectorTextWidth_Multiline_BF(text$)+VectorTextHeight(" ")/3
      Protected get_text_height.d=VectorParagraphHeight(text$, get_text_width.d, 3e4)+VectorTextHeight(" ")/6
    CompilerEndIf
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Linux
      ; Enlarge width prevent automatically line breaks on DrawVectorParagraph()
      Protected get_text_width.d=VectorTextWidth_Multiline_BF(text$)+VectorTextHeight(" ")
      Protected get_text_height.d=VectorParagraphHeight(text$, get_text_width.d, 3e4)+VectorTextHeight(" ")*1.2
    CompilerEndIf
    
    CompilerIf #PB_Compiler_OS=#PB_OS_MacOS
      ; Enlarge width prevent automatically line breaks on DrawVectorParagraph()
      Protected get_text_width.d=VectorTextWidth_Multiline_BF(text$)+VectorTextHeight(" ")/3
      Protected get_text_height.d=VectorParagraphHeight(text$, get_text_width.d, 3e4)+VectorTextHeight(" ")/6
    CompilerEndIf
    
    StopVectorDrawing()
    Protected scaling_width.d=button_width/resize_factor_fitted_image_width/get_text_width
    Protected scaling_height.d=button_height/resize_factor_fitted_image_height/get_text_height
    ResizeImage(image_2_ID,
                button_width/resize_factor_fitted_image_width*DPI_ResizeFactor_BF(),
                button_height/resize_factor_fitted_image_height*DPI_ResizeFactor_BF(), resize_interpolation)
    
    If centered_text_toggle_gadget
      used_value=image_1_ID
    Else
      used_value=image_2_ID
    EndIf
    If Not StartVectorDrawing(ImageVectorOutput(used_value))
      FreeImage(image_1_ID)
      FreeImage(image_2_ID)
      macro_reset_variables_toggle_gadget
      ProcedureReturn 0
    EndIf
    
    VectorFont(FontID(font_ID))
    
    If Not Alpha(text_color)
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), 255))
    Else
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), Alpha(text_color)))
    EndIf
    
    If activate_linear_gradient
      VectorSourceLinearGradient(linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy)
    EndIf
    
    If activate_circular_gradient
      VectorSourceCircularGradient(circular_gradient_x, circular_gradient_y, circular_gradient_radius)
    EndIf
    
    Protected i
    For i=0 To #Size_GradientColors_Array
      If gradient_activate_flag(i)
        VectorSourceGradientColor(gradient_color(i), gradient_position.f(i))
      EndIf
    Next 
    
    If deactivate_text_stretching
      preset_stretchfactor_x_=preset_stretchfactor_x
      preset_stretchfactor_y_=preset_stretchfactor_y
      If preset_stretchfactor_x_>1 : preset_stretchfactor_x_=1 : EndIf
      If preset_stretchfactor_y_>1 : preset_stretchfactor_y_=1 : EndIf
      If scaling_width*DPI_ResizeFactor_BF()>scaling_height*DPI_ResizeFactor_BF()
        ScaleCoordinates(scaling_height*preset_stretchfactor_x_*DPI_ResizeFactor_BF(), scaling_height*preset_stretchfactor_y_*DPI_ResizeFactor_BF())
      ElseIf scaling_width*DPI_ResizeFactor_BF()<scaling_height*DPI_ResizeFactor_BF()
        ScaleCoordinates(scaling_width*preset_stretchfactor_x_*DPI_ResizeFactor_BF(), scaling_width*preset_stretchfactor_y_*DPI_ResizeFactor_BF())
      Else
        ScaleCoordinates(scaling_width*DPI_ResizeFactor_BF(), scaling_height*DPI_ResizeFactor_BF())
      EndIf
    Else
      ScaleCoordinates(scaling_width*DPI_ResizeFactor_BF(), scaling_height*DPI_ResizeFactor_BF())
    EndIf
    
    If state=-1
      DrawVectorParagraph(text$, get_text_width.d, 3e4, text_flag_stretchtext_gadget)
    Else
      DrawVectorParagraph(text$, get_text_width.d, 3e4, text_flag_toggle_gadget)
    EndIf
    
    StopVectorDrawing()
    
    Protected old_image_width=ImageWidth(used_value)
    Protected old_image_height=ImageHeight(used_value)
    
    used_value=ReduceSpriteMask_BF(used_value, 0, 0, 0)
    
    If Not IsImage(used_value) ; Empty
      used_value=CreateImage(#PB_Any, old_image_width, old_image_height, 24, slider_color)
    EndIf
    
    If Not IsImage(used_value)
      FreeImage(image_1_ID)
      FreeImage(image_2_ID)
      macro_reset_variables_toggle_gadget
      ProcedureReturn used_value
    EndIf
    
    If centered_text_toggle_gadget
      FreeImage(image_1_ID)
      image_1_ID=FitImage_DualPadding_BF(0,                ; Mode
                                         used_value,       ; Image_ID
                                         old_image_width,  ; Fit image to this width
                                         old_image_height, ; Fit image to this height
                                         slider_color,     ; Background color
                                         padding_x,        ; Padding x
                                         padding_y,        ; Padding y
                                         0,                ; Grid color
                                         0,                ; Grid size
                                         0,                ; Grid tiling
                                         0,                ; Frame color
                                         0)                ; frame_size      
      If Not IsImage(image_1_ID)
        FreeImage(image_2_ID)
        macro_reset_variables_toggle_gadget
        ProcedureReturn image_1_ID
      EndIf
    Else
      FreeImage(image_2_ID)
      image_2_ID=FitImage_DualPadding_BF(0,                ; Mode
                                         used_value,       ; Image_ID
                                         old_image_width,  ; Fit image to this width
                                         old_image_height, ; Fit image to this height
                                         slider_color,     ; Background color
                                         padding_x,        ; Padding x
                                         padding_y,        ; Padding y
                                         0,                ; Grid color
                                         0,                ; Grid size
                                         0,                ; Grid tiling
                                         0,                ; Frame color
                                         0)                ; frame_size  
      If Not IsImage(image_2_ID)
        FreeImage(image_1_ID)
        macro_reset_variables_toggle_gadget
        ProcedureReturn image_2_ID
      EndIf
    EndIf
    
    FreeImage(used_value)
    
    If Not IsGadget(gadget_ID)
      If state=-1
        If canvas_stretchtext_gadget
          gadget_ID=CanvasGadget(#PB_Any, x_pos, y_pos, button_width, button_height, flags_stretchtext_gadget)
        Else
          gadget_ID=ButtonImageGadget(#PB_Any, x_pos, y_pos, button_width, button_height, 0, flags_stretchtext_gadget)
        EndIf
      Else
        If canvas_toggle_gadget
          gadget_ID=CanvasGadget(#PB_Any, x_pos, y_pos, button_width, button_height, flags_toggle_gadget)
        Else
          gadget_ID=ButtonImageGadget(#PB_Any, x_pos, y_pos, button_width, button_height, 0, flags_toggle_gadget)
        EndIf
      EndIf
    EndIf
    
    If Not centered_text_toggle_gadget
      If StartDrawing(ImageOutput(image_1_ID))
        Protected slider_1_x_pos=(button_width/2-button_width/resize_factor_fitted_image_width)*DPI_ResizeFactor_BF()+adjust_fitted_image_x
        Protected slider_0_x_pos=button_width/2*DPI_ResizeFactor_BF()+(button_width/2-button_width/resize_factor_fitted_image_width)/4*DPI_ResizeFactor_BF()+adjust_fitted_image_x
        slider_moving_space=slider_0_x_pos-slider_1_x_pos-2
        If state
          slider_1_x_pos+slider_1_offset
          DrawImage(ImageID(image_2_ID),
                    slider_1_x_pos,
                    (button_height-button_height/resize_factor_fitted_image_height)/2*DPI_ResizeFactor_BF()+adjust_fitted_image_y)
        Else
          slider_0_x_pos-slider_0_offset
          DrawImage(ImageID(image_2_ID),
                    slider_0_x_pos,
                    (button_height-button_height/resize_factor_fitted_image_height)/2*DPI_ResizeFactor_BF()+adjust_fitted_image_y)
        EndIf
        StopDrawing()
      EndIf
    EndIf
    
    ResizeGadget(gadget_ID, x_pos, y_pos, button_width, button_height)
    SetGadgetAttribute(gadget_ID, #PB_Button_Image, ImageID(image_1_ID))
    
    FreeImage(image_2_ID)
    macro_reset_variables_toggle_gadget
    
    gadget_bf()\gadget_ID=gadget_ID
    gadget_bf()\temporary_image_ID=image_1_ID
    gadget_bf()\state=state
    gadget_bf()\x=x_pos
    gadget_bf()\y=y_pos
    gadget_bf()\button_width=button_width
    gadget_bf()\button_height=button_height
    gadget_bf()\padding_x_gadget=padding_x
    gadget_bf()\padding_y_gadget=padding_y
    gadget_bf()\background_color=background_color
    If state=1
      gadget_bf()\text_state_1$=text$
      gadget_bf()\text_color_state_1=text_color
      gadget_bf()\slider_color_state_1=slider_color
    Else
      gadget_bf()\text_state_0$=text$
      gadget_bf()\text_color_state_0=text_color
      gadget_bf()\slider_color_state_0=slider_color
    EndIf
    
  EndMacro
  
  Procedure StretchTextGadget_BF(text$,                    ; Text
                                 x_pos,                    ; Gadget output x pos
                                 y_pos,                    ; Gadget output y pos
                                 button_width,             ; Gadget width
                                 button_height,            ; Gadget height
                                 padding_x=0,              ; Padding x
                                 padding_y=0,              ; Padding y
                                 text_color=#White,        ; Text color
                                 background_color=$EEEEEE, ; Background color
                                 font$="Arial",            ; Font type - more is not needed - accept also ""
                                 font_style=0)             ; Font style
    
    Protected result=ToggleGadget_BF(-1,               ; Toggle state 0 or 1
                                     text$,            ; Text
                                     x_pos,            ; Gadget output x pos
                                     y_pos,            ; Gadget output x pos
                                     button_width,     ; Gadget width
                                     button_height,    ; Gadget height
                                     padding_x,        ; padding_x
                                     padding_y,        ; Padding y
                                     text_color,       ; Text color
                                     background_color, ; Slider color
                                     -1,               ; Background color
                                     font$,            ; Font type - more is not needed - accept also ""
                                     font_style)       ; Font style
    
    ProcedureReturn result
  EndProcedure
  
  Procedure SetStretchTextGadget_BF(gadget_ID,           ; Gadget_ID - look the demo code for using
                                    text$="",            ; Text
                                    x_pos=-1,            ; Gadget output x pos
                                    y_pos=-1,            ; Gadget output x pos
                                    button_width=-1,     ; Gadget width
                                    button_height=-1,    ; Gadget height
                                    padding_x=-1,        ; Padding x
                                    padding_y=-1,        ; Padding_y
                                    text_color=-1,       ; Text color
                                    background_color=-1) ; Background color
    
    Protected result=SetToggleGadget_BF(gadget_ID,        ; Gadget_ID - look the demo code for using
                                        -1,               ; Toggle state 0 or 1
                                        text$,            ; Text
                                        x_pos,            ; Gadget output x pos
                                        y_pos,            ; Gadget output x pos
                                        button_width,     ; Gadget width
                                        button_height,    ; Gadget height
                                        padding_x,        ; Padding x
                                        padding_y,        ; Padding_y
                                        text_color,       ; Text color
                                        background_color, ; Slider color
                                        -1)               ; Background color
    
    ProcedureReturn result
  EndProcedure
  
  Procedure GetPadding_X_StretchTextGadget_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\padding_x_gadget
      EndIf
    Next
    ProcedureReturn 0
  EndProcedure
  
  Procedure GetPadding_Y_StretchTextGadget_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\padding_y_gadget
      EndIf
    Next
    ProcedureReturn 0
  EndProcedure
  
  Procedure CanvasStretchTextGadget_BF()
    canvas_stretchtext_gadget=1
    ProcedureReturn canvas_stretchtext_gadget
  EndProcedure
  
  Procedure SetFlagsStretchTextGadget_BF(flags_stretchtext_gadget_)
    flags_stretchtext_gadget=flags_stretchtext_gadget_
    ProcedureReturn flags_stretchtext_gadget
  EndProcedure
  
  Procedure SetTextFlagsStretchTextGadget_BF(text_flag_Stretchtext_gadget_=#PB_VectorParagraph_Center)
    text_flag_stretchtext_gadget=text_flag_stretchtext_gadget_
    ProcedureReturn text_flag_stretchtext_gadget
  EndProcedure
  
  Procedure ToggleGadget_BF(state,                    ; Toggle state 0 or 1
                            text$,                    ; Text
                            x_pos,                    ; Gadget output x pos
                            y_pos,                    ; Gadget output x pos
                            button_width,             ; Gadget width
                            button_height,            ; Gadget height
                            padding_x=0,              ; padding_x
                            padding_y=0,              ; Padding y
                            text_color=#White,        ; Text color - Alpha channel supported
                            slider_color=#Red,        ; Slider color
                            background_color=$EEEEEE, ; Background color
                            font$="Arial",            ; Font type - more is not needed - accept also ""
                            font_style=0)             ; Font style
    
    If FindString(text$, #CR$) ; A part from DrawText_BF ;-)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    Protected font_ID, used_value, preset_stretchfactor_x_.d, preset_stretchfactor_y_.d
    Protected gadget_ID, image_1_ID
    
    ForEach(gadget_bf())
      If gadget_bf()\font$=font$ And gadget_bf()\font_style=font_style
        font$=gadget_bf()\font$
        font_ID=gadget_bf()\font_ID
        Break
      EndIf
    Next
    
    If Not IsFont(font_ID)
      font_ID=LoadFont(#PB_Any, font$, 11, font_style) ; Change not fontsize 11
      If Not IsFont(font_ID) : ProcedureReturn 0 : EndIf
    EndIf
    
    AddElement(gadget_bf())
    If state=-1
      centered_text_toggle_gadget=1
      If canvas_stretchtext_gadget
        gadget_bf()\gadget_type=#StretchTextCanvasGadget_BF
      Else
        gadget_bf()\gadget_type=#StretchTextImageGadget_BF
      EndIf
    Else
      If canvas_toggle_gadget
        gadget_bf()\gadget_type=#ToggleCanvasGadget_BF
      Else
        gadget_bf()\gadget_type=#ToggleImageGadget_BF
      EndIf
    EndIf
    
    common_toggle_gadget
    
    gadget_bf()\font$=font$
    gadget_bf()\font_ID=font_ID
    gadget_bf()\font_style=font_style
    
    ProcedureReturn gadget_ID
  EndProcedure
  
  Procedure SetToggleGadget_BF(gadget_ID,           ; Gadget_ID - look the demo code for using
                               state=-1,            ; Toggle state 0 or 1
                               text$="",            ; Text
                               x_pos=-1,            ; Gadget output x pos
                               y_pos=-1,            ; Gadget output x pos
                               button_width=-1,     ; Gadget width
                               button_height=-1,    ; Gadget height
                               padding_x=-1,        ; Padding x
                               padding_y=-1,        ; Padding_y
                               text_color=-1,       ; Text color
                               slider_color=-1,     ; Slider color
                               background_color=-1) ; Background color
    
    If Not IsGadget(gadget_ID)
      ProcedureReturn 0
    EndIf
    
    If FindString(text$, #CR$) ; A part from DrawText_BF ;-)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    Protected font_ID, font$, used_value, preset_stretchfactor_x_.d, preset_stretchfactor_y_.d
    Protected image_1_ID
    
    If state=-1
      centered_text_toggle_gadget=1
    EndIf
    
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        If state=-1
          state=gadget_bf()\state
        EndIf
        If text$=""
          If state=1
            text$=gadget_bf()\text_state_1$
          Else
            text$=gadget_bf()\text_state_0$
          EndIf
        EndIf
        If x_pos=-1
          x_pos=gadget_bf()\x
        EndIf
        If y_pos=-1
          y_pos=gadget_bf()\y
        EndIf
        If button_width < 0
          button_width=gadget_bf()\button_width
        EndIf
        If button_height < 0
          button_height=gadget_bf()\button_height
        EndIf
        If padding_x < 0
          padding_x=gadget_bf()\padding_x_gadget
        EndIf
        If padding_y < 0
          padding_y=gadget_bf()\padding_y_gadget
        EndIf
        If text_color=-1
          If state=1
            text_color=gadget_bf()\text_color_state_1
          Else
            text_color=gadget_bf()\text_color_state_0
          EndIf
        EndIf
        If slider_color=-1
          If state=1
            slider_color=gadget_bf()\slider_color_state_1
          Else
            slider_color=gadget_bf()\slider_color_state_0
          EndIf
        EndIf
        If background_color=-1
          background_color=gadget_bf()\background_color
        EndIf
        
        font$=gadget_bf()\font$
        font_ID=gadget_bf()\font_ID
        image_1_ID=gadget_bf()\temporary_image_ID
        
        Break
      EndIf
    Next
    
    If (gadget_bf()\gadget_type=#StretchTextImageGadget_BF Or gadget_bf()\gadget_type=#StretchTextCanvasGadget_BF) And
       Not state=-1
      ProcedureReturn 0
    EndIf
    
    If Not IsFont(font_ID)
      ProcedureReturn 0
    EndIf
    
    common_toggle_gadget
    
    ProcedureReturn 1
  EndProcedure 
  
  Procedure GetStateToggleGadget_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\state
      EndIf
    Next
    ProcedureReturn 0
  EndProcedure
  
  Procedure GetPadding_X_ToggleGadget_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\padding_x_gadget
      EndIf
    Next
    ProcedureReturn 0
  EndProcedure
  
  Procedure GetPadding_Y_ToggleGadget_BF(gadget_ID)
    ForEach(gadget_bf())
      If gadget_bf()\gadget_ID=gadget_ID
        ProcedureReturn gadget_bf()\padding_y_gadget
      EndIf
    Next
    ProcedureReturn 0
  EndProcedure
  
  Procedure ToggleToggleGadget_BF(gadget_ID,
                                  state,
                                  text$="",
                                  text_color=-1,
                                  slider_color=-1)
    If Not IsGadget(gadget_ID) : ProcedureReturn 0 : EndIf
    ProcedureReturn SetToggleGadget_BF(gadget_ID,    ; Gadget_ID
                                       state,        ; state=-1            ; Toggle state 0 or 1
                                       text$,        ; text$=""            ; Text
                                       -1,           ; x_pos=-1            ; Gadget output x pos
                                       -1,           ; y_pos=-1            ; Gadget output x pos
                                       -1,           ; button_width=-1     ; Gadget width
                                       -1,           ; button_height=-1    ; Gadget height
                                       -1,           ; padding_x           ; Padding x
                                       -1,           ; padding_y           ; Padding y
                                       text_color,   ; text_color=-1       ; Text color
                                       slider_color, ; slider_color=-1     ; Slider color
                                       -1)           ; background_color=-1 ; Background color
  EndProcedure
  
  Procedure SetCenteredTextToggleGadget_BF(centered_text_toggle_gadget_)
    centered_text_toggle_gadget=centered_text_toggle_gadget_
    If centered_text_toggle_gadget<0 Or centered_text_toggle_gadget>1
      centered_text_toggle_gadget=0
    EndIf
    ProcedureReturn centered_text_toggle_gadget
  EndProcedure
  
  Procedure CanvasToggleGadget_BF()
    canvas_toggle_gadget=1
    ProcedureReturn canvas_toggle_gadget
  EndProcedure
  
  Procedure SetFlagsToggleGadget_BF(flags_toggle_gadget_)
    flags_toggle_gadget=flags_toggle_gadget_
    ProcedureReturn flags_toggle_gadget
  EndProcedure
  
  Procedure SetTextFlagsToggleGadget_BF(text_flag_toggle_gadget_=#PB_VectorParagraph_Center)
    text_flag_toggle_gadget=text_flag_toggle_gadget_
    ProcedureReturn text_flag_toggle_gadget
  EndProcedure
  
  Procedure MoveSliderToggleGadget_BF(slider, slider_offset_)
    If slider=1
      slider_1_offset=slider_offset_
    Else
      slider_0_offset=slider_offset_
    EndIf
    ProcedureReturn slider_offset_  
  EndProcedure
  
  Procedure.d GetSliderMovingSpaceToggleGadget_BF()
    ProcedureReturn slider_moving_space
  EndProcedure
  
  Procedure StretchedText_Vector_BF(output_ID,                                           ; Output_ID
                                    font_ID,                                             ; Font ID      
                                    text$,                                               ; Text
                                    x_pos,                                               ; Output x pos
                                    y_pos,                                               ; Output y pos
                                    output_width,                                        ; Output width
                                    output_height,                                       ; Output height
                                    background_color=$FF000000,                          ; Background color - Alpha channel available - Alpha=0 ignore the alpha channel - set to $01, this is equal 0
                                    text_color=$FFFFFFFF,                                ; Text color - Alpha channel available - Alpha=0 ignore the alpha channel - set to $01, this is equal 0
                                    text_flag_vector_paragraph=#PB_VectorParagraph_Left) ; Text flag
    
    If FindString(text$, #CR$) ; A part from DrawText_BF ;-)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    If Not IsFont(font_ID)
      Protected font_loaded=1
      font_ID=LoadFont(#PB_Any, "Arial", 11)
      If Not IsFont(font_ID)
        ProcedureReturn 0
      EndIf
    EndIf
    
    If output_ID<>-1
      If IsImage(output_ID)
        StartVectorDrawing(ImageVectorOutput(output_ID))
      ElseIf IsGadget(output_ID)
        StartVectorDrawing(CanvasVectorOutput(output_ID))
      ElseIf IsWindow(output_ID)
        StartVectorDrawing(WindowVectorOutput(output_ID))
      Else
        ProcedureReturn -11
      EndIf
    EndIf
    
    VectorFont(FontID(font_ID))
    
    CompilerIf #PB_Compiler_OS=#PB_OS_Linux
      ; Enlarge width prevent automatically line breaks on DrawVectorParagraph()
      Protected get_text_width.d=VectorTextWidth_Multiline_BF(text$)+VectorTextHeight(" ")
    CompilerElse
      ; Enlarge width prevent automatically line breaks on DrawVectorParagraph()
      Protected get_text_width.d=VectorTextWidth_Multiline_BF(text$)+VectorTextHeight(" ")/3
    CompilerEndIf
    
    Protected get_text_height.d=VectorParagraphHeight(text$, get_text_width.d, 3e4)+VectorTextHeight(" ")/6
    
    Protected scaling_width.d=output_width/get_text_width
    Protected scaling_height.d=output_height/get_text_height
    
    If Not Alpha(background_color)
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), 255))
    ElseIf Alpha(background_color)=1
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), 0))
    Else
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), Alpha(background_color)))
    EndIf
    
    If activate_dpi_aware
      ScaleCoordinates(scaling_width*DPI_ResizeFactor_BF(), scaling_height*DPI_ResizeFactor_BF())
    Else
      ScaleCoordinates(scaling_width, scaling_height)
    EndIf
    
    AddPathBox(x_pos/scaling_width, y_pos/scaling_height, get_text_width, get_text_height)
    FillPath()
    MovePathCursor(x_pos/scaling_width, y_pos/scaling_height)
    
    If Not Alpha(text_color)
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), 255))
    ElseIf Alpha(text_color)=1
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), 0))
    Else
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), Alpha(text_color)))
    EndIf
    
    If activate_linear_gradient
      VectorSourceLinearGradient(linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy)
    EndIf
    
    If activate_circular_gradient
      VectorSourceCircularGradient(circular_gradient_x, circular_gradient_y, circular_gradient_radius)
    EndIf
    
    Protected i
    For i=0 To #Size_GradientColors_Array
      If gradient_activate_flag(i)
        VectorSourceGradientColor(gradient_color(i), gradient_position.f(i))
      EndIf
    Next 
    
    DrawVectorParagraph(text$, get_text_width.d, 3e4, text_flag_vector_paragraph)
    
    If font_loaded
      FreeFont(font_ID)
    EndIf
    StopVectorDrawing()
    ProcedureReturn 1
  EndProcedure
  
  ; Rotate MagicStretchText
  ; This function works sprite based
  ; -The sprite mask is black, don't use black less than $04 as border colors
  Procedure RotateMagic_Stretchtext(activate_rotating_stretchtext_=0, degree_stretchtext_=0)
    activate_rotating_stretchtext=activate_rotating_stretchtext_
    degree_stretchtext=degree_stretchtext_
    ProcedureReturn activate_rotating_stretchtext
  EndProcedure
  
  Macro bucket_Fill_MagicStretchText
    If MagicText_BucketFill_image_ID<>-1 
      If IsImage(MagicText_BucketFill_image_ID)
        Protected alpha_=MagicText_BucketFill_texture_alpha*-1
        If alpha_>-2
          alpha_=-2
        EndIf
      Else
        alpha_=-1
      EndIf
      Protected old_no_caching=no_caching
      no_caching=1
      ; If you want to change the source permanently, switch off temporary the caching before with NoCaching_BF(1)
      BucketFill_BF(alpha_, temp_image_ID, MagicText_BucketFill_Image_ID,
                    -1,                                  ; Grab transparence color from sprite pos x - For preset set ..x=-1  
                    text_color,                          ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF 
                    MagicText_BucketFill_x_pos,          ; Startposition texture output x - Preset 0
                    MagicText_BucketFill_y_pos,          ; Startposition texture output y - Preset 0
                    MagicText_BucketFill_xx_pos,         ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                    MagicText_BucketFill_yy_pos,         ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                    0,                                   ; Startposition inside the texture x - Preset 0
                    0,                                   ; Startposition inside the texture y - Preset 0
                    0,                                   ; Endposition inside the texture - Preset 0 = full texture width
                    0,                                   ; Endposition inside the texture - Preset 0 = full texture height
                    MagicText_BucketFill_texture_width,  ; Pre resize texture x
                    MagicText_BucketFill_texture_height) ; Pre resize texture y
                                                         ; mode = 0         - Set only a positioning marker
                                                         ;       -1         - Ignore a texture and use a color - Set as texture_ID a color, as sample $FFFF00
                                                         ;       -2         - Standard preset texture mode
                                                         ;       -3 to -256 - Texture mode with alpha blending
                                                         ; For compatibility with BucketFill this function use negative mode values, but it works at the same also with positive values
                                                         ; This function can output on images, PB sprites and canvas
      no_caching=old_no_caching 
    EndIf
    
    If MagicText_BucketFill_1_image_ID<>-1 
      If IsImage(MagicText_BucketFill_1_image_ID)
        Protected alpha_1=MagicText_BucketFill_1_texture_alpha*-1
        If alpha_1>-2
          alpha_1=-2
        EndIf
      Else
        alpha_1=-1
      EndIf
      Protected old_no_caching_1=no_caching
      no_caching=1
      ; If you want to change the source permanently, switch off temporary the caching before with NoCaching_BF(1)
      BucketFill_BF(alpha_1, temp_image_ID, MagicText_BucketFill_1_Image_ID,
                    -1,                                    ; Grab transparence color from sprite pos x - For preset set ..x=-1  
                    text_color,                            ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF 
                    MagicText_BucketFill_1_x_pos,          ; Startposition texture output x - Preset 0
                    MagicText_BucketFill_1_y_pos,          ; Startposition texture output y - Preset 0
                    MagicText_BucketFill_1_xx_pos,         ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                    MagicText_BucketFill_1_yy_pos,         ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                    0,                                     ; Startposition inside the texture x - Preset 0
                    0,                                     ; Startposition inside the texture y - Preset 0
                    0,                                     ; Endposition inside the texture - Preset 0 = full texture width
                    0,                                     ; Endposition inside the texture - Preset 0 = full texture height
                    MagicText_BucketFill_1_texture_width,  ; Pre resize texture x
                    MagicText_BucketFill_1_texture_height) ; Pre resize texture y
                                                           ; mode = 0         - Set only a positioning marker
                                                           ;       -1         - Ignore a texture and use a color - Set as texture_ID a color, as sample $FFFF00
                                                           ;       -2         - Standard preset texture mode
                                                           ;       -3 to -256 - Texture mode with alpha blending
                                                           ; For compatibility with BucketFill this function use negative mode values, but it works at the same also with positive values
                                                           ; This function can output on images, PB sprites and canvas
      no_caching=old_no_caching_1 
    EndIf
    
    If MagicText_BucketFill_2_image_ID<>-1 
      If IsImage(MagicText_BucketFill_2_image_ID)
        Protected alpha_2=MagicText_BucketFill_2_texture_alpha*-1
        If alpha_2>-2
          alpha_2=-2
        EndIf
      Else
        alpha_2=-1
      EndIf
      Protected old_no_caching_2=no_caching
      no_caching=1
      ; If you want to change the source permanently, switch off temporary the caching before with NoCaching_BF(1)
      BucketFill_BF(alpha_2, temp_image_ID, MagicText_BucketFill_2_Image_ID,
                    -1,                                    ; Grab transparence color from sprite pos x - For preset set ..x=-1  
                    text_color,                            ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF 
                    MagicText_BucketFill_2_x_pos,          ; Startposition texture output x - Preset 0
                    MagicText_BucketFill_2_y_pos,          ; Startposition texture output y - Preset 0
                    MagicText_BucketFill_2_xx_pos,         ; Endposition texture output - Preset 0 = Clipping is automatic to available width 
                    MagicText_BucketFill_2_yy_pos,         ; Endposition texture output - Preset 0 = Clipping is automatic to available height 
                    0,                                     ; Startposition inside the texture x - Preset 0
                    0,                                     ; Startposition inside the texture y - Preset 0
                    0,                                     ; Endposition inside the texture - Preset 0 = full texture width
                    0,                                     ; Endposition inside the texture - Preset 0 = full texture height
                    MagicText_BucketFill_2_texture_width,  ; Pre resize texture x
                    MagicText_BucketFill_2_texture_height) ; Pre resize texture y
                                                           ; mode = 0         - Set only a positioning marker
                                                           ;       -1         - Ignore a texture and use a color - Set as texture_ID a color, as sample $FFFF00
                                                           ;       -2         - Standard preset texture mode
                                                           ;       -3 to -256 - Texture mode with alpha blending
                                                           ; For compatibility with BucketFill this function use negative mode values, but it works at the same also with positive values
                                                           ; This function can output on images, PB sprites and canvas
      no_caching=old_no_caching_2 
    EndIf
  EndMacro
  
  ; Can output on all channels
  ; With the function "ActivateScreenOutput_BF" you can activate the screen output
  ; For output only as image set output_ID = -1 - Use a output function with alpha channel support or you see nothing
  Procedure Magic_StretchText_BF(output_ID,           ; All channels supported
                                 text$,               ; Text
                                 x_pos,               ; Output x pos
                                 y_pos,               ; Output y pos
                                 output_width,        ; Output width
                                 output_height,       ; output height
                                 text_color=#Black,   ; Alpha channel available - Sample $FFAAAAAA - $FF is the alpha channel 
                                 text_stretching_x=0, ; Text stretching x - Headline mode
                                 text_stretching_y=0, ; Text stretching y - Headline mode
                                 padding_x.d=0,       ; Padding x - This is a virtual padding for resizing and boxed output - Negative values available
                                 padding_y.d=0,       ; Padding y - This is a virtual padding for resizing and boxed output - Negative values available
                                 flag=0,              ; Text center=1 - Text right=2 - Text vertical=3
                                 mode=0,              ; Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
                                 font$="Arial",       ; Font type - more is not needed - accept also ""
                                 font_style=0)        ; Font style
    
    Static font_ID, font_style_, font_$
    
    If Not screen_output
      If IsImage(output_ID) ; check for image
        Protected image=1
      ElseIf IsSprite(output_ID) ; Check for PB sprite
        Protected sprite=1
      ElseIf IsGadget(output_ID) ; Check for canvas
        Protected canvas=1
      ElseIf IsWindow(output_ID) ; Check for window
        Protected window=1  
      Else
        ProcedureReturn -11
      EndIf
    EndIf
    
    If Not IsFont(font_ID) Or font_style<>font_style_ Or font$<>font_$
      If IsFont(font_ID) ; PB 5.72 Bug - Linux free not fonts - This method load only absolut necessary fonts
        FreeFont(font_ID)
      EndIf
      font_ID=LoadFont(#PB_Any, font$, 11, font_style) ; Fontsize 11 you must not change
      If Not IsFont(font_ID) : ProcedureReturn 0 : EndIf
      font_style_=font_style
      font_$=font$
    EndIf
    
    If Not Alpha(text_color)
      Protected alpha=255
    Else
      alpha=Alpha(text_color)
    EndIf
    
    If activate_dpi_aware
      padding_x*DPI_ResizeFactor_BF()
      padding_y*DPI_ResizeFactor_BF()
      Protected temp_image_ID=CreateImage(#PB_Any, (output_width+6)*DPI_ResizeFactor_BF(), (output_height+6)*DPI_ResizeFactor_BF())
    Else
      temp_image_ID=CreateImage(#PB_Any, output_width+6, output_height+6)
    EndIf
    
    If Not temp_image_ID
      ProcedureReturn -4
    EndIf
    
    If Not StartVectorDrawing(ImageVectorOutput(temp_image_ID))
      FreeImage(temp_image_ID)
      ProcedureReturn -6
    EndIf
    
    VectorFont(FontID(font_ID))
    
    ; StartVectorDrawing needed
    Fitted_DrawText_Vector_BF(text$,             ; Text - Multiline supported
                              0,                 ; Output x pos
                              0,                 ; Output y pos
                              output_width,      ; Output width
                              output_height,     ; output height
                              $FFFFFFFF,         ; Alpha channel available - Sample $FFAAAAAA - $FF is the alpha channel 
                              text_stretching_x, ; Text stretching x - Headline mode
                              text_stretching_y, ; Text stretching x - Headline mode
                              0,                 ; Padding x - This is a virtual padding for resizing and boxed output
                              0,                 ; Padding y - This is a virtual padding for resizing and boxed output
                              flag,              ; Text center=1 - Text right=2 - Text vertical=3
                              mode)              ; Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
    
    StopVectorDrawing()
    
    GrabSpriteOffsets_BF(temp_image_ID)
    
    CompilerIf #PB_Compiler_OS=#PB_OS_MacOS
      Protected get_x=GetSprite_X_BF()
      Protected get_y=GetSprite_Y_BF()+1
      Protected get_width=GetSprite_width_BF()
      Protected get_height=GetSprite_height_BF()
    CompilerElse
      Protected get_x=GetSprite_X_BF()-1
      Protected get_y=GetSprite_Y_BF()-1
      Protected get_width=GetSprite_width_BF()+2
      Protected get_height=GetSprite_height_BF()+2
    CompilerEndIf
    
    If get_x<0 : get_x=0 : EndIf
    If get_y<0 : get_y=0 : EndIf
    If get_width<1 : get_width=1 : EndIf
    If get_height<1 : get_height=1 : EndIf
    
    FreeImage(temp_image_ID)
    
    Protected old_Change_Embossing_Color=Change_Embossing_Color
    Protected old_Embossing_Color=Embossing_Color
    
    If Not Change_Embossing_Color
      Change_Embossing_Color=1
      Embossing_Color=text_color
    EndIf
    
    If activate_dpi_aware
      temp_image_ID=CreateImage(#PB_Any, (output_width+6)*DPI_ResizeFactor_BF(), (output_height+6)*DPI_ResizeFactor_BF(), 32, #PB_Image_Transparent)
    Else
      temp_image_ID=CreateImage(#PB_Any, output_width+6, output_height+6, 32, #PB_Image_Transparent)
    EndIf
    
    Change_Embossing_Color=old_change_Embossing_Color
    Embossing_Color=old_Embossing_Color
    
    If Not temp_image_ID
      ProcedureReturn -4
    EndIf
    
    If Not StartVectorDrawing(ImageVectorOutput(temp_image_ID))
      FreeImage(temp_image_ID)
      ProcedureReturn -6
    EndIf
    
    VectorFont(FontID(font_ID))
    
    If activate_linear_gradient
      VectorSourceLinearGradient(linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy)
    EndIf
    
    If activate_circular_gradient
      VectorSourceCircularGradient(circular_gradient_x, circular_gradient_y, circular_gradient_radius)
    EndIf
    
    Protected i
    For i=0 To #Size_GradientColors_Array
      If gradient_activate_flag(i)
        VectorSourceGradientColor(gradient_color(i), gradient_position.f(i))
      EndIf
    Next 
    
    ; StartVectorDrawing needed
    Fitted_DrawText_Vector_BF(text$,                ; Text - Multiline supported
                              0,                    ; Output x pos
                              0,                    ; Output y pos
                              output_width,         ; Output width
                              output_height,        ; output height
                              text_color|$FF000000, ; Alpha channel available - Sample $FFAAAAAA - $FF is the alpha channel 
                              text_stretching_x,    ; Text stretching x - Headline mode
                              text_stretching_y,    ; Text stretching x - Headline mode
                              0,                    ; Padding x - This is a virtual padding for resizing and boxed output
                              0,                    ; Padding y - This is a virtual padding for resizing and boxed output
                              flag,                 ; Text center=1 - Text right=2 - Text vertical=3
                              mode)                 ; Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
    
    StopVectorDrawing()
    
    bucket_Fill_MagicStretchText
    
    Protected grabed_image_ID=GrabImage(temp_image_ID, #PB_Any, get_x, get_y, get_width, Get_height)
    If Not grabed_image_ID
      FreeImage(temp_image_ID)
      ProcedureReturn -6
    EndIf
    
    FreeImage(temp_image_ID)
    
    If activate_dpi_aware
      output_width*DPI_ResizeFactor_BF()
      output_height*DPI_ResizeFactor_BF()
    EndIf
    
    Protected output_width_=output_width-padding_x
    Protected output_height_=output_height-padding_y
    
    If output_width_<1 : output_width_=1 : EndIf
    If output_height_<1 : output_height_=1 : EndIf
    
    If Not ResizeImage(grabed_image_ID, output_width_, output_height_, resize_interpolation)
      ProcedureReturn -27
    EndIf
    
    If activate_rotating_stretchtext
      Protected old_alpha_sprite=alpha_sprite
      alpha_sprite=1
      RotateSprite_BF(grabed_image_ID, 
                      0, ; Grab transparence color from sprite pos x - For preset set ..x=-1 - x=-2 ignore the invisible color and set image_get_color_y as mask color
                      0, ; Grab transparence color from sprite pos y - For preset set ..y= the color you want - as sample $FF00FF
                      degree_stretchtext) ; Rotating degree - The sprite rotates around its center point - With the rotating_offsets you can relocate this point                    
      alpha_sprite=old_alpha_sprite
    EndIf
    
    If output_ID=-1
      ProcedureReturn grabed_image_ID
    EndIf
    
    If screen_output
      Protected result=StartDrawing(ScreenOutput())
    Else
      If image
        result=StartDrawing(ImageOutput(output_ID))
      ElseIf sprite
        result=StartDrawing(SpriteOutput(output_ID))
      ElseIf canvas
        result=StartDrawing(CanvasOutput(output_ID))
      Else
        result=StartDrawing(WindowOutput(output_ID)) 
      EndIf
    EndIf
    
    If Not result
      FreeImage(grabed_image_ID)
      ProcedureReturn -11
    EndIf
    
    DrawAlphaImage(ImageID(grabed_image_ID), x_pos+padding_x/2, y_pos+padding_y/2, alpha)
    
    StopDrawing()
    
    FreeImage(grabed_image_ID)
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure Magic_StretchText_Pro_Border_BF(frame_size_stretchtext_=0,
                                            frame_padding_stretchtext_=0,
                                            frame_rounding_stretchtext_=0,
                                            frame_color_stretchtext_=$FFAAAAAA)
    
    If stretchtext_pro_offset_x Or stretchtext_pro_offset_y
      frame_size_stretchtext_=0
    EndIf
    
    frame_size_stretchtext=frame_size_stretchtext_
    frame_padding_stretchtext_x=frame_padding_stretchtext_
    frame_padding_stretchtext_y=frame_padding_stretchtext_
    frame_rounding_stretchtext=frame_rounding_stretchtext_
    frame_color_stretchtext=frame_color_stretchtext_
    ProcedureReturn frame_size_stretchtext
  EndProcedure
  
  Procedure Magic_StretchText_Pro_Border_DualPadding_BF(frame_size_stretchtext_=0,
                                                        frame_padding_stretchtext_x_=0,
                                                        frame_padding_stretchtext_y_=0,
                                                        frame_rounding_stretchtext_=0,
                                                        frame_color_stretchtext_=$FFAAAAAA)
    
    frame_size_stretchtext=frame_size_stretchtext_
    frame_padding_stretchtext_x=frame_padding_stretchtext_x_
    frame_padding_stretchtext_y=frame_padding_stretchtext_y_
    frame_rounding_stretchtext=frame_rounding_stretchtext_
    frame_color_stretchtext=frame_color_stretchtext_
    ProcedureReturn frame_size_stretchtext
  EndProcedure
  
  Procedure Magic_StretchText_Pro_TextOffset_BF(stretchtext_pro_offset_x_=0, stretchtext_pro_offset_y_=0)
    stretchtext_pro_offset_x=stretchtext_pro_offset_x_
    stretchtext_pro_offset_y=stretchtext_pro_offset_y_
  EndProcedure
  
  ; If letters are cut off, adjust cut_offset
  Procedure Magic_StretchText_Pro_BF(output_ID,                  ; All channels supported
                                     text$,                      ; Text
                                     x_pos,                      ; Output x pos
                                     y_pos,                      ; Output y pos
                                     text_width,                 ; Text width
                                     text_height,                ; Text height
                                     fitted_text_width,          ; Fitted text width  - Negative values available
                                     fitted_text_height,         ; fitted text height - Negative values available
                                     text_color=#Black,          ; Alpha channel available - Sample $FFAAAAAA - $FF is the alpha channel
                                     background_color=$FFFAFAFA, ; Alpha channel available - For invisible background set to -1
                                     text_stretching_x=0,        ; Text stretching x - Headline mode
                                     text_stretching_y=0,        ; Text stretching y - Headline mode
                                     padding_x.d=0,              ; Padding x - This is a virtual padding for resizing and boxed output - Negative values available
                                     padding_y.d=0,              ; Padding y - This is a virtual padding for resizing and boxed output - Negative values available
                                     flag=0,                     ; Text center=1 - Text right=2 - Text vertical=3
                                     mode=0,                     ; Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
                                     font$="Arial",              ; Font type - more is not needed - accept also ""
                                     font_style=0,               ; Font style
                                     inherited_font_ID=0)        ; You can inherit a font
    
    Static font_ID, font_style_, font_$
    
    If output_ID<>-1
      If Not screen_output
        If IsImage(output_ID) ; check for image
          Protected image=1
        ElseIf IsSprite(output_ID) ; Check for PB sprite
          Protected sprite=1
        ElseIf IsGadget(output_ID) ; Check for canvas
          Protected canvas=1
        ElseIf IsWindow(output_ID) ; Check for window
          Protected window=1  
        Else
          ProcedureReturn -11
        EndIf
      EndIf
    EndIf
    
    If IsFont(inherited_font_ID)
      font_ID=inherited_font_ID
    Else 
      If Not IsFont(font_ID) Or font_style<>font_style_ Or font$<>font_$
        If IsFont(font_ID) ; PB 5.72 Bug - Linux free not fonts - This method load only absolut necessary fonts
          FreeFont(font_ID)
        EndIf
        font_ID=LoadFont(#PB_Any, font$, 11, font_style) ; Fontsize 11 you must not change
        If Not IsFont(font_ID) : ProcedureReturn 0 : EndIf
        font_style_=font_style
        font_$=font$
      EndIf
    EndIf
    
    If degree_stretchtext And Not text_color ; Simple blackmask rotating fuse
      text_color=4                           ; Change black a little
    EndIf
    
    If Not Alpha(text_color)
      Protected alpha=255
    Else
      alpha=Alpha(text_color)
    EndIf
    
    Protected cut_offset=7 ; Cut offset
    
    If activate_dpi_aware
      padding_x*DPI_ResizeFactor_BF()
      padding_y*DPI_ResizeFactor_BF()
      fitted_text_width*DPI_ResizeFactor_BF()
      fitted_text_height*DPI_ResizeFactor_BF()
      Protected temp_image_ID=CreateImage(#PB_Any, (text_width+cut_offset)*DPI_ResizeFactor_BF(), (text_height+cut_offset)*DPI_ResizeFactor_BF())
    Else
      temp_image_ID=CreateImage(#PB_Any, text_width+cut_offset, text_height+cut_offset)
    EndIf
    
    If Not temp_image_ID
      ProcedureReturn -4
    EndIf
    
    If Not StartVectorDrawing(ImageVectorOutput(temp_image_ID))
      FreeImage(temp_image_ID)
      ProcedureReturn -6
    EndIf
    
    VectorFont(FontID(font_ID))
    
    ; StartVectorDrawing needed
    Fitted_DrawText_Vector_BF(text$,             ; Text - Multiline supported
                              0,                 ; Output x pos
                              0,                 ; Output y pos
                              text_width,        ; Output width
                              text_height,       ; output height
                              $FFFFFFFF,         ; Alpha channel available - Sample $FFAAAAAA - $FF is the alpha channel 
                              text_stretching_x, ; Text stretching x - Headline mode
                              text_stretching_y, ; Text stretching x - Headline mode
                              0,                 ; Padding x - This is a virtual padding for resizing and boxed output
                              0,                 ; Padding y - This is a virtual padding for resizing and boxed output
                              flag,              ; Text center=1 - Text right=2 - Text vertical=3
                              mode)              ; Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
    
    StopVectorDrawing()
    
    GrabSpriteOffsets_BF(temp_image_ID)
    
    Protected get_x=GetSprite_X_BF()-cut_offset/2
    Protected get_y=GetSprite_Y_BF()-cut_offset/2
    Protected get_width=GetSprite_width_BF()+cut_offset-1
    Protected get_height=GetSprite_height_BF()+cut_offset-1
    
    If get_x<0 : get_x=0 : EndIf ; Fuse
    If get_y<0 : get_y=0 : EndIf
    If get_width<1 : get_width=1 : EndIf
    If get_height<1 : get_height=1 : EndIf
    
    FreeImage(temp_image_ID)
    
    Protected old_Change_Embossing_Color=Change_Embossing_Color
    Protected old_Embossing_Color=Embossing_Color
    
    If Not Change_Embossing_Color
      Change_Embossing_Color=1
      Embossing_Color=text_color
    EndIf
    
    If activate_dpi_aware
      temp_image_ID=CreateImage(#PB_Any, (text_width+cut_offset+1)*DPI_ResizeFactor_BF(), (text_height+cut_offset+1)*DPI_ResizeFactor_BF(), 32, #PB_Image_Transparent)
    Else
      temp_image_ID=CreateImage(#PB_Any, text_width+cut_offset+1, text_height+cut_offset+1, 32, #PB_Image_Transparent)
    EndIf
    
    Change_Embossing_Color=old_Change_Embossing_Color
    Embossing_Color=old_Embossing_Color
    
    If Not temp_image_ID
      ProcedureReturn -4
    EndIf
    
    If Not StartVectorDrawing(ImageVectorOutput(temp_image_ID))
      FreeImage(temp_image_ID)
      ProcedureReturn -6
    EndIf
    
    VectorFont(FontID(font_ID))
    
    If activate_linear_gradient
      VectorSourceLinearGradient(linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy)
    EndIf
    
    If activate_circular_gradient
      VectorSourceCircularGradient(circular_gradient_x, circular_gradient_y, circular_gradient_radius)
    EndIf
    
    Protected i
    For i=0 To #Size_GradientColors_Array
      If gradient_activate_flag(i)
        VectorSourceGradientColor(gradient_color(i), gradient_position.f(i))
      EndIf
    Next 
    
    ; StartVectorDrawing needed
    Fitted_DrawText_Vector_BF(text$,                ; Text - Multiline supported
                              0,                    ; Output x pos
                              0,                    ; Output y pos
                              text_width,           ; Output width
                              text_height,          ; output height
                              text_color|$FF000000, ; Alpha channel available - Sample $FFAAAAAA - $FF is the alpha channel 
                              text_stretching_x,    ; Text stretching x - Headline mode
                              text_stretching_y,    ; Text stretching x - Headline mode
                              0,                    ; Padding x - This is a virtual padding for resizing and boxed output
                              0,                    ; Padding y - This is a virtual padding for resizing and boxed output
                              flag,                 ; Text center=1 - Text right=2 - Text vertical=3
                              mode)                 ; Revers text mode=1 - RTL text mode=2 - Give only back the needed width mode=3, height mode=4
    
    StopVectorDrawing()
    
    bucket_Fill_MagicStretchText
    
    Protected grabed_image_ID=GrabImage(temp_image_ID, #PB_Any, get_x, get_y, get_width, get_height)
    If Not grabed_image_ID
      FreeImage(temp_image_ID)
      ProcedureReturn -6
    EndIf
    
    FreeImage(temp_image_ID)
    
    If activate_dpi_aware
      text_width*DPI_ResizeFactor_BF()
      text_height*DPI_ResizeFactor_BF()
    EndIf
    
    Protected text_width_=text_width-padding_x
    Protected text_height_=text_height-padding_y
    
    If text_width_<1 : text_width_=1 : EndIf
    If text_height_<1 : text_height_=1 : EndIf
    
    If Not ResizeImage(grabed_image_ID, text_width_, text_height_, resize_interpolation)
      ProcedureReturn -27
    EndIf
    
    Protected upscaling=FitImageGetUpscalingState_BF() 
    
    FitImageActivateUpscaling_BF(1) ; Activate FitImage upscaling
    
    Protected frame_padding_stretchtext_x_=frame_padding_stretchtext_x
    Protected frame_padding_stretchtext_y_=frame_padding_stretchtext_y
    Protected frame_rounding_stretchtext_=frame_rounding_stretchtext
    Protected frame_color_stretchtext_=frame_color_stretchtext
    
    If Not Alpha(frame_color_stretchtext_)
      Protected frame_alpha=255
    Else
      frame_alpha=Alpha(frame_color_stretchtext_)
    EndIf
    
    If Not Alpha(background_color)
      Protected background_color_alpha=255
    Else
      background_color_alpha=Alpha(background_color)
    EndIf
    
    If activate_dpi_aware
      frame_padding_stretchtext_x_=(frame_padding_stretchtext_x+frame_size_stretchtext)*DPI_ResizeFactor_BF()
      frame_padding_stretchtext_y_=(frame_padding_stretchtext_y+frame_size_stretchtext)*DPI_ResizeFactor_BF()
    Else
      frame_padding_stretchtext_x_=(frame_padding_stretchtext_x+frame_size_stretchtext)
      frame_padding_stretchtext_y_=(frame_padding_stretchtext_y+frame_size_stretchtext)
    EndIf
    
    ; This function give back a fitted image
    Protected grabed_image_1_ID=FitImage_StretchText(grabed_image_ID,
                                                     fitted_text_width-padding_x,        ; Fit image to this width
                                                     fitted_text_height-padding_y,       ; Fit image to this height
                                                     RGBA(Red(background_color),         ; Background color 
                                                          Green(background_color),
                                                          Blue(background_color),
                                                          background_color_alpha),
                                                     frame_padding_stretchtext_x_*2,     ; Frame padding x
                                                     frame_padding_stretchtext_y_*2,     ; Frame padding y
                                                     frame_rounding_stretchtext_,        ; Edge Rounding
                                                     RGBA(Red(frame_color_stretchtext_), ; Frame color 
                                                          Green(frame_color_stretchtext_),
                                                          Blue(frame_color_stretchtext_),
                                                          frame_alpha),
                                                     frame_size_stretchtext)             ; Frame size  
    
    FitImageActivateUpscaling_BF(upscaling)
    
    FreeImage(grabed_image_ID)
    
    If activate_rotating_stretchtext
      Protected old_alpha_sprite=alpha_sprite
      alpha_sprite=1
      RotateSprite_BF(grabed_image_1_ID, 
                      -1,
                      0,
                      degree_stretchtext)                  
      alpha_sprite=old_alpha_sprite
    EndIf
    
    If output_ID=-1
      ProcedureReturn grabed_image_1_ID
    EndIf
    
    If screen_output
      Protected result=StartDrawing(ScreenOutput())
    Else
      If image
        result=StartDrawing(ImageOutput(output_ID))
      ElseIf sprite
        result=StartDrawing(SpriteOutput(output_ID))
      ElseIf canvas
        result=StartDrawing(CanvasOutput(output_ID))
      Else
        result=StartDrawing(WindowOutput(output_ID)) 
      EndIf
    EndIf
    
    DrawAlphaImage(ImageID(grabed_image_1_ID), x_pos+padding_x/2, y_pos+padding_y/2, alpha)
    
    StopDrawing()
    
    FreeImage(grabed_image_1_ID)
    
    ProcedureReturn 1
  EndProcedure
  
  Procedure StretchedText_Roundbox_Vector_BF(output_ID,                                           ; Output_ID
                                             font_ID,                                             ; Font ID      
                                             text$,                                               ; Text
                                             x_pos,                                               ; Output x pos
                                             y_pos,                                               ; Output y pos
                                             output_width,                                        ; Output width
                                             output_height,                                       ; Output height
                                             radius=4,                                            ; Rounding radius
                                             frame_thickness.d=3,                                 ; Frame thickness - Set zero for ignore a frame - A negative value reduce the background
                                             frame_color=$00000000,                               ; Frame color - Alpha channel available - Alpha=0 ignore the alpha channel - set to $01, this is equal 0
                                             background_color=$FFFFFFFF,                          ; Background color - Alpha channel available - Alpha=0 ignore the alpha channel - set to $01, this is equal 0
                                             padding_x.d=0,                                       ; Padding x
                                             padding_y.d=0,                                       ; Padding y
                                             text_color=$00000000,                                ; Text color - Alpha channel available - Alpha=0 ignore the alpha channel - set to $01, this is equal 0
                                             text_flag_vector_paragraph=#PB_VectorParagraph_Left) ; Text flag
    
    If frame_thickness<0 : frame_thickness=0 : EndIf
    
    If activate_dpi_aware
      radius*DPI_ResizeFactor_BF()
    EndIf
    
    x_pos+(frame_thickness+padding_x)
    y_pos+(frame_thickness+padding_y)
    
    output_width-(frame_thickness+padding_x)*2
    output_height-(frame_thickness+padding_y)*2
    
    If IsImage(output_ID)
      StartVectorDrawing(ImageVectorOutput(output_ID))
    ElseIf IsGadget(output_ID)
      StartVectorDrawing(CanvasVectorOutput(output_ID))
    ElseIf IsWindow(output_ID)
      StartVectorDrawing(WindowVectorOutput(output_ID))
    Else
      ProcedureReturn -11
    EndIf
    
    If activate_dpi_aware
      If frame_thickness
        frame_thickness+1
      EndIf
      If Rotate_StretchedText_Roundbox_Vector
        RotateCoordinates((x_pos-frame_thickness-padding_x+output_width/2)*DPI_ResizeFactor_BF(),
                          (y_pos-frame_thickness-padding_y+output_height/2)*DPI_ResizeFactor_BF(),
                          Rotate_StretchedText_Roundbox_Vector)
      EndIf
      Protected result=RoundBox_Vector_RAW_BF(output_ID,
                                              (x_pos-frame_thickness-padding_x)*DPI_ResizeFactor_BF(),
                                              (y_pos-frame_thickness-padding_y)*DPI_ResizeFactor_BF(),
                                              (output_width+(frame_thickness+padding_x)*2)*DPI_ResizeFactor_BF(),
                                              (output_height+(frame_thickness+padding_y)*2)*DPI_ResizeFactor_BF(),
                                              radius,
                                              frame_thickness*DPI_ResizeFactor_BF(),
                                              frame_color,
                                              background_color)
    Else
      If Rotate_StretchedText_Roundbox_Vector
        RotateCoordinates((x_pos-frame_thickness-padding_x+output_width/2),
                          (y_pos-frame_thickness-padding_y+output_height/2),
                          Rotate_StretchedText_Roundbox_Vector)
      EndIf
      result=RoundBox_Vector_RAW_BF(output_ID,
                                    x_pos-frame_thickness-padding_x,
                                    y_pos-frame_thickness-padding_y,
                                    output_width+(frame_thickness+padding_x)*2,
                                    output_height+(frame_thickness+padding_y)*2,
                                    radius,
                                    frame_thickness,
                                    frame_color,
                                    background_color)
    EndIf
    
    If result <1 : ProcedureReturn result : EndIf
    
    If output_width<0 : output_width=0 : EndIf
    If output_height<0 : output_height=0 : EndIf  
    
    If FindString(text$, #CR$) ; A part from DrawText_BF ;-)
      text$=ReplaceString(text$, #CRLF$, #LF$)
      text$=ReplaceString(text$, #LFCR$, #LF$)
      ReplaceString(text$, #CR$, #LF$, #PB_String_InPlace)
    EndIf
    
    If Not IsFont(font_ID)
      Protected font_loaded=1
      font_ID=LoadFont(#PB_Any, "Arial", 11)
      If Not IsFont(font_ID)
        ProcedureReturn 0
      EndIf
    EndIf
    
    VectorFont(FontID(font_ID)) 
    
    ; Enlarge width prevent automatically line breaks on DrawVectorParagraph()
    Protected get_text_width.d=VectorTextWidth_Multiline_BF(text$)+VectorTextHeight(" ")/3
    
    Protected get_text_height.d=VectorParagraphHeight(text$, get_text_width.d, 3e4)+VectorTextHeight(" ")/6
    
    Protected scaling_width.d=output_width/get_text_width
    Protected scaling_height.d=output_height/get_text_height
    
    If Not Alpha(background_color)
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), 255))
    ElseIf Alpha(background_color)=1
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), 0))
    Else
      VectorSourceColor(RGBA(Red(background_color), Green(background_color), Blue(background_color), Alpha(background_color)))
    EndIf
    
    If activate_dpi_aware
      ScaleCoordinates(scaling_width*DPI_ResizeFactor_BF(), scaling_height*DPI_ResizeFactor_BF())
    Else
      ScaleCoordinates(scaling_width, scaling_height)
    EndIf
    
    MovePathCursor(x_pos/scaling_width, y_pos/scaling_height)
    
    If Not Alpha(text_color)
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), 255))
    ElseIf Alpha(text_color)=1
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), 0))
    Else
      VectorSourceColor(RGBA(Red(text_color), Green(text_color), Blue(text_color), Alpha(text_color)))
    EndIf
    
    If activate_linear_gradient
      VectorSourceLinearGradient(linear_gradient_x, linear_gradient_y, linear_gradient_xx, linear_gradient_yy)
    EndIf
    
    If activate_circular_gradient
      VectorSourceCircularGradient(circular_gradient_x, circular_gradient_y, circular_gradient_radius)
    EndIf
    
    Protected i
    For i=0 To #Size_GradientColors_Array
      If gradient_activate_flag(i)
        VectorSourceGradientColor(gradient_color(i), gradient_position.f(i))
      EndIf
    Next 
    
    DrawVectorParagraph(text$, get_text_width.d, 3e4, text_flag_vector_paragraph)
    
    If font_loaded
      FreeFont(font_ID)
    EndIf
    StopVectorDrawing()
    ProcedureReturn 1
  EndProcedure
  
  Procedure Deactivate_TextStretching_BF(deactivate_text_stretching_, preset_stretchfactor_x_.d=1, preset_stretchfactor_y_.d=1)
    deactivate_text_stretching=deactivate_text_stretching_
    preset_stretchfactor_x.d=preset_stretchfactor_x_.d
    preset_stretchfactor_y.d=preset_stretchfactor_y_.d
    ProcedureReturn deactivate_text_stretching
  EndProcedure
  
  Procedure Rotate_StretchedText_Roundbox_Vector_BF(Rotate_StretchedText_Roundbox_Vector_)
    Rotate_StretchedText_Roundbox_Vector=Rotate_StretchedText_Roundbox_Vector_
    ProcedureReturn Rotate_StretchedText_Roundbox_Vector
  EndProcedure
  
  ; ---------- Data functions ----------
  Procedure GetData_int_1_BF(ID)
    Protected result
    ForEach(data_bf())
      If data_bf()\ID=ID
        result=data_bf()\data_int_1
        Break
      EndIf
    Next
    ProcedureReturn result
  EndProcedure
  
  Procedure GetData_int_2_BF(ID)
    Protected result
    ForEach(data_bf())
      If data_bf()\ID=ID
        result=data_bf()\data_int_2
        Break
      EndIf
    Next
    ProcedureReturn result 
  EndProcedure
  
  Procedure.d GetData_double_1_BF(ID)
    Protected result.d
    ForEach(data_bf())
      If data_bf()\ID=ID
        result=data_bf()\data_double_1
        Break
      EndIf
    Next
    ProcedureReturn result
  EndProcedure
  
  Procedure.d GetData_double_2_BF(ID)
    Protected result.d
    ForEach(data_bf())
      If data_bf()\ID=ID
        result=data_bf()\data_double_2
        Break
      EndIf
    Next
    ProcedureReturn result 
  EndProcedure
  
  Procedure.s GetData_string_1_BF(ID)
    Protected result$
    ForEach(data_bf())
      If data_bf()\ID=ID
        result$=data_bf()\data_string_1$
        Break
      EndIf
    Next
    ProcedureReturn result$
  EndProcedure
  
  Procedure.s GetData_string_2_BF(ID)
    Protected result$
    ForEach(data_bf())
      If data_bf()\ID=ID
        result$=data_bf()\data_string_2$
        Break
      EndIf
    Next
    ProcedureReturn result$ 
  EndProcedure
  
  Procedure SetData_int_1_BF(ID, data_int_1)
    If Not ListSize(data_bf())
      AddElement(data_bf())
    EndIf
    ForEach(data_bf())
      If data_bf()\ID=ID
        data_bf()\data_int_1=data_int_1
        ProcedureReturn 1
      EndIf
    Next
    AddElement(data_bf())
    data_bf()\data_int_1=data_int_1
    data_bf()\ID=ID
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetData_int_2_BF(ID, data_int_2)
    If Not ListSize(data_bf())
      AddElement(data_bf())
    EndIf
    ForEach(data_bf())
      If data_bf()\ID=ID
        data_bf()\data_int_2=data_int_2
        ProcedureReturn 1
      EndIf
    Next
    AddElement(data_bf())
    data_bf()\data_int_2=data_int_2
    data_bf()\ID=ID
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetData_double_1_BF(ID, data_double_1.d)
    If Not ListSize(data_bf())
      AddElement(data_bf())
    EndIf
    ForEach(data_bf())
      If data_bf()\ID=ID
        data_bf()\data_double_1=data_double_1
        ProcedureReturn 1
      EndIf
    Next
    AddElement(data_bf())
    data_bf()\data_double_1=data_double_1
    data_bf()\ID=ID
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetData_double_2_BF(ID, data_double_2.d)
    If Not ListSize(data_bf())
      AddElement(data_bf())
    EndIf
    ForEach(data_bf())
      If data_bf()\ID=ID
        data_bf()\data_double_2=data_double_2
        ProcedureReturn 1
      EndIf
    Next
    AddElement(data_bf())
    data_bf()\data_double_2=data_double_2
    data_bf()\ID=ID
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetData_string_1_BF(ID, data_string_1$)
    If Not ListSize(data_bf())
      AddElement(data_bf())
    EndIf
    ForEach(data_bf())
      If data_bf()\ID=ID
        data_bf()\data_string_1$=data_string_1$
        ProcedureReturn 1
      EndIf
    Next
    AddElement(data_bf())
    data_bf()\data_string_1$=data_string_1$
    data_bf()\ID=ID
    ProcedureReturn 1
  EndProcedure
  
  Procedure SetData_string_2_BF(ID, data_string_2$)
    If Not ListSize(data_bf())
      AddElement(data_bf())
    EndIf
    ForEach(data_bf())
      If data_bf()\ID=ID
        data_bf()\data_string_2$=data_string_2$
        ProcedureReturn 1
      EndIf
    Next
    AddElement(data_bf())
    data_bf()\data_string_2$=data_string_2$
    data_bf()\ID=ID
    ProcedureReturn 1
  EndProcedure
  
  Procedure FreeData_BF(ID)
    Protected result
    ForEach data_BF()
      While NextElement(data_BF())
        If data_bf()\ID=ID
          DeleteElement(data_bf())
          result+1
        EndIf
      Wend
    Next
    ProcedureReturn result
  EndProcedure
  ; ------- End - Data functions -------
  
  Procedure Embossing_Color(Change_Embossing_Color_=0, Embossing_Color_=#Black)
    If Change_Embossing_Color_=1
      Change_Embossing_Color=Change_Embossing_Color_
      Embossing_Color=Embossing_Color_
    Else
      Change_Embossing_Color=0
      Embossing_Color=#Black
    EndIf
    ProcedureReturn used_Embossing_Color
  EndProcedure
  
  Procedure MagicText_BucketFill_BF(MagicText_BucketFill_image_ID_=-1,
                                    MagicText_BucketFill_x_pos_=0,
                                    MagicText_BucketFill_y_pos_=0,
                                    MagicText_BucketFill_xx_pos_=0,
                                    MagicText_BucketFill_yy_pos_=0,
                                    MagicText_BucketFill_texture_width_=0,
                                    MagicText_BucketFill_texture_height_=0,
                                    MagicText_BucketFill_texture_alpha_=0)
    MagicText_BucketFill_image_ID=MagicText_BucketFill_image_ID_
    MagicText_BucketFill_x_pos=MagicText_BucketFill_x_pos_
    MagicText_BucketFill_y_pos=MagicText_BucketFill_y_pos_
    MagicText_BucketFill_xx_pos=MagicText_BucketFill_xx_pos_
    MagicText_BucketFill_yy_pos=MagicText_BucketFill_yy_pos_
    MagicText_BucketFill_texture_width=MagicText_BucketFill_texture_width_
    MagicText_BucketFill_texture_height=MagicText_BucketFill_texture_height_
    MagicText_BucketFill_texture_alpha=MagicText_BucketFill_texture_alpha_
    ProcedureReturn 1
  EndProcedure
  
  Procedure MagicText_BucketFill_1_BF(MagicText_BucketFill_1_image_ID_=-1,
                                      MagicText_BucketFill_1_x_pos_=0,
                                      MagicText_BucketFill_1_y_pos_=0,
                                      MagicText_BucketFill_1_xx_pos_=0,
                                      MagicText_BucketFill_1_yy_pos_=0,
                                      MagicText_BucketFill_1_texture_width_=0,
                                      MagicText_BucketFill_1_texture_height_=0,
                                      MagicText_BucketFill_1_texture_alpha_=0)
    MagicText_BucketFill_1_image_ID=MagicText_BucketFill_1_image_ID_
    MagicText_BucketFill_1_x_pos=MagicText_BucketFill_1_x_pos_
    MagicText_BucketFill_1_y_pos=MagicText_BucketFill_1_y_pos_
    MagicText_BucketFill_1_xx_pos=MagicText_BucketFill_1_xx_pos_
    MagicText_BucketFill_1_yy_pos=MagicText_BucketFill_1_yy_pos_
    MagicText_BucketFill_1_texture_width=MagicText_BucketFill_1_texture_width_
    MagicText_BucketFill_1_texture_height=MagicText_BucketFill_1_texture_height_
    MagicText_BucketFill_1_texture_alpha=MagicText_BucketFill_1_texture_alpha_
    ProcedureReturn 1
  EndProcedure
  
  Procedure MagicText_BucketFill_2_BF(MagicText_BucketFill_2_image_ID_=-1,
                                      MagicText_BucketFill_2_x_pos_=0,
                                      MagicText_BucketFill_2_y_pos_=0,
                                      MagicText_BucketFill_2_xx_pos_=0,
                                      MagicText_BucketFill_2_yy_pos_=0,
                                      MagicText_BucketFill_2_texture_width_=0,
                                      MagicText_BucketFill_2_texture_height_=0,
                                      MagicText_BucketFill_2_texture_alpha_=0)
    MagicText_BucketFill_2_image_ID=MagicText_BucketFill_2_image_ID_
    MagicText_BucketFill_2_x_pos=MagicText_BucketFill_2_x_pos_
    MagicText_BucketFill_2_y_pos=MagicText_BucketFill_2_y_pos_
    MagicText_BucketFill_2_xx_pos=MagicText_BucketFill_2_xx_pos_
    MagicText_BucketFill_2_yy_pos=MagicText_BucketFill_2_yy_pos_
    MagicText_BucketFill_2_texture_width=MagicText_BucketFill_2_texture_width_
    MagicText_BucketFill_2_texture_height=MagicText_BucketFill_2_texture_height_
    MagicText_BucketFill_2_texture_alpha=MagicText_BucketFill_2_texture_alpha_
    ProcedureReturn 1
  EndProcedure
  
  Procedure Is_AlphaChannel_BF(image_ID)
    If IsImage(image_ID) And StartDrawing(ImageOutput(image_ID))
      Protected pixel_format.a=DrawingBufferPixelFormat()
      StopDrawing()
      If pixel_format=#PB_PixelFormat_32Bits_RGB Or pixel_format=#PB_PixelFormat_32Bits_BGR
        ProcedureReturn 1
      EndIf
    EndIf
    ProcedureReturn 0
  EndProcedure
  
  Procedure CreateRoundCornersImage_BF(image_ID, radius, background_color)
    If Not StartVectorDrawing(ImageVectorOutput(image_ID))
      ProcedureReturn -4
    EndIf
    
    VectorSourceColor($FF000000|background_color)
    
    Protected x, y, width, height, width_1, height_1
    x=-radius/4+1
    y=-radius/4+1
    width=VectorOutputWidth()+radius/2-2
    height=VectorOutputHeight()+radius/2-2
    width_1=VectorOutputWidth()
    height_1=VectorOutputHeight()
    
    If radius
      MovePathCursor(x+radius, y)
      AddPathArc(width-radius, 0, width-radius, radius, radius, #PB_Path_Relative)
      AddPathArc(0, height-radius, -radius, height-radius, radius, #PB_Path_Relative)
      AddPathArc(-width+radius, 0, -width+radius, -radius, radius, #PB_Path_Relative)
      AddPathArc(0, -height+radius, radius, -height+radius, radius, #PB_Path_Relative)
    EndIf
    
    StrokePath(radius/2)
    
    AddPathBox(0, 0, radius/4.5, radius/4.5)
    AddPathBox(width_1-radius/4.5, 0, radius/4.5, radius/4.5)
    AddPathBox(0, height_1-radius/4.5, radius/4.5, radius/4.5)
    AddPathBox(width_1-radius/4.5, height_1-radius/4.5, radius/4.5, radius/4.5)
    FillPath()
    
    StopVectorDrawing()
    ProcedureReturn 1
  EndProcedure
  
EndModule

UseModule GFX_Wizzard_BF

CompilerIf #PB_Compiler_IsMainFile
  MessageRequester("GFX_Wizzard_BF", "This is only the Module"+#LF$+#LF$+
                                     "So you see nothing !"+#LF$+#LF$+
                                     "This is the GFX_Wizzard_BF main Module !")
CompilerEndIf
; IDE Options = PureBasic 5.73 LTS (Windows - x64)
; CursorPosition = 13955
; FirstLine = 13954
; Folding = -----------------------------------------------------------------------------------------------------------
; EnableXP
; Optimizer
; EnableUnicode